
#include <unistd.h>
#include <cstring>
#include <pthread.h>
#include <cstdlib>
#include <fstream>
#include "iString.h"
#include "paics_view.h"
#include "paics_view_tk.h"
#include "paics_view_cl.h"
#include "paics_view_gl.h"
#include "paics_view_global.h"

int           lock_molecule;
int           lock_trajectory;
pthread_t     pv_thread_id_mol;
pthread_t     pv_thread_id_trj;

extern string pv_path_babel;
extern string pv_path_gnuplot;
extern string pv_path_platex;
extern string pv_path_dvipdfm;
extern int    pv_thread_tk_on;
extern int    pv_thread_gl_on;
extern int    pv_thread_cl_on;
extern int    pv_debug;

// 20160827
namespace X11 {
#include <X11/Xlib.h>
}


void pv_sleep(const int t)
{
  usleep(t*1000);
}

void pv_lock_mol(const int t)
{
  pthread_t id=pthread_self();
  if(lock_molecule==0){
    lock_molecule=1;
    pv_thread_id_mol=id;
  } 
  else if(pv_thread_id_mol==id){
  }
  else {
    while(lock_molecule==1){
      pv_sleep(t);
    }
    lock_molecule=1;
  }
}

void pv_unlock_mol(int * chk)
{
  lock_molecule=0;
}

void pv_lock_trj(const int t)
{
  pthread_t id=pthread_self();
  if(lock_trajectory==0){
    lock_trajectory=1;
    pv_thread_id_mol=id;
  }
  else if(pv_thread_id_trj==id){
  }
  else {
    while(lock_trajectory==1){
      pv_sleep(t);
    }
    lock_trajectory=1;
  }
}

void pv_unlock_trj(int * chk)
{
  lock_trajectory=0;
}


void pv_check_command()
{
  int dummy;
  char line[1024];
  char buff[1024];
  string home,filename;
  dummy=system("rm -f $HOME/._pvcom");
  dummy=system("pvcom=`(which babel   > $HOME/._pvtmp) ; cat $HOME/._pvtmp`; echo \"babel   $pvcom\" >> $HOME/._pvcom");
  dummy=system("pvcom=`(which gnuplot > $HOME/._pvtmp) ; cat $HOME/._pvtmp`; echo \"gnuplot $pvcom\" >> $HOME/._pvcom");
  dummy=system("pvcom=`(which platex  > $HOME/._pvtmp) ; cat $HOME/._pvtmp`; echo \"platex  $pvcom\" >> $HOME/._pvcom");
  dummy=system("pvcom=`(which dvipdfm > $HOME/._pvtmp) ; cat $HOME/._pvtmp`; echo \"dvipdfm $pvcom\" >> $HOME/._pvcom");
  dummy=system("rm -f $HOME/._pvtmp");
  home=getenv("HOME");
  filename=home+"/._pvcom";
  fstream file(filename.c_str(), std::ios::in);
  while(! file.eof()){
    file.getline(line, sizeof(line));
    if(iPickUpNum(line)==2){
      iPickUp(line,0,buff);
      if(iFind(buff,"babel"  )>=0){iPickUp(line,1,buff);pv_path_babel   = buff;}
      if(iFind(buff,"gnuplot")>=0){iPickUp(line,1,buff);pv_path_gnuplot = buff;}
      if(iFind(buff,"plate"  )>=0){iPickUp(line,1,buff);pv_path_platex  = buff;}
      if(iFind(buff,"divpdfm")>=0){iPickUp(line,1,buff);pv_path_dvipdfm = buff;}
    }
  }
}

int main(int argc, char** argv)
{
  // 20160827
  X11::XInitThreads();

  pthread_t gl_handle, tk_handle, cl_handle;
  char * dummy = new char [1] ;

  lock_molecule=0;
  lock_trajectory=0;

  pv_os=1;
  //pv_check_command();
  
  if(argc>1){
    if(strcmp(argv[1],"-command")==0){
      if(argc==2){
        cout << "paics view: \"-commands\" option requires \"commands\"." << endl ;
      }
      else {
        pv_cl_read_command(argc, argv);
      }
      return 0;
    }
  }

  void * ( * pv_tk_main_pointer ) ( void * argv );
  pv_tk_main_pointer = pv_tk_main ;
  pthread_create( &tk_handle , NULL , pv_tk_main_pointer , dummy );
  pv_thread_tk_on=1;
  
  void * ( * pv_gl_main_pointer ) ( void * dummy );
  pv_gl_main_pointer = pv_gl_main ;
  pthread_create( &gl_handle , NULL , pv_gl_main_pointer , dummy );
  pv_thread_gl_on=1;

  void * ( * pv_cl_main_pointer ) ( void * dummy );
  pv_cl_main_pointer = pv_cl_main ;
  pthread_create( &cl_handle , NULL , pv_cl_main_pointer , dummy );
  pv_thread_cl_on=1;

  if(argc>1){
    if(strcmp(argv[1],"-start_command")==0){
      pv_cl_read_command(argc, argv);
    }
  }

  while(1){
    if(pv_exit==1){
      break;
    }
    pv_idle();
    pv_sleep(1);
  }
  
  delete [] dummy ;

  pthread_join( cl_handle , NULL );
  pthread_join( gl_handle , NULL );
  pthread_join( tk_handle , NULL );
  
  return 0;
}
