
#ifndef _ISHI_ATOM_H_
#define _ISHI_ATOM_H_

#include <string>
#include "Dvector3D.h"
#include "ViewAtom.h"
#include "AtomData.h"

using namespace std ;

namespace ishi {

class Atom{
  
 public:

  //-------------------//
  //--- constractor ---//
  //-------------------//

  Atom ( ) ;    
  Atom ( const Atom& a ) ;

  //------------------//
  //--- destractor ---//
  //------------------//
  
  ~Atom ( ) ;

  //------------//
  //--- copy ---//
  //------------//

  void copy ( const Atom& a ) ;

  //------------//
  //--- atom ---//
  //------------//

  void       clear_bond_atom     (                                  )       ;
  int        get_atomic_number   (                                  ) const ;
  double     get_mass_number     (                                  ) const ;
  double     get_bond_distance   (                                  ) const ;
  int        get_bond_atom_n     (                                  ) const ;
  int        get_bond_atom       ( const int iatom                  ,
                                   const int ierr=0                 ) const ;
  string     get_symbol          (                                  ) const ;
  int        get_chk_view        (                                  ) const ;
  string     get_basis           (                                  ) const ;
  int        get_opt             (                                  ) const ;
  int        get_md              (                                  ) const ;
  int        get_lnk1            (                                  ) const ;
  int        get_bio1            (                                  ) const ;
  int        get_bio2            (                                  ) const ;
  int        get_bio3            (                                  ) const ;
  int        get_bio4            (                                  ) const ;
  int        get_not_ca          (                                  ) const ;

  int        get_frg1            (                                  ) const ;
  int        get_frg2            (                                  ) const ;
  Dvector3D  get_R               (                                  ) const ;
  Dvector3D  get_V               (                                  ) const ;
  Dvector3D  get_G               (                                  ) const ;
  ViewAtom   get_va              (                                  ) const ;
  int        get_link_read_pdb   (                                  ) const ;
  string     get_atyp_read_pdb   (                                  ) const ;
  string     get_rtyp_read_pdb   (                                  ) const ;
  string     get_nseq_read_pdb   (                                  ) const ;
  string     get_chin_read_pdb   (                                  ) const ;
  string     get_mol2_1          (                                  ) const ;
  string     get_mol2_2          (                                  ) const ;
  string     get_mol2_3          (                                  ) const ;
  double     get_ad_charge       (                                  ) const ;
  //string     get_amber_ff_type   (                                  ) const ;
  //double     get_amber_ff_chg    (                                  ) const ;

  void       set_atomic_number   ( const int        n               ,
                                   const AtomData&  ad              )       ;
  void       set_atomic_number   ( const int        n               )       ;
  void       set_chk_view        ( const int        n               )       ;
  void       set_opt             ( const int        n_              )       ;
  void       set_md              ( const int        n_              )       ;
  void       set_lnk1            ( const int        n_              )       ;
  void       set_bio1            ( const int        n_              )       ;
  void       set_bio2            ( const int        n_              )       ;
  void       set_bio3            ( const int        n_              )       ;
  void       set_bio4            ( const int        n_              )       ;
  void       set_not_ca          ( const int        n_              )       ;
  void       set_frg1            ( const int        n_              )       ;
  void       set_frg2            ( const int        n_              )       ;
  void       set_basis           ( const string&    b               )       ;
  void       set_R               ( const Dvector3D& R_              )       ;
  void       set_V               ( const Dvector3D& V_              )       ;
  void       set_G               ( const Dvector3D& V_              )       ;
  void       add_V               ( const Dvector3D& V_              )       ;
  void       set_va              ( const ViewAtom&  v_              )       ;
  void       set_color           ( const double     R               ,
                                   const double     G               ,
                                   const double     B               )       ;
  void       set_color           (                                  )       ;
  void       set_color           ( const AtomData&  ad              )       ;
  void       set_text_color      ( const double     R               ,
                                   const double     G               ,
                                   const double     B               )       ;
  void       set_link_read_pdb   ( const int        il              )       ;
  void       set_atyp_read_pdb   ( const string     s_              )       ;
  void       set_rtyp_read_pdb   ( const string     s_              )       ;
  void       set_nseq_read_pdb   ( const string     s_              )       ;
  void       set_chin_read_pdb   ( const string     s_              )       ;
  void       set_mol2_1          ( const string     s_              )       ;
  void       set_mol2_2          ( const string     s_              )       ;
  void       set_mol2_3          ( const string     s_              )       ;
  void       push_back_bond_atom ( const int        iatom           ,
                                   const int        buff_increase=10)       ;
  void       pick_up_bond_atom   ( const int        iatom           )       ;
  void       set_ad_charge       ( const double     ch_             )       ;

  static int SymbolToNumber      ( const string     s               )       ;
  static int SymbolToNumber      ( const string     s               ,
                                   const AtomData&  ad              )       ;
  
 private:

  /*----------------------*/
  /*--- private member ---*/
  /*----------------------*/
  
  int        atomic_number    ;
  double     mass_number      ;
  string     symbol           ;
  double     bond_distance    ;
  int        chk_view         ; // -1 : view off , 1 : view on
  Dvector3D  R                ;
  Dvector3D  V                ;
  Dvector3D  G                ;
  ViewAtom   va               ;
  string     basis            ;
  int        opt              ; // 0 : fix , 1 : move
  int        md               ; // 0 : fix , 1 : move
  int        bond_atom_n      ;
  int        bond_atom_n_buff ;
  int      * bond_atom        ;
  int        lnk1             ; // linkgroup belonging to
  int        bio1             ; // type of atom
  int        bio2             ; // c-alpha atom belonging to
  int        bio3             ; // biounit belonging to
  int        bio4             ; // type of biounit belonging to
  int        not_ca           ; // 1 = not c-alpha , 0 = normal
  int        frg1             ; // fragment belonging to
  int        frg2             ; // bda or not (-1:not bda, 1:bda)
  int        link_read_pdb    ; // pdb information
  string     atyp_read_pdb    ; // pdb informatoin
  string     rtyp_read_pdb    ; // pdb information
  string     nseq_read_pdb    ; // pdb information
  string     chin_read_pdb    ; // pdb information
  string     mol2_1           ; // mol2 information (2nd colomun)
  string     mol2_2           ; // mol2 information (6th colomun, atom type)
  string     mol2_3           ; // mol2 information (7th colomun--end)
  double     ad_charge        ; // autodock-charge

  void create_bond_atom ( const int n ) ;
  void resize_bond_atom ( const int n ) ;
};

}

#endif
