
#include <cstdlib>
#include <iostream>
#include "Atom.h"
#include "AtomData.h"

extern int pv_debug; // for debug , 20160827

using namespace std;
using namespace ishi;

Atom::Atom()
{
  set_atomic_number(0);
  R.set_xyz(0.00,0.00,0.00);
  V.set_xyz(0.00,0.00,0.00);
  G.set_xyz(0.00,0.00,0.00);
  mass_number=0.0;
  bond_distance=0.0;
  bond_atom_n=0;
  bond_atom_n_buff=0;
  symbol="";
  chk_view=-1;
  basis="cc-pVDZso";
  opt=0;
  md=1;
  lnk1=-1;
  bio1=-1;
  bio2=-1;
  bio3=-1;
  bio4=-1;
  not_ca=0;
  frg1=-1;
  frg2=-1;
  link_read_pdb=-1;
  atyp_read_pdb="";
  rtyp_read_pdb="";
  nseq_read_pdb="";
  chin_read_pdb="";
  mol2_1="";
  mol2_2="";
  mol2_3="";
  ad_charge=0.0;
}

Atom::Atom(const Atom& a)
{
  set_atomic_number(0);
  R.set_xyz(0.00,0.00,0.00);
  V.set_xyz(0.00,0.00,0.00);
  G.set_xyz(0.00,0.00,0.00);
  mass_number=0.0;
  bond_distance=0.0;
  bond_atom_n=0;
  bond_atom_n_buff=0;
  symbol="";
  chk_view=-1;
  basis="cc-pVDZso";
  opt=0;
  md=1;
  lnk1=-1;
  bio1=-1;
  bio2=-1;
  bio3=-1;
  bio4=-1;
  not_ca= 0;
  frg1=-1;
  frg2=-1;
  link_read_pdb=-1;
  atyp_read_pdb="";
  rtyp_read_pdb="";
  nseq_read_pdb="";
  chin_read_pdb="";
  mol2_1="";
  mol2_2="";
  mol2_3="";
  ad_charge=0.0;
  copy(a);
}

Atom::~Atom()
{
  if(pv_debug==1){cout << "ishi in  ~Atom()" << endl ;}
  clear_bond_atom();
  if(pv_debug==1){cout << "ishi out ~Atom()" << endl ;}
}

void Atom::copy(const Atom& a)
{
  if(this==&a) return;
  atomic_number=a.atomic_number;
  mass_number=a.mass_number;
  bond_distance=a.bond_distance;
  symbol=a.symbol;
  chk_view=a.chk_view;
  basis=a.basis;
  opt=a.opt;
  md=a.md;
  lnk1=a.lnk1;
  bio1=a.bio1;
  bio2=a.bio2;
  bio3=a.bio3;
  bio4=a.bio4;
  not_ca=a.not_ca;
  frg1=a.frg1;
  frg2=a.frg2;
  link_read_pdb=a.link_read_pdb;
  atyp_read_pdb=a.atyp_read_pdb;
  rtyp_read_pdb=a.rtyp_read_pdb;
  nseq_read_pdb=a.nseq_read_pdb;
  chin_read_pdb=a.chin_read_pdb;
  mol2_1=a.mol2_1;
  mol2_2=a.mol2_2;
  mol2_3=a.mol2_3;
  ad_charge=a.ad_charge;
  create_bond_atom(a.bond_atom_n);
  for(int i=0;i<a.bond_atom_n;i++){
    bond_atom[i]=a.bond_atom[i];
    bond_atom_n++;
  }
  R.copy(a.R);
  V.copy(a.V);
  G.copy(a.G);
  va.copy(a.va);
}

void Atom::clear_bond_atom( )
{
  if(pv_debug==1){cout << "ishi in     clear_bond_atom()" << endl ;}
  if(bond_atom_n_buff==0) {
  if(pv_debug==1){cout << "ishi out 01 clear_bond_atom()" << endl ;}
    return ;
  }
  if(pv_debug==1){cout << "ishi 01     clear_bond_atom()" << endl ;}
  delete [] bond_atom ;
  if(pv_debug==1){cout << "ishi 02     clear_bond_atom()" << endl ;}
  bond_atom_n_buff=0;
  if(pv_debug==1){cout << "ishi 03     clear_bond_atom()" << endl ;}
  bond_atom_n=0;
  if(pv_debug==1){cout << "ishi out 02 clear_bond_atom()" << endl ;}
}

void Atom::create_bond_atom(const int n)
{
  clear_bond_atom();
  if(n==0) return ;
  bond_atom=new int [n];
  bond_atom_n_buff=n;
  bond_atom_n=0;
}

void Atom::resize_bond_atom(const int n)
{
  int bond_atom_n_;
  int * bond_atom_;
  bond_atom_n_ = bond_atom_n;
  bond_atom_ = new int [bond_atom_n+1];
  for(int i=0;i<bond_atom_n_;i++){
    bond_atom_[i]=bond_atom[i];
  }
  create_bond_atom(n);
  if(n>bond_atom_n_){
    for(int i=0;i<bond_atom_n_;i++){
      bond_atom[i]=bond_atom_[i];
      bond_atom_n++;
    }
  } else {
    for(int i=0;i<n;i++){
      bond_atom[i]=bond_atom_[i];
      bond_atom_n++;
    }
  }
  delete [] bond_atom_;
}

int Atom::SymbolToNumber(const string s)
{
  return AtomData::Default_atomic_number(s);
}

int Atom::SymbolToNumber(const string    s ,
                         const AtomData& ad)
{ 
  return ad.get_atomic_number(s);
}

int Atom::get_atomic_number ( ) const
{ 
  return atomic_number ;
}

double Atom::get_mass_number ( ) const
{
  return mass_number ;
}

double Atom::get_bond_distance ( ) const
{
  return bond_distance ;
}

string Atom::get_symbol ( ) const
{ 
  return symbol ;
}

int Atom::get_chk_view ( ) const
{ 
  return chk_view ;
}

string Atom::get_basis ( ) const
{
  return basis;
}

int Atom::get_opt ( ) const 
{
  return opt;
}

int Atom::get_md ( ) const 
{
  return md;
}

int Atom::get_lnk1 ( ) const 
{
  return lnk1;
}

int Atom::get_bio1 ( ) const 
{
  return bio1;
}

int Atom::get_bio2 ( ) const 
{
  return bio2;
}

int Atom::get_bio3 ( ) const 
{
  return bio3;
}

int Atom::get_bio4 ( ) const 
{
  return bio4;
}

int Atom::get_not_ca ( ) const 
{
  return not_ca;
}

int Atom::get_frg1 ( ) const 
{
  return frg1;
}

int Atom::get_frg2 ( ) const 
{
  return frg2;
}

Dvector3D Atom::get_R ( ) const
{
  return R ;
}

Dvector3D Atom::get_V ( ) const 
{ 
  return V ;
}

Dvector3D Atom::get_G ( ) const 
{ 
  return G ;
}

ViewAtom Atom::get_va ( ) const
{
  return va;
}

int Atom::get_bond_atom_n() const
{
  return bond_atom_n;
}

int Atom::get_bond_atom(const int iatom ,
                        const int ierr  ) const
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Atom::get_bond_atom , iatom << 0" << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=bond_atom_n){
    if(ierr==0 || ierr==1){
      cerr << "error in Atom::get_bond_atom , iatom >= bond_atom_n" << endl ;
      cerr << "iatom = " << iatom << " , bond_atom_n = " << bond_atom_n << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return bond_atom[iatom];
}

int Atom::get_link_read_pdb() const
{
  return link_read_pdb;
}

string Atom::get_atyp_read_pdb() const
{
  return atyp_read_pdb;
}

string Atom::get_rtyp_read_pdb() const
{
 return rtyp_read_pdb;
}

string Atom::get_nseq_read_pdb() const
{
  return nseq_read_pdb;
}

string Atom::get_chin_read_pdb() const
{
  return chin_read_pdb;
}

string Atom::get_mol2_1() const
{
  return mol2_1;
}

string Atom::get_mol2_2() const
{
  return mol2_2;
}

string Atom::get_mol2_3() const
{
  return mol2_3;
}

void Atom::set_atomic_number ( const int n)
{
  const double w_bond = 1.20/0.5291 ;
  va.set_color(n);
  atomic_number = n;
  mass_number   = AtomData::Default_mass_number(n);
  bond_distance = AtomData::Default_bond_distance(n)*w_bond;
  symbol        = AtomData::Default_symbol(n);
}

void Atom::set_atomic_number ( const int       n ,
                               const AtomData& ad)
{ 
  const double w_bond = 1.20/0.5291 ;
  va.set_color(n,ad);
  atomic_number = n;
  mass_number   = ad.get_mass_number(n);
  bond_distance = ad.get_bond_distance(n)*w_bond;
  symbol        = ad.get_symbol(n);
}

void Atom::set_chk_view ( const int n )
{
  chk_view=n;
}

void Atom::set_basis ( const string& b )
{
  basis=b;
}

void Atom::set_R ( const Dvector3D& R_ )
{ 
  R.copy(R_) ;
}

void Atom::set_V ( const Dvector3D& V_ )
{
  V.copy(V_);
}

void Atom::set_G ( const Dvector3D& G_ )
{
  G.copy(G_);
}

void Atom::add_V ( const Dvector3D& V_ )
{
  V.copy(V+V_);
}

void Atom::set_va ( const ViewAtom& va_ ) 
{
  va.copy(va_);
}

void Atom::set_opt ( const int n_ )
{
  opt=n_;
}

void Atom::set_md ( const int n_ )
{
  md=n_;
}

void Atom::set_lnk1 ( const int n_ )
{
  lnk1=n_;
}

void Atom::set_bio1 ( const int n_ )
{
  bio1=n_;
}

void Atom::set_bio2 ( const int n_ )
{
  bio2=n_;
}

void Atom::set_bio3 ( const int n_ )
{
  bio3=n_;
}

void Atom::set_bio4 ( const int n_ )
{
  bio4=n_;
}

void Atom::set_not_ca ( const int n_ )
{
  not_ca=n_;
}

void Atom::set_frg1 ( const int n_ )
{
  frg1=n_;
}

void Atom::set_frg2 ( const int n_ )
{
  frg2=n_;
}

void Atom::push_back_bond_atom ( const int iatom ,
                                 const int buff_increase )
{
  for(int i=0;i<bond_atom_n;i++){
    if(bond_atom[i]==iatom){
      return;
    }
  }
  if(bond_atom_n==bond_atom_n_buff){
    resize_bond_atom(bond_atom_n+buff_increase);
  }
  bond_atom[bond_atom_n]=iatom;
  bond_atom_n++;
}

void Atom::pick_up_bond_atom ( const int iatom )
{
  int chk=0;
  for(int i=0;i<bond_atom_n;i++){
    if(chk==1){
      bond_atom[i-1]=bond_atom[i];
    }
    if(bond_atom[i]==iatom){
      chk=1;
    }    
  }
  if(chk==1){
    bond_atom_n--;
  }
}

void Atom::set_color( const double R ,
                      const double G ,
                      const double B )
{
  va.set_color_R (R);
  va.set_color_G (G);
  va.set_color_B (B);
}

void Atom::set_color()
{
  va.set_color(atomic_number);
}

void Atom::set_color( const AtomData& ad )
{
  va.set_color(atomic_number,ad);
}

void Atom::set_text_color( const double R,
                           const double G,
                           const double B)
{
  va.set_text_color_R(R);
  va.set_text_color_G(G);
  va.set_text_color_B(B);
}


void Atom::set_link_read_pdb(const int il)
{
  link_read_pdb=il;
}

void Atom::set_atyp_read_pdb(const string s_)
{
  atyp_read_pdb=s_;
}

void Atom::set_rtyp_read_pdb(const string s_)
{
  rtyp_read_pdb=s_;
}

void Atom::set_nseq_read_pdb(const string s_)
{
  nseq_read_pdb=s_;
}

void Atom::set_chin_read_pdb(const string s_)
{
  chin_read_pdb=s_;
}

void Atom::set_mol2_1(const string s_)
{
  mol2_1=s_;
}

void Atom::set_mol2_2(const string s_)
{
  mol2_2=s_;
}

void Atom::set_mol2_3(const string s_)
{
  mol2_3=s_;
}


double Atom::get_ad_charge() const 
{
  return ad_charge;
}

void Atom::set_ad_charge(const double ch_)
{
  ad_charge=ch_;;
}
