
#include <iostream>
#include <cstdlib>
#include <algorithm>
#include "AtomData.h"

AtomData::AtomData()
{
  natom=0;
}

AtomData::AtomData(const int n)
{
  natom=0;
  create(n);
}

AtomData::AtomData(const AtomData& ad)
{
  natom=0;
  copy(ad);
}

AtomData::~AtomData()
{
  clear();
}

void AtomData::copy(const AtomData& ad)
{
  if(this==&ad) return;
  create(ad.natom);
  for(int i=0;i<natom;i++){
    symbol        [i] = ad.symbol        [i];
    bond_distance [i] = ad.bond_distance [i];
    mass_number   [i] = ad.mass_number   [i];
    color_R       [i] = ad.color_R       [i];
    color_G       [i] = ad.color_G       [i];
    color_B       [i] = ad.color_B       [i];
  }
}

void AtomData::clear()
{
  if(natom==0) return;
  delete [] symbol;
  delete [] bond_distance;
  delete [] mass_number;
  delete [] color_R;
  delete [] color_G;
  delete [] color_B;
  natom=0;
}

void AtomData::create(const int n)
{
  clear();
  if(n==0) return ;
  symbol        = new string [n];
  bond_distance = new double [n];
  mass_number   = new double [n];
  color_R       = new double [n];
  color_G       = new double [n];
  color_B       = new double [n];
  natom=n;
  for(int i=0;i<n;i++){
    symbol        [i] = AtomData::Default_symbol        (i);
    bond_distance [i] = AtomData::Default_bond_distance (i);
    mass_number   [i] = AtomData::Default_mass_number   (i);
    color_R       [i] = AtomData::Default_color_R       (i);
    color_G       [i] = AtomData::Default_color_G       (i);
    color_B       [i] = AtomData::Default_color_B       (i);
  }
}

string AtomData::get_symbol ( const int ia ) const 
{
  return symbol[ia];
}

double AtomData::get_bond_distance ( const int ia ) const
{
  return bond_distance[ia];
}

double AtomData::get_mass_number ( const int ia ) const
{
  return mass_number[ia];
}

double AtomData::get_color_R ( const int ia ) const
{
  return color_R[ia];
}

double AtomData::get_color_G ( const int ia ) const
{
  return color_G[ia];
}

double AtomData::get_color_B ( const int ia ) const
{
  return color_B[ia];
}

int AtomData::get_atomic_number( const string& as ) const 
{
  string stmp=as;
  std::transform(stmp.begin(),stmp.end(),stmp.begin(),::toupper);
  for(int i=0;i<natom;i++){
    if(symbol[i]==stmp){
      return i;
    }
  }
  cerr << "error : AtomData::Default_atomic_number : No defined symbol : symbol = " << as << endl ;
  exit(1);
}

void AtomData::read(const string& filename)
{
}

void AtomData::write(const string& filename)
{
}

string AtomData::Default_symbol(const int ia)
{
  switch(ia) {
  case   0 : return "X"  ;
  case   1 : return "H"  ;  case   2 : return "HE" ;  case   3 : return "Li" ;  case   4 : return "BE" ;  case   5 : return "B"  ;
  case   6 : return "C"  ;  case   7 : return "N"  ;  case   8 : return "O"  ;  case   9 : return "F"  ;  case  10 : return "NE" ;
  case  11 : return "NA" ;  case  12 : return "MG" ;  case  13 : return "AL" ;  case  14 : return "SI" ;  case  15 : return "P"  ;
  case  16 : return "S"  ;  case  17 : return "CL" ;  case  18 : return "AR" ;  case  19 : return "K"  ;  case  20 : return "CA" ;
  case  21 : return "SC" ;  case  22 : return "TI" ;  case  23 : return "V"  ;  case  24 : return "CR" ;  case  25 : return "MN" ;
  case  26 : return "FE" ;  case  27 : return "CO" ;  case  28 : return "NI" ;  case  29 : return "CU" ;  case  30 : return "ZN" ;
  case  31 : return "GA" ;  case  32 : return "GE" ;  case  33 : return "AS" ;  case  34 : return "SE" ;  case  35 : return "BR" ;
  case  36 : return "KR" ;  case  37 : return "RB" ;  case  38 : return "SR" ;  case  39 : return "Y"  ;  case  40 : return "ZR" ;
  case  41 : return "NB" ;  case  42 : return "MO" ;  case  43 : return "TC" ;  case  44 : return "RU" ;  case  45 : return "RH" ;
  case  46 : return "PD" ;  case  47 : return "AG" ;  case  48 : return "CD" ;  case  49 : return "IN" ;  case  50 : return "SN" ;
  case  51 : return "SB" ;  case  52 : return "TE" ;  case  53 : return "I"  ;  case  54 : return "XE" ;  case  55 : return "SC" ;
  case  56 : return "BA" ;  case  57 : return "LA" ;  case  58 : return "CE" ;  case  59 : return "PR" ;  case  60 : return "ND" ;
  case  61 : return "PM" ;  case  62 : return "SM" ;  case  63 : return "EU" ;  case  64 : return "GD" ;  case  65 : return "TB" ;
  case  66 : return "DY" ;  case  67 : return "HO" ;  case  68 : return "ER" ;  case  69 : return "Tm" ;  case  70 : return "YB" ;
  case  71 : return "LU" ;  case  72 : return "HF" ;  case  73 : return "TA" ;  case  74 : return "W"  ;  case  75 : return "RE" ;
  case  76 : return "OS" ;  case  77 : return "IR" ;  case  78 : return "PT" ;  case  79 : return "AU" ;  case  80 : return "HG" ;
  case  81 : return "TL" ;  case  82 : return "PB" ;  case  83 : return "BI" ;  case  84 : return "PO" ;  case  85 : return "AT" ;
  case  86 : return "RN" ;  case  87 : return "FR" ;  case  88 : return "RA" ;  case  89 : return "Ac" ;  case  90 : return "TH" ;
  case  91 : return "PA" ;  case  92 : return "U"  ;  case  93 : return "NP" ;  case  94 : return "PU" ;  case  95 : return "Am" ;
  case  96 : return "CM" ;  case  97 : return "BK" ;  case  98 : return "CF" ;  case  99 : return "ES" ;  case 100 : return "FM" ;
  case 101 : return "MD" ;  case 102 : return "NO" ;  case 103 : return "LR" ;  case 104 : return "RF" ;  case 105 : return "DD" ;
  case 106 : return "SG" ;  case 107 : return "BH" ;  case 108 : return "HS" ;  case 109 : return "MT" ;
  default  : 
    cerr << "error : AtomData::Default_symbol : No defined atom : atomic number = " << ia << endl ;
    exit(1);
  }
}

double AtomData::Default_bond_distance(const int ia)
{
  switch(ia) {
  case   0 : return  0.0000 ;
  case   1 : return  0.3500 ;
  case   2 : return  0.3500 ;
  case   3 : return  1.1500 ;
  case   4 : return  0.8870 ;
  case   5 : return  0.8220 ;
  case   6 : return  0.7720 ;
  case   7 : return  0.7500 ;
  case   8 : return  0.7300 ;
  case   9 : return  0.7200 ;
  case  10 : return  0.7100 ;
  case  11 : return  1.5390 * 0.6 ; 
  case  12 : return  0.9000 ; // 1.1730
  case  13 : return  1.1800 ;
  case  14 : return  1.1690 ;
  case  15 : return  1.1070 ;
  case  16 : return  1.0490 ;
  case  17 : return  0.9940 ;
  case  18 : return  0.9800 ;
  case  19 : return  1.2000 ;
  case  20 : return  1.7400 ;
  case  21 : return  1.4400 ;
  case  22 : return  1.3200 ;
  case  23 : return  1.2200 ;
  case  24 : return  1.1800 ;
  case  25 : return  1.1700 ;
  case  26 : return  1.1700 ;
  case  27 : return  1.1600 ;
  case  28 : return  1.1500 ;
  case  29 : return  1.1700 ;
  case  30 : return  1.2500 ;
  case  31 : return  1.2560 ;
  case  32 : return  1.2230 ;
  case  33 : return  1.1940 ;
  case  34 : return  1.1670 ;
  case  35 : return  1.1420 ;
  case  36 : return  1.8900 ;
  case  37 : return  1.8900 ;
  case  38 : return  1.8900 ;
  case  39 : return  1.8900 ;
  case  40 : return  1.8900 ;
  case  41 : return  1.8900 ;
  case  42 : return  1.8900 ;
  case  43 : return  1.8900 ;
  case  44 : return  1.8900 ;
  case  45 : return  1.8900 ;
  case  46 : return  1.8900 ;
  case  47 : return  1.8900 ;
  case  48 : return  1.8900 ;
  case  49 : return  1.8900 ;
  case  50 : return  1.8900 ;
  case  51 : return  1.8900 ;
  case  52 : return  1.8900 ;
  case  53 : return  1.8900 ;
  case  54 : return  1.8900 ;
  case  55 : return  1.8900 ;
  case  56 : return  1.8900 ;
  case  57 : return  1.8900 ;
  case  58 : return  1.8900 ;
  case  59 : return  1.8900 ;
  case  60 : return  1.8900 ;
  case  61 : return  1.8900 ;
  case  62 : return  1.8900 ;
  case  63 : return  1.8900 ;
  case  64 : return  1.8900 ;
  case  65 : return  1.8900 ;
  case  66 : return  1.8900 ;
  case  67 : return  1.8900 ;
  case  68 : return  1.8900 ;
  case  69 : return  1.8900 ;
  case  70 : return  1.8900 ;
  case  71 : return  1.8900 ;
  case  72 : return  1.8900 ;
  case  73 : return  1.8900 ;
  case  74 : return  1.8900 ;
  case  75 : return  1.8900 ;
  case  76 : return  1.8900 ;
  case  77 : return  1.8900 ;
  case  78 : return  1.5000 ;
  case  79 : return  1.5000 ;
  case  80 : return  1.5000 ;
  case  81 : return  1.5000 ;
  case  82 : return  1.5000 ;
  case  83 : return  1.5000 ;
  case  84 : return  1.5000 ;
  case  85 : return  1.5000 ;
  case  86 : return  1.5000 ;
  case  87 : return  1.5000 ;
  case  88 : return  1.5000 ;
  case  89 : return  1.5000 ;
  case  90 : return  1.5000 ;
  case  91 : return  1.5000 ;
  case  92 : return  1.5000 ;
  case  93 : return  1.5000 ;
  case  94 : return  1.5000 ;
  case  95 : return  1.5000 ;
  case  96 : return  1.5000 ;
  case  97 : return  1.5000 ;
  case  98 : return  1.5000 ;
  case  99 : return  1.5000 ;
  case 100 : return  1.5000 ;
  case 101 : return  1.5000 ;
  case 102 : return  1.5000 ;
  case 103 : return  1.5000 ;
  case 104 : return  1.5000 ;
  case 105 : return  1.5000 ;
  case 106 : return  1.5000 ;
  case 107 : return  1.5000 ;
  case 108 : return  1.5000 ;
  case 109 : return  1.5000 ;
  default  :
    cerr << "error : AtomData::Default_bond_distance : No defined atom : atomic number = " << ia << endl ;
    exit(1);
  }
}

double AtomData::Default_mass_number(const int ia)
{
  switch(ia) {
  case   0 : return   0.000000 ;
  case   1 : return   1.008000 ;
  case   2 : return   4.003000 ;
  case   3 : return   6.941000 ;
  case   4 : return   9.012100 ;
  case   5 : return  10.811000 ;
  case   6 : return  12.000000 ;
  case   7 : return  14.010000 ;
  case   8 : return  16.000000 ;
  case   9 : return  19.000000 ;
  case  10 : return  20.180000 ;
  case  11 : return  22.990000 ;
  case  12 : return  24.310000 ;
  case  13 : return  26.980000 ;
  case  14 : return  28.090000 ;
  case  15 : return  30.970000 ;
  case  16 : return  32.070000 ;
  case  17 : return  35.450000 ;
  case  18 : return  39.948000 ;
  case  19 : return  39.098300 ;
  case  20 : return  40.078300 ;
  case  21 : return  44.955910 ;
  case  22 : return  47.867000 ;
  case  23 : return  50.941500 ;
  case  24 : return  51.996100 ;
  case  25 : return  54.938045 ;
  case  26 : return  55.845000 ;
  case  27 : return  58.933195 ;
  case  28 : return  58.693400 ;
  case  29 : return  63.546000 ;
  case  30 : return  65.380000 ;
  case  31 : return  69.723000 ;
  case  32 : return  72.460000 ;
  case  33 : return  74.921600 ;
  case  34 : return  78.960000 ;
  case  35 : return  79.904000 ;
  case  36 : return  83.798000 ;
  case  44 : return 101.070000 ;
  case  47 : return 107.868200 ;
  case  53 : return 126.904470 ;
  case  55 : return 132.905452 ;
  case  78 : return 195.100000 ;
  case  79 : return 196.960000 ;
  case  80 : return 200.590000 ;
  case  81 : return 204.380000 ;
  default  :
    cerr << "error : AtomData::Default_mass_number : No defined atom : atomic number = " << ia << endl ;
    exit(1);
  }
}

double AtomData::Default_color_R(const int ia)
{
  switch(ia) {
  case   0: return 0.25;
  case   1: return 1.00;  case   2: return 0.85;  case   3: return 0.80;  case   4: return 0.76;  case   5: return 1.00;
  case   6: return 0.56;  case   7: return 0.19;  case   8: return 1.00;  case   9: return 0.56;  case  10: return 0.70;
  case  11: return 0.67;  case  12: return 0.54;  case  13: return 0.75;  case  14: return 0.94;  case  15: return 1.00;
  case  16: return 1.00;  case  17: return 0.12;  case  18: return 0.50;  case  19: return 0.56;  case  20: return 0.24;
  case  21: return 0.90;  case  22: return 0.75;  case  23: return 0.65;  case  24: return 0.54;  case  25: return 0.61;
  case  26: return 0.88;  case  27: return 0.94;  case  28: return 0.31;  case  29: return 0.78;  case  30: return 0.49;
  case  31: return 0.76;  case  32: return 0.40;  case  33: return 0.74;  case  34: return 1.00;  case  35: return 0.65;
  case  36: return 0.36;  case  37: return 0.44;  case  38: return 0.00;  case  39: return 0.58;  case  40: return 0.58;
  case  41: return 0.45;  case  42: return 0.33;  case  43: return 0.23;  case  44: return 0.14;  case  45: return 0.04;
  case  46: return 0.00;  case  47: return 0.75;  case  48: return 1.00;  case  49: return 0.65;  case  50: return 0.40;
  case  51: return 0.62;  case  52: return 0.83;  case  53: return 0.58;  case  54: return 0.26;  case  55: return 0.34;
  case  56: return 0.00;  case  57: return 0.44;  case  58: return 1.00;  case  59: return 0.85;  case  60: return 0.78;
  case  61: return 0.64;  case  62: return 0.56;  case  63: return 0.38;  case  64: return 0.27;  case  65: return 0.19;
  case  66: return 0.12;  case  67: return 0.00;  case  68: return 0.00;  case  69: return 0.00;  case  70: return 0.00;
  case  71: return 0.00;  case  72: return 0.30;  case  73: return 0.30;  case  74: return 0.13;  case  75: return 0.15;
  case  76: return 0.15;  case  77: return 0.09;  case  78: return 0.82;  case  79: return 1.00;  case  80: return 0.72;
  case  81: return 0.65;  case  82: return 0.34;  case  83: return 0.62;  case  84: return 0.67;  case  85: return 0.46;
  case  86: return 0.26;  case  87: return 0.26;  case  88: return 0.00;  case  89: return 0.44;  case  90: return 0.00;
  case  91: return 0.00;  case  92: return 0.00;  case  93: return 0.00;  case  94: return 0.00;  case  95: return 0.33;
  case  96: return 0.47;  case  97: return 0.54;  case  98: return 0.63;  case  99: return 0.70;  case 100: return 0.70;
  case 101: return 0.70;  case 102: return 0.74;  case 103: return 0.78;  case 104: return 0.80;  case 105: return 0.82;
  case 106: return 0.85;  case 107: return 0.88;  case 108: return 0.90;  case 109: return 0.92;
  default : return 1.00;
  }
}

double AtomData::Default_color_G(const int ia)
{
  switch(ia) {
  case   0: return 0.25;
  case   1: return 1.00;  case   2: return 1.00;  case   3: return 0.50;  case   4: return 1.00;  case   5: return 0.71;
  case   6: return 0.56;  case   7: return 0.31;  case   8: return 0.05;  case   9: return 0.88;  case  10: return 0.89;
  case  11: return 0.36;  case  12: return 1.00;  case  13: return 0.65;  case  14: return 0.78;  case  15: return 0.50;
  case  16: return 1.00;  case  17: return 0.94;  case  18: return 0.82;  case  19: return 0.25;  case  20: return 1.00;
  case  21: return 0.90;  case  22: return 0.76;  case  23: return 0.65;  case  24: return 0.60;  case  25: return 0.48;
  case  26: return 0.40;  case  27: return 0.56;  case  28: return 0.82;  case  29: return 0.50;  case  30: return 0.50;
  case  31: return 0.56;  case  32: return 0.56;  case  33: return 0.50;  case  34: return 0.63;  case  35: return 0.16;
  case  36: return 0.72;  case  37: return 0.18;  case  38: return 1.00;  case  39: return 1.00;  case  40: return 0.88;
  case  41: return 0.76;  case  42: return 0.71;  case  43: return 0.62;  case  44: return 0.56;  case  45: return 0.49;
  case  46: return 0.41;  case  47: return 0.75;  case  48: return 0.85;  case  49: return 0.46;  case  50: return 0.50;
  case  51: return 0.39;  case  52: return 0.48;  case  53: return 0.00;  case  54: return 0.62;  case  55: return 0.09;
  case  56: return 0.79;  case  57: return 0.83;  case  58: return 1.00;  case  59: return 1.00;  case  60: return 1.00;
  case  61: return 1.00;  case  62: return 1.00;  case  63: return 1.00;  case  64: return 1.00;  case  65: return 1.00;
  case  66: return 1.00;  case  67: return 1.00;  case  68: return 0.90;  case  69: return 0.83;  case  70: return 0.75;
  case  71: return 0.67;  case  72: return 0.76;  case  73: return 0.65;  case  74: return 0.58;  case  75: return 0.49;
  case  76: return 0.40;  case  77: return 0.33;  case  78: return 0.82;  case  79: return 0.82;  case  80: return 0.72;
  case  81: return 0.33;  case  82: return 0.35;  case  83: return 0.31;  case  84: return 0.36;  case  85: return 0.31;
  case  86: return 0.51;  case  87: return 0.00;  case  88: return 0.49;  case  89: return 0.67;  case  90: return 0.73;
  case  91: return 0.63;  case  92: return 0.56;  case  93: return 0.50;  case  94: return 0.42;  case  95: return 0.36;
  case  96: return 0.36;  case  97: return 0.31;  case  98: return 0.21;  case  99: return 0.12;  case 100: return 0.12;
  case 101: return 0.05;  case 102: return 0.05;  case 103: return 0.00;  case 104: return 0.00;  case 105: return 0.00;
  case 106: return 0.00;  case 107: return 0.00;  case 108: return 0.00;  case 109: return 0.00;
  default : return 1.00;
  }
}
double AtomData::Default_color_B(const int ia)
{
  switch(ia) {
  case   0: return 0.25;
  case   1: return 1.00;  case   2: return 1.00;  case   3: return 1.00;  case   4: return 0.00;  case   5: return 0.71;
  case   6: return 0.56;  case   7: return 0.97;  case   8: return 0.05;  case   9: return 0.31;  case  10: return 0.96;
  case  11: return 0.95;  case  12: return 0.00;  case  13: return 0.65;  case  14: return 0.63;  case  15: return 0.00;
  case  16: return 0.19;  case  17: return 0.12;  case  18: return 0.89;  case  19: return 0.83;  case  20: return 0.00;
  case  21: return 0.90;  case  22: return 0.78;  case  23: return 0.67;  case  24: return 0.78;  case  25: return 0.78;
  case  26: return 0.20;  case  27: return 0.63;  case  28: return 0.31;  case  29: return 0.20;  case  30: return 0.69;
  case  31: return 0.56;  case  32: return 0.56;  case  33: return 0.89;  case  34: return 0.00;  case  35: return 0.16;
  case  36: return 0.82;  case  37: return 0.69;  case  38: return 0.00;  case  39: return 1.00;  case  40: return 0.88;
  case  41: return 0.79;  case  42: return 0.71;  case  43: return 0.62;  case  44: return 0.56;  case  45: return 0.55;
  case  46: return 0.52;  case  47: return 0.75;  case  48: return 0.56;  case  49: return 0.45;  case  50: return 0.50;
  case  51: return 0.71;  case  52: return 0.00;  case  53: return 0.58;  case  54: return 0.69;  case  55: return 0.56;
  case  56: return 0.00;  case  57: return 1.00;  case  58: return 0.78;  case  59: return 0.78;  case  60: return 0.78;
  case  61: return 0.78;  case  62: return 0.78;  case  63: return 0.78;  case  64: return 0.78;  case  65: return 0.78;
  case  66: return 0.78;  case  67: return 0.61;  case  68: return 0.46;  case  69: return 0.32;  case  70: return 0.22;
  case  71: return 0.14;  case  72: return 1.00;  case  73: return 1.00;  case  74: return 0.84;  case  75: return 0.67;
  case  76: return 0.59;  case  77: return 0.53;  case  78: return 0.88;  case  79: return 0.14;  case  80: return 0.82;
  case  81: return 0.30;  case  82: return 0.38;  case  83: return 0.71;  case  84: return 0.00;  case  85: return 0.27;
  case  86: return 0.59;  case  87: return 0.40;  case  88: return 0.00;  case  89: return 0.98;  case  90: return 1.00;
  case  91: return 1.00;  case  92: return 1.00;  case  93: return 1.00;  case  94: return 1.00;  case  95: return 0.95;
  case  96: return 0.89;  case  97: return 0.89;  case  98: return 0.83;  case  99: return 0.83;  case 100: return 0.73;
  case 101: return 0.65;  case 102: return 0.53;  case 103: return 0.40;  case 104: return 0.35;  case 105: return 0.31;
  case 106: return 0.27;  case 107: return 0.22;  case 108: return 0.18;  case 109: return 0.15;
  default : return 1.00;
  }
}

int AtomData::Default_atomic_number ( const string& as )
{
  string s=as;
  std::transform(s.begin(),s.end(),s.begin(),::toupper);
  if(0){}
  else if(s=="X" ){return   0;} 
  else if(s=="H" ){return   1;} else if(s=="HE"){return   2;} else if(s=="LI"){return   3;} else if(s=="BE"){return   4;} else if(s=="B" ){return   5;}
  else if(s=="C" ){return   6;} else if(s=="N" ){return   7;} else if(s=="O" ){return   8;} else if(s=="F" ){return   9;} else if(s=="NE"){return  10;}
  else if(s=="NA"){return  11;} else if(s=="MG"){return  12;} else if(s=="AL"){return  13;} else if(s=="SI"){return  14;} else if(s=="P" ){return  15;}
  else if(s=="S" ){return  16;} else if(s=="CL"){return  17;} else if(s=="AR"){return  18;} else if(s=="K" ){return  19;} else if(s=="CA"){return  20;}
  else if(s=="SC"){return  21;} else if(s=="TI"){return  22;} else if(s=="V" ){return  23;} else if(s=="CR"){return  24;} else if(s=="MN"){return  25;}
  else if(s=="FE"){return  26;} else if(s=="CO"){return  27;} else if(s=="NI"){return  28;} else if(s=="CU"){return  29;} else if(s=="ZN"){return  30;}
  else if(s=="GA"){return  31;} else if(s=="GE"){return  32;} else if(s=="AS"){return  33;} else if(s=="SE"){return  34;} else if(s=="BR"){return  35;}
  else if(s=="KR"){return  36;} else if(s=="RB"){return  37;} else if(s=="SR"){return  38;} else if(s=="Y" ){return  39;} else if(s=="ZR"){return  40;}
  else if(s=="NB"){return  41;} else if(s=="MO"){return  42;} else if(s=="TC"){return  43;} else if(s=="RU"){return  44;} else if(s=="RH"){return  45;}
  else if(s=="PD"){return  46;} else if(s=="AG"){return  47;} else if(s=="CD"){return  48;} else if(s=="IN"){return  49;} else if(s=="SN"){return  50;}
  else if(s=="SB"){return  51;} else if(s=="TE"){return  52;} else if(s=="I" ){return  53;} else if(s=="XE"){return  54;} else if(s=="SC"){return  55;}
  else if(s=="BA"){return  56;} else if(s=="LA"){return  57;} else if(s=="CE"){return  58;} else if(s=="PR"){return  59;} else if(s=="ND"){return  60;}
  else if(s=="PM"){return  61;} else if(s=="SM"){return  62;} else if(s=="EU"){return  63;} else if(s=="GD"){return  64;} else if(s=="TB"){return  65;}
  else if(s=="DY"){return  66;} else if(s=="HO"){return  67;} else if(s=="ER"){return  68;} else if(s=="TM"){return  69;} else if(s=="YB"){return  70;}
  else if(s=="LU"){return  71;} else if(s=="HF"){return  72;} else if(s=="TA"){return  73;} else if(s=="W" ){return  74;} else if(s=="RE"){return  75;}
  else if(s=="OS"){return  76;} else if(s=="IR"){return  77;} else if(s=="PT"){return  78;} else if(s=="AU"){return  79;} else if(s=="HG"){return  80;}
  else if(s=="HG"){return  81;} else if(s=="PB"){return  82;} else if(s=="BI"){return  83;} else if(s=="PO"){return  84;} else if(s=="AT"){return  85;}
  else if(s=="RN"){return  86;} else if(s=="FR"){return  87;} else if(s=="RA"){return  88;} else if(s=="AC"){return  89;} else if(s=="TH"){return  90;}
  else if(s=="PA"){return  91;} else if(s=="U" ){return  92;} else if(s=="NP"){return  93;} else if(s=="PU"){return  94;} else if(s=="AM"){return  95;}
  else if(s=="CM"){return  96;} else if(s=="BK"){return  97;} else if(s=="CF"){return  98;} else if(s=="ES"){return  99;} else if(s=="FM"){return 100;}
  else if(s=="MD"){return 101;} else if(s=="NO"){return 102;} else if(s=="LR"){return 103;} else if(s=="RF"){return 104;} else if(s=="DB"){return 105;}
  else if(s=="SG"){return 106;} else if(s=="BH"){return 107;} else if(s=="HS"){return 108;} else if(s=="MT"){return 109;}
  else {
    cerr << "error : AtomData::Default_atomic_number : No defined symbol : symbol = " << as << endl ;
    exit(1);
  }
}
