 
#ifndef _ISHI_BIOUNIT_H_
#define _ISHI_BIOUNIT_H_

#include <string>
#include "Dvector3D.h"

using namespace std;

class BioUnit{

 public:

  BioUnit                     (                                   )       ;
  BioUnit                     ( const BioUnit&   bu               )       ;
  
  ~BioUnit                    (                                   )       ;
  
  void   copy                 ( const BioUnit&   bu               )       ;
  void   clear                (                                   )       ;

  int    get_type             (                                   ) const ;
  int    get_seq              (                                   ) const ;
  int    get_chk_view         (                                   ) const ;
  int    get_n_atom_index     (                                   ) const ;
  int    get_atom_index       ( const int        iatom_in_bio     ) const ;
  int    get_atom_type_2      ( const int        iatom_in_mol     ) const ; 
  int    get_atom_type        ( const int        iatom_in_bio     ) const ;
  void   set_type             ( const int        type_            )       ;
  void   set_seq              ( const int        seq_             )       ;
  void   set_chk_view         ( const int        n                )       ;
  void   set_ss_bond          ( const int        ibu              )       ;
  void   push_back_atom_index ( const int        atom_index_      ,
                                const int        atom_type_       ,
                                const int        buff_increase=10 )       ;
  void   index_order          (                                   )       ;
  
  /* peptide */
  int    get_ss_bond          (                                   ) const ;
  int    get_terminal_c       (                                   ) const ;
  int    get_terminal_n       (                                   ) const ;
  int    get_link_unit_c      (                                   ) const ;
  int    get_link_unit_n      (                                   ) const ;
  string get_abbr_1           (                                   ) const ;
  string get_abbr_3           (                                   ) const ;
  int    get_c0               (                                   ) const ; // Ca
  int    get_c1               (                                   ) const ; // Ca-C
  int    get_n2               (                                   ) const ; // Ca-N
  int    get_h3               (                                   ) const ; // Ca-H
  int    get_o4               (                                   ) const ; // Ca-C=0
  int    get_h5               (                                   ) const ; // Ca-N-H
  int    get_h1814            (                                   ) const ; // NE1 in HIS
  int    get_h1816            (                                   ) const ; // HE2 in HIS
  void   set_terminal_c       ( const int        type_            )       ;
  void   set_terminal_n       ( const int        type_            )       ;
  void   set_link_unit_c      ( const int        ibu              )       ;
  void   set_link_unit_n      ( const int        ibu              )       ;
  void   set_abbr_1           ( const string&    s                )       ;
  void   set_abbr_3           ( const string&    s                )       ;

  /* dna deoxyribose */
  int    get_terminal_5       (                                   ) const ;
  int    get_terminal_3       (                                   ) const ;
  int    get_link_unit_5      (                                   ) const ;
  int    get_link_unit_3      (                                   ) const ;
  int    get_link_unit_b      (                                   ) const ;
  int    get_c20              (                                   ) const ; // 5'C
  int    get_o21              (                                   ) const ;
  int    get_c22              (                                   ) const ; // 4'C
  int    get_c23              (                                   ) const ; // 3'C
  int    get_c24              (                                   ) const ; // 2'C
  int    get_c25              (                                   ) const ; // 1'C
  int    get_p27              (                                   ) const ; // P
  int    get_h28              (                                   ) const ; // 5'C-term.-H
  int    get_o29              (                                   ) const ; // 3'C-term.-O
  int    get_h30              (                                   ) const ; // 3'C-term.-H
  int    get_o31              (                                   ) const ;
  void   set_terminal_5       ( const int        type_            )       ;
  void   set_terminal_3       ( const int        type_            )       ;
  void   set_link_unit_5      ( const int        ibu              )       ;
  void   set_link_unit_3      ( const int        ibu              )       ;
  void   set_link_unit_b      ( const int        ibu              )       ;
  
  /* dna base */
  int    get_n26              (                                   ) const ; // --N
  int    get_link_unit_d      (                                   ) const ;
  void   set_link_unit_d      ( const int        ibu              )       ;

  /* pdb tag */
  static string res_abbr_3    ( const int        index            )       ;
  static string res_abbr_1    ( const int        index            )       ;
  static string pdb_tag       ( const int        atom_type        ,
                                const int        info01=-1        )       ;
  static int    atm_type      ( const string     pdb_tag          ,
                                const string     res_typ          )       ;
  static int    res_type      ( const string     res_typ          ,
                                string *         out=NULL         )       ;

  static int    bnd_type_rotatable ( const int index1,
				     const int index2,
                                     const int bio_type_1,
                                     const int bio_type_2) ;

 private:

  int    type;
  int    n_atom_index;
  int    n_atom_index_buff;
  int*   atom_index;
  int*   atom_type;
  int    seq;
  int    chk_view;
  string abbr_1;
  string abbr_3;

  /* peptide */
  int    ss_bond;     // -1: none
  int    terminal_c;  // -1: none , 1:C-term(0) , 2:C-term(-1) , 6:C-term(-CNCH3)
  int    terminal_n;  // -1: none , 1:N-term(0) , 2:N-term(+1) , 6:N-term(-COCH3)
  int    link_unit_c;
  int    link_unit_n;

  /* dna deoxyribose */
  int    terminal_5; // -1: none , 1:5-term(-OH)
  int    terminal_3; // -1: none , 1:3-term(-OH)
  int    link_unit_5;
  int    link_unit_3;
  int    link_unit_b;

  /* dna base */
  int    link_unit_d;

  void clear_atom_index  ( );
  void create_atom_index ( const int n );
  void resize_atom_index ( const int n );

  int static d4_(const string a1,
		 const string a2,
		 const string a3,
		 const string a4,
		 const string b1,
		 const string b2,
		 const string b3,
		 const string b4,
                 const int    improper=0);
};

#endif
