
#include <iostream>
#include <cstdlib>
#include "BioUnit.h"
#include "BioUnitData.h"
#include "Dvector3D.h"
#include "iString.h"

using namespace std;

BioUnit::BioUnit()
{
  type=-1;
  n_atom_index=0;
  n_atom_index_buff=0 ;
  seq=-1;
  chk_view=-1;

  /* peptide */
  ss_bond=-1;
  terminal_c=-1;
  terminal_n=-1;
  link_unit_n=-1;
  link_unit_c=-1;
  abbr_1="";
  abbr_3="";

  /* dna deoxyribose */
  terminal_5=-1;
  terminal_3=-1;
  link_unit_5=-1;
  link_unit_3=-1;
  link_unit_b=-1;
  
  /* dna base */
  link_unit_d=-1;
}

BioUnit::BioUnit(const BioUnit& bu)
{
  type=-1;
  n_atom_index=0;
  n_atom_index_buff=0;
  seq=-1;
  chk_view=-1;

  /* peptide */
  ss_bond=-1;
  terminal_c=-1;
  terminal_n=-1;
  link_unit_n=-1;
  link_unit_c=-1;
  abbr_1="";
  abbr_3="";

  /* dna */
  terminal_5=-1;
  terminal_3=-1;
  link_unit_5=-1;
  link_unit_3=-1;
  link_unit_b=-1;
  
  /* dna base */
  link_unit_d=-1;

  copy(bu);
}

BioUnit::~BioUnit()
{
  clear();
}

void BioUnit::copy(const BioUnit& bu)
{
  if(this==&bu) return;
  create_atom_index(bu.n_atom_index_buff);

  type=bu.type;
  seq=bu.seq;
  chk_view=bu.chk_view;
  for(int i=0;i<bu.n_atom_index;i++){
    atom_index[i]=bu.atom_index[i];
    atom_type [i]=bu.atom_type [i];
    n_atom_index++;
  }

  // peptide
  ss_bond=bu.ss_bond;
  terminal_c=bu.terminal_c;
  terminal_n=bu.terminal_n;
  link_unit_n=bu.link_unit_n;
  link_unit_c=bu.link_unit_c;
  abbr_1=bu.abbr_1;
  abbr_3=bu.abbr_3;

  // dna
  terminal_5=bu.terminal_5;
  terminal_3=bu.terminal_3;
  link_unit_5=bu.link_unit_5;
  link_unit_3=bu.link_unit_3;
  link_unit_b=bu.link_unit_b;
  
  // dna base
  link_unit_d=bu.link_unit_d;
}

void BioUnit::clear()
{
  type=-1;
  seq=-1;
  chk_view=-1;
  clear_atom_index();

  // peptide
  ss_bond=-1;
  terminal_c=-1;
  terminal_n=-1;
  link_unit_n=-1;
  link_unit_c=-1;
  abbr_1="";
  abbr_3="";

  // dna deoxyribose
  terminal_5=-1;
  terminal_3=-1;
  link_unit_5=-1;
  link_unit_3=-1;
  link_unit_b=-1;
  
  // dna base
  link_unit_d=-1;
}

void BioUnit::clear_atom_index()
{
  if(n_atom_index_buff==0) return ;
  delete [] atom_index ;
  delete [] atom_type  ;
  n_atom_index_buff=0;
  n_atom_index=0;
}

void BioUnit::create_atom_index(const int n)
{
  clear_atom_index();
  if(n==0) return;
  atom_index = new int [n] ;
  atom_type  = new int [n] ;
  n_atom_index_buff=n;
  n_atom_index=0;
}

void BioUnit::resize_atom_index(const int n)
{
  int     natom_buf ;
  int*    atom_index_buf ;
  int*    atom_type_buf;
  natom_buf=n_atom_index;
  atom_index_buf = new int [n_atom_index+1];
  atom_type_buf  = new int [n_atom_index+1];
  for(int i=0;i<n_atom_index;i++){
    atom_index_buf[i]=atom_index[i];
    atom_type_buf [i]=atom_type [i];
  }
  create_atom_index(n);
  if(n>natom_buf){
    for(int i=0;i<natom_buf;i++){
      atom_index[i]=atom_index_buf[i];
      atom_type [i]=atom_type_buf [i];
      n_atom_index++;
    }
  } else {
    for(int i=0;i<n;i++){
      atom_index[i]=atom_index_buf[i];
      atom_type [i]=atom_type_buf [i];
      n_atom_index++;
    }
  }
  delete [] atom_index_buf;
  delete [] atom_type_buf;
}

int BioUnit::get_type ( ) const
{
  return type;
}

int BioUnit::get_n_atom_index ( ) const
{
  return n_atom_index;
}

int BioUnit::get_atom_index ( const int iatom ) const
{
  if(iatom<0){
    cerr << "error in BioUnit::get_atom_index , iatom < 0" << endl ;
    cerr << "iatom = " << iatom << endl ;
    exit(1);
  }
  if(iatom>=n_atom_index){
    cerr << "error in BioUnit::get_atom_index , iatom >= n_atom_index" << endl ;
    cerr << "iatom = " << iatom << " , n_atom_index = " << n_atom_index << endl ;
    exit(1);
  }
  return atom_index[iatom];
}

void BioUnit::push_back_atom_index ( const int atom_index_ , const int atom_type_, const int buff_increase )
{
  if(n_atom_index==n_atom_index_buff){
    resize_atom_index(n_atom_index+buff_increase);
  }
  atom_index[n_atom_index]=atom_index_ ;
  atom_type [n_atom_index]=atom_type_  ;
  n_atom_index++;
}

int BioUnit::get_seq( ) const
{
  return seq;
}

int BioUnit::get_chk_view( ) const
{
  return chk_view;
}

int BioUnit::get_atom_type ( const int iatom ) const 
{
  // iatom ---> number in the biounit
  if(iatom<0){
    cerr << "error in BioUnit::get_atom_type , iatom < 0" << endl ;
    cerr << "iatom = " << iatom << endl ;
    exit(1);
  }
  if(iatom>=n_atom_index){
    cerr << "error in BioUnit::get_atom_type , iatom >= n_atom_index" << endl ;
    cerr << "iatom = " << iatom << " , n_atom_index = " << n_atom_index << endl ;
    exit(1);
  }
  return atom_type[iatom];
}

int BioUnit::get_atom_type_2 ( const int iatom ) const 
{
  // iatom ---> number in the Molecule
  for(int i=0;i<n_atom_index;i++){
    if(atom_index[i]==iatom){
      return atom_type[i];
    }
  }
  return -1;
}

void BioUnit::set_type ( const int type_ )
{
  type=type_;
}

void BioUnit::set_seq ( const int seq_ )
{
  seq=seq_;
}

void BioUnit::set_chk_view ( const int n )

{
  chk_view=n;
}

//---------//
// peptide //
//---------//

int BioUnit::get_terminal_c ( ) const
{
  return terminal_c;
}

int BioUnit::get_terminal_n ( ) const
{
  return terminal_n;
}

int BioUnit::get_ss_bond ( ) const
{
  return ss_bond;
}


int BioUnit::get_link_unit_c() const
{
  return link_unit_c;
}

int BioUnit::get_link_unit_n() const
{
  return link_unit_n;
}

string BioUnit::get_abbr_1 ( ) const
{
  return abbr_1;
}

string BioUnit::get_abbr_3 ( ) const
{
  return abbr_3;
}

int BioUnit::get_c0 ( ) const 
{
  // Ca
  if((100<=type && type<=2100) || type==-999){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==0){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_c1 ( ) const
{
  // Ca-C
  if((100<=type && type<=2100) || type==-999){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==1){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_n2 ( ) const
{
  // Ca-N
  if((100<=type && type<=2100) || type==-999){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==2){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_h3 ( ) const
{
  // Ca-H
  if((100<=type && type<=2100) || type==-999){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==3){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_o4 ( ) const
{
  // Ca-C=O
  if((100<=type && type<=2100) || type==-999){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==4){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_h5 ( ) const
{
  // Ca-N-H
  if((100<=type && type<=2100) || type==-999){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==5){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_h1814 ( ) const
{
  // HD1 in HIS
  //if(type==1800 || type==1850){ // <-- ishiishi
  if(type==1800){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==1814){
        return i;
      }
    }
  }
  return -1;
}
  
int BioUnit::get_h1816 ( ) const
{
  // HE2 in HIS
  //if(type==1800 || type==1850){ // <-- ishiishi
  if(type==1800){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==1816){
        return i;
      }
    }
  }
  return -1;
}

void BioUnit::set_terminal_c ( const int type_ )
{
  terminal_c=type_;

  // -CNCH3 (20130815)
  int n90=0;
  for(int i=0;i<n_atom_index;i++){
    if(atom_type[i]==90){
      n90++;
    }
  }
  if(n90>1){
    int idx=90;
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==90){
        atom_type[i]=idx;
        idx++;
      }
    }
  }    
}

void BioUnit::set_terminal_n ( const int type_ )
{
  terminal_n=type_;

  // -NH3+
  int n5=0;
  for(int i=0;i<n_atom_index;i++){
    if(atom_type[i]==5){
      n5++;
    }
  }
  if(n5>1){
    int idx=10;
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==5){
        atom_type[i]=idx;
        idx++;
      }
    }
  }

  // -COCH3 (20130815)
  int n84=0;
  for(int i=0;i<n_atom_index;i++){
    if(atom_type[i]==84){
      n84++;
    }
  }
  if(n84>1){
    int idx=84;
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==84){
        atom_type[i]=idx;
        idx++;
      }
    }
  }  
}

void BioUnit::set_ss_bond ( const int ibu )
{
  ss_bond=ibu;
}

void BioUnit::index_order()
{
  int itmp;
  int n;
  int * atype = new int [n_atom_index+1];
  int * order = new int [n_atom_index+1];

  //////////////////
  // order define //
  //////////////////
  
  n=0;
  if(0){}
  else if(terminal_n==6){
    order[n]= 0              ; atype[n]=83 ; n++;
    order[n]= 1              ; atype[n]=84 ; n++;
    order[n]= 2              ; atype[n]=85 ; n++;
    order[n]= 3              ; atype[n]=86 ; n++;
    order[n]= 4              ; atype[n]=82 ; n++;
    order[n]= 5              ; atype[n]=81 ; n++;
    order[n]= 6              ; atype[n]= 2 ; n++;
    order[n]= 7              ; atype[n]= 0 ; n++;
    order[n]= 8              ; atype[n]= 1 ; n++;
  }
  else if(terminal_c==6){
    order[n]= 0              ; atype[n]= 2 ; n++;
    order[n]= 1              ; atype[n]= 0 ; n++;
    order[n]= 2              ; atype[n]= 1 ; n++;
    order[n]= n_atom_index-6 ; atype[n]=87 ; n++;
    order[n]= n_atom_index-5 ; atype[n]=88 ; n++;
    order[n]= n_atom_index-4 ; atype[n]=89 ; n++;
    order[n]= n_atom_index-3 ; atype[n]=90 ; n++;
    order[n]= n_atom_index-2 ; atype[n]=91 ; n++;
    order[n]= n_atom_index-1 ; atype[n]=92 ; n++;
  } else {
    order[n]= 0              ; atype[n]= 2 ; n++;
    order[n]= 1              ; atype[n]= 0 ; n++;
    order[n]= 2              ; atype[n]= 1 ; n++;
  }

  //////////////
  // exchange //
  //////////////
  
  for(int j=0;j<n;j++){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==atype[j]){
        itmp=atom_type [order[j]];atom_type [order[j]]=atom_type [i];atom_type [i]=itmp;
        itmp=atom_index[order[j]];atom_index[order[j]]=atom_index[i];atom_index[i]=itmp;
      }
    }
  }
  
  delete [] atype;
  delete [] order;
}


void BioUnit::set_link_unit_c ( const int ibu )
{
  link_unit_c=ibu;
}

void BioUnit::set_link_unit_n ( const int ibu )
{
  link_unit_n=ibu;
}

void BioUnit::set_abbr_1 ( const string& s )
{
  abbr_1=s;
}

void BioUnit::set_abbr_3 ( const string& s )
{
  abbr_3=s;
}

//-----------------//
// dna deoxyribose //
//-----------------//

int BioUnit::get_terminal_5 ( ) const
{
  return terminal_5;
}

int BioUnit::get_terminal_3 ( ) const
{
  return terminal_3;
}

int BioUnit::get_link_unit_5() const
{
  return link_unit_5;
}

int BioUnit::get_link_unit_3() const
{
  return link_unit_3;
}

int BioUnit::get_link_unit_b() const
{
  return link_unit_b;
}

void BioUnit::set_terminal_5 ( const int type_ )
{
  terminal_5=type_;
}

void BioUnit::set_terminal_3 ( const int type_ )
{
  terminal_3=type_;
}

void BioUnit::set_link_unit_5 ( const int ibu )
{
  link_unit_5=ibu;
}

void BioUnit::set_link_unit_3 ( const int ibu )
{
  link_unit_3=ibu;
}

void BioUnit::set_link_unit_b ( const int ibu )
{
  link_unit_b=ibu;
}

int BioUnit::get_c20() const
{
  //if(4000<=type && type<4001){ // ishi 20160315
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==20){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_o21() const
{
  //if(4000<=type && type<4001){ // ishi 20160315
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==21){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_c22() const
{
  //if(4000<=type && type<4001){  // ishi 20160315
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==22){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_c23() const
{
  //if(4000<=type && type<4001){  // ishi 20160315
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==23){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_c24() const
{
  //if(4000<=type && type<4001){  // ishi 20160315
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==24){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_c25() const
{
  //if(4000<=type && type<4001){ // ishi 20160315
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==25){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_p27() const
{
  //if(4000<=type && type<4001){  // ishi 20160351
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==27){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_h28() const
{
  //if(4000<=type && type<4001){ // ishi 20160315
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==28){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_o29() const
{
  //if(4000<=type && type<4001){ // ishi 20160315
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==29){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_h30() const
{
  //if(4000<=type && type<4001){ // ishi 20160315
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==30){
        return i;
      }
    }   
  }
  return -1;
}

int BioUnit::get_o31() const
{
  //if(4000<=type && type<4001){ // ishi 20160315
  if(4000<=type && type<5000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==31){
        return i;
      }
    }   
  }
  return -1;
}

//----------//
// dna base //
//----------//

int BioUnit::get_link_unit_d() const
{
  return link_unit_d;
}

void BioUnit::set_link_unit_d ( const int ibu )
{
  link_unit_d=ibu;
}

int BioUnit::get_n26() const
{
  if(5000<=type && type<=6000){
    for(int i=0;i<n_atom_index;i++){
      if(atom_type[i]==26){
        return i;
      }
    }   
  }
  return -1;
}

//------------//
// res abbr 3 //
//------------//

string BioUnit::res_abbr_3 ( const int index )
{
  return BioUnitData::Default_abbr_3(index);
  /*
  string s;
  switch(index){
  case  100: s="GLY"; break;
  case  200: s="ALA"; break;
  case  300: s="VAL"; break;
  case  400: s="LEU"; break;
  case  500: s="ILE"; break;
  case  600: s="MET"; break;
  case  700: s="PRO"; break;
  case  800: s="PHE"; break;
  case  900: s="TRP"; break;
  case 1000: s="SER"; break;
  case 1100: s="THR"; break;
  case 1200: s="ASN"; break;
  case 1300: s="GLN"; break;
  case 1400: s="TYR"; break;
  case 1500: s="CYS"; break;
  case 1550: s="CYX"; break;
  case 1600: s="LYS"; break;
  case 1601: s="LYS"; break;
  case 1700: s="ARG"; break;
  case 1701: s="ARG"; break;
  case 1800: s="HIS"; break;
  //case 1850: s="HIP"; break; // <-- ishiishi 
  case 1900: s="ASP"; break;
  case 1901: s="ASP"; break;
  case 2000: s="GLU"; break;
  case 2001: s="GLU"; break;
  default  : s="***"; break;
  }
  return s;
  */
}

//------------//
// res abbr 1 //
//------------//

string BioUnit::res_abbr_1 ( const int index )
{
  return BioUnitData::Default_abbr_1(index);
  /*
  string s;
  switch(index){
  case  100: s="G"; break;
  case  200: s="A"; break;
  case  300: s="V"; break;
  case  400: s="L"; break;
  case  500: s="I"; break;
  case  600: s="M"; break;
  case  700: s="P"; break;
  case  800: s="F"; break;
  case  900: s="W"; break;
  case 1000: s="S"; break;
  case 1100: s="T"; break;
  case 1200: s="N"; break;
  case 1300: s="Q"; break;
  case 1400: s="Y"; break;
  case 1500: s="C"; break;
  case 1550: s="C"; break;
  case 1600: s="K"; break;
  case 1601: s="K"; break;
  case 1700: s="R"; break;
  case 1701: s="R"; break;
  case 1800: s="H"; break;
  //case 1850: s="H"; break; // <-- ishiishi
  case 1900: s="D"; break;
  case 1901: s="D"; break;
  case 2000: s="E"; break;
  case 2001: s="E"; break;
  default  : s="*"; break;
  }
  return s;
  */
}

//---------//
// pdb tag //
//---------//

string BioUnit::pdb_tag ( const int atom_index ,
			  const int info01     ) 
{
  string s;

  // main chain

  if (atom_index==   0) {s="  CA  "; return s;}
  if (atom_index==   1) {s="  C   "; return s;}
  if (atom_index==   2) {s="  N   "; return s;}
  if (atom_index==   3) {s="  HA  "; return s;}
  if (atom_index==   4) {s="  O   "; return s;}
  if (atom_index==   5) {s="  H   "; return s;} 
  if (atom_index==   6) {s="  H2  "; return s;}
  if (atom_index==   7) {s="  HC  "; return s;}
  if (atom_index==   8) {s="  HA2 "; return s;}
  if (atom_index==   9) {s="  HA3 "; return s;}
  if (atom_index==  10) {s="  H1  "; return s;} //<---pro:H2,H3
  if (atom_index==  11) {s="  H2  "; return s;}
  if (atom_index==  12) {s="  H3  "; return s;}
  if (atom_index==  81) {s="  C   "; return s;} // nterm -COCH3
  if (atom_index==  82) {s="  O   "; return s;} // nterm -COCH3
  if (atom_index==  83) {s="  CH3 "; return s;} // nterm -COCH3
  if (atom_index==  84) {s=" HH31 "; return s;} // nterm -COCH3
  if (atom_index==  85) {s=" HH32 "; return s;} // nterm -COCH3
  if (atom_index==  86) {s=" HH33 "; return s;} // nterm -COCH3
  if (atom_index==  87) {s="  N   "; return s;} // cterm -CNCH3
  if (atom_index==  88) {s="  H   "; return s;} // cterm -CNCH3
  if (atom_index==  89) {s="  CH3 "; return s;} // cterm -CNCH3
  if (atom_index==  90) {s=" HH31 "; return s;} // cterm -CNCH3
  if (atom_index==  91) {s=" HH32 "; return s;} // cterm -CNCH3
  if (atom_index==  92) {s=" HH33 "; return s;} // cterm -CNCH3

  // deoxyribose

  if (atom_index==  20) {s="  C5* "; return s;}
  if (atom_index==  21) {s="  O5* "; return s;}
  if (atom_index==  22) {s="  C4* "; return s;}
  if (atom_index==  23) {s="  C3* "; return s;}
  if (atom_index==  24) {s="  C2* "; return s;}
  if (atom_index==  25) {s="  C1* "; return s;}
  if (atom_index==  27) {s="  P   "; return s;}
  if (atom_index==  28) {s="  H5T "; return s;}
  if (atom_index==  29) {s="  O3* "; return s;}
  if (atom_index==  30) {s="  H3T "; return s;}
  if (atom_index==  31) {s="  O3* "; return s;}
  if (atom_index==  32) {s="  O1P "; return s;}
  if (atom_index==  33) {s="  O2P "; return s;}
  if (atom_index==  34) {s="  O4* "; return s;}
  if (atom_index==  35) {s="      "; return s;}
  if (atom_index==  36) {s=" 1H5* "; return s;}
  if (atom_index==  37) {s=" 2H5* "; return s;}
  if (atom_index==  38) {s="  H4* "; return s;}
  if (atom_index==  39) {s="  H3* "; return s;}
  if (atom_index==  40) {s=" 1H2* "; return s;}
  if (atom_index==  41) {s="      "; return s;}
  if (atom_index==  42) {s="  H1* "; return s;}
  if (atom_index==  43) {s=" 2H2* "; return s;}
  if (atom_index==  44) {s="  O5* "; return s;}
  if (atom_index==  26) {
    if   (info01==5000) {s="  N9  "; return s;} // ade
    if   (info01==5100) {s="  N9  "; return s;} // gua
    if   (info01==5200) {s="  N1  "; return s;} // thy
    if   (info01==5300) {s="  N1  "; return s;} // cyt
    if   (info01==5400) {s="  N1  "; return s;} // ura
    s=" **** "; return s;
  }

  // ala

  if (atom_index== 205) {s="  CB  "; return s;}
  if (atom_index== 206) {s="  HB1 "; return s;}
  if (atom_index== 207) {s="  HB2 "; return s;}
  if (atom_index== 208) {s="  HB3 "; return s;}

  // val 

  if (atom_index== 305) {s="  CB  "; return s;}
  if (atom_index== 306) {s="  CG1 "; return s;}
  if (atom_index== 307) {s="  CG2 "; return s;}
  if (atom_index== 308) {s="  HB  "; return s;}
  if (atom_index== 309) {s=" HG11 "; return s;}
  if (atom_index== 310) {s=" HG12 "; return s;}
  if (atom_index== 311) {s=" HG13 "; return s;}
  if (atom_index== 312) {s=" HG21 "; return s;}
  if (atom_index== 313) {s=" HG22 "; return s;}
  if (atom_index== 314) {s=" HG23 "; return s;}

  // Leu

  if (atom_index== 405) {s="  CB  "; return s;}
  if (atom_index== 406) {s="  CG  "; return s;}
  if (atom_index== 407) {s="  CD1 "; return s;}
  if (atom_index== 408) {s="  CD2 "; return s;}
  if (atom_index== 409) {s="  HB2 "; return s;}
  if (atom_index== 410) {s="  HB3 "; return s;}
  if (atom_index== 411) {s="  HG  "; return s;}
  if (atom_index== 412) {s=" HD11 "; return s;}
  if (atom_index== 413) {s=" HD12 "; return s;}
  if (atom_index== 414) {s=" HD13 "; return s;}
  if (atom_index== 415) {s=" HD21 "; return s;}
  if (atom_index== 416) {s=" HD22 "; return s;}
  if (atom_index== 417) {s=" HD23 "; return s;}

  // Ile

  if (atom_index== 505) {s="  CB  "; return s;}
  if (atom_index== 506) {s="  CG2 "; return s;}
  if (atom_index== 507) {s="  CG1 "; return s;}
  if (atom_index== 508) {s="  CD1 "; return s;}
  if (atom_index== 509) {s="  HB  "; return s;}
  if (atom_index== 510) {s=" HG21 "; return s;}
  if (atom_index== 511) {s=" HG22 "; return s;}
  if (atom_index== 512) {s=" HG23 "; return s;}
  if (atom_index== 513) {s=" HG12 "; return s;}
  if (atom_index== 514) {s=" HG13 "; return s;}
  if (atom_index== 515) {s=" HD11 "; return s;}
  if (atom_index== 516) {s=" HD12 "; return s;}
  if (atom_index== 517) {s=" HD13 "; return s;}

  // Met

  if (atom_index== 605) {s="  CB  "; return s;}
  if (atom_index== 606) {s="  CG  "; return s;}
  if (atom_index== 607) {s="  SD  "; return s;}
  if (atom_index== 608) {s="  CE  "; return s;}
  if (atom_index== 609) {s="  HB2 "; return s;}
  if (atom_index== 610) {s="  HB3 "; return s;}
  if (atom_index== 611) {s="  HG2 "; return s;}
  if (atom_index== 612) {s="  HG3 "; return s;}
  if (atom_index== 613) {s="  HE1 "; return s;}
  if (atom_index== 614) {s="  HE2 "; return s;}
  if (atom_index== 615) {s="  HE3 "; return s;}

  // Pro

  if (atom_index== 705) {s="  CB  "; return s;}
  if (atom_index== 706) {s="  CG  "; return s;}
  if (atom_index== 707) {s="  CD  "; return s;}
  if (atom_index== 708) {s="  HB2 "; return s;}
  if (atom_index== 709) {s="  HB3 "; return s;}
  if (atom_index== 710) {s="  HG2 "; return s;}
  if (atom_index== 711) {s="  HG3 "; return s;}
  if (atom_index== 712) {s="  HD2 "; return s;}
  if (atom_index== 713) {s="  HD3 "; return s;}

  // Phe

  if (atom_index== 805) {s="  CB  "; return s;}
  if (atom_index== 806) {s="  CG  "; return s;}
  if (atom_index== 807) {s="  CD1 "; return s;}
  if (atom_index== 808) {s="  CD2 "; return s;}
  if (atom_index== 809) {s="  CE1 "; return s;}
  if (atom_index== 810) {s="  CE2 "; return s;}
  if (atom_index== 811) {s="  CZ  "; return s;}
  if (atom_index== 812) {s="  HB2 "; return s;}
  if (atom_index== 813) {s="  HB3 "; return s;}
  if (atom_index== 814) {s="  HD1 "; return s;}
  if (atom_index== 815) {s="  HD2 "; return s;}
  if (atom_index== 816) {s="  HE1 "; return s;}
  if (atom_index== 817) {s="  HE2 "; return s;}
  if (atom_index== 818) {s="  HZ  "; return s;}

  // Trp

  if (atom_index== 905) {s="  CB  "; return s;}
  if (atom_index== 906) {s="  CG  "; return s;}
  if (atom_index== 907) {s="  CD1 "; return s;}
  if (atom_index== 908) {s="  CD2 "; return s;}
  if (atom_index== 909) {s="  NE1 "; return s;}
  if (atom_index== 910) {s="  CE2 "; return s;}
  if (atom_index== 911) {s="  CE3 "; return s;}
  if (atom_index== 912) {s="  CZ3 "; return s;}
  if (atom_index== 913) {s="  CH2 "; return s;}
  if (atom_index== 914) {s="  CZ2 "; return s;}
  if (atom_index== 915) {s="  HB2 "; return s;}
  if (atom_index== 916) {s="  HB3 "; return s;}
  if (atom_index== 917) {s="  HD1 "; return s;}
  if (atom_index== 918) {s="  HE1 "; return s;}
  if (atom_index== 919) {s="  HE3 "; return s;}
  if (atom_index== 920) {s="  HZ3 "; return s;}
  if (atom_index== 921) {s="  HH2 "; return s;}
  if (atom_index== 922) {s="  HZ2 "; return s;}

  // Ser

  if (atom_index==1005) {s="  CB  "; return s;}
  if (atom_index==1006) {s="  OG  "; return s;}
  if (atom_index==1007) {s="  HB2 "; return s;}
  if (atom_index==1008) {s="  HB3 "; return s;}
  if (atom_index==1009) {s="  HG  "; return s;}

  // Thr

  if (atom_index==1105) {s="  CB  "; return s;}
  if (atom_index==1106) {s="  CG2 "; return s;}
  if (atom_index==1107) {s="  OG1 "; return s;}
  if (atom_index==1108) {s="  HB  "; return s;}
  if (atom_index==1109) {s=" HG21 "; return s;}
  if (atom_index==1110) {s=" HG22 "; return s;}
  if (atom_index==1111) {s=" HG23 "; return s;}
  if (atom_index==1112) {s="  HG1 "; return s;}

  // Asn

  if (atom_index==1205) {s="  CB  "; return s;}
  if (atom_index==1206) {s="  CG  "; return s;}
  if (atom_index==1207) {s="  ND2 "; return s;}
  if (atom_index==1208) {s="  OD1 "; return s;}
  if (atom_index==1209) {s="  HB2 "; return s;}
  if (atom_index==1210) {s="  HB3 "; return s;}
  if (atom_index==1211) {s=" HD21 "; return s;}
  if (atom_index==1212) {s=" HD22 "; return s;}

  // Gln

  if (atom_index==1305) {s="  CB  "; return s;}
  if (atom_index==1306) {s="  CG  "; return s;}
  if (atom_index==1307) {s="  CD  "; return s;}
  if (atom_index==1308) {s="  NE2 "; return s;}
  if (atom_index==1309) {s="  OE1 "; return s;}
  if (atom_index==1310) {s="  HB2 "; return s;}
  if (atom_index==1311) {s="  HB3 "; return s;}
  if (atom_index==1312) {s="  HG2 "; return s;}
  if (atom_index==1313) {s="  HG3 "; return s;}
  if (atom_index==1314) {s=" HE21 "; return s;}
  if (atom_index==1315) {s=" HE22 "; return s;}

  // Tyr

  if (atom_index==1405) {s="  CB  "; return s;}
  if (atom_index==1406) {s="  CG  "; return s;}
  if (atom_index==1407) {s="  CD1 "; return s;}
  if (atom_index==1408) {s="  CD2 "; return s;}
  if (atom_index==1409) {s="  CE1 "; return s;}
  if (atom_index==1410) {s="  CE2 "; return s;}
  if (atom_index==1411) {s="  CZ  "; return s;}
  if (atom_index==1412) {s="  OH  "; return s;}
  if (atom_index==1413) {s="  HB2 "; return s;}
  if (atom_index==1414) {s="  HB3 "; return s;}
  if (atom_index==1415) {s="  HD1 "; return s;}
  if (atom_index==1416) {s="  HD2 "; return s;}
  if (atom_index==1417) {s="  HE1 "; return s;}
  if (atom_index==1418) {s="  HE2 "; return s;}
  if (atom_index==1419) {s="  HH  "; return s;}

  // Cys

  if (atom_index==1505) {s="  CB  "; return s;}
  if (atom_index==1506) {s="  SG  "; return s;}
  if (atom_index==1507) {s="  HB2 "; return s;}
  if (atom_index==1508) {s="  HB3 "; return s;}
  if (atom_index==1509) {s="  HG  "; return s;}

  // Lys

  if (atom_index==1605) {s="  CB  "; return s;}
  if (atom_index==1606) {s="  CG  "; return s;}
  if (atom_index==1607) {s="  CD  "; return s;}
  if (atom_index==1608) {s="  CE  "; return s;}
  if (atom_index==1609) {s="  NZ  "; return s;}
  if (atom_index==1610) {s="  HB2 "; return s;}
  if (atom_index==1611) {s="  HB3 "; return s;}
  if (atom_index==1612) {s="  HG2 "; return s;}
  if (atom_index==1613) {s="  HG3 "; return s;} 
  if (atom_index==1614) {s="  HD2 "; return s;}
  if (atom_index==1615) {s="  HD3 "; return s;}
  if (atom_index==1616) {s="  HE2 "; return s;}
  if (atom_index==1617) {s="  HE3 "; return s;}
  if (atom_index==1618) {s="  HZ1 "; return s;}
  if (atom_index==1619) {s="  HZ2 "; return s;}
  if (atom_index==1620) {s="  HZ3 "; return s;}

  // Arg

  if (atom_index==1705) {s="  CB  "; return s;}
  if (atom_index==1706) {s="  CG  "; return s;}
  if (atom_index==1707) {s="  CD  "; return s;}
  if (atom_index==1708) {s="  NE  "; return s;}
  if (atom_index==1709) {s="  CZ  "; return s;}
  if (atom_index==1710) {s="  NH1 "; return s;} 
  if (atom_index==1711) {s="  NH2 "; return s;}
  if (atom_index==1712) {s="  HB2 "; return s;}
  if (atom_index==1713) {s="  HB3 "; return s;}
  if (atom_index==1714) {s="  HG2 "; return s;}
  if (atom_index==1715) {s="  HG3 "; return s;}
  if (atom_index==1716) {s="  HD2 "; return s;} 
  if (atom_index==1717) {s="  HD3 "; return s;}
  if (atom_index==1718) {s="  HE  "; return s;}
  if (atom_index==1719) {s=" HH11 "; return s;}
  if (atom_index==1720) {s=" HH12 "; return s;}
  if (atom_index==1721) {s=" HH21 "; return s;}
  if (atom_index==1722) {s=" HH22 "; return s;}

  // His

  if (atom_index==1805) {s="  CB  "; return s;}
  if (atom_index==1806) {s="  CG  "; return s;}
  if (atom_index==1807) {s="  CD2 "; return s;}
  if (atom_index==1808) {s="  ND1 "; return s;}
  if (atom_index==1809) {s="  NE2 "; return s;}
  if (atom_index==1810) {s="  CE1 "; return s;}
  if (atom_index==1811) {s="  HB2 "; return s;}
  if (atom_index==1812) {s="  HB3 "; return s;}
  if (atom_index==1813) {s="  HD2 "; return s;} 
  if (atom_index==1814) {s="  HD1 "; return s;}
  if (atom_index==1815) {s="  HE1 "; return s;}
  if (atom_index==1816) {s="  HE2 "; return s;}

  // Asp

  if (atom_index==1905) {s="  CB  "; return s;}
  if (atom_index==1906) {s="  CG  "; return s;}
  if (atom_index==1907) {s="  OD1 "; return s;}
  if (atom_index==1908) {s="  OD2 "; return s;}
  if (atom_index==1909) {s="  HB2 "; return s;}
  if (atom_index==1910) {s="  HB3 "; return s;}
  if (atom_index==1911) {s="  HD2 "; return s;} // <---
  if (atom_index==1912) {s="  HD2 "; return s;} // <---

  // Glu

  if (atom_index==2005) {s="  CB  "; return s;}
  if (atom_index==2006) {s="  CG  "; return s;}
  if (atom_index==2007) {s="  CD  "; return s;}
  if (atom_index==2008) {s="  OE1 "; return s;}
  if (atom_index==2009) {s="  OE2 "; return s;}
  if (atom_index==2010) {s="  HB2 "; return s;}
  if (atom_index==2011) {s="  HB3 "; return s;}
  if (atom_index==2012) {s="  HG2 "; return s;}
  if (atom_index==2013) {s="  HG3 "; return s;}
  if (atom_index==2014) {s="  HE2 "; return s;} // <---
  if (atom_index==2015) {s="  HE2 "; return s;} // <---

  // ade

  if (atom_index==5001) {s="  C8  "; return s;}
  if (atom_index==5002) {s="  N7  "; return s;}
  if (atom_index==5003) {s="  C5  "; return s;}
  if (atom_index==5004) {s="  C4  "; return s;}
  if (atom_index==5005) {s="  N3  "; return s;}
  if (atom_index==5006) {s="  C2  "; return s;}
  if (atom_index==5007) {s="  N1  "; return s;}
  if (atom_index==5008) {s="  C6  "; return s;}
  if (atom_index==5009) {s="  N6  "; return s;}
  if (atom_index==5010) {s="  H8  "; return s;}
  if (atom_index==5011) {s="  H2  "; return s;}
  if (atom_index==5012) {s="  H61 "; return s;}
  if (atom_index==5013) {s="  H62 "; return s;}

  // gua

  if (atom_index==5101) {s="  C8  "; return s;}
  if (atom_index==5102) {s="  N7  "; return s;}
  if (atom_index==5103) {s="  C5  "; return s;}
  if (atom_index==5104) {s="  C4  "; return s;}
  if (atom_index==5105) {s="  N3  "; return s;}
  if (atom_index==5106) {s="  C2  "; return s;}
  if (atom_index==5107) {s="  N1  "; return s;}
  if (atom_index==5108) {s="  C6  "; return s;}
  if (atom_index==5109) {s="  N2  "; return s;}
  if (atom_index==5110) {s="  O6  "; return s;}
  if (atom_index==5111) {s="  H8  "; return s;}
  if (atom_index==5112) {s="  H21 "; return s;}
  if (atom_index==5113) {s="  H22 "; return s;}
  if (atom_index==5114) {s="  H1  "; return s;}

  // thy

  if (atom_index==5201) {s="  C6  "; return s;}
  if (atom_index==5202) {s="  C5  "; return s;}
  if (atom_index==5203) {s="  C4  "; return s;}
  if (atom_index==5204) {s="  N3  "; return s;}
  if (atom_index==5205) {s="  C2  "; return s;}
  if (atom_index==5206) {s="  C7  "; return s;}
  if (atom_index==5207) {s="  O4  "; return s;}
  if (atom_index==5208) {s="  O2  "; return s;}
  if (atom_index==5209) {s="  H6  "; return s;}
  if (atom_index==5210) {s="  H71 "; return s;}
  if (atom_index==5211) {s="  H72 "; return s;}
  if (atom_index==5212) {s="  H73 "; return s;}
  if (atom_index==5213) {s="  H3  "; return s;}

  // cyt

  if (atom_index==5301) {s="  C6  "; return s;}
  if (atom_index==5302) {s="  C5  "; return s;}
  if (atom_index==5303) {s="  C4  "; return s;}
  if (atom_index==5304) {s="  N3  "; return s;}
  if (atom_index==5305) {s="  C2  "; return s;}
  if (atom_index==5306) {s="  N4  "; return s;}
  if (atom_index==5307) {s="  O2  "; return s;}
  if (atom_index==5308) {s="  H6  "; return s;}
  if (atom_index==5309) {s="  H5  "; return s;}
  if (atom_index==5310) {s="  H41 "; return s;}
  if (atom_index==5311) {s="  H42 "; return s;}

  // ura

  if (atom_index==5401) {s=" 5401 "; return s;}
  if (atom_index==5402) {s=" 5402 "; return s;}
  if (atom_index==5403) {s=" 5403 "; return s;}
  if (atom_index==5404) {s=" 5404 "; return s;}
  if (atom_index==5405) {s=" 5405 "; return s;}
  if (atom_index==5406) {s=" 5406 "; return s;}
  if (atom_index==5407) {s=" 5407 "; return s;}
  if (atom_index==5408) {s=" 5408 "; return s;}
  if (atom_index==5409) {s=" 5409 "; return s;}
  if (atom_index==5410) {s=" 5410 "; return s;}

  // WAT

  if (atom_index==6501) {s="  O   "; return s;}
  if (atom_index==6502) {s="  H1  "; return s;}
  if (atom_index==6503) {s="  H2  "; return s;}

  // etc. 
  //cout << atom_index << endl ;
  s=" **** ";
  return s;
}

int BioUnit::res_type(const string s_in,
		      string * out)
{
  string s;
  char c[5];

  if(s_in.size()==3){
    s=s_in;
  }
  if(s_in.size()==4){
    c[0]=s_in.at(0);
    c[1]=s_in.at(1);
    c[2]=s_in.at(2);
    c[3]=s_in.at(3);
    c[4]='\0';
    if(c[0]=='A'){
      s=&c[1];
    } else {
      return -1;
    }
  }

  if(out!=NULL){
    (*out)=s;
  }

  // peptide
  if(s=="GLY"){return  100;}
  if(s=="ALA"){return  200;}
  if(s=="VAL"){return  300;}
  if(s=="LEU"){return  400;}
  if(s=="ILE"){return  500;}
  if(s=="MET"){return  600;}
  if(s=="PRO"){return  700;}
  if(s=="PHE"){return  800;}
  if(s=="TRP"){return  900;}
  if(s=="SER"){return 1000;}
  if(s=="THR"){return 1100;}
  if(s=="ASN"){return 1200;}
  if(s=="GLN"){return 1300;}
  if(s=="TYR"){return 1400;}
  if(s=="CYS"){return 1500;}
  if(s=="CYX"){return 1550;}
  if(s=="LYS"){return 1600;}
  if(s=="ARG"){return 1700;}
  if(s=="HIS"){return 1800;}
//if(s=="HIP"){return 1850;} // <-- ishiishi
  if(s=="HIP"){return 1800;} // <-- ishiishi
  if(s=="HIE"){return 1800;} // <-- ishiishi
  if(s=="HID"){return 1800;} // <-- ishiishi
  if(s=="ASP"){return 1900;}
  if(s=="GLU"){return 2000;}
  if(s=="ACE"){return 2100;} // NTERM -COCH3
  if(s=="NME"){return 2200;} // CTERM -NHCH3

  return -1;
}

int BioUnit::atm_type(const string s,
		      const string res_typ)
{
  if(res_typ=="ACE"){
    if (s=="C"   ){return 81;} // nterm -COCH3
    if (s=="O"   ){return 82;} // nterm -COCH3
    if (s=="CH3" ){return 83;} // nterm -COCH3
    if (s=="HH31"){return 84;} // nterm -COCH3
    if (s=="HH32"){return 85;} // nterm -COCH3
    if (s=="HH33"){return 86;} // nterm -COCH3
  }
  
  if(res_typ=="NME"){
    if (s=="N"   ){return 87;} // cterm -NHCH3
    if (s=="H"   ){return 88;} // cterm -NHCH3
    if (s=="CH3" ){return 89;} // cterm -NHCH3
    if (s=="HH31"){return 90;} // cterm -NHCH3
    if (s=="HH32"){return 91;} // cterm -NHCH3
    if (s=="HH33"){return 92;} // cterm -NHCH3
  }

  if (s=="CA"  ){return    0;}
  if (s=="C"   ){return    1;}
  if (s=="N"   ){return    2;}
  if (s=="HA"  ){return    3;}
  if (s=="O"   ){return    4;} 
  if (s=="OXT" ){return    4;}
  if (s=="H"   ){return    5;}
  if (s=="HN"  ){return    5;}
  if (s=="HN1" ){return    5;}
  if (s=="HN2" ){return    5;}
  if (s=="HN3" ){return    5;}
  if (s=="HC"  ){return    7;}
  if (s=="HA2" ){return    8;}
  if (s=="HA3" ){return    9;}
  if (s=="HA1" ){return    9;}
  if (s=="H1"  ){return   10;}
  if (s=="H2"  ){return   11;}
  if (s=="H3"  ){return   12;}

  if(res_typ=="ALA" || res_typ=="AALA") {
    if (s=="CB"  ){return  205;}
    if (s=="HB1" ){return  206;}
    if (s=="HB2" ){return  207;}
    if (s=="HB3" ){return  208;}
  }

  if(res_typ=="VAL" || res_typ=="AVAL") {
    if (s=="CB"  ){return  305;}
    if (s=="CG1" ){return  306;}
    if (s=="CG2" ){return  307;}
    if (s=="HB"  ){return  308;}
    if (s=="HG11"){return  309;} if (s=="1HG1"){return  309;}
    if (s=="HG12"){return  310;} if (s=="2HG1"){return  310;}
    if (s=="HG13"){return  311;} if (s=="3HG1"){return  311;}
    if (s=="HG21"){return  312;} if (s=="1HG2"){return  312;}
    if (s=="HG22"){return  313;} if (s=="2HG2"){return  313;}
    if (s=="HG23"){return  314;} if (s=="3HG2"){return  314;}
  }

  if(res_typ=="LEU" || res_typ=="ALEU") { 
    if (s=="CB"  ){return  405;}
    if (s=="CG"  ){return  406;}
    if (s=="CD1" ){return  407;}
    if (s=="CD2" ){return  408;}
    if (s=="HB2" ){return  409;}
    if (s=="HB3" ){return  410;} if (s=="HB1" ){return  410;}
    if (s=="HG"  ){return  411;}
    if (s=="HD11"){return  412;} if (s=="1HD1"){return  412;}
    if (s=="HD12"){return  413;} if (s=="2HD1"){return  413;}
    if (s=="HD13"){return  414;} if (s=="3HD1"){return  414;}
    if (s=="HD21"){return  415;} if (s=="1HD2"){return  415;}
    if (s=="HD22"){return  416;} if (s=="2HD2"){return  416;}
    if (s=="HD23"){return  417;} if (s=="3HD2"){return  417;}
  }

  if(res_typ=="ILE" || res_typ=="AILE") {
    if (s=="CB"  ){return  505;}
    if (s=="CG2" ){return  506;}
    if (s=="CG1" ){return  507;}
    if (s=="CD1" ){return  508;}
    if (s=="HB"  ){return  509;}
    if (s=="HG21"){return  510;} if (s=="1HG2"){return  510;}
    if (s=="HG22"){return  511;} if (s=="2HG2"){return  511;}
    if (s=="HG23"){return  512;} if (s=="3HG2"){return  512;}
    if (s=="HG12"){return  513;} if (s=="2HG1"){return  513;}
    if (s=="HG13"){return  514;} if (s=="HG11"){return  514;} if (s=="3HG1"){return  514;} if (s=="1HG1"){return  514;}
    if (s=="HD11"){return  515;} if (s=="1HD1"){return  515;}
    if (s=="HD12"){return  516;} if (s=="2HD1"){return  516;}
    if (s=="HD13"){return  517;} if (s=="3HD1"){return  517;}
  }

  if(res_typ=="MET" || res_typ=="AMET") {
    if (s=="CB"  ){return  605;}
    if (s=="CG"  ){return  606;}
    if (s=="SD"  ){return  607;} if (s=="SE"  ){return  607;}
    if (s=="CE"  ){return  608;}
    if (s=="HB2" ){return  609;}
    if (s=="HB3" ){return  610;} if (s=="HB1" ){return  610;}
    if (s=="HG2" ){return  611;}
    if (s=="HG3" ){return  612;} if (s=="HG1" ){return  612;}
    if (s=="HE1" ){return  613;}
    if (s=="HE2" ){return  614;}
    if (s=="HE3" ){return  615;}
  }

  if(res_typ=="PRO" || res_typ=="APRO") {
    if (s=="CB"  ){return  705;}
    if (s=="CG"  ){return  706;}
    if (s=="CD"  ){return  707;}
    if (s=="HB2" ){return  708;}
    if (s=="HB3" ){return  709;} if (s=="HB1" ){return  709;}
    if (s=="HG2" ){return  710;}
    if (s=="HG3" ){return  711;} if (s=="HG1" ){return  711;}
    if (s=="HD2" ){return  712;}
    if (s=="HD3" ){return  713;} if (s=="HD1" ){return  713;}
  }

  if(res_typ=="PHE" || res_typ=="APHE") {
    if (s=="CB"  ){return  805;}
    if (s=="CG"  ){return  806;}
    if (s=="CD1" ){return  807;}
    if (s=="CD2" ){return  808;}
    if (s=="CE1" ){return  809;}
    if (s=="CE2" ){return  810;}
    if (s=="CZ"  ){return  811;}
    if (s=="HB2" ){return  812;}
    if (s=="HB3" ){return  813;} if (s=="HB1" ){return  813;}
    if (s=="HD1" ){return  814;}
    if (s=="HD2" ){return  815;}
    if (s=="HE1" ){return  816;}
    if (s=="HE2" ){return  817;}
    if (s=="HZ"  ){return  818;}
  }

  if(res_typ=="TRP" || res_typ=="ATRP") {
    if (s=="CB"  ){return  905;}
    if (s=="CG"  ){return  906;}
    if (s=="CD1" ){return  907;}
    if (s=="CD2" ){return  908;}
    if (s=="NE1" ){return  909;}
    if (s=="CE2" ){return  910;}
    if (s=="CE3" ){return  911;}
    if (s=="CZ3" ){return  912;}
    if (s=="CH2" ){return  913;}
    if (s=="CZ2" ){return  914;}
    if (s=="HB2" ){return  915;}
    if (s=="HB3" ){return  916;} if (s=="HB1" ){return  916;}
    if (s=="HD1" ){return  917;}
    if (s=="HE1" ){return  918;}
    if (s=="HE3" ){return  919;}
    if (s=="HZ3" ){return  920;}
    if (s=="HH2" ){return  921;}
    if (s=="HZ2" ){return  922;}
  }

  if(res_typ=="SER" || res_typ=="ASER") {
    if (s=="CB"  ){return 1005;}
    if (s=="OG"  ){return 1006;}
    if (s=="HB2" ){return 1007;}
    if (s=="HB3" ){return 1008;} if (s=="HB1" ){return 1008;}
    if (s=="HG"  ){return 1009;}
  }

  if(res_typ=="THR" || res_typ=="ATHR") {
    if (s=="CB"  ){return 1105;}
    if (s=="CG2" ){return 1106;}
    if (s=="OG1" ){return 1107;}
    if (s=="HB"  ){return 1108;}
    if (s=="HG21"){return 1109;} if (s=="1HG2"){return 1109;}
    if (s=="HG22"){return 1110;} if (s=="2HG2"){return 1110;}
    if (s=="HG23"){return 1111;} if (s=="3HG2"){return 1111;}
    if (s=="HG1" ){return 1112;}
  }

  if(res_typ=="ASN" || res_typ=="AASN") {
    if (s=="CB"  ){return 1205;}
    if (s=="CG"  ){return 1206;}
    if (s=="ND2" ){return 1207;}
    if (s=="OD1" ){return 1208;}
    if (s=="HB2" ){return 1209;}
    if (s=="HB3" ){return 1210;} if (s=="HB1" ){return 1210;}
    if (s=="HD21"){return 1211;} if (s=="1HD2"){return 1211;}
    if (s=="HD22"){return 1212;} if (s=="2HD2"){return 1212;}
  }

  if(res_typ=="GLN" || res_typ=="AGLN") {
    if (s=="CB"  ){return 1305;}
    if (s=="CG"  ){return 1306;}
    if (s=="CD"  ){return 1307;}
    if (s=="NE2" ){return 1308;}
    if (s=="OE1" ){return 1309;}
    if (s=="HB2" ){return 1310;}
    if (s=="HB3" ){return 1311;} if (s=="HB1" ){return 1311;}
    if (s=="HG2" ){return 1312;}
    if (s=="HG3" ){return 1313;} if (s=="HG1" ){return 1313;}
    if (s=="HE21"){return 1314;} if (s=="1HE2"){return 1314;}
    if (s=="HE22"){return 1315;} if (s=="2HE2"){return 1315;}
  }

  if(res_typ=="TYR" || res_typ=="ATYR") {
    if (s=="CB"  ){return 1405;}
    if (s=="CG"  ){return 1406;}
    if (s=="CD1" ){return 1407;}
    if (s=="CD2" ){return 1408;}
    if (s=="CE1" ){return 1409;}
    if (s=="CE2" ){return 1410;}
    if (s=="CZ"  ){return 1411;}
    if (s=="OH"  ){return 1412;}
    if (s=="HB2" ){return 1413;}
    if (s=="HB3" ){return 1414;} if (s=="HB1" ){return 1414;}
    if (s=="HD1" ){return 1415;}
    if (s=="HD2" ){return 1416;}
    if (s=="HE1" ){return 1417;}
    if (s=="HE2" ){return 1418;}
    if (s=="HH"  ){return 1419;}
  }

  if(res_typ=="CYS" || res_typ=="ACYS" || res_typ=="CYX") {
    if (s=="CB"  ){return 1505;}
    if (s=="SG"  ){return 1506;}
    if (s=="HB2" ){return 1507;}
    if (s=="HB3" ){return 1508;} if (s=="HB1" ){return 1508;}
    if (s=="HG"  ){return 1509;}
  }

  if(res_typ=="LYS" || res_typ=="ALYS") {
    if (s=="CB"  ){return 1605;}
    if (s=="CG"  ){return 1606;}
    if (s=="CD"  ){return 1607;}
    if (s=="CE"  ){return 1608;}
    if (s=="NZ"  ){return 1609;}
    if (s=="HB2" ){return 1610;}
    if (s=="HB3" ){return 1611;} if (s=="HB1" ){return 1611;}
    if (s=="HG2" ){return 1612;}
    if (s=="HG3" ){return 1613;} if (s=="HG1" ){return 1613;}
    if (s=="HD2" ){return 1614;}
    if (s=="HD3" ){return 1615;} if (s=="HD1" ){return 1615;}
    if (s=="HE2" ){return 1616;}
    if (s=="HE3" ){return 1617;} if (s=="HE1" ){return 1617;}
    if (s=="HZ1" ){return 1618;}
    if (s=="HZ2" ){return 1619;}
    if (s=="HZ3" ){return 1620;}
  }

  if(res_typ=="ARG" || res_typ=="AARG") {
    if (s=="CB"  ){return 1705;}
    if (s=="CG"  ){return 1706;}
    if (s=="CD"  ){return 1707;}
    if (s=="NE"  ){return 1708;}
    if (s=="CZ"  ){return 1709;}
    if (s=="NH1" ){return 1710;}
    if (s=="NH2" ){return 1711;}
    if (s=="HB2" ){return 1712;}
    if (s=="HB3" ){return 1713;} if (s=="HB1" ){return 1713;}
    if (s=="HG2" ){return 1714;}
    if (s=="HG3" ){return 1715;} if (s=="HG1" ){return 1715;}
    if (s=="HD2" ){return 1716;}
    if (s=="HD3" ){return 1717;} if (s=="HD1" ){return 1717;}
    if (s=="HE"  ){return 1718;}
    if (s=="HH11"){return 1719;} if (s=="1HH1"){return 1719;}
    if (s=="HH12"){return 1720;} if (s=="2HH1"){return 1720;}
    if (s=="HH21"){return 1721;} if (s=="1HH2"){return 1721;}
    if (s=="HH22"){return 1722;} if (s=="2HH2"){return 1722;}
  }

  if(res_typ=="HIS" || res_typ=="AHIS" || res_typ=="HIP" || res_typ=="AHIP" ||
     res_typ=="HIE" || res_typ=="AHIE" || res_typ=="HID" || res_typ=="AHID" ){
    if (s=="CB"  ){return 1805;}
    if (s=="CG"  ){return 1806;}
    if (s=="CD2" ){return 1807;}
    if (s=="ND1" ){return 1808;}
    if (s=="NE2" ){return 1809;}
    if (s=="CE1" ){return 1810;}
    if (s=="HB2" ){return 1811;}
    if (s=="HB3" ){return 1812;} if (s=="HB1" ){return 1812;}
    if (s=="HD2" ){return 1813;} 
    if (s=="HD1" ){return 1814;}
    if (s=="HE1" ){return 1815;}
    if (s=="HE2" ){return 1816;}
  }

  if(res_typ=="ASP" || res_typ=="AASP") {
    if (s=="CB"  ){return 1905;}
    if (s=="CG"  ){return 1906;}
    if (s=="OD1" ){return 1907;}
    if (s=="OD2" ){return 1908;}
    if (s=="HB2" ){return 1909;}
    if (s=="HB3" ){return 1910;} if (s=="HB1" ){return 1910;}
  }

  if(res_typ=="GLU" || res_typ=="AGLU") {
    if (s=="CB"  ){return 2005;}
    if (s=="CG"  ){return 2006;}
    if (s=="CD"  ){return 2007;}
    if (s=="OE1" ){return 2008;}
    if (s=="OE2" ){return 2009;}
    if (s=="HB2" ){return 2010;}
    if (s=="HB3" ){return 2011;} if (s=="HB1" ){return 2011;}
    if (s=="HG2" ){return 2012;}
    if (s=="HG3" ){return 2013;} if (s=="HG1" ){return 2013;}
  }

  if(res_typ=="NME"){
    //if (s=="CH3"  ){return 0;}
  }
  if(res_typ=="ACE"){
  }

  return -1;
}

int BioUnit::bnd_type_rotatable ( const int index1 ,
				  const int index2 ,
                                  const int bio_type_1,
                                  const int bio_type_2) 
{
  // return  0 : not rotatable
  // return  1 :     rotatable
  // return -1 : not biounit atom

  int idx1,idx2,bio1,bio2;

  if(index1<index2){
    idx1=index1;
    idx2=index2;
    bio1=bio_type_1;
    bio2=bio_type_2;
  } else {
    idx1=index2;
    idx2=index1;
    bio1=bio_type_2;
    bio2=bio_type_1;
  }

  if(idx1==   0 && idx2==   2 && bio1==700 && bio2==700) { return 0;}

  // main-chain
  if(idx1==   0 && idx2==   1){return 1;}
  if(idx1==   0 && idx2==   2){return 1;}
  if(idx1==   0 && idx2==   3){return 1;}
  if(idx1==   0 && idx2==   8){return 1;}
  if(idx1==   0 && idx2==   9){return 1;}
  if(idx1==   1 && idx2==   2){return 1;}
  if(idx1==   1 && idx2==   4){return 0;}
  if(idx1==   2 && idx2==   5){return 1;}
  if(idx1==   2 && idx2==  10){return 1;}
  if(idx1==   2 && idx2==  11){return 1;}
  if(idx1==   2 && idx2==  12){return 1;}
  if(idx1==   2 && idx2==  81){return 1;} // ACE
  if(idx1==  81 && idx2==  82){return 0;} // ACE
  if(idx1==  81 && idx2==  83){return 1;} // ACE
  if(idx1==  83 && idx2==  84){return 1;} // ACE
  if(idx1==  83 && idx2==  85){return 1;} // ACE
  if(idx1==  83 && idx2==  86){return 1;} // ACE
  if(idx1==   1 && idx2==  87){return 1;} // NME
  if(idx1==  87 && idx2==  88){return 1;} // NME
  if(idx1==  87 && idx2==  89){return 1;} // NME
  if(idx1==  89 && idx2==  90){return 1;} // NME
  if(idx1==  89 && idx2==  91){return 1;} // NME
  if(idx1==  89 && idx2==  92){return 1;} // NME

  // ala
  if(idx1==   0 && idx2== 205){return 1;}
  if(idx1== 205 && idx2== 206){return 1;}
  if(idx1== 205 && idx2== 207){return 1;}
  if(idx1== 205 && idx2== 208){return 1;}

  // val
  if(idx1==   0 && idx2== 305){return 1;}
  if(idx1== 305 && idx2== 306){return 1;}
  if(idx1== 305 && idx2== 307){return 1;}
  if(idx1== 305 && idx2== 308){return 1;}
  if(idx1== 306 && idx2== 309){return 1;}
  if(idx1== 306 && idx2== 310){return 1;}
  if(idx1== 306 && idx2== 311){return 1;}
  if(idx1== 307 && idx2== 312){return 1;}
  if(idx1== 307 && idx2== 313){return 1;}
  if(idx1== 307 && idx2== 314){return 1;}

  // leu
  if(idx1==   0 && idx2== 405){return 1;}
  if(idx1== 405 && idx2== 406){return 1;}
  if(idx1== 405 && idx2== 409){return 1;}
  if(idx1== 405 && idx2== 410){return 1;}
  if(idx1== 406 && idx2== 407){return 1;}
  if(idx1== 406 && idx2== 408){return 1;}
  if(idx1== 406 && idx2== 411){return 1;}
  if(idx1== 407 && idx2== 412){return 1;}
  if(idx1== 407 && idx2== 413){return 1;}
  if(idx1== 407 && idx2== 414){return 1;}
  if(idx1== 408 && idx2== 415){return 1;}
  if(idx1== 408 && idx2== 416){return 1;}
  if(idx1== 408 && idx2== 417){return 1;}

  // ile
  if(idx1==   0 && idx2== 505){return 1;}
  if(idx1== 505 && idx2== 506){return 1;}
  if(idx1== 505 && idx2== 507){return 1;}
  if(idx1== 505 && idx2== 509){return 1;}
  if(idx1== 506 && idx2== 510){return 1;}
  if(idx1== 506 && idx2== 511){return 1;}
  if(idx1== 506 && idx2== 512){return 1;}
  if(idx1== 507 && idx2== 508){return 1;}
  if(idx1== 507 && idx2== 513){return 1;}
  if(idx1== 507 && idx2== 514){return 1;}
  if(idx1== 508 && idx2== 515){return 1;}
  if(idx1== 508 && idx2== 516){return 1;}
  if(idx1== 508 && idx2== 517){return 1;}

  // met
  if(idx1==   0 && idx2== 605){return 1;}
  if(idx1== 605 && idx2== 606){return 1;}
  if(idx1== 605 && idx2== 609){return 1;}
  if(idx1== 605 && idx2== 610){return 1;}
  if(idx1== 606 && idx2== 607){return 1;}
  if(idx1== 606 && idx2== 611){return 1;}
  if(idx1== 606 && idx2== 612){return 1;}
  if(idx1== 607 && idx2== 608){return 1;}
  if(idx1== 607 && idx2== 608){return 1;}
  if(idx1== 608 && idx2== 613){return 1;}
  if(idx1== 608 && idx2== 614){return 1;}
  if(idx1== 608 && idx2== 615){return 1;}

  // pro
  if(idx1==   0 && idx2== 705){return 0;}
  if(idx1== 705 && idx2== 706){return 0;}
  if(idx1== 705 && idx2== 708){return 1;} 
  if(idx1== 705 && idx2== 709){return 1;}
  if(idx1== 706 && idx2== 707){return 0;}
  if(idx1== 706 && idx2== 710){return 1;}
  if(idx1== 706 && idx2== 711){return 1;}
  if(idx1== 707 && idx2== 712){return 1;}
  if(idx1== 707 && idx2== 713){return 1;}
  if(idx1==   2 && idx2== 707){return 0;} // 20150125

  // phe
  if(idx1==   0 && idx2== 805){return 1;}
  if(idx1== 805 && idx2== 806){return 1;}
  if(idx1== 805 && idx2== 812){return 1;}
  if(idx1== 805 && idx2== 813){return 1;}
  if(idx1== 806 && idx2== 807){return 0;}
  if(idx1== 806 && idx2== 808){return 0;} // 20150125
  if(idx1== 807 && idx2== 809){return 0;}
  if(idx1== 807 && idx2== 814){return 1;}
  if(idx1== 808 && idx2== 810){return 0;}
  if(idx1== 808 && idx2== 815){return 1;}
  if(idx1== 809 && idx2== 811){return 0;}
  if(idx1== 809 && idx2== 816){return 1;}
  if(idx1== 810 && idx2== 811){return 0;}
  if(idx1== 810 && idx2== 817){return 1;}
  if(idx1== 811 && idx2== 818){return 1;}

  // trp
  if(idx1==   0 && idx2== 905){return 1;}
  if(idx1== 905 && idx2== 906){return 1;}
  if(idx1== 905 && idx2== 915){return 1;}
  if(idx1== 905 && idx2== 916){return 1;}
  if(idx1== 906 && idx2== 907){return 0;}
  if(idx1== 906 && idx2== 908){return 0;}
  if(idx1== 907 && idx2== 909){return 0;}
  if(idx1== 907 && idx2== 917){return 1;}
  if(idx1== 908 && idx2== 910){return 0;}
  if(idx1== 908 && idx2== 911){return 0;}
  if(idx1== 909 && idx2== 910){return 0;}
  if(idx1== 909 && idx2== 918){return 1;}
  if(idx1== 910 && idx2== 914){return 0;}
  if(idx1== 911 && idx2== 912){return 0;}
  if(idx1== 911 && idx2== 919){return 1;}
  if(idx1== 912 && idx2== 913){return 0;}
  if(idx1== 912 && idx2== 920){return 1;}
  if(idx1== 913 && idx2== 914){return 0;}
  if(idx1== 913 && idx2== 921){return 1;}
  if(idx1== 914 && idx2== 922){return 1;}

  // ser
  if(idx1==   0 && idx2==1005){return 1;}
  if(idx1==1005 && idx2==1006){return 1;}
  if(idx1==1005 && idx2==1007){return 1;}
  if(idx1==1005 && idx2==1008){return 1;}
  if(idx1==1006 && idx2==1009){return 1;}

  // thr
  if(idx1==   0 && idx2==1105){return 1;}
  if(idx1==1105 && idx2==1106){return 1;}
  if(idx1==1105 && idx2==1107){return 1;}
  if(idx1==1105 && idx2==1108){return 1;}
  if(idx1==1106 && idx2==1109){return 1;}
  if(idx1==1106 && idx2==1110){return 1;}
  if(idx1==1106 && idx2==1111){return 1;}
  if(idx1==1107 && idx2==1112){return 1;}

  // asn
  if(idx1==   0 && idx2==1205){return 1;}
  if(idx1==1205 && idx2==1206){return 1;}
  if(idx1==1205 && idx2==1209){return 1;}
  if(idx1==1205 && idx2==1210){return 1;}
  if(idx1==1206 && idx2==1207){return 1;}
  if(idx1==1206 && idx2==1208){return 0;}
  if(idx1==1207 && idx2==1211){return 1;}
  if(idx1==1207 && idx2==1212){return 1;}

  // gln
  if(idx1==   0 && idx2==1305){return 1;}
  if(idx1==1305 && idx2==1306){return 1;}
  if(idx1==1305 && idx2==1310){return 1;}
  if(idx1==1305 && idx2==1311){return 1;}
  if(idx1==1306 && idx2==1307){return 1;}
  if(idx1==1306 && idx2==1312){return 1;}
  if(idx1==1306 && idx2==1313){return 1;}
  if(idx1==1307 && idx2==1308){return 1;}
  if(idx1==1307 && idx2==1309){return 0;}
  if(idx1==1308 && idx2==1314){return 1;}
  if(idx1==1308 && idx2==1315){return 2;}

  // tyr
  if(idx1==   0 && idx2==1405){return 1;}
  if(idx1==1405 && idx2==1406){return 1;}
  if(idx1==1405 && idx2==1413){return 1;}
  if(idx1==1405 && idx2==1414){return 1;}
  if(idx1==1406 && idx2==1407){return 0;}
  if(idx1==1406 && idx2==1408){return 0;}
  if(idx1==1407 && idx2==1409){return 0;}
  if(idx1==1407 && idx2==1415){return 1;}
  if(idx1==1408 && idx2==1410){return 0;}
  if(idx1==1408 && idx2==1416){return 1;}
  if(idx1==1409 && idx2==1411){return 0;}
  if(idx1==1409 && idx2==1417){return 1;}
  if(idx1==1410 && idx2==1411){return 0;}
  if(idx1==1410 && idx2==1418){return 1;}
  if(idx1==1411 && idx2==1412){return 1;}
  if(idx1==1412 && idx2==1419){return 1;}

  // cys,cyx
  if(idx1==   0 && idx2==1505){return 1;}
  if(idx1==1505 && idx2==1506){return 1;}
  if(idx1==1505 && idx2==1507){return 1;}
  if(idx1==1505 && idx2==1508){return 1;}
  if(idx1==1506 && idx2==1509){return 1;}

  // lys
  if(idx1==   0 && idx2==1605){return 1;}
  if(idx1==1605 && idx2==1606){return 1;}
  if(idx1==1605 && idx2==1610){return 1;}
  if(idx1==1605 && idx2==1611){return 1;}
  if(idx1==1606 && idx2==1607){return 1;}
  if(idx1==1606 && idx2==1612){return 1;}
  if(idx1==1606 && idx2==1613){return 1;}
  if(idx1==1607 && idx2==1608){return 1;}
  if(idx1==1607 && idx2==1614){return 1;}
  if(idx1==1607 && idx2==1615){return 1;}
  if(idx1==1608 && idx2==1609){return 1;}
  if(idx1==1608 && idx2==1616){return 1;}
  if(idx1==1608 && idx2==1617){return 1;}
  if(idx1==1609 && idx2==1618){return 1;}
  if(idx1==1609 && idx2==1619){return 1;}
  if(idx1==1609 && idx2==1620){return 1;}

  // arg
  if(idx1==   0 && idx2==1705){return 1;}
  if(idx1==1705 && idx2==1706){return 1;}
  if(idx1==1705 && idx2==1712){return 1;}
  if(idx1==1705 && idx2==1713){return 1;}
  if(idx1==1706 && idx2==1707){return 1;}
  if(idx1==1706 && idx2==1714){return 1;}
  if(idx1==1706 && idx2==1715){return 1;}
  if(idx1==1707 && idx2==1708){return 1;}
  if(idx1==1707 && idx2==1716){return 1;}
  if(idx1==1707 && idx2==1717){return 1;}
  if(idx1==1708 && idx2==1709){return 1;} // <--
  if(idx1==1708 && idx2==1718){return 1;}
  if(idx1==1709 && idx2==1710){return 0;}
  if(idx1==1709 && idx2==1711){return 0;}
  if(idx1==1710 && idx2==1719){return 1;}
  if(idx1==1710 && idx2==1720){return 1;}
  if(idx1==1711 && idx2==1721){return 1;}
  if(idx1==1711 && idx2==1722){return 1;}

  // his
  if(idx1==   0 && idx2==1805){return 1;}
  if(idx1==1805 && idx2==1806){return 1;}
  if(idx1==1805 && idx2==1811){return 1;}
  if(idx1==1805 && idx2==1812){return 1;}
  if(idx1==1806 && idx2==1807){return 0;}
  if(idx1==1806 && idx2==1808){return 0;}
  if(idx1==1807 && idx2==1809){return 0;}
  if(idx1==1807 && idx2==1813){return 1;}
  if(idx1==1808 && idx2==1810){return 0;}
  if(idx1==1808 && idx2==1814){return 1;}
  if(idx1==1809 && idx2==1810){return 0;}
  if(idx1==1809 && idx2==1816){return 1;}
  if(idx1==1810 && idx2==1815){return 1;}

  // asp
  if(idx1==   0 && idx2==1905){return 1;}
  if(idx1==1905 && idx2==1906){return 1;}
  if(idx1==1905 && idx2==1909){return 1;}
  if(idx1==1905 && idx2==1910){return 1;}
  if(idx1==1906 && idx2==1907){return 0;}
  if(idx1==1906 && idx2==1908){return 0;}

  // glu
  if(idx1==   0 && idx2==2005){return 1;}
  if(idx1==2005 && idx2==2006){return 1;}
  if(idx1==2005 && idx2==2010){return 1;}
  if(idx1==2005 && idx2==2011){return 1;}
  if(idx1==2006 && idx2==2007){return 1;}
  if(idx1==2006 && idx2==2012){return 1;}
  if(idx1==2006 && idx2==2013){return 1;}
  if(idx1==2007 && idx2==2008){return 0;}
  if(idx1==2007 && idx2==2009){return 0;}

  return -1;  
}
