
#include "BioUnitData.h"

///////////////////////////////////////////////
//////////          bu_data          //////////
///////////////////////////////////////////////

bu_data::bu_data ()
{
  index=-1;
  pdb_tag="";
  abbr_1="";
  abbr_3="";
  atom_n=0;
  atom_n_buff=0;
  bond_n=0;
  bond_n_buff=0;
}

bu_data::bu_data ( const bu_data& bud )
{
  index=-1;
  pdb_tag="";
  abbr_1="";
  abbr_3="";
  atom_n=0;
  atom_n_buff=0;
  bond_n=0;
  bond_n_buff=0;
  copy(bud);
}

bu_data::~bu_data ( )
{
  clear();
}

void bu_data::copy( const bu_data& bud )
{
  if(this==&bud) return;
  index   = bud.index;
  pdb_tag = bud.pdb_tag;
  abbr_1  = bud.abbr_1;
  abbr_3  = bud.abbr_3;
  create_atom(bud.atom_n);
  create_bond(bud.atom_n);
  for(int i=0;i<bud.atom_n;i++) {
    atom_index        [i]=bud.atom_index        [i];
    atom_pdb_tag      [i]=bud.atom_pdb_tag      [i];
    atom_atomic_number[i]=bud.atom_atomic_number[i];
    atom_n++;
  }
  for(int i=0;i<bud.bond_n;i++) {
    bond_index_1[i]=bud.bond_index_1[i];
    bond_index_2[i]=bud.bond_index_2[i];
    bond_n++;
  }
}

void bu_data::clear()
{
  index=-1;
  pdb_tag="";
  abbr_1="";
  abbr_3="";
  clear_atom();
  clear_bond();
}

void bu_data::clear_atom()
{
  if(atom_n_buff==0) return ;
  delete [] atom_index;
  delete [] atom_pdb_tag;
  delete [] atom_atomic_number;
  atom_n=0;
  atom_n_buff=0;
}

void bu_data::create_atom(const int n)
{
  clear_atom();
  if(n==0) return ;
  atom_index         = new int    [n];
  atom_pdb_tag       = new string [n];
  atom_atomic_number = new int    [n];
  atom_n_buff=n;
}

void bu_data::resize_atom(const int n)
{
  int      atom_n_tmp;
  int    * atom_index_tmp;
  string * atom_pdb_tag_tmp;
  int    * atom_atomic_number_tmp;
  atom_n_tmp = atom_n ;
  atom_index_tmp         = new int    [atom_n_tmp+1];
  atom_pdb_tag_tmp       = new string [atom_n_tmp+1];
  atom_atomic_number_tmp = new int    [atom_n_tmp+1];
  for(int i=0;i<atom_n_tmp;i++){
    atom_index_tmp        [i]=atom_index        [i];
    atom_pdb_tag_tmp      [i]=atom_pdb_tag      [i];
    atom_atomic_number_tmp[i]=atom_atomic_number[i];
  }
  create_atom(n);
  if(n>=atom_n_tmp){
    for(int i=0;i<atom_n_tmp;i++){
      atom_index        [i]=atom_index_tmp        [i];
      atom_pdb_tag      [i]=atom_pdb_tag_tmp      [i];
      atom_atomic_number[i]=atom_atomic_number_tmp[i];
      atom_n++;
    }
  } else {
    for(int i=0;i<n;i++){
      atom_index        [i]=atom_index_tmp        [i];
      atom_pdb_tag      [i]=atom_pdb_tag_tmp      [i];
      atom_atomic_number[i]=atom_atomic_number_tmp[i];
      atom_n++;
    }
  }
  delete [] atom_index_tmp;
  delete [] atom_pdb_tag_tmp;
  delete [] atom_atomic_number_tmp;
}

void bu_data::push_back_atom(const int    index,
                             const string pdb_tag,
                             const int    atomic_number,
                             const int    buff_inc)
{
  if(atom_n==atom_n_buff){
    resize_atom(atom_n_buff+buff_inc);
  }
  atom_index        [atom_n]=index;
  atom_pdb_tag      [atom_n]=pdb_tag;
  atom_atomic_number[atom_n]=atomic_number;
  atom_n++;  
}

void bu_data::push_back_bond(const int index_1,
                             const int index_2,
                             const int buff_inc)
{
  if(bond_n==bond_n_buff){
    resize_bond(bond_n_buff+buff_inc);
  }
  bond_index_1[bond_n]=index_1;
  bond_index_2[bond_n]=index_2;
  bond_n++;
}

void bu_data::clear_bond()
{
  if(bond_n_buff==0) return ;
  delete [] bond_index_1;
  delete [] bond_index_2;
  bond_n=0;
  bond_n_buff=0;
}

void bu_data::create_bond(const int n)
{
  clear_bond();
  if(n==0) return ;
  bond_index_1 = new int [n];
  bond_index_2 = new int [n];
  bond_n_buff=n;
}

void bu_data::resize_bond(const int n)
{
  int   bond_n_tmp;
  int * bond_index_1_tmp;
  int * bond_index_2_tmp;
  bond_n_tmp=bond_n;
  bond_index_1 = new int [bond_n_tmp+1];
  bond_index_2 = new int [bond_n_tmp+1];
  for(int i=0;i<bond_n_tmp;i++){
    bond_index_1_tmp[i]=bond_index_1[i];
    bond_index_2_tmp[i]=bond_index_2[i];
  }
  create_bond(n);
  if(n>=bond_n_tmp){
    for(int i=0;i<bond_n_tmp;i++){
      bond_index_1[i]=bond_index_1_tmp[i];
      bond_index_2[i]=bond_index_2_tmp[i];
      bond_n++;
    }
  } else {
    for(int i=0;i<n;i++){
      bond_index_1[i]=bond_index_1_tmp[i];
      bond_index_2[i]=bond_index_2_tmp[i];
      bond_n++;
    }
  }
  delete [] bond_index_1_tmp;
  delete [] bond_index_2_tmp;
}

///////////////////////////////////////////////////
//////////          BioUnitData          //////////
///////////////////////////////////////////////////

BioUnitData::BioUnitData()
{
  data_n=0;
  data_n_buff=0;
}

BioUnitData::BioUnitData(const BioUnitData& bud)
{
  data_n=0;
  data_n_buff=0;
  copy(bud);
}

BioUnitData::~BioUnitData()
{
  clear();
}

void BioUnitData::copy(const BioUnitData& bud)
{
  create(bud.data_n);
  for(int i=0;i<bud.data_n;i++){
    data[i].copy(bud.data[i]);
    data_n++;
  }
}

void BioUnitData::clear()
{
  if(data_n_buff==0) return ;
  delete [] data ;
  data_n=0;
  data_n_buff=0;
}

void BioUnitData::create(const int n)
{
  clear();
  if(n==0) return ;
  data = new bu_data [n];
  data_n_buff = n; 
}

void BioUnitData::resize(const int n)
{
  int data_n_tmp;
  bu_data * data_tmp;
  data_n_tmp = data_n;
  data_tmp = new bu_data [data_n_tmp+1];
  for(int i=0;i<data_n_tmp;i++){
    data_tmp[i].copy(data[i]);
  }
  create(n);
  if(n>=data_n_tmp){
    for(int i=0;i<data_n_tmp;i++){
      data[i].copy(data_tmp[i]);
      data_n++;
    }
  } else {
    for(int i=0;i<n;i++){
      data[i].copy(data_tmp[i]);
      data_n++;
    }
  }
  delete [] data_tmp;
}

string BioUnitData::Default_abbr_1 ( const int index_ )
{
  string s;
  switch(index_){
  case  100: s="G"; break;
  case  200: s="A"; break;
  case  300: s="V"; break;
  case  400: s="L"; break;
  case  500: s="I"; break;
  case  600: s="M"; break;
  case  700: s="P"; break;
  case  800: s="F"; break;
  case  900: s="W"; break;
  case 1000: s="S"; break;
  case 1100: s="T"; break;
  case 1200: s="N"; break;
  case 1300: s="Q"; break;
  case 1400: s="Y"; break;
  case 1500: s="C"; break;
  case 1550: s="C"; break;
  case 1600: s="K"; break;
  case 1601: s="K"; break;
  case 1700: s="R"; break;
  case 1701: s="R"; break;
  case 1800: s="H"; break;
  case 1900: s="D"; break;
  case 1901: s="D"; break;
  case 2000: s="E"; break;
  case 2001: s="E"; break;
  default  : s="*"; break;
  }
  return s;
}

string BioUnitData::Default_abbr_3 ( const int index_ )
{
  string s;
  switch(index_){
  case  100: s="GLY"; break;
  case  200: s="ALA"; break;
  case  300: s="VAL"; break;
  case  400: s="LEU"; break;
  case  500: s="ILE"; break;
  case  600: s="MET"; break;
  case  700: s="PRO"; break;
  case  800: s="PHE"; break;
  case  900: s="TRP"; break;
  case 1000: s="SER"; break;
  case 1100: s="THR"; break;
  case 1200: s="ASN"; break;
  case 1300: s="GLN"; break;
  case 1400: s="TYR"; break;
  case 1500: s="CYS"; break;
  case 1550: s="CYX"; break;
  case 1600: s="LYS"; break;
  case 1601: s="LYS"; break;
  case 1700: s="ARG"; break;
  case 1701: s="ARG"; break;
  case 1800: s="HIS"; break;
  case 1900: s="ASP"; break;
  case 1901: s="ASP"; break;
  case 2000: s="GLU"; break;
  case 2001: s="GLU"; break;
  default  : s="***"; break;
  }
  return s;
}

int BioUnitData::Default_index ( const string pdb_tag_ )
{
  int i;
  return i;
}

string BioUnitData::Default_atom_pdb_tag ( const int ai )
{
  string s;
  if (ai ==    0) {s="  CA  "; return s;}
  if (ai ==    1) {s="  C   "; return s;}
  if (ai ==    2) {s="  N   "; return s;}
  if (ai ==    3) {s="  HA  "; return s;}
  if (ai ==    4) {s="  O   "; return s;}
  if (ai ==    5) {s="  H   "; return s;} 
  if (ai ==    6) {s="  H2  "; return s;}
  if (ai ==    7) {s="  HC  "; return s;}
  if (ai ==    8) {s="  HA2 "; return s;}
  if (ai ==    9) {s="  HA3 "; return s;}
  if (ai ==   10) {s="  H1  "; return s;} //<---pro:H2,H3
  if (ai ==   11) {s="  H2  "; return s;}
  if (ai ==   12) {s="  H3  "; return s;}
  if (ai ==   81) {s="  C   "; return s;} // nterm -COCH3
  if (ai ==   82) {s="  O   "; return s;} // nterm -COCH3
  if (ai ==   83) {s="  CH3 "; return s;} // nterm -COCH3
  if (ai ==   84) {s="  HH31"; return s;} // nterm -COCH3
  if (ai ==   85) {s="  HH32"; return s;} // nterm -COCH3
  if (ai ==   86) {s="  HH33"; return s;} // nterm -COCH3
  if (ai ==   87) {s="  N   "; return s;} // cterm -CNCH3
  if (ai ==   88) {s="  H   "; return s;} // cterm -CNCH3
  if (ai ==   89) {s="  CH3 "; return s;} // cterm -CNCH3
  if (ai ==   90) {s="  HH31"; return s;} // cterm -CNCH3
  if (ai ==   91) {s="  HH32"; return s;} // cterm -CNCH3
  if (ai ==   92) {s="  HH33"; return s;} // cterm -CNCH3

  // ala

  if (ai ==  205) {s="  CB  "; return s;}
  if (ai ==  206) {s="  HB1 "; return s;}
  if (ai ==  207) {s="  HB2 "; return s;}
  if (ai ==  208) {s="  HB3 "; return s;}

  // val 

  if (ai ==  305) {s="  CB  "; return s;}
  if (ai ==  306) {s="  CG1 "; return s;}
  if (ai ==  307) {s="  CG2 "; return s;}
  if (ai ==  308) {s="  HB  "; return s;}
  if (ai ==  309) {s=" HG11 "; return s;}
  if (ai ==  310) {s=" HG12 "; return s;}
  if (ai ==  311) {s=" HG13 "; return s;}
  if (ai ==  312) {s=" HG21 "; return s;}
  if (ai ==  313) {s=" HG22 "; return s;}
  if (ai ==  314) {s=" HG23 "; return s;}

  // Leu

  if (ai ==  405) {s="  CB  "; return s;}
  if (ai ==  406) {s="  CG  "; return s;}
  if (ai ==  407) {s="  CD1 "; return s;}
  if (ai ==  408) {s="  CD2 "; return s;}
  if (ai ==  409) {s="  HB2 "; return s;}
  if (ai ==  410) {s="  HB3 "; return s;}
  if (ai ==  411) {s="  HG  "; return s;}
  if (ai ==  412) {s=" HD11 "; return s;}
  if (ai ==  413) {s=" HD12 "; return s;}
  if (ai ==  414) {s=" HD13 "; return s;}
  if (ai ==  415) {s=" HD21 "; return s;}
  if (ai ==  416) {s=" HD22 "; return s;}
  if (ai ==  417) {s=" HD23 "; return s;}

  // Ile

  if (ai ==  505) {s="  CB  "; return s;}
  if (ai ==  506) {s="  CG2 "; return s;}
  if (ai ==  507) {s="  CG1 "; return s;}
  if (ai ==  508) {s="  CD1 "; return s;}
  if (ai ==  509) {s="  HB  "; return s;}
  if (ai ==  510) {s=" HG21 "; return s;}
  if (ai ==  511) {s=" HG22 "; return s;}
  if (ai ==  512) {s=" HG23 "; return s;}
  if (ai ==  513) {s=" HG12 "; return s;}
  if (ai ==  514) {s=" HG13 "; return s;}
  if (ai ==  515) {s=" HD11 "; return s;}
  if (ai ==  516) {s=" HD12 "; return s;}
  if (ai ==  517) {s=" HD13 "; return s;}

  // Met

  if (ai ==  605) {s="  CB  "; return s;}
  if (ai ==  606) {s="  CG  "; return s;}
  if (ai ==  607) {s="  SD  "; return s;}
  if (ai ==  608) {s="  CE  "; return s;}
  if (ai ==  609) {s="  HB2 "; return s;}
  if (ai ==  610) {s="  HB3 "; return s;}
  if (ai ==  611) {s="  HG2 "; return s;}
  if (ai ==  612) {s="  HG3 "; return s;}
  if (ai ==  613) {s="  HE1 "; return s;}
  if (ai ==  614) {s="  HE2 "; return s;}
  if (ai ==  615) {s="  HE3 "; return s;}

  // Pro

  if (ai ==  705) {s="  CB  "; return s;}
  if (ai ==  706) {s="  CG  "; return s;}
  if (ai ==  707) {s="  CD  "; return s;}
  if (ai ==  708) {s="  HB2 "; return s;}
  if (ai ==  709) {s="  HB3 "; return s;}
  if (ai ==  710) {s="  HG2 "; return s;}
  if (ai ==  711) {s="  HG3 "; return s;}
  if (ai ==  712) {s="  HD2 "; return s;}
  if (ai ==  713) {s="  HD3 "; return s;}

  // Phe

  if (ai ==  805) {s="  CB  "; return s;}
  if (ai ==  806) {s="  CG  "; return s;}
  if (ai ==  807) {s="  CD1 "; return s;}
  if (ai ==  808) {s="  CD2 "; return s;}
  if (ai ==  809) {s="  CE1 "; return s;}
  if (ai ==  810) {s="  CE2 "; return s;}
  if (ai ==  811) {s="  CZ  "; return s;}
  if (ai ==  812) {s="  HB2 "; return s;}
  if (ai ==  813) {s="  HB3 "; return s;}
  if (ai ==  814) {s="  HD1 "; return s;}
  if (ai ==  815) {s="  HD2 "; return s;}
  if (ai ==  816) {s="  HE1 "; return s;}
  if (ai ==  817) {s="  HE2 "; return s;}
  if (ai ==  818) {s="  HZ  "; return s;}

  // Trp

  if (ai ==  905) {s="  CB  "; return s;}
  if (ai ==  906) {s="  CG  "; return s;}
  if (ai ==  907) {s="  CD1 "; return s;}
  if (ai ==  908) {s="  CD2 "; return s;}
  if (ai ==  909) {s="  NE1 "; return s;}
  if (ai ==  910) {s="  CE2 "; return s;}
  if (ai ==  911) {s="  CE3 "; return s;}
  if (ai ==  912) {s="  CZ3 "; return s;}
  if (ai ==  913) {s="  CH2 "; return s;}
  if (ai ==  914) {s="  CZ2 "; return s;}
  if (ai ==  915) {s="  HB2 "; return s;}
  if (ai ==  916) {s="  HB3 "; return s;}
  if (ai ==  917) {s="  HD1 "; return s;}
  if (ai ==  918) {s="  HE1 "; return s;}
  if (ai ==  919) {s="  HE3 "; return s;}
  if (ai ==  920) {s="  HZ3 "; return s;}
  if (ai ==  921) {s="  HH2 "; return s;}
  if (ai ==  922) {s="  HZ2 "; return s;}

  // Ser

  if (ai == 1005) {s="  CB  "; return s;}
  if (ai == 1006) {s="  OG  "; return s;}
  if (ai == 1007) {s="  HB2 "; return s;}
  if (ai == 1008) {s="  HB3 "; return s;}
  if (ai == 1009) {s="  HG  "; return s;}

  // Thr

  if (ai == 1105) {s="  CB  "; return s;}
  if (ai == 1106) {s="  CG2 "; return s;}
  if (ai == 1107) {s="  OG1 "; return s;}
  if (ai == 1108) {s="  HB  "; return s;}
  if (ai == 1109) {s=" HG21 "; return s;}
  if (ai == 1110) {s=" HG22 "; return s;}
  if (ai == 1111) {s=" HG23 "; return s;}
  if (ai == 1112) {s="  HG1 "; return s;}

  // Asn

  if (ai == 1205) {s="  CB  "; return s;}
  if (ai == 1206) {s="  CG  "; return s;}
  if (ai == 1207) {s="  ND2 "; return s;}
  if (ai == 1208) {s="  OD1 "; return s;}
  if (ai == 1209) {s="  HB2 "; return s;}
  if (ai == 1210) {s="  HB3 "; return s;}
  if (ai == 1211) {s=" HD21 "; return s;}
  if (ai == 1212) {s=" HD22 "; return s;}

  // Gln

  if (ai == 1305) {s="  CB  "; return s;}
  if (ai == 1306) {s="  CG  "; return s;}
  if (ai == 1307) {s="  CD  "; return s;}
  if (ai == 1308) {s="  NE2 "; return s;}
  if (ai == 1309) {s="  OE1 "; return s;}
  if (ai == 1310) {s="  HB2 "; return s;}
  if (ai == 1311) {s="  HB3 "; return s;}
  if (ai == 1312) {s="  HG2 "; return s;}
  if (ai == 1313) {s="  HG3 "; return s;}
  if (ai == 1314) {s=" HE21 "; return s;}
  if (ai == 1315) {s=" HE22 "; return s;}

  return s;
}

int BioUnitData::Default_atom_index ( const string pdb_tag_      ,
                                      const string atom_pdb_tag_ )
{
  int i;
  return i;
}

int BioUnitData::Default_atom_atomic_number ( const string pdb_tag_      ,
                                              const string atom_pdb_tag_ )
{
  int i;
  return i;
}

int BioUnitData::Default_bond_n ( const int index_ )
{
  int i;
  return i;
}

int BioUnitData::Default_bond_index_1 ( const int index_ ,
                                        const int ibond  )
{
  int i;
  return i;
}

int BioUnitData::Default_bond_index_2 ( const int index_ ,
                                        const int ibond  )
{
  int i;
  return i;
}
