
////////////////////                              ///////////////////
////////////////////     Dihedral class ; CPP     ///////////////////
////////////////////                              ///////////////////

#include <math.h>
#include <cstdlib>
#include "Dihedral.h"
#include "Dvector3D.h"

using namespace std ;

Dihedral::Dihedral()
{
  iatom1=-1;
  iatom2=-1;
  iatom3=-1;
  iatom4=-1;
  chk_improper=0;
}

Dihedral::Dihedral(const int ia1, 
		   const int ia2,
		   const int ia3,
		   const int ia4)
{
  iatom1=ia1;
  iatom2=ia2;
  iatom3=ia3;
  iatom4=ia4;
  chk_improper=0;
}

Dihedral::Dihedral(const Dihedral& di)
{
  iatom1=-1;
  iatom2=-1;
  iatom3=-1;
  iatom4=-1;
  chk_improper=0;
  copy(di);
}

Dihedral::~Dihedral()
{
}

void Dihedral::copy(const Dihedral& di)
{
  if(this==&di) return;
  iatom1=di.iatom1;
  iatom2=di.iatom2;
  iatom3=di.iatom3;
  iatom4=di.iatom4;
  chk_improper=di.chk_improper;
}

int Dihedral::get_iatom1 ( ) const 
{
  return iatom1;
}

int Dihedral::get_iatom2 ( ) const
{
  return iatom2;
}
 
int Dihedral::get_iatom3 ( ) const
{
  return iatom3;
} 

int Dihedral::get_iatom4 ( ) const
{
  return iatom4;
} 

int Dihedral::get_chk_improper() const
{
  return chk_improper;
}

void Dihedral::set_iatom1(const int ia_)
{
  iatom1=ia_;
}

void Dihedral::set_iatom2(const int ia_)
{
  iatom2=ia_;
}

void Dihedral::set_iatom3(const int ia_)
{
  iatom3=ia_;
}

void Dihedral::set_iatom4(const int ia_)
{
  iatom4=ia_;
}

void Dihedral::set_improper_on ()
{
  chk_improper=1;
}

void Dihedral::set_improper_off()
{
  chk_improper=0;
}

double Dihedral::calc(const double    x1,
		      const double    y1,
		      const double    z1,
		      const double    x2,
		      const double    y2,
		      const double    z2,
		      const double    x3,
		      const double    y3,
		      const double    z3,
		      const double    x4,
		      const double    y4,
		      const double    z4)
{
  double r32n,n_12_32_n_32_34,nn1232,nn3234,r,r12_r32,r32_r34;
  double r12[3]; double r32[3]; double r34[3]; double n_12_32[3]; double n_32_34[3]; double n[3];

  //cout.setf(ios::fixed,ios::floatfield);
  //cout.precision(10);
  //cout.width(20);cout << x1 ;
  //cout.width(20);cout << y1 ;
  //cout.width(20);cout << z1 << endl ;
  //cout.width(20);cout << x2 ;
  //cout.width(20);cout << y2 ;
  //cout.width(20);cout << z2 << endl ;
  //cout.width(20);cout << x3 ;
  //cout.width(20);cout << y3 ;
  //cout.width(20);cout << z3 << endl ;
  //cout.width(20);cout << x4 ;
  //cout.width(20);cout << y4 ;
  //cout.width(20);cout << z4 << endl ;
 
  r12[0]=x2-x1;
  r12[1]=y2-y1;
  r12[2]=z2-z1;
  r32[0]=x2-x3;
  r32[1]=y2-y3;
  r32[2]=z2-z3;
  r34[0]=x4-x3;
  r34[1]=y4-y3;
  r34[2]=z4-z3;

  r12_r32=r12[0]*r32[0]+r12[1]*r32[1]+r12[2]*r32[2];
  r32_r34=r32[0]*r34[0]+r32[1]*r34[1]+r32[2]*r34[2];

  //
  if(-0.00000001 < r12_r32 && r12_r32 < +0.00000001) { cout << "r12_r32 = 0.0 !!" << endl ; return NULL; }
  if(-0.00000001 < r32_r34 && r32_r34 < +0.00000001) { cout << "r32_r34 = 0.0 !!" << endl ; return NULL; }

  n_12_32[0]=r12[1]*r32[2]-r12[2]*r32[1];
  n_12_32[1]=r12[2]*r32[0]-r12[0]*r32[2];
  n_12_32[2]=r12[0]*r32[1]-r12[1]*r32[0];
  n_32_34[0]=r32[1]*r34[2]-r32[2]*r34[1];
  n_32_34[1]=r32[2]*r34[0]-r32[0]*r34[2];
  n_32_34[2]=r32[0]*r34[1]-r32[1]*r34[0];

  nn1232=sqrt(n_12_32[0]*n_12_32[0]+n_12_32[1]*n_12_32[1]+n_12_32[2]*n_12_32[2]);
  nn3234=sqrt(n_32_34[0]*n_32_34[0]+n_32_34[1]*n_32_34[1]+n_32_34[2]*n_32_34[2]);
  n_12_32_n_32_34=n_12_32[0]*n_32_34[0]+n_12_32[1]*n_32_34[1]+n_12_32[2]*n_32_34[2];

  // 
  if(n_12_32_n_32_34/(nn1232*nn3234) >  0.99999999){ return 0.0      ; /*  0.0*/ }
  if(n_12_32_n_32_34/(nn1232*nn3234) < -0.99999999){ return 3.141592 ; /*180.0*/ }

  n[0]=n_12_32[1]*n_32_34[2]-n_12_32[2]*n_32_34[1];
  n[1]=n_12_32[2]*n_32_34[0]-n_12_32[0]*n_32_34[2];
  n[2]=n_12_32[0]*n_32_34[1]-n_12_32[1]*n_32_34[0];

  //cout.setf(ios::fixed,ios::floatfield);
  //cout.precision(10);
  //cout << "n_x : "; cout.width(20);cout << n[0]<< endl ;
  //cout << "n_y : "; cout.width(20);cout << n[0]<< endl ;
  //cout << "n_z : "; cout.width(20);cout << n[0]<< endl ;

  r32n=r32[0]*n[0]+r32[1]*n[1]+r32[2]*n[2];

  //cout.setf(ios::fixed,ios::floatfield);
  //cout.precision(10);
  //cout << "r32n                                  : "; cout.width(20);cout << r32n << endl ;
  //cout << "n_12_32_n_32_34/(nn1232*nn3234)       : "; cout.width(20);cout << n_12_32_n_32_34/(nn1232*nn3234) << endl ;
  //cout << "acos(n_12_32_n_32_34/(nn1232*nn3234)) : "; cout.width(20);cout << acos(n_12_32_n_32_34/(nn1232*nn3234)) << endl ;
 
  if  (r32n < 0.0){return -(-1.0)*acos(n_12_32_n_32_34/(nn1232*nn3234));}
  else            {return -(+1.0)*acos(n_12_32_n_32_34/(nn1232*nn3234));}
}

double Dihedral::calc(const Dvector3D r1,
		      const Dvector3D r2,
		      const Dvector3D r3,
		      const Dvector3D r4)
{
  return calc(r1.get_x(),
	      r1.get_y(),
	      r1.get_z(),
	      r2.get_x(),
	      r2.get_y(),
	      r2.get_z(),
	      r3.get_x(),
	      r3.get_y(),
	      r3.get_z(),
	      r4.get_x(),
	      r4.get_y(),
	      r4.get_z());
}
