
#ifndef _ISHI_DVECTOR3D_H_
#define _ISHI_DVECTOR3D_H_

#include <iostream>

class Dvector3D {
  
 public:
  
  /*-------------------*/
  /*--- constractor ---*/
  /*-------------------*/

  Dvector3D                     (                               )       ;
  Dvector3D                     ( const double     x_           ,
                                  const double     y_           ,
                                  const double     z_           )       ;
  Dvector3D                     ( const Dvector3D& v            )       ;

  /*------------------*/
  /*--- destractor ---*/
  /*------------------*/

  ~Dvector3D                    (                               )       ;

  /*------------*/
  /*--- copy ---*/
  /*------------*/

  void             copy         ( const Dvector3D& R_           )       ;

  /*----------------*/
  /*--- operator ---*/
  /*----------------*/

  void             operator +=  ( const Dvector3D& r            )       ;
  void             operator -=  ( const Dvector3D& r            )       ;
  void             operator +=  ( const double     a            )       ;
  void             operator -=  ( const double     a            )       ;
  void             operator *=  ( const double     a            )       ;
  void             operator /=  ( const double     a            )       ;
  Dvector3D        operator +   ( const Dvector3D& r            ) const ;
  Dvector3D        operator -   ( const Dvector3D& r            ) const ;
  double           operator *   ( const Dvector3D& r            ) const ;
  Dvector3D        operator +   ( const double     a            ) const ;
  Dvector3D        operator -   ( const double     a            ) const ;
  Dvector3D        operator *   ( const double     a            ) const ;
  Dvector3D        operator /   ( const double     a            ) const ;
  
  /*--------------------*/
  /*--- public metod ---*/
  /*--------------------*/
  
  static bool      chkSame      ( const Dvector3D& r1           ,
                                  const Dvector3D& r2           )       ;
  static double    distance1    ( const Dvector3D& r1           ,
                                  const Dvector3D& r2           )       ;
  static double    distance2    ( const Dvector3D& r1           ,
                                  const Dvector3D& r2           )       ;
  static double    norm         ( const Dvector3D& r1           )       ;
  static double    dot          ( const Dvector3D& r1           ,
                                  const Dvector3D& r2           )       ;
  static double    angle        ( const Dvector3D& r1           ,
                                  const Dvector3D& r2           )       ;
  static double    dihedral     ( const Dvector3D& r1           ,
                                  const Dvector3D& r2           ,
                                  const Dvector3D& r3           ,
                                  const Dvector3D& r4           )       ;
  static Dvector3D vertical     ( const Dvector3D& r1           )       ;
  static Dvector3D Vproduct     ( const Dvector3D& r1           ,
                                  const Dvector3D& r2           )       ;
  static Dvector3D rotation_x   ( const Dvector3D& r            ,
                                  const double     ang          )       ;
  static Dvector3D rotation_y   ( const Dvector3D& r            ,
                                  const double     ang          )       ;
  static Dvector3D rotation_z   ( const Dvector3D& r            ,
                                  const double     ang          )       ;
  static Dvector3D rotation_x   ( const Dvector3D& r            ,
                                  const double     ang          ,
                                  const Dvector3D& origin       )       ;
  static Dvector3D rotation_y   ( const Dvector3D& r            ,
                                  const double     ang          ,
                                  const Dvector3D& origin       )       ;
  static Dvector3D rotation_z   ( const Dvector3D& r            ,
                                  const double     ang          ,
                                  const Dvector3D& origin       )       ;
  static Dvector3D rotation     ( const Dvector3D& r            ,
                                  const double     ang          ,
                                  const Dvector3D& axis         )       ;
  static Dvector3D rotation     ( const Dvector3D& r            ,
                                  const double     ang          ,
                                  const Dvector3D& axis         ,
                                  const Dvector3D& origin       )       ;
  static Dvector3D LatticeToXYZ ( const Dvector3D& r            ,
                                  const double*    axis         ,
                                  const double*    angle        ,
                                  Dvector3D*       newAxis=NULL )       ;
  
  void set_x                    ( const double     x_           )       ;
  void set_y                    ( const double     y_           )       ;
  void set_z                    ( const double     z_           )       ;
  void set_xyz                  ( const double     x_           ,
                                  const double     y_           ,
                                  const double     z_           )       ;

  void change_xy                (                               )       ;
  void change_yz                (                               )       ;
  void change_zx                (                               )       ;
  
  double get_x                  (                               ) const ;
  double get_y                  (                               ) const ;
  double get_z                  (                               ) const ;

 private:

  double x ;
  double y ;
  double z ;
};

#endif
