
#include <iostream>
#include <iomanip>
#include <cstdlib>
#include <math.h>
#include "Dvector3D.h"

using namespace std ;

Dvector3D::Dvector3D()
{
  x=0.0 ;
  y=0.0 ;
  z=0.0 ;
}

Dvector3D::Dvector3D(const double x_, const double y_, const double z_)
{
  x=x_ ;
  y=y_ ;
  z=z_ ;
}

Dvector3D::Dvector3D(const Dvector3D& v)
{
  x=v.x;
  y=v.y;
  z=v.z;
}

Dvector3D::~Dvector3D()
{
}

void Dvector3D::operator+=(const Dvector3D& r)
{
  x=x+r.x;
  y=y+r.y;
  z=z+r.z;
}

void Dvector3D::operator-=(const Dvector3D& r)
{
  x=x-r.x;
  y=y-r.y;
  z=z-r.z;
}

void Dvector3D::operator+=(const double a)
{
  x=x+a;
  y=y+a;
  z=z+a;
}

void Dvector3D::operator-=(const double a)
{
  x=x-a;
  y=y-a;
  z=z-a;
}
  
void Dvector3D::operator*=(const double a)
{
  x=x*a;
  y=y*a;
  z=z*a;
}

void Dvector3D::operator/=(const double a)
{
  x=x/a;
  y=y/a;
  z=z/a;
}

Dvector3D Dvector3D::operator+(const Dvector3D& r) const
{
  Dvector3D tmp(*this);
  tmp+=r;
  return tmp;
}

Dvector3D Dvector3D::operator-(const Dvector3D& r) const
{
  Dvector3D tmp(*this);
  tmp-=r;
  return tmp;
}

double Dvector3D::operator*(const Dvector3D& r) const
{
  return x*r.x+y*r.y+z*r.z;
}

Dvector3D Dvector3D::operator+(const double a) const
{
  Dvector3D tmp(*this);
  tmp+=a;
  return tmp;
}

Dvector3D Dvector3D::operator-(const double a) const
{
  Dvector3D tmp(*this);
  tmp-=a;
  return tmp;
}

Dvector3D Dvector3D::operator*(const double a) const
{
  Dvector3D tmp(*this);
  tmp*=a;
  return tmp;
}

Dvector3D Dvector3D::operator/(const double a) const
{
  Dvector3D tmp(*this);
  tmp/=a;
  return tmp;
}

void Dvector3D::set_xyz(const double x_, const double y_, const double z_)
{
  x=x_ ;
  y=y_ ;
  z=z_ ;
}

void Dvector3D::set_x(const double x_)
{
  x=x_ ;
}

void Dvector3D::set_y(const double y_)
{
  y=y_ ;
}

void Dvector3D::set_z(const double z_)
{
  z=z_ ;
}

void Dvector3D::copy(const Dvector3D& R)
{
  if(this==&R) return;
  x=R.x;
  y=R.y;
  z=R.z;
}

void Dvector3D::change_xy()
{
  double tmp;
  tmp=x;
  x=y;
  y=tmp;
}

void Dvector3D::change_yz()
{
  double tmp;
  tmp=y;
  y=z;
  z=tmp;
}

void Dvector3D::change_zx()
{
  double tmp;
  tmp=z;
  z=x;
  x=tmp;
}

double Dvector3D::get_x() const
{
  return x;
}

double Dvector3D::get_y() const
{
  return y;
}

double Dvector3D::get_z() const
{
  return z;
}

bool Dvector3D::chkSame(const Dvector3D& r1, const Dvector3D& r2)
{
  double d=0.0000001;
  double dx, dy, dz;
  dx=r1.x-r2.x;
  dy=r1.y-r2.y;
  dz=r1.z-r2.z;
  if(-d<dx && dx<d && -d<dy && dy<d && -d<dz && dz<d) {
    return true;
  } else {
    return false;
  }
}

double Dvector3D::distance1(const Dvector3D& r1, const Dvector3D& r2)
{
  return (sqrt((r1.x-r2.x)*(r1.x-r2.x)+(r1.y-r2.y)*(r1.y-r2.y)+(r1.z-r2.z)*(r1.z-r2.z)));
}

double Dvector3D::distance2(const Dvector3D& r1, const Dvector3D& r2)
{
  return ((r1.x-r2.x)*(r1.x-r2.x)+(r1.y-r2.y)*(r1.y-r2.y)+(r1.z-r2.z)*(r1.z-r2.z));
}

double Dvector3D::norm(const Dvector3D& r1)
{
  return sqrt(r1.x*r1.x+r1.y*r1.y+r1.z*r1.z);
}  

double Dvector3D::dot(const Dvector3D& r1, const Dvector3D& r2)
{
  return (r1.get_x()*r2.get_x()+r1.get_y()*r2.get_y()+r1.get_z()*r2.get_z());
}

double Dvector3D::angle(const Dvector3D& r1, const Dvector3D& r2)
{
  double tmp=(r1*r2)/((norm(r1))*(norm(r2)));
  if( 0.9999999 < tmp && tmp <  1.0000001 ) return 0.0;
  if(-1.0000001 < tmp && tmp < -0.9999999 ) return 3.14159265359;
  return acos(tmp);
}

double Dvector3D::dihedral(const Dvector3D& r1 ,
                           const Dvector3D& r2 ,
                           const Dvector3D& r3 ,
                           const Dvector3D& r4 )
{
  Dvector3D r12, r13, r14, n1, n2;
  r12=r2-r1;
  r13=r3-r1;
  r14=r4-r1;
  n1=Vproduct(r13,r12);
  n2=Vproduct(r13,r14);
  return angle(n1,n2);
}

Dvector3D Dvector3D::rotation_x(const Dvector3D& r, const double ang)
{
  Dvector3D tmp;
  tmp.set_x(r.x);
  tmp.set_y(r.y*cos(ang)-r.z*sin(ang));
  tmp.set_z(r.y*sin(ang)+r.z*cos(ang));
  return tmp ;
}

Dvector3D Dvector3D::rotation_y(const Dvector3D& r, const double ang)
{
  Dvector3D tmp;
  tmp.set_z(r.z*cos(ang)-r.x*sin(ang));
  tmp.set_y(r.y);
  tmp.set_x(r.z*sin(ang)+r.x*cos(ang));
  return tmp ;
}

Dvector3D Dvector3D::rotation_z(const Dvector3D& r, const double ang)
{
  Dvector3D tmp;
  tmp.set_x(r.x*cos(ang)-r.y*sin(ang));
  tmp.set_y(r.x*sin(ang)+r.y*cos(ang));
  tmp.set_z(r.z);
  return tmp ;
}

Dvector3D Dvector3D::rotation_x(const Dvector3D& r, const double ang, const Dvector3D& origin)
{
  return (Dvector3D::rotation_x(r-origin,ang))+origin;
}

Dvector3D Dvector3D::rotation_y(const Dvector3D& r, const double ang, const Dvector3D& origin)
{
  return (Dvector3D::rotation_y(r-origin,ang))+origin;
}

Dvector3D Dvector3D::rotation_z(const Dvector3D& r, const double ang, const Dvector3D& origin)
{
  return (Dvector3D::rotation_z(r-origin,ang))+origin;
}

Dvector3D Dvector3D::rotation(const Dvector3D& r, const double ang, const Dvector3D& axis)
{
  if(-0.000001 < axis.x && axis.x < 0.0000001 &&
     -0.000001 < axis.y && axis.y < 0.0000001 &&
     -0.000001 < axis.z && axis.z < 0.0000001){
    cerr << "error : Dvector3D::rotation(const Dvector3D& axis, const Dvector3D r2, const double ang) : " ;
    cerr << "axis is zero vector !!" << endl ;
    exit(1);
  }
  if(-0.0000001 < axis.y && axis.y < 0.000001 &&
     -0.0000001 < axis.z && axis.z < 0.000001){
    if(axis.x>=0.0){
      return rotation_x(r,  ang);
    }
    else{
      return rotation_x(r, -ang);
    }
  }
  if(-0.0000001 < axis.x && axis.x < 0.000001 &&
     -0.0000001 < axis.z && axis.z < 0.000001){
    if(axis.y>=0.0){
      return rotation_y(r,  ang);
    }
    else{
      return rotation_y(r, -ang);
    }
  }
  if(-0.000001 < axis.x && axis.x < 0.000001 &&
     -0.000001 < axis.y && axis.y < 0.000001){
    if(axis.z>=0.0){
      return rotation_z(r,  ang);
    }
    else{
      return rotation_z(r, -ang);
    }
  }
  Dvector3D e_x(1.0, 0.0, 0.0);
  Dvector3D e_y(0.0, 1.0, 0.0);
  Dvector3D e_z(0.0, 0.0, 1.0); 
  Dvector3D axis_xy(axis.x, axis.y, 0.0);
  Dvector3D r_tmp1;
  Dvector3D r_tmp2;
  double ang1=angle(axis_xy, e_x);
  double ang2=angle(axis, e_z);
  if(axis.y>=0.0){
    r_tmp1=rotation_y(rotation_z(r, -ang1), -ang2);
    r_tmp2=rotation_z(r_tmp1, ang);
    return rotation_z(rotation_y(r_tmp2, ang2), ang1); 
  }
  else{
    r_tmp1=rotation_y(rotation_z(r, ang1), -ang2);
    r_tmp2=rotation_z(r_tmp1, ang);
    return rotation_z(rotation_y(r_tmp2, ang2), -ang1);
  }
}

Dvector3D Dvector3D::rotation(const Dvector3D& r, const double ang, const Dvector3D& axis, const Dvector3D& origin)
{
  return (Dvector3D::rotation(r-origin,ang,axis))+origin;
}

Dvector3D Dvector3D::vertical(const Dvector3D& r1)
{
  Dvector3D tmp;
  double x=r1.get_x();
  double y=r1.get_y();
  double z=r1.get_z();
  if(x < -0.00001 || 0.00001 < x){
    tmp.set_x((-y-z)/x);
    tmp.set_y(1.0);
    tmp.set_z(1.0);
  }
  else if(y < -0.00001 || 0.00001 < y){
    tmp.set_x(1.0);
    tmp.set_y((-z-x)/y);
    tmp.set_z(1.0);
  }
  else {
    tmp.set_x(1.0);
    tmp.set_y(1.0);
    tmp.set_z((-x-y)/z);
  }
  return tmp/Dvector3D::norm(tmp);
}

Dvector3D Dvector3D::Vproduct(const Dvector3D& r1, const Dvector3D& r2)
{
  Dvector3D tmp;
  tmp.set_x(r1.y*r2.z-r1.z*r2.y);
  tmp.set_y(r1.z*r2.x-r1.x*r2.z);
  tmp.set_z(r1.x*r2.y-r1.y*r2.x);
  return tmp;
}

Dvector3D Dvector3D::LatticeToXYZ(const Dvector3D& r, const double* axis, const double* angle, Dvector3D* newAxis)
{
  Dvector3D axis_x, axis_y, axis_z, tmp;
  double a1, b1, b2, c1, c2, c3;
  a1=axis[0]; 
  axis_x.set_xyz(a1, 0.00, 0.00);
  tmp.set_xyz(axis[1], 0.00, 0.00);
  axis_y.copy(rotation_z(tmp,angle[2]));
  b1=axis_y.x;
  b2=axis_y.y;
  c1=cos(angle[1]);
  c2=(sqrt(b1*b1+b2*b2)*cos(angle[0])-c1*b1)/b2;
  c3=1-c1*c1-c2*c2;
  axis_z.set_xyz(c1*axis[2], c2*axis[2], c3*axis[2]);
  if(newAxis!=NULL){
    newAxis[0].copy(axis_x);
    newAxis[1].copy(axis_y);
    newAxis[2].copy(axis_z);
  }
  tmp=axis_x*r.x+axis_y*r.y+axis_z*r.z;
  return tmp;
}
