
#include <math.h>
#include "Eye.h"

using namespace std ;

Eye::Eye()
{
  fovy=60.0;
  zfar=5000.0;
  znear=0.1;
  R.set_xyz(0.00, 0.00, 10.00);
  eyeAxis_x.set_xyz(1.00, 0.00,  0.00);
  eyeAxis_y.set_xyz(0.00, 1.00,  0.00);
  eyeAxis_z.set_xyz(0.00, 0.00, -1.00);
}

Eye::Eye(const Eye& e)
{
  copy(e);
}

Eye::~Eye()
{
  //cout << "ishi in ~Eye in " << endl ;
  //cout << "ishi in ~Eye out " << endl ;
}

void Eye::copy(const Eye& e)
{
  if(this==&e) return;
  fovy=e.fovy;
  zfar=e.zfar;
  znear=e.znear;
  R.copy(e.R);
  eyeAxis_x.copy(e.eyeAxis_x);
  eyeAxis_y.copy(e.eyeAxis_y);
  eyeAxis_z.copy(e.eyeAxis_z);
}

double Eye::get_zfar( ) const 
{
  return zfar;
}

void Eye::set_zfar( const double zfar_ )
{
  zfar=zfar_;
}

double Eye::get_znear( ) const 
{
  return znear;
}

void Eye::set_znear( const double znear_ )
{
  znear=znear_;
}

double Eye::get_fovy( ) const 
{
  return fovy;
}

void Eye::set_fovy( const double fovy_ )
{
  fovy=fovy_;
}

Dvector3D Eye::R_to_eyeR(const Dvector3D& r) const
{
  Dvector3D tmp1, tmp2;
  tmp1=r-R;
  tmp2.set_xyz(eyeAxis_x*tmp1, eyeAxis_y*tmp1, eyeAxis_z*tmp1);
  return tmp2;
}

Dvector3D Eye::eyeR_to_R(const Dvector3D& r) const
{
  Dvector3D tmp;
  double x_=r.get_x()*eyeAxis_x.get_x()+r.get_y()*eyeAxis_y.get_x()+r.get_z()*eyeAxis_z.get_x();
  double y_=r.get_x()*eyeAxis_x.get_y()+r.get_y()*eyeAxis_y.get_y()+r.get_z()*eyeAxis_z.get_y();
  double z_=r.get_x()*eyeAxis_x.get_z()+r.get_y()*eyeAxis_y.get_z()+r.get_z()*eyeAxis_z.get_z();
  tmp.set_xyz(x_+R.get_x(),y_+R.get_y(),z_+R.get_z());
  return tmp;
}

void Eye::eyeAxis_rotate_eyeAxis_x(const double angle)
{
  eyeAxis_y=Dvector3D::rotation(eyeAxis_y,angle,eyeAxis_x);
  eyeAxis_z=Dvector3D::rotation(eyeAxis_z,angle,eyeAxis_x);
}

void Eye::eyeAxis_rotate_eyeAxis_y(const double angle)
{
  eyeAxis_z=Dvector3D::rotation(eyeAxis_z,angle,eyeAxis_y);
  eyeAxis_x=Dvector3D::rotation(eyeAxis_x,angle,eyeAxis_y);
}

void Eye::eyeAxis_rotate_eyeAxis_z(const double angle)
{
  eyeAxis_x=Dvector3D::rotation(eyeAxis_x,angle,eyeAxis_z);
  eyeAxis_y=Dvector3D::rotation(eyeAxis_y,angle,eyeAxis_z);
}

void Eye::eyeAxis_z_to_r(const Dvector3D& r_)
{
  Dvector3D z(0.00,0.00,1.00);
  Dvector3D r(R_to_eyeR(r_));
  double ang=Dvector3D::angle(r,z);
  if(-0.0000001 < ang && ang < 0.0000001){
    return;
  }
  if(-0.0000001 < ang-180.00 && ang-180.00 < 0.0000001){
    eyeAxis_z=eyeAxis_z*(-1);
    return ;
  }
  Dvector3D n=Dvector3D::Vproduct(r,z);
  eyeAxis_x.copy(Dvector3D::rotation(eyeAxis_x,ang,eyeR_to_R(n)-R));
  eyeAxis_y.copy(Dvector3D::rotation(eyeAxis_y,ang,eyeR_to_R(n)-R));
  eyeAxis_z.copy(Dvector3D::rotation(eyeAxis_z,ang,eyeR_to_R(n)-R));
  eyeAxis_reset_for_z();
}

void Eye::eyeAxis_reset_for_z ( )
{
  Dvector3D rxz=Dvector3D::Vproduct(eyeAxis_x,eyeAxis_z);
  eyeAxis_y.copy(rxz/Dvector3D::norm(rxz));
  Dvector3D rzy=Dvector3D::Vproduct(eyeAxis_z,eyeAxis_y);
  eyeAxis_x.copy(rzy/Dvector3D::norm(rzy));
}

void Eye::translate(const double l, const Dvector3D& to)
{
  R.copy(R+((to/Dvector3D::norm(to))*l));
}

void Eye::translate_x(const double l)
{
  R.set_xyz(R.get_x()+l, R.get_y(), R.get_z());
}

void Eye::translate_y(const double l)
{
  R.set_xyz(R.get_x(), R.get_y()+l, R.get_z());
}

void Eye::translate_z(const double l)
{
  R.set_xyz(R.get_x(), R.get_y(), R.get_z()+l);
}

void Eye::translate_fixed_view(const double l, const Dvector3D& to, const Dvector3D& view_point)
{
  translate(l,to);
  eyeAxis_z_to_r(view_point);
}

void Eye::translate_fixed_view_x(const double l, const Dvector3D& view_point)
{
  translate_x(l);
  eyeAxis_z_to_r(view_point);
}

void Eye::translate_fixed_view_y(const double l, const Dvector3D& view_point)
{
  translate_z(l);
  eyeAxis_z_to_r(view_point);
}

void Eye::translate_fixed_view_z(const double l, const Dvector3D& view_point)
{
  translate_z(l);
  eyeAxis_z_to_r(view_point);
}

void Eye::translate_eyeAxis (const double l, const Dvector3D& to)
{
  translate(l,eyeR_to_R(to)-R);
}

void Eye::translate_eyeAxis_x(const double l)
{
  Dvector3D xtmp(1.0,0.0,0.0);
  translate(l,eyeR_to_R(xtmp)-R);
}

void Eye::translate_eyeAxis_y(const double l)
{
  Dvector3D ytmp(0.0,1.0,0.0);
  translate(l,eyeR_to_R(ytmp)-R);
}

void Eye::translate_eyeAxis_z(const double l)
{
  Dvector3D ztmp(0.0,0.0,1.0);
  translate(l,eyeR_to_R(ztmp)-R);
}

void Eye::translate_eyeAxis_fixed_view(const double l, const Dvector3D& to, const Dvector3D& view_point)
{
  translate_eyeAxis(l,to);
  eyeAxis_z_to_r(view_point);
}

void Eye::translate_eyeAxis_fixed_view_x(const double l, const Dvector3D& view_point)
{
  translate_eyeAxis_x(l);
  eyeAxis_z_to_r(view_point);
}

void Eye::translate_eyeAxis_fixed_view_y(const double l, const Dvector3D& view_point)
{
  translate_eyeAxis_y(l);
  eyeAxis_z_to_r(view_point);
}

void Eye::translate_eyeAxis_fixed_view_z(const double l, const Dvector3D& view_point)
{
  translate_eyeAxis_z(l);
  eyeAxis_z_to_r(view_point);
}

void Eye::rotate_x(const double ang_, const Dvector3D& origin)
{
  Dvector3D tmp;
  tmp=Dvector3D::rotation_x(R,ang_,origin);
  R=tmp;
}

void Eye::rotate_y(const double ang_, const Dvector3D& origin)
{
  Dvector3D tmp;
  tmp=Dvector3D::rotation_y(R,ang_,origin);
  R=tmp;
}

void Eye::rotate_z(const double ang_, const Dvector3D& origin)
{
  Dvector3D tmp;
  tmp=Dvector3D::rotation_z(R,ang_,origin);
  R=tmp;
}

void Eye::rotate(const double ang_, const Dvector3D& axis, const Dvector3D& origin)
{
  Dvector3D tmp;
  tmp=Dvector3D::rotation(R,ang_,axis,origin);
  R=tmp;
}

void Eye::rotate_fixed_view_x ( const double ang_ , const Dvector3D& origin, const Dvector3D& view_point)
{
  rotate_x(ang_,origin);
  eyeAxis_z_to_r(view_point);
}

void Eye::rotate_fixed_view_y ( const double ang_ , const Dvector3D& origin, const Dvector3D& view_point)
{
  rotate_y(ang_,origin);
  eyeAxis_z_to_r(view_point);
}

void Eye::rotate_fixed_view_z ( const double ang_ , const Dvector3D& origin , const Dvector3D& view_point)
{
  rotate_z(ang_,origin);
  eyeAxis_z_to_r(view_point);
}

void Eye::rotate_fixed_view ( const double ang_ , const Dvector3D& axis, const Dvector3D& origin , const Dvector3D& view_point)
{
  rotate(ang_,axis,origin);
  eyeAxis_z_to_r(view_point);
}

void Eye::rotate_eyeAxis_x ( const double ang_ , const Dvector3D& origin )
{
  Dvector3D tmp(1.00, 0.00, 0.00);
  rotate(ang_, eyeR_to_R(tmp)-R, origin);
}

void Eye::rotate_eyeAxis_y ( const double ang_ , const Dvector3D& origin )
{
  Dvector3D tmp(0.00, 1.00, 0.00);
  rotate(ang_, eyeR_to_R(tmp)-R, origin);
}

void Eye::rotate_eyeAxis_z ( const double ang_ , const Dvector3D& origin )
{
  Dvector3D tmp(0.00, 0.00, 1.00);
  rotate(ang_, eyeR_to_R(tmp)-R, origin);
}

void Eye::rotate_eyeAxis ( const double ang_ , const Dvector3D& axis, const Dvector3D& origin)
{
  rotate(ang_, eyeR_to_R(axis)-R, origin);
}

void Eye::rotate_eyeAxis_fixed_view_x ( const double ang_ , const Dvector3D& origin , const Dvector3D& view_point )
{
  Dvector3D tmp(1.00, 0.00, 0.00);
  rotate_fixed_view(ang_, eyeR_to_R(tmp)-R, origin, view_point);
}

void Eye::rotate_eyeAxis_fixed_view_y ( const double ang_ , const Dvector3D& origin , const Dvector3D& view_point )
{
  Dvector3D tmp(0.00, 1.00, 0.00);
  rotate_fixed_view(ang_, eyeR_to_R(tmp)-R, origin, view_point);
}

void Eye::rotate_eyeAxis_fixed_view_z ( const double ang_ , const Dvector3D& origin , const Dvector3D& view_point )
{
  Dvector3D tmp(0.00, 0.00, 1.00);
  rotate_fixed_view(ang_, eyeR_to_R(tmp)-R, origin, view_point);
}

void Eye::rotate_eyeAxis_fixed_view ( const double ang_ , const Dvector3D& axis, const Dvector3D& origin , const Dvector3D& view_point)
{
  rotate_fixed_view(ang_, eyeR_to_R(axis)-R, origin, view_point);
}

Dvector3D Eye::get_R() const
{
  return R;
}

Dvector3D Eye::get_eyeAxis_x () const
{
  return eyeAxis_x ;
}

Dvector3D Eye::get_eyeAxis_y () const
{
  return eyeAxis_y ;
}

Dvector3D Eye::get_eyeAxis_z () const
{
  return eyeAxis_z ;
}

void Eye::set_R ( const Dvector3D& R_ )
{
  R.copy(R_) ;
}

void Eye::add_R ( const Dvector3D& R_ )
{
  R=R+R_;
}

void Eye::set_eyeAxis_x ( const Dvector3D& x_ )
{
  eyeAxis_x.copy(x_) ;
}

void Eye::set_eyeAxis_y ( const Dvector3D& y_ )
{
  eyeAxis_y.copy(y_) ;
}

void Eye::set_eyeAxis_z ( const Dvector3D& z_ )
{
  eyeAxis_z.copy(z_) ;
}
