
#ifndef _ISHI_FORCE_FIELD_H_
#define _ISHI_FORCE_FIELD_H_

#include <string>
#include "Dvector3D.h"
#include "ViewAtom.h"
#include "Molecule.h"

using namespace std ;

class ForceField{

 public:
  
  ForceField (                     );
  ForceField ( const ForceField& f );

  ~ForceField ();

  void          copy                                  ( const ForceField& m               );
  void          copy_atm                              ( const ForceField& m               );
  void          copy_bnd                              ( const ForceField& m               );
  void          copy_ang                              ( const ForceField& m               );
  void          copy_dih                              ( const ForceField& m               );
  void          copy_p14                              ( const ForceField& m               );

  void          clear                                 (                                   );
  void          atm_clear                             (                                   );
  void          bnd_clear                             (                                   );
  void          ang_clear                             (                                   );
  void          dih_clear                             (                                   );
  void          p14_clear                             (                                   );

  int           get_natm                              (                                   ) const ;
  int           get_atm1                              ( const int ia                      ) const ;
  int           get_atm_paics                         ( const int ia                      ) const ;
  string        get_atm_amber                         ( const int ia                      ) const ;
  string        get_atm_pdb                           ( const int ia                      ) const ;
  double        get_atm_chg                           ( const int ia                      ) const ;
  double        get_atm_vdw_r                         ( const int ia                      ) const ;
  double        get_atm_vdw_e                         ( const int ia                      ) const ;
  
  void          push_back_atm                         ( const int         atm1_           ,
                                                        const int         atm_paics_      ,
                                                        const string      atm_amber_      ,
                                                        const string      atm_pdb_        ,
                                                        const double      atm_chg_        ,
                                                        const double      atm_vdw_r_      ,
                                                        const double      atm_vdw_e_      ,
                                                        const int         atm_excluded_n_ ,
                                                        const int *       atm_excluded_   ,
                                                        const int         inc=10000       );
  void          push_back_bnd                         ( const int         bnd1_           ,
                                                        const int         bnd2_           ,
                                                        const double      bnd_coeff_      ,
                                                        const double      bnd_eq_         ,
                                                        const int         inc=10000       );
  void          push_back_ang                         ( const int         ang1_           ,
                                                        const int         ang2_           ,
                                                        const int         ang3_           ,
                                                        const double      ang_coeff_      ,
                                                        const double      ang_eq_         ,
                                                        const int         inc=10000       );
  void          push_back_dih                         ( const int         dih1_           ,
                                                        const int         dih2_           ,
                                                        const int         dih3_           ,
                                                        const int         dih4_           ,
                                                        const double      dih_coeff_      ,
                                                        const double      dih_div_        ,
                                                        const double      dih_eq_         ,
                                                        const double      dih_phase_      ,
                                                        const int         dih_improper_   ,
                                                        const int         inc=10000       );
  void          push_back_p14                         ( const int         p14_1_          ,
                                                        const int         p14_2_          ,
                                                        const int         inc=10000       );

  ///////////////////////////////////////
  ////////// AMBER FORCE FIELD //////////
  ///////////////////////////////////////

  static void   amber_force_field_get_atom_info       ( const Molecule&   m               ,
                                                        const int         iatom           ,
                                                        int             * ptyp            ,
                                                        string          * pdb             ,
                                                        int             * ibu             ,
                                                        int             * res_type        ,
                                                        int             * t               ,
                                                        int             * his             ,
                                                        string          * amber           ) ;//const ;
  void          amber_force_field_make_param_atom     ( const Molecule&   m               );
  void          amber_force_field_make_param_bond     ( const Molecule&   m               );
  void          amber_force_field_make_param_angle    ( const Molecule&   m               );
  void          amber_force_field_make_param_dihedral ( const Molecule&   m               );
  void          amber_force_field_make_14_pair        ( const Molecule&   m               );
  static int    amber_force_field_charge              ( const string      a1              ,
                                                        const string      pdb             ,
                                                        const int         res_type        ,
                                                        const int         terminal        ,
                                                        double          * out_ch          ,
                                                        const int         info_1=0        ); // info_1 --> 0:HID, 1:HIE, 2:HIP
  int           amber_force_field_vdw                 ( const string      a1              ,
                                                        double          * out_r           ,
                                                        double          * out_e           );
  int           amber_force_field_bond                ( const string      a1              ,
                                                        const string      a2              ,
                                                        double          * out_coeff       ,
                                                        double          * out_eq          );
  int           amber_force_field_angle               ( const string      a1              ,
                                                        const string      a2              ,
                                                        const string      a3              ,
                                                        double          * out_coeff       ,
                                                        double          * out_eq          );
  int           amber_force_field_dihedral            ( const string      a1              ,
                                                        const string      a2              ,
                                                        const string      a3              ,
                                                        const string      a4              ,
                                                        const int         improper        ,
                                                        int             * out_n           ,
                                                        double          * out_coeff       ,
                                                        double          * div             ,
                                                        double          * out_eq          ,
                                                        double          * out_phase       );
  void          amber_force_field_get_atm_coeff_A     ( double *          coeff_A_        ) const ;
  void          amber_force_field_get_atm_coeff_B     ( double *          coeff_B_        ) const ;
  void          write_amber_force_field_paics         ( const string      filename        ) const ;
  string        get_amber_force_field_paics           (                                   ) const ; 
  void          amber_force_field_calc_energy         ( const Molecule&   m               ,
                                                        double          * bnd_eng         ,
                                                        double          * ang_eng         ,
                                                        double          * dih_eng         ,
                                                        double          * chg_eng_exc     ,
                                                        double          * chg_eng_14p     ,
                                                        double          * vdw_eng_exc     ,
                                                        double          * vdw_eng_14p     ,
                                                        const double    * atm_coeff_A=NULL,
                                                        const double    * atm_coeff_B=NULL) const ;
                                                        //const int         chk_bnd=1       ,
                                                        //const int         chk_ang=1       ,
                                                        //const int         chk_dih=1       ,
                                                        //const int         chk_exc=1       ,
                                                        //const int         chk_14p=1       ,  
                                                        //const int         calc_type=0     ,
                                                        //const double      max=1e+99       ) const ;
  void          amber_force_field_calc_energy         ( const int         n               ,
                                                        const double    * x               ,
                                                        const double    * y               ,
                                                        const double    * z               ,
                                                        double          * bnd_eng         ,
                                                        double          * ang_eng         ,
                                                        double          * dih_eng         ,
                                                        double          * chg_eng_exc     ,
                                                        double          * chg_eng_14p     ,
                                                        double          * vdw_eng_exc     ,
                                                        double          * vdw_eng_14p     ,
                                                        const double    * atm_coeff_A=NULL,
                                                        const double    * atm_coeff_B=NULL) const ;
                                                        //const int         chk_bnd=1       ,
                                                        //const int         chk_ang=1       ,
                                                        //const int         chk_dih=1       ,
                                                        //const int         chk_exc=1       ,
                                                        //const int         chk_14p=1       ,  
                                                        //const int         calc_type=0     ,
                                                        //const double      max=1e+99       ) const ;
  void          amber_force_field_calc_gradient       ( const Molecule&   m               ,
                                                        double          * grad_x          ,
                                                        double          * grad_y          ,
                                                        double          * grad_z          ) const ;
  void          amber_force_field_calc_gradient       ( const double    * x               ,
                                                        const double    * y               ,
                                                        const double    * z               ,
                                                        double          * grad_x          ,
                                                        double          * grad_y          ,
                                                        double          * grad_z          ) const ;

  static string amber_force_field_get_atom_type       ( const int         atom_index      ,
                                                        const int         term=0          ,
                                                        const int         info_1=0        ); // info_1 --> 0:HID, 1:HIE, 2:HIP
  static void   amber_force_field_get_improper        ( const int         rt              ,
                                                        const int         tn              ,
                                                        const int         tc              ,
                                                        const int         his             ,
                                                        int             * n               ,
                                                        int             * a1              ,
                                                        int             * a2              ,
                                                        int             * a3              ,
                                                        int             * a4              ,
                                                        int             * b1              ,
                                                        int             * b2              ,
                                                        int             * b3              ,
                                                        int             * b4              );

  //////////////////////////////////////////
  ////////// AUTODOCK FORCE FIELD //////////
  //////////////////////////////////////////

  static string autodock_get_atom_type                ( const int         ai              );
  static int    autodock_charge                       ( const string      a1              ,
                                                        const string      pdb_in          ,
                                                        const int         res_type        ,
                                                        const int         terminal_       ,
                                                        double          * out_ch          ,
                                                        const int         info_1          );
  static int    autodock_field_vdw                    ( const string      atype           ,
                                                        double          * out_r           ,
                                                        double          * out_e           );


  /*
  void          copy_def_atm                          ( const ForceField& m               );
  void          copy_def_bnd                          ( const ForceField& m               );
  void          copy_def_ang                          ( const ForceField& m               );
  void          copy_def_dih                          ( const ForceField& m               );
  void          def_atm_clear                         (                                   );
  void          def_bnd_clear                         (                                   );
  void          def_ang_clear                         (                                   );
  void          def_dih_clear                         (                                   );
  void          push_back_def_atm                     ( const string      atm_amber_      ,
                                                        const double      atm_vdw_r_      ,
                                                        const double      atm_vdw_e_      ,
                                                        const int         inc=100         );
  void          push_back_def_bnd                     ( const string      bnd1_           ,
                                                        const string      bnd2_           ,
                                                        const double      bnd_coeff_      ,
                                                        const double      bnd_eq_         ,
                                                        const int         inc=100         );
  void          push_back_def_ang                     ( const string      ang1_           ,
                                                        const string      ang2_           ,
                                                        const string      ang3_           ,
                                                        const double      ang_coeff_      ,
                                                        const double      ang_eq_         ,
                                                        const int         inc=100         );
  void          push_back_def_dih                     ( const string      dih1_           ,
                                                        const string      dih2_           ,
                                                        const string      dih3_           ,
                                                        const string      dih4_           ,
                                                        const double      dih_div_        ,
                                                        const double      dih_coeff_      ,
                                                        const double      dih_eq_         ,
                                                        const double      dih_phase_      ,
                                                        const int         dih_improper_   ,
                                                        const int         inc=100         );
  void          read_force_field_param_define         ( const string&     filename        )       ;
  int           amber_force_field_vdw_from_define     ( const string      a1              ,
                                                        double          * out_r           ,
                                                        double          * out_e           ) const ;
  */
  
 private:

  int          natm_buff;
  int          natm;
  int       *  atm1;                // [natm] ( <--- 0,1,2,3,4.....)
  int       *  atm_paics;           // [natm]
  string    *  atm_amber;           // [natm]
  string    *  atm_pdb;             // [natm]
  double    *  atm_chg;             // [natm]
  double    *  atm_vdw_r;           // [natm]
  double    *  atm_vdw_e;           // [natm]
  int       *  atm_excluded_n;      // [natm]
  int       *  atm_excluded_n_buff; // [natm]
  int       ** atm_excluded;

  void atm_create          ( const int n  );
  void atm_resize          ( const int n  );
  void atm_excluded_clear  ( const int ai );
  void atm_excluded_create ( const int ai ,
                             const int n  );
  void atm_excluded_resize ( const int ai ,
                             const int n  );

  int         nbnd_buff;
  int         nbnd;
  int       * bnd1;                 // [nbnd]
  int       * bnd2;                 // [nbnd]
  double    * bnd_coeff;            // [nbnd]
  double    * bnd_eq;               // [nbnd]

  void bnd_create ( const int n );
  void bnd_resize ( const int n );

  int         nang_buff;
  int         nang;
  int       * ang1;                 // [nang]
  int       * ang2;                 // [nang]
  int       * ang3;                 // [nang]
  double    * ang_coeff;            // [nang]
  double    * ang_eq;               // [nang]

  void ang_create ( const int n );
  void ang_resize ( const int n );
  
  int         ndih_buff;
  int         ndih; 
  int       * dih1;                 // [ndih]
  int       * dih2;                 // [ndih]
  int       * dih3;                 // [ndih]
  int       * dih4;                 // [ndih]
  double    * dih_coeff;            // [ndih] 
  double    * dih_div;              // [ndih] <--- dih_coeff / dih_div = force constant @ amber topology file
  double    * dih_eq;               // [ndih] <--- phase                                @ amber topology file
  double    * dih_phase;            // [ndih] <--- periodicity                          @ amber topology file 
  int       * dih_improper;         // [ndih]

  void dih_create ( const int n );
  void dih_resize ( const int n );

  int         np14_buff;
  int         np14;
  int       * p14_1;                // [np14]
  int       * p14_2;                // [np14]

  void p14_create ( const int n );
  void p14_resize ( const int n );

  static int d4_  ( const string a1         ,
                    const string a2         ,
                    const string a3         ,
                    const string a4         ,
                    const string b1         ,
                    const string b2         ,
                    const string b3         ,
                    const string b4         ,
                    const int    improper=0 );

  /*
  int          def_natm_buff;
  int          def_natm;
  string    *  def_atm_amber;       // [natm]
  double    *  def_atm_vdw_r;       // [natm]
  double    *  def_atm_vdw_e;       // [natm]

  void def_atm_create ( const int n );
  void def_atm_resize ( const int n );

  int          def_nbnd_buff;
  int          def_nbnd;
  string    *  def_bnd1;            // [nbnd]
  string    *  def_bnd2;            // [nbnd]
  double    *  def_bnd_coeff;       // [nbnd]
  double    *  def_bnd_eq;          // [nbnd]

  void def_bnd_create ( const int n );
  void def_bnd_resize ( const int n );

  int          def_nang_buff;
  int          def_nang;
  string    *  def_ang1;            // [nang]
  string    *  def_ang2;            // [nang]
  string    *  def_ang3;            // [nang]
  double    *  def_ang_coeff;       // [nang]
  double    *  def_ang_eq;          // [nang]

  void def_ang_create ( const int n );
  void def_ang_resize ( const int n );

  int          def_ndih_buff;
  int          def_ndih; 
  string    *  def_dih1;            // [ndih]
  string    *  def_dih2;            // [ndih]
  string    *  def_dih3;            // [ndih]
  string    *  def_dih4;            // [ndih]
  double    *  def_dih_coeff;       // [ndih]
  double    *  def_dih_div;         // [ndih]
  double    *  def_dih_eq;          // [ndih]
  double    *  def_dih_phase;       // [ndih]
  int       *  def_dih_improper;    // [ndih]

  void def_dih_create ( const int n );
  void def_dih_resize ( const int n );
  */
};

#endif
