
#include <cstdlib>
#include <cstdio>
#include <cmath>
#include <iostream>
#include <fstream>
#include <sstream>
#include "ForceField.h"
#include "iString.h"
#include "Molecule.h"

ForceField::ForceField ()
{
  natm_buff=0;
  natm=0;
  nbnd_buff=0;
  nbnd=0;
  nang_buff=0;
  nang=0;
  ndih_buff=0;
  ndih=0;
  np14_buff=0;
  np14=0;
  /*
  def_natm_buff=0;
  def_natm=0;
  def_nbnd_buff=0;
  def_nbnd=0;
  def_nang_buff=0;
  def_nang=0;
  def_ndih_buff=0;
  def_ndih=0;
  */
}

ForceField::ForceField ( const ForceField& f )
{
  natm_buff=0;
  natm=0;
  nbnd_buff=0;
  nbnd=0;
  nang_buff=0;
  nang=0;
  ndih_buff=0;
  ndih=0;
  np14_buff=0;
  np14=0;
  /*
  def_natm_buff=0;
  def_natm=0;
  def_nbnd_buff=0;
  def_nbnd=0;
  def_nang_buff=0;
  def_nang=0;
  def_ndih_buff=0;
  def_ndih=0;
  */
  copy(f);
}

ForceField::~ForceField ()
{
  clear();
}

void ForceField::copy ( const ForceField& f )
{
  clear();
  copy_atm(f);
  copy_bnd(f);
  copy_ang(f);
  copy_dih(f);
  copy_p14(f);
}

void ForceField::copy_atm( const ForceField& f)
{
  atm_create(f.natm);
  for(int i=0;i<f.natm;i++){
    atm_excluded_create(i,f.atm_excluded_n[i]);
  }
  for(int i=0;i<f.natm;i++){
    atm1     [i]=f.atm1     [i];
    atm_paics[i]=f.atm_paics[i];
    atm_amber[i]=f.atm_amber[i];
    atm_pdb  [i]=f.atm_pdb  [i];
    atm_chg  [i]=f.atm_chg  [i];
    atm_vdw_r[i]=f.atm_vdw_r[i];
    atm_vdw_e[i]=f.atm_vdw_e[i];
    for(int j=0;j<f.atm_excluded_n[i];j++){
      atm_excluded[i][j]=f.atm_excluded[i][j];
      atm_excluded_n[i]++;
    }
    natm++;
  }
}

void ForceField::copy_bnd( const ForceField& f )
{
  bnd_create(f.nbnd);
  for(int i=0;i<f.nbnd;i++){
    bnd1     [i]=f.bnd1     [i];
    bnd2     [i]=f.bnd2     [i];
    bnd_coeff[i]=f.bnd_coeff[i];
    bnd_eq   [i]=f.bnd_eq   [i];
    nbnd++;
  }
}

void ForceField::copy_ang( const ForceField& f )
{
  ang_create(f.nang);
  for(int i=0;i<f.nang;i++){
    ang1     [i]=f.ang1     [i];
    ang2     [i]=f.ang2     [i];
    ang3     [i]=f.ang3     [i];
    ang_coeff[i]=f.ang_coeff[i];
    ang_eq   [i]=f.ang_eq   [i];
    nang++;
  }
}

void ForceField::copy_dih( const ForceField& f )
{
  dih_create(f.ndih);
  for(int i=0;i<f.ndih;i++){
    dih1        [i]=f.dih1        [i];
    dih2        [i]=f.dih2        [i];
    dih3        [i]=f.dih3        [i];
    dih4        [i]=f.dih4        [i];
    dih_coeff   [i]=f.dih_coeff   [i];
    dih_eq      [i]=f.dih_eq      [i];
    dih_div     [i]=f.dih_div     [i];
    dih_phase   [i]=f.dih_phase   [i];
    dih_improper[i]=f.dih_improper[i];
    ndih++;
  }
}

void ForceField::copy_p14( const ForceField& f )
{
  p14_create(f.np14);
  for(int i=0;i<f.np14;i++){
    p14_1[i]=f.p14_1[i];
    p14_2[i]=f.p14_2[i];
    np14++;
  }
}

void ForceField::clear ( )
{
  atm_clear();
  bnd_clear();
  ang_clear();
  dih_clear();
  p14_clear();
  /*
  def_atm_clear();
  def_bnd_clear();
  def_ang_clear();
  def_dih_clear();
  */
}

void ForceField::atm_clear()
{
  if(natm_buff==0) return ;
  delete [] atm1;
  delete [] atm_paics;
  delete [] atm_amber;
  delete [] atm_pdb;
  delete [] atm_chg;
  delete [] atm_vdw_r;
  delete [] atm_vdw_e;
  for(int i=0;i<natm;i++){
    atm_excluded_clear(i);
  }
  delete [] atm_excluded_n;
  delete [] atm_excluded_n_buff;
  delete [] atm_excluded;
  natm_buff=0;
  natm=0;
}

void ForceField::atm_create(const int n)
{
  atm_clear();
  if(n==0) return ;
  atm1                = new int    [n];
  atm_paics           = new int    [n];
  atm_amber           = new string [n];
  atm_pdb             = new string [n];
  atm_chg             = new double [n];
  atm_vdw_r           = new double [n];
  atm_vdw_e           = new double [n];
  atm_excluded_n      = new int    [n];
  atm_excluded_n_buff = new int    [n];
  atm_excluded        = new int*   [n];
  for(int i=0;i<n;i++){
    atm_excluded_n_buff[i]=0;
    atm_excluded_n     [i]=0;
  }
  natm_buff=n;
}

void ForceField::atm_resize(const int n)
{
  int natm_tmp = natm;
  int    *  atm1_tmp           = new int    [natm+1];
  int    *  atm_paics_tmp      = new int    [natm+1]; 
  string *  atm_amber_tmp      = new string [natm+1];
  string *  atm_pdb_tmp        = new string [natm+1];
  double *  atm_chg_tmp        = new double [natm+1];
  double *  atm_vdw_r_tmp      = new double [natm+1];
  double *  atm_vdw_e_tmp      = new double [natm+1];
  int    *  atm_excluded_n_tmp = new int    [natm+1];
  int    ** atm_excluded_tmp   = new int *  [natm+1];
  for(int i=0;i<natm;i++){
    atm1_tmp          [i] = atm1          [i];
    atm_paics_tmp     [i] = atm_paics     [i];
    atm_amber_tmp     [i] = atm_amber     [i];
    atm_pdb_tmp       [i] = atm_pdb       [i];
    atm_chg_tmp       [i] = atm_chg       [i];
    atm_vdw_r_tmp     [i] = atm_vdw_r     [i];
    atm_vdw_e_tmp     [i] = atm_vdw_e     [i];
    atm_excluded_n_tmp[i] = atm_excluded_n[i];
    atm_excluded_tmp  [i] = new int    [atm_excluded_n[i]+1];
    for(int j=0;j<atm_excluded_n[i];j++){
      atm_excluded_tmp[i][j]=atm_excluded[i][j];
    }
  }
  atm_create(n);
  if(n>natm_tmp){
    for(int i=0;i<natm_tmp;i++){
      atm1          [i] = atm1_tmp          [i];
      atm_paics     [i] = atm_paics_tmp     [i];
      atm_amber     [i] = atm_amber_tmp     [i];
      atm_pdb       [i] = atm_pdb_tmp       [i];
      atm_chg       [i] = atm_chg_tmp       [i];
      atm_vdw_r     [i] = atm_vdw_r_tmp     [i];
      atm_vdw_e     [i] = atm_vdw_e_tmp     [i];
      atm_excluded_n[i] = atm_excluded_n_tmp[i];
      atm_excluded_create(i,atm_excluded_n_tmp[i]);
      for(int j=0;j<atm_excluded_n_tmp[i];j++){
        atm_excluded[i][j]=atm_excluded_tmp[i][j];
      }
      natm++;
    }
  } else {
    for(int i=0;i<n;i++){
      atm1          [i] = atm1_tmp          [i];
      atm_paics     [i] = atm_paics_tmp     [i];
      atm_amber     [i] = atm_amber_tmp     [i];
      atm_pdb       [i] = atm_pdb_tmp       [i];
      atm_chg       [i] = atm_chg_tmp       [i];
      atm_vdw_r     [i] = atm_vdw_r_tmp     [i];
      atm_vdw_e     [i] = atm_vdw_e_tmp     [i];
      atm_excluded_n[i] = atm_excluded_n_tmp[i];
      atm_excluded_create(i,atm_excluded_n_tmp[i]);
      for(int j=0;j<atm_excluded_n_tmp[i];j++){
        atm_excluded[i][j]=atm_excluded_tmp[i][j];
      }
      natm++;
    }
  }
  for(int i=0;i<natm_tmp;i++){
    delete [] atm_excluded_tmp[i];
  }
  delete [] atm1_tmp;
  delete [] atm_paics_tmp;
  delete [] atm_amber_tmp;
  delete [] atm_pdb_tmp;
  delete [] atm_chg_tmp;
  delete [] atm_vdw_r_tmp;
  delete [] atm_vdw_e_tmp;
  delete [] atm_excluded_n_tmp;
  delete [] atm_excluded_tmp;
}

void ForceField::atm_excluded_clear (const int ai)
{
  if(atm_excluded_n_buff[ai]==0) return ;
  delete [] atm_excluded[ai] ;
  atm_excluded_n_buff[ai]=0;
  atm_excluded_n     [ai]=0;
}

void ForceField::atm_excluded_create(const int ai,
                                     const int n)
{
  atm_excluded_clear(ai);
  if(n==0) return ;
  atm_excluded[ai]=new int [n];
  atm_excluded_n_buff[ai]=n;
}

void ForceField::atm_excluded_resize(const int ai,
                                     const int n)
{
  int   n_tmp=atm_excluded_n[ai];
  int * tmp;
  tmp = new int [n_tmp+1];
  for(int i=0;i<n_tmp;i++){
    tmp[i]=atm_excluded[ai][i];
  }
  atm_excluded_create(ai,n);
  if(n>n_tmp){
    for(int i=0;i<n_tmp;i++){
      atm_excluded[ai][i]=tmp[i];
      atm_excluded_n[ai]++;
    }
  } else {
    for(int i=0;i<n;i++){
      atm_excluded[ai][i]=tmp[i];
      atm_excluded_n[ai]++;
    }
  }
  delete [] tmp;
}

void ForceField::bnd_clear()
{
  if(nbnd_buff==0) return ;
  delete [] bnd1;
  delete [] bnd2;
  delete [] bnd_coeff;
  delete [] bnd_eq;
  nbnd_buff=0;
  nbnd=0;
}

void ForceField::bnd_create(const int n)
{
  bnd_clear();
  if(n==0) return ;
  bnd1      = new int    [n];
  bnd2      = new int    [n];
  bnd_coeff = new double [n];
  bnd_eq    = new double [n];
  nbnd_buff = n;
}

void ForceField::bnd_resize(const int n)
{
  int nbnd_tmp=nbnd;
  int    * bnd1_tmp      = new int    [nbnd+1];
  int    * bnd2_tmp      = new int    [nbnd+1];
  double * bnd_coeff_tmp = new double [nbnd+1];
  double * bnd_eq_tmp    = new double [nbnd+1];
  for(int i=0;i<nbnd;i++){
    bnd1_tmp     [i]=bnd1     [i];
    bnd2_tmp     [i]=bnd2     [i];
    bnd_coeff_tmp[i]=bnd_coeff[i];
    bnd_eq_tmp   [i]=bnd_eq   [i];
  }
  bnd_create(n);
  if(n>nbnd_tmp){
    for(int i=0;i<nbnd_tmp;i++){
      bnd1     [i]=bnd1_tmp     [i];
      bnd2     [i]=bnd2_tmp     [i];
      bnd_coeff[i]=bnd_coeff_tmp[i];
      bnd_eq   [i]=bnd_eq_tmp   [i];
      nbnd++;
    }
  } else {
    for(int i=0;i<n;i++){
      bnd1     [i]=bnd1_tmp     [i];
      bnd2     [i]=bnd2_tmp     [i];
      bnd_coeff[i]=bnd_coeff_tmp[i];
      bnd_eq   [i]=bnd_eq_tmp   [i];
      nbnd++;
    }
  }
  delete [] bnd1_tmp;
  delete [] bnd2_tmp;
  delete [] bnd_coeff_tmp;
  delete [] bnd_eq_tmp;
}

void ForceField::ang_clear()
{
  if(nang_buff==0) return ;
  delete [] ang1;
  delete [] ang2;
  delete [] ang3;
  delete [] ang_coeff;
  delete [] ang_eq;
  nang_buff=0;
  nang=0;
}

void ForceField::ang_create(const int n)
{
  ang_clear();
  if(n==0) return ;
  ang1      = new int    [n];
  ang2      = new int    [n];
  ang3      = new int    [n];
  ang_coeff = new double [n];
  ang_eq    = new double [n];
  nang_buff = n;
}

void ForceField::ang_resize(const int n)
{
  int nang_tmp=nang;
  int    * ang1_tmp      = new int    [nang+1];
  int    * ang2_tmp      = new int    [nang+1];
  int    * ang3_tmp      = new int    [nang+1];
  double * ang_coeff_tmp = new double [nang+1];
  double * ang_eq_tmp    = new double [nang+1];
  for(int i=0;i<nang;i++){
    ang1_tmp     [i]=ang1     [i];
    ang2_tmp     [i]=ang2     [i];
    ang3_tmp     [i]=ang3     [i];
    ang_coeff_tmp[i]=ang_coeff[i];
    ang_eq_tmp   [i]=ang_eq   [i];
  }
  ang_create(n);
  if(n>nang_tmp){
    for(int i=0;i<nang_tmp;i++){
      ang1     [i]=ang1_tmp     [i];
      ang2     [i]=ang2_tmp     [i];
      ang3     [i]=ang3_tmp     [i];
      ang_coeff[i]=ang_coeff_tmp[i];
      ang_eq   [i]=ang_eq_tmp   [i];
      nang++;
    }
  } else {
    for(int i=0;i<n;i++){
      ang1     [i]=ang1_tmp     [i];
      ang2     [i]=ang2_tmp     [i];
      ang3     [i]=ang3_tmp     [i];
      ang_coeff[i]=ang_coeff_tmp[i];
      ang_eq   [i]=ang_eq_tmp   [i];
      nang++;
    }
  }
  delete [] ang1_tmp;
  delete [] ang2_tmp;
  delete [] ang3_tmp;
  delete [] ang_coeff_tmp;
  delete [] ang_eq_tmp;
}

void ForceField::dih_clear()
{
  if(ndih_buff==0) return ;
  delete [] dih1;
  delete [] dih2;
  delete [] dih3;
  delete [] dih4;
  delete [] dih_coeff;
  delete [] dih_eq;
  delete [] dih_div;
  delete [] dih_phase;
  delete [] dih_improper;
  ndih_buff=0;
  ndih=0;
}

void ForceField::dih_create(const int n)
{
  dih_clear();
  if(n==0) return ;
  dih1         = new int    [n];
  dih2         = new int    [n];
  dih3         = new int    [n];
  dih4         = new int    [n];
  dih_coeff    = new double [n];
  dih_eq       = new double [n];
  dih_div      = new double [n];
  dih_phase    = new double [n];
  dih_improper = new int    [n];
  ndih_buff = n;
}

void ForceField::dih_resize(const int n)
{
  int ndih_tmp=ndih;
  int    * dih1_tmp         = new int    [ndih+1];
  int    * dih2_tmp         = new int    [ndih+1];
  int    * dih3_tmp         = new int    [ndih+1];
  int    * dih4_tmp         = new int    [ndih+1];
  double * dih_coeff_tmp    = new double [ndih+1];
  double * dih_eq_tmp       = new double [ndih+1];
  double * dih_div_tmp      = new double [ndih+1];
  double * dih_phase_tmp    = new double [ndih+1];
  int    * dih_improper_tmp = new int    [ndih+1];
  for(int i=0;i<ndih;i++){
    dih1_tmp        [i]=dih1        [i];
    dih2_tmp        [i]=dih2        [i];
    dih3_tmp        [i]=dih3        [i];
    dih4_tmp        [i]=dih4        [i];
    dih_coeff_tmp   [i]=dih_coeff   [i];
    dih_eq_tmp      [i]=dih_eq      [i];
    dih_div_tmp     [i]=dih_div     [i];
    dih_phase_tmp   [i]=dih_phase   [i];
    dih_improper_tmp[i]=dih_improper[i];
  }
  dih_create(n);
  if(n>ndih_tmp){
    for(int i=0;i<ndih_tmp;i++){
      dih1        [i]=dih1_tmp        [i];
      dih2        [i]=dih2_tmp        [i];
      dih3        [i]=dih3_tmp        [i];
      dih4        [i]=dih4_tmp        [i];
      dih_coeff   [i]=dih_coeff_tmp   [i];
      dih_eq      [i]=dih_eq_tmp      [i];
      dih_div     [i]=dih_div_tmp     [i];
      dih_phase   [i]=dih_phase_tmp   [i];
      dih_improper[i]=dih_improper_tmp[i];
      ndih++;
    }
  } else {
    for(int i=0;i<n;i++){
      dih1        [i]=dih1_tmp        [i];
      dih2        [i]=dih2_tmp        [i];
      dih3        [i]=dih3_tmp        [i];
      dih4        [i]=dih4_tmp        [i];
      dih_coeff   [i]=dih_coeff_tmp   [i];
      dih_eq      [i]=dih_eq_tmp      [i];
      dih_div     [i]=dih_div_tmp     [i];
      dih_phase   [i]=dih_phase_tmp   [i];
      dih_improper[i]=dih_improper_tmp[i];
      ndih++;
    }
  }
  delete [] dih1_tmp;
  delete [] dih2_tmp;
  delete [] dih3_tmp;
  delete [] dih4_tmp;
  delete [] dih_coeff_tmp;
  delete [] dih_eq_tmp;
  delete [] dih_div_tmp;
  delete [] dih_phase_tmp;
  delete [] dih_improper_tmp;
}

void ForceField::p14_clear()
{
  if(np14_buff==0) return ;
  delete [] p14_1;
  delete [] p14_2;
  np14_buff=0;
  np14=0;
}

void ForceField::p14_create(const int n)
{
  p14_clear();
  if(n==0) return ;
  p14_1 = new int [n];
  p14_2 = new int [n];
  np14_buff = n;
}

void ForceField::p14_resize(const int n)
{
  int np14_tmp=np14;
  int * p14_1_tmp = new int [np14+1];
  int * p14_2_tmp = new int [np14+1];
  for(int i=0;i<np14;i++){
    p14_1_tmp[i]=p14_1[i];
    p14_2_tmp[i]=p14_2[i];
  }
  p14_create(n);
  if(n>np14_tmp){
    for(int i=0;i<np14_tmp;i++){
      p14_1 [i]=p14_1_tmp [i];
      p14_2 [i]=p14_2_tmp [i];
      np14++;
    }
  } else {
    for(int i=0;i<n;i++){
      p14_1 [i]=p14_1_tmp [i];
      p14_2 [i]=p14_2_tmp [i];
      np14++;
    }
  }
  delete [] p14_1_tmp;
  delete [] p14_2_tmp;
}

int    ForceField::get_natm      (              ) const { return natm          ; }
int    ForceField::get_atm1      ( const int ia ) const { return atm1      [ia]; }
int    ForceField::get_atm_paics ( const int ia ) const { return atm_paics [ia]; }
string ForceField::get_atm_amber ( const int ia ) const { return atm_amber [ia]; }
string ForceField::get_atm_pdb   ( const int ia ) const { return atm_pdb   [ia]; }
double ForceField::get_atm_chg   ( const int ia ) const { return atm_chg   [ia]; }
double ForceField::get_atm_vdw_r ( const int ia ) const { return atm_vdw_r [ia]; }
double ForceField::get_atm_vdw_e ( const int ia ) const { return atm_vdw_e [ia]; }

void ForceField::push_back_atm(const int    atm1_,
                               const int    atm_paics_,
                               const string atm_amber_,
                               const string atm_pdb_,
                               const double atm_chg_,
                               const double atm_vdw_r_,
                               const double atm_vdw_e_,
                               const int    atm_excluded_n_,
                               const int *  atm_excluded_,
                               const int    inc)
{
  //cout << "ForceField::push_back_atm in " << endl ;
  if(natm==natm_buff){
    atm_resize(natm+inc);
  }
  atm1     [natm]=atm1_;
  atm_paics[natm]=atm_paics_;
  atm_amber[natm]=atm_amber_;
  atm_pdb  [natm]=atm_pdb_;
  atm_chg  [natm]=atm_chg_;
  atm_vdw_r[natm]=atm_vdw_r_;
  atm_vdw_e[natm]=atm_vdw_e_;
  atm_excluded_create(natm,atm_excluded_n_);
  for(int i=0;i<atm_excluded_n_;i++){
    atm_excluded[natm][i]=atm_excluded_[i];
    atm_excluded_n[natm]++;
  }
  natm++;  
  //cout << "ForceField::push_back_atm out" << endl ;
}

void ForceField::push_back_bnd(const int    bnd1_,
                               const int    bnd2_,
                               const double bnd_coeff_,
                               const double bnd_eq_,
                               const int    inc)
{
  if(nbnd==nbnd_buff){
    bnd_resize(nbnd+inc);
  }
  bnd1     [nbnd]=bnd1_;  
  bnd2     [nbnd]=bnd2_;
  bnd_coeff[nbnd]=bnd_coeff_;
  bnd_eq   [nbnd]=bnd_eq_;
  nbnd++;
}

void ForceField::push_back_ang(const int    ang1_,
                               const int    ang2_,
                               const int    ang3_,
                               const double ang_coeff_,
                               const double ang_eq_,
                               const int    inc)
{
  if(nang==nang_buff){
    ang_resize(nang+inc);
  }
  ang1     [nang]=ang1_;  
  ang2     [nang]=ang2_;
  ang3     [nang]=ang3_;
  ang_coeff[nang]=ang_coeff_;
  ang_eq   [nang]=ang_eq_;
  nang++;
}

void ForceField::push_back_dih(const int    dih1_,
                               const int    dih2_,
                               const int    dih3_,
                               const int    dih4_,
                               const double dih_coeff_,
                               const double dih_div_,
                               const double dih_eq_,
                               const double dih_phase_,
                               const int    dih_improper_,
                               const int    inc)
{
  if(ndih==ndih_buff){
    dih_resize(ndih+inc);
  }
  dih1        [ndih]=dih1_;  
  dih2        [ndih]=dih2_;
  dih3        [ndih]=dih3_;
  dih4        [ndih]=dih4_;
  dih_coeff   [ndih]=dih_coeff_;
  dih_eq      [ndih]=dih_eq_;
  dih_div     [ndih]=dih_div_;
  dih_phase   [ndih]=dih_phase_;
  dih_improper[ndih]=dih_improper_;
  ndih++;
}

void ForceField::push_back_p14(const int    p14_1_,
                               const int    p14_2_,
                               const int    inc)
{
  if(np14==np14_buff){
    p14_resize(np14+inc);
  }
  if(p14_1_<p14_2_){p14_1[np14]=p14_1_; p14_2[np14]=p14_2_;}
  else             {p14_1[np14]=p14_2_; p14_2[np14]=p14_1_;}
  np14++;
}

/*
void ForceField::calc_external_interaction_energy ( const Molecule& m         ,
                                                    const int       n_ext     ,
                                                    const double *  ext_x_    ,
                                                    const double *  ext_y_    ,
                                                    const double *  ext_z_    ,
                                                    const double *  ext_vdw_r ,
                                                    const double *  ext_vdw_e ,
                                                    const double *  ext_chg   ,
                                                    double       *  eng_vdw   ,
                                                    double       *  eng_chg   ) const 
{
  double * x;
  double * y;
  double * z;
  int natm=m.get_natom();
  x = new double [natm+1];
  y = new double [natm+1];
  z = new double [natm+1];
  for(int i=0;i<natm;i++){
    x[i]=m.get_atom(i).get_R().get_x();
    y[i]=m.get_atom(i).get_R().get_y();
    z[i]=m.get_atom(i).get_R().get_z();
  }
  calc_external_interaction_energy (natm,
                                    x,
                                    y,
                                    z,
                                    n_ext,
                                    ext_x_,
                                    ext_y_,
                                    ext_z_,
                                    ext_vdw_r,
                                    ext_vdw_e,
                                    ext_chg,
                                    eng_vdw,
                                    eng_chg);
  delete [] x;
  delete [] y;
  delete [] z;
}

void ForceField::calc_external_interaction_energy ( const int       n,
                                                    const double *  x_,
                                                    const double *  y_,
                                                    const double *  z_,
                                                    const int       n_ext     ,
                                                    const double *  ext_x_    ,
                                                    const double *  ext_y_    ,
                                                    const double *  ext_z_    ,
                                                    const double *  ext_vdw_r ,
                                                    const double *  ext_vdw_e ,
                                                    const double *  ext_chg   ,
                                                    double       *  eng_vdw   ,
                                                    double       *  eng_chg   ) const 
{
  //cout << "ForceField::calc_enternal_interaction_energy in " << endl ;
  double * x;
  double * y;
  double * z;
  double * ext_x;
  double * ext_y;
  double * ext_z;
  double sum_v1,sum_v2,v1,v2,x1,y1,z1,x2,y2,z2,d,coeffA,coeffB;

  x     = new double [n+1];
  y     = new double [n+1];
  z     = new double [n+1];
  ext_x = new double [n_ext+1];
  ext_y = new double [n_ext+1];
  ext_z = new double [n_ext+1];

  for(int i=0;i<natm;i++){
    x[i]=x_[i]*0.5291772108;
    y[i]=y_[i]*0.5291772108;
    z[i]=z_[i]*0.5291772108;
  }
  for(int i=0;i<n_ext;i++){
    ext_x[i]=ext_x_[i]*0.5291772108;
    ext_y[i]=ext_y_[i]*0.5291772108;
    ext_z[i]=ext_z_[i]*0.5291772108;
  }

  sum_v1=0.0;
  sum_v2=0.0;
  
  for(int i=0;i<natm;i++){

    x1=x[i];
    y1=y[i];
    z1=z[i];

    for(int j=0;j<n_ext;j++){

      x2=ext_x[j];
      y2=ext_y[j];
      z2=ext_z[j];

      //////////////////
      // distance (d) //
      //////////////////

      d=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));

      ////////////////////////////////////////////////
      // non-bonding energy (exclude 12,13,14 pair) //
      ////////////////////////////////////////////////

      coeffA =     sqrt(atm_vdw_e[i]*ext_vdw_e[j])*pow(atm_vdw_r[i]+ext_vdw_r[j],12);
      coeffB = 2.0*sqrt(atm_vdw_e[i]*ext_vdw_e[j])*pow(atm_vdw_r[i]+ext_vdw_r[j], 6);
      
      v1=coeffA/pow(d,12)-coeffB/pow(d,6);
      v2=(atm_chg[i]*18.2223)*(ext_chg[j]*18.2223)/d;
      sum_v1=sum_v1+v1;
      sum_v2=sum_v2+v2;
    }
  }
  (*eng_vdw)=sum_v1/627.51;
  (*eng_chg)=sum_v2/627.51;
  
  delete [] x;
  delete [] y;
  delete [] z;
  delete [] ext_x;
  delete [] ext_y;
  delete [] ext_z;
  //cout << "ForceField::calc_enternal_interaction_energy out " << endl ;
}
*/
/*
void ForceField::copy_def_atm( const ForceField& f)
{
  def_atm_create(f.def_natm);
  for(int i=0;i<f.def_natm;i++){
    def_atm_amber[i]=f.def_atm_amber[i];
    //def_atm_chg  [i]=f.def_atm_chg  [i];
    def_atm_vdw_r[i]=f.def_atm_vdw_r[i];
    def_atm_vdw_e[i]=f.def_atm_vdw_e[i];
    def_natm++;
  }
}

void ForceField::copy_def_bnd( const ForceField& f )
{
  def_bnd_create(f.def_nbnd);
  for(int i=0;i<f.def_nbnd;i++){
    def_bnd1     [i]=f.def_bnd1     [i];
    def_bnd2     [i]=f.def_bnd2     [i];
    def_bnd_coeff[i]=f.def_bnd_coeff[i];
    def_bnd_eq   [i]=f.def_bnd_eq   [i];
    def_nbnd++;
  }
}

void ForceField::copy_def_ang( const ForceField& f )
{
  def_ang_create(f.def_nang);
  for(int i=0;i<f.def_nang;i++){
    def_ang1     [i]=f.def_ang1     [i];
    def_ang2     [i]=f.def_ang2     [i];
    def_ang3     [i]=f.def_ang3     [i];
    def_ang_coeff[i]=f.def_ang_coeff[i];
    def_ang_eq   [i]=f.def_ang_eq   [i];
    def_nang++;
  }
}

void ForceField::copy_def_dih( const ForceField& f )
{
  def_dih_create(f.def_ndih);
  for(int i=0;i<f.def_ndih;i++){
    def_dih1        [i]=f.def_dih1        [i];
    def_dih2        [i]=f.def_dih2        [i];
    def_dih3        [i]=f.def_dih3        [i];
    def_dih4        [i]=f.def_dih4        [i];
    def_dih_coeff   [i]=f.def_dih_coeff   [i];
    def_dih_eq      [i]=f.def_dih_eq      [i];
    def_dih_div     [i]=f.def_dih_div     [i];
    def_dih_phase   [i]=f.def_dih_phase   [i];
    def_dih_improper[i]=f.def_dih_improper[i];
    def_ndih++;
  }
}

void ForceField::def_atm_clear()
{
  if(def_natm_buff==0) return ;
  delete [] def_atm_amber;
  delete [] def_atm_vdw_r;
  delete [] def_atm_vdw_e;
  def_natm_buff=0;
  def_natm=0;
}

void ForceField::def_atm_create(const int n)
{
  def_atm_clear();
  if(n==0) return ;
  def_atm_amber = new string [n];
  def_atm_vdw_r = new double [n];
  def_atm_vdw_e = new double [n];
  def_natm_buff=n;
}

void ForceField::def_atm_resize(const int n)
{
  int def_natm_tmp = def_natm;
  string *  def_atm_amber_tmp      = new string [def_natm+1];
  double *  def_atm_vdw_r_tmp      = new double [def_natm+1];
  double *  def_atm_vdw_e_tmp      = new double [def_natm+1];
  for(int i=0;i<def_natm;i++){
    def_atm_amber_tmp[i] = def_atm_amber[i];
    def_atm_vdw_r_tmp[i] = def_atm_vdw_r[i];
    def_atm_vdw_e_tmp[i] = def_atm_vdw_e[i];
  }
  def_atm_create(n);
  if(n>def_natm_tmp){
    for(int i=0;i<def_natm_tmp;i++){
      def_atm_amber[i] = def_atm_amber_tmp[i];
      def_atm_vdw_r[i] = def_atm_vdw_r_tmp[i];
      def_atm_vdw_e[i] = def_atm_vdw_e_tmp[i];
      def_natm++;
    }
  } else {
    for(int i=0;i<n;i++){
      def_atm_amber[i] = def_atm_amber_tmp[i];
      def_atm_vdw_r[i] = def_atm_vdw_r_tmp[i];
      def_atm_vdw_e[i] = def_atm_vdw_e_tmp[i];
      def_natm++;
    }
  }
  delete [] def_atm_amber_tmp;
  delete [] def_atm_vdw_r_tmp;
  delete [] def_atm_vdw_e_tmp;
}

void ForceField::def_bnd_clear()
{
  if(def_nbnd_buff==0) return ;
  delete [] def_bnd1;
  delete [] def_bnd2;
  delete [] def_bnd_coeff;
  delete [] def_bnd_eq;
  def_nbnd_buff=0;
  def_nbnd=0;
}

void ForceField::def_bnd_create(const int n)
{
  def_bnd_clear();
  if(n==0) return ;
  def_bnd1      = new string [n];
  def_bnd2      = new string [n];
  def_bnd_coeff = new double [n];
  def_bnd_eq    = new double [n];
  def_nbnd_buff = n;
}

void ForceField::def_bnd_resize(const int n)
{
  int def_nbnd_tmp=def_nbnd;
  string * def_bnd1_tmp      = new string [def_nbnd+1];
  string * def_bnd2_tmp      = new string [def_nbnd+1];
  double * def_bnd_coeff_tmp = new double [def_nbnd+1];
  double * def_bnd_eq_tmp    = new double [def_nbnd+1];
  for(int i=0;i<def_nbnd;i++){
    def_bnd1_tmp     [i]=def_bnd1     [i];
    def_bnd2_tmp     [i]=def_bnd2     [i];
    def_bnd_coeff_tmp[i]=def_bnd_coeff[i];
    def_bnd_eq_tmp   [i]=def_bnd_eq   [i];
  }
  def_bnd_create(n);
  if(n>def_nbnd_tmp){
    for(int i=0;i<def_nbnd_tmp;i++){
      def_bnd1     [i]=def_bnd1_tmp     [i];
      def_bnd2     [i]=def_bnd2_tmp     [i];
      def_bnd_coeff[i]=def_bnd_coeff_tmp[i];
      def_bnd_eq   [i]=def_bnd_eq_tmp   [i];
      def_nbnd++;
    }
  } else {
    for(int i=0;i<n;i++){
      def_bnd1     [i]=def_bnd1_tmp     [i];
      def_bnd2     [i]=def_bnd2_tmp     [i];
      def_bnd_coeff[i]=def_bnd_coeff_tmp[i];
      def_bnd_eq   [i]=def_bnd_eq_tmp   [i];
      def_nbnd++;
    }
  }
  delete [] def_bnd1_tmp;
  delete [] def_bnd2_tmp;
  delete [] def_bnd_coeff_tmp;
  delete [] def_bnd_eq_tmp;
}

void ForceField::def_ang_clear()
{
  if(def_nang_buff==0) return ;
  delete [] def_ang1;
  delete [] def_ang2;
  delete [] def_ang3;
  delete [] def_ang_coeff;
  delete [] def_ang_eq;
  def_nang_buff=0;
  def_nang=0;
}

void ForceField::def_ang_create(const int n)
{
  def_ang_clear();
  if(n==0) return ;
  def_ang1      = new string [n];
  def_ang2      = new string [n];
  def_ang3      = new string [n];
  def_ang_coeff = new double [n];
  def_ang_eq    = new double [n];
  def_nang_buff = n;
}

void ForceField::def_ang_resize(const int n)
{
  int def_nang_tmp=def_nang;
  string * def_ang1_tmp      = new string [def_nang+1];
  string * def_ang2_tmp      = new string [def_nang+1];
  string * def_ang3_tmp      = new string [def_nang+1];
  double * def_ang_coeff_tmp = new double [def_nang+1];
  double * def_ang_eq_tmp    = new double [def_nang+1];
  for(int i=0;i<def_nang;i++){
    def_ang1_tmp     [i]=def_ang1     [i];
    def_ang2_tmp     [i]=def_ang2     [i];
    def_ang3_tmp     [i]=def_ang3     [i];
    def_ang_coeff_tmp[i]=def_ang_coeff[i];
    def_ang_eq_tmp   [i]=def_ang_eq   [i];
  }
  def_ang_create(n);
  if(n>def_nang_tmp){
    for(int i=0;i<def_nang_tmp;i++){
      def_ang1     [i]=def_ang1_tmp     [i];
      def_ang2     [i]=def_ang2_tmp     [i];
      def_ang3     [i]=def_ang3_tmp     [i];
      def_ang_coeff[i]=def_ang_coeff_tmp[i];
      def_ang_eq   [i]=def_ang_eq_tmp   [i];
      def_nang++;
    }
  } else {
    for(int i=0;i<n;i++){
      def_ang1     [i]=def_ang1_tmp     [i];
      def_ang2     [i]=def_ang2_tmp     [i];
      def_ang3     [i]=def_ang3_tmp     [i];
      def_ang_coeff[i]=def_ang_coeff_tmp[i];
      def_ang_eq   [i]=def_ang_eq_tmp   [i];
      def_nang++;
    }
  }
  delete [] def_ang1_tmp;
  delete [] def_ang2_tmp;
  delete [] def_ang3_tmp;
  delete [] def_ang_coeff_tmp;
  delete [] def_ang_eq_tmp;
}

void ForceField::def_dih_clear()
{
  if(def_ndih_buff==0) return ;
  delete [] def_dih1;
  delete [] def_dih2;
  delete [] def_dih3;
  delete [] def_dih4;
  delete [] def_dih_coeff;
  delete [] def_dih_eq;
  delete [] def_dih_div;
  delete [] def_dih_phase;
  delete [] def_dih_improper;
  def_ndih_buff=0;
  def_ndih=0;
}

void ForceField::def_dih_create(const int n)
{
  def_dih_clear();
  if(n==0) return ;
  def_dih1         = new string [n];
  def_dih2         = new string [n];
  def_dih3         = new string [n];
  def_dih4         = new string [n];
  def_dih_coeff    = new double [n];
  def_dih_eq       = new double [n];
  def_dih_div      = new double [n];
  def_dih_phase    = new double [n];
  def_dih_improper = new int    [n];
  def_ndih_buff = n;
}

void ForceField::def_dih_resize(const int n)
{
  int def_ndih_tmp=def_ndih;
  string * def_dih1_tmp         = new string [def_ndih+1];
  string * def_dih2_tmp         = new string [def_ndih+1];
  string * def_dih3_tmp         = new string [def_ndih+1];
  string * def_dih4_tmp         = new string [def_ndih+1];
  double * def_dih_coeff_tmp    = new double [def_ndih+1];
  double * def_dih_eq_tmp       = new double [def_ndih+1];
  double * def_dih_div_tmp      = new double [def_ndih+1];
  double * def_dih_phase_tmp    = new double [def_ndih+1];
  int    * def_dih_improper_tmp = new int    [def_ndih+1];
  for(int i=0;i<def_ndih;i++){
    def_dih1_tmp        [i]=def_dih1        [i];
    def_dih2_tmp        [i]=def_dih2        [i];
    def_dih3_tmp        [i]=def_dih3        [i];
    def_dih4_tmp        [i]=def_dih4        [i];
    def_dih_coeff_tmp   [i]=def_dih_coeff   [i];
    def_dih_eq_tmp      [i]=def_dih_eq      [i];
    def_dih_div_tmp     [i]=def_dih_div     [i];
    def_dih_phase_tmp   [i]=def_dih_phase   [i];
    def_dih_improper_tmp[i]=def_dih_improper[i];
  }
  def_dih_create(n);
  if(n>def_ndih_tmp){
    for(int i=0;i<def_ndih_tmp;i++){
      def_dih1        [i]=def_dih1_tmp        [i];
      def_dih2        [i]=def_dih2_tmp        [i];
      def_dih3        [i]=def_dih3_tmp        [i];
      def_dih4        [i]=def_dih4_tmp        [i];
      def_dih_coeff   [i]=def_dih_coeff_tmp   [i];
      def_dih_eq      [i]=def_dih_eq_tmp      [i];
      def_dih_div     [i]=def_dih_div_tmp     [i];
      def_dih_phase   [i]=def_dih_phase_tmp   [i];
      def_dih_improper[i]=def_dih_improper_tmp[i];
      def_ndih++;
    }
  } else {
    for(int i=0;i<n;i++){
      def_dih1        [i]=def_dih1_tmp        [i];
      def_dih2        [i]=def_dih2_tmp        [i];
      def_dih3        [i]=def_dih3_tmp        [i];
      def_dih4        [i]=def_dih4_tmp        [i];
      def_dih_coeff   [i]=def_dih_coeff_tmp   [i];
      def_dih_eq      [i]=def_dih_eq_tmp      [i];
      def_dih_div     [i]=def_dih_div_tmp     [i];
      def_dih_phase   [i]=def_dih_phase_tmp   [i];
      def_dih_improper[i]=def_dih_improper_tmp[i];
      def_ndih++;
    }
  }
  delete [] def_dih1_tmp;
  delete [] def_dih2_tmp;
  delete [] def_dih3_tmp;
  delete [] def_dih4_tmp;
  delete [] def_dih_coeff_tmp;
  delete [] def_dih_eq_tmp;
  delete [] def_dih_div_tmp;
  delete [] def_dih_phase_tmp;
  delete [] def_dih_improper_tmp;
}

void ForceField::push_back_def_atm(const string def_atm_amber_,
                                   const double def_atm_vdw_r_,
                                   const double def_atm_vdw_e_,
                                   const int    inc)
{
  if(def_natm==def_natm_buff){
    def_atm_resize(def_natm+inc);
  }
  def_atm_amber[def_natm]=def_atm_amber_;
  def_atm_vdw_r[def_natm]=def_atm_vdw_r_;
  def_atm_vdw_e[def_natm]=def_atm_vdw_e_;
  def_natm++;  
}

void ForceField::push_back_def_bnd(const string def_bnd1_,
                                   const string def_bnd2_,
                                   const double def_bnd_coeff_,
                                   const double def_bnd_eq_,
                                   const int    inc)
{
  if(def_nbnd==def_nbnd_buff){
    def_bnd_resize(def_nbnd+inc);
  }
  def_bnd1     [def_nbnd]=def_bnd1_;  
  def_bnd2     [def_nbnd]=def_bnd2_;
  def_bnd_coeff[def_nbnd]=def_bnd_coeff_;
  def_bnd_eq   [def_nbnd]=def_bnd_eq_;
  def_nbnd++;
}

void ForceField::push_back_def_ang(const string def_ang1_,
                                   const string def_ang2_,
                                   const string def_ang3_,
                                   const double def_ang_coeff_,
                                   const double def_ang_eq_,
                                   const int    inc)
{
  if(def_nang==def_nang_buff){
    def_ang_resize(def_nang+inc);
  }
  def_ang1     [def_nang]=def_ang1_;  
  def_ang2     [def_nang]=def_ang2_;
  def_ang3     [def_nang]=def_ang3_;
  def_ang_coeff[def_nang]=def_ang_coeff_;
  def_ang_eq   [def_nang]=def_ang_eq_;
  def_nang++;
}

void ForceField::push_back_def_dih(const string def_dih1_,
                                   const string def_dih2_,
                                   const string def_dih3_,
                                   const string def_dih4_,
                                   const double def_dih_div_,
                                   const double def_dih_coeff_,
                                   const double def_dih_eq_,
                                   const double def_dih_phase_,
                                   const int    def_dih_improper_,
                                   const int    inc)
{
  if(def_ndih==def_ndih_buff){
    def_dih_resize(def_ndih+inc);
  }
  def_dih1        [def_ndih]=def_dih1_;  
  def_dih2        [def_ndih]=def_dih2_;
  def_dih3        [def_ndih]=def_dih3_;
  def_dih4        [def_ndih]=def_dih4_;
  def_dih_coeff   [def_ndih]=def_dih_coeff_;
  def_dih_eq      [def_ndih]=def_dih_eq_;
  def_dih_div     [def_ndih]=def_dih_div_;
  def_dih_phase   [def_ndih]=def_dih_phase_;
  def_dih_improper[def_ndih]=def_dih_improper_;
  def_ndih++;
}

void ForceField::read_force_field_param_define( const string& filename )
{
  char line[1024];
  char buff[1024];
  fstream file(filename.c_str(), std::ios::in);
  string atype1;
  string atype2;
  string atype3;
  string atype4;
  double eq,coeff,div,phase,vdw_r,vdw_e;
  int n,nterm,improper;
  if(file==NULL){
    cout << "\n warning , can not open the file : " << filename << "\n\n";
    return;
  }
  while(! file.eof()){
    file.getline(line, sizeof(line));
    if(iFind(line, "ATOM_PARAM")>=0){
      iPickUp(line,1,buff);n=atoi(buff);
      for(int i=0;i<n;i++){
        file.getline(line, sizeof(line));
        iPickUp(line,0,buff); atype1=buff;
        iPickUp(line,1,buff); vdw_r=atof(buff);
        iPickUp(line,2,buff); vdw_e=atof(buff);
        push_back_def_atm(atype1,vdw_r,vdw_e);
      }
    }
    if(iFind(line, "BOND_PARAM")>=0){
      iPickUp(line,1,buff);n=atoi(buff);
      for(int i=0;i<n;i++){
        file.getline(line, sizeof(line));
        iPickUp(line,0,buff); atype1=buff;
        iPickUp(line,1,buff); atype2=buff;
        iPickUp(line,2,buff); coeff =atof(buff);
        iPickUp(line,3,buff); eq    =atof(buff);
        push_back_def_bnd(atype1,atype2,coeff,eq);
      }
    }
    if(iFind(line, "ANGLE_PARAM")>=0){
      iPickUp(line,1,buff);n=atoi(buff);
      for(int i=0;i<n;i++){
        file.getline(line, sizeof(line));
        iPickUp(line,0,buff); atype1=buff;
        iPickUp(line,1,buff); atype2=buff;
        iPickUp(line,2,buff); atype3=buff;
        iPickUp(line,3,buff); coeff =atof(buff);
        iPickUp(line,4,buff); eq    =atof(buff);
        push_back_def_ang(atype1,atype2,atype3,coeff,eq);
      }
    }
    if(iFind(line, "DIHEDRAL_PARAM")>=0){

      iPickUp(line,1,buff);n=atoi(buff);
      for(int i=0;i<n;i++){
        file.getline(line, sizeof(line));
        iPickUp(line,0,buff); atype1  =buff;
        iPickUp(line,1,buff); atype2  =buff;
        iPickUp(line,2,buff); atype3  =buff;
        iPickUp(line,3,buff); atype4  =buff;
        iPickUp(line,4,buff); improper=atoi(buff);
        iPickUp(line,5,buff); nterm   =atoi(buff);
        iPickUp(line,6,buff); div     =atof(buff);
        iPickUp(line,7,buff); coeff   =atof(buff); cout << buff << " " << coeff << endl ;
        iPickUp(line,8,buff); eq      =atof(buff);
        iPickUp(line,9,buff); phase   =atof(buff);
        if(nterm>1){
          cout << "error , ForceField::read_force_field_param_define , nterm > 1 , corrent version is not available" << endl ;
          exit(1);
        }
        push_back_def_dih(atype1,atype2,atype3,atype4,div,coeff,eq,phase,improper);
      }
    }
  }
}
*/
// print out for develop
/*
void ForceField::print_amber_force_field_atom (const Molecule& m)
{
  int a1,ptyp1,ibu1,tc1,tn1,t1,res_type,chk,his1814,his1816,his;
  double charge,vdw_r,vdw_e,charge_sum;
  string amber1,pdb1;
  Dvector3D r1;
  double * vdw_r_;
  double * vdw_e_;
  
  vdw_r_ = new double [m.get_natom()+1];
  vdw_e_ = new double [m.get_natom()+1];

  charge_sum=0.0; 
  cout << "< ATOM >" << m.get_natom() << endl ;
  for(int i=0;i<m.get_natom();i++){
  
    // atom number
    a1=i;

    // atom coordinate
    r1.copy(m.get_atom(a1).get_R());

    // atom type of paics
    ptyp1=m.get_atom(a1).get_bio1();

    // biounit of the atom
    ibu1=m.get_atom(a1).get_bio3();

    // cterm
    tc1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_c()>0){tc1=1;}}

    // nterm
    tn1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_n()>0){tn1=1;}}

    // terminal
    t1=0; if(tn1>0){t1=1;} if(tc1>0){t1=2;}

    // residue type
    if(ibu1>=0){res_type=m.get_biounit(ibu1).get_type();} 

    // pdb tag
    pdb1=BioUnit::pdb_tag(ptyp1);

    // atom type of amber
    if(ibu1>=0 && res_type==1800){
      his1814=m.get_biounit(ibu1).get_h1814();
      his1816=m.get_biounit(ibu1).get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
	cout << "error in his check!! @ print atom type4 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
	exit(1);
      }
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,his);
    } else if(ibu1>=0 && res_type==1550){
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,1);
    } else {
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1);
    }

    if(amber1 != "**"){
      chk=ForceField::amber_force_field_charge (amber1,pdb1,res_type,t1,&charge,his);
      charge_sum=charge_sum+charge;
      if(chk<0){
	cout << "error!! in force_field_charge" ;
	cout.width(6); cout << amber1 ;
	cout.width(6); cout << res_type ;
	cout.width(6); cout << t1 ;
	cout << endl ;
      }
      //chk=ForceField::amber_force_field_vdw(amber1,&vdw_r,&vdw_e);
      chk=amber_force_field_vdw(amber1,&vdw_r,&vdw_e);
      if(chk<0){
	cout << "error!! in force_field_abcoeff" << endl ;
	cout.width(6); cout << amber1 ;
	cout.width(6); cout << res_type ;
	cout.width(6); cout << t1 ;
	cout << endl ;
      }
    }
    vdw_r_[i]=vdw_r;
    vdw_e_[i]=vdw_e;

    // print out
    cout.setf(ios::fixed,ios::floatfield);
    cout.precision(10);
    cout.width( 6); cout << i+1 << ":" ;
    cout.width( 6); cout << a1+1 ;
    cout.width( 6); cout << amber1 ;
    cout.width(18); cout << charge ;
    cout.width(18); cout << vdw_r ;
    cout.width(18); cout << vdw_e ;
    cout.width(18); cout << r1.get_x() * 0.529177211;
    cout.width(18); cout << r1.get_y() * 0.529177211;
    cout.width(18); cout << r1.get_z() * 0.529177211;
    cout << endl ;
  }

  //--- charge sum

  cout << "charge_sum             = " << charge_sum << endl ;

  //--- distinct atom

  int dist_atom_n;
  int * dist_atom;
  double d1,d2;
  dist_atom=new int [m.get_natom()];

  dist_atom_n=0;
  dist_atom[dist_atom_n]=0;
  dist_atom_n++;

  for(int i=1;i<m.get_natom();i++){
    chk=0;
    for(int j=0;j<dist_atom_n;j++){
      d1=fabs(vdw_r_[dist_atom[j]]-vdw_r_[i]);
      d2=fabs(vdw_e_[dist_atom[j]]-vdw_e_[i]);
      if(d1<0.000000001 && d2<0.000000001){
	chk=1;
      }
    }
    if(chk==0){
      dist_atom[dist_atom_n]=i;
      dist_atom_n++;
    }
  }
  cout << "distinct atoms(NTYPES) = " << dist_atom_n << endl ;
  delete [] vdw_r_;
  delete [] vdw_e_;
  delete [] dist_atom;

  cout << "ishi in print_atom out" << endl ;
}
*/
// print out for develop
/*
void ForceField::print_amber_force_field_bond ( const Molecule& m )
{
  int chk,a1,a2,a3,ptyp1,ptyp2,ptyp3,ibu1,ibu2,ibu3,tc1,tc2,tc3,tn1,tn2,tn3,t1,t2,t3,n_h,n_not_h,atomic1,atomic2;
  int res_type1,res_type2,his1814,his1816,his;
  string amber1,amber2,amber3;
  double coeff,deq,v,d,sum_v;
  Dvector3D r1,r2;

  cout << "< BOND >" << m.get_nbond() << endl ;
  sum_v=0.0;
  n_h=0;
  n_not_h=0;
  for(int i=0;i<m.get_nbond();i++){

    // atom number
    a1=m.get_bond(i).get_iatom1();
    a2=m.get_bond(i).get_iatom2();

    // atomic number
    atomic1=m.get_atom(a1).get_atomic_number();
    atomic2=m.get_atom(a2).get_atomic_number();
    if(atomic1 != 1 && atomic2 != 1){
      n_not_h++;
    } else {
      n_h++;
    }

    // atom coordinate
    r1.copy(m.get_atom(a1).get_R());
    r2.copy(m.get_atom(a2).get_R());

    // atom type of paics
    ptyp1=m.get_atom(a1).get_bio1();
    ptyp2=m.get_atom(a2).get_bio1();

    // biounit of the atom
    ibu1=m.get_atom(a1).get_bio3();
    ibu2=m.get_atom(a2).get_bio3();

    // residue type
    if(ibu1>=0){res_type1=m.get_biounit(ibu1).get_type();}
    if(ibu2>=0){res_type2=m.get_biounit(ibu2).get_type();}

    // cterm
    tc1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_c()>0){tc1=1;}}
    tc2=0; if(ibu2>=0){if(m.get_biounit(ibu2).get_terminal_c()>0){tc2=1;}}

    // nterm
    tn1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_n()>0){tn1=1;}}
    tn2=0; if(ibu2>=0){if(m.get_biounit(ibu2).get_terminal_n()>0){tn2=1;}}

    // terminal
    t1=0; if(ibu1>=0){if(tn1>0){t1=1;} if(tc1>0){t1=2;}}
    t2=0; if(ibu2>=0){if(tn2>0){t2=1;} if(tc2>0){t2=2;}}

    // atom type of amber
    if(ibu1>=0 && res_type1==1800){
      his1814=m.get_biounit(ibu1).get_h1814();
      his1816=m.get_biounit(ibu1).get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
        cout << "error in his check!! @ print bond type1 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
        exit(1);
      }
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,his);
    } else if(ibu1>=0 && res_type1==1550){
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,1);
    } else {
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1);
    }
    if(ibu2>=0 && res_type2==1800){
      his1814=m.get_biounit(ibu2).get_h1814();
      his1816=m.get_biounit(ibu2).get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
        cout << "error in his check!! @ print bond type2 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
        exit(1);
      }
      amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2,his);
    } else if(ibu2 >=0 && res_type2==1550){
      amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2,1);
    } else {
      amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2);
    }

    // amber force field
    if(amber1 != "**" && 
       amber2 != "**" ){
      chk=ForceField::amber_force_field_bond(amber1,amber2,&coeff,&deq);
      if(chk<0){
	cout << "error!! in force_field_bond" << endl ;
	cout.width(6); cout << amber1 ;
	cout.width(6); cout << amber2 ;
	cout << endl ;
      }
    }
    
    // distance
    d=Dvector3D::distance1(r1,r2)*0.529177211;
    
    // potential energy
    v=coeff*(d-deq)*(d-deq);
    sum_v=sum_v+v;
    
    // print out
    cout.setf(ios::fixed,ios::floatfield);
    cout.precision(10);
    cout.width( 6); cout << i+1 << ":" ;
    cout.width( 6); cout << a1+1 ;
    cout.width( 6); cout << a2+1 ;
    cout.width( 6); cout << amber1 ;
    cout.width( 6); cout << amber2 ;
    cout.width(18); cout << d ;
    cout.width(18); cout << deq ;
    cout.width(18); cout << d-deq ;
    cout.width(18); cout << coeff ;
    cout.width(18); cout << v ;
    cout << endl ;
  }
  cout << "bond     containing hydrogen , NBONH = " << n_h     << endl ;
  cout << "bond not containing hydrogen , NBONA = " << n_not_h << endl ;
  cout.width(12); cout << sum_v << endl ;
}
*/
// print out for develop 
/*
void ForceField::print_amber_force_field_angle(const Molecule& m)
{
  int chk,a1,a2,a3,ptyp1,ptyp2,ptyp3,ibu1,ibu2,ibu3,tc1,tc2,tc3,tn1,tn2,tn3,t1,t2,t3,atomic1,atomic2,atomic3,n_h,n_not_h;
  int res_type1,res_type2,res_type3,his1814,his1816,his;
  string amber1,amber2,amber3;
  double coeff,aeq,ang,v,sum_v;
  Dvector3D r1,r2,r3;

  cout << "< ANGLE >" << m.get_nangle() << endl ;
  sum_v=0.0;
  n_h=0;
  n_not_h=0;
  for(int i=0;i<m.get_nangle();i++){

    // atom number
    a1=m.get_angle_iatom1(i);
    a2=m.get_angle_iatom2(i);
    a3=m.get_angle_iatom3(i);

    // atomic number
    atomic1=m.get_atom(a1).get_atomic_number();
    atomic2=m.get_atom(a2).get_atomic_number();
    atomic3=m.get_atom(a3).get_atomic_number();
    if(atomic1 != 1 && atomic2 != 1 && atomic3 != 1){
      n_not_h++;
    } else {
      n_h++;
    }

    // atom coordinate
    r1.copy(m.get_atom(a1).get_R());
    r2.copy(m.get_atom(a2).get_R());
    r3.copy(m.get_atom(a3).get_R());

    // atom type of paics
    ptyp1=m.get_atom(a1).get_bio1();
    ptyp2=m.get_atom(a2).get_bio1();
    ptyp3=m.get_atom(a3).get_bio1();

    // biounit of the atom
    ibu1=m.get_atom(a1).get_bio3();
    ibu2=m.get_atom(a2).get_bio3();
    ibu3=m.get_atom(a3).get_bio3();

    // residue type
    if(ibu1>=0){res_type1=m.get_biounit(ibu1).get_type();}
    if(ibu2>=0){res_type2=m.get_biounit(ibu2).get_type();}
    if(ibu3>=0){res_type3=m.get_biounit(ibu3).get_type();}

    // cterm
    tc1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_c()>0){tc1=1;}}
    tc2=0; if(ibu2>=0){if(m.get_biounit(ibu2).get_terminal_c()>0){tc2=1;}}
    tc3=0; if(ibu3>=0){if(m.get_biounit(ibu3).get_terminal_c()>0){tc3=1;}}

    // nterm
    tn1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_n()>0){tn1=1;}}
    tn2=0; if(ibu2>=0){if(m.get_biounit(ibu2).get_terminal_n()>0){tn2=1;}}
    tn3=0; if(ibu3>=0){if(m.get_biounit(ibu3).get_terminal_n()>0){tn3=1;}}

    // terminal
    t1=0; if(ibu1>=0){if(tn1>0){t1=1;} if(tc1>0){t1=2;}}
    t2=0; if(ibu2>=0){if(tn2>0){t2=1;} if(tc2>0){t2=2;}}
    t3=0; if(ibu3>=0){if(tn3>0){t3=1;} if(tc3>0){t3=2;}}

    // atom type of amber
    if(ibu1>=0 && res_type1==1800){
      his1814=m.get_biounit(ibu1).get_h1814();
      his1816=m.get_biounit(ibu1).get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
        cout << "error in his check!! @ print angle type1 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
        exit(1);
      }
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,his);
    } else if(ibu1>=0 && res_type1==1550){
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,1);
    } else {
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1);
    }
    if(ibu2>=0 && res_type2==1800){
      his1814=m.get_biounit(ibu2).get_h1814();
      his1816=m.get_biounit(ibu2).get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
        cout << "error in his check!! @ print angle type2 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
        exit(1);
      }
      amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2,his);
    } else if(ibu2>=0 && res_type2==1550){
      amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2,1);
    } else {
      amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2);
    }
    if(ibu3>=0 && res_type3==1800){
      his1814=m.get_biounit(ibu3).get_h1814();
      his1816=m.get_biounit(ibu3).get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
        cout << "error in his check!! @ print angle type3 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
        exit(1);
      }
      amber3=ForceField::amber_force_field_get_atom_type(ptyp3,t3,his);
    } else if(ibu3 >= 0 && res_type3==1550){
      amber3=ForceField::amber_force_field_get_atom_type(ptyp3,t3,1);
    } else {
      amber3=ForceField::amber_force_field_get_atom_type(ptyp3,t3);
    }

    // amber force field
    if(amber1 != "**" &&
       amber2 != "**" &&
       amber3 != "**" ){
      chk=ForceField::amber_force_field_angle(amber1,amber2,amber3,&coeff,&aeq);
      if(chk<0){
	cout << "error!! in force_field_angle" << endl ;
	cout.width( 6); cout << amber1 ;
	cout.width( 6); cout << amber2 ;
	cout.width( 6); cout << amber3 ;
	cout << endl ;
      }
    }

    // angle
    ang=Angle::calc(r1,r2,r3);

    // potential energy
    v=coeff*(ang-aeq*3.141594/180.0)*(ang-aeq*3.141594/180.0);
    sum_v=sum_v+v;

    // print out
    cout.setf(ios::fixed,ios::floatfield);
    cout.precision(10);
    cout.width( 6); cout << i+1 << ":" ;
    cout.width( 6); cout << a1+1 ;
    cout.width( 6); cout << a2+1 ;
    cout.width( 6); cout << a3+1 ;
    cout.width( 6); cout << amber1 ;
    cout.width( 6); cout << amber2 ;
    cout.width( 6); cout << amber3 ;
    cout.width(18); cout << ang ;
    cout.width(18); cout << aeq ;
    cout.width(18); cout << ang-aeq ;
    cout.width(18); cout << coeff ;
    cout.width(18); cout << v ;
    cout << endl ;
  }
  cout << "angle     containing hydrogen (NTHETH) = " << n_h     << endl ;
  cout << "angle not containing hydrogen (NTHETA) = " << n_not_h << endl ;   
  cout.width(12); cout << sum_v << endl ;
}
*/
// print out for develop
/*
void ForceField::print_amber_force_field_dihedral( const Molecule& m )
{
  int n,chk,a1,a2,a3,a4,ptyp1,ptyp2,ptyp3,ptyp4,ibu1,ibu2,ibu3,ibu4,tc1,tc2,tc3,tc4,tn1,tn2,tn3,tn4,t1,t2,t3,t4,improper;
  int atomic1,atomic2,atomic3,atomic4,n_h,n_not_h,n_amber,amber_count,n_improper,n_improper_h,n_improper_not_h,his_h;
  int res_type1,res_type2,res_type3,res_type4,his1814,his1816,his;
  string amber1,amber2,amber3,amber4;
  double dih,v,v_sum;
  Dvector3D r1,r2,r3,r4;
  double  coeff[16];
  double  div  [16];
  double  eq   [16];
  double  phase[16];

  cout << "< DIHEDRAL > " << m.get_ndihedral() << endl ;

  v_sum=0.0;
  n_h=0;
  n_not_h=0;
  n_amber=0;
  n_improper=0;
  n_improper_h=0;
  n_improper_not_h=0;
  amber_count=0;
  for(int i=0;i<m.get_ndihedral();i++){
    
    //atom number
    a1=m.get_dihedral_iatom1(i);
    a2=m.get_dihedral_iatom2(i);
    a3=m.get_dihedral_iatom3(i);
    a4=m.get_dihedral_iatom4(i);

    // improper
    improper=m.get_dihedral_chk_improper(i);

    // atomic number
    atomic1=m.get_atom(a1).get_atomic_number();
    atomic2=m.get_atom(a2).get_atomic_number();
    atomic3=m.get_atom(a3).get_atomic_number();
    atomic4=m.get_atom(a4).get_atomic_number();

    // atom coordinate
    r1.copy(m.get_atom(a1).get_R());
    r2.copy(m.get_atom(a2).get_R());
    r3.copy(m.get_atom(a3).get_R());
    r4.copy(m.get_atom(a4).get_R());

    // atom type of paics
    ptyp1=m.get_atom(a1).get_bio1();
    ptyp2=m.get_atom(a2).get_bio1();
    ptyp3=m.get_atom(a3).get_bio1();
    ptyp4=m.get_atom(a4).get_bio1();

    // biounit of the atom
    ibu1=m.get_atom(a1).get_bio3();
    ibu2=m.get_atom(a2).get_bio3();
    ibu3=m.get_atom(a3).get_bio3();
    ibu4=m.get_atom(a4).get_bio3();

    // residue type
    if(ibu1>=0){res_type1=m.get_biounit(ibu1).get_type();}
    if(ibu2>=0){res_type2=m.get_biounit(ibu2).get_type();}
    if(ibu3>=0){res_type3=m.get_biounit(ibu3).get_type();}
    if(ibu4>=0){res_type4=m.get_biounit(ibu4).get_type();}

    // cterm
    tc1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_c()>0){tc1=1;}}
    tc2=0; if(ibu2>=0){if(m.get_biounit(ibu2).get_terminal_c()>0){tc2=1;}}
    tc3=0; if(ibu3>=0){if(m.get_biounit(ibu3).get_terminal_c()>0){tc3=1;}}
    tc4=0; if(ibu4>=0){if(m.get_biounit(ibu4).get_terminal_c()>0){tc4=1;}}
    
    // nterm
    tn1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_n()>0){tn1=1;}}
    tn2=0; if(ibu2>=0){if(m.get_biounit(ibu2).get_terminal_n()>0){tn2=1;}}
    tn3=0; if(ibu3>=0){if(m.get_biounit(ibu3).get_terminal_n()>0){tn3=1;}}
    tn4=0; if(ibu4>=0){if(m.get_biounit(ibu4).get_terminal_n()>0){tn4=1;}}

    // terminal
    t1=0; if(ibu1>=0){if(tn1>0){t1=1;} if(tc1>0){t1=2;}}
    t2=0; if(ibu2>=0){if(tn2>0){t2=1;} if(tc2>0){t2=2;}}
    t3=0; if(ibu3>=0){if(tn3>0){t3=1;} if(tc3>0){t3=2;}}
    t4=0; if(ibu4>=0){if(tn4>0){t4=1;} if(tc4>0){t4=2;}}

    // atom type of amber
    if(ibu1 >=0 && res_type1==1800){
      his1814=m.get_biounit(ibu1).get_h1814();
      his1816=m.get_biounit(ibu1).get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
        cout << "error in his check!! @ print dihedral type1 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
        exit(1);
      }
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,his);
    } else if(ibu1 >=0 && res_type1==1550){
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,1);
    } else {
      amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1);
    }
    if(ibu2 >=0 && res_type2==1800){
      his1814=m.get_biounit(ibu2).get_h1814();
      his1816=m.get_biounit(ibu2).get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
        cout << "error in his check!! @ print dihedral type2 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
        exit(1);
      }
      amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2,his);
    } else if(ibu2 >=0 && res_type2==1550){
      amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2,1);
    } else {
      amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2);
    }
    if(ibu3 >= 0 && res_type3==1800){
      his1814=m.get_biounit(ibu3).get_h1814();
      his1816=m.get_biounit(ibu3).get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
        cout << "error in his check!! @ print dihedral type3 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
        exit(1);
      }
      amber3=ForceField::amber_force_field_get_atom_type(ptyp3,t3,his);
    } else if(ibu3 >= 0 && res_type3==1550){
      amber3=ForceField::amber_force_field_get_atom_type(ptyp3,t3,1);
    } else {
      amber3=ForceField::amber_force_field_get_atom_type(ptyp3,t3);
    }
    if(ibu4>=0 && res_type4==1800){
      his1814=m.get_biounit(ibu4).get_h1814();
      his1816=m.get_biounit(ibu4).get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
        cout << "error in his check!! @ print dihedral type4 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
        exit(1);
      }
      amber4=ForceField::amber_force_field_get_atom_type(ptyp4,t4,his);
    } else if(ibu4 >= 0 && res_type4==1550){
      amber4=ForceField::amber_force_field_get_atom_type(ptyp4,t4,1);
    } else {
      amber4=ForceField::amber_force_field_get_atom_type(ptyp4,t4);
    }

    // amber force field
    if(amber1 != "**" &&
       amber2 != "**" &&
       amber3 != "**" &&
       amber4 != "**" ){
      chk=ForceField::amber_force_field_dihedral(amber1,amber2,amber3,amber4,improper,&n,coeff,div,eq,phase);
    }
    
    if(chk != -1) {
      
      n_amber=n_amber+n;
      if(improper==1){
        n_improper=n_improper+n;
      }
      if(atomic1!=1 && atomic2!=1 && atomic3!=1 && atomic4!=1){
        n_not_h=n_not_h+n;
        if(improper==1){
          n_improper_not_h=n_improper_not_h+n;
        } 
      } else {
        n_h=n_h+n;
        if(improper==1){
          n_improper_h=n_improper_h+n;
        } 
      }

      dih=Dihedral::calc(r1,r2,r3,r4);

      cout.width (6); cout << amber_count+1 ; amber_count++; 
      cout.width (6); cout << a1+1 ;
      cout.width (6); cout << a2+1 ;
      cout.width (6); cout << a3+1 ;
      cout.width (6); cout << a4+1 ;
      cout.width (6); cout << amber1 ;
      cout.width (6); cout << amber2 ;
      cout.width (6); cout << amber3 ;
      cout.width (6); cout << amber4 ;
      for(int j=0;j<n;j++){
        cout.setf(ios::fixed,ios::floatfield);
        cout.precision(4);
        cout.width( 6); cout << n;
        cout.width(10); cout << coeff[j]/div  [j];
        cout << " (";
        cout.width(10); cout << coeff[j];
        cout.width(10); cout << div  [j];
        cout << ")";
        cout.width(10); cout << eq   [j];
        cout.width(10); cout << phase[j];
        cout.setf(ios::fixed,ios::floatfield);
        cout.precision(4);
        cout.width(10); cout << dih * 180.00/3.141594;
        v=(coeff[j]/div[j])*(1.0+cos(phase[j]*dih-eq[j]*3.141594/180.00));
        v_sum=v_sum+v;
        cout.setf(ios::fixed,ios::floatfield);
        cout.precision(10);
        cout.width(18); cout << v ;
        if(improper==1) {cout << " <-- improper";}
          if(n>1 && (j+1)!=n) {
            cout << endl ;
            cout.width (6); cout << amber_count+1 ; amber_count++;
            cout.width (6); cout << a1+1 ;
            cout.width (6); cout << a2+1 ;
            cout.width (6); cout << a3+1 ;
            cout.width (6); cout << a4+1 ;
            cout.width (6); cout << amber1 ;
            cout.width (6); cout << amber2 ;
            cout.width (6); cout << amber3 ;
            cout.width (6); cout << amber4 ;
          }
      }
      cout << endl ;
    }
    
  }
  cout << "dihedral     containing hydrogen (NPHIH) = " << n_h     << endl ;
  cout << "dihedral not containing hydrogen (NPHIA) = " << n_not_h << endl ;
  cout << "dihedral (amber like count)              = " << n_amber << endl ; 
  cout << "improper       = " << n_improper << endl ;
  cout << "improper_h     = " << n_improper_h << endl ;
  cout << "improper_not_h = " << n_improper_not_h << endl ;
  cout << " v_sum=" << v_sum << endl ;
}
*/
// print out for develop
/*
void ForceField::print_amber_force_field_excluded_atom ( const Molecule& m ,
							 const int * out_n ,
							 const int * out   )
{
  int natom,n_sum,n,a1,a2,ptyp1,ptyp2,ibu1,ibu2,tc1,tc2,tn1,tn2,t1,t2,res_type1,res_type2,his1,his2,his1814,his1816,chk;
  double chg1,vdw_r1,vdw_e1,chg2,vdw_r2,vdw_e2,d,coeffA,coeffB,v1,v2,v_sum1,v_sum2;
  string amber1,amber2,pdb1,pdb2;
  Dvector3D r1,r2;

  natom=m.get_natom();

  n_sum=0;
  for(int i=0;i<natom;i++){
    n_sum=n_sum+out_n[i];
  }
  cout << "< EXCLUDED_ATOMS >" << n_sum << "(" << n_sum+1 << ")" << endl;
  for(int i=0;i<natom;i++){
    cout.width(6); cout << i+1 ;
    cout.width(6); cout << out_n[i] ;
    cout << " : " ;
    for(int j=0;j<out_n[i];j++){
      cout.width(6); cout << out[i*natom+j]+1 ;
    }
    cout << endl ;
  }

  n=0;
  v_sum1=0.0;
  v_sum2=0.0;
  for(int i=0;i<natom;i++){
    for(int j=i+1;j<natom;j++){
      chk=0;
      for(int k=0;k<out_n[i];k++){
	if(j==out[i*natom+k]){
	  chk=1;
	}
      }
      if(chk==0){

	n++;

	// atom number
	a1=i;
	a2=j;

	// atom coordinate
	r1.copy(m.get_atom(a1).get_R());
	r2.copy(m.get_atom(a2).get_R());

	// atom type of paics
	ptyp1=m.get_atom(a1).get_bio1();
	ptyp2=m.get_atom(a2).get_bio1();

	// biounit of the atom
	ibu1=m.get_atom(a1).get_bio3();
	ibu2=m.get_atom(a2).get_bio3();

	// cterm
	tc1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_c()>0){tc1=1;}}
	tc2=0; if(ibu2>=0){if(m.get_biounit(ibu2).get_terminal_c()>0){tc2=1;}}

	// nterm
	tn1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_n()>0){tn1=1;}}
	tn2=0; if(ibu2>=0){if(m.get_biounit(ibu2).get_terminal_n()>0){tn2=1;}}

	// terminal
	t1=0; if(ibu1>=0){if(tn1>0){t1=1;} if(tc1>0){t1=2;}}
	t2=0; if(ibu2>=0){if(tn2>0){t2=1;} if(tc2>0){t2=2;}}

	// residue type
	if(ibu1>=0){res_type1=m.get_biounit(ibu1).get_type();}
	if(ibu2>=0){res_type2=m.get_biounit(ibu2).get_type();}

        // his_type (only res_type==1800)
        if(ibu1>=0 && res_type1==1800){
          his1814=m.get_biounit(ibu1).get_h1814();
          his1816=m.get_biounit(ibu1).get_h1816();
          if     (his1814>=0 && his1816< 0) {his1=0;} // HID
          else if(his1814 <0 && his1816>=0) {his1=1;} // HIE
          else if(his1814>=0 && his1816>=0) {his1=2;} // HIP
          else {
            cout << "error in his check!!" << endl ;
            exit(1);
          }
        }
        if(ibu2>=0 && res_type2==1800){
          his1814=m.get_biounit(ibu2).get_h1814();
          his1816=m.get_biounit(ibu2).get_h1816();
          if     (his1814>=0 && his1816< 0) {his2=0;} // HID
          else if(his1814 <0 && his1816>=0) {his2=1;} // HIE
          else if(his1814>=0 && his1816>=0) {his2=2;} // HIP
          else {
            cout << "error in his check!!" << endl ;
            exit(1);
          }
        }

	// pdb tag
	pdb1=BioUnit::pdb_tag(ptyp1);
	pdb2=BioUnit::pdb_tag(ptyp2);

	// atom type of amber
	if     (ibu1>=0 && res_type1==1800){amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,his1);}
        else if(ibu1>=0 && res_type1==1550){amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,1)   ;}
	else                               {amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1)     ;}
	if     (ibu2>=0 && res_type2==1800){amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2,his2);}
        else if(ibu2>=0 && res_type2==1550){amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2,1)   ;}
	else                               {amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2)     ;}

	d=Dvector3D::distance1(r1,r2)*0.529177211;
	chk=ForceField::amber_force_field_charge (amber1,pdb1,res_type1,t1,&chg1,his1);
	if(chk<0){
	  cout << "error!! in force_field_charge" ;
	  cout.width(6); cout << amber1 ;
	  cout.width(6); cout << res_type1 ;
	  cout.width(6); cout << t1 ;
	  cout << endl ;
	}
	chk=ForceField::amber_force_field_charge (amber2,pdb2,res_type2,t2,&chg2,his2);
	if(chk<0){
	  cout << "error!! in force_field_charge" ;
	  cout.width(6); cout << amber2 ;
	  cout.width(6); cout << res_type2 ;
	  cout.width(6); cout << t2 ;
	  cout << endl ;
	}
	//chk=ForceField::amber_force_field_vdw    (amber1,&vdw_r1,&vdw_e1);
	chk=amber_force_field_vdw    (amber1,&vdw_r1,&vdw_e1);
	if(chk<0){
	  cout << "error!! in force_field_vdw" << endl ;
	  cout.width(6); cout << amber1 ;
	  cout << endl ;
	}
	//chk=ForceField::amber_force_field_vdw    (amber2,&vdw_r2,&vdw_e2);
	chk=amber_force_field_vdw    (amber2,&vdw_r2,&vdw_e2);
	if(chk<0){
	  cout << "error!! in force_field_vdw" << endl ;
	  cout.width(6); cout << amber2 ;
	  cout << endl ;
	}
	coeffA=    sqrt(vdw_e1*vdw_e2)*pow(vdw_r1+vdw_r2,12);
	coeffB=2.0*sqrt(vdw_e1*vdw_e2)*pow(vdw_r1+vdw_r2, 6);
	v1=coeffA/pow(d,12)-coeffB/pow(d,6);
	v_sum1=v_sum1+v1;
	v2=(chg1*18.2223)*(chg2*18.2223)/d;
	v_sum2=v_sum2+v2;
      }
    }
  }
  printf("v_sum1=%20.10f\n",v_sum1);
  printf("v_sum2=%20.10f\n",v_sum2);
}
*/
// print out for develop
/*
void ForceField::print_amber_force_field_14_pair ( const Molecule& m,
						   const int * out_n,
						   const int * out_1,
						   const int * out_2)
{
  int chk,a1,a2,ptyp1,ptyp2,ibu1,ibu2,tc1,tc2,tn1,tn2,t1,t2,res_type1,res_type2,his1,his2,his1814,his1816;
  double chg1,vdw_r1,vdw_e1,chg2,vdw_r2,vdw_e2,d,coeffA,coeffB,v1,v2,v_sum1,v_sum2;
  string amber1,amber2,pdb1,pdb2;
  Dvector3D r1,r2;

  cout << "< 14 PAIR > " << *out_n << endl ;

  v_sum1=0.0;
  v_sum2=0.0;
  for(int i=0;i<(*out_n);i++){

    // atom number
    a1=out_1[i];
    a2=out_2[i];

    // atom coordinate
    r1.copy(m.get_atom(a1).get_R());
    r2.copy(m.get_atom(a2).get_R());

    
    // atom type of paics
    ptyp1=m.get_atom(a1).get_bio1();
    ptyp2=m.get_atom(a2).get_bio1();
    
    // biounit of the atom
    ibu1=m.get_atom(a1).get_bio3();
    ibu2=m.get_atom(a2).get_bio3();
    
    // cterm
    tc1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_c()>0){tc1=1;}}
    tc2=0; if(ibu2>=0){if(m.get_biounit(ibu2).get_terminal_c()>0){tc2=1;}}
    
    // nterm
    tn1=0; if(ibu1>=0){if(m.get_biounit(ibu1).get_terminal_n()>0){tn1=1;}}
    tn2=0; if(ibu2>=0){if(m.get_biounit(ibu2).get_terminal_n()>0){tn2=1;}}
    
    // terminal
    t1=0; if(ibu1>=0){if(tn1>0){t1=1;} if(tc1>0){t1=2;}}
    t2=0; if(ibu2>=0){if(tn2>0){t2=1;} if(tc2>0){t2=2;}}
    
    // residue type
    if(ibu1>=0){res_type1=m.get_biounit(ibu1).get_type();}
    if(ibu2>=0){res_type2=m.get_biounit(ibu2).get_type();}

    // his type (only res_type==1800)
    if(ibu1>=0 && res_type1==1800){
      his1814=m.get_biounit(ibu1).get_h1814();
      his1816=m.get_biounit(ibu1).get_h1816();
      if     (his1814>=0 && his1816< 0) {his1=0;} // HID
      else if(his1814 <0 && his1816>=0) {his1=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his1=2;} // HIP
      else {
        cout << "error in his check!!" << endl ;
        exit(1);
      }
    }
    if(ibu2>=0 && res_type2==1800){
      his1814=m.get_biounit(ibu2).get_h1814();
      his1816=m.get_biounit(ibu2).get_h1816();
      if     (his1814>=0 && his1816< 0) {his2=0;} // HID
      else if(his1814 <0 && his1816>=0) {his2=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his2=2;} // HIP
      else {
        cout << "error in his check!!" << endl ;
        exit(1);
      }
    }

    // pdb tag
    pdb1=BioUnit::pdb_tag(ptyp1);
    pdb2=BioUnit::pdb_tag(ptyp2);
    
    // atom type of amber
    if     (ibu1>=0 && res_type1==1800){amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,his1);}
    else if(ibu1>=0 && res_type1==1550){amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1,1)   ;}
    else                               {amber1=ForceField::amber_force_field_get_atom_type(ptyp1,t1)     ;}
    if     (ibu2>=0 && res_type2==1800){amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2,his2);}
    else if(ibu2>=0 && res_type2==1550){amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2,1)   ;}
    else                               {amber2=ForceField::amber_force_field_get_atom_type(ptyp2,t2)     ;}
    
    d=Dvector3D::distance1(r1,r2)*0.529177211;
    chk=ForceField::amber_force_field_charge(amber1,pdb1,res_type1,t1,&chg1,his1);
    if(chk<0){
      cout << "error!! in force_field_charge" ;
      cout.width(6); cout << amber1 ;
      cout.width(6); cout << res_type1 ;
      cout.width(6); cout << t1 ;
      cout << endl ;
    }
    chk=ForceField::amber_force_field_charge(amber2,pdb2,res_type2,t2,&chg2,his2);
    if(chk<0){
      cout << "error!! in force_field_charge" ;
      cout.width(6); cout << amber2 ;
      cout.width(6); cout << res_type2 ;
      cout.width(6); cout << t2 ;
      cout << endl ;
    }
    //chk=ForceField::amber_force_field_vdw(amber1,&vdw_r1,&vdw_e1);
    chk=amber_force_field_vdw(amber1,&vdw_r1,&vdw_e1);
    if(chk<0){
      cout << "error!! in force_field_vdw" << endl ;
      cout.width(6); cout << amber1 ;
      cout << endl ;
    }
    //chk=ForceField::amber_force_field_vdw(amber2,&vdw_r2,&vdw_e2);
    chk=amber_force_field_vdw(amber2,&vdw_r2,&vdw_e2);
    if(chk<0){
      cout << "error!! in force_field_vdw" << endl ;
      cout.width(6); cout << amber2 ;
      cout << endl ;
    }
    coeffA=    sqrt(vdw_e1*vdw_e2)*pow(vdw_r1+vdw_r2,12);
    coeffB=2.0*sqrt(vdw_e1*vdw_e2)*pow(vdw_r1+vdw_r2, 6);
    v1=coeffA/pow(d,12)-coeffB/pow(d,6);
    v_sum1=v_sum1+v1/2.00;
    printf("%6d,%6d%6d,%6s%6s,%20.8e%20.8e%20.10f%20.10f\n",i+1,a1+1,a2+1,amber1.c_str(),amber2.c_str(),coeffA,coeffB,d,v1);
    v2=(chg1*18.2223)*(chg2*18.2223)/d;
    v_sum2=v_sum2+v2/1.20;
  }
  printf("v_sum1=%20.10f\n",v_sum1);
  printf("v_sum2=%20.10f\n",v_sum2);
}
*/
