
#include <iostream>
#include <cstdlib>
#include "Fragment.h"

using namespace std;

Fragment::Fragment()
{
  n_atom_index = 0;
  n_link = 0;
  n_elec = 0;
  //n_nuc  = 0;
  n_atom_index_buff = 0;
  n_link_buff = 0;
  opt = 0;
  abmd = 0;
  chk_view = -1;
}

Fragment::Fragment(const Fragment& f)
{
  n_atom_index = 0;
  n_link = 0;
  n_elec = 0;
  //n_nuc  = 0;
  n_atom_index_buff = 0;
  n_link_buff = 0;
  opt = 0;
  abmd = 0;
  chk_view = -1;
  copy(f);
}

Fragment::~Fragment()
{
  clear_atom_index();
  clear_atom_index_link();
}

void Fragment::copy(const Fragment& f)
{
  if(this==&f) return;
  create_atom_index(f.n_atom_index_buff);
  for(int i=0;i<f.n_atom_index;i++){
    atom_index[i]=f.atom_index[i];
    n_atom_index++;
  }
  create_atom_index_link(f.n_link_buff);
  for(int i=0;i<f.n_link;i++){
    atom_index_link1[i]=f.atom_index_link1[i];
    atom_index_link2[i]=f.atom_index_link2[i];
    n_link++;
  }
  n_elec=f.n_elec;
  //n_nuc =f.n_nuc;
  opt=f.opt;
  abmd=f.abmd;
  chk_view=f.chk_view;
}

void Fragment::clear()
{
  clear_atom_index();
  clear_atom_index_link();
  n_elec=0;
  opt=0;
  abmd=0;
  chk_view=-1;
}

void Fragment::clear_atom_index()
{
  if(n_atom_index_buff==0) return ;
  delete [] atom_index ;
  n_atom_index_buff=0;
  n_atom_index     =0;
}

void Fragment::create_atom_index(const int n)
{
  clear_atom_index();
  if(n==0) return;
  atom_index=new int [n] ;
  n_atom_index_buff=n;
  n_atom_index=0;
}

void Fragment::resize_atom_index(const int n)
{
  int     natom_buf ;
  int*    atom_index_buf ;
  natom_buf=n_atom_index;
  atom_index_buf = new int [n_atom_index+1];
  for(int i=0;i<n_atom_index;i++){
    atom_index_buf[i]=atom_index[i];
  }
  create_atom_index(n);
  if(n>natom_buf){
    for(int i=0;i<natom_buf;i++){
      atom_index[i]=atom_index_buf[i];
      n_atom_index++;
    }
  } else {
    for(int i=0;i<n;i++){
      atom_index[i]=atom_index_buf[i];
      n_atom_index++;
    }
  }
  delete [] atom_index_buf;
}

void Fragment::clear_atom_index_link()
{
  if(n_link_buff==0) return ;
  delete [] atom_index_link1 ;
  delete [] atom_index_link2 ;
  n_link_buff=0;
  n_link     =0;
}

void Fragment::create_atom_index_link(const int n)
{
  clear_atom_index_link();
  if(n==0) return;
  atom_index_link1 = new int [n] ;
  atom_index_link2 = new int [n] ;
  n_link_buff=n;
  n_link     =0;
}

void Fragment::resize_atom_index_link(const int n)
{
  int  nlink_buf ;
  int* link_index1_buf ;
  int* link_index2_buf ;
  nlink_buf=n_link;
  link_index1_buf = new int [n_link+1];
  link_index2_buf = new int [n_link+1];
  for(int i=0;i<n_link;i++){
    link_index1_buf[i]=atom_index_link1[i];
    link_index2_buf[i]=atom_index_link2[i];
  }
  create_atom_index_link(n);
  if(n>nlink_buf){
    for(int i=0;i<nlink_buf;i++){
      atom_index_link1[i]=link_index1_buf[i];
      atom_index_link2[i]=link_index2_buf[i];
      n_link++;
    }
  } else {
    for(int i=0;i<n;i++){
      atom_index_link1[i]=link_index1_buf[i];
      atom_index_link2[i]=link_index2_buf[i];
      n_link++;
    }
  }
  delete [] link_index1_buf;
  delete [] link_index2_buf;
}

int Fragment::get_n_atom_index ( ) const
{
  return n_atom_index;
}

int Fragment::get_n_link ( ) const
{
  return n_link;
}

int Fragment::get_n_elec ( ) const
{
  return n_elec;
}

int Fragment::get_chk_view ( ) const
{
  return chk_view;
}

//int Fragment::get_n_nuc ( ) const
//{
//  return n_nuc;
//}

int Fragment::get_atom_index (const int iatom) const
{
  if(iatom<0){
    cerr << "error in Fragment::get_atom_index , iatom < 0" << endl ;
    cerr << "iatom = " << iatom << endl ;
    exit(1);
  }
  if(iatom>=n_atom_index){
    cerr << "error in Fragment::get_atom_index , iatom >= n_atom_index" << endl ;
    cerr << "iatom = " << iatom << " , n_atom_index = " << n_atom_index << endl ;
    exit(1);
  }
  return atom_index[iatom];
}

int Fragment::get_atom_index_link1 (const int ilink) const
{
  if(ilink<0){
    cerr << "error in Fragment::get_atom_index_link1 , ilink < 0" << endl ;
    cerr << "ilink = " << ilink << endl ;
    exit(1);
  }
  if(ilink>=n_link){
    cerr << "error in Fragment::get_atom_index_link1 , ilink >= n_link" << endl ;
    cerr << "ilink = " << ilink << " , n_link = " << n_link << endl ;
    exit(1);
  }
  return atom_index_link1[ilink];
}

int Fragment::get_atom_index_link2 (const int ilink) const
{
  if(ilink<0){
    cerr << "error in Fragment::get_atom_index_link2 , ilink < 0" << endl ;
    cerr << "ilink = " << ilink << endl ;
    exit(1);
  }
  if(ilink>=n_link){
    cerr << "error in Fragment::get_atom_index_link2 , ilink >= n_link" << endl ;
    cerr << "ilink = " << ilink << " , n_link = " << n_link << endl ;
    exit(1);
  }
  return atom_index_link2[ilink];
}

int Fragment::get_opt() const 
{
  return opt;
}

int Fragment::get_abmd() const 
{
  return abmd;
}

int Fragment::chk_atom_in (const int index_) const
{
  for(int i=0;i<n_atom_index;i++){
    if(index_==atom_index[i]){
      return i;
    }
  }
  return -1;
}

void Fragment::push_back_atom_index ( const int iatom , const int buff_increase)
{
  if(n_atom_index==n_atom_index_buff){
    resize_atom_index(n_atom_index+buff_increase);
  }
  atom_index[n_atom_index]=iatom;
  n_atom_index++;
}

void Fragment::pick_up_atom_index ( const int iatom )
{
  if(iatom<0){
    cerr << "error in Fragment::pick_up_atom_index , iatom < 0" << endl ;
    cerr << "iatom = " << iatom << endl ;
    exit(1);
  }
  if(iatom>=n_atom_index) {
    cerr << "error in Fragment::pick_up_atom_index : iatom >= n_atom_index" << endl ;
    cerr << "iatom = " << iatom << " , n_atom_index = " << n_atom_index << endl ;
    exit(1);
  }
  for(int i=iatom;i<n_atom_index-1;i++){
    atom_index[i]=atom_index[i+1];
  }
  n_atom_index--;
}

void Fragment::push_back_atom_index_link ( const int iatom1, const int iatom2, const int buff_increase)
{
  if(n_link==n_link_buff){
    resize_atom_index_link(n_link+buff_increase);
  }
  atom_index_link1[n_link]=iatom1;
  atom_index_link2[n_link]=iatom2;
  n_link++;
}

void Fragment::pick_up_atom_index_link ( const int ilink )
{
  if(ilink<0){
    cerr << "error in Fragment::pick_up_atom_index_link , ilink < 0" << endl;
    cerr << "ilink = " << ilink << endl ;
    exit(1);
  }
  if(ilink>=n_link){
    cerr << "error in Fragment::pick_up_atom_index_link , ilink >= n_link" << endl ;
    cerr << "ilink = " << ilink << " , n_link = " << n_link << endl ;
    exit(1);
  }
  for(int i=ilink;i<n_link-1;i++){
    atom_index_link1[i]=atom_index_link1[i+1];
    atom_index_link2[i]=atom_index_link2[i+1];
  }
  n_link--;
}

void Fragment::push_back_elec ( const int n )
{
  n_elec=n_elec+n;
}

void Fragment::pick_up_elec ( const int n )
{
  n_elec=n_elec-n;
}

Fragment Fragment::unit_fragment( const Fragment& f1 ,
                                  const Fragment& f2 )
{
  Fragment f;
  for(int i=0;i<f1.n_atom_index;i++){
    f.push_back_atom_index(f1.atom_index[i]);
  }
  for(int i=0;i<f2.n_atom_index;i++){
    f.push_back_atom_index(f2.atom_index[i]);
  }
  for(int i=0;i<f1.n_link;i++){
    int chk=0;
    for(int j=0;j<f2.n_atom_index;j++){
      if(f1.atom_index_link1[i]==f2.atom_index[j]) chk=1;
    }
    if(chk==0){
      f.push_back_atom_index_link(f1.atom_index_link1[i],f1.atom_index_link2[i]);
    }
  }
  for(int i=0;i<f2.n_link;i++){
    int chk=0;
    for(int j=0;j<f1.n_atom_index;j++){
      if(f2.atom_index_link1[i]==f1.atom_index[j]) chk=1;
    }
    if(chk==0){
      f.push_back_atom_index_link(f2.atom_index_link1[i],f2.atom_index_link2[i]);
    }
  }
  f.push_back_elec(f1.n_elec);
  f.push_back_elec(f2.n_elec);
  return f;
}

void Fragment::set_opt(const int opt_)
{
  opt=opt_;
}

void Fragment::set_abmd(const int abmd_)
{
  abmd=abmd_;
}

void Fragment::set_chk_view(const int n)
{
  chk_view=n;
}
