

#include <cstdlib>
#include <iostream>
#include <string>
#include "LinkGroup.h"

using namespace std;

LinkGroup::LinkGroup()
{
  n_atom_index=0;
  n_atom_index_buff=0;
  residue_index=1;
  type="unknown";
  not_peptide=0;
  not_dna=0;
}

LinkGroup::LinkGroup( const LinkGroup& lg )
{
  n_atom_index=0;
  n_atom_index_buff=0;
  residue_index=1 ;
  type="unknown";
  not_peptide=0;
  not_dna=0;
  copy(lg);
}

LinkGroup::~LinkGroup()
{
  clear_atom_index();
}

void LinkGroup::copy( const LinkGroup& lg )
{
  if(this==&lg) return;
  create_atom_index(lg.n_atom_index_buff);
  for(int i=0;i<lg.n_atom_index;i++){
    atom_index[i]=lg.atom_index[i];
    n_atom_index++;
  }
  residue_index=lg.residue_index;
  type=lg.type;
  not_peptide=lg.not_peptide;
  not_dna=lg.not_dna;
}

void LinkGroup::clear()
{
  clear_atom_index();
}

void LinkGroup::clear_atom_index()
{
  if(n_atom_index_buff==0) return ;
  delete [] atom_index;
  n_atom_index_buff=0;
  n_atom_index=0;
}

void LinkGroup::create_atom_index( const int n )
{
  clear_atom_index();
  if(n==0) return;
  atom_index = new int [n] ;
  n_atom_index_buff=n;
  n_atom_index=0;
}

void LinkGroup::resize_atom_index( const int n )
{
  int     natom_buf ;
  int*    atom_index_buf ;
  natom_buf=n_atom_index;
  atom_index_buf = new int [n_atom_index+1];
  for(int i=0;i<n_atom_index;i++){
    atom_index_buf[i]=atom_index[i];
  }
  create_atom_index(n);
  if(n>natom_buf){
    for(int i=0;i<natom_buf;i++){
      atom_index[i]=atom_index_buf[i];
      n_atom_index++;
    }
  } else {
    for(int i=0;i<n;i++){
      atom_index[i]=atom_index_buf[i];
      n_atom_index++;
    }
  }
  delete [] atom_index_buf;
}

int LinkGroup::get_n_atom_index ( ) const
{
  return n_atom_index;
}

int LinkGroup::get_atom_index ( const int iatom ) const
{
  if(iatom<0){
    cerr << "error in LinkGroup::get_atom_index , iatom < 0" << endl ;
    cerr << "iatom = " << iatom << endl ;
    exit(1);
  }
  if(iatom>=n_atom_index){
    cerr << "error in LinkGroup::get_atom_index , iatom >= n_atom_index" << endl ;
    cerr << "iatom = " << iatom << " , n_atom_index = " << n_atom_index << endl ;
    exit(1);
  }
  return atom_index[iatom];
}

int LinkGroup::get_residue_index ( ) const
{
  return residue_index;
}

int LinkGroup::chk_atom_in ( const int index_ ) const
{
  for(int i=0;i<n_atom_index;i++){
    if(index_==atom_index[i]){
      return i;
    }
  }
  return -1;
}

void LinkGroup::push_back_atom_index ( const int iatom , const int buff_increase )
{
  if(n_atom_index==n_atom_index_buff){
    resize_atom_index(n_atom_index+buff_increase);
  }
  atom_index[n_atom_index]=iatom;
  n_atom_index++;
}

void LinkGroup::pick_up_atom_index ( const int iatom )
{
  if(iatom>=n_atom_index) {
    cerr << "error : LinkGroup::pick_up_atom : iatom >= n_atom_index";
    exit(1);
  }
  for(int i=iatom;i<n_atom_index-1;i++){
    atom_index[i]=atom_index[i+1];
  }
  n_atom_index--;
}

void LinkGroup::set_residue_index ( const int n )
{
  residue_index=n;
}

string LinkGroup::get_type( ) const
{
  return type;
}

void LinkGroup::set_type(const string& t_)
{
  type=t_;
}

int LinkGroup::get_not_peptide() const 
{
  return not_peptide;
}

int LinkGroup::get_not_dna() const {
  return not_dna;
}

void LinkGroup::set_not_peptide(const int n_)
{
  not_peptide=n_;
}

void LinkGroup::set_not_dna(const int n_)
{
  not_dna=n_;
}
