
#include <cstdlib>
#include "Molecule.h"
#include "Angle.h"
#include "ForceField.h"

void Molecule::clear_angle()
{
  //cout << "ishi in Molecule::clear_angle() nangle_buff = " << nangle_buff << endl ;
  if(nangle_buff==0){
    return;
  }
  for(int i=(nangle-1);i>=0;i--){
    pick_up_angle(i);
  }
  delete [] angle;
  nangle_buff=0;
  //cout << "ishi out Molecule::clear_angle()" << endl ;
}

void Molecule::create_angle(const int n)
{
  clear_angle();
  if(n==0) return;
  angle = new Angle [n];
  nangle_buff=n;
}

void Molecule::resize_angle(const int n)
{
  int   tmp_nangle=nangle;
  Angle* tmp_angle;
  tmp_angle = new Angle [tmp_nangle+1];
  for(int i=0;i<tmp_nangle;i++){
    tmp_angle[i].copy(angle[i]);
  }
  create_angle(n);
  if(n>tmp_nangle){
    for(int i=0;i<tmp_nangle;i++){
      push_back_angle(tmp_angle[i]);
    } 
  } else {
    for(int i=0;i<n;i++){
      push_back_angle(tmp_angle[i]);
    }
  }
  delete [] tmp_angle;
}

// private method //
void Molecule::push_back_angle( const Angle& an,
				const int   buff_increase)
{
  push_back_angle(an.get_iatom1(),an.get_iatom2(),an.get_iatom3(),buff_increase);
}

// private method //
void Molecule::push_back_angle( const int iatom1, 
				const int iatom2,
				const int iatom3,
				const int buff_increase,
				const int ierr)
{
  if(nangle==nangle_buff){
    resize_angle(nangle+buff_increase);
  }
  angle[nangle].set_iatom1(iatom1);
  angle[nangle].set_iatom2(iatom2);
  angle[nangle].set_iatom3(iatom3);
  nangle++;
}

// private method //
Angle Molecule::pick_up_angle(const int iangle,
			      const int ierr )
{
  Angle tmp1(angle[iangle]);
  for(int i=iangle;i<nangle-1;i++){
    angle[i].copy(angle[i+1]);
  }
  nangle--;
  return tmp1;
}

int Molecule::get_nangle( ) const
{
  return nangle;
}
  
int Molecule::get_angle_iatom1( const int iangle ,
				const int ierr   ) const
{
  return angle[iangle].get_iatom1();
}
  
int Molecule::get_angle_iatom2( const int iangle ,
				const int ierr   ) const 
{
  return angle[iangle].get_iatom2();
}

int Molecule::get_angle_iatom3( const int iangle ,
				const int ierr   ) const
{
  return angle[iangle].get_iatom3();
}

void Molecule::add_angle_auto( )
{
  int ia1,ia2,ib1,ib2;
  //cout << "ishi in Molecule::add_angle_auto" << endl ;
  clear_angle();
  for(int i=0;i<nbond;i++){
    ia1=bond[i].get_iatom1();
    ia2=bond[i].get_iatom2();
    for(int j=i+1;j<nbond;j++){
      ib1=bond[j].get_iatom1();
      ib2=bond[j].get_iatom2();
      if     (ia1==ib1){push_back_angle(ia2,ia1,ib2,nbond*6+1);}
      else if(ia1==ib2){push_back_angle(ia2,ia1,ib1,nbond*6+1);}
      else if(ia2==ib1){push_back_angle(ia1,ia2,ib2,nbond*6+1);}
      else if(ia2==ib2){push_back_angle(ia1,ia2,ib1,nbond*6+1);}
    }
  }
  //cout << "ishi out Molecule::add_angle_auto" << endl ;
}
