
#include <cstdlib>
#include <cstdio>
#include <math.h>
#include "Molecule.h"
#include "iString.h"
#include "ForceField.h"

extern int pv_debug; // for debug 20160827

// ierr = 0 : message and exit
// ierr = 1 : message and not exit
// ierr = 2 : not message and not exit

void Molecule::copy(const Atom& a_ , 
                    const int   iatom, 
                    const int   ierr)
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::copy(const Atom& a_, const int iatom) , iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Moelcule::copy(const Atom& a_, const int iatom) , iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;  
    }
    if(ierr==0) exit(1);
  }
  atom[iatom].copy(a_);
}

void Molecule::copy(const ViewAtom& va,
                    const int       iatom,
                    const int       ierr)
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::copy(const ViewAtom& va, const int iatom) , iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Moelcule::copy(const ViewAtom& va, const int iatom) , iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }  
    if(ierr==0) exit(1);
  }
  atom[iatom].set_va(va);
}

void Molecule::clear_atom()
{
  if(pv_debug==1){ cout << "ishi in    Molecule::clear_atom()" << endl ;}
  clear_bond();
  if(pv_debug==1){ cout << "ishi 01    Molecule::clear_atom()" << endl ;}
  clear_linkgroup();
  if(pv_debug==1){ cout << "ishi 02    Molecule::clear_atom()" << endl ;}
  clear_biounit();
  if(pv_debug==1){ cout << "ishi 03    Molecule::clear_atom()" << endl ;}
  clear_fragment();
  if(pv_debug==1){ cout << "ishi 04    Molecule::clear_atom()" << endl ;}
  if(natom_buff==0) {
  if(pv_debug==1){ cout << "ishi out-1 Molecule::clear_atom()" << endl ;}
    return;
  }
  if(pv_debug==1){ cout << "ishi 05    Molecule::clear_atom()" << endl ;}
  delete [] atom;
  if(pv_debug==1){ cout << "ishi 06    Molecule::clear_atom()" << endl ;}
  natom=0;
  if(pv_debug==1){ cout << "ishi 07    Molecule::clear_atom()" << endl ;}
  natom_buff=0;
  if(pv_debug==1){ cout << "ishi out-2 Molecule::clear_atom()" << endl ;}
}

void Molecule::create_atom(const int n)
{
  clear_atom();
  if(n==0) return;
  atom = new Atom [n];
  natom_buff=n;
}

void Molecule::resize_atom(const int n)
{
  int tmp_natom=natom;
  Atom * tmp_atom;
  tmp_atom = new Atom [tmp_natom+1];
  for(int i=0;i<tmp_natom;i++){
    tmp_atom[i].copy(atom[i]);
  }
  create_atom(n);
  if(n>tmp_natom){
    for(int i=0;i<tmp_natom;i++){
      atom[i].copy(tmp_atom[i]);
      natom++;
    }
  } else {
    for(int i=0;i<n;i++){
      atom[i].copy(tmp_atom[i]);
      natom++;
    }
  }
  delete [] tmp_atom ;
}

void Molecule::push_back_atom (const Atom& a, 
                               const int   buff_increase)
{
  if(natom==natom_buff){
    resize_atom(natom+buff_increase);
  }
  atom[natom].copy(a);
  natom++;
} 

Atom Molecule::pick_up_atom(const int iatom,
                            const int ierr)
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::puck_up_atom , iatom < 0" << endl;
      cerr << "iatom = " << iatom << " , this = " << this << endl;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::pick_up_atom , iatom >= natom" << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  Atom tmp1(atom[iatom]);
  for(int i=iatom;i<natom-1;i++){
    atom[i].copy(atom[i+1]);
  }
  natom--;
  return tmp1;
}

void Molecule::add_atom (const Atom& a,
                         const int increase_buff)
{
  clear_fragment();
  clear_biounit();
  clear_linkgroup(); 
  //clear_bond(); 20130803
  push_back_atom(a,increase_buff);
}

void Molecule::add_atom ( const int    an            ,
                          const double x_            ,
                          const double y_            ,
                          const double z_            ,
                          const int    increase_buff )
{
  //clear_fragment();
  //clear_biounit();
  //clear_linkgroup(); 
  //clear_bond();
  Dvector3D r(x_,y_,z_);
  Atom a;
  a.set_R(r);
  a.set_atomic_number(an);
  add_atom(a,increase_buff);
}

void Molecule::add_atom_sp3 ( const int iatom_        ,
                              const int atomic        ,
                              const int buff_increase ,
                              const int ierr          )
{
  if(iatom_<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_atom_sp3 , iatom_ < 0" << endl;
      cerr << "iatom_ = " << iatom_ << " , this = " << this << endl;
    }
    if(ierr==0) exit(1);
  }
  if(iatom_>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_atom_sp3 , iatom_ >= natom" << endl ;
      cerr << "iatom_ = " << iatom_ << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  Dvector3D r1,r2,r3,r4;
  double d1,d2;
  Atom a;
  a.set_atomic_number(atomic);
  d1=atom[iatom_].get_bond_distance()*0.8;
  d2=a.get_bond_distance()*0.8;
  int n=get_sp3_pos(iatom_,d1+d2,r1,r2,r3,r4);
  if(n>0){
    add_atom(atomic,r1.get_x(),r1.get_y(),r1.get_z(),buff_increase);
  }
}

void Molecule::add_atom_sp2 ( const int iatom_        ,
                              const int atomic        ,
                              const int buff_increase ,
                              const int ierr          )
{
  if(iatom_<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_atom_sp2 , iatom_ < 0" << endl;
      cerr << "iatom_ = " << iatom_ << " , this = " << this << endl;
    }
    if(ierr==0) exit(1);
  }
  if(iatom_>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_atom_sp2 , iatom_ >= natom" << endl ;
      cerr << "iatom_ = " << iatom_ << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  Dvector3D r1,r2,r3;
  double d1,d2;
  Atom a;
  a.set_atomic_number(atomic);
  d1=atom[iatom_].get_bond_distance()*0.8;
  d2=a.get_bond_distance()*0.8;
  int n=get_sp2_pos(iatom_,d1+d2,r1,r2,r3);
  if(n>0){
    add_atom(atomic,r1.get_x(),r1.get_y(),r1.get_z(),buff_increase);
  }
}

void Molecule::add_atom_sp1 ( const int iatom_        ,
                              const int atomic        ,
                              const int buff_increase ,
                              const int ierr          )
{
  if(iatom_<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_atom_sp1 , iatom_ < 0" << endl;
      cerr << "iatom_ = " << iatom_ << " , this = " << this << endl;
    }
    if(ierr==0) exit(1);
  }
  if(iatom_>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_atom_sp1 , iatom_ >= natom" << endl ;
      cerr << "iatom_ = " << iatom_ << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
}

void Molecule::add_atom_lattice( const int    an1  ,
				 const int    an2  ,
				 const int    bn1  ,
				 const int    bn2  ,
				 const int    cn1  ,
				 const int    cn2  ,
				 const double a    ,
				 const double b    ,
				 const double c    ,
				 const double alpha,
				 const double beta ,
				 const double gamma)
{
  Molecule mtmp;
  Dvector3D axis_a, axis_b, axis_c, tmptmp;
  double tmp1[3],tmp2[3];
  Dvector3D tmp_axis[3];
  Atom atom_;
  int ibuf[16];
  tmptmp.set_xyz(0.5, 0.5, 0.5);
  tmp1[0]=a;     tmp1[1]=b;    tmp1[2]=c;
  tmp2[0]=alpha; tmp2[1]=beta; tmp2[2]=gamma;
  Dvector3D::LatticeToXYZ(tmptmp, tmp1, tmp2, tmp_axis);
  axis_a.copy(tmp_axis[0]);
  axis_b.copy(tmp_axis[1]);
  axis_c.copy(tmp_axis[2]);
  mtmp.copy(*this);
  for(int i=an1 ; i<=an2 ; i++){
    for(int j=bn1 ; j<=bn2 ; j++) {
      for(int k=cn1 ; k<=cn2 ; k++) {
        if(i!=0 || j!=0 || k!=0) {
          for(int l=0 ; l<mtmp.get_natom() ; l++) {
            atom_.copy(mtmp.get_atom(l));
            tmptmp.copy(atom_.get_R());
            tmptmp=tmptmp+axis_a*i+axis_b*j+axis_c*k;
	    // same check
	    if(pick_up_atom_pos(tmptmp,ibuf)==0){
	      atom_.set_R(tmptmp);
	      add_atom(atom_);
	    }
          }
        }
      }
    }
  }
}

int Molecule::get_iatom_dist_in_atom( const double     dist     ,
                                      const int        na       ,
                                      const int      * list_in  ,
                                      int            * list_out ) const 
{
  Dvector3D r1,r2;
  double x1,y1,z1,x2,y2,z2,d;
  int n=0;
  int * chk = new int [natom+1];
  for(int i=0;i<natom;i++){
    chk[i]=0;
  }
  for(int i=0;i<na;i++){
    chk[list_in[i]]=1;
  }
  for(int i=0;i<na;i++){
    r1=atom[list_in[i]].get_R();
    x1=r1.get_x();
    y1=r1.get_y();
    z1=r1.get_z();
    for(int j=0;j<natom;j++){
      if(chk[j]==0){
        r2=atom[j].get_R();
        x2=r2.get_x();
        y2=r2.get_y();
        z2=r2.get_z();
        if(fabs(x1-x2)<dist && fabs(y1-y2)<dist && fabs(z1-z2)<dist){
          d=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
          if(d<dist){
            list_out[n]=j;
            chk[j]=1;
            n++;
          }
        }
      } 
    }
  }
  return n;
}

void Molecule::add_atom ( const int     n             ,
                          const int*    an            ,
                          const double* x_            ,
                          const double* y_            ,
                          const double* z_            ,
                          const int     buff_increase )
{
  clear_fragment();
  clear_biounit();
  clear_linkgroup(); 
  //clear_bond();
  for(int i=0;i<n;i++){
    add_atom(an[i],x_[i],y_[i],z_[i],buff_increase);
  }
}

void Molecule::delete_atom ( const int nlist     ,
                             const int * atom_list )
{
  clear_fragment();
  clear_biounit();
  clear_linkgroup(); 
  //clear_bond(); 20130803
  int imax,itmp,nl_buf,chk,nb,ia1,ia2;
  string type;
  int * list_buf = new int [natom+1];
  int * change = new int [natom+1];
  Bond * b = new Bond [nbond+1];

  nl_buf=0;
  for(int i=0;i<nlist;i++){
    chk=0;
    for(int j=0;j<nl_buf;j++){
      if(list_buf[j]==atom_list[i]){
        chk=1;
        break;
      }
    }
    if(chk==0){
      //list_buf[i]=atom_list[i]; ishi 20160226
      list_buf[nl_buf]=atom_list[i];
      nl_buf++;
    }
  }
  for(int i=0;i<(nl_buf-1);i++){
    imax=i;
    for(int j=(i+1);j<nl_buf;j++){
      if(list_buf[j]>list_buf[imax]) imax=j;
    }
    itmp=list_buf[i];
    list_buf[i]=list_buf[imax];
    list_buf[imax]=itmp;
  }

  cout << "nl_buf = " << nl_buf <<endl ;
  for(int i=0;i<nl_buf;i++){
    cout << list_buf[i] << endl ;
  }

  // 20130803
  for(int i=0;i<natom;i++){
    change[i]=i;
  }
  for(int i=0;i<nl_buf;i++){
    change[list_buf[i]]=-1;
    for(int j=(list_buf[i]+1);j<natom;j++){
      change[j]--;
    }
  }

  nb=nbond;
  for(int i=0;i<nb;i++){
    b[i].copy(bond[i]);
  }

  clear_bond();
  
  for(int i=0;i<nl_buf;i++){
    pick_up_atom(list_buf[i]);
  }

  // 20130803
  for(int i=0;i<nb;i++){
    ia1=b[i].get_iatom1();
    ia2=b[i].get_iatom2();
    if(!(change[ia1]<0 || change[ia2]<0)){
      ia1=change[ia1];
      ia2=change[ia2];
      type=b[i].get_type_mol2();
      add_bond(ia1,ia2,type,nb);
      // copy
      bond[nbond-1].set_chk_view(b[i].get_chk_view());
      bond[nbond-1].copy(b[i].get_vb());
    }
  }

  delete [] list_buf;
  delete [] change;
  delete [] b;
}

int Molecule::get_natom() const
{
  return natom;
}

int Molecule::get_natom_atomic(const int atomic,
                               const int ierr) const
{
  if(atomic<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_natom_atomic , atomic < 0" << endl ;
      cerr << "atomic = " << atomic << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int count=0;
  for(int i=0;i<natom;i++){
    if(atom[i].get_atomic_number()==atomic) {
      count++;
    }
  }
  return count;
}

Atom Molecule::get_atom(const int iatom,
                        const int ierr) const
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "erorr in Molecule::get_atom : iatom < 0 " << endl;
      cerr << "iatom = " << iatom << " , this = " << this << endl;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "erorr in Molecule::get_atom : iatom >= natom " << endl;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this =" << this << endl;
    }
    if(ierr==0) exit(1);
  }
  return atom[iatom];
}

void Molecule::change_atom_order(const int iatom1, 
                                 const int iatom2,
                                 const int ierr)
{
  if(iatom1>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::change_atom_order , iatom1 >= natom " << endl ;
      cerr << "iatom1 = " << iatom1 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::change_atom_order , iatom2 >= natom " << endl ;
      cerr << "iatom2 = " << iatom2 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  clear_fragment();
  clear_biounit();
  clear_linkgroup(); 
  clear_bond();
  Atom tmp;
  tmp.copy(atom[iatom1]);
  atom[iatom1].copy(atom[iatom2]);
  atom[iatom2].copy(tmp);
}

void Molecule::change_atom_order_up(const int iatom,
                                    const int ierr)
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::change_atom_order_up , iatom < 0" << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::charge_atom_order_up , iatom >= natom" << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  //clear_fragment();
  //clear_biounit();
  //clear_linkgroup(); 
  //clear_bond();
  if(iatom==0){
    return;
  } else {
    change_atom_order(iatom, iatom-1);
  }
}

void Molecule::change_atom_order_down(const int iatom,
                                      const int ierr)
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::change_atom_order_down , iatom < 0" << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::charge_atom_order_down , iatom >= natom" << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  //clear_fragment();
  //clear_biounit();
  //clear_linkgroup(); 
  //clear_bond();
  if(iatom==natom-1){
    return;
  } else {
    change_atom_order(iatom, iatom+1);
  }
}

double Molecule::get_atom_distance(const int iatom,
                                   const int jatom,
                                   const int ierr) const
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_atom_distance (1) : iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1); 
  }
  if(jatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_atom_distance (1) : jatom < 0 " << endl ;
      cerr << "jatom = "  << jatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_atom_distance (1) : iatom >= natom" << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_atom_distance (1) : jatom >= natom" << endl ;
      cerr << "jatom = " << jatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return Dvector3D::distance1(atom[iatom].get_R(),atom[jatom].get_R());
}

double Molecule::get_atom_distance(const int        iatom,
                                   const Dvector3D& r,
                                   const int        ierr) const
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_atom_distance (2) : iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_atom_distance (2) : iatom >= natom" << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return Dvector3D::distance1(r,atom[iatom].get_R());
}

void Molecule::rotate_atom ( const int         iatom    ,
                             const Dvector3D&  axis     ,
                             const double      angle    ,
                             const Dvector3D&  r        ,
                             const int         ierr     )
{
  //// angle ---> radian
  //// r ---> center of the rotaton
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::rotate_atom , iatom < 0 " << endl;
      cerr << "iatom = " << iatom << " , this = " << this << endl;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::rotate_atom , iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  Dvector3D r_atom,r_atom_new;
  r_atom = atom[iatom].get_R()-r;
  r_atom_new = Dvector3D::rotation(r_atom,angle,axis);
  atom[iatom].set_R(r+r_atom_new);
}

int Molecule::get_alpha_carbon(int* out)
{
  int n=0;
  for(int i=0;i<natom;i++){
    if(atom[i].get_bio1()==0){
      out[n]=i;
      n++;
    }
  }
  return n;
}

void Molecule::set_atom_coordinate(const int    iatom,
				   const double x    ,
				   const double y    ,
				   const double z    ,
                                   const int    ierr )
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in set_atom_coordinate (1) , iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in set_atom_coordinate (1) , iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  Dvector3D r(x,y,z);
  atom[iatom].set_R(r);
}

void Molecule::set_atom_atomic_number(const int iatom,
                                      const int atomic,
                                      const int ierr)
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in set_atom_atomic , iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in set_atom_atomic , iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  atom[iatom].set_atomic_number(atomic);
}


void Molecule::set_atom_coordinate(const int        iatom,
				   const Dvector3D& r    ,
                                   const int        ierr )
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in set_atom_coordinate (2) , iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in set_atom_coordinate (2) , iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  atom[iatom].set_R(r);
}

void Molecule::set_atom_velocity( const int        iatom ,
				  const double     x     ,
				  const double     y     ,
				  const double     z     ,
                                  const int        ierr  )
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in set_atom_velocity (1) , iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in set_atom_velocity (1) , iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  Dvector3D r(x,y,z);
  atom[iatom].set_V(r);
}

void Molecule::set_atom_velocity( const int        iatom ,
				  const Dvector3D& r     ,
                                  const int        ierr  )
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in set_atom_velocity (2) , iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in set_atom_velocity (2) , iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  atom[iatom].set_V(r);
}

void Molecule::add_atom_velocity( const int        iatom ,
				  const double     x     ,
				  const double     y     ,
				  const double     z     ,
                                  const int        ierr  )
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in add_atom_velocity (1) , iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in add_atom_velocity (1) , iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  Dvector3D r(x,y,z);
  atom[iatom].add_V(r);
}

void Molecule::add_atom_velocity( const int        iatom ,
				  const Dvector3D& r     ,
                                  const int        ierr  )
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in add_atom_velocity (2) , iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in add_atom_velocity (2) , iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  atom[iatom].add_V(r);
}

void Molecule::view_atom_set_color_atomic(const int iatom,
                                          const int ierr)
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_atom_set_color_atomic , iatom < 0" << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_atom_set_color_atomic , iatom >= natom" << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  ViewAtom va(atom[iatom].get_va());
  va.set_color(atom[iatom].get_atomic_number());
  copy(va,iatom);
}

void Molecule::view_atom_set_color_atomic()
{
  for(int i=0;i<natom;i++){
    view_atom_set_color_atomic(i);
  }
}

void Molecule::view_atom_set_color( const int    na   ,
                                    const int*   list ,
                                    const double R    ,
                                    const double G    ,
                                    const double B    ,
                                    const int    ierr )
{
  for(int i=0;i<na;i++){
    atom[list[i]].set_color(R,G,B);
  }
}

void Molecule::view_atom_set_color( const int    iatom,
                                    const double R    ,
                                    const double G    ,
                                    const double B    )
{
  atom[iatom].set_color(R,G,B);
}

void Molecule::view_atom_set_color( const int  na   ,
                                    const int* list ,
                                    const int  ierr )
{
  for(int i=0;i<na;i++){
    atom[list[i]].set_color();
  }
}

void Molecule::view_atom_set_color( const int       na   ,
                                    const int*      list ,
                                    const AtomData& ad   ,
                                    const int       ierr )
{
  for(int i=0;i<na;i++){
    atom[list[i]].set_color(ad);
  }
}

void Molecule::view_atom_set_text_color( const int    na,
                                         const int*   list,
                                         const double R,
                                         const double G,
                                         const double B,
                                         const int    ierr)
{
  for(int i=0;i<na;i++){
    atom[list[i]].set_text_color(R,G,B);
  }
}

void Molecule::view_atom_set_radius(const int iatom,
				    const int r,
				    const int ierr)
{
  if(iatom<0){
    if(ierr==0 && ierr==1){
      cerr << "error in Molecule::view_atom_set_radius , iatom < 0" << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_atom_set_radius , iatom >= natom" << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  ViewAtom va(atom[iatom].get_va());
  va.set_radius(r);
  copy(va,iatom);
}

void Molecule::view_atom_set_radius(const int r)
{
  for(int i=0;i<natom;i++){
    view_atom_set_radius(i,r);
  }
}

void Molecule::view_atom_set_radius(const int    n,
				    const int*   list_in,
				    const int    r,
				    const int    ierr)
{
  for(int i=0;i<n;i++){
    view_atom_set_radius(list_in[i],r,ierr);
  }
}

void Molecule::view_atom_set_radius_lnk(const int    n,
					const int*   list_in,
					const int    r,
					const int    ierr)
{
  int na;
  int* atm;
  atm = new int [natom+1];
  na=get_atom_index_linkgroup(n,list_in,atm,ierr);
  for(int j=0;j<na;j++){
    view_atom_set_radius(atm[j],r);
  }
  delete [] atm ;
}

void Molecule::view_atom_set_radius_bio(const int    n,
					const int*   list_in,
					const int    r,
					const int    ierr)
{
  int na;
  int* atm;
  atm = new int [natom+1];
  na=get_atom_index_biounit(n,list_in,atm,ierr);
  for(int j=0;j<na;j++){
    view_atom_set_radius(atm[j],r);
  }
  delete [] atm ;
}

void Molecule::view_atom_set_radius_frg(const int    n,
					const int*   list_in,
					const int    r,
					const int    ierr)
{
  int na;
  int* atm;
  atm = new int [natom+1];
  na=get_atom_index_fragment(n,list_in,atm,ierr);
  for(int j=0;j<na;j++){
    view_atom_set_radius(atm[j],r);
  }
  delete [] atm ;
}



void Molecule::view_atom_set_bond_w(const int iatom,
				    const int w,
				    const int ierr)
{
  if(iatom<0){
    if(ierr==0 && ierr==1){
      cerr << "error in Molecule::view_atom_set_bond_w , iatom < 0" << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_atom_set_bond_w , iatom >= natom" << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  ViewAtom va(atom[iatom].get_va());
  va.set_bond_w(w);
  copy(va,iatom);
}

void Molecule::view_atom_set_bond_w(const int w)
{
  for(int i=0;i<natom;i++){
    view_atom_set_bond_w(i,w);
  }
}

void Molecule::view_atom_set_bond_w(const int    n,
				    const int*   list_in,
				    const int    w,
				    const int    ierr)
{
  for(int i=0;i<n;i++){
    view_atom_set_bond_w(list_in[i],w,ierr);
  }
}

void Molecule::view_atom_set_bond_w_lnk(const int    n,
					const int*   list_in,
					const int    w,
					const int    ierr)
{
  int na;
  int* atm;
  atm = new int [natom+1];
  na=get_atom_index_linkgroup(n,list_in,atm,ierr);
  for(int j=0;j<na;j++){
    view_atom_set_bond_w(atm[j],w);
  }
  delete [] atm ;
}

void Molecule::view_atom_set_bond_w_bio(const int    n,
					const int*   list_in,
					const int    w,
					const int    ierr)
{
  int na;
  int* atm;
  atm = new int [natom+1];
  na=get_atom_index_biounit(n,list_in,atm,ierr);
  for(int j=0;j<na;j++){
    view_atom_set_bond_w(atm[j],w);
  }
  delete [] atm ;
}

void Molecule::view_atom_set_bond_w_frg(const int    n,
					const int*   list_in,
					const int    w,
					const int    ierr)
{
  int na;
  int* atm;
  atm = new int [natom+1];
  na=get_atom_index_fragment(n,list_in,atm,ierr);
  for(int j=0;j<na;j++){
    view_atom_set_bond_w(atm[j],w);
  }
  delete [] atm ;
}

void Molecule::view_atom_set_radius_alpha(const int r)
{
  int * list = new int [natom+1];
  int n=get_alpha_carbon(list);
  for(int i=0;i<n;i++){
    view_atom_set_radius(list[i],r);
  }
  delete [] list;
}

void Molecule::view_atom_set_type(const int iatom,
				  const int t,
				  const int ierr)
{
  if(iatom<0){
    if(ierr==0 && ierr==1){
      cerr << "error in Molecule::view_atom_set_type , iatom < 0" << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_atom_set_view_type , iatom >= natom" << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  ViewAtom va(atom[iatom].get_va());
  va.set_type(t);
  copy(va,iatom);
}

void Molecule::view_atom_set_type(const int t)
{
  for(int i=0;i<natom;i++){
    view_atom_set_type(i,t);
  }
}

void Molecule::view_atom_set_type(const int    n,
				  const int*   list_in,
				  const int    t,
				  const int    ierr)
{
  for(int i=0;i<n;i++){
    view_atom_set_type(list_in[i],t,ierr);
  }
}

void Molecule::view_atom_set_type_lnk(const int    n,
				      const int*   list_in,
				      const int    t,
				      const int    ierr)
{
  int na;
  int* atm;
  atm = new int [natom+1];
  na=get_atom_index_linkgroup(n,list_in,atm,ierr);
  for(int j=0;j<na;j++){
    view_atom_set_type(atm[j],t);
  }
  delete [] atm ;
}

void Molecule::view_atom_set_type_bio(const int  n,
				      const int* list_in,
				      const int  t,
				      const int  ierr)
{
  int na;
  int* atm;
  atm = new int [natom+1];
  na=get_atom_index_biounit(n,list_in,atm,ierr);
  for(int j=0;j<na;j++){
    view_atom_set_type(atm[j],t);
  }
  delete [] atm ;
}

void Molecule::view_atom_set_type_frg(const int    n,
				      const int*   list_in,
				      const int    t,
				      const int    ierr)
{
  int na;
  int* atm;
  atm = new int [natom+1];
  na=get_atom_index_fragment(n,list_in,atm,ierr);
  for(int j=0;j<na;j++){
    view_atom_set_type(atm[j],t);
  }
  delete [] atm ;
}

void Molecule::set_atom_basis(const int     nlist,
                              const int*    atom_list,
                              const string& basis,
                              const int     ierr)
{
  for(int i=0;i<nlist;i++){
    if(atom_list[i]<0){
      if(ierr==0 && ierr==1){
        cerr << "error in Molecule::set_atom_basis , atom_list[i] < 0" << endl ;
        cerr << "atom_list[i] = " << atom_list[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    } else if(atom_list[i]>=natom){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::set_atom_basis , atom_list[i] >= natom" << endl ;
        cerr << "atom_list[i] = " << atom_list[i] << " , natom = " << natom << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    } else {
      atom[atom_list[i]].set_basis(basis);
    }
  }
}

void Molecule::set_atom_opt(const int  nlist,
                            const int* atom_list,
                            const int  ierr)
{
  for(int i=0;i<nlist;i++){
    if(atom_list[i]<0){
      if(ierr==0 && ierr==1){
        cerr << "error in Molecule::set_atom_opt , atom_list[i] < 0" << endl ;
        cerr << "atom_list[i] = " << atom_list[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    } else if(atom_list[i]>=natom){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::set_atom_opt , atom_list[i] >= natom" << endl ;
        cerr << "atom_list[i] = " << atom_list[i] << " , natom = " << natom << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    } else {
      atom[atom_list[i]].set_opt(1);
      atom[atom_list[i]].set_md (1);
    }
  }
}
 
void Molecule::set_atom_fix(const int  nlist,
                            const int* atom_list,
                            const int ierr)
{
  for(int i=0;i<nlist;i++){
    if(atom_list[i]<0){
      if(ierr==0 && ierr==1){
        cerr << "error in Molecule::set_atom_fix , atom_list[i] < 0" << endl ;
        cerr << "atom_list[i] = " << atom_list[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    } else if(atom_list[i]>=natom){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::set_atom_fix , atom_list[i] >= natom" << endl ;
        cerr << "atom_list[i] = " << atom_list[i] << " , natom = " << natom << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    } else {
      atom[atom_list[i]].set_opt(0);
      atom[atom_list[i]].set_md (0);
    }
  }
}

void Molecule::set_atom_not_ca(const int iatom,
			       const int n_,
			       const int ierr)
{
  atom[iatom].set_not_ca(n_);
}

void Molecule::set_atom_type_bio1_clear()
{
  for(int i=0;i<natom;i++){
    atom[i].set_bio1(-1);
  }
}

void Molecule::set_atom_type_bio2_clear()
{
  for(int i=0;i<natom;i++){
    atom[i].set_bio2(-1);
  }
}

void Molecule::view_atom_set_text_on()
{
  ViewAtom va;
  for(int i=0;i<natom;i++){
    va.copy(atom[i].get_va());
    va.set_text_chk(1);
    atom[i].set_va(va);
  }
}

void Molecule::view_atom_set_text_on(const int  n,
                                     const int* list,
                                     const int  ierr)
{
  ViewAtom va;
  for(int i=0;i<n;i++){
    va.copy(atom[list[i]].get_va());
    va.set_text_chk(1);
    atom[list[i]].set_va(va);
  }
}

void Molecule::view_atom_set_text_off()
{
  ViewAtom va;
  for(int i=0;i<natom;i++){
    va.copy(atom[i].get_va());
    va.set_text_chk(0);
    atom[i].set_va(va);
  }
}

void Molecule::view_atom_set_text_off(const int  n,
                                      const int* list,
                                      const int  ierr)
{
  ViewAtom va;
  for(int i=0;i<n;i++){
    va.copy(atom[list[i]].get_va());
    va.set_text_chk(0);
    atom[list[i]].set_va(va);
  }
}

void Molecule::view_atom_set_text_on_alpha_carbon()
{
  ViewAtom va;
  int type_bio1;
  for(int i=0;i<natom;i++){
    type_bio1=atom[i].get_bio1();
    if(0==type_bio1){
      va.copy(atom[i].get_va());
      va.set_text_chk(1);
      atom[i].set_va(va);
    }
  }
}

void Molecule::view_atom_set_text_on_main_chain()
{
  ViewAtom va;
  int type_bio1;
  for(int i=0;i<natom;i++){
    type_bio1=atom[i].get_bio1();
    if(0<=type_bio1 && type_bio1<=10){
      va.copy(atom[i].get_va());
      va.set_text_chk(1);
      atom[i].set_va(va);
    }
  }
}

void Molecule::view_atom_set_text_on_side_chain()
{
  ViewAtom va;
  int type_bio1;
  for(int i=0;i<natom;i++){
    type_bio1=atom[i].get_bio1();
    if((100<=type_bio1 && type_bio1<=2100) || type_bio1==-999){
      va.copy(atom[i].get_va());
      va.set_text_chk(1);
      atom[i].set_va(va);
    }
  }
}

int Molecule::get_sp3_pos(const int    iatom_ ,
                          const double d_     ,
                          Dvector3D&   r1     ,
                          Dvector3D&   r2     ,
                          Dvector3D&   r3     ,
                          Dvector3D&   r4     ,
                          const int    ierr   ) const 
{
  if(iatom_<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_sp3_pos , iatom_ < 0 " << endl ;
      cerr << "iatom_ = " << iatom_ << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom_>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_sp3_pos , iatom_ >= natom " << endl ;
      cerr << "iatom_ = " << iatom_ << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int nb=0;
  int a1,a2;
  int ba[128];
  Dvector3D br[128];
  for(int i=0;i<nbond;i++){
    a1=bond[i].get_iatom1();
    a2=bond[i].get_iatom2();
    if(a1==iatom_){
      ba[nb]=a2;
      br[nb]=atom[a2].get_R(); 
      nb++;
    }
    if(a2==iatom_){
      ba[nb]=a1;
      br[nb]=atom[a1].get_R(); 
      nb++;
    }
  }
  if(nb==0){
    return 0;
    br[0].set_x(atom[iatom_].get_R().get_x());
    br[0].set_y(atom[iatom_].get_R().get_y());
    br[0].set_z(atom[iatom_].get_R().get_z());
  }
  if(nb<=1){
    double rot_x,rot_y,rot_z;
    Dvector3D v0(atom[iatom_].get_R());
    Dvector3D v1(atom[ba[0]].get_R());
    Dvector3D v01(v1-v0);
    if(-0.001 < v01.get_x() && v01.get_x() <0.001){
      if(-0.001 < v01.get_y() && v01.get_y() < 0.001) {
        // v01_x --> 0.0 , v01_y --> 0.0
        rot_x =  1.0;
        rot_y =  1.0;
        rot_z = -1.0*(v01.get_x()*rot_x+v01.get_y()*rot_y)/v01.get_z();
      } else {
        // v01_x --> 0.0
        rot_x =  1.0;
        rot_z =  1.0; 
        rot_y = -1.0*(v01.get_x()*rot_x+v01.get_z()*rot_z)/v01.get_y();
      }
    } else {
      // v01_x --> not 0.0
      rot_y =  1.0;
      rot_z =  1.0; 
      rot_x = -1.0*(v01.get_y()*rot_y+v01.get_z()*rot_z)/v01.get_x();
    }
    Dvector3D rot;
    rot.set_x(rot_x);
    rot.set_y(rot_y);
    rot.set_z(rot_z);

    //cout << rot_x << endl ;
    //cout << rot_y << endl ;
    //cout << rot_z << endl ;
    //cout << endl ;
    //cout << rot_x << endl ;
    //cout << rot_y << endl ;
    //cout << rot_z << endl ;
    
    Dvector3D v02;
    //v02.copy(Dvector3D::rotation_x(v01,+108*(3.141592/180)));
    v02.copy(Dvector3D::rotation(v01,+108*(3.141592/180),rot));
    br[1].copy(v0+((v02/Dvector3D::norm(v02)))*d_);
    // r1.copy(v0+((v02/Dvector3D::norm(v02)))*d_);
  }
  if(nb<=2){
    Dvector3D v0(atom[iatom_].get_R());
    Dvector3D v1(atom[ba[0]].get_R());
    Dvector3D v2(atom[ba[1]].get_R());
    Dvector3D v01(v1-v0);
    Dvector3D v02(v2-v0);
    Dvector3D v12(v01+v02);
    v01.copy(v01*(-1.0));
    v02.copy(v02*(-1.0));
    v01.copy(Dvector3D::rotation(v01,+90*(3.141592/180.0),v12));
    v02.copy(Dvector3D::rotation(v01,-90*(3.141592/180.0),v12));
    br[2].copy(v0+((v01/Dvector3D::norm(v01)))*d_);
    //r1.copy(v0+((v01/Dvector3D::norm(v01)))*d_);
  }
  if(nb<=3){
    Dvector3D v0(atom[iatom_].get_R());
    Dvector3D v1(atom[ba[0]].get_R());
    Dvector3D v2(atom[ba[1]].get_R());
    Dvector3D v3(atom[ba[2]].get_R());
    Dvector3D v01(v1-v0);
    Dvector3D v02(v2-v0);
    Dvector3D v03(v3-v0);
    Dvector3D v123(v01+(v02+v03));
    br[3].copy(v0-((v123/Dvector3D::norm(v123)))*d_);
    //r1.copy(v0-((v123/Dvector3D::norm(v123)))*d_);
  }
  if (nb==0) { r1.copy(br[0]); r2.copy(br[1]); r3.copy(br[2]); r4.copy(br[3]); return 4; }
  if (nb==1) { r1.copy(br[1]); r2.copy(br[2]); r3.copy(br[3]);                 return 3; }
  if (nb==2) { r1.copy(br[2]); r2.copy(br[3]);                                 return 2; }
  if (nb==3) { r1.copy(br[3]);                                                 return 1; }
  return 0;
}

int Molecule::get_sp2_pos(const int  iatom_,
                          const double d_  ,
                          Dvector3D& r1    ,
                          Dvector3D& r2    ,
                          Dvector3D& r3    ,
                          const int  ierr  ) const
{
  if(iatom_<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_sp2_pos , iatom_ < 0 " << endl ;
      cerr << "iatom_ = " << iatom_ << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom_>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_sp2_pos , iatom_ >= natom " << endl ;
      cerr << "iatom_ = " << iatom_ << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int a1,a2;
  int nb=0;
  int ba[128];
  for(int i=0;i<nbond;i++){
    a1=bond[i].get_iatom1();
    a2=bond[i].get_iatom2();
    if(a1==iatom_){
      ba[nb]=a2;
      nb++;
    }
    if(a2==iatom_){
      ba[nb]=a1;
      nb++;
    }
  }
  if(nb==0){
    return 0;
  }
  if(nb==1){
    int nbb=0;
    int bba[128];
    for(int i=0;i<nbond;i++){
      a1=bond[i].get_iatom1();
      a2=bond[i].get_iatom2();
      if(a1==ba[0] && a2!=iatom_){
        bba[nbb]=a2;
        nbb++;
      }
      if(a2==ba[0] && a1!=iatom_){
        bba[nbb]=a1;
        nbb++;
      }
    }
    if(nbb>0){
      Dvector3D v0(atom[iatom_].get_R());
      Dvector3D v1(atom[ba[0]].get_R());
      Dvector3D v2(atom[bba[0]].get_R());
      Dvector3D v01(v1-v0);
      Dvector3D v02(v2-v0);
      Dvector3D n12(Dvector3D::Vproduct(v01,v02));
      v01.copy(Dvector3D::rotation(v01,120*(3.141592/180.0),n12));
      r1.copy(v0+((v01/Dvector3D::norm(v01)))*d_);
      v01.copy(Dvector3D::rotation(v01,120*(3.141592/180.0),n12));
      r2.copy(v0+((v01/Dvector3D::norm(v01)))*d_);
      return 2;
    } else {
      return 0;
    }
  }
  if(nb==2){
    Dvector3D v0(atom[iatom_].get_R());
    Dvector3D v1(atom[ba[0]].get_R());
    Dvector3D v2(atom[ba[1]].get_R());
    Dvector3D v01(v1-v0);
    Dvector3D v02(v2-v0);
    Dvector3D v12(v01+v02);
    r1.copy(v0-((v12/Dvector3D::norm(v12)))*d_);
    return 1;
  }
  return 0;
}

int Molecule::get_sp1_pos(const int  iatom_,
                          const double d_  ,
                          Dvector3D& r1    ,
                          Dvector3D& r2    ,
                          const int  ierr  ) const
{
  if(iatom_<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_sp1_pos , iatom_ < 0 " << endl ;
      cerr << "iatom_ = " << iatom_ << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom_>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_sp1_pos , iatom_ >= natom " << endl ;
      cerr << "iatom_ = " << iatom_ << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int nb=0;
  int a1,a2;
  int ba[128];
  for(int i=0;i<nbond;i++){
    a1=bond[i].get_iatom1();
    a2=bond[i].get_iatom2();
    if(a1==iatom_){
      ba[nb]=a2;
      nb++;
    }
    if(a2==iatom_){
      ba[nb]=a1;
      nb++;
    }
  }
  if(nb==0){
    return 0;
  }
  if(nb==1){
    return 0;
  }
  return 0;
}

void Molecule::translate_atom_to_atom( const int    iatom,
                                       const int    jatom,
                                       const double dist ,
                                       const int    ierr )
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::move_atom_to_atom, iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::move_atom_to_atom, iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::move_atom_to_atom, jatom < 0 " << endl ;
      cerr << "jatom = " << jatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::move_atom_to_atom, jatom >= natom " << endl ;
      cerr << "jatom = " << jatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  // iatom ---> jatom
  Dvector3D ri(atom[iatom].get_R());
  Dvector3D rj(atom[jatom].get_R());
  Dvector3D rij(rj-ri);
  Dvector3D to((rij/Dvector3D::norm(rij))*dist);
  atom[iatom].set_R(ri+to);
}

void Molecule::translate_atom( const int        iatom  ,
                               const Dvector3D  r      ,
                               const int        ierr   )
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::move_atom, iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::move_atom, iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  Dvector3D ri(atom[iatom].get_R());
  atom[iatom].set_R(ri+r);
}

int Molecule::pick_up_atom_pos( const Dvector3D& r      ,
				int*             iatom_ ,
				const double     th     ) const 
{
  int n=0;
  double x ,y ,z ;
  double x_,y_,z_;
  Dvector3D r_;
  y_=r.get_y();
  z_=r.get_z();
  for(int i=0;i<natom;i++){
    r_.copy(atom[i].get_R());
    x =r .get_x();
    x_=r_.get_x();
    if(-th<(x-x_) && (x-x_)<th){
      y =r .get_y();
      y_=r_.get_y();
      if(-th<(y-y_) && (y-y_)<th){
	z =r .get_z();
	z_=r_.get_z();
	if(-th<(z-z_) && (z-z_)<th){
	  iatom_[n]=i;
	  n++;
	}
      }
    }
  }
  return n;
}

void Molecule::atom_gl_point( const int        iatom_            ,
                              float            *tx               ,
                              float            *ty               ,
                              float            *tz               ,
                              float            *cr               ,
                              float            *cg               ,
                              float            *cb               ,
                              float            *rd               ,
                              const int        ierr              ) const
{
  ViewAtom  va   = atom[iatom_].get_va();
  Dvector3D r    = atom[iatom_].get_R();
  *tx            = (float)r.get_x();
  *ty            = (float)r.get_y();
  *tz            = (float)r.get_z();
  *cr            = (float)va.get_color_R();
  *cg            = (float)va.get_color_G();
  *cb            = (float)va.get_color_B();
  *rd            = (float)va.get_radius();
}

void Molecule::atom_gl_sphere( const int        iatom_            ,
                               float            *tx               ,
                               float            *ty               ,
                               float            *tz               ,
                               float            *cr               ,
                               float            *cg               ,
                               float            *cb               ,
                               float            *rd               ,
                               const int        ierr              ) const
{
  ViewAtom  va   = atom[iatom_].get_va();
  Dvector3D r    = atom[iatom_].get_R();
  float size = (float)va.get_radius ();
  //float     rad  = (float)atom[iatom_].get_radius();
  float rad;
  if(atom[iatom_].get_atomic_number()<3){
    rad=1.0;
  } else {
    rad=2.0;
  }

  *cr            = (float)va.get_color_R();
  *cg            = (float)va.get_color_G();
  *cb            = (float)va.get_color_B();
  *tx            = (float)r.get_x();
  *ty            = (float)r.get_y();
  *tz            = (float)r.get_z();
  *rd            = (float)rad*(2.0+size)*0.1;
}

void Molecule::set_atom_amber_force_field()
{

}

void Molecule::delete_atom_not_autodock()
{
  int * list = new int [natom+1];
  int n=0;
  for(int i=(natom-1);i>=0;i--){
    int itype=atom[i].get_bio1();
    if(ForceField::autodock_get_atom_type(itype)=="**"){
      list[n]=i;
      n++;
    }
  }
  delete_atom(n,list);
  delete [] list ;
}
