
#include <fstream>
#include <cstdlib>
#include <cstdio>
#include <math.h>
#include "Molecule.h"
#include "Bond.h"
#include "ViewBond.h"
#include "iString.h"
#include "ForceField.h"

void Molecule::copy(const ViewBond& vb   ,
                    const int       ibond,
                    const int       ierr )
{
  if(ibond<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::copy(const ViewBond&, const int) , ibond < 0" << endl ;
      cerr << "ibond = " << ibond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  } 
  if(ibond>=nbond){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::copy(const ViewBond&, const int) , ibond >= nbond" << endl ;
      cerr << "ibond = " << ibond << " , nbond = " << nbond << " , this = " << this << endl;
    }
    if(ierr==0) exit(1);
  }
  bond[ibond].copy(vb);
}

void Molecule::clear_bond()
{
  if(nbond_buff==0){
    return;
  }
  for(int i=(nbond-1);i>=0;i--){
    pick_up_bond(i);
  }
  delete [] bond;
  nbond_buff=0;
}

void Molecule::create_bond(const int n)
{
  clear_bond();
  if(n==0) return;
  bond = new Bond [n];
  nbond_buff=n;
}

void Molecule::resize_bond(const int n)
{
  int   tmp_nbond=nbond;
  Bond* tmp_bond;
  tmp_bond = new Bond [tmp_nbond+1];
  for(int i=0;i<tmp_nbond;i++){
    tmp_bond[i].copy(bond[i]);
  }
  create_bond(n);
  if(n>tmp_nbond){
    for(int i=0;i<tmp_nbond;i++){
      push_back_bond(tmp_bond[i]);
    } 
  } else {
    for(int i=0;i<n;i++){
      push_back_bond(tmp_bond[i]);
    }
  }
  delete [] tmp_bond;
}

// private method //
void Molecule::push_back_bond( const Bond& b,
                               const int   buff_increase)
{
  push_back_bond(b.get_iatom1(),b.get_iatom2(),b.get_type_mol2(),buff_increase);
}

// private method //
void Molecule::push_back_bond( const int iatom1, 
                               const int iatom2,
			       const string type,
                               const int buff_increase,
                               const int ierr)
{
  if(iatom1<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::push_back_bond , iatom1 < 0" << endl ;
      cerr << "iatom1 = " << iatom1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  } 
  if(iatom2<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::push_back_bond , iatom2 < 0" << endl ;
      cerr << "iatom2 = " << iatom2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  } 
  if(iatom1>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::push_back_bond , iatom1 >= natom" << endl ;
      cerr << "iatom1 = " << iatom1 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::push_back_bond , iatom2 >= natom" << endl ;
      cerr << "iatom2 = " << iatom2 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(nbond==nbond_buff){
    resize_bond(nbond+buff_increase);
  }
  bond[nbond].set_iatom1(iatom1);
  bond[nbond].set_iatom2(iatom2);
  bond[nbond].set_type_mol2(type);
  nbond++;

  ViewBond vb(bond[nbond-1].get_vb());
  ViewAtom va1(atom[iatom1].get_va());
  ViewAtom va2(atom[iatom2].get_va());
  int t1=atom[iatom1].get_va().get_type();
  int t2=atom[iatom2].get_va().get_type();
  int w1=atom[iatom1].get_va().get_bond_w();
  int w2=atom[iatom2].get_va().get_bond_w();
  vb.set_dist_ratio_1(atom[iatom1].get_bond_distance());
  vb.set_dist_ratio_2(atom[iatom2].get_bond_distance());
  if(t1==t2){
    vb.set_type(t1);
  }
  if(w1<=w2){
    vb.set_width(w1);
  } else {
    vb.set_width(w2);
  }
  vb.set_color_R1(va1.get_color_R());
  vb.set_color_G1(va1.get_color_G());
  vb.set_color_B1(va1.get_color_B());
  vb.set_color_R2(va2.get_color_R());
  vb.set_color_G2(va2.get_color_G());
  vb.set_color_B2(va2.get_color_B());
  copy(vb,nbond-1);

  atom[iatom1].push_back_bond_atom(iatom2);
  atom[iatom2].push_back_bond_atom(iatom1);
}

// private method //
void Molecule::pick_up_bond(const int ibond,
                            const int ierr )
{
  if(ibond<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::pick_up_bond , ibond < 0" << endl ;
      cerr << "ibond = " << ibond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibond>=nbond){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::pick_up_bond , ibond >= nbond" << endl ;
      cerr << "ibond = " << ibond << " , nbond = " << nbond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int iatom1=bond[ibond].get_iatom1(); 
  int iatom2=bond[ibond].get_iatom2();
  for(int i=ibond;i<nbond-1;i++){
    bond[i].copy(bond[i+1]);
  }
  nbond--;
  atom[iatom1].pick_up_bond_atom(iatom2);
  atom[iatom2].pick_up_bond_atom(iatom1);
}

int Molecule::chk_bond_exist( const int iatom1,
                              const int iatom2,
                              const int ierr) const 
{
  if(iatom1<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::chk_bond_exist , iatom1 < 0" << endl ;
      cerr << "iatom1 = " << iatom1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  } 
  if(iatom2<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::chk_bond_exist , iatom2 < 0" << endl ;
      cerr << "iatom2 = " << iatom2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  } 
  if(iatom1>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::chk_bond_exist , iatom1 >= natom" << endl ;
      cerr << "iatom1 = " << iatom1 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::chk_bond_exist , iatom2 >= natom" << endl ;
      cerr << "iatom2 = " << iatom2 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int ia1,ia2;
  for(int i=0;i<nbond;i++){
    ia1=bond[i].get_iatom1();
    ia2=bond[i].get_iatom2();
    if(ia1==iatom1 && ia2==iatom2) return i;
    if(ia1==iatom2 && ia2==iatom1) return i;
  }
  return -1;
}

void Molecule::add_bond(const int iatom1,
                        const int iatom2,
                        const int buff_increase,
                        const int ierr)
{
  if(iatom1<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_bond (1), iatom1 < 0" << endl ;
      cerr << "iatom1 = " << iatom1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  } 
  if(iatom2<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_bond (1), iatom2 < 0" << endl ;
      cerr << "iatom2 = " << iatom2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  } 
  if(iatom1>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_bond (1), iatom1 >= natom" << endl ;
      cerr << "iatom1 = " << iatom1 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_bond (1), iatom2 >= natom" << endl ;
      cerr << "iatom2 = " << iatom2 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  push_back_bond(iatom1,iatom2,"",buff_increase,ierr);
}

void Molecule::add_bond(const int iatom1,
                        const int iatom2,
			const string type,
                        const int buff_increase,
                        const int ierr)
{
  if(iatom1<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_bond (2), iatom1 < 0" << endl ;
      cerr << "iatom1 = " << iatom1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  } 
  if(iatom2<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_bond (2), iatom2 < 0" << endl ;
      cerr << "iatom2 = " << iatom2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  } 
  if(iatom1>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_bond (2), iatom1 >= natom" << endl ;
      cerr << "iatom1 = " << iatom1 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_bond (2), iatom2 >= natom" << endl ;
      cerr << "iatom2 = " << iatom2 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  push_back_bond(iatom1,iatom2,type,buff_increase,ierr);
}

void Molecule::add_bond_auto(const double ratio)
{
  clear_bond();
  int n;
  double leng1,leng2;
  Dvector3D r;
  int* atomic_num_;
  double* x;
  double* y;
  double* z;
  double* leng;
  n=natom;
  if(n<2) return ;
  x = new double [n] ;
  y = new double [n] ;
  z = new double [n] ;
  leng = new double [n] ;
  atomic_num_ = new int [n] ;
  for(int i=0;i<n;i++){
    r=atom[i].get_R();
    x[i]=r.get_x();
    y[i]=r.get_y();
    z[i]=r.get_z();
    atomic_num_[i]=atom[i].get_atomic_number();
    leng[i]=atom[i].get_bond_distance()*ratio;
  }
  for(int i=0;i<n;i++){
    if(atomic_num_[i]!=0) {
      for(int j=i+1;j<n;j++){
        if(atomic_num_[j]!=0){
          leng2=(leng[i]+leng[j]);
          if(fabs(x[i]-x[j])<leng2 && fabs(y[i]-y[j])<leng2 && fabs(z[i]-z[j])<leng2){
            leng1=sqrt((x[i]-x[j])*(x[i]-x[j])+(y[i]-y[j])*(y[i]-y[j])+(z[i]-z[j])*(z[i]-z[j]));
            if(leng1<leng2 && 0.00001 < leng1){
              push_back_bond(i,j,"",n*2);
            }
          }
        }
      }
    }
  }
  delete [] x;
  delete [] y;
  delete [] z;
  delete [] leng;
  delete [] atomic_num_;
}

void Molecule::add_bond_auto(const int na,
                             const int * range,
                             const double ratio,
                             const int exist_chk,
                             const int ierr)
{
  for(int i=0;i<na;i++){
    if(range[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::add_bond_auto , range[i]<0" << endl ;
        cerr << "range[i] = " << range[i] << " , i = " << i << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(range[i]>=natom){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::add_bond_auto , range[i] >= natom" << endl ;
        cerr << "range[i] = " << range[i] << " , i = " << i << " , natom = " << natom << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  int n;
  double leng1,leng2;
  Dvector3D r;
  int* atomic_num_;
  double* x;
  double* y;
  double* z;
  double* leng;
  n=natom;
  if(n <2) return ;
  if(na<2) return ;
  x           = new double [na] ;
  y           = new double [na] ;
  z           = new double [na] ;
  leng        = new double [na] ;
  atomic_num_ = new int    [na] ;
  for(int i=0;i<na;i++){
    r=atom[range[i]].get_R();
    x[i]=r.get_x();
    y[i]=r.get_y();
    z[i]=r.get_z();
    atomic_num_[i]=atom[range[i]].get_atomic_number();
    leng[i]=atom[range[i]].get_bond_distance()*ratio;
  }
  for(int i=0;i<na;i++){
    if(atomic_num_[i]!=0) {
      for(int j=i+1;j<na;j++){
        if(atomic_num_[j]!=0){
          leng2=(leng[i]+leng[j]);
          if(fabs(x[i]-x[j])<leng2 && fabs(y[i]-y[j])<leng2 && fabs(z[i]-z[j])<leng2){
            leng1=sqrt((x[i]-x[j])*(x[i]-x[j])+(y[i]-y[j])*(y[i]-y[j])+(z[i]-z[j])*(z[i]-z[j]));
            if(leng1<leng2 && 0.00001 < leng1){
              // ishi 20100406
              if(exist_chk==1){
                if(chk_bond_exist(range[i],range[j])==-1){
                  push_back_bond(range[i],range[j],"",n*2);
                }
              } else {
                push_back_bond(range[i],range[j],"",n*2);
              }
            }
          }
        }
      }
    }
  }
  delete [] x;
  delete [] y;
  delete [] z;
  delete [] leng;
  delete [] atomic_num_;
}

void Molecule::add_bond_auto_fragment(const int ifrag,
                                      const double ratio,
                                      const int ierr)
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_bond_auto_fragment : ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::add_bond_auto_fragment , ifrag >= nfragment " << endl ;
      cerr << "ifrag =" << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int * range = new int [natom];
  int n=0;
  for(int i=0;i<fragment[ifrag].get_n_atom_index();i++){
    range[n]=fragment[ifrag].get_atom_index(i);
    n++;
  }
  for(int i=0;i<fragment[ifrag].get_n_link();i++){
    range[n]=fragment[ifrag].get_atom_index_link1(i);
    n++;
  }
  add_bond_auto(n,range,ratio);
  delete [] range;
}

void Molecule::add_bond_auto_fragment(const double ratio)
{
  clear_bond();
  for(int i=0;i<nfragment;i++){
    add_bond_auto_fragment(i,ratio);
  }
}

void Molecule::add_bond_auto_ter_pdb(const string& filename) 
{
  if(natom<2){
    return ;
  }
  ifstream file(filename.c_str(), std::ios::in);
  if(!file.is_open()){
    return ;
  }
  char line[1024];
  int n,nsum;
  int ter_n=0;
  int * ter=new int [natom+1];
  clear_bond () ;
  nsum=0;
  while(!file.eof()){
    n=0;
    while(!file.eof()){
      file.getline(line, sizeof(line));
      if(line[0]=='A' || line[0]=='T' || line[0]=='H'){
        if(iFind(line, "TER"   )>=0){
          break;
        }
        if(iFind(line, "ATOM"  )>=0){
          if(iPickUpNum(line)>4){
            n++;
            nsum++;
          }
        }
        if(iFind(line, "HETATM")>=0){
          n++;
          nsum++;
        }
      }
    }
    ter[ter_n]=n;
    ter_n++;
  }
  file.close();
  if(nsum>natom){
    ter_n=1;
    ter[0]=natom;
  }
  int * ibuf = new int [natom+1];
  int n1=0;
  int n2=0;
  for(int i=0;i<ter_n;i++){
    n2=0;
    for(int j=0;j<ter[i];j++){
      ibuf[j]=n1;
      n1++;
      n2++;
    }
    if(n2>1){
      add_bond_auto(n2,ibuf,1.0,0);
    }
  }
  delete [] ibuf ;
  int i1,i2,j1,j2;
  double dist,leng;
  i1=0;i2=0;
  for(int i=0;i<ter_n;i++){
    i1=i2;i2=i1+ter[i];
    for(int k=i1;k<i2;k++){
      if(atom[k].get_atomic_number()==16){    
        j1=0;j2=0;
        for(int j=0;j<ter_n;j++){
          j1=j2;j2=j1+ter[j];
          if(i<j){
            for(int l=j1;l<j2;l++){
              if(atom[l].get_atomic_number()==16){
                dist=Dvector3D::distance1(atom[k].get_R(),atom[l].get_R());
                leng=atom[k].get_bond_distance()+atom[l].get_bond_distance();
                if(dist<leng){
                  add_bond(k,l);
                }
              }
            }
          }
        }
      }
    }
  }
  delete [] ter;
}

void Molecule::add_bond_auto_from_pdb_tag(string& s_out)
{
  int * list;
  int * check;
  int * notdefine;
  int a1,a2,n,alink1,alink2,atm_type,res_type,anseq1_i,anseq2_i;
  double d;
  string artyp1,anseq1,achin1,aatyp1,artyp2,anseq2,achin2,aatyp2;
  char c[1024];
  clear_bond () ;
  list      = new int [natom+1];
  check     = new int [natom+1];
  notdefine = new int [natom+1];
  for(int i=0;i<natom;i++){
    check[i]=0;
    notdefine[i]=0;
  }
  for(int i=0;i<natom;i++){
    if(check[i]==0){
      n=0;
      alink1=atom[i].get_link_read_pdb();
      artyp1=atom[i].get_rtyp_read_pdb();
      anseq1=atom[i].get_nseq_read_pdb();
      achin1=atom[i].get_chin_read_pdb();
      aatyp1=atom[i].get_atyp_read_pdb();
      check[i]=1;
      atm_type=BioUnit::atm_type(aatyp1,artyp1);
      res_type=BioUnit::res_type(artyp1);
      if(atm_type<0 || res_type<0){
        notdefine[i]=1;
        //if(!(res_type==-1 && aatyp1=="O" && artyp1=="HOH")){
        sprintf(c,"\n   ** warning : unknown pdb-tag : %5s %5s",aatyp1.c_str(),artyp1.c_str());s_out=s_out+c;
        //}
      } 
      list[n]=i;
      n++;
      for(int j=i+1;j<natom;j++){
	alink2=atom[j].get_link_read_pdb();
	artyp2=atom[j].get_rtyp_read_pdb();
	anseq2=atom[j].get_nseq_read_pdb();
	achin2=atom[j].get_chin_read_pdb();
	aatyp2=atom[j].get_atyp_read_pdb();
	if(alink1==alink2 && artyp1==artyp2 && anseq1==anseq2 && achin1==achin2){
	  list[n]=j;
	  n++;
	  check[j]=1;
	  atm_type=BioUnit::atm_type(aatyp1,artyp1);
	  res_type=BioUnit::res_type(artyp1);
	  if(atm_type<0 || res_type<0){
            notdefine[j]=1;
            //if(!(res_type==-1 && aatyp2=="O" && artyp2=="HOH")){
            sprintf(c,"\n   ** warning : unknown pdb-tag : %5s %5s",aatyp2.c_str(),artyp2.c_str());s_out=s_out+c;
            //}
	  }
	}
      }
      add_bond_auto_from_pdb_tag_intra_res(n,list,artyp1);
    }
  }
  for(int i=0;i<natom;i++){
    alink1=atom[i].get_link_read_pdb();
    aatyp1=atom[i].get_atyp_read_pdb();
    if(aatyp1=="C"){

      /*
      // 20140718  
      n=0;
      while(1){
	n++;
	if((i+n)>=natom){ // ishi 20140718
	  break;
	}
	if(alink1!=atom[i+n].get_link_read_pdb()){ // ishi 20140718
	  break;
	}
	if("N"==atom[i+n].get_atyp_read_pdb()){
          add_bond(i,i+n);
          break;
	}
      }
      */
      anseq1_i=atoi(atom[i].get_nseq_read_pdb().c_str());
      for(int j=0;j<natom;j++){
	if(alink1==atom[j].get_link_read_pdb()){
          if("N"==atom[j].get_atyp_read_pdb()){
            anseq2_i=atoi(atom[j].get_nseq_read_pdb().c_str());
            if((anseq2_i-1)==anseq1_i){
              add_bond(i,j);
              break;
            }
          }
        }
      }
      
      
      
    }
  }
  //////////////////
  // make ss-bond //
  //////////////////
  double dist,leng;
  for(int i=0;i<natom;i++){
    if(atom[i].get_atomic_number()==16){    
      for(int k=(i+1);k<natom;k++){
        if(atom[k].get_atomic_number()==16){    
          dist=Dvector3D::distance1(atom[i].get_R(),atom[k].get_R());
          leng=atom[i].get_bond_distance()+atom[k].get_bond_distance();
          if(dist<leng){
            add_bond(i,k);
          }
        }
      }
    }
  }
  ////////////////////////////////////
  // make bond for not defined atom //
  ////////////////////////////////////
  n=0;
  for(int i=0;i<natom;i++){
    if(notdefine[i]==1){
      list[n]=i;
      n++;
    }
  }
  add_bond_auto(n,list);
  for(int i=0;i<nbond;i++){
    a1=bond[i].get_iatom1();
    a2=bond[i].get_iatom2();
    d=Dvector3D::distance1(atom[a1].get_R(),atom[a2].get_R());
    if(d>5.0){
      sprintf(c,"\n   ** warning : bond distance is too large : atom1= %6d , atom2= %6d , dist. = %10.2f A",a1+1,a2+1,d*0.5291);s_out=s_out+c;
    }
  }
  delete [] list;
  delete [] check;
  delete [] notdefine;
}

void Molecule::add_bond_auto_from_pdb_tag_intra_res(const int n,
						    const int * list,
						    const string artyp)
{
  string a1,a2;
  int t1,t2;

  for(int i=0;i<n;i++){

    a1=atom[list[i]].get_atyp_read_pdb();
    t1=BioUnit::atm_type(a1,artyp);

    for(int j=i+1;j<n;j++){

      a2=atom[list[j]].get_atyp_read_pdb();
      t2=BioUnit::atm_type(a2,artyp);

      if(0){}

      ////////////////
      // main chain //
      ////////////////

      else if((t1==0 && t2== 1) || (t2==0 && t1== 1)){add_bond(list[i],list[j]);}
      else if((t1==0 && t2== 2) || (t2==0 && t1== 2)){add_bond(list[i],list[j]);}
      else if((t1==0 && t2== 3) || (t2==0 && t1== 3)){add_bond(list[i],list[j]);}
      else if((t1==1 && t2== 4) || (t2==1 && t1== 4)){add_bond(list[i],list[j]);}
      else if((t1==2 && t2== 5) || (t2==2 && t1== 5)){add_bond(list[i],list[j]);}

      // N-tarminal
      else if((t1==2 && t2==10) || (t2==2 && t1==10)){add_bond(list[i],list[j]);}
      else if((t1==2 && t2==11) || (t2==2 && t1==11)){add_bond(list[i],list[j]);}
      else if((t1==2 && t2==12) || (t2==2 && t1==12)){add_bond(list[i],list[j]);}

      // C-tarminal
      else if((t1==1 && t2== 4) || (t2==1 && t1== 4)){add_bond(list[i],list[j]);}

      else if(artyp=="ACE"){
        if(0){}
        else if((t1==81 && t2==82) || (t2==81 && t1==82)){add_bond(list[i],list[j]);}
        else if((t1==81 && t2==83) || (t2==81 && t1==83)){add_bond(list[i],list[j]);}
        else if((t1==83 && t2==84) || (t2==83 && t1==84)){add_bond(list[i],list[j]);}
        else if((t1==83 && t2==85) || (t2==83 && t1==85)){add_bond(list[i],list[j]);}
        else if((t1==83 && t2==86) || (t2==83 && t1==86)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="NME"){
        if(0){}
        else if((t1==87 && t2==88) || (t2==87 && t1==88)){add_bond(list[i],list[j]);}
        else if((t1==87 && t2==89) || (t2==87 && t1==89)){add_bond(list[i],list[j]);}
        else if((t1==89 && t2==90) || (t2==89 && t1==90)){add_bond(list[i],list[j]);}
        else if((t1==89 && t2==91) || (t2==89 && t1==91)){add_bond(list[i],list[j]);}
        else if((t1==89 && t2==92) || (t2==89 && t1==92)){add_bond(list[i],list[j]);}
      }

      ////////////////
      // side chain //
      ////////////////

      else if(artyp=="GLY"){
	if(0){}
	//else if((a1=="CA"   && a2=="HA2"  ) || (a2=="CA"   && a1=="HA2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CA"   && a2=="HA3"  ) || (a2=="CA"   && a1=="HA3"  )){add_bond(list[i],list[j]);}
	else if((t1==0 && t2==8) || (t2==0 && t1==8)){add_bond(list[i],list[j]);}
	else if((t1==0 && t2==9) || (t2==0 && t1==9)){add_bond(list[i],list[j]);}
      }
      
      else if(artyp=="ALA"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB1"  ) || (a2=="CB"   && a1=="HB1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	else if((t1==0   && t2==205) || (t2==0   && t1==205)){add_bond(list[i],list[j]);}
	else if((t1==205 && t2==206) || (t2==205 && t1==206)){add_bond(list[i],list[j]);}
	else if((t1==205 && t2==207) || (t2==205 && t1==207)){add_bond(list[i],list[j]);}
	else if((t1==205 && t2==208) || (t2==205 && t1==208)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="VAL"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG1"  ) || (a2=="CB"   && a1=="CG1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG2"  ) || (a2=="CB"   && a1=="CG2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB"   ) || (a2=="CB"   && a1=="HB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG1"  && a2=="HG11" ) || (a2=="CG1"  && a1=="HG11" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG1"  && a2=="HG12" ) || (a2=="CG1"  && a1=="HG12" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG1"  && a2=="HG13" ) || (a2=="CG1"  && a1=="HG13" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG2"  && a2=="HG21" ) || (a2=="CG2"  && a1=="HG21" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG2"  && a2=="HG22" ) || (a2=="CG2"  && a1=="HG22" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG2"  && a2=="HG23" ) || (a2=="CG2"  && a1=="HG23" )){add_bond(list[i],list[j]);}
	else if((t1==0   && t2==305) || (t2==0   && t1==305)){add_bond(list[i],list[j]);}
	else if((t1==305 && t2==306) || (t2==305 && t1==306)){add_bond(list[i],list[j]);}
	else if((t1==305 && t2==307) || (t2==305 && t1==307)){add_bond(list[i],list[j]);}
	else if((t1==305 && t2==308) || (t2==305 && t1==308)){add_bond(list[i],list[j]);}
	else if((t1==306 && t2==309) || (t2==306 && t1==309)){add_bond(list[i],list[j]);}
	else if((t1==306 && t2==310) || (t2==306 && t1==310)){add_bond(list[i],list[j]);}
	else if((t1==306 && t2==311) || (t2==306 && t1==311)){add_bond(list[i],list[j]);}
	else if((t1==307 && t2==312) || (t2==307 && t1==312)){add_bond(list[i],list[j]);}
	else if((t1==307 && t2==313) || (t2==307 && t1==313)){add_bond(list[i],list[j]);}
	else if((t1==307 && t2==314) || (t2==307 && t1==314)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="LEU"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD1"  ) || (a2=="CG"   && a1=="CD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD2"  ) || (a2=="CG"   && a1=="CD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG"   ) || (a2=="CG"   && a1=="HG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="HD11" ) || (a2=="CD1"  && a1=="HD11" )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="HD12" ) || (a2=="CD1"  && a1=="HD12" )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="HD13" ) || (a2=="CD1"  && a1=="HD13" )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="HD21" ) || (a2=="CD2"  && a1=="HD21" )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="HD22" ) || (a2=="CD2"  && a1=="HD22" )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="HD23" ) || (a2=="CD2"  && a1=="HD23" )){add_bond(list[i],list[j]);}
	else if((t1==0   && t2==405) || (t2==0   && t1==405)){add_bond(list[i],list[j]);}
	else if((t1==405 && t2==409) || (t2==405 && t1==409)){add_bond(list[i],list[j]);}
	else if((t1==405 && t2==410) || (t2==405 && t1==410)){add_bond(list[i],list[j]);}
	else if((t1==405 && t2==406) || (t2==405 && t1==406)){add_bond(list[i],list[j]);}
	else if((t1==406 && t2==407) || (t2==406 && t1==407)){add_bond(list[i],list[j]);}
	else if((t1==406 && t2==408) || (t2==406 && t1==408)){add_bond(list[i],list[j]);}
	else if((t1==406 && t2==411) || (t2==406 && t1==411)){add_bond(list[i],list[j]);}
	else if((t1==407 && t2==412) || (t2==407 && t1==412)){add_bond(list[i],list[j]);}
	else if((t1==407 && t2==413) || (t2==407 && t1==413)){add_bond(list[i],list[j]);}
	else if((t1==407 && t2==414) || (t2==407 && t1==414)){add_bond(list[i],list[j]);}
	else if((t1==408 && t2==415) || (t2==408 && t1==415)){add_bond(list[i],list[j]);}
	else if((t1==408 && t2==416) || (t2==408 && t1==416)){add_bond(list[i],list[j]);}
	else if((t1==408 && t2==417) || (t2==408 && t1==417)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="ILE"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG1"  ) || (a2=="CB"   && a1=="CG1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG2"  ) || (a2=="CB"   && a1=="CG2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB"   ) || (a2=="CB"   && a1=="HB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG2"  && a2=="HG21" ) || (a2=="CG2"  && a1=="HG21" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG2"  && a2=="HG22" ) || (a2=="CG2"  && a1=="HG22" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG2"  && a2=="HG23" ) || (a2=="CG2"  && a1=="HG23" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG1"  && a2=="HG12" ) || (a2=="CG1"  && a1=="HG12" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG1"  && a2=="HG13" ) || (a2=="CG1"  && a1=="HG13" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG1"  && a2=="CD1"  ) || (a2=="CG1"  && a1=="CD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="HD11" ) || (a2=="CD1"  && a1=="HD11" )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="HD12" ) || (a2=="CD1"  && a1=="HD12" )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="HD13" ) || (a2=="CD1"  && a1=="HD13" )){add_bond(list[i],list[j]);}
	else if((t1==0   && t2==505) || (t2==0   && t1==505)){add_bond(list[i],list[j]);}
	else if((t1==505 && t2==507) || (t2==505 && t1==507)){add_bond(list[i],list[j]);}
	else if((t1==505 && t2==506) || (t2==505 && t1==506)){add_bond(list[i],list[j]);}
	else if((t1==505 && t2==509) || (t2==505 && t1==509)){add_bond(list[i],list[j]);}
	else if((t1==506 && t2==510) || (t2==506 && t1==510)){add_bond(list[i],list[j]);}
	else if((t1==506 && t2==511) || (t2==506 && t1==511)){add_bond(list[i],list[j]);}
	else if((t1==506 && t2==512) || (t2==506 && t1==512)){add_bond(list[i],list[j]);}
	else if((t1==507 && t2==513) || (t2==507 && t1==513)){add_bond(list[i],list[j]);}
	else if((t1==507 && t2==514) || (t2==507 && t1==514)){add_bond(list[i],list[j]);}
	else if((t1==507 && t2==508) || (t2==507 && t1==508)){add_bond(list[i],list[j]);}
	else if((t1==508 && t2==515) || (t2==508 && t1==515)){add_bond(list[i],list[j]);}
	else if((t1==508 && t2==516) || (t2==508 && t1==516)){add_bond(list[i],list[j]);}
	else if((t1==508 && t2==517) || (t2==508 && t1==517)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="MET"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG2"  ) || (a2=="CG"   && a1=="HG2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG3"  ) || (a2=="CG"   && a1=="HG3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="SD"   ) || (a2=="CG"   && a1=="SD"   )){add_bond(list[i],list[j]);}
	//else if((a1=="SD"   && a2=="CE"   ) || (a2=="SD"   && a1=="CE"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CE"   && a2=="HE1"  ) || (a2=="CE"   && a1=="HE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE"   && a2=="HE2"  ) || (a2=="CE"   && a1=="HE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE"   && a2=="HE3"  ) || (a2=="CE"   && a1=="HE3"  )){add_bond(list[i],list[j]);}
	else if((t1==0   && t2==605) || (t2==0   && t1==605)){add_bond(list[i],list[j]);}
	else if((t1==605 && t2==609) || (t2==605 && t1==609)){add_bond(list[i],list[j]);}
	else if((t1==605 && t2==610) || (t2==605 && t1==610)){add_bond(list[i],list[j]);}
	else if((t1==605 && t2==606) || (t2==605 && t1==606)){add_bond(list[i],list[j]);}
	else if((t1==606 && t2==611) || (t2==606 && t1==611)){add_bond(list[i],list[j]);}
	else if((t1==606 && t2==612) || (t2==606 && t1==612)){add_bond(list[i],list[j]);}
	else if((t1==606 && t2==607) || (t2==606 && t1==607)){add_bond(list[i],list[j]);}
	else if((t1==607 && t2==608) || (t2==607 && t1==608)){add_bond(list[i],list[j]);}
	else if((t1==608 && t2==613) || (t2==608 && t1==613)){add_bond(list[i],list[j]);}
	else if((t1==608 && t2==614) || (t2==608 && t1==614)){add_bond(list[i],list[j]);}
	else if((t1==608 && t2==615) || (t2==608 && t1==615)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="PRO"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG2"  ) || (a2=="CG"   && a1=="HG2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG3"  ) || (a2=="CG"   && a1=="HG3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD"   ) || (a2=="CG"   && a1=="CD"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="HD2"  ) || (a2=="CD"   && a1=="HD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="HD3"  ) || (a2=="CD"   && a1=="HD3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="N"    ) || (a2=="CD"   && a1=="N"    )){add_bond(list[i],list[j]);}
	else if((t1==0   && t2==705) || (t2==0   && t1==705)){add_bond(list[i],list[j]);}
	else if((t1==705 && t2==708) || (t2==705 && t1==708)){add_bond(list[i],list[j]);}
	else if((t1==705 && t2==709) || (t2==705 && t1==709)){add_bond(list[i],list[j]);}
	else if((t1==705 && t2==706) || (t2==705 && t1==706)){add_bond(list[i],list[j]);}
	else if((t1==706 && t2==710) || (t2==706 && t1==710)){add_bond(list[i],list[j]);}
	else if((t1==706 && t2==711) || (t2==706 && t1==711)){add_bond(list[i],list[j]);}
	else if((t1==706 && t2==707) || (t2==706 && t1==707)){add_bond(list[i],list[j]);}
	else if((t1==707 && t2==712) || (t2==707 && t1==712)){add_bond(list[i],list[j]);}
	else if((t1==707 && t2==713) || (t2==707 && t1==713)){add_bond(list[i],list[j]);}
	else if((t1==707 && t2==2  ) || (t2==707 && t1==2  )){add_bond(list[i],list[j]);}
      }

      else if(artyp=="PHE"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD1"  ) || (a2=="CG"   && a1=="CD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD2"  ) || (a2=="CG"   && a1=="CD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="HD1"  ) || (a2=="CD1"  && a1=="HD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="CE1"  ) || (a2=="CD1"  && a1=="CE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="HD2"  ) || (a2=="CD2"  && a1=="HD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="CE2"  ) || (a2=="CD2"  && a1=="CE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE1"  && a2=="HE1"  ) || (a2=="CE1"  && a1=="HE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE1"  && a2=="CZ"   ) || (a2=="CE1"  && a1=="CZ"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CE2"  && a2=="HE2"  ) || (a2=="CE2"  && a1=="HE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE2"  && a2=="CZ"   ) || (a2=="CE2"  && a1=="CZ"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CZ"   && a2=="HZ"   ) || (a2=="CZ"   && a1=="HZ"   )){add_bond(list[i],list[j]);}
	else if((t1==0   && t2==805) || (t2==0   && t1==805)){add_bond(list[i],list[j]);}
	else if((t1==805 && t2==812) || (t2==805 && t1==812)){add_bond(list[i],list[j]);}
	else if((t1==805 && t2==813) || (t2==805 && t1==813)){add_bond(list[i],list[j]);}
	else if((t1==805 && t2==806) || (t2==805 && t1==806)){add_bond(list[i],list[j]);}
	else if((t1==806 && t2==807) || (t2==806 && t1==807)){add_bond(list[i],list[j]);}
	else if((t1==806 && t2==808) || (t2==806 && t1==808)){add_bond(list[i],list[j]);}
	else if((t1==807 && t2==814) || (t2==807 && t1==814)){add_bond(list[i],list[j]);}
	else if((t1==807 && t2==809) || (t2==807 && t1==809)){add_bond(list[i],list[j]);}
	else if((t1==808 && t2==815) || (t2==808 && t1==815)){add_bond(list[i],list[j]);}
	else if((t1==808 && t2==810) || (t2==808 && t1==810)){add_bond(list[i],list[j]);}
	else if((t1==809 && t2==816) || (t2==809 && t1==816)){add_bond(list[i],list[j]);}
	else if((t1==809 && t2==811) || (t2==809 && t1==811)){add_bond(list[i],list[j]);}
	else if((t1==810 && t2==817) || (t2==810 && t1==817)){add_bond(list[i],list[j]);}
	else if((t1==810 && t2==811) || (t2==810 && t1==811)){add_bond(list[i],list[j]);}
	else if((t1==811 && t2==818) || (t2==811 && t1==818)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="TRP"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD1"  ) || (a2=="CG"   && a1=="CD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD2"  ) || (a2=="CG"   && a1=="CD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="HD1"  ) || (a2=="CD1"  && a1=="HD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="NE1"  ) || (a2=="CD1"  && a1=="NE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="CE2"  ) || (a2=="CD2"  && a1=="CE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="CE3"  ) || (a2=="CD2"  && a1=="CE3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="NE1"  && a2=="HE1"  ) || (a2=="NE1"  && a1=="HE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="NE1"  && a2=="CE2"  ) || (a2=="NE1"  && a1=="CE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE2"  && a2=="CZ2"  ) || (a2=="CE2"  && a1=="CZ2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CZ2"  && a2=="HZ2"  ) || (a2=="CZ2"  && a1=="HZ2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CZ2"  && a2=="CH2"  ) || (a2=="CZ2"  && a1=="CH2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE3"  && a2=="HE3"  ) || (a2=="CE3"  && a1=="HE3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE3"  && a2=="CZ3"  ) || (a2=="CE3"  && a1=="CZ3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CH2"  && a2=="HH2"  ) || (a2=="CH2"  && a1=="HH2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CZ3"  && a2=="HZ3"  ) || (a2=="CZ3"  && a1=="HZ3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CZ3"  && a2=="CH2"  ) || (a2=="CZ3"  && a1=="CH2"  )){add_bond(list[i],list[j]);}
	else if((t1==0   && t2==905) || (t2==0   && t1==905)){add_bond(list[i],list[j]);}
	else if((t1==905 && t2==915) || (t2==905 && t1==915)){add_bond(list[i],list[j]);}
	else if((t1==905 && t2==916) || (t2==905 && t1==916)){add_bond(list[i],list[j]);}
	else if((t1==905 && t2==906) || (t2==905 && t1==906)){add_bond(list[i],list[j]);}
	else if((t1==906 && t2==907) || (t2==906 && t1==907)){add_bond(list[i],list[j]);}
	else if((t1==906 && t2==908) || (t2==906 && t1==908)){add_bond(list[i],list[j]);}
	else if((t1==907 && t2==917) || (t2==907 && t1==917)){add_bond(list[i],list[j]);}
	else if((t1==907 && t2==909) || (t2==907 && t1==909)){add_bond(list[i],list[j]);}
	else if((t1==908 && t2==910) || (t2==908 && t1==910)){add_bond(list[i],list[j]);}
	else if((t1==908 && t2==911) || (t2==908 && t1==911)){add_bond(list[i],list[j]);}
	else if((t1==909 && t2==918) || (t2==909 && t1==918)){add_bond(list[i],list[j]);}
	else if((t1==909 && t2==910) || (t2==909 && t1==910)){add_bond(list[i],list[j]);}
	else if((t1==910 && t2==914) || (t2==910 && t1==914)){add_bond(list[i],list[j]);}
	else if((t1==914 && t2==922) || (t2==914 && t1==922)){add_bond(list[i],list[j]);}
	else if((t1==914 && t2==913) || (t2==914 && t1==913)){add_bond(list[i],list[j]);}
	else if((t1==911 && t2==919) || (t2==911 && t1==919)){add_bond(list[i],list[j]);}
	else if((t1==911 && t2==912) || (t2==911 && t1==912)){add_bond(list[i],list[j]);}
	else if((t1==913 && t2==921) || (t2==913 && t1==921)){add_bond(list[i],list[j]);}
	else if((t1==912 && t2==920) || (t2==912 && t1==920)){add_bond(list[i],list[j]);}
	else if((t1==912 && t2==913) || (t2==912 && t1==913)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="SER"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="OG"   ) || (a2=="CB"   && a1=="OG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="OG"   && a2=="HG"   ) || (a2=="OG"   && a1=="HG"   )){add_bond(list[i],list[j]);}
	else if((t1==0    && t2==1005) || (t2==0    && t1==1005)){add_bond(list[i],list[j]);}
	else if((t1==1005 && t2==1007) || (t2==1005 && t1==1007)){add_bond(list[i],list[j]);}
	else if((t1==1005 && t2==1008) || (t2==1005 && t1==1008)){add_bond(list[i],list[j]);}
	else if((t1==1005 && t2==1006) || (t2==1005 && t1==1006)){add_bond(list[i],list[j]);}
	else if((t1==1006 && t2==1009) || (t2==1006 && t1==1009)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="THR"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB1"  ) || (a2=="CB"   && a1=="HB1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="OG1"  ) || (a2=="CB"   && a1=="OG1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG2"  ) || (a2=="CB"   && a1=="CG2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="OG1"  && a2=="HG1"  ) || (a2=="OG1"  && a1=="HG1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG2"  && a2=="HG21" ) || (a2=="CG2"  && a1=="HG21" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG2"  && a2=="HG22" ) || (a2=="CG2"  && a1=="HG22" )){add_bond(list[i],list[j]);}
	//else if((a1=="CG2"  && a2=="HG23" ) || (a2=="CG2"  && a1=="HG23" )){add_bond(list[i],list[j]);}
	//else {
	//  if(0){}
	//  else if((a1=="CB"   && a2=="HB"   ) || (a2=="CB"   && a1=="HB"   )){add_bond(list[i],list[j]);}
	//}
	else if((t1==0    && t2==1105) || (t2==0    && t1==1105)){add_bond(list[i],list[j]);}
	else if((t1==1105 && t2==1108) || (t2==1105 && t1==1108)){add_bond(list[i],list[j]);}
	else if((t1==1105 && t2==1107) || (t2==1105 && t1==1107)){add_bond(list[i],list[j]);}
	else if((t1==1105 && t2==1106) || (t2==1105 && t1==1106)){add_bond(list[i],list[j]);}
	else if((t1==1107 && t2==1112) || (t2==1107 && t1==1112)){add_bond(list[i],list[j]);}
	else if((t1==1106 && t2==1109) || (t2==1106 && t1==1109)){add_bond(list[i],list[j]);}
	else if((t1==1106 && t2==1110) || (t2==1106 && t1==1110)){add_bond(list[i],list[j]);}
	else if((t1==1106 && t2==1111) || (t2==1106 && t1==1111)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="ASN"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="OD1"  ) || (a2=="CG"   && a1=="OD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="ND2"  ) || (a2=="CG"   && a1=="ND2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="ND2"  && a2=="HD21" ) || (a2=="ND2"  && a1=="HD21" )){add_bond(list[i],list[j]);}
	//else if((a1=="ND2"  && a2=="HD22" ) || (a2=="ND2"  && a1=="HD22" )){add_bond(list[i],list[j]);}
	else if((t1==0    && t2==1205) || (t2==0    && t1==1205)){add_bond(list[i],list[j]);}
	else if((t1==1205 && t2==1209) || (t2==1205 && t1==1209)){add_bond(list[i],list[j]);}
	else if((t1==1205 && t2==1210) || (t2==1205 && t1==1210)){add_bond(list[i],list[j]);}
	else if((t1==1205 && t2==1206) || (t2==1205 && t1==1206)){add_bond(list[i],list[j]);}
	else if((t1==1206 && t2==1208) || (t2==1206 && t1==1208)){add_bond(list[i],list[j]);}
	else if((t1==1206 && t2==1207) || (t2==1206 && t1==1207)){add_bond(list[i],list[j]);}
	else if((t1==1207 && t2==1211) || (t2==1207 && t1==1211)){add_bond(list[i],list[j]);}
	else if((t1==1207 && t2==1212) || (t2==1207 && t1==1212)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="GLN"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG2"  ) || (a2=="CG"   && a1=="HG2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG3"  ) || (a2=="CG"   && a1=="HG3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD"   ) || (a2=="CG"   && a1=="CD"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="OE1"  ) || (a2=="CD"   && a1=="OE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="NE2"  ) || (a2=="CD"   && a1=="NE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="NE2"  && a2=="HE21" ) || (a2=="NE2"  && a1=="HE21" )){add_bond(list[i],list[j]);}
	//else if((a1=="NE2"  && a2=="HE22" ) || (a2=="NE2"  && a1=="HE22" )){add_bond(list[i],list[j]);}
	else if((t1==0    && t2==1305) || (t2==0    && t1==1305)){add_bond(list[i],list[j]);}
	else if((t1==1305 && t2==1310) || (t2==1305 && t1==1310)){add_bond(list[i],list[j]);}
	else if((t1==1305 && t2==1311) || (t2==1305 && t1==1311)){add_bond(list[i],list[j]);}
	else if((t1==1305 && t2==1306) || (t2==1305 && t1==1306)){add_bond(list[i],list[j]);}
	else if((t1==1306 && t2==1312) || (t2==1306 && t1==1312)){add_bond(list[i],list[j]);}
	else if((t1==1306 && t2==1313) || (t2==1306 && t1==1313)){add_bond(list[i],list[j]);}
	else if((t1==1306 && t2==1307) || (t2==1306 && t1==1307)){add_bond(list[i],list[j]);}
	else if((t1==1307 && t2==1309) || (t2==1307 && t1==1309)){add_bond(list[i],list[j]);}
	else if((t1==1307 && t2==1308) || (t2==1307 && t1==1308)){add_bond(list[i],list[j]);}
	else if((t1==1308 && t2==1314) || (t2==1308 && t1==1314)){add_bond(list[i],list[j]);}
	else if((t1==1308 && t2==1315) || (t2==1308 && t1==1315)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="TYR"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD1"  ) || (a2=="CG"   && a1=="CD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD2"  ) || (a2=="CG"   && a1=="CD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="HD1"  ) || (a2=="CD1"  && a1=="HD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD1"  && a2=="CE1"  ) || (a2=="CD1"  && a1=="CE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="HD2"  ) || (a2=="CD2"  && a1=="HD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="CE2"  ) || (a2=="CD2"  && a1=="CE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE1"  && a2=="HE1"  ) || (a2=="CE1"  && a1=="HE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE1"  && a2=="CZ"   ) || (a2=="CE1"  && a1=="CZ"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CE2"  && a2=="HE2"  ) || (a2=="CE2"  && a1=="HE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE2"  && a2=="CZ"   ) || (a2=="CE2"  && a1=="CZ"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CZ"   && a2=="OH"   ) || (a2=="CZ"   && a1=="OH"   )){add_bond(list[i],list[j]);}
	//else if((a1=="OH"   && a2=="HH"   ) || (a2=="OH"   && a1=="HH"   )){add_bond(list[i],list[j]);}
	else if((t1==0    && t2==1405) || (t2==0    && t1==1405)){add_bond(list[i],list[j]);}
	else if((t1==1405 && t2==1413) || (t2==1405 && t1==1413)){add_bond(list[i],list[j]);}
	else if((t1==1405 && t2==1414) || (t2==1405 && t1==1414)){add_bond(list[i],list[j]);}
	else if((t1==1405 && t2==1406) || (t2==1405 && t1==1406)){add_bond(list[i],list[j]);}
	else if((t1==1406 && t2==1407) || (t2==1406 && t1==1407)){add_bond(list[i],list[j]);}
	else if((t1==1406 && t2==1408) || (t2==1406 && t1==1408)){add_bond(list[i],list[j]);}
	else if((t1==1407 && t2==1415) || (t2==1407 && t1==1415)){add_bond(list[i],list[j]);}
	else if((t1==1407 && t2==1409) || (t2==1407 && t1==1409)){add_bond(list[i],list[j]);}
	else if((t1==1408 && t2==1416) || (t2==1408 && t1==1416)){add_bond(list[i],list[j]);}
	else if((t1==1408 && t2==1410) || (t2==1408 && t1==1410)){add_bond(list[i],list[j]);}
	else if((t1==1409 && t2==1417) || (t2==1409 && t1==1417)){add_bond(list[i],list[j]);}
	else if((t1==1409 && t2==1411) || (t2==1409 && t1==1411)){add_bond(list[i],list[j]);}
	else if((t1==1410 && t2==1418) || (t2==1410 && t1==1418)){add_bond(list[i],list[j]);}
	else if((t1==1410 && t2==1411) || (t2==1410 && t1==1411)){add_bond(list[i],list[j]);}
	else if((t1==1411 && t2==1412) || (t2==1411 && t1==1412)){add_bond(list[i],list[j]);}
	else if((t1==1412 && t2==1419) || (t2==1412 && t1==1419)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="CYS" || artyp=="CYX"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="SG"   ) || (a2=="CB"   && a1=="SG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="SG"   && a2=="HG"   ) || (a2=="SG"   && a1=="HG"   )){add_bond(list[i],list[j]);}
	else if((t1==0    && t2==1505) || (t2==0    && t1==1505)){add_bond(list[i],list[j]);}
	else if((t1==1505 && t2==1507) || (t2==1505 && t1==1507)){add_bond(list[i],list[j]);}
	else if((t1==1505 && t2==1508) || (t2==1505 && t1==1508)){add_bond(list[i],list[j]);}
	else if((t1==1505 && t2==1506) || (t2==1505 && t1==1506)){add_bond(list[i],list[j]);}
	else if((t1==1506 && t2==1509) || (t2==1506 && t1==1509)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="LYS"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB1"  ) || (a2=="CB"   && a1=="HB1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="SG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG1"  ) || (a2=="CG"   && a1=="HG1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG3"  ) || (a2=="CG"   && a1=="HG3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG2"  ) || (a2=="CG"   && a1=="HG2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD"   ) || (a2=="CG"   && a1=="CD"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="HD1"  ) || (a2=="CD"   && a1=="HD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="HD3"  ) || (a2=="CD"   && a1=="HD3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="HD2"  ) || (a2=="CD"   && a1=="HD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="CE"   ) || (a2=="CD"   && a1=="CE"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CE"   && a2=="HE1"  ) || (a2=="CE"   && a1=="HE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE"   && a2=="HE3"  ) || (a2=="CE"   && a1=="HE3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE"   && a2=="HE2"  ) || (a2=="CE"   && a1=="HE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE"   && a2=="NZ"   ) || (a2=="CE"   && a1=="NZ"   )){add_bond(list[i],list[j]);}
	//else if((a1=="NZ"   && a2=="HZ1"  ) || (a2=="NZ"   && a1=="NZ1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="NZ"   && a2=="HZ3"  ) || (a2=="NZ"   && a1=="NZ3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="NZ"   && a2=="HZ2"  ) || (a2=="NZ"   && a1=="NZ2"  )){add_bond(list[i],list[j]);}
	else if((t1==0    && t2==1605) || (t2==0    && t1==1605)){add_bond(list[i],list[j]);}
	else if((t1==1605 && t2==1611) || (t2==1605 && t1==1611)){add_bond(list[i],list[j]);}
	else if((t1==1605 && t2==1610) || (t2==1605 && t1==1610)){add_bond(list[i],list[j]);}
	else if((t1==1605 && t2==1606) || (t2==1605 && t1==1606)){add_bond(list[i],list[j]);}
	else if((t1==1606 && t2==1613) || (t2==1606 && t1==1613)){add_bond(list[i],list[j]);}
	else if((t1==1606 && t2==1612) || (t2==1606 && t1==1612)){add_bond(list[i],list[j]);}
	else if((t1==1606 && t2==1607) || (t2==1606 && t1==1607)){add_bond(list[i],list[j]);}
	else if((t1==1607 && t2==1615) || (t2==1607 && t1==1615)){add_bond(list[i],list[j]);}
	else if((t1==1607 && t2==1614) || (t2==1607 && t1==1614)){add_bond(list[i],list[j]);}
	else if((t1==1607 && t2==1608) || (t2==1607 && t1==1608)){add_bond(list[i],list[j]);}
	else if((t1==1608 && t2==1617) || (t2==1608 && t1==1617)){add_bond(list[i],list[j]);}
	else if((t1==1608 && t2==1616) || (t2==1608 && t1==1616)){add_bond(list[i],list[j]);}
	else if((t1==1608 && t2==1609) || (t2==1608 && t1==1609)){add_bond(list[i],list[j]);}
	else if((t1==1609 && t2==1618) || (t2==1609 && t1==1618)){add_bond(list[i],list[j]);}
	else if((t1==1609 && t2==1620) || (t2==1609 && t1==1620)){add_bond(list[i],list[j]);}
	else if((t1==1609 && t2==1619) || (t2==1609 && t1==1619)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="ARG"){
	if(0){}
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB1"  ) || (a2=="CB"   && a1=="HB1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG1"  ) || (a2=="CG"   && a1=="HG1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG3"  ) || (a2=="CG"   && a1=="HG3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="HG2"  ) || (a2=="CG"   && a1=="HG2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD"   ) || (a2=="CG"   && a1=="CD"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="HD1"  ) || (a2=="CD"   && a1=="HD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="HD3"  ) || (a2=="CD"   && a1=="HD3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="HD2"  ) || (a2=="CD"   && a1=="HD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD"   && a2=="NE"   ) || (a2=="CD"   && a1=="NE"   )){add_bond(list[i],list[j]);}
	//else if((a1=="NE"   && a2=="HE"   ) || (a2=="NE"   && a1=="HE"   )){add_bond(list[i],list[j]);}
	//else if((a1=="NE"   && a2=="CZ"   ) || (a2=="NE"   && a1=="CZ"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CZ"   && a2=="NH1"  ) || (a2=="CZ"   && a1=="NH1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CZ"   && a2=="NH2"  ) || (a2=="CZ"   && a1=="NH2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="NH1"  && a2=="HH11" ) || (a2=="NH1"  && a1=="HH11" )){add_bond(list[i],list[j]);}
	//else if((a1=="NH1"  && a2=="HH12" ) || (a2=="NH1"  && a1=="HH12" )){add_bond(list[i],list[j]);}
	//else if((a1=="NH2"  && a2=="HH21" ) || (a2=="NH2"  && a1=="HH21" )){add_bond(list[i],list[j]);}
	//else if((a1=="NH2"  && a2=="HH22" ) || (a2=="NH2"  && a1=="HH22" )){add_bond(list[i],list[j]);}
	else if((t1==0    && t2==1705) || (t2==0    && t1==1705)){add_bond(list[i],list[j]);}
	else if((t1==1705 && t2==1713) || (t2==1705 && t1==1713)){add_bond(list[i],list[j]);}
	else if((t1==1705 && t2==1712) || (t2==1705 && t1==1712)){add_bond(list[i],list[j]);}
	else if((t1==1705 && t2==1706) || (t2==1705 && t1==1706)){add_bond(list[i],list[j]);}
	else if((t1==1706 && t2==1715) || (t2==1706 && t1==1715)){add_bond(list[i],list[j]);}
	else if((t1==1706 && t2==1714) || (t2==1706 && t1==1714)){add_bond(list[i],list[j]);}
	else if((t1==1706 && t2==1707) || (t2==1706 && t1==1707)){add_bond(list[i],list[j]);}
	else if((t1==1707 && t2==1717) || (t2==1707 && t1==1717)){add_bond(list[i],list[j]);}
	else if((t1==1707 && t2==1716) || (t2==1707 && t1==1716)){add_bond(list[i],list[j]);}
	else if((t1==1707 && t2==1708) || (t2==1707 && t1==1708)){add_bond(list[i],list[j]);}
	else if((t1==1708 && t2==1718) || (t2==1708 && t1==1718)){add_bond(list[i],list[j]);}
	else if((t1==1708 && t2==1709) || (t2==1708 && t1==1709)){add_bond(list[i],list[j]);}
	else if((t1==1709 && t2==1710) || (t2==1709 && t1==1710)){add_bond(list[i],list[j]);}
	else if((t1==1709 && t2==1711) || (t2==1709 && t1==1711)){add_bond(list[i],list[j]);}
	else if((t1==1710 && t2==1719) || (t2==1710 && t1==1719)){add_bond(list[i],list[j]);}
	else if((t1==1710 && t2==1720) || (t2==1710 && t1==1720)){add_bond(list[i],list[j]);}
	else if((t1==1711 && t2==1721) || (t2==1711 && t1==1721)){add_bond(list[i],list[j]);}
	else if((t1==1711 && t2==1722) || (t2==1711 && t1==1722)){add_bond(list[i],list[j]);}
      }

      else if(artyp=="HIS" || artyp=="HID" || artyp=="HIE" || artyp=="HIP"){ 
	if(0){}
        //cout << "ishi" << endl ;
	//else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB1"  ) || (a2=="CB"   && a1=="HB1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="ND1"  ) || (a2=="CG"   && a1=="ND1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CG"   && a2=="CD2"  ) || (a2=="CG"   && a1=="CD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="ND1"  && a2=="HD1"  ) || (a2=="ND1"  && a1=="HD1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="ND1"  && a2=="CE1"  ) || (a2=="ND1"  && a1=="CE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="HD2"  ) || (a2=="CD2"  && a1=="HD2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CD2"  && a2=="NE2"  ) || (a2=="CD2"  && a1=="NE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE1"  && a2=="HE1"  ) || (a2=="CE1"  && a1=="HE1"  )){add_bond(list[i],list[j]);}
	//else if((a1=="NE2"  && a2=="HE2"  ) || (a2=="NE2"  && a1=="HE2"  )){add_bond(list[i],list[j]);}
	//else if((a1=="CE1"  && a2=="NE2"  ) || (a2=="CE1"  && a1=="NE2"  )){add_bond(list[i],list[j]);}
	else if((t1==0    && t2==1805) || (t2==0    && t1==1805)){add_bond(list[i],list[j]);}
	else if((t1==1805 && t2==1812) || (t2==1805 && t1==1812)){add_bond(list[i],list[j]);}
	else if((t1==1805 && t2==1811) || (t2==1805 && t1==1811)){add_bond(list[i],list[j]);}
	else if((t1==1805 && t2==1806) || (t2==1805 && t1==1806)){add_bond(list[i],list[j]);}
	else if((t1==1806 && t2==1808) || (t2==1806 && t1==1808)){add_bond(list[i],list[j]);}
	else if((t1==1806 && t2==1807) || (t2==1806 && t1==1807)){add_bond(list[i],list[j]);}
	else if((t1==1808 && t2==1814) || (t2==1808 && t1==1814)){add_bond(list[i],list[j]);}
	else if((t1==1808 && t2==1810) || (t2==1808 && t1==1810)){add_bond(list[i],list[j]);}
	else if((t1==1807 && t2==1813) || (t2==1807 && t1==1813)){add_bond(list[i],list[j]);}
	else if((t1==1807 && t2==1809) || (t2==1807 && t1==1809)){add_bond(list[i],list[j]);}
	else if((t1==1810 && t2==1815) || (t2==1810 && t1==1815)){add_bond(list[i],list[j]);}
	else if((t1==1809 && t2==1816) || (t2==1809 && t1==1816)){add_bond(list[i],list[j]);}
	else if((t1==1810 && t2==1809) || (t2==1810 && t1==1809)){add_bond(list[i],list[j]);}
        //else {
        //  cout << list[i]+1 << " " << list[j]+1 << " " << t1 << " " << t2 << endl;
        //}
      }

      else if(artyp=="ASP"){
	if(0){}
	else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	else if((a1=="CB"   && a2=="HB1"  ) || (a2=="CB"   && a1=="HB1"  )){add_bond(list[i],list[j]);}
	else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
 	else if((a1=="CG"   && a2=="OD1"  ) || (a2=="CG"   && a1=="OD1"  )){add_bond(list[i],list[j]);}
	else if((a1=="CG"   && a2=="OD2"  ) || (a2=="CG"   && a1=="OD2"  )){add_bond(list[i],list[j]);}
      }

      else if(artyp=="GLU"){
	if(0){}
	else if((a1=="CA"   && a2=="CB"   ) || (a2=="CA"   && a1=="CB"   )){add_bond(list[i],list[j]);}
	else if((a1=="CB"   && a2=="HB1"  ) || (a2=="CB"   && a1=="HB1"  )){add_bond(list[i],list[j]);}
	else if((a1=="CB"   && a2=="HB2"  ) || (a2=="CB"   && a1=="HB2"  )){add_bond(list[i],list[j]);}
	else if((a1=="CB"   && a2=="HB3"  ) || (a2=="CB"   && a1=="HB3"  )){add_bond(list[i],list[j]);}
	else if((a1=="CB"   && a2=="CG"   ) || (a2=="CB"   && a1=="CG"   )){add_bond(list[i],list[j]);}
	else if((a1=="CG"   && a2=="HG1"  ) || (a2=="CG"   && a1=="HG1"  )){add_bond(list[i],list[j]);}
	else if((a1=="CG"   && a2=="HG2"  ) || (a2=="CG"   && a1=="HG2"  )){add_bond(list[i],list[j]);}
	else if((a1=="CG"   && a2=="HG3"  ) || (a2=="CG"   && a1=="HG3"  )){add_bond(list[i],list[j]);}
	else if((a1=="CG"   && a2=="CD"   ) || (a2=="CG"   && a1=="CD"   )){add_bond(list[i],list[j]);}
	else if((a1=="CD"   && a2=="OE1"  ) || (a2=="CD"   && a1=="OE1"  )){add_bond(list[i],list[j]);}
	else if((a1=="CD"   && a2=="OE2"  ) || (a2=="CD"   && a1=="OE2"  )){add_bond(list[i],list[j]);}
      }
    }
  }
}

void Molecule::add_bond_ff_tip3p()
{
  for(int i=0;i<nlinkgroup;i++){
    if(linkgroup[i].get_n_atom_index()==3){
      int nh=0;
      int no=0;
      for(int j=0;j<3;j++){
	if(atom[linkgroup[i].get_atom_index(j)].get_atomic_number()==1) nh++;
	if(atom[linkgroup[i].get_atom_index(j)].get_atomic_number()==8) no++;
      }
      if(nh==2 && no==1){
	int iatom1=-1;
	int iatom2=-1;
	for(int j=0;j<3;j++){
	  if(atom[linkgroup[i].get_atom_index(j)].get_atomic_number()==1){
	    if(iatom1==-1){iatom1=linkgroup[i].get_atom_index(j);}	  
	    else          {iatom2=linkgroup[i].get_atom_index(j);}
	  }
	}
	if(iatom1 != -1 && iatom2 != -1){
	  int chk=0;
	  for(int j=0;j<nbond;j++){
	    int ba1=bond[j].get_iatom1();
	    int ba2=bond[j].get_iatom2();
	    if(ba1==iatom1 && ba2==iatom2) chk=1;
	    if(ba1==iatom2 && ba2==iatom1) chk=1;
	  }
	  if(chk==0){
	    add_bond(iatom1,iatom2);
	  }
	} else {
	  cout << "ishi in add_bond_ff_tip3p , error : (iatom1,iatom2)= " << iatom1 << "," << iatom2 << endl;
	}
      }
    }
  }
}

void Molecule::add_bond_in_fragment()
{
  int a1,a2,iatom,ifrag1,ifrag2,chk1,n,imin;
  double x1,y1,z1,x2,y2,z2,dmin,d;
  Dvector3D r1,r2;
  int * out = new int [natom] ;
  for(int i=0;i<natom;i++){
    ifrag1=get_ifragment_from_iatom(i);
    if(ifrag1>=0){
      chk1=0;
      for(int j=0;j<nbond;j++){
        a1=bond[j].get_iatom1();
        a2=bond[j].get_iatom1();
        iatom=-1;
        if(i==a1){
          iatom=a2;
        }
        if(i==a2){
          iatom=a1;
        }
        if(iatom>=0){
          ifrag2=get_ifragment_from_iatom(iatom);
          if(ifrag1==ifrag2){
            chk1=1;
            break;
          }
        }
      }
      if(chk1==0){
        r1.copy(atom[i].get_R());
        x1=r1.get_x();
        y1=r1.get_y();
        z1=r1.get_z();
        n=get_atom_index_fragment(1,&ifrag1,out);
        dmin=999999.0;
        for(int j=0;j<n;j++){
          if(out[j]!=i){
            r2.copy(atom[out[j]].get_R());
            x2=r2.get_x();
            y2=r2.get_y();
            z2=r2.get_z();
            d=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
            if(dmin>d){
              dmin=d;
              imin=out[j];
            }
          }
        }
        add_bond(i,imin);
      }
    }
  }
  delete [] out;
}

void Molecule::delete_bond(const int iatom1,
                           const int iatom2,
                           const int ierr)
{
  if(iatom1<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Moelcule::delete_bond , iatom1 < 0 " << endl ;
      cerr << "iatom1 = " << iatom1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::delete_bond , iatom2 < 0 " << endl ;
      cerr << "iatom2 = " << iatom2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom1>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::delete_bond , iatom1 >= natom " << endl ;
      cerr << "iatom1 = " << iatom1 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::delete_bond , iatom2 >= natom " << endl ;
      cerr << "iatom2 = " << iatom2 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  for(int i=0;i<nbond;i++){
    if(iatom1==bond[i].get_iatom1() && iatom2==bond[i].get_iatom2()){pick_up_bond(i);return;}
    if(iatom1==bond[i].get_iatom2() && iatom2==bond[i].get_iatom1()){pick_up_bond(i);return;}
  }    
}

void Molecule::delete_bond_atom_range_in(const int nlist,
                                        const int * list,
                                        const int ierr)
{
  int n,a1,a2,chk1,chk2;
  int * ia1 = new int [nbond+1];
  int * ia2 = new int [nbond+1];
  n=0;
  for(int i=0;i<nbond;i++){
    a1=bond[i].get_iatom1();
    a2=bond[i].get_iatom2();
    chk1=0;
    chk2=0;
    for(int j=0;j<nlist;j++){
      if(a1==list[j]){chk1=1;}
      if(a2==list[j]){chk2=1;}
    }
    if(chk1==1 && chk2==1){
      ia1[n]=a1;
      ia2[n]=a2;
      n++;
    }
  }
  for(int i=0;i<n;i++){
    delete_bond(ia1[i],ia2[i],ierr);
  }
  delete [] ia1;
  delete [] ia2;
}

void Molecule::delete_bond_atom_range_out(const int nlist,
                                         const int * list,
                                         const int ierr)
{
  int n,a1,a2,chk1,chk2;
  int * ia1 = new int [nbond+1];
  int * ia2 = new int [nbond+1];
  n=0;
  for(int i=0;i<nbond;i++){
    a1=bond[i].get_iatom1();
    a2=bond[i].get_iatom2();
    chk1=0;
    chk2=0;
    for(int j=0;j<nlist;j++){
      if(a1==list[j]){chk1=1;}
      if(a2==list[j]){chk2=1;}
    }
    if((chk1==1 && chk2==0) || (chk1==0 && chk2==1)){
      ia1[n]=a1;
      ia2[n]=a2;
      n++;
    }
  }
  for(int i=0;i<n;i++){
    delete_bond(ia1[i],ia2[i],ierr);
  }
  delete [] ia1;
  delete [] ia2;
}

void Molecule::bond_trace(const int  iatom,
                          int*       num  ,
                          int*       group,
                          const int  nb   ,
                          const int* a1   ,
                          const int* a2   ) const
{
  int bond1, bond2, chk, chk_bond, n;
  group[*num]=iatom;
  *num = *num + 1;
  if(iatom>=0){
    n=atom[iatom].get_bond_atom_n();
    for(int i=0;i<n;i++){
      bond1=iatom;
      bond2=atom[iatom].get_bond_atom(i);
      chk_bond=0;
      for(int j=0;j<nb;j++){
	if(a1[j]==bond1 && a2[j]==bond2) chk_bond=1;
	if(a2[j]==bond1 && a1[j]==bond2) chk_bond=1;
      }
      if(chk_bond==0){
	chk=0;
	for(int j=0;j<*num;j++){
	  if(group[j]==bond2) {
	    chk=1;
	    break;
	  }
	}
	if(chk==0) {
	  bond_trace(bond2,num,group,nb,a1,a2);
	}
      }
    }
  }  
}

void Molecule::bond_trace(const int   iatom           ,
                          int*        num             , 
                          int*        group           ,
                          const int*  atom_bond_list_n,
                          int**       atom_bond_list  ) const
{
  int bond2, chk;
  group[*num]=iatom;
  *num = *num + 1;
  for(int i=0;i<atom_bond_list_n[iatom];i++){
    bond2=atom_bond_list[iatom][i];
    chk=0;
    for(int j=0;j<*num;j++){
      if(group[j]==bond2) {
        chk=1;
        break;
      }
    }
    if(chk==0) {
      bond_trace(bond2,num,group,atom_bond_list_n,atom_bond_list);
    }
  }
}

int Molecule::get_bond_iatom1(const int ibond,
                              const int ierr ) const
{
  if(ibond<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_bond_iatom1 , ibond < 0 " << endl ;
      cerr << "ibond = " << ibond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibond>=nbond){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_bond_iatom1 , ibond >= nbond " << endl ;
      cerr << "ibond = " << ibond << " , nbond = " << nbond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return bond[ibond].get_iatom1();
}

int Molecule::get_bond_iatom2(const int ibond,
                              const int ierr ) const 
{
  if(ibond<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_bond_iatom2 , ibond < 0 " << endl ;
      cerr << "ibond = " << ibond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibond>=nbond){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_bond_iatom2 , ibond >= nbond " << endl ;
      cerr << "ibond = " << ibond << " , nbond = " << nbond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return bond[ibond].get_iatom2();
}

void Molecule::get_bond_iatom(int* iatom1,
                              int* iatom2) const
{
  for(int i=0;i<nbond;i++){
    iatom1[i]=bond[i].get_iatom1();
    iatom2[i]=bond[i].get_iatom2();
  }
}

Bond Molecule::get_bond (const int ibond,
                         const int ierr ) const 
{
  if(ibond<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_bond , ibond < 0 " << endl ;
      cerr << "ibond = " << ibond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibond>=nbond){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_bond , ibond >= nbond " << endl ;
      cerr << "ibond = " << ibond << " , nbond = " << nbond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return bond[ibond];
}

int Molecule::get_nbond( ) const
{ 
  return nbond ;
}

int Molecule::get_bond_atom(const int iatom, 
                            int * jatom    ,
                            const int ierr ) const
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_bond_atom , iatom < 0" << endl ;
      cerr << "iatom = " << iatom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Moelcule::get_bond_atom , iatom >= natom" << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int iatom1,iatom2;
  int n=0;
  for(int i=0;i<nbond;i++){
    iatom1=bond[i].get_iatom1();
    iatom2=bond[i].get_iatom2();
    if(iatom1==iatom){
      jatom[n]=iatom2;
      n++;
    }
    if(iatom2==iatom){
      jatom[n]=iatom1;
      n++;
    }
  }
  return n;
}

int Molecule::get_bond_atom(const int  n     ,
                            const int* iatom ,
                            int*       jatom ,
                            const int  ierr  ) const 
{
  int n_out=0;
  int * buf = new int [natom];
  int ntmp,chk;
  for(int i=0;i<n;i++){
    ntmp=get_bond_atom(iatom[i],buf,ierr);
    for(int j=0;j<ntmp;j++){
      chk=0;
      for(int k=0;k<n;k++){
        if(buf[j]==iatom[k]){
          chk=1;
          break;
        }
      }
      if(chk==0){
        for(int k=0;k<n_out;k++){
          if(buf[j]==jatom[k]){
            chk=1;
            break;
          }
        }
      }
      if(chk==0){
        jatom[n_out]=buf[j];
        n_out++;
      }
    }
  }
  delete [] buf;
  return n_out;
}

void Molecule::get_bond_rotatable( int * out  ,
                                   int * type ) const
{

  // set   0 : not rotatable 
  // set   1 :     rotatable (     internal degree of freedom ) 
  // set   2 :     rotatable ( not internal degree of freedom )
  // set  -1 : not ditected
  // type  0 : main-chain
  // type -1 : not ditected

  int n,b1,b2,ba1,ba2,type1,type2,chk,bio1,bio2,bio_type_1,bio_type_2;
  for(int i=0;i<nbond;i++){
    b1=bond[i].get_iatom1();
    b2=bond[i].get_iatom2();
    type1=atom[b1].get_bio1();
    type2=atom[b2].get_bio1();
    bio1=get_ibiounit_from_iatom(b1);
    bio2=get_ibiounit_from_iatom(b2);
    if(bio1>=0){bio_type_1=biounit[bio1].get_type();} else {bio_type_1=-1;}
    if(bio2>=0){bio_type_2=biounit[bio2].get_type();} else {bio_type_2=-1;}
    if(type != NULL){
      if( (type1==0 || type1==1 || type1==2) &&
          (type2==0 || type2==1 || type2==2) ){
        type[i]= 0;
      } else {
        type[i]=-1;
      }
    }
    chk=BioUnit::bnd_type_rotatable(type1,type2,bio_type_1,bio_type_2);
    if(chk==1){
      ba1=atom[b1].get_bond_atom_n();
      ba2=atom[b2].get_bond_atom_n();
      if(ba1>1 && ba2>1){
        out[i]=1;
      } else {
        out[i]=2;
      }
    }
    if(chk==0){
      out[i]=0;
    }
    if(chk==-1){
      out[i]=-1;
      cout << "Molecule::get_bond_rotatable , chk==-1 (not biounit atom) : b1 b2 type1 type2 = " ;
      cout.width(10); cout << b1 ;
      cout.width(10); cout << b2 ;
      cout.width(10); cout << type1 ;
      cout.width(10); cout << type2 ;
      cout << endl ;
    }
  }
}

void Molecule::get_bond_side_atom( const int ibond_atom1,
                                   const int ibond_atom2,
                                   const int iatom      ,
                                   int*      out_n_1    ,
                                   int*      out_n_2    ,
                                   int*      out_list_1 ,
                                   int*      out_list_2 ) const 
{
  int a1,a2;
  for(int i=0;i<nbond;i++){
    a1=bond[i].get_iatom1();
    a2=bond[i].get_iatom2();
    if(ibond_atom1==a1 && ibond_atom2==a2){get_bond_side_atom(i,iatom,out_n_1,out_n_2,out_list_1,out_list_2);return;}
    if(ibond_atom1==a2 && ibond_atom2==a1){get_bond_side_atom(i,iatom,out_n_1,out_n_2,out_list_1,out_list_2);return;}
  }
  cout << "error in get_bond_side_atom <1>" << endl ;
  exit(1);
}

void Molecule::get_bond_side_atom( const int ibond     ,
                                   const int iatom     ,
                                   int*      out_n_1   ,
                                   int*      out_n_2   ,
                                   int*      out_list_1,
                                   int*      out_list_2) const 
{
  //cout << "get_bond_side_atom : start" << endl ;
  //cout << "ibond=" << ibond << endl ;
  int ia1,ia2,n1,n2,nb,a1,a2,chk;
  int * group1;
  int * group2;
  ia1=bond[ibond].get_iatom1();
  ia2=bond[ibond].get_iatom2();
  //cout << " ia1 = " << ia1 << endl ;
  //cout << " ia2 = " << ia2 << endl ;
  nb=1;
  n1=0;
  n2=0;
  a1=ia1;
  a2=ia2;
  group1=new int [natom+1];
  group2=new int [natom+1];
  //cout << ia1 << " " << nb << " " << a1 << " " << a2 << endl;
  bond_trace(ia1,&n1,group1,nb,&a1,&a2);
  //cout << ia2 << " " << nb << " " << a1 << " " << a2 << endl;
  bond_trace(ia2,&n2,group2,nb,&a1,&a2);
  chk=0;
  for(int i=0;i<n1;i++){
    if(group1[i]==iatom){
      chk=1;
      break;
    }
  }
  if(chk==0){
    for(int i=0;i<n2;i++){
      if(group2[i]==iatom){
        chk=2;
        break;
      }
    }
  }
  if(chk==1){
    for(int i=0;i<n1;i++){out_list_1[i]=group1[i];};(*out_n_1)=n1;
    for(int i=0;i<n2;i++){out_list_2[i]=group2[i];};(*out_n_2)=n2;
  }
  else if(chk==2){
    for(int i=0;i<n2;i++){out_list_1[i]=group2[i];};(*out_n_1)=n2;
    for(int i=0;i<n1;i++){out_list_2[i]=group1[i];};(*out_n_2)=n1;
  } else {
    cout << "error in get bond side atom <2>" << endl ;
    exit(1);
  }  
  //cout << "get_bond_side_atom : end" << endl ;
}

void Molecule::chk_bond_num_H (int * bond0     ,
                               int * bond2     ,
                               int * bond0_list,
                               int * bond2_list) const 
{
  int * ibuf = new int [natom];
  int iatom1,iatom2;
  for(int i=0;i<natom;i++){
    ibuf[i]=0;
  }
  for(int i=0;i<nbond;i++){
    iatom1=bond[i].get_iatom1();
    iatom2=bond[i].get_iatom2();
    ibuf[iatom1]++;
    ibuf[iatom2]++;
  }
  *bond0=0;
  *bond2=0;
  for(int i=0;i<natom;i++){
    if(atom[i].get_atomic_number()==1 && ibuf[i]<=0){
      bond0_list[(*bond0)]=i;
      (*bond0)++;
    }
    if(atom[i].get_atomic_number()==1 && ibuf[i]>=2){
      bond2_list[(*bond2)]=i;
      (*bond2)++;
    }
  }
  delete [] ibuf;  
}

void Molecule::chk_bond_num_C (int * bond1     ,
                               int * bond5     ,
                               int * bond1_list,
                               int * bond5_list) const 
{
  int * ibuf = new int [natom];
  int iatom1,iatom2;
  for(int i=0;i<natom;i++){
    ibuf[i]=0;
  }
  for(int i=0;i<nbond;i++){
    iatom1=bond[i].get_iatom1();
    iatom2=bond[i].get_iatom2();
    ibuf[iatom1]++;
    ibuf[iatom2]++;
  }
  *bond1=0;
  *bond5=0;
  for(int i=0;i<natom;i++){
    if(atom[i].get_atomic_number()==6 && ibuf[i]<=1){
      bond1_list[(*bond1)]=i;
      (*bond1)++;
    }
    if(atom[i].get_atomic_number()==6 && ibuf[i]>=5){
      bond5_list[(*bond5)]=i;
      (*bond5)++;
    }
  }
  delete [] ibuf;  
}

void Molecule::view_bond_set_dist_ratio_atomic(const int ibond,
                                               const int ierr )
{
  if(ibond<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_bond_set_dist_ratio_atomic , ibond < 0" << endl ;
      cerr << "ibond = " << ibond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibond>=nbond){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_bond_set_dist_ratio_atomic , ibond >= nbond" << endl ;
      cerr << "ibond = " << ibond << " , nbond = " << nbond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  ViewBond vb(bond[ibond].get_vb());
  int a1=bond[ibond].get_iatom1();
  int a2=bond[ibond].get_iatom2();
  vb.set_dist_ratio_1(atom[a1].get_bond_distance());
  vb.set_dist_ratio_2(atom[a2].get_bond_distance());
  copy(vb,ibond);
}

void Molecule::view_bond_set_dist_ratio_atomic( )
{
  for(int i=0;i<get_nbond();i++){
    view_bond_set_dist_ratio_atomic(i);
  }
}

void Molecule::view_bond_set_type_atomic(const int ibond,
					 const int ierr )
{
  if(ibond<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_bond_set_type_atomic , ibond < 0" << endl ;
      cerr << "ibond = " << ibond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibond>=nbond){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_bond_set_type_atomic , ibond >= nbond" << endl ;
      cerr << "ibond = " << ibond << " , nbond = " << nbond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  ViewBond vb(bond[ibond].get_vb());
  int a1=bond[ibond].get_iatom1();
  int a2=bond[ibond].get_iatom2();
  int t1=atom[a1].get_va().get_type();
  int t2=atom[a2].get_va().get_type();
  if(t1==t2){
    vb.set_type(t1);
    copy(vb,ibond);
  }
}

void Molecule::view_bond_set_type_atomic()
{
  for(int i=0;i<get_nbond();i++){
    view_bond_set_type_atomic(i);
  }
}

void Molecule::view_bond_set_width_atomic(const int ibond,
					  const int ierr )
{
  if(ibond<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_bond_set_width_atomic , ibond < 0" << endl ;
      cerr << "ibond = " << ibond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibond>=nbond){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_bond_set_width_atomic , ibond >= nbond" << endl ;
      cerr << "ibond = " << ibond << " , nbond = " << nbond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  ViewBond vb(bond[ibond].get_vb());
  int a1=bond[ibond].get_iatom1();
  int a2=bond[ibond].get_iatom2();
  int w1=atom[a1].get_va().get_bond_w();
  int w2=atom[a2].get_va().get_bond_w();
  if(w1<=w2){
    vb.set_width(w1);
  } else {
    vb.set_width(w2);
  }
  copy(vb,ibond);
}

void Molecule::view_bond_set_width_atomic()
{
  for(int i=0;i<get_nbond();i++){
    view_bond_set_width_atomic(i);
  }
}

void Molecule::view_bond_set_color_atomic(const int ibond,
                                          const int ierr )
{
  if(ibond<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_bond_set_color_atomic , ibond < 0" << endl ;
      cerr << "ibond = " << ibond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibond>=nbond){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::view_bond_set_color_atomic , ibond >= nbond" << endl ;
      cerr << "ibond = " << ibond << " , nbond = " << nbond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  ViewBond vb(bond[ibond].get_vb());
  int a1=bond[ibond].get_iatom1();
  int a2=bond[ibond].get_iatom2();
  ViewAtom va1(atom[a1].get_va());
  ViewAtom va2(atom[a2].get_va());
  vb.set_color_R1(va1.get_color_R());
  vb.set_color_G1(va1.get_color_G());
  vb.set_color_B1(va1.get_color_B());
  vb.set_color_R2(va2.get_color_R());
  vb.set_color_G2(va2.get_color_G());
  vb.set_color_B2(va2.get_color_B());
  copy(vb,ibond);
}

void Molecule::view_bond_set_color_atomic( )
{
  for(int i=0;i<get_nbond();i++){
    view_bond_set_color_atomic(i);
  }
}

void Molecule::view_bond_set_color( const int    ibond,
                                    const double R    ,
                                    const double G    ,
                                    const double B    )
{
  bond[ibond].set_color(R,G,B);
}

void Molecule::view_bond_set_color_fragment()
{
  double c_r[12];
  double c_g[12];
  double c_b[12];
  c_r[  0]=1.00; c_g[  0]=1.00; c_b[  0]=1.00;
  c_r[  1]=1.00; c_g[  1]=1.00; c_b[  1]=1.00;
  c_r[  2]=0.85; c_g[  2]=1.00; c_b[  2]=1.00;
  c_r[  3]=0.80; c_g[  3]=0.50; c_b[  3]=1.00;
  c_r[  4]=0.76; c_g[  4]=1.00; c_b[  4]=0.00;
  c_r[  5]=1.00; c_g[  5]=0.71; c_b[  5]=0.71;
  c_r[  6]=0.56; c_g[  6]=0.56; c_b[  6]=0.56;
  c_r[  7]=0.19; c_g[  7]=0.31; c_b[  7]=0.97;
  c_r[  8]=1.00; c_g[  8]=0.05; c_b[  8]=0.05;
  c_r[  9]=0.56; c_g[  9]=0.88; c_b[  9]=0.31;
  c_r[ 10]=0.70; c_g[ 10]=0.89; c_b[ 10]=0.96;
  c_r[ 11]=0.67; c_g[ 11]=0.36; c_b[ 11]=0.95;
  int a1,a2,if1,if2,ic1,ic2,nfl1,nfl2,chk;
  int * list = new int[natom];
  get_list_atom_to_fragment(list);
  for(int i=0;i<nbond;i++){
    ViewBond vb(bond[i].get_vb());
    a1=bond[i].get_iatom1();
    a2=bond[i].get_iatom2();
    if1=list[a1];
    if2=list[a2];
    if(if1==if2){
      vb.set_dist_ratio_1(1.0);
      vb.set_dist_ratio_2(1.0);
    }
    if(if1!=if2){
      chk=0;
      nfl1=0;
      nfl2=0;
      if(if1>=0) nfl1=fragment[if1].get_n_link();
      if(if2>=0) nfl2=fragment[if2].get_n_link();
      for(int i=0;i<nfl1;i++){
        if(fragment[if1].get_atom_index_link1(i)==a2) chk=1;
      }
      for(int i=0;i<nfl2;i++){
        if(fragment[if2].get_atom_index_link1(i)==a1) chk=2;
      }
      if(chk==0) {
        vb.set_dist_ratio_1(0.0);
        vb.set_dist_ratio_2(1.0);
      }
      if(chk==1) {
        vb.set_dist_ratio_1(8.0);
        vb.set_dist_ratio_2(2.0);
      }
      if(chk==2) {     
        vb.set_dist_ratio_1(2.0);
        vb.set_dist_ratio_2(8.0);
      }
    }
    ic1=(if1%10)+2;
    ic2=(if2%10)+2;
    vb.set_color_R1(c_r[ic1]);
    vb.set_color_G1(c_g[ic1]);
    vb.set_color_B1(c_b[ic1]);
    vb.set_color_R2(c_r[ic2]);
    vb.set_color_G2(c_g[ic2]);
    vb.set_color_B2(c_b[ic2]);
    copy(vb,i);
  }
  delete [] list;
}

void Molecule::view_bond_set_color_fragment_H2O ( const double cr ,
                                                  const double cg ,
                                                  const double cb )
{
  int * list = new int [natom];
  int if1,if2;
  get_list_atom_to_fragment(list);
  for(int i=0;i<nbond;i++){
    if1=list[bond[i].get_iatom1()];
    if2=list[bond[i].get_iatom2()];
    if(if1==if2){
      if(if1>=0){
        if(!(chk_fragment_H2O(if1)==0)){
          ViewBond vb(bond[i].get_vb());
          vb.set_dist_ratio_1(1.0);
          vb.set_dist_ratio_2(1.0);
          vb.set_color_R1(cr);
          vb.set_color_G1(cg);
          vb.set_color_B1(cb);
          vb.set_color_R2(cr);
          vb.set_color_G2(cg);
          vb.set_color_B2(cb);
          copy(vb,i);
        }
      }
    }
  }
  delete [] list;
}

void Molecule::view_bond_set_color_inter_fragment ( const double cr ,
                                                    const double cg ,
                                                    const double cb )
{
  int * list = new int [natom];
  int if1,if2;
  get_list_atom_to_fragment(list);
  for(int i=0;i<nbond;i++){
    if1=list[bond[i].get_iatom1()];
    if2=list[bond[i].get_iatom2()];
    if(if1!=if2){
      ViewBond vb(bond[i].get_vb());
      vb.set_dist_ratio_1(1.0);
      vb.set_dist_ratio_2(1.0);
      vb.set_color_R1(cr);
      vb.set_color_G1(cg);
      vb.set_color_B1(cb);
      vb.set_color_R2(cr);
      vb.set_color_G2(cg);
      vb.set_color_B2(cb);
      copy(vb,i);
    }
  }
  delete [] list ;
}

void Molecule::view_bond_set_color_side_chain(const double cr,
                                              const double cg,
                                              const double cb)
{
  int a1_type_bio1,a2_type_bio1;
  ViewBond vb;
  for(int i=0;i<nbond;i++){
    vb.copy(bond[i].get_vb());
    a1_type_bio1=atom[bond[i].get_iatom1()].get_bio1();
    a2_type_bio1=atom[bond[i].get_iatom2()].get_bio1();
    if((100<=a1_type_bio1 && a1_type_bio1<=2100) || a1_type_bio1==-999){
      vb.set_color_R1(cr);
      vb.set_color_G1(cg);
      vb.set_color_B1(cb);
    }
    if((100<=a2_type_bio1 && a2_type_bio1<=2100) || a2_type_bio1==-999){
      vb.set_color_R2(cr);
      vb.set_color_G2(cg);
      vb.set_color_B2(cb);
    }
    bond[i].copy(vb);
  }
}

void Molecule::view_bond_set_color_main_chain(const double cr,
                                              const double cg,
                                              const double cb)
{
  int a1_type_bio1,a2_type_bio1;
  ViewBond vb;
  for(int i=0;i<nbond;i++){
    vb.copy(bond[i].get_vb());
    a1_type_bio1=atom[bond[i].get_iatom1()].get_bio1();
    a2_type_bio1=atom[bond[i].get_iatom2()].get_bio1();
    if(0<=a1_type_bio1 && a1_type_bio1<=10){
      vb.set_color_R1(cr);
      vb.set_color_G1(cg);
      vb.set_color_B1(cb);
    }
    if(0<=a2_type_bio1 && a2_type_bio1<=10){
      vb.set_color_R2(cr);
      vb.set_color_G2(cg);
      vb.set_color_B2(cb);
    }
    bond[i].copy(vb);    
  }
}

void Molecule::bond_gl_line( const int ibond,
			     float * w  ,
			     float * x1 , float * y1 , float * z1 ,
			     float * x2 , float * y2 , float * z2 ,
			     float * x3 , float * y3 , float * z3 ,
			     float * cr1, float * cg1, float * cb1,
			     float * cr2, float * cg2, float * cb2,
			     const int ierr) const
{
  ViewBond  vb = bond[ibond].get_vb();
  double    d1 = vb.get_dist_ratio_1();
  double    d2 = vb.get_dist_ratio_2();
  Dvector3D r1 = atom[bond[ibond].get_iatom1()].get_R();
  Dvector3D r2 = atom[bond[ibond].get_iatom2()].get_R();
  Dvector3D r3 = r1+((r2-r1)*(d1/(d1+d2)));
  *w   = (float)vb.get_width();
  *cr1 = (float)vb.get_color_R1();
  *cg1 = (float)vb.get_color_G1();
  *cb1 = (float)vb.get_color_B1();
  *cr2 = (float)vb.get_color_R2();
  *cg2 = (float)vb.get_color_G2();
  *cb2 = (float)vb.get_color_B2();
  *x1  = (float)r1.get_x();
  *y1  = (float)r1.get_y();
  *z1  = (float)r1.get_z();
  *x2  = (float)r2.get_x();
  *y2  = (float)r2.get_y();
  *z2  = (float)r2.get_z();
  *x3  = (float)r3.get_x();
  *y3  = (float)r3.get_y();
  *z3  = (float)r3.get_z();
}

void Molecule::bond_gl_cylinder( const int ibond,
				 float * w   , float * ang , 
				 float * l1  , float * l2  ,
				 float * x13c, float * y13c, float * z13c,
				 float * x23c, float * y23c, float * z23c,
				 float * nx  , float * ny  , float * nz  ,
				 float * cr1 , float * cg1 , float * cb1 ,
				 float * cr2 , float * cg2 , float * cb2 ,
				 const int ierr)const
{
  ViewBond  vb    = bond[ibond].get_vb();
  double    width = vb.get_width();
  double    d1    = vb.get_dist_ratio_1();
  double    d2    = vb.get_dist_ratio_2();
  Dvector3D r1    = atom[bond[ibond].get_iatom1()].get_R();
  Dvector3D r2    = atom[bond[ibond].get_iatom2()].get_R();
  Dvector3D r3    = r1+((r2-r1)*(d1/(d1+d2)));
  Dvector3D r13   = r3-r1;
  Dvector3D r23   = r3-r2;
  Dvector3D r13c  = r1+(r13/2.0);
  Dvector3D r23c  = r2+(r23/2.0);
  Dvector3D n     = Dvector3D::Vproduct(Dvector3D(0,0,1),r13);
  double    angle = Dvector3D::angle(Dvector3D(0,0,1),r13)*180/3.141592;
  *w    = (float)width*0.1;
  *ang  = (float)angle;
  *x13c = (float)r13c.get_x();
  *y13c = (float)r13c.get_y();
  *z13c = (float)r13c.get_z();
  *x23c = (float)r23c.get_x();
  *y23c = (float)r23c.get_y();
  *z23c = (float)r23c.get_z();
  *nx   = (float)n.get_x();
  *ny   = (float)n.get_y();
  *nz   = (float)n.get_z();
  *l1   = (float)Dvector3D::norm(r13);
  *l2   = (float)Dvector3D::norm(r23);
  *cr1  = (float)vb.get_color_R1();
  *cg1  = (float)vb.get_color_G1();
  *cb1  = (float)vb.get_color_B1();
  *cr2  = (float)vb.get_color_R2();
  *cg2  = (float)vb.get_color_G2();
  *cb2  = (float)vb.get_color_B2();
}
