
#include "Molecule.h"

Molecule Molecule::over_lattice( const int an1, const int an2,
                                 const int bn1, const int bn2,
                                 const int cn1, const int cn2,
                                 double const a,     double const b,    double const c,
                                 double const alpha, double const beta, double const gamma) {

  Molecule mtmp;
  Dvector3D axis_a, axis_b, axis_c, tmptmp;
  
  double tmp1[3],tmp2[3];
  Dvector3D tmp_axis[3];
  Atom atom_;

  int ibuf[16];
  
  tmptmp.set_xyz(0.5, 0.5, 0.5);
  tmp1[0]=a;     tmp1[1]=b;    tmp1[2]=c;
  tmp2[0]=alpha; tmp2[1]=beta; tmp2[2]=gamma;
  Dvector3D::LatticeToXYZ(tmptmp, tmp1, tmp2, tmp_axis);
  axis_a.copy(tmp_axis[0]);
  axis_b.copy(tmp_axis[1]);
  axis_c.copy(tmp_axis[2]);
  for(int l=0 ; l<natom;l++) {
    atom_.copy(atom[l]);
    tmptmp.copy(atom_.get_R());
    atom_.set_R(tmptmp);
    mtmp.add_atom(atom_);
  }
  for(int i=an1 ; i<=an2; i++){
    for(int j=bn1 ; j<=bn2; j++) {
      for(int k=cn1 ; k<=cn2; k++) {
        if(i!=0 || j!=0 || k!=0) {
          for(int l=0 ; l<natom;l++) {
            atom_.copy(atom[l]);
            tmptmp.copy(atom_.get_R());
            tmptmp=tmptmp+axis_a*i+axis_b*j+axis_c*k;
	    // same check
	    if(mtmp.pick_up_atom_pos(tmptmp,ibuf)==0){
	      atom_.set_R(tmptmp);
	      mtmp.add_atom(atom_);
	    }
          }
        }
      }
    }
  }
  return mtmp;
}

int Molecule::space_group ( const int igroup, const Dvector3D& r, Dvector3D* symmR ) {

  int num=0;
  double x,y,z,xx,yy,zz;

  x=r.get_x();
  y=r.get_y();
  z=r.get_z();
  
  if (igroup==1) {

    // 1. x    , y    , z

    symmR[0].set_xyz( x    , y    , z    ); space_group_trans_0_1(symmR[0]); num++;
    return num;
  } 
  else if (igroup==2) {

    // 1. x    , y    , z
    // 2.-x    ,-y    ,-z

    symmR[0].set_xyz( x    , y    , z    ); space_group_trans_0_1(symmR[0]); num++;
    symmR[1].set_xyz(-x    ,-y    ,-z    ); space_group_trans_0_1(symmR[1]); num++;
    return num;
  } 
  else if (igroup==14) {

    // 1.  x    , y    , z
    // 2. -x    , y+0.5,-z+0.5
    // 3. inv(1)
    // 4. inv(2)

    symmR[0].set_xyz( x    , y    , z    ); space_group_trans_0_1(symmR[0]); num++;
    symmR[1].set_xyz(-x    , y+0.5,-z+0.5); space_group_trans_0_1(symmR[1]); num++;
    symmR[2].copy(space_group_inv(symmR[0])); num++;
    symmR[3].copy(space_group_inv(symmR[1])); num++;
    return num;
  }
  else if (igroup==62) {

    // 1.  x    , y    ,     z
    // 2. -x+0.5,-y    , z+0.5
    // 3. -x    , y+0.5,-z
    // 4.  x+0.5,-y+0.5,-z+0.5
    // 5-8. inv(1-4)

    symmR[ 0].set_xyz( x    , y    , z    ); space_group_trans_0_1(symmR[ 0]); num++;
    symmR[ 1].set_xyz(-x+0.5,-y    , z+0.5); space_group_trans_0_1(symmR[ 1]); num++;
    symmR[ 2].set_xyz(-x    , y+0.5,-z    ); space_group_trans_0_1(symmR[ 2]); num++;
    symmR[ 3].set_xyz( x+0.5,-y+0.5,-z+0.5); space_group_trans_0_1(symmR[ 3]); num++;
    symmR[ 4].copy(space_group_inv(symmR[ 0])); num++;
    symmR[ 5].copy(space_group_inv(symmR[ 1])); num++;
    symmR[ 6].copy(space_group_inv(symmR[ 2])); num++;
    symmR[ 7].copy(space_group_inv(symmR[ 3])); num++;
    return num;
  }
  else if (igroup==119) {

    // 1.  x    , y    ,     z
    // 2. -x    ,-y    ,     z
    // 3.  y    ,-x    ,    -z
    // 4. -y    , x    ,    -z
    // 5.  x    ,-y    ,     z
    // 6. -x    , y    ,     z
    // 7.  y    , x    ,    -z
    // 8. -y    ,-x    ,    -z
    // 9-16 . [1-8]+(+0.5,+0.5,+0.5)

    symmR[ 0].set_xyz( x    , y    , z    ); space_group_trans_0_1(symmR[ 0]); num++;
    symmR[ 1].set_xyz(-x    ,-y    , z    ); space_group_trans_0_1(symmR[ 1]); num++;
    symmR[ 2].set_xyz( y    ,-x    ,-z    ); space_group_trans_0_1(symmR[ 2]); num++;
    symmR[ 3].set_xyz(-y    , x    ,-z    ); space_group_trans_0_1(symmR[ 3]); num++;
    symmR[ 4].set_xyz( x    ,-y    , z    ); space_group_trans_0_1(symmR[ 4]); num++;
    symmR[ 5].set_xyz(-x    , y    , z    ); space_group_trans_0_1(symmR[ 5]); num++;
    symmR[ 6].set_xyz( y    , x    ,-z    ); space_group_trans_0_1(symmR[ 6]); num++;
    symmR[ 7].set_xyz(-y    ,-x    ,-z    ); space_group_trans_0_1(symmR[ 7]); num++;
    symmR[ 8].set_xyz( x+0.5, y+0.5, z+0.5); space_group_trans_0_1(symmR[ 8]); num++;
    symmR[ 9].set_xyz(-x+0.5,-y+0.5, z+0.5); space_group_trans_0_1(symmR[ 9]); num++;
    symmR[10].set_xyz( y+0.5,-x+0.5,-z+0.5); space_group_trans_0_1(symmR[10]); num++;
    symmR[11].set_xyz(-y+0.5, x+0.5,-z+0.5); space_group_trans_0_1(symmR[11]); num++;
    symmR[12].set_xyz( x+0.5,-y+0.5, z+0.5); space_group_trans_0_1(symmR[12]); num++;
    symmR[13].set_xyz(-x+0.5, y+0.5, z+0.5); space_group_trans_0_1(symmR[13]); num++;
    symmR[14].set_xyz( y+0.5, x+0.5,-z+0.5); space_group_trans_0_1(symmR[14]); num++;
    symmR[15].set_xyz(-y+0.5,-x+0.5,-z+0.5); space_group_trans_0_1(symmR[15]); num++;
    return num;
  }
  else if (igroup==167) {

    //x      ,y      ,    z
    //-y     ,x-y    ,    z
    //-x+y   ,-x     ,    z
    //-y     ,-x     ,1/2+z
    //x      ,x-y    ,1/2+z
    //-x+y   ,y      ,1/2+z
    //-x     ,-y     ,   -z
    //y      ,-x+y   ,   -z
    //x-y    ,x      ,   -z
    //y      ,x      ,1/2-z
    //-x     ,-x+y   ,1/2-z
    //x-y    ,-y     ,1/2-z

    //2/3+x  ,1/3+y  ,1/3+z
    //2/3-y  ,1/3+x-y,1/3+z
    //2/3-x+y,1/3-x  ,1/3+z
    //2/3-y  ,1/3-x  ,5/6+z
    //2/3+x  ,1/3+x-y,5/6+z
    //2/3-x+y,1/3+y  ,5/6+z
    //2/3-x  ,1/3-y  ,1/3-z
    //2/3+y  ,1/3-x+y,1/3-z
    //2/3+x-y,1/3+x  ,1/3-z
    //2/3+y  ,1/3+x  ,5/6-z
    //2/3-x  ,1/3-x+y,5/6-z
    //2/3+x-y,1/3-y  ,5/6-z

    //1/3+x  ,2/3+y  ,2/3+z
    //1/3-y  ,2/3+x-y,2/3+z
    //1/3-x+y,2/3-x  ,2/3+z
    //1/3-y  ,2/3-x  ,   +z
    //1/3+x  ,2/3+x-y,   +z
    //1/3-x+y,2/3+y  ,   +z
    //1/3-x  ,2/3-y  ,2/3-z
    //1/3+y  ,2/3-x+y,2/3-z
    //1/3+x-y,2/3+x  ,2/3-z
    //1/3+y  ,2/3+x  ,   -z
    //1/3-x  ,2/3-x+y,   -z
    //1/3+x-y,2/3-y  ,   -z

    xx=0.0; yy=0.0; zz=0.0;
    symmR[ 0].set_xyz( x  +xx, y  +yy, z    +zz); space_group_trans_0_1(symmR[ 0]); num++;
    symmR[ 1].set_xyz(-y  +xx, x-y+yy, z    +zz); space_group_trans_0_1(symmR[ 1]); num++;
    symmR[ 2].set_xyz(-x+y+xx,-x  +yy, z    +zz); space_group_trans_0_1(symmR[ 2]); num++;
    symmR[ 3].set_xyz(-y  +xx,-x  +yy, z+0.5+zz); space_group_trans_0_1(symmR[ 3]); num++;
    symmR[ 4].set_xyz( x  +xx, x-y+yy, z+0.5+zz); space_group_trans_0_1(symmR[ 4]); num++;
    symmR[ 5].set_xyz(-x+y+xx, y  +yy, z+0.5+zz); space_group_trans_0_1(symmR[ 5]); num++;
    symmR[ 6].set_xyz(-x  +xx,-y  +yy,-z    +zz); space_group_trans_0_1(symmR[ 6]); num++;
    symmR[ 7].set_xyz( y  +xx,-x+y+yy,-z    +zz); space_group_trans_0_1(symmR[ 7]); num++;
    symmR[ 8].set_xyz( x-y+xx, x  +yy,-z    +zz); space_group_trans_0_1(symmR[ 8]); num++;
    symmR[ 9].set_xyz( y  +xx, x  +yy,-z+0.5+zz); space_group_trans_0_1(symmR[ 9]); num++;
    symmR[10].set_xyz(-x  +xx,-x+y+yy,-z+0.5+zz); space_group_trans_0_1(symmR[10]); num++;
    symmR[11].set_xyz( x-y+xx,-y  +yy,-z+0.5+zz); space_group_trans_0_1(symmR[11]); num++;
    xx=2.0/3.0; yy=1.0/3.0; zz=1.0/3.0;
    symmR[12].set_xyz( x  +xx, y  +yy, z    +zz); space_group_trans_0_1(symmR[12]); num++;
    symmR[13].set_xyz(-y  +xx, x-y+yy, z    +zz); space_group_trans_0_1(symmR[13]); num++;
    symmR[14].set_xyz(-x+y+xx,-x  +yy, z    +zz); space_group_trans_0_1(symmR[14]); num++;
    symmR[15].set_xyz(-y  +xx,-x  +yy, z+0.5+zz); space_group_trans_0_1(symmR[15]); num++;
    symmR[16].set_xyz( x  +xx, x-y+yy, z+0.5+zz); space_group_trans_0_1(symmR[16]); num++;
    symmR[17].set_xyz(-x+y+xx, y  +yy, z+0.5+zz); space_group_trans_0_1(symmR[17]); num++;
    symmR[18].set_xyz(-x  +xx,-y  +yy,-z    +zz); space_group_trans_0_1(symmR[18]); num++;
    symmR[19].set_xyz( y  +xx,-x+y+yy,-z    +zz); space_group_trans_0_1(symmR[19]); num++;
    symmR[20].set_xyz( x-y+xx, x  +yy,-z    +zz); space_group_trans_0_1(symmR[20]); num++;
    symmR[21].set_xyz( y  +xx, x  +yy,-z+0.5+zz); space_group_trans_0_1(symmR[21]); num++;
    symmR[22].set_xyz(-x  +xx,-x+y+yy,-z+0.5+zz); space_group_trans_0_1(symmR[22]); num++;
    symmR[23].set_xyz( x-y+xx,-y  +yy,-z+0.5+zz); space_group_trans_0_1(symmR[23]); num++;
    xx=1.0/3.0; yy=2.0/3.0; zz=2.0/3.0;
    symmR[24].set_xyz( x  +xx, y  +yy, z    +zz); space_group_trans_0_1(symmR[24]); num++;
    symmR[25].set_xyz(-y  +xx, x-y+yy, z    +zz); space_group_trans_0_1(symmR[25]); num++;
    symmR[26].set_xyz(-x+y+xx,-x  +yy, z    +zz); space_group_trans_0_1(symmR[26]); num++;
    symmR[27].set_xyz(-y  +xx,-x  +yy, z       ); space_group_trans_0_1(symmR[27]); num++;
    symmR[28].set_xyz( x  +xx, x-y+yy, z       ); space_group_trans_0_1(symmR[28]); num++;
    symmR[29].set_xyz(-x+y+xx, y  +yy, z       ); space_group_trans_0_1(symmR[29]); num++;
    symmR[30].set_xyz(-x  +xx,-y  +yy,-z    +zz); space_group_trans_0_1(symmR[30]); num++;
    symmR[31].set_xyz( y  +xx,-x+y+yy,-z    +zz); space_group_trans_0_1(symmR[31]); num++;
    symmR[32].set_xyz( x-y+xx, x  +yy,-z    +zz); space_group_trans_0_1(symmR[32]); num++;
    symmR[33].set_xyz( y  +xx, x  +yy,-z       ); space_group_trans_0_1(symmR[33]); num++;
    symmR[34].set_xyz(-x  +xx,-x+y+yy,-z       ); space_group_trans_0_1(symmR[34]); num++;
    symmR[35].set_xyz( x-y+xx,-y  +yy,-z       ); space_group_trans_0_1(symmR[35]); num++;
    return num;
  }
  return -1;
}

void Molecule::space_group_trans_0_1(Dvector3D& r) {

  double dtmp;
  
  if(r.get_x() < -0.000000001 || 1.000000001 < r.get_x()) {
    dtmp=r.get_x();
    while(dtmp < -0.000000001 || 1.000000001 < dtmp) {
      if(dtmp<-0.000000001) {dtmp=dtmp+1.0;}
      if(dtmp> 1.000000001) {dtmp=dtmp-1.0;}
    }
    r.set_x(dtmp);
  }
  if(r.get_y() < -0.000000001 || 1.000000001 < r.get_y()) {
    dtmp=r.get_y();
    while(dtmp < -0.000000001 || 1.000000001 < dtmp) {
      if(dtmp<-0.000000001) {dtmp=dtmp+1.0;}
      if(dtmp> 1.000000001) {dtmp=dtmp-1.0;}
    }
    r.set_y(dtmp);
  }
  if(r.get_z() < -0.000000001 || 1.000000001 < r.get_z()) {
    dtmp=r.get_z();
    while(dtmp < -0.000000001 || 1.000000001 < dtmp) {
      if(dtmp<-0.000000001) {dtmp=dtmp+1.0;}
      if(dtmp> 1.000000001) {dtmp=dtmp-1.0;}
    }
    r.set_z(dtmp);
  }
}

Dvector3D Molecule::space_group_inv(const Dvector3D& r) {

  Dvector3D tmp;

  tmp.set_x(0.5-(r.get_x()-0.5));
  tmp.set_y(0.5-(r.get_y()-0.5));
  tmp.set_z(0.5-(r.get_z()-0.5));
  return tmp;
}
