
#include <math.h>
#include <cstdlib>
#include "Molecule.h"
#include "Dihedral.h"
#include "ForceField.h"

void Molecule::clear_dihedral()
{
  if(ndihedral_buff==0){
    return;
  }
  for(int i=(ndihedral-1);i>=0;i--){
    pick_up_dihedral(i);
  }
  delete [] dihedral;
  ndihedral_buff=0;
}

void Molecule::create_dihedral(const int n)
{
  clear_dihedral();
  if(n==0) return;
  dihedral = new Dihedral [n];
  ndihedral_buff=n;
}

void Molecule::resize_dihedral(const int n)
{
  int   tmp_ndihedral=ndihedral;
  Dihedral* tmp_dihedral;
  tmp_dihedral = new Dihedral [tmp_ndihedral+1];
  for(int i=0;i<tmp_ndihedral;i++){
    tmp_dihedral[i].copy(dihedral[i]);
  }
  create_dihedral(n);
  if(n>tmp_ndihedral){
    for(int i=0;i<tmp_ndihedral;i++){
      push_back_dihedral(tmp_dihedral[i]);
    } 
  } else {
    for(int i=0;i<n;i++){
      push_back_dihedral(tmp_dihedral[i]);
    }
  }
  delete [] tmp_dihedral;
}

// private method //
void Molecule::push_back_dihedral( const Dihedral& di,
				   const int   buff_increase)
{
  push_back_dihedral(di.get_iatom1(),
		     di.get_iatom2(),
		     di.get_iatom3(),
		     di.get_iatom4(),
		     buff_increase );
}

// private method //
void Molecule::push_back_dihedral( const int iatom1, 
				   const int iatom2,
				   const int iatom3,
				   const int iatom4,
				   const int buff_increase,
				   const int ierr)
{
  if(ndihedral==ndihedral_buff){
    resize_dihedral(ndihedral+buff_increase);
  }
  dihedral[ndihedral].set_iatom1(iatom1);
  dihedral[ndihedral].set_iatom2(iatom2);
  dihedral[ndihedral].set_iatom3(iatom3);
  dihedral[ndihedral].set_iatom4(iatom4);
  ndihedral++;
}

// private method //
Dihedral Molecule::pick_up_dihedral ( const int idihedral,
				      const int ierr )
{
  Dihedral tmp1(dihedral[idihedral]);
  for(int i=idihedral;i<ndihedral-1;i++){
    dihedral[i].copy(dihedral[i+1]);
  }
  ndihedral--;
  return tmp1;
}

int Molecule::get_ndihedral( ) const
{
  return ndihedral;
}

int Molecule::get_dihedral_iatom1( const int idihedral ,
				   const int ierr      ) const
{
  return dihedral[idihedral].get_iatom1();
}

int Molecule::get_dihedral_iatom2( const int idihedral ,
				   const int ierr      ) const
{
  return dihedral[idihedral].get_iatom2();
}

int Molecule::get_dihedral_iatom3( const int idihedral ,
				   const int ierr      ) const
{
  return dihedral[idihedral].get_iatom3();
}

int Molecule::get_dihedral_iatom4( const int idihedral ,
				   const int ierr      ) const
{
  return dihedral[idihedral].get_iatom4();
}

int Molecule::get_dihedral_chk_improper( const int        idihedral ,
					 const int        ierr      ) const
{
  return dihedral[idihedral].get_chk_improper();
}

void Molecule::add_dihedral_auto( )
{
  int ia1,ia2,ib1,ib2,n1,n2,dih1,dih2,dih3,dih4;
  int buf1[128];
  int buf2[128];
  clear_dihedral();
  for(int i=0;i<nbond;i++){
    ia1=bond[i].get_iatom1();
    ia2=bond[i].get_iatom2();
    (*this).get_bond_bond(i,&n1,&n2,buf1,buf2);
    for(int j=0;j<n1;j++){
      for(int k=0;k<n2;k++){
	ib1=bond[buf1[j]].get_iatom1();
	ib2=bond[buf1[j]].get_iatom2();
	if(ib1==ia1){dih1=ib2; dih2=ia1;}
	if(ib2==ia1){dih1=ib1; dih2=ia1;}
	ib1=bond[buf2[k]].get_iatom1();
	ib2=bond[buf2[k]].get_iatom2();
	if(ib1==ia2){dih3=ia2; dih4=ib2;}
	if(ib2==ia2){dih3=ia2; dih4=ib1;}
	if(dih1 != dih4){
	  push_back_dihedral(dih1,dih2,dih3,dih4,nbond*9+1);
	}
      }
    }
  }
}

void Molecule::add_dihedral_amber_improper()
{
  int atom1,atom2,atom3,atom4,n,rt,tc,tn,res_nterm,res_cterm,itmp,ibio,natom_bio,aidx,atyp,his1814,his1816,his;

  int a1[32]; // a1[]--->atom_type
  int a2[32]; // a2[]--->atom_type
  int a3[32]; // a3[]--->atom_type
  int a4[32]; // a4[]--->atom_type
  int b1[32]; // b1[]--->residue( 0: this residue, +1: next residue, -1: previous residue)
  int b2[32]; // b2[]--->residue( 0: this residue, +1: next residue, -1: previous residue)
  int b3[32]; // b3[]--->residue( 0: this residue, +1: next residue, -1: previous residue)
  int b4[32]; // b4[]--->residue( 0: this residue, +1: next residue, -1: previous residue)

  for(int i=0;i<nbiounit;i++){

    rt=biounit[i].get_type();
    tc=biounit[i].get_terminal_c();
    tn=biounit[i].get_terminal_n();
    res_cterm=-1;
    res_nterm=-1;
    itmp=biounit[i].get_link_unit_c();if(itmp>=0){res_cterm=atom[itmp].get_bio3();}
    itmp=biounit[i].get_link_unit_n();if(itmp>=0){res_nterm=atom[itmp].get_bio3();}
    
    for(int j=0;j<32;j++){
      b1[j]=i;
      b2[j]=i;
      b3[j]=i;
      b4[j]=i;
    }

    if(rt==1800){
      his1814=biounit[i].get_h1814();
      his1816=biounit[i].get_h1816();
      if     (his1814>=0 && his1816< 0) {his=0;} // HID
      else if(his1814 <0 && his1816>=0) {his=1;} // HIE
      else if(his1814>=0 && his1816>=0) {his=2;} // HIP
      else {
        cout << "error in his check!! biounit = " << i << endl ;
        exit(1);
      }
    }

    /*---------------------------------------*/
    /*-- get improper of amber force field --*/
    /*---------------------------------------*/

    ForceField::amber_force_field_get_improper (rt,tn,tc,his,&n,a1,a2,a3,a4,b1,b2,b3,b4);

    /*---------------------------*/
    /*-- add improper dihedral --*/
    /*---------------------------*/
    
    //cout << rt  << endl ;
    //cout << tn  << endl ;
    //cout << tc  << endl ;
    //cout << his << endl ;
    //cout << n   << endl ;
    //cout << ""  << endl ;

    for(int j=0;j<n;j++){

      //cout << "a1 = " << a1[j] << endl ;
      //cout << "a2 = " << a2[j] << endl ;
      //cout << "a3 = " << a3[j] << endl ;
      //cout << "a4 = " << a4[j] << endl ;
      // atom 1 //

      atom1=-1;
      ibio=b1[j];
      natom_bio=biounit[ibio].get_n_atom_index();
      for(int k=0;k<natom_bio;k++){
	aidx=biounit[ibio].get_atom_index(k);
	atyp=atom[aidx].get_bio1();
	if(atyp==a1[j]){
	  atom1=aidx;
	  break;
	}
      }
      // atom 2 //
      atom2=-1;
      ibio=b2[j];
      natom_bio=biounit[ibio].get_n_atom_index();
      for(int k=0;k<natom_bio;k++){
	aidx=biounit[ibio].get_atom_index(k);
	atyp=atom[aidx].get_bio1();
	if(atyp==a2[j] && aidx!=atom1){
	  atom2=aidx;
	  break;
	}
      }
      // atom 3 //
      atom3=-1;
      ibio=b3[j];
      natom_bio=biounit[ibio].get_n_atom_index();
      for(int k=0;k<natom_bio;k++){
	aidx=biounit[ibio].get_atom_index(k);
	atyp=atom[aidx].get_bio1();
	if(atyp==a3[j] && aidx!=atom1 && aidx!=atom2){
	  atom3=aidx;
	  break;
	}
      }
      // atom 4 //
      atom4=-1;
      ibio=b4[j];
      natom_bio=biounit[ibio].get_n_atom_index();
      for(int k=0;k<natom_bio;k++){
	aidx=biounit[ibio].get_atom_index(k);
	atyp=atom[aidx].get_bio1();
	if(atyp==a4[j] && aidx!=atom1 && aidx!=atom2 && aidx!=atom3){
	  atom4=aidx;
	  break;
	}
      }
      // add dihedral //
      if(atom1>=0 && atom2>=0 && atom3>=0 && atom4>=0){
	push_back_dihedral(atom1,atom2,atom3,atom4);
	dihedral[ndihedral-1].set_improper_on();
      } else {
	cout << "error in add_dihedral_amber_improper!! ibiounit = " << i << endl ;
	cout << "atom1 = " << atom1 << endl ;
	cout << "atom2 = " << atom2 << endl ;
	cout << "atom3 = " << atom3 << endl ;
	cout << "atom4 = " << atom4 << endl ;
      }
    }    
  }
}
