
#include <cstdlib>
#include <cstdio>
#include <math.h>
#include "Molecule.h"
#include "iString.h"


/* private method */
void Molecule::create_fragment(const int n)
{
  clear_fragment();
  if(n==0) return ;
  fragment = new Fragment [n];
  nfragment_buff=n;
}

/* private method */
void Molecule::resize_fragment(const int n)
{
  int tmp_nfragment=nfragment;
  Fragment* tmp_fragment;
  tmp_fragment = new Fragment [tmp_nfragment+1];
  for(int i=0;i<tmp_nfragment;i++){
    tmp_fragment[i].copy(fragment[i]);
  }
  create_fragment(n);
  if(n>tmp_nfragment){
    for(int i=0;i<tmp_nfragment;i++){
      push_back_fragment(tmp_fragment[i]);
    }
  } else {
    for(int i=0;i<n;i++){
      push_back_fragment(tmp_fragment[i]);
    }
  }
  delete [] tmp_fragment ;
}

/* private method */
void Molecule::push_back_fragment ( const Fragment& f, 
                                    const int buff_increase )
{
  if(nfragment==nfragment_buff){
    resize_fragment(nfragment+buff_increase);
  }
  fragment[nfragment].copy(f);
  for(int i=0;i<f.get_n_atom_index();i++){
    atom[f.get_atom_index(i)].set_frg1(nfragment);
  }
  for(int j=0;j<f.get_n_link();j++){
    atom[f.get_atom_index_link1(j)].set_frg2(1);
  }
  nfragment++;
} 

/* private method */
void Molecule::insert_fragment ( const Fragment& f, 
                                 const int index, 
                                 const int buff_increase )
{
  if(nfragment<=index) {
    push_back_fragment(f,buff_increase);
  } else {
    Fragment f_;
    push_back_fragment(f_,buff_increase);
    for(int i=(nfragment-1);i>index;i--){
      fragment[i].copy(fragment[i-1]);
    }
    fragment[index].copy(f);
    for(int i=0;i<f.get_n_atom_index();i++){
      atom[f.get_atom_index(i)].set_frg1(nfragment); // <--- ??? 20150704
    }
    for(int j=0;j<f.get_n_link();j++){
      atom[f.get_atom_index_link1(j)].set_frg2(1);
    }
  }
}

/* private method */
void Molecule::pick_up_fragment ( const int ifrag )
{
  if(ifrag<0){
    cerr << "error : Molecule::pick_up_fragment : ifrag < 0 , ifrag = " << ifrag << endl ;
    exit(1);
  }
  if(ifrag>=nfragment) {
    cerr << "error : Molecule::pick_up_fragment : ifrag >= nfragment" << endl ;
    cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << endl ;
    exit(1);
  }
  for(int i=0;i<fragment[ifrag].get_n_atom_index();i++){
    atom[fragment[ifrag].get_atom_index(i)].set_frg1(-1);
  }
  for(int j=0;j<fragment[ifrag].get_n_link();j++){
    atom[fragment[ifrag].get_atom_index_link1(j)].set_frg2(-1);
  }
  for(int i=ifrag;i<nfragment-1;i++){
    fragment[i].copy(fragment[i+1]);
  }
  nfragment--;
}

void Molecule::clear_fragment()
{
  if(nfragment==0) return ;
  for(int i=(nfragment-1);i>=0;i--){
    pick_up_fragment(i);
  }
  delete [] fragment ;
  nfragment_buff=0;
}

void Molecule::push_back_atom_to_fragment(const int iatom, 
                                          const int ifrag,
                                          const int ierr )
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::push_back_atom_to_fragment : ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment) {
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::push_back_atom_to_fragment : ifrag >= nfragment" << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  fragment[ifrag].push_back_atom_index(iatom);
  fragment[ifrag].push_back_elec(atom[iatom].get_atomic_number());
  atom[iatom].set_frg1(ifrag);
}

void Molecule::push_back_link_to_fragment_ifrag(const int iatom1,
                                                const int iatom2,
                                                const int ifrag ,
                                                const int ierr  )
{
  // iatom1 --> bde (this atom belong to another fragment)
  // iatom2 -->     (this atom belong to this fragment)
  if(iatom1<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::push_back_link_to_fragment_ifrag , iatom1 < 0" << endl ;
      cerr << "iatom1 = " << iatom1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::push_back_link_to_fragment_ifrag , iatom2 < 0" << endl ;
      cerr << "iatom2 = " << iatom2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom1>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Moelcule::push_back_link_to_fragment_ifrag , iatom1 >= natom" << endl ;
      cerr << "iatom1 = " << iatom1 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Moelcule::push_back_link_to_fragment_ifrag , iatom2 >= natom" << endl ;
      cerr << "iatom2 = " << iatom1 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::push_back_link_to_fragment_ifrag : ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::push_back_link_to_fragment_ifrag : ifrag >= nfragment" << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  fragment[ifrag].push_back_atom_index_link(iatom1,iatom2);
  fragment[ifrag].push_back_elec(1);
  fragment[get_ifragment_from_iatom(iatom1)].pick_up_elec(1);
  atom[iatom1].set_frg2(1);
}

void Molecule::push_back_link_to_fragment(const int iatom1,
                                          const int iatom2,
                                          const int ierr  )
{
  // iatom1 --> bde (this atom belong to another fragment)
  // iatom2 -->     (this atom belong to this fragment)
  if(iatom1<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::push_back_link_to_fragment , iatom1 < 0" << endl ;
      cerr << "iatom1 = " << iatom1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::push_back_link_to_fragment , iatom2 < 0" << endl ;
      cerr << "iatom2 = " << iatom2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom1>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Moelcule::push_back_link_to_fragment , iatom1 >= natom" << endl ;
      cerr << "iatom1 = " << iatom1 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom2>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Moelcule::push_back_link_to_fragment , iatom2 >= natom" << endl ;
      cerr << "iatom2 = " << iatom1 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  push_back_link_to_fragment_ifrag(iatom1,iatom2,get_ifragment_from_iatom(iatom2));
}

void Molecule::push_back_elec_to_fragment(const int nelec,
                                          const int ifrag,
                                          const int ierr )
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::push_back_elec_to_fragment : ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::push_back_elec_to_fragment : ifrag >= nfragment" << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  fragment[ifrag].push_back_elec(nelec);
}

void Molecule::pick_up_elec_from_fragment(const int nelec,
                                          const int ifrag,
                                          const int ierr )
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::pick_up_elec_from_fragment : ifrag < 0 " ;
      cerr << "frag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::pick_up_elec_from_fragment : ifrag >= nfragment" << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  fragment[ifrag].pick_up_elec(nelec);
}

int Molecule::get_ifragment_from_iatom(const int iatom,
                                       const int ierr ) const
{
  if(iatom<0){
    if(ierr==0 || ierr==1){
      cerr << "error : molecule::get_ifragment_from_iatom : iatom < 0 " << endl ;
      cerr << "iatom = " << iatom << " this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(iatom>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error : molecule::get_ifragment_from_iatom : iatom >= natom " << endl ;
      cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  for(int i=0;i<nfragment;i++){
    for(int j=0;j<fragment[i].get_n_atom_index();j++){
      if(iatom==fragment[i].get_atom_index(j)){
        return i;
      }
    }
  }
  return -1;
}

int Molecule::get_max_iatom_from_ifragment(const int ifrag) const 
{
  if(ifrag<0){
    cerr << "error in Molecule::get_max_iatom_from_ifragment , ifrag < 0" << endl ;
    cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    exit(1);
  }
  if(ifrag>=nfragment){
    cerr << "error in Molecule::get_max_iatom_from_ifragment , ifrag >= nfragment" << endl ;
    cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    exit(1);
  }
  int imax=fragment[ifrag].get_atom_index(0);
  for(int i=1;i<fragment[ifrag].get_n_atom_index();i++){
    if(imax<fragment[ifrag].get_atom_index(i)) imax=fragment[ifrag].get_atom_index(i);
  }
  return imax;
}

int Molecule::get_min_iatom_from_ifragment(const int ifrag) const 
{
  if(ifrag<0){
    cerr << "error in Molecule::get_min_iatom_from_ifragment , ifrag < 0" << endl ;
    cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    exit(1);
  }
  if(ifrag>=nfragment){
    cerr << "error in Molecule::get_min_iatom_from_ifragment , ifrag >= nfragment" << endl ;
    cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    exit(1);
  }
  int imin=fragment[ifrag].get_atom_index(0);
  for(int i=1;i<fragment[ifrag].get_n_atom_index();i++){
    if(imin>fragment[ifrag].get_atom_index(i)) imin=fragment[ifrag].get_atom_index(i);
  }
  return imin;
}

int Molecule::chk_iatom_in_ifragment(const int iatom,
                                     const int ifrag) const
{
  if(ifrag<0){
    cerr << "error in Molecule::chk_iatom_in_ifragment , ifrag < 0" << endl ;
    cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    exit(1);
  }
  if(ifrag>=nfragment){
    cerr << "error in Molecule::chk_iatom_in_ifragment , ifrag >= nfragment" << endl ;
    cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    exit(1);
  }
  if(iatom<0){
    cerr << "error in Molecule::chk_iatom_in_ifragment , iatom < 0" << endl ;
    cerr << "iatom = " << iatom << " , this = " << this << endl ;
    exit(1);
  }
  if(iatom>=natom){
    cerr << "error in Molecule::chk_iatom_in_ifragment , iatom >= natom" << endl ;
    cerr << "iatom = " << iatom << " , natom = " << natom << " , this = " << this << endl ;
    exit(1);
  }
  for(int i=0;i<fragment[ifrag].get_n_atom_index();i++){
    if(iatom==fragment[ifrag].get_atom_index(i)){
      return i;
    }
  }
  return -1;
}

int Molecule::get_ifragment_from_ibond(const int ibond,
                                       const int ierr ) const
{
  if(ibond<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_ifragment_from_ibond , ibond < 0" << endl ;
      cerr << "ibond = " << ibond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibond>=nbond){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_ifragment_from_ibond , ibond >= nbond " << endl;
      cerr << "ibond = " << ibond << " , nbond = " << nbond << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int ia1=bond[ibond].get_iatom1();
  int ia2=bond[ibond].get_iatom2();
  int if1=get_ifragment_from_iatom(ia1);
  int if2=get_ifragment_from_iatom(ia2);
  if(if1==if2){
    return if1;
  } else {
    // ishi 20100824
    return -1;
  }
}

double Molecule::get_fragment_distance_from_fragment(const int ifrag ,
                                                     const int jfrag ,
                                                     const int ierr  ) const 
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_fragment_distance_from_fragment , ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_fragment_distance_from_fragment , ifrag >= nfragment" << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jfrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_fragment_distance_from_fragment , jfrag < 0" << endl ;
      cerr << "jfrag = " << jfrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jfrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_fragment_distance_from_fragment , jfrag >= nfragment" << endl ;
      cerr << "jfrag = " << jfrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  double min2, dist2;
  int iatom1, iatom2, natom1, natom2;
  min2=99999999.0;
  natom1 = fragment[ifrag].get_n_atom_index();
  natom2 = fragment[jfrag].get_n_atom_index();
  for(int i=0;i<natom1;i++){
    for(int j=0;j<natom2;j++){
      iatom1 = fragment[ifrag].get_atom_index(i);
      iatom2 = fragment[jfrag].get_atom_index(j);
      dist2=Dvector3D::distance2(atom[iatom1].get_R(),atom[iatom2].get_R());
      if(dist2<min2){
        min2=dist2;
      }
    }
  }
  return sqrt(min2);
}

double Molecule::get_fragment_distance_from_fragment(const int  nf1,
                                                     const int* list1,
                                                     const int  nf2,
                                                     const int* list2,
                                                     const int  ierr) const
{
  for(int i=0;i<nf1;i++){
    if(list1[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_fragment_distance_from_fragment , list1[i] < 0" << endl ;
        cerr << "list1[i] = " << list1[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(list1[i]>=nfragment){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_fragment_distance_from_fragment , list1[i] >= nfragment" << endl ;
        cerr << "list1[i] = " << list1[i] << " , nfragment = " << nfragment << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  for(int i=0;i<nf2;i++){
    if(list2[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_fragment_distance_from_fragment , list2[i] < 0" << endl ;
        cerr << "list2[i] = " << list2[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(list1[i]>=nfragment){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_fragment_distance_from_fragment , list2[i] >= nfragment" << endl ;
        cerr << "list2[i] = " << list2[i] << " , nfragment = " << nfragment << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  double min_dist=99999999.0;
  double dist;
  for(int i=0;i<nf1;i++){
    for(int j=0;j<nf2;j++){
      dist=get_fragment_distance_from_fragment(list1[i],list2[j]);
      if(min_dist>dist) min_dist=dist;
    }
  }
  return min_dist;
}

double Molecule::get_fragment_distance_from_linkgroup(const int ifrag , 
                                                      const int jlink ,
                                                      const int ierr  ) const
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_from_linkgroup , ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_from_linkgroup , ifrag >= nfragment" << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jlink<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_from_linkgroup , jlink < 0" << endl ;
      cerr << "jlink = " << jlink << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jlink>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_from_linkgroup , jlink >= nlinkgroup" << endl ;
      cerr << "jlink = " << jlink << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }  
  double min2, dist2;
  int iatom1, iatom2, natom1, natom2;
  min2=99999999.0;
  natom1 = fragment [ifrag].get_n_atom_index();
  natom2 = linkgroup[jlink].get_n_atom_index();
  for(int i=0;i<natom1;i++){
    for(int j=0;j<natom2;j++){
      iatom1 = fragment [ifrag].get_atom_index(i);
      iatom2 = linkgroup[jlink].get_atom_index(j);
      dist2=Dvector3D::distance2(atom[iatom1].get_R(),atom[iatom2].get_R());
      if(dist2<min2){
        min2=dist2;
      }
    }
  }
  return sqrt(min2);
}

double Molecule::get_fragment_distance_from_biounit(const int ifrag , 
                                                    const int jbio  ,
                                                    const int ierr  ) const
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_from_biounit , ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_from_biounit , ifrag >= nfragment" << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jbio<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_from_biounit , jbio < 0" << endl ;
      cerr << "jbio = " << jbio << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jbio>=nbiounit){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_from_biounit , jbio >= nbiounit" << endl ;
      cerr << "jbio = " << jbio << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  double min2, dist2;
  int iatom1, iatom2, natom1, natom2;
  min2=99999999.0;
  natom1 = fragment [ifrag].get_n_atom_index();
  natom2 = biounit  [jbio ].get_n_atom_index();
  for(int i=0;i<natom1;i++){
    for(int j=0;j<natom2;j++){
      iatom1 = fragment[ifrag].get_atom_index(i);
      iatom2 = biounit [jbio ].get_atom_index(j);
      dist2=Dvector3D::distance2(atom[iatom1].get_R(),atom[iatom2].get_R());
      if(dist2<min2){
        min2=dist2;
      }
    }
  }
  return sqrt(min2);
}

double Molecule::get_fragment_distance_SS(const int ifrag ,
                                          const int jfrag ,
                                          const int ierr  ) const
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_SS , ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_SS , ifrag >= nfragment" << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jfrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_SS , jfrag < 0" << endl ;
      cerr << "jfrag = " << jfrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(jfrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_SS , jfrag >= nfragment" << endl ;
      cerr << "jfrag = " << jfrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  double min2, dist2;
  int iatom1, iatom2, natom1, natom2;
  min2=99999999.0;
  natom1 = fragment[ifrag].get_n_atom_index();
  natom2 = fragment[jfrag].get_n_atom_index();
  for(int i=0;i<natom1;i++){
    iatom1=fragment[ifrag].get_atom_index(i);
    if(atom[iatom1].get_atomic_number()==16){
      for(int j=0;j<natom2;j++){
        iatom2=fragment[jfrag].get_atom_index(j);
        if(atom[iatom2].get_atomic_number()==16){
          dist2=Dvector3D::distance2(atom[iatom1].get_R(),atom[iatom2].get_R());
          if(dist2<min2){
            min2=dist2;
          }
        }
      }
    }
  }
  return sqrt(min2);
}

Dvector3D Molecule::get_fragment_center_of_mass( const int ifrag ,
                                                 const int ierr  ) const
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_center_of_mass , ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_center_of_mass , ifrag >= nfragment" << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  double sumMass = get_fragment_total_mass(ifrag);
  Dvector3D sum(0.00,0.00,0.00);
  int iatom;
  for(int i=0;i<fragment[ifrag].get_n_atom_index();i++){
    iatom=fragment[ifrag].get_atom_index(i);
    sum=sum+atom[iatom].get_R()*atom[iatom].get_mass_number();
  }
  if(-0.0000001 < sumMass && sumMass < 0.000001){
    //return innerAxis_origin;
  } else {
    return sum/sumMass;
  }
}

double Molecule::get_fragment_total_mass( const int ifrag ,
                                          const int ierr  ) const
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_total_mass , ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_total_mass , ifrag >= nfragment" << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  double sum=0.0;
  for(int i=0;i<fragment[ifrag].get_n_atom_index();i++){
    sum=sum+atom[fragment[ifrag].get_atom_index(i)].get_mass_number();
  }
  return sum;
}

int Molecule::chk_fragment_H2O(const int ifrag,
                               const int ierr ) const
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::chk_fragment_H2O : ifrag < 0 " << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::chk_fragment_H2O : ifrag >= nfragment " << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  } 
  if(fragment[ifrag].get_n_atom_index()!=3){ 
    return 0;
  }
  int ia1=atom[fragment[ifrag].get_atom_index(0)].get_atomic_number();
  int ia2=atom[fragment[ifrag].get_atom_index(1)].get_atomic_number();
  int ia3=atom[fragment[ifrag].get_atom_index(2)].get_atomic_number();
  if(ia1==8 && ia2==1 && ia3==1) return 1;
  if(ia2==8 && ia3==1 && ia1==1) return 1;
  if(ia3==8 && ia1==1 && ia2==1) return 1;
  return 0;
}

double Molecule::get_fragment_distance_R(const int ifrag    ,
                                         const Dvector3D& r ,
                                         int* iatom         ,
                                         const int ierr     ) const
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_R : ifrag < 0 " << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment_distance_R : ifrag >= nfragment " << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  double min2,dist2;
  int ia,n;
  min2=99999999.0;
  n=fragment[ifrag].get_n_atom_index();
  for(int i=0;i<n;i++){
    ia=fragment[ifrag].get_atom_index(i);
    dist2=Dvector3D::distance2(atom[ia].get_R(),r);
    if(dist2<min2){
      (*iatom)=ia;
      min2=dist2;
    }
  }
  return sqrt(dist2);
}

int Molecule::get_nfragment ( ) const
{
  return nfragment ;
}

Fragment Molecule::get_fragment(const int ifrag,
                                const int ierr ) const
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment , ifrag < 0" << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_fragment , ifrag >= nfragment " << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return fragment[ifrag];
}

int Molecule::get_atom_index_fragment(const int  nfrag,
                                      const int* ifrag,
                                      int*       out  ,
                                      const int  ierr ) const
{
  int n=0;
  for(int j=0;j<nfrag;j++){
    if(ifrag[j]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_atom_index_fragment : ifrag < 0 " << endl ;
        cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(ifrag[j]>=nfragment){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_atom_index_fragment : ifrag > nfragment " << endl ;
        cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    for(int i=0;i<fragment[ifrag[j]].get_n_atom_index();i++){
      out[n]=fragment[ifrag[j]].get_atom_index(i);
      n++;
    }
  }
  return n;
}

int Molecule::get_nelec_fragment ( const int ifrag ,
                                   const int ierr  ) const
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_nelec_fragment : ifrag < 0 " << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_nelec_fragment : ifrag > nfragment " << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return fragment[ifrag].get_n_elec();
}

int Molecule::get_ncharge_fragment ( const int ifrag ,
                                     const int ierr  ) const
{
  if(ifrag<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_ncharge_fragment : ifrag < 0 " << endl ;
      cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_ncharge_fragment : ifrag > nfragment " << endl ;
      cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int sum,iatom;
  sum=-1*fragment[ifrag].get_n_elec();
  for(int i=0;i<fragment[ifrag].get_n_atom_index();i++){
    sum=sum+atom[fragment[ifrag].get_atom_index(i)].get_atomic_number();
  }
  sum=sum+fragment[ifrag].get_n_link();
  for(int i=0;i<nfragment;i++){
    if(i!=ifrag){
      for(int j=0;j<fragment[i].get_n_link();j++){
        iatom=fragment[i].get_atom_index_link1(j);
        for(int k=0;k<fragment[ifrag].get_n_atom_index();k++){
          if(fragment[ifrag].get_atom_index(k)==iatom) sum=sum-1;
        }
      }
    }
  }
  return sum;
}

void Molecule::unit_fragment(const int nf, 
                             const int* in)
{
  int* buff=new int [nf+1];
  for(int i=0;i<nf;i++){
    buff[i]=in[i];
  }
  for(int i=0;i<nf-1;i++){
    int imax=i;
    for(int j=i+1;j<nf;j++){
      if(buff[imax]<buff[j]) imax=j;
    }
    int ibuf=buff[i];
    buff[i]=buff[imax];
    buff[imax]=ibuf;
  }
  unit_fragment(buff[0],buff[1]);
  for(int i=2;i<nf;i++){
    unit_fragment(buff[i-1],buff[i]);
  }
  delete [] buff;
}

void Molecule::unit_fragment(const int ifrag1,
                             const int ifrag2)
{
  Fragment f1,f2;
  if(ifrag1==ifrag2) return ;
  f1.copy(fragment[ifrag1]);
  f2.copy(fragment[ifrag2]);
  if(ifrag1>ifrag2){
    pick_up_fragment(ifrag1);
    pick_up_fragment(ifrag2);
  } else {
    pick_up_fragment(ifrag2);
    pick_up_fragment(ifrag1);
  }
  if(ifrag1<ifrag2) insert_fragment(Fragment::unit_fragment(f1,f2),ifrag1);
  if(ifrag2<ifrag1) insert_fragment(Fragment::unit_fragment(f2,f1),ifrag2);
}

void Molecule::unit_fragment_SS()
{
  int chk,chk_ss;
  while(1){
    chk=-1;
    for(int i=0;i<(get_nfragment()-1);i++){
      if(chk_s_exist_in_fragment(i)>0){
        for(int j=i+1;j<(get_nfragment());j++){
          if(chk_s_exist_in_fragment(j)>0){
            chk_ss=-1;
            for(int k=0;k<fragment[i].get_n_atom_index();k++){
              if(atom[fragment[i].get_atom_index(k)].get_atomic_number()==16){
                for(int l=0;l<fragment[j].get_n_atom_index();l++){
                  if(atom[fragment[j].get_atom_index(l)].get_atomic_number()==16){
                    if(chk_bond_exist(fragment[i].get_atom_index(k),fragment[j].get_atom_index(l))>=0){
                      chk_ss=1;
                    }
                  }
                  if(chk_ss>=0) break;
                }
              }
              if(chk_ss>=0) break;
            }
            if(chk_ss>=0){
              chk=1;
              unit_fragment(i,j);
            }
            if(chk==1) break;
          }
        }
        if(chk==1) break;
      }
    }
    if(chk==-1) break;
  }
}

void Molecule::unit_fragment_solvent_ion(const int atomic,
                                         const double dist)
{
  int* ibuf = new int [get_nfragment()];
  int n;
  for(int i=0;i<get_nfragment();i++){
    if(get_fragment(i).get_n_atom_index( )==1){
      if(get_atom(get_fragment(i).get_atom_index(0)).get_atomic_number()==atomic){
        n=0;
        for(int j=0;j<get_nfragment();j++){
          if(chk_fragment_H2O(j)==1){
            if(get_fragment_distance_from_fragment(i,j)<dist){
              ibuf[n]=j;
              n++;
            }
          }
        }
        for(int j=(n-1);j>=1;j--){
          unit_fragment(ibuf[0],ibuf[j]);
        }
        unit_fragment(i,ibuf[0]);
      }
    }
  }
  delete [] ibuf;
}

void Molecule::make_fragment_from_molecule()
{
  clear_fragment();
  Fragment f;
  for(int i=0;i<natom;i++){
    f.push_back_atom_index(i);
    f.push_back_elec(atom[i].get_atomic_number());
  }  
  push_back_fragment(f,1); 
}

void Molecule::make_fragment_from_linkgroup(const int nlnk,
                                            const int* in,
                                            const int inc)
{
  for(int i=0;i<nlnk;i++){
    clear_fragment_linkgroup(in[i]);
    make_fragment_from_linkgroup(in[i],inc);
  }
}

void Molecule::make_fragment_from_linkgroup( const int igroup,
                                             const int inc)
{
  Fragment f;
  int ia,na,atomic,nH,nO;
  na=linkgroup[igroup].get_n_atom_index();
  for(int i=0;i<na;i++){
    ia=linkgroup[igroup].get_atom_index(i);
    f.push_back_atom_index(ia);
    atomic=atom[ia].get_atomic_number();
    f.push_back_elec(atomic);
  }
  na=f.get_n_atom_index();
  if(na==1){
    ia=f.get_atom_index(0);
    atomic=atom[ia].get_atomic_number();
    if(atomic==11){f.pick_up_elec  (1);} // Na+
    if(atomic==12){f.pick_up_elec  (2);} // Mg2+
    if(atomic==17){f.push_back_elec(1);} // Cl-
    if(atomic==19){f.pick_up_elec  (1);} // K+
  } else {
    nH=0;
    nO=0;
    for(int i=0;i<na;i++){
      ia=f.get_atom_index(i);
      atomic=atom[ia].get_atomic_number();
      if(atomic==1) nH++;
      if(atomic==8) nO++;
    }
    if(na==2 && nH==1 && nO==1){f.push_back_elec(1);}
    if(na==4 && nH==3 && nO==1){f.pick_up_elec  (1);}
  }
  push_back_fragment(f,inc);
}

void Molecule::make_fragment_from_linkgroup_nobio()
{
  for(int i=0;i<get_nlinkgroup();i++){
    if(get_nbiounit_linkgroup(i)==0){
      clear_fragment_linkgroup(i);
    }
  }
  for(int i=0;i<get_nlinkgroup();i++){
    if(get_nbiounit_linkgroup(i)==0){
      make_fragment_from_linkgroup(i,get_nlinkgroup());
    }
  }
}

void Molecule::make_fragment_from_biounit(string& s_out)
{
  char c[1024];
  for(int i=0;i<get_nlinkgroup();i++){
    clear_fragment_linkgroup(i);
    make_fragment_from_biounit_peptide(i);
    make_fragment_from_biounit_dna(i,s_out);
  }
  unit_fragment_SS();
}

void Molecule::make_fragment_from_biounit(const int  nl  ,
                                          const int* in  ,
                                          string&    s_out,
                                          const int  ierr)
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::make_fragment_from_biounit : in[i] < 0 " << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nlinkgroup){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::make_fragment_from_biounit : in[i] >= nlinkgroup " << endl ;
        cerr << "in[i] = " << in[i] << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  for(int i=0;i<nl;i++){
    clear_fragment_linkgroup(in[i]);
    make_fragment_from_biounit_peptide(in[i]);
    make_fragment_from_biounit_dna(in[i],s_out);
  }
  unit_fragment_SS();
}

void Molecule::make_fragment_from_biounit_peptide(const int igroup,
                                                  const int ierr  )
{
  if(igroup<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::make_fragment_from_biounit_cc : igroup < 0 " << endl ;
      cerr << "igroup = " << igroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(igroup>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::make_fragment_from_biounit_cc : igroup >= nlinkgroup " << endl ;
      cerr << "igroup = " << igroup << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int ia,it,ibu,il1,il2,ic0,ic1,type,ibu_nterm,ibu_cterm,ibu_n,ibu_c,ibu_b,cterm_type,nterm_type,count;
  Fragment f;
  BioUnit bu;
  string s;
  ibu_cterm=get_ibiounit_cterm(igroup);
  ibu_nterm=get_ibiounit_nterm(igroup);
  if(ibu_cterm>=0 && ibu_nterm>=0){
    ibu=ibu_nterm;
    count=0;
    while(1){
      if(count>nbiounit){
        break;
      }
      f.clear();
      ibu_n=biounit[ibu].get_link_unit_n();
      ibu_c=biounit[ibu].get_link_unit_c();
      cterm_type=biounit[ibu].get_terminal_c();
      nterm_type=biounit[ibu].get_terminal_n();
      type=biounit[ibu].get_type();
      for(int i=0;i<biounit[ibu].get_n_atom_index();i++){
        ia=biounit[ibu].get_atom_index(i);
        it=biounit[ibu].get_atom_type (i);
        if(it== 0)           {f.push_back_atom_index(ia);f.push_back_elec(6);}
        if(it== 2)           {f.push_back_atom_index(ia);f.push_back_elec(7);}
        if(it== 3)           {f.push_back_atom_index(ia);f.push_back_elec(1);}
        if(it== 8)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for gly
        if(it== 9)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for gly
        if(it== 5)           {f.push_back_atom_index(ia);f.push_back_elec(1);}
        if(it==10)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for nterm
        if(it==11)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for nterm
        if(it==12)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for nterm
        if(it==81)           {f.push_back_atom_index(ia);f.push_back_elec(6);} // only for nterm (-COCH3) // 20130815
        if(it==82)           {f.push_back_atom_index(ia);f.push_back_elec(8);} // only for nterm (-COCH3) // 20130815
        if(it==83)           {f.push_back_atom_index(ia);f.push_back_elec(6);} // only for nterm (-COCH3) // 20130815
        if(it==84)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for nterm (-COCH3) // 20130815
        if(it==85)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for nterm (-COCH3) // 20130815
        if(it==86)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for nterm (-COCH3) // 20130815
        if(it==87)           {f.push_back_atom_index(ia);f.push_back_elec(7);} // only for cterm (-CNCH3) // 20130815
        if(it==88)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for cterm (-CNCH3) // 20130815
        if(it==89)           {f.push_back_atom_index(ia);f.push_back_elec(6);} // only for cterm (-CNCH3) // 20130815
        if(it==90)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for cterm (-CNCH3) // 20130815
        if(it==91)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for cterm (-CNCH3) // 20130815
        if(it==92)           {f.push_back_atom_index(ia);f.push_back_elec(1);} // only for cterm (-CNCH3) // 20130815
        if(it== 1 && ibu_c<0){f.push_back_atom_index(ia);f.push_back_elec(6);}
        if(it== 4 && ibu_c<0){f.push_back_atom_index(ia);f.push_back_elec(8);}
        if(it== 6 && ibu_c<0){f.push_back_atom_index(ia);f.push_back_elec(1);}
        if(it== 7 && ibu_c<0){f.push_back_atom_index(ia);f.push_back_elec(1);}
        if(it>= 100)         {f.push_back_atom_index(ia);f.push_back_elec(atom[ia].get_atomic_number());}
        if(it==-999)         {f.push_back_atom_index(ia);f.push_back_elec(atom[ia].get_atomic_number());}
        if(it==  -1)         {f.push_back_atom_index(ia);f.push_back_elec(atom[ia].get_atomic_number());} // 20130903
      }
      if(ibu_n>=0){
        for(int i=0;i<biounit[ibu_n].get_n_atom_index();i++){
          ia=biounit[ibu_n].get_atom_index(i);
          it=biounit[ibu_n].get_atom_type (i);
          if(it==1){f.push_back_atom_index(ia);f.push_back_elec(6);}
          if(it==4){f.push_back_atom_index(ia);f.push_back_elec(8);}
        }
      }
      if(type==1600   ){f.pick_up_elec  (1);}
      if(type==1700   ){f.pick_up_elec  (1);}
      
      //if(type==1850   ){f.pick_up_elec  (1);} // ishi 20110627 <-- ishiishi
      if(type==1800   ) {
        int chk_hd1=0;
        int chk_he2=0;
        for(int j=0;j<biounit[ibu].get_n_atom_index();j++){
          if(biounit[ibu].get_atom_type(j)==1814){chk_hd1=1;}
          if(biounit[ibu].get_atom_type(j)==1816){chk_he2=1;}
        }
        if(chk_hd1==1 && chk_he2==1){f.pick_up_elec(1);} // ishi 20110627 <-- ishi 20140324
      }
      if(type==1900   ){f.push_back_elec(1);}
      if(type==2000   ){f.push_back_elec(1);}
      if(cterm_type==2){f.push_back_elec(1);}
      if(nterm_type==2){f.pick_up_elec  (1);}
      push_back_fragment(f);
      if(ibu_c<0) break ;
      ibu=ibu_c;
      count++;
    }
    ibu=ibu_nterm;
    count=0;
    while(1){
      if(count>nbiounit){
        break;
      }
      ibu_n=biounit[ibu].get_link_unit_n();
      ibu_c=biounit[ibu].get_link_unit_c();
      if(ibu_n>=0){
        ic0=biounit[ibu_n].get_c0();
        ic1=biounit[ibu_n].get_c1();
        if(ic0>=0 && ic1>=0){
          il1=biounit[ibu_n].get_atom_index(ic0);
          il2=biounit[ibu_n].get_atom_index(ic1);
          push_back_link_to_fragment(il1,il2);
        }
      }
      if(ibu_c<0) break ;
      ibu=ibu_c;
      count++;
    }
  }
}

void Molecule::make_fragment_from_biounit_dna(const int igroup,
                                              string&   s_out,
                                              const int ierr  )
{
  if(igroup<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::make_fragment_from_biounit_dna : igroup < 0 " << endl ;
      cerr << "igroup = " << igroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(igroup>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::make_fragment_from_biounit_dna : igroup >= nlinkgroup " << endl ;
      cerr << "igroup = " << igroup << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int ia,it,ic23,io31,ic25,in26,ic23_,io31_,ic25_,in26_,
    term3_type,term5_type,ibu_5,ibu_3,ibu_b,ibu_3term,ibu_5term,count,ibu;
  char c[1024];
  Fragment f;
  BioUnit bu;
  ibu_3term=get_ibiounit_3term(igroup);
  ibu_5term=get_ibiounit_5term(igroup);
  if(ibu_3term>=0 && ibu_5term>=0){
    ibu=ibu_5term;
    count=0;
    while(1){
      if(count>nbiounit){
        break;
      }
      ibu_5=biounit[ibu].get_link_unit_5();
      ibu_3=biounit[ibu].get_link_unit_3();
      ibu_b=biounit[ibu].get_link_unit_b();
      term3_type=biounit[ibu].get_terminal_3();
      term5_type=biounit[ibu].get_terminal_5();
      f.clear();
      for(int i=0;i<biounit[ibu].get_n_atom_index();i++){
        ia=biounit[ibu].get_atom_index(i);
        f.push_back_atom_index(ia);
        if(biounit[ibu].get_atom_type(i)==27){f.push_back_elec(1);}
        f.push_back_elec(atom[ia].get_atomic_number());
      }
      push_back_fragment(f);
      count++;
      f.clear();
      for(int i=0;i<biounit[ibu_b].get_n_atom_index();i++){
        ia=biounit[ibu_b].get_atom_index(i);
        f.push_back_atom_index(ia);
        f.push_back_elec(atom[ia].get_atomic_number());
      }
      push_back_fragment(f);
      count++;
      if(ibu_3<0) break ;
      ibu=ibu_3;
    }
    ibu=ibu_5term;
    while(1){
      ibu_5=biounit[ibu].get_link_unit_5();
      ibu_3=biounit[ibu].get_link_unit_3();
      ibu_b=biounit[ibu].get_link_unit_b();
      if(ibu_5>=0){
        ic23_ = biounit[ibu_5].get_c23();
        io31_ = biounit[ibu  ].get_o31();
        if(ic23_>=0) ic23=biounit[ibu_5].get_atom_index(ic23_);
        if(io31_>=0) io31=biounit[ibu  ].get_atom_index(io31_);
        if(ic23>=0 && io31>=0){
          push_back_link_to_fragment(ic23,io31);
        } else {
          sprintf(c,"   ** warning : failed to specify BDA atomes between bio-units No. %d and %d\n",ibu_5+1,ibu+1);
          s_out=s_out+c;
        }
      }
      if(ibu_b>=0){
        in26_ = biounit[ibu_b].get_n26();
        ic25_ = biounit[ibu  ].get_c25();
        if(in26_>=0) in26=biounit[ibu_b].get_atom_index(in26_);
        if(ic25_>=0) ic25=biounit[ibu  ].get_atom_index(ic25_);
        if(in26>=0 && ic25>=0){
          push_back_link_to_fragment(ic25,in26);
        } else {
          sprintf(c,"   ** warning : failed to specify BDA atomes between bio-units No. %d and %d\n",ibu_b+1,ibu+1);
          s_out=s_out+c;
        }
      }
      if(ibu_3<0) break ;
      ibu=ibu_3;
    }
  }
}

// ishi 20150704
void Molecule::exchange_fragment ( const int ifrag,
                                   const int jfrag)
{
  Fragment f_;
  f_.copy(fragment[ifrag]);
  fragment[ifrag].copy(fragment[jfrag]);
  fragment[jfrag].copy(f_);
  // set frg1 for each atom
  for(int i=0;i<fragment[ifrag].get_n_atom_index();i++){atom[fragment[ifrag].get_atom_index(i)].set_frg1(ifrag);}
  for(int i=0;i<fragment[jfrag].get_n_atom_index();i++){atom[fragment[jfrag].get_atom_index(i)].set_frg1(jfrag);}
  // set frg2 for each atom
}


void Molecule::div_fragment(const int nb,
                            const int* a1,
                            const int* a2)
{
  int ifrag;
  for(int i=0;i<nb;i++){
    ifrag=get_ifragment_from_iatom(a1[i]);
    if(ifrag>=0){
      div_fragment(ifrag,1,&a1[i],&a2[i]);
    }
  }
}

void Molecule::div_fragment(const int ifrag,
                            const int nb,
                            const int* a1_,
                            const int* a2_)
{
  Fragment frag;
  int chk, nf, num, nb2, iatom, a1_tmp, a2_tmp;
  int * group = new int [natom];
  int * itmp  = new int [natom];
  int * a1    = new int [nbond];
  int * a2    = new int [nbond];

  nb2=0;
  for(int i=0;i<nb;i++){
    if(chk_bond_exist(a1_[i],a2_[i])>=0){
      a1[nb2]=a1_[i];
      a2[nb2]=a2_[i];
      nb2++;
    }
  }
  for(int i=0;i<fragment[ifrag].get_n_link();i++){
    a1[nb2]=fragment[ifrag].get_atom_index_link1(i);
    a2[nb2]=fragment[ifrag].get_atom_index_link2(i);
    nb2++;
  }
  for(int i=0;i<nfragment;i++){
    if(i!=ifrag){
      for(int j=0;j<fragment[i].get_n_link();j++){
        a1_tmp=fragment[i].get_atom_index_link1(j);
        a2_tmp=fragment[i].get_atom_index_link2(j);
        if(get_ifragment_from_iatom(a1_tmp)==ifrag){
          a1[nb2]=a1_tmp;
          a2[nb2]=a2_tmp;
          nb2++;
        }
      }
    }
  }
  //
  for(int i=0;i<natom;i++) group[i]=-1;
  chk=1;
  nf=0;
  while(chk==1){
    for(int i=0;i<fragment[ifrag].get_n_atom_index();i++){
      iatom=fragment[ifrag].get_atom_index(i);
      if(group[iatom]==-1){
        num=0;
        bond_trace(iatom,&num,itmp,nb2,a1,a2);
        break;
      }
    }
    frag.clear();
    for(int i=0;i<num;i++){
      group[itmp[i]]=nf;
      frag.push_back_atom_index(itmp[i]);
      frag.push_back_elec(atom[itmp[i]].get_atomic_number());
      for(int j=0;j<nb2;j++){
        if(a2[j]==itmp[i]){
          frag.push_back_atom_index_link(a1[j],a2[j]);
          frag.push_back_elec(1);
        }
        if(a1[j]==itmp[i]){
          frag.pick_up_elec(1);
        }
      }
    }
    insert_fragment(frag,ifrag+1);
    nf++;
    chk=0;
    for(int i=0;i<fragment[ifrag].get_n_atom_index();i++){
      iatom=fragment[ifrag].get_atom_index(i);
      if(group[iatom]==-1){
        chk=1;
        break;
      }
    }
  }
  pick_up_fragment(ifrag);
  delete [] group ;
  delete [] itmp ;
  delete [] a1 ;
  delete [] a2 ;
}

int Molecule::chk_s_exist_in_fragment(const int ifrag) const 
{
  for(int i=0;i<fragment[ifrag].get_n_atom_index();i++){
    if(atom[fragment[ifrag].get_atom_index(i)].get_atomic_number()==16){
      return 1;
    }
  }
  return -1;
}

int Molecule::get_ifragment_dist_in_linkgroup(const double dist,
                                              const int    nl,
                                              const int*   in,
                                              int*         out,
                                              double*      out_d,
                                              const int    ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_in_linkgroup , in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nlinkgroup){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_in_linkgroup , in[i] >= nlinkgroup" << endl ;
        cerr << "in[i] = " << in[i] << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  int n,imin,itmp;
  double d,dmin,dtmp;
  n=0;
  for(int i=0;i<get_nfragment();i++){
    dmin=999999999.0;
    for(int j=0;j<nl;j++){
      d=get_fragment_distance_from_linkgroup(i,in[j]);
      if(d<dmin) {
        dmin=d;
      }
    }
    if(dmin<dist){
      out[n]=i;
      out_d[n]=dmin;
      n++;
    }
    //cout << i << " " << n << " " << dmin << endl ; 
    //if(i==200) {
    //  exit;
    //}
  }
  // sort
  for(int i=0;i<n-1;i++){
    imin=i;
    for(int j=i+1;j<n;j++){
      if(out_d[imin]>out_d[j]){
        imin=j;
      }
    }
    itmp       =out  [i]   ;
    dtmp       =out_d[i]   ;
    out  [i]   =out  [imin];
    out_d[i]   =out_d[imin];
    out  [imin]=itmp       ;
    out_d[imin]=dtmp       ;
  }
  return n;
}

int Molecule::get_ifragment_dist_in_biounit(const double dist,
                                            const int    nl,
                                            const int*   in,
                                            int*         out,
                                            const int    ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_in_biounit , in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nbiounit){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_in_biounit , in[i] >= nbiounit" << endl ;
        cerr << "in[i] = " << in[i] << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nfragment();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_fragment_distance_from_biounit(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==1){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ifragment_dist_in_fragment(const double dist,
                                             const int nl,
                                             const int* in,
                                             int* out,
                                             double* out_d,
                                             const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_in_fragment , in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nfragment){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_in_fragment , in[i] >= nfragment" << endl ;
        cerr << "in[i] = " << in[i] << " , nfragment = " << nfragment << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  int n,imin,itmp;
  double d,dmin,dtmp;
  n=0;
  for(int i=0;i<get_nfragment();i++){
    dmin=999999999.0;
    for(int j=0;j<nl;j++){
      d=get_fragment_distance_from_fragment(i,in[j]);
      if(dmin>d){
        dmin=d;
      }
    }
    if(dmin<dist){
      out[n]  =i;
      out_d[n]=dmin;
      n++;
    }    
  }
  // sort 
  for(int i=0;i<n-1;i++){
    imin=i;
    for(int j=i+1;j<n;j++){
      if(out_d[imin]>out_d[j]){
        imin=j;
      }
    }
    itmp       =out  [i]   ;
    dtmp       =out_d[i]   ;
    out  [i]   =out  [imin];
    out_d[i]   =out_d[imin];
    out  [imin]=itmp       ;
    out_d[imin]=dtmp       ;
  }
  return n;
}

int Molecule::get_ifragment_dist_out_linkgroup(const double dist,
                                               const int nl,
                                               const int* in,
                                               int* out,
                                               const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_out_linkgroup , in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nlinkgroup){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_out_linkgroup , in[i] >= nlinkgroup" << endl ;
        cerr << "in[i] = " << in[i] << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nfragment();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_fragment_distance_from_linkgroup(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==0){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ifragment_dist_out_biounit(const double dist,
                                             const int nl,
                                             const int* in,
                                             int* out,
                                             const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_out_biounit , in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nbiounit){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_out_biounit , in[i] >= nbiounit" << endl ;
        cerr << "in[i] = " << in[i] << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nfragment();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_fragment_distance_from_biounit(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==0){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ifragment_dist_out_fragment(const double dist,
                                              const int nl,
                                              const int* in,
                                              int* out,
                                              const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_out_fragment , in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nfragment){
      if(ierr==0 || ierr==1){
        cerr << "error : Molecule::get_ifragment_dist_out_fragment , in[i] >= nfragment" << endl ;
        cerr << "in[i] = " << in[i] << " , nfragment = " << nfragment << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nfragment();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_fragment_distance_from_fragment(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==0){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

void Molecule::delete_fragment(const int nl,
                               const int* in)
{
  int * list_atom = new int [get_natom()+1];
  int n_list_atom=0;
  for(int i=0;i<nl;i++){
    for(int j=0;j<get_fragment(in[i]).get_n_atom_index();j++){
      list_atom[n_list_atom]=get_fragment(in[i]).get_atom_index(j);
      n_list_atom++;
    }
  }
  delete_atom(n_list_atom,list_atom);
  delete [] list_atom;
}

void Molecule::delete_fragment_dist_in_linkgroup(const double dist,
                                                 const int nl,
                                                 const int* in)
{
  int * list_buf = new int [get_nfragment()+1];
  double * out_d = new double [get_nfragment()+1];
  int n=get_ifragment_dist_in_linkgroup(dist,nl,in,list_buf,out_d);
  delete_fragment(n,list_buf);
  delete [] list_buf;
  delete [] out_d;
}

void Molecule::delete_fragment_dist_in_biounit(const double dist,
                                               const int nl,
                                               const int* in)
{
  int * list_buf = new int [get_nfragment()];
  int n=get_ifragment_dist_in_biounit(dist,nl,in,list_buf);
  delete_fragment(n,list_buf);
  delete [] list_buf;
}

void Molecule::delete_fragment_dist_in_fragment(const double dist,
                                                const int nl,
                                                const int* in)
{
  int    * list_buf   = new int    [nfragment+1];
  double * list_buf_d = new double [nfragment+1];
  int n=get_ifragment_dist_in_fragment(dist,nl,in,list_buf,list_buf_d);
  delete_fragment(n,list_buf);
  delete [] list_buf ;
  delete [] list_buf_d;
}

void Molecule::delete_fragment_dist_out_linkgroup(const double dist,
                                                  const int nl,
                                                  const int* in)
{
  int * list_buf = new int [get_nfragment()];
  int n=get_ifragment_dist_out_linkgroup(dist,nl,in,list_buf);
  delete_fragment(n,list_buf);
  delete [] list_buf;
}

void Molecule::delete_fragment_dist_out_biounit(const double dist,
                                                const int nl,
                                                const int* in)
{
  int * list_buf = new int [get_nfragment()];
  int n=get_ifragment_dist_out_biounit(dist,nl,in,list_buf);
  delete_fragment(n,list_buf);
  delete [] list_buf;
}

void Molecule::delete_fragment_dist_out_fragment(const double dist,
                                                 const int nl,
                                                 const int* in)
{
  int * list_buf = new int [get_nfragment()];
  int n=get_ifragment_dist_out_fragment(dist,nl,in,list_buf);
  delete_fragment(n,list_buf);
  delete [] list_buf;
}

string Molecule::get_fragment_info_str_ifrag(const int ifrag,
                                             const int type ,
                                             const int ibu_ ,
                                             const int ilg_ ,
                                             const int nsq_ ) const
{
  string buf="";
  Fragment f(get_fragment(ifrag));
  int ia,natom,nelec,nlink,atomic,atomic1,atomic2,atomic3;
  natom=f.get_n_atom_index();
  nelec=f.get_n_elec();
  nlink=f.get_n_link();
  if(type==0){
    buf=buf
      +iIntToString(nelec,3)+ " "
      +iIntToString(natom,3)+ " "
      +iIntToString(nlink,3)+", ";
  }
  if(natom<=4){
    int nH=0;
    int nO=0;
    for(int i=0;i<natom;i++){
      ia=f.get_atom_index(i);
      atomic=atom[ia].get_atomic_number();
      if(atomic==1) nH++;
      if(atomic==8) nO++;
    }
    if(natom==1){
      Atom a1(get_atom(f.get_atom_index(0)));
      atomic1=a1.get_atomic_number();
      if     (atomic1==11 && nelec==10){buf=buf+"Na+   ";}
      else if(atomic1==17 && nelec==18){buf=buf+"Cl-   ";}
      else if(atomic1==19 && nelec==18){buf=buf+"K+    ";}
      else {
        buf=buf+"      ";
      }
    } else if(natom==2){
      if(nH==1 && nO==1){buf=buf+"OH-   ";}
    } else if(natom==3){
      Atom a1(get_atom(f.get_atom_index(0)));
      Atom a2(get_atom(f.get_atom_index(1)));
      Atom a3(get_atom(f.get_atom_index(2)));
      atomic1=a1.get_atomic_number();
      atomic2=a2.get_atomic_number();
      atomic3=a3.get_atomic_number();
      if     (atomic1==8 && atomic2==1 && atomic3==1){buf=buf+"H2O   ";}
      else if(atomic1==1 && atomic2==8 && atomic3==1){buf=buf+"H2O   ";}
      else if(atomic1==1 && atomic2==1 && atomic3==8){buf=buf+"H2O   ";}
      else { 
        buf=buf+"      ";
      }
    } else if(natom==4){
      if(nH==3 && nO==1){buf=buf+"H3O+  ";}
    }
  } else {
    int ibu,ilg,t,nseq,iss,ilg_ss,nseq_ss;
    if(ibu_<0){
      ibu=get_ibiounit_from_fragment(ifrag);
    } else {
      ibu=ibu_;
    }
    //cout << ibu_ << " " << ibu << endl ;
    if(ibu>=0){
      if(ilg_<0){
        ilg=get_ilinkgroup_from_biounit(ibu);
      } else {
        ilg=ilg_;
      }
      BioUnit bu(get_biounit(ibu));
      t=bu.get_type();
      iss=bu.get_ss_bond();
      if(ilg>=0){
        LinkGroup lg(get_linkgroup(ilg));
        if(nsq_<0){
          nseq=biounit[ibu].get_seq();
        } else {
          nseq=nsq_;
        }
        if     (100 <=t && t<4000){buf=buf+bu.get_abbr_3()     +iIntToString(nseq,3,'0');}
        else if(4000<=t && t<5000){buf=buf+"DOX"               +iIntToString(nseq,3,'0');}
        else if(5000<=t          ){buf=buf+bu.get_abbr_1()+"  "+iIntToString(nseq,3,'0');}
      } else {
        buf=buf+"      ";
      }
      if(iss>=0){
        ilg_ss=get_ilinkgroup_from_biounit(iss); 
        BioUnit bu_ss(get_biounit(iss));
        if(ilg_ss>=0){
          LinkGroup lg_ss(get_linkgroup(ilg_ss));
          nseq_ss=biounit[iss].get_seq();
          buf=buf+"-"+bu_ss.get_abbr_3()+iIntToString(nseq_ss,3,'0');
        } else {
          buf=buf+"-"+bu_ss.get_abbr_3()+"   ";
        }
      } else {
        buf=buf+"    ";
      } 
    } else {
      buf=buf+"      ";
    }
  }
  return buf;
}

void Molecule::get_list_atom_to_fragment(int* list) const 
{
  for(int i=0;i<natom;i++){
    list[i]=-1;
  }
  for(int i=0;i<nfragment;i++){
    for(int j=0;j<fragment[i].get_n_atom_index();j++){
      list[fragment[i].get_atom_index(j)]=i;
    }
  }
}

void Molecule::set_fragment_R1(const int  nlist  ,
                               const int* list_in)
{
  for(int i=0;i<nlist;i++){
    if(list_in[i]<0){
      cerr << "error : Molecule::set_fragment_R1 , list_in[i] < 0" << endl ;
      cerr << "list_in[i] = " << list_in[i] << " , this = " << this <<endl ;
      exit(1);
    }
    if(list_in[i]>=nfragment){
      cerr << "error : Molecule::set_fragment_R1 , list_in[i] >= nfragment" << endl ;
      cerr << "list_in[i] = " << list_in[i] << " , nfragment = " << nfragment << " , this = " << this << endl ;
      exit(1);
    }
    fragment[list_in[i]].set_opt(0);
  }
}

void Molecule::set_fragment_R2(const int  nlist  ,
                               const int* list_in)
{
  for(int i=0;i<nlist;i++){
    if(list_in[i]<0){
      cerr << "error : Molecule::set_fragment_R2 , list_in[i] < 0" << endl ;
      cerr << "list_in[i] = " << list_in[i] << " , this = " << this <<endl ;
      exit(1);
    }
    if(list_in[i]>=nfragment){
      cerr << "error : Molecule::set_fragment_R2 , list_in[i] >= nfragment" << endl ;
      cerr << "list_in[i] = " << list_in[i] << " , nfragment = " << nfragment << " , this = " << this << endl ;
      exit(1);
    }
    fragment[list_in[i]].set_opt(1);
  }
}

void Molecule::add_fragment_R1_from_atom()
{
  int ifrag;
  for(int i=0;i<natom;i++){
    if(atom[i].get_opt()==1){
      ifrag=get_ifragment_from_iatom(i);
      if(ifrag>=0){
        fragment[ifrag].set_opt(0);
      }
    }
  }
}

void Molecule::set_fragment_qm(const int  nlist  ,
                               const int* list_in)
{
  for(int i=0;i<nlist;i++){
    if(list_in[i]<0){
      cerr << "error : Molecule::set_fragment_qm , list_in[i] < 0" << endl ;
      cerr << "list_in[i] = " << list_in[i] << " , this = " << this <<endl ;
      exit(1);
    }
    if(list_in[i]>=nfragment){
      cerr << "error : Molecule::set_fragment_qm , list_in[i] >= nfragment" << endl ;
      cerr << "list_in[i] = " << list_in[i] << " , nfragment = " << nfragment << " , this = " << this << endl ;
      exit(1);
    }
    fragment[list_in[i]].set_abmd(0);
  }
}

void Molecule::set_fragment_mm(const int  nlist  ,
                               const int* list_in)
{
  for(int i=0;i<nlist;i++){
    if(list_in[i]<0){
      cerr << "error : Molecule::set_fragment_mm , list_in[i] < 0" << endl ;
      cerr << "list_in[i] = " << list_in[i] << " , this = " << this <<endl ;
      exit(1);
    }
    if(list_in[i]>=nfragment){
      cerr << "error : Molecule::set_fragment_mm , list_in[i] >= nfragment" << endl ;
      cerr << "list_in[i] = " << list_in[i] << " , nfragment = " << nfragment << " , this = " << this << endl ;
      exit(1);
    }
    fragment[list_in[i]].set_abmd(1);
  }
}
