
#include <cstdlib>
#include <cstdio>
#include <math.h>
#include "Molecule.h"
#include "LinkGroup.h"
#include "iString.h"

/* private method */
void Molecule::create_linkgroup(const int n)
{
  clear_linkgroup();
  if(n==0) return ;
  linkgroup = new LinkGroup [n];
  nlinkgroup_buff=n;
}

/* private method */
void Molecule::resize_linkgroup(const int n)
{
  int tmp_nlinkgroup=nlinkgroup;
  LinkGroup * tmp_linkgroup;
  tmp_linkgroup = new LinkGroup [tmp_nlinkgroup+1];
  for(int i=0;i<tmp_nlinkgroup;i++){
    tmp_linkgroup[i].copy(linkgroup[i]);
  }
  create_linkgroup(n);
  if(n>tmp_nlinkgroup){
    for(int i=0;i<tmp_nlinkgroup;i++){
      push_back_linkgroup(tmp_linkgroup[i]);
    }
  } else {
    for(int i=0;i<n;i++){
      push_back_linkgroup(tmp_linkgroup[i]);
    }
  }
  delete [] tmp_linkgroup ;
}

/* private method */
void Molecule::push_back_linkgroup(const LinkGroup& lg,
                                   const int        buff_increase)
{
  if(nlinkgroup==nlinkgroup_buff){
    resize_linkgroup(nlinkgroup+buff_increase);
  }
  linkgroup[nlinkgroup].copy(lg);

  // set atom[].bio1() for H2O molecule
  if(lg.get_n_atom_index()==3){

    // check this linkgroup is H2O or not 
    int nh=0;
    int no=0;
    for(int i=0;i<3;i++){
      if(atom[lg.get_atom_index(i)].get_atomic_number()==1) nh++;
      if(atom[lg.get_atom_index(i)].get_atomic_number()==8) no++;
    }
    if(nh==2 && no==1){

      // set atom[].bio1() , 6501, 6502, 6503
      int chk=0;
      for(int i=0;i<3;i++){
	if(atom[lg.get_atom_index(i)].get_atomic_number()==1) {
	  if(chk==0) {atom[lg.get_atom_index(i)].set_bio1(6502);chk=1;}
	  else       {atom[lg.get_atom_index(i)].set_bio1(6503);      }
	} 
	if(atom[lg.get_atom_index(i)].get_atomic_number()==8) {
	  atom[lg.get_atom_index(i)].set_bio1(6501);
	}
      }
    }
  }
    
  for(int j=0;j<lg.get_n_atom_index();j++){
    atom[lg.get_atom_index(j)].set_lnk1(nlinkgroup);
  }
  nlinkgroup++;
} 

/* private method */
void Molecule::pick_up_linkgroup(const int ilg ,
                                 const int ierr)
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::pick_up_linkgroup , ilg < 0" << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::pick_up_linkgroup , ilg >= nlinkgroup" << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  for(int j=0;j<linkgroup[ilg].get_n_atom_index();j++){
    atom[linkgroup[ilg].get_atom_index(j)].set_lnk1(-1);
  }
  for(int i=ilg;i<nlinkgroup-1;i++){
    linkgroup[i].copy(linkgroup[i+1]);
  }
  nlinkgroup--;
}

void Molecule::copy(const LinkGroup& lg_ , 
                    const int        ilg ,
                    const int        ierr)
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::copy(LinkGroup, int, int) , ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::copy(Linkgroup, int, int) , ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  linkgroup[ilg].copy(lg_);
}

void Molecule::clear_linkgroup()
{
  if(nlinkgroup==0) return ;
  for(int i=(nlinkgroup-1);i>=0;i--){
    pick_up_linkgroup(i);
  }
  delete [] linkgroup ;
  nlinkgroup_buff=0;
}

int Molecule::get_nlinkgroup() const
{
  return nlinkgroup ;
}

int Molecule::get_natom_linkgroup(const int ilg , 
                                  const int ierr) const 
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_natom_linkgroup , ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_natom_linkgroup , ilg >= nlinkgroup" << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return linkgroup[ilg].get_n_atom_index();
}

int Molecule::get_atom_index_linkgroup(const int  nlg ,
                                       const int* ilg ,
                                       int*       out ,
                                       const int  ierr) const
{
  for(int i=0;i<nlg;i++){
    if(ilg[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::get_atom_index_linkgroup , ilg[i] < 0" << endl;
        cerr << "ilg[i] = " << ilg[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(ilg[i]>=nlinkgroup){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::get_atom_index_linkgroup , ilg[i] >= nlinkgroup" << endl ;
        cerr << "ilg[i] = " << ilg[i] << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  int n=0;
  for(int j=0;j<nlg;j++){
    if(0<=ilg[j] && ilg[j]<nlinkgroup){
      for(int i=0;i<linkgroup[ilg[j]].get_n_atom_index();i++){
        out[n]=linkgroup[ilg[j]].get_atom_index(i);
        n++;
      }
    }
  }
  return n;
}

int Molecule::get_natom_linkgroup_atomic(const int ilg   ,
                                         const int atomic,
                                         const int ierr  ) const 
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_natom_linkgroup_atomic , ilg < 0" << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_natom_linkgroup_atomic , ilg >= nlinkgroup" << endl ;
      cerr << "ilg = " << ilg << " nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int n=0;
  for(int i=0;i<linkgroup[ilg].get_n_atom_index();i++){
    if(atom[linkgroup[ilg].get_atom_index(i)].get_atomic_number()==atomic){
      n++;
    }
  }
  return n;
}

LinkGroup Molecule::get_linkgroup(const int ilg , 
                                  const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_linkgroup , ilg < 0" << endl;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_linkgroup , ilg >= nlinkgroup" << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup " << nlinkgroup << " , this = " << this << endl;
    }
    if(ierr==0) exit(1);
  }
  return linkgroup[ilg];
}

int Molecule::get_ilinkgroup_from_biounit(const int ibu ,
                                          const int ierr) const
{
  if(ibu<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_ilinkgroup_from_biounit , ibu < 0" << endl ;
      cerr << "ibu = " << ibu << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibu>=nbiounit){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_ilinkgroup_from_biounit , ibu >= nbiounit" << endl ;
      cerr << "ibu = " << ibu << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int t=biounit[ibu].get_type();
  if(100<=t && t<4000){
    int ic0=biounit[ibu].get_c0();
    if(ic0>=0){
      int ia=biounit[ibu].get_atom_index(ic0);
      return get_ilinkgroup_from_iatom(ia);
    }
  }
  if(4000<=t && t<5000){
    int ic20=biounit[ibu].get_c20();
    if(ic20>=0){
      int ia=biounit[ibu].get_atom_index(ic20);
      return get_ilinkgroup_from_iatom(ia);
    }
  }
  if(5000<=t){
    int in26=biounit[ibu].get_n26();
    if(in26>=0){
      int ia=biounit[ibu].get_atom_index(in26);
      return get_ilinkgroup_from_iatom(ia);
    }
  }
  if(biounit[ibu].get_n_atom_index()>0){
    return get_ilinkgroup_from_iatom(biounit[ibu].get_atom_index(0));
  }
  return -1;
}

int Molecule::get_ilinkgroup_from_iatom(const int iatom) const
{
  for(int i=0;i<nlinkgroup;i++){
    for(int j=0;j<linkgroup[i].get_n_atom_index();j++){
      if(iatom==linkgroup[i].get_atom_index(j)) return i;
    }
  }
  return -1;
}

int Molecule::get_max_iatom_from_ilinkgroup(const int ilg , 
                                            const int ierr) const 
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_max_iatom_from_ilinkgroup , ilg < 0" << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_max_iatom_from_ilinkgroup , ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int imax=linkgroup[ilg].get_atom_index(0);
  for(int i=1;i<linkgroup[ilg].get_n_atom_index();i++){
    if(imax<linkgroup[ilg].get_atom_index(i)) imax=linkgroup[ilg].get_atom_index(i);
  }
  return imax;
}

int Molecule::get_min_iatom_from_ilinkgroup(const int ilg ,
                                            const int ierr) const 
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_min_iatom_from_ilinkgroup , ilg < 0" << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_min_iatom_from_ilinkgroup , ilg >= nlinkgroup" << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int imin=linkgroup[ilg].get_atom_index(0);
  for(int i=1;i<linkgroup[ilg].get_n_atom_index();i++){
    if(imin>linkgroup[ilg].get_atom_index(i)) imin=linkgroup[ilg].get_atom_index(i);
  }
  return imin;
} 

int Molecule::chk_iatom_in_ilinkgroup( const int iatom_,
                                       const int ilg_  ,
                                       const int ierr  ) const
{
  if(ilg_<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::chk_iatom_in_ilinkgroup : ilg_ < 0 " << endl ;
      cerr << "ilg_ = " << ilg_ << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg_>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::chk_iatom_in_ilinkgroup : ilg_ >= nlinkgroup " << endl ;
      cerr << "ilg_ = " << ilg_ << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }  
  for(int i=0;i<linkgroup[ilg_].get_n_atom_index();i++){
    if(iatom_==linkgroup[ilg_].get_atom_index(i)) return i;
  }
  return -1;
}

int Molecule::get_ibiounit_cterm ( const int ilg_ ,
                                   const int ierr ) const 
{
  if(ilg_<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_ibiounit_cterm : ilg_ < 0 " << endl ;
      cerr << "ilg_ = " << ilg_ << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg_>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_ibiounit_cterm : ilg_ >= nlinkgroup " << endl ;
      cerr << "ilg_ = " << ilg_ << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }  
  int ibu;
  for(int i=0;i<linkgroup[ilg_].get_n_atom_index();i++){
    ibu=get_ibiounit_from_iatom(linkgroup[ilg_].get_atom_index(i));
    if(ibu>=0){
      if(biounit[ibu].get_terminal_c()>0){
        return ibu;
      } 
    }
  }
  return -1;
}

int Molecule::get_ibiounit_nterm (const int ilg_,
                                  const int ierr) const
{
  if(ilg_<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_ibiounit_nterm : ilg_ < 0 " << endl ;
      cerr << "ilg_ = " << ilg_ << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg_>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_ibiounit_nterm : ilg_ >= nlinkgroup " << endl ;
      cerr << "ilg_ = " << ilg_ << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }  
  int ibu;
  for(int i=0;i<linkgroup[ilg_].get_n_atom_index();i++){
    ibu=get_ibiounit_from_iatom(linkgroup[ilg_].get_atom_index(i));
    if(ibu>=0){
      if(biounit[ibu].get_terminal_n()>0){
        return ibu;
      } 
    }
  }
  return -1;
}

int Molecule::get_ibiounit_5term( const int ilg_,
                                  const int ierr) const
{
  if(ilg_<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_ibiounit_5term : ilg_ < 0 " << endl ;
      cerr << "ilg_ = " << ilg_ << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg_>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_ibiounit_5term : ilg_ >= nlinkgroup " << endl ;
      cerr << "ilg_ = " << ilg_ << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }  
  int ibu;
  for(int i=0;i<linkgroup[ilg_].get_n_atom_index();i++){
    ibu=get_ibiounit_from_iatom(linkgroup[ilg_].get_atom_index(i));
    if(ibu>=0){
      if(biounit[ibu].get_terminal_5()>0){
        return ibu;
      } 
    }
  }
  return -1;
}

int Molecule::get_ibiounit_3term( const int ilg_,
                                  const int ierr) const
{
  if(ilg_<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_ibiounit_3term : ilg_ < 0 " << endl ;
      cerr << "ilg_ = " << ilg_ << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg_>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_ibiounit_3term : ilg_ >= nlinkgroup " << endl ;
      cerr << "ilg_ = " << ilg_ << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }  
  int ibu;
  for(int i=0;i<linkgroup[ilg_].get_n_atom_index();i++){
    ibu=get_ibiounit_from_iatom(linkgroup[ilg_].get_atom_index(i));
    if(ibu>=0){
      if(biounit[ibu].get_terminal_3()>0){
        return ibu;
      } 
    }
  }
  return -1;
}

int Molecule::get_ibiounit_nterm_plus(const int ilg_,
                                      const int plus,
                                      const int ierr) const
{
  if(ilg_<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_ibiounit_nterm_plus : ilg_ < 0 " << endl ;
      cerr << "ilg_ = " << ilg_ << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg_>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_ibiounit_nterm_plus : ilg_ >= nlinkgroup " << endl ;
      cerr << "ilg_ = " << ilg_ << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }  
  int ibu;
  ibu=get_ibiounit_nterm(ilg_);
  for(int i=0;i<plus;i++){
    ibu=biounit[ibu].get_link_unit_c();
  }
  return ibu;
}

double Molecule::get_linkgroup_total_mass (const int ilg ,
                                           const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_total_mass : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_total_mass : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }  
  double sum=0.0;
  for(int i=0;i<linkgroup[ilg].get_n_atom_index();i++){
    sum=sum+atom[linkgroup[ilg].get_atom_index(i)].get_mass_number();
  }
  return sum;
}

Dvector3D Molecule::get_linkgroup_center_of_mass (const int ilg ,
                                                  const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_center_of_mass : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_center_of_mass : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }  
  double sumMass = get_linkgroup_total_mass(ilg);
  Dvector3D sum(0.00,0.00,0.00);
  int iatom;
  for(int i=0;i<linkgroup[ilg].get_n_atom_index();i++){
    iatom=linkgroup[ilg].get_atom_index(i);
    sum=sum+atom[iatom].get_R()*atom[iatom].get_mass_number();
  }
  if(-0.0000001 < sumMass && sumMass < 0.000001){
    //return innerAxis_origin;
  } else {
    return sum/sumMass;
  }
}

void Molecule::make_linkgroup(string& s_out,
                              const int trace_ss)
{
  // trase_ss == 1 : ss bonds are not included in not trace bond list
  // trase_ss == 0 : ss bonds are included in not trace bond list
  clear_linkgroup();
  if(natom==0){
    return ;
  }

  char c[1024];

  sprintf(c,"\n"                       ); s_out=s_out+c;
  sprintf(c,"  -------------------\n"  ); s_out=s_out+c;
  sprintf(c,"   # make link-group \n"  ); s_out=s_out+c;
  sprintf(c,"  -------------------\n\n"); s_out=s_out+c;

  LinkGroup lg;
  int chk,chkchk,ngroup,num,inc,bond1,bond2,atomic1,atomic2;
  int *  group            = new int   [natom+1];
  int *  itmp             = new int   [natom+1];
  int *  atom_bond_list_n = new int   [natom+1]; // not traced bond list
  int ** atom_bond_list   = new int * [natom+1]; // not traced bond list
  for(int i=0;i<natom;i++){
    group[i]=-1;
    atom_bond_list_n[i]=0;
  }
  for(int i=0;i<nbond;i++){
    atom_bond_list_n[bond[i].get_iatom1()]++;
    atom_bond_list_n[bond[i].get_iatom2()]++;
  }
  for(int i=0;i<natom;i++){
    atom_bond_list[i] = new int [atom_bond_list_n[i]+1];
    atom_bond_list_n[i]=0;
  }
  for(int i=0;i<nbond;i++){
    bond1=bond[i].get_iatom1();
    bond2=bond[i].get_iatom2();
    atomic1=atom[bond1].get_atomic_number();
    atomic2=atom[bond2].get_atomic_number();
    chkchk=0;
    if(atomic1==16 && atomic2==16){
      if(trace_ss==0){
        chkchk=1;
      }
    }
    if((atomic1==16 && atomic2==30)||(atomic2==16 && atomic1==30)){
      if(trace_ss==0){
        chkchk=1;
      }
    }
    if(chkchk==0){
      atom_bond_list  [bond1][atom_bond_list_n[bond1]]=bond2;
      atom_bond_list  [bond2][atom_bond_list_n[bond2]]=bond1;
      atom_bond_list_n[bond1]++;
      atom_bond_list_n[bond2]++;
    }
  }
  chk=1;
  ngroup=0;
  inc=natom/3+1;
  while(chk==1){
    for(int i=0;i<natom;i++){
      if(group[i]==-1){
        num=0;
        bond_trace(i,&num,itmp,atom_bond_list_n,atom_bond_list);
        break;
      }
    }
    lg.clear();
    for(int i=0;i<num;i++){
      group[itmp[i]]=ngroup;
      lg.push_back_atom_index(itmp[i]);
    }
    push_back_linkgroup(lg,inc);
    ngroup++;
    chk=0;
    for(int i=0;i<natom;i++){
      if(group[i]==-1){
        chk=1;
        break;
      }
    }
  }
  set_linkgroup_type();
  for(int i=0;i<natom;i++){
    delete [] atom_bond_list[i];
  }
  delete [] group;
  delete [] itmp;
  delete [] atom_bond_list_n;
  delete [] atom_bond_list;
}

void Molecule::check_linkgroup_distance(string& s_out) const
{
  char c[1024];
  int atm1,atm2,atomic1,atomic2;
  string symb1,symb2;
  double dist;
  for(int i=0;i<(nlinkgroup-1);i++){
    for(int j=(i+1);j<nlinkgroup;j++){
      dist=get_linkgroup_distance_from_linkgroup(i,j,&atm1,&atm2);
      atomic1=atom[atm1].get_atomic_number();
      atomic2=atom[atm2].get_atomic_number();
      symb1=atom[atm1].get_symbol();
      symb2=atom[atm2].get_symbol();
      if(atomic1==1 && atomic2==1){
        if((dist*0.5291)<1.0){
          sprintf(c,"   ** warning : link-group distance between No. %d and %d is %4.2f A (atom No. %d and %d, element %s and %s)\n",
                  i+1,j+1,dist*0.5291,atm1+1,atm2+1,symb1.c_str(),symb2.c_str());
          s_out=s_out+c;
        }
      } else if(atomic1==1 || atomic2==1) {
        if((dist*0.5291)<1.4){
          sprintf(c,"   ** warning : link-group distance between No. %d and %d is %4.2f A (atom No. %d and %d, element %s and %s)\n",
                  i+1,j+1,dist*0.5291,atm1+1,atm2+1,symb1.c_str(),symb2.c_str());
          s_out=s_out+c;
        }
      }
    }
  }
  for(int i=0;i<(nlinkgroup-1);i++){
    for(int j=(i+1);j<nlinkgroup;j++){
      dist=get_linkgroup_distance_from_linkgroup_noH(i,j,&atm1,&atm2);
      atomic1=atom[atm1].get_atomic_number();
      atomic2=atom[atm2].get_atomic_number();
      symb1=atom[atm1].get_symbol();
      symb2=atom[atm2].get_symbol();
      if((dist*0.5291)<2.0){
        sprintf(c,"   ** warning : link-group distance between No. %d and %d is %4.2f A (atom No. %d and %d, element %s and %s)\n",
                i+1,j+1,dist*0.5291,atm1+1,atm2+1,symb1.c_str(),symb2.c_str());
        s_out=s_out+c;
      }
    }
  }
}

double Molecule::get_linkgroup_distance_from_atom(const int ilg1,
                                                  const int ia2,
                                                  int * iatm1,
                                                  const int ierr) const
{
  if(ilg1<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_atom : ilg1 < 0 " << endl ;
      cerr << "ilg1 = " << ilg1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg1>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_atom : ilg1 >= nlinkgroup " << endl ;
      cerr << "ilg1 = " << ilg1 << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ia2<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_atom : ia2 < 0 " << endl ;
      cerr << "ia2 = " << ia2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ia2>=natom){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_atom : ia2 >= natom " << endl ;
      cerr << "ia2 = " << ia2 << " , natom = " << natom << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int n1=linkgroup[ilg1].get_n_atom_index();
  Dvector3D r1;
  Dvector3D r2;
  double x1,x2,y1,y2,z1,z2,dist2,min_dist2;
  r2.copy(atom[ia2].get_R());
  x2=r2.get_x();
  y2=r2.get_y();
  z2=r2.get_z();
  min_dist2=999999.0;
  for(int i=0;i<n1;i++){
    r1.copy(atom[linkgroup[ilg1].get_atom_index(i)].get_R());
    x1=r1.get_x();
    y1=r1.get_y();
    z1=r1.get_z();
    dist2=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
    if(dist2<min_dist2) {
      min_dist2=dist2;
      *iatm1=linkgroup[ilg1].get_atom_index(i);
    }
  }
  return sqrt(min_dist2);
}

double Molecule::get_linkgroup_distance_from_linkgroup(const int ilg1 ,
                                                       const int ilg2 ,
                                                       int *     iatm1,
                                                       int *     iatm2,
                                                       const int ierr ) const 
{
  if(ilg1<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_linkgroup : ilg1 < 0 " << endl ;
      cerr << "ilg1 = " << ilg1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg1>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_linkgroup : ilg1 >= nlinkgroup " << endl ;
      cerr << "ilg1 = " << ilg1 << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg2<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_linkgroup : ilg2 < 0 " << endl ;
      cerr << "ilg2 = " << ilg2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg2>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_linkgroup : ilg2 >= nlinkgroup " << endl ;
      cerr << "ilg2 = " << ilg2 << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg1==ilg2) return 0.0;
  int n1=linkgroup[ilg1].get_n_atom_index();
  int n2=linkgroup[ilg2].get_n_atom_index();
  Dvector3D r1;
  Dvector3D r2;
  double x1,x2,y1,y2,z1,z2,dist2,min_dist2;
  int a1,a2;
  min_dist2=999999.0;
  for(int i=0;i<n1;i++){
    a1=atom[linkgroup[ilg1].get_atom_index(i)].get_atomic_number();
    r1.copy(atom[linkgroup[ilg1].get_atom_index(i)].get_R());
    x1=r1.get_x();
    y1=r1.get_y();
    z1=r1.get_z();
    for(int j=0;j<n2;j++){
      a2=atom[linkgroup[ilg2].get_atom_index(j)].get_atomic_number();
      r2.copy(atom[linkgroup[ilg2].get_atom_index(j)].get_R());
      //if     (a1==16 && a2==16) {}
      //else if(a1==16 && a2==30) {}
      //else if(a1==30 && a2==16) {}
      //else {
      x2=r2.get_x();
      y2=r2.get_y();
      z2=r2.get_z();
      dist2=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
      if(dist2<min_dist2) {
        min_dist2=dist2;
        *iatm1=linkgroup[ilg1].get_atom_index(i);
        *iatm2=linkgroup[ilg2].get_atom_index(j);
      }
      //}
    }
  }
  return sqrt(min_dist2);
}

double Molecule::get_linkgroup_distance_from_linkgroup_noH(const int ilg1 ,
                                                           const int ilg2 ,
                                                           int *     iatm1,
                                                           int *     iatm2,
                                                           const int ierr ) const 
{
  if(ilg1<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_linkgroup_noH : ilg1 < 0 " << endl ;
      cerr << "ilg1 = " << ilg1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg1>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_linkgroup_noH : ilg1 >= nlinkgroup " << endl ;
      cerr << "ilg1 = " << ilg1 << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg2<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_linkgroup_noH : ilg2 < 0 " << endl ;
      cerr << "ilg2 = " << ilg2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg2>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_linkgroup_noH : ilg2 >= nlinkgroup " << endl ;
      cerr << "ilg2 = " << ilg2 << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg1==ilg2) return 0.0;
  int n1=linkgroup[ilg1].get_n_atom_index();
  int n2=linkgroup[ilg2].get_n_atom_index();
  Dvector3D r1;
  Dvector3D r2;
  double x1,x2,y1,y2,z1,z2,dist2,min_dist2;
  int a1,a2;
  min_dist2=999999.0;
  for(int i=0;i<n1;i++){
    a1=atom[linkgroup[ilg1].get_atom_index(i)].get_atomic_number();
    if(a1!=1){
      r1.copy(atom[linkgroup[ilg1].get_atom_index(i)].get_R());
      x1=r1.get_x();
      y1=r1.get_y();
      z1=r1.get_z();
      for(int j=0;j<n2;j++){
        a2=atom[linkgroup[ilg2].get_atom_index(j)].get_atomic_number();
        if(a2!=1){
          if     (a1==16 && a2==16) {}
          else if(a1==16 && a2==30) {}
          else if(a1==30 && a2==16) {}
          else {
            r2.copy(atom[linkgroup[ilg2].get_atom_index(j)].get_R());
            x2=r2.get_x();
            y2=r2.get_y();
            z2=r2.get_z();
            dist2=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
            if(dist2<min_dist2) {
              min_dist2=dist2;
              *iatm1=linkgroup[ilg1].get_atom_index(i);
              *iatm2=linkgroup[ilg2].get_atom_index(j);
            }
          }
        }
      }
    }
  }
  return sqrt(min_dist2);
}

double Molecule::get_linkgroup_distance_from_biounit(const int ilg1,
                                                     const int ibu2,
                                                     const int ierr) const
{
  if(ilg1<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_biounit : ilg1 < 0 " << endl ;
      cerr << "ilg1 = " << ilg1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg1>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_biounit : ilg1 >= nlinkgroup " << endl ;
      cerr << "ilg1 = " << ilg1 << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibu2<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_biounit : ibu2 < 0 " << endl ;
      cerr << "ibu2 = " << ibu2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibu2>=nbiounit){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_biounit : ibu2 >= nbiounit " << endl ;
      cerr << "ibu2 = " << ibu2 << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int n1=linkgroup[ilg1].get_n_atom_index();
  int n2=biounit  [ibu2].get_n_atom_index();
  Dvector3D r1;
  Dvector3D r2;
  double x1,x2,y1,y2,z1,z2,dist2,min_dist2;
  min_dist2=999999.0;
  for(int i=0;i<n1;i++){
    r1.copy(atom[linkgroup[ilg1].get_atom_index(i)].get_R());
    x1=r1.get_x();
    y1=r1.get_y();
    z1=r1.get_z();
    for(int j=0;j<n2;j++){
      r2.copy(atom[biounit[ibu2].get_atom_index(j)].get_R());
      x2=r2.get_x();
      y2=r2.get_y();
      z2=r2.get_z();
      dist2=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
      if(dist2<min_dist2) min_dist2=dist2;
    }
  }
  return sqrt(min_dist2);
}

double Molecule::get_linkgroup_distance_from_fragment (const int ilg1  , 
                                                       const int ifrag2,
                                                       const int ierr  ) const
{
  if(ilg1<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_fragment : ilg1 < 0 " << endl ;
      cerr << "ilg1 = " << ilg1 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg1>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_fragment : ilg1 >= nlinkgroup " << endl ;
      cerr << "ilg1 = " << ilg1 << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag2<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_fragment : ifrag2 < 0 " << endl ;
      cerr << "ifrag2 = " << ifrag2 << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ifrag2>=nfragment){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_distance_from_fragment : ifrag2 >= nfragment " << endl ;
      cerr << "ifrag2 = " << ifrag2 << " , nfragment = " << nfragment << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int n1=linkgroup[ilg1  ].get_n_atom_index();
  int n2=fragment [ifrag2].get_n_atom_index();
  Dvector3D r1;
  Dvector3D r2;
  double x1,x2,y1,y2,z1,z2,dist2,min_dist2;
  min_dist2=999999.0;
  for(int i=0;i<n1;i++){
    r1.copy(atom[linkgroup[ilg1].get_atom_index(i)].get_R());
    x1=r1.get_x();
    y1=r1.get_y();
    z1=r1.get_z();
    for(int j=0;j<n2;j++){
      r2.copy(atom[fragment[ifrag2].get_atom_index(j)].get_R());
      x2=r2.get_x();
      y2=r2.get_y();
      z2=r2.get_z();
      dist2=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
      if(dist2<min_dist2) min_dist2=dist2;
    }
  }
  return sqrt(min_dist2);
}

void Molecule::clear_fragment_linkgroup(const int il  ,
                                        const int ierr)
{
  if(il<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::clear_fragment_linkgroup : il < 0 " << endl ;
      cerr << "il = " << il << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(il>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::clear_fragment_linkgroup : il >= nlinkgroup " << endl ;
      cerr << "il = " << il << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(nfragment>0){
    int * ifrag_list = new int [nfragment];
    int na = linkgroup[il].get_n_atom_index();
    for(int i=0;i<nfragment;i++) ifrag_list[i]=-1;
    for(int i=0;i<na;i++){
      int ifrag=get_ifragment_from_iatom(linkgroup[il].get_atom_index(i));
      if(ifrag>=0) ifrag_list[ifrag]=1;
    }
    for(int i=(nfragment-1);i>=0;i--){
      if(ifrag_list[i]>=0) pick_up_fragment(i);
    }
    delete [] ifrag_list;
  }
}

void Molecule::clear_fragment_linkgroup(const int nl,
                                        const int* in)
{
  for(int i=0;i<nl;i++){
    clear_fragment_linkgroup(in[i]);
  }
}

void Molecule::clear_biounit_linkgroup(const int ilg ,
                                       const int ierr)
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::clear_biounit_linkgroup : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::clear_biounit_linkgroup : il >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    exit(1);
  }
  if(nbiounit>0){
    for(int i=(nbiounit-1);i>=0;i--){
      if(get_ilinkgroup_from_biounit(i)==ilg){
        pick_up_biounit(i);
      }
    }
    linkgroup[ilg].set_residue_index(1);
  }
}

void Molecule::clear_biounit_linkgroup(const int nl,
                                       const int* in)
{
  for(int i=0;i<nl;i++){
    clear_biounit_linkgroup(in[i]);
  }
}

int Molecule::get_nbiounit_linkgroup(const int ilg ,
                                     const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Molecule::get_nbiounit_linkgroup : ilg < 0" << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
    cerr << "error : Moleucle::get_nbiounit_linkgroup ; ilg >= nlinkgroup" << endl ;
    cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl;
    }
    if(ierr==0) exit(1);
  }
  int ibu,ibu_n,ibu_c,ibu_5,ibu_3,ibu_b;
  int n=0;
  ibu=get_ibiounit_nterm(ilg);
  while(1){
    if(ibu<0){
      break ;
    }
    if(n>nbiounit){
      return 0;
    }    
    ibu_n=biounit[ibu].get_link_unit_n();
    ibu_c=biounit[ibu].get_link_unit_c();
    ibu=ibu_c;
    n++;
  }
  ibu=get_ibiounit_5term(ilg);
  while(1){
    if(ibu<0){
      break ;
    }
    if(n>nbiounit){
      return 0;
    }    
    ibu_5=biounit[ibu].get_link_unit_5();
    ibu_3=biounit[ibu].get_link_unit_3();
    ibu=ibu_3;
    n++;
    ibu_b=biounit[ibu].get_link_unit_b();
    if(ibu_b>=0){
      n++;
    }
  }
  return n;
}

int Molecule::get_nfragment_linkgroup(const int ilg ,
                                      const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_nfragment_linkgroup : ilg < 0" << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_nfragment_linkgroup : ilg >= nlinkgroup" << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int ibuf;
  int n=0;
  if(nfragment>0){
    int nfrag_ = nfragment;
    int natom_ = linkgroup[ilg].get_n_atom_index();
    int * ifrag_list = new int [nfrag_];
    for(int i=0;i<nfrag_;i++) ifrag_list[i]=-1;
    for(int i=0;i<natom_;i++){
      ibuf=get_ifragment_from_iatom(linkgroup[ilg].get_atom_index(i));
      if(ibuf>=0){
        ifrag_list[ibuf]=1;
      }
    }
    for(int i=0;i<nfrag_;i++) n++;
    delete [] ifrag_list;
  }
  return n;
}

int Molecule::get_ilinkgroup_dist_in_atom(const double dist,
                                          const int nl,
                                          const int* in,
                                          int* out,
                                          double* out_d,
                                          const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_in_atom : in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=natom){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_in_atom : in[i] >= atom" << endl ;
        cerr << "in[i] = " << in[i] << " , natom = " << natom << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);      
    }
  }
  int n,imin,itmp,ia1;
  double d,dmin,dtmp;
  n=0; // 20150110
  for(int i=0;i<get_nlinkgroup();i++){
    dmin=999999999.0;
    for(int j=0;j<nl;j++){
      d=get_linkgroup_distance_from_atom(i,in[j],&ia1);
      if(dmin>d){
        dmin=d;
      }
    }
    if(dmin<dist){
      out  [n]=i;
      out_d[n]=dmin;
      n++;
    }    
  }
  // sort
  for(int i=0;i<n-1;i++){
    imin=i;
    for(int j=i+1;j<n;j++){
      if(out_d[imin]>out_d[j]){
        imin=j;
      }
    }
    itmp       =out  [i]   ;
    dtmp       =out_d[i]   ;
    out  [i]   =out  [imin];
    out_d[i]   =out_d[imin];
    out  [imin]=itmp       ;
    out_d[imin]=dtmp       ;
  }
  return n;
}

int Molecule::get_ilinkgroup_dist_in_linkgroup(const double dist,
                                               const int nl,
                                               const int * in,
                                               int* out,
                                               double* out_d,
                                               const int ierr) const 
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_in_linkgroup : in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nlinkgroup){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_in_linkgroup : in[i] >= nlinkgroup" << endl ;
        cerr << "in[i] = " << in[i] << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);      
    }
  }
  int n,imin,itmp,ia1,ia2;
  double d,dmin,dtmp;
  n=0;
  for(int i=0;i<get_nlinkgroup();i++){
    dmin=999999999.0;
    for(int j=0;j<nl;j++){
      d=get_linkgroup_distance_from_linkgroup(i,in[j],&ia1,&ia2);
      if(dmin>d){
        dmin=d;
      }
    }
    if(dmin<dist){
      out  [n]=i;
      out_d[n]=dmin;
      n++;
    }    
  }
  // sort
  for(int i=0;i<n-1;i++){
    imin=i;
    for(int j=i+1;j<n;j++){
      if(out_d[imin]>out_d[j]){
        imin=j;
      }
    }
    itmp       =out  [i]   ;
    dtmp       =out_d[i]   ;
    out  [i]   =out  [imin];
    out_d[i]   =out_d[imin];
    out  [imin]=itmp       ;
    out_d[imin]=dtmp       ;
  }
  return n;
}

int Molecule::get_ilinkgroup_dist_in_biounit(const double dist,
                                             const int nl,
                                             const int* in,
                                             int* out,
                                             const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_in_biounit : in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nbiounit){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_in_biounit : in[i] >= nbiounit" << endl ;
        cerr << "in[i] = " << in[i] << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);      
    }
  }
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nlinkgroup();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_linkgroup_distance_from_biounit(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==1){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ilinkgroup_dist_in_fragment(const double dist,
                                              const int nl,
                                              const int* in,
                                              int* out,
                                              const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_in_fragment : in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nfragment){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_in_fragment : in[i] >= nfragment" << endl ;
        cerr << "in[i] = " << in[i] << " , nfragment = " << nfragment << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);      
    }
  }
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nlinkgroup();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_linkgroup_distance_from_fragment(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==1){
      out[n]=i;
      n++;
    }    
  }
  return n;
}


int Molecule::get_ilinkgroup_dist_out_atom(const double dist,
                                           const int nl,
                                           const int* in,
                                           int* out,
                                           const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_out_atom : in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=natom){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_out_atom : in[i] >= natom" << endl ;
        cerr << "in[i] = " << in[i] << " , natom = " << natom << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);      
    }
  }
  int n,chk,ia1;
  double d;
  n=0;
  for(int i=0;i<get_nlinkgroup();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_linkgroup_distance_from_atom(i,in[j],&ia1);
      if(d<dist){
        chk=1;
        break;
      }
    }
    if(chk==0){
      out[n]=i;
      n++;
    }    
  }
  return n;
}


int Molecule::get_ilinkgroup_dist_out_linkgroup(const double dist,
                                                const int nl,
                                                const int* in,
                                                int* out,
                                                const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_out_linkgroup : in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nlinkgroup){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_out_linkgroup : in[i] >= nlinkgroup" << endl ;
        cerr << "in[i] = " << in[i] << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);      
    }
  }
  int n,chk,ia1,ia2;
  double d;
  n=0;
  for(int i=0;i<get_nlinkgroup();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_linkgroup_distance_from_linkgroup(i,in[j],&ia1,&ia2);
      if(d<dist){
        chk=1;
        break;
      }
    }
    if(chk==0){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ilinkgroup_dist_out_biounit(const double dist,
                                              const int nl,
                                              const int* in,
                                              int* out,
                                              const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_out_biounit : in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nbiounit){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_out_biounit : in[i] >= nbiounit" << endl ;
        cerr << "in[i] = " << in[i] << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);      
    }
  }
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nlinkgroup();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_linkgroup_distance_from_biounit(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==0){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ilinkgroup_dist_out_fragment(const double dist,
                                               const int nl,
                                               const int* in,
                                               int* out,
                                               const int ierr) const
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_out_fragment : in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nfragment){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::get_ilinkgroup_dist_out_fragment : in[i] >= nfragment" << endl ;
        cerr << "in[i] = " << in[i] << " , nfragment = " << nfragment << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);      
    }
  }
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nlinkgroup();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_linkgroup_distance_from_fragment(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==0){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

void Molecule::delete_linkgroup(const int nl,
                                const int* in,
                                const int ierr)
{
  for(int i=0;i<nl;i++){
    if(in[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::delete_linkgroup : in[i] < 0" << endl ;
        cerr << "in[i] = " << in[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(in[i]>=nlinkgroup){
      if(ierr==0 || ierr==1){
        cerr << "error : Moleucle::delete_linkgroup : in[i] >= nlinkgroup" << endl ;
        cerr << "in[i] = " << in[i] << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);      
    }
  }
  int * list_atom = new int [get_natom()+1];
  int n_list_atom=0;
  for(int i=0;i<nl;i++){
    for(int j=0;j<get_linkgroup(in[i]).get_n_atom_index();j++){
      list_atom[n_list_atom]=get_linkgroup(in[i]).get_atom_index(j);
      n_list_atom++;
    }
  }

  //cout << natom << " " << n_list_atom << endl ;
  //for(int i=0;i<n_list_atom;i++){
  //  if(list_atom[i]<0 || list_atom[i]>natom){
  //    cout << list_atom[i] << endl ;
  //  }
  //}
  //cout << "ishiishiishi" << endl ;

  delete_atom(n_list_atom,list_atom);
  delete [] list_atom;
}

void Molecule::set_residue_index(const int ilg ,
                                 const int n   ,
                                 const int ierr)
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::set_residue_index : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::set_residue_index : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  linkgroup[ilg].set_residue_index(n);
}

string Molecule::get_linkgroup_info_str(const int ilg ,
                                        const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_info_str : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_info_str : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  LinkGroup l(get_linkgroup(ilg));
  string buf="";
  buf=buf+iIntToString(l.get_n_atom_index(),6)+", ";
  if     (l.get_type()=="peptide"){ buf=buf+"   PEPT"+", "+"res="+iIntToString(l.get_residue_index(),3,'0');}
  else if(l.get_type()=="H2O"    ){ buf=buf+"    H2O"+", "                                                 ;}
  else if(l.get_type()=="Na+"    ){ buf=buf+"    Na+"+", "                                                 ;}
  else if(l.get_type()=="Cl-"    ){ buf=buf+"    Cl-"+", "                                                 ;}
  else                            { buf=buf+"       "+", "                                                 ;}
  return buf;
}

void Molecule::set_linkgroup_type( const int ilg ,
                                   const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::set_linkgroup_type : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::set_linkgroup_type : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  int nO=0;
  int nH=0;
  int nNa=0;
  int nCl=0;
  int natom=linkgroup[ilg].get_n_atom_index();
  int atomic;
  for(int i=0;i<natom;i++){
    atomic=atom[linkgroup[ilg].get_atom_index(i)].get_atomic_number();
    if(atomic== 1) nH++;
    if(atomic== 8) nO++;
    if(atomic==11) nNa++;
    if(atomic==17) nCl++;
  }
  if(natom==3 && nH==2 && nO==1){
    linkgroup[ilg].set_type("H2O");
  }
  if(natom==1 && nNa==1){
    linkgroup[ilg].set_type("Na+");
  }
  if(natom==1 && nCl==1){
    linkgroup[ilg].set_type("Cl-");
  }
}

void Molecule::set_linkgroup_type( ) const
{
  for(int i=0;i<nlinkgroup;i++){
    set_linkgroup_type(i);
  }
}

string Molecule::get_linkgroup_type( const int ilg ,
                                     const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_type : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_type : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return linkgroup[ilg].get_type();
}

int Molecule::get_linkgroup_not_peptide(const int ilg,
                                        const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_not_peptide : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_not_peptide : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return linkgroup[ilg].get_not_peptide();
}

int Molecule::get_linkgroup_not_dna(const int ilg,
                                    const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_not_dna : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::get_linkgroup_not_dna : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return linkgroup[ilg].get_not_dna();
}

void Molecule::add_linkgroup_velocity(const int        ilg,
                                      const Dvector3D& v,
                                      const int        ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::add_linkgroup_velocity : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::add_linkgroup_velocity : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);    
  }
  int na=linkgroup[ilg].get_n_atom_index();
  for(int i=0;i<na;i++){
    atom[linkgroup[ilg].get_atom_index(i)].add_V(v);
  }
}

void Molecule::set_linkgroup_not_peptide(const int ilg,
                                         const int n_,
                                         const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::set_linkgroup_not_peptide : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::set_linkgroup_not_peptide : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);    
  }
  linkgroup[ilg].set_not_peptide(n_);
}

void Molecule::set_linkgroup_not_dna(const int ilg,
                                     const int n_,
                                     const int ierr) const
{
  if(ilg<0){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::set_linkgroup_not_dna : ilg < 0 " << endl ;
      cerr << "ilg = " << ilg << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilg>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error : Moleucle::set_linkgroup_not_dna : ilg >= nlinkgroup " << endl ;
      cerr << "ilg = " << ilg << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);    
  }
  linkgroup[ilg].set_not_dna(n_);
}
