
#include <cstdio>
#include <cstdlib>
#include <math.h>
#include <time.h>
#include <random>
#include "Molecule.h"
#include "Dvector3D.h"
#include "Trajectory.h"

void Molecule::translate(const Dvector3D& r)
{
  for(int i=0;i<natom;i++){
    translate_atom (i,r);
  }
}

void Molecule::rotate(const Dvector3D& axis ,
                      const double     angle,
                      const Dvector3D& r    )
{
  for(int i=0;i<natom;i++){
    rotate_atom (i,axis,angle,r);
  }
}

void Molecule::move_123(const int        iatom1,
                        const int        iatom2,
                        const int        iatom3, 
                        const Dvector3D& r1,
                        const Dvector3D& r2,
                        const Dvector3D& r3)
{
  Dvector3D tmp1,tmp2,tmp3,tmp12,tmp23,r12,r23;
  double ang1,ang2;
  
  ///// translate
  tmp1.copy(atom[iatom1].get_R());
  translate(r1-tmp1);

  ///// rotate
  tmp1.copy(atom[iatom1].get_R());
  tmp2.copy(atom[iatom2].get_R());
  tmp12=tmp2-tmp1;
  r12=r2-r1;
  ang1=Dvector3D::angle(tmp12,r12);
  if(ang1 < -0.00001 || 0.00001 < ang1){
    if(((-3.141592-0.00001) < ang1 && ang1 < (-3.141592+0.00001)) ||
       ((+3.141592-0.00001) < ang1 && ang1 < (+3.141592+0.00001))) {
      tmp3.set_xyz(1.00, -tmp3.get_x()/tmp3.get_y(),0.00);
    } else {
      tmp3.copy(Dvector3D::Vproduct(tmp12, r12));
    }
    rotate(tmp3,ang1,atom[iatom1].get_R());            
  }
  
  ///// rotate
  ang1=Dvector3D::dihedral(atom[iatom1].get_R(),r3,atom[iatom2].get_R(),atom[iatom3].get_R());
  if(ang1 < -0.00001 || 0.00001 < ang1){
    tmp1.copy(atom[iatom1].get_R());
    tmp2.copy(atom[iatom2].get_R());
    tmp3.copy(atom[iatom3].get_R());
    tmp12=tmp2-tmp1;
    tmp23=tmp3-tmp2;
    r23=r3-r2;
    tmp3.copy(Dvector3D::rotation(tmp23,ang1,tmp12));
    ang2=Dvector3D::angle(tmp3,r23);
    if(-0.001 < ang2 && ang2 < 0.001){
      rotate(tmp12,ang1,atom[iatom1].get_R());
    } else {
      rotate(tmp12,-ang1,atom[iatom1].get_R());
    }
  }
}

double Molecule::get_max_X () const 
{
  double maxX;
  if(natom==0) return 0.0;
  maxX=atom[0].get_R().get_x();
  for(int i=1;i<natom;i++){
    if(maxX<atom[0].get_R().get_x()) maxX=atom[0].get_R().get_x();
  }
  return maxX;
}

double Molecule::get_max_Y () const
{
  double maxY;
  if(natom==0) return 0.0;
  maxY=atom[0].get_R().get_y();
  for(int i=1;i<natom;i++){
    if(maxY<atom[0].get_R().get_y()) maxY=atom[0].get_R().get_y();
  }
  return maxY;
}

double Molecule::get_max_Z () const
{
  double maxZ;
  if(natom==0) return 0.0;
  maxZ=atom[0].get_R().get_z();
  for(int i=1;i<natom;i++){
    if(maxZ<atom[0].get_R().get_z()) maxZ=atom[0].get_R().get_z();
  }
  return maxZ;
}

double Molecule::get_min_X () const
{
  double minX;
  if(natom==0) return 0.0;
  minX=atom[0].get_R().get_x();
  for(int i=1;i<natom;i++){
    if(minX<atom[0].get_R().get_x()) minX=atom[0].get_R().get_x();
  }
  return minX;
}

double Molecule::get_min_Y () const
{
  double minY;
  if(natom==0) return 0.0;
  minY=atom[0].get_R().get_y();
  for(int i=1;i<natom;i++){
    if(minY<atom[0].get_R().get_y()) minY=atom[0].get_R().get_y();
  }
  return minY;
}

double Molecule::get_min_Z () const
{
  double minZ;
  if(natom==0) return 0.0;
  minZ=atom[0].get_R().get_z();
  for(int i=1;i<natom;i++){
    if(minZ<atom[0].get_R().get_z()) minZ=atom[0].get_R().get_z();
  }
  return minZ;
}

double Molecule::get_total_mass() const
{
  double sum=0.0;
  for(int i=0;i<natom;i++){
    sum=sum+atom[i].get_mass_number();
  }
  return sum;
}

Dvector3D Molecule::get_center_of_mass() const
{
  double sumMass = get_total_mass();
  Dvector3D sum(0.00,0.00,0.00);
  for(int i=0;i<natom;i++){
    sum=sum+atom[i].get_R()*atom[i].get_mass_number();
  }
  if(-0.0000001 < sumMass && sumMass < 0.000001){
    //return innerAxis_origin;
  } else {
    return sum/sumMass;
  }
}

Dvector3D Molecule::get_center_of_mass(const int n, const int* in) const
{
  Dvector3D c(0.0,0.0,0.0);
  if(n<=0){
    return c;
  }
  Atom a;
  double sum=0.0,x=0.0,y=0.0,z=0.0,mn;
  for(int i=0;i<n;i++){
    a.copy(get_atom(in[i]));
    mn=a.get_mass_number();
    x=x+a.get_R().get_x()*mn;
    y=y+a.get_R().get_y()*mn;
    z=z+a.get_R().get_z()*mn;
    sum=sum+mn;
  }
  x=x/sum;
  y=y/sum;
  z=z/sum;
  c.set_x(x);
  c.set_y(y);
  c.set_z(z);
  return c;
}

double Molecule::get_nuclear_repulsion() const
{
  Atom a1,a2;
  Dvector3D r1,r2;
  double tmp;
  int n1,n2;
  tmp=0.0;
  for(int i=0;i<get_natom();i++){
    a1.copy(atom[i]) ;
    r1.copy(a1.get_R()) ;
    n1=a1.get_atomic_number() ;
    for(int j=0;j<i;j++){
      a2.copy(atom[j]) ;
      r2.copy(a2.get_R()) ;
      n2=a2.get_atomic_number() ;
      tmp=tmp+(n1*n2)/sqrt(Dvector3D::distance2(r1,r2));
    }
  }
  return tmp;
}

void Molecule::get_extension(double * x_out,
                             double * y_out,
                             double * z_out,
                             const int type) const 
{
  // type=0 : all atom
  // type=1 : main chain
  double x,sum_x;
  double y,sum_y;
  double z,sum_z;
  int n,atype;
  sum_x=0.0;
  sum_y=0.0;
  sum_z=0.0;
  n=0.0;
  for(int i=0;i<natom;i++){
    x=fabs(atom[i].get_R().get_x());
    y=fabs(atom[i].get_R().get_y());
    z=fabs(atom[i].get_R().get_z());
    if(type==1){
      atype=atom[i].get_bio1();
      if(atype==0 || atype==1 || atype==2 || atype==4 ){
        n++;
        sum_x=sum_x+fabs(atom[i].get_R().get_x());
        sum_y=sum_y+fabs(atom[i].get_R().get_y());
        sum_z=sum_z+fabs(atom[i].get_R().get_z());
      }
    } else {
      n++;
      sum_x=sum_x+fabs(atom[i].get_R().get_x());
      sum_y=sum_y+fabs(atom[i].get_R().get_y());
      sum_z=sum_z+fabs(atom[i].get_R().get_z());
    }
  }
  (*x_out)=sum_x/n;
  (*y_out)=sum_y/n;
  (*z_out)=sum_z/n;
}

void Molecule::add_molecule(const Molecule& m)
{
  int na  = m.get_natom();
  int nb  = m.get_nbond();
  int na_ = natom;
  int nb_ = nbond;
  int a1,a2;
  Bond * b = new Bond [nb_+1];
  for(int i=0;i<nb_;i++){
    b[i].copy(bond[i]);
  }
  for(int i=0;i<na;i++){
    add_atom(m.get_atom(i));
  }
  for(int i=0;i<nb_;i++){
    a1=b[i].get_iatom1();
    a2=b[i].get_iatom2();
    add_bond(a1,a2,nb_+nb+1);
  }
  for(int i=0;i<nb;i++){
    a1=(m.get_bond(i).get_iatom1())+na_;
    a2=(m.get_bond(i).get_iatom2())+na_;
    add_bond(a1,a2,nb+1);
  }
  delete [] b;
}

Molecule Molecule::pick_up_overlap(int* iend, const Molecule& m_, const double th, const int istart)
{
  Molecule tmp_m,tmp_mm;
  
  ///// m_ molecule
  Dvector3D tmp_r1,tmp_r2,tmp_r3;
  double tmp_leng12, tmp_leng13, tmp_leng23;
  int tmp_symb1, tmp_symb2, tmp_symb3;
  
  ///// this molecule
  Dvector3D R1, R2, R3;
  double Leng12, Leng13, Leng23;
  int Symb1,Symb2,Symb3;
  
  Dvector3D rtmp1, rtmp2;
  int chk,miniatom,symbchk1,symbchk2,symbchk3,atomic1,atomic2;
  double lengleng, minlengleng;
  Atom a_;

  int j1,j2,j3;
  int itmp;

  j1=0;
  j2=1;
  j3=2;
  
  tmp_symb1=m_.get_atom(j1).get_atomic_number();
  tmp_symb2=m_.get_atom(j2).get_atomic_number();
  tmp_symb3=m_.get_atom(j3).get_atomic_number();
  tmp_r1.copy(m_.get_atom(j1).get_R());
  tmp_r2.copy(m_.get_atom(j2).get_R());
  tmp_r3.copy(m_.get_atom(j3).get_R());
  tmp_leng12=sqrt(Dvector3D::distance2(tmp_r1,tmp_r2));
  tmp_leng13=sqrt(Dvector3D::distance2(tmp_r1,tmp_r3));
  tmp_leng23=sqrt(Dvector3D::distance2(tmp_r2,tmp_r3));
  for(int i1=istart;i1<get_natom();i1++) {
    iend[0]=i1;
    symbchk1=0;
    symbchk2=0;
    symbchk3=0;
    Symb1=atom[i1].get_atomic_number();
    if(Symb1==tmp_symb1) {
      R1.copy(atom[i1].get_R());
      for(int i2=0;i2<get_natom();i2++){
        Symb2=atom[i2].get_atomic_number();
        if(Symb2==tmp_symb2 && i1!=i2) {
          R2.copy(atom[i2].get_R());
          Leng12=sqrt(Dvector3D::distance2(R1,R2));
          if(Leng12<(tmp_leng12+th)) {
            for(int i3=0;i3<get_natom();i3++){
              Symb3=atom[i3].get_atomic_number();
              if(Symb3==tmp_symb3 && i1!=i3 && i2!=i3) {
                R3.copy(atom[i3].get_R());
                Leng13=sqrt(Dvector3D::distance2(R1,R3));
                Leng23=sqrt(Dvector3D::distance2(R2,R3));
                if(Leng13<(tmp_leng13+th) && Leng23<(tmp_leng23+th)) {
                  tmp_m.copy(m_);
                  tmp_m.move_123(j1,j2,j3,R1,R2,R3);
                  chk=0;
                  for(int i=0;i<tmp_m.get_natom();i++){
                    rtmp1.copy(tmp_m.get_atom(i).get_R());
                    atomic1=tmp_m.get_atom(i).get_atomic_number();
                    minlengleng=9999.99;
                    itmp=0;
                    for(int j=0;j<get_natom();j++){
                      rtmp2.copy(atom[j].get_R());
                      atomic2=atom[j].get_atomic_number();
                      if(atomic1==atomic2){
                        lengleng=Dvector3D::distance2(rtmp1,rtmp2);
                        if(lengleng<minlengleng) {
                          minlengleng=lengleng;
                          itmp=j;
                        }
                      }
                    }
                    if(sqrt(minlengleng)>th) {
                      chk=1;
                      break;
                    }
                  }
                  if(chk==0) {
                    for(int i=0;i<tmp_m.get_natom();i++){
                      rtmp1.copy(tmp_m.get_atom(i).get_R());
                      miniatom=-1;
                      minlengleng=9999.99;
                      for(int j=0;j<get_natom();j++){
                        rtmp2.copy(atom[j].get_R());
                        lengleng=Dvector3D::distance2(rtmp1,rtmp2);
                        if(lengleng<minlengleng) {
                          minlengleng=lengleng;
                          miniatom=j;
                        }
                      }
                      a_.copy(pick_up_atom(miniatom));
                      tmp_mm.add_atom(a_);
                    }
                    return tmp_mm;
                  } else {
                    tmp_m.clear();
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  return tmp_mm;
}

double Molecule::get_distance_R(const Dvector3D& r) const
{
  double dist,min;
  min=999999.0;
  for(int i=0;i<get_natom();i++){
    dist=sqrt(Dvector3D::distance2(r,atom[i].get_R()));
    if(dist<min){min=dist;}
  }
  return min;
}

double Molecule::get_distance_ilg_ifrag(const int ilg, const int ifrag) const 
{
  double * xyz1;
  double * xyz2;
  int n1=linkgroup[  ilg].get_n_atom_index();
  int n2=fragment [ifrag].get_n_atom_index();
  xyz1 = new double [n1*3];
  xyz2 = new double [n2*3];
  for(int i=0;i<n1;i++){
    xyz1[i*3+0]=atom[linkgroup[ilg].get_atom_index(i)].get_R().get_x();
    xyz1[i*3+1]=atom[linkgroup[ilg].get_atom_index(i)].get_R().get_y();
    xyz1[i*3+2]=atom[linkgroup[ilg].get_atom_index(i)].get_R().get_z();
  }
  for(int i=0;i<n2;i++){
    xyz2[i*3+0]=atom[fragment[ifrag].get_atom_index(i)].get_R().get_x();
    xyz2[i*3+1]=atom[fragment[ifrag].get_atom_index(i)].get_R().get_y();
    xyz2[i*3+2]=atom[fragment[ifrag].get_atom_index(i)].get_R().get_z();
  }
  double min_dist2=999999.0;
  double dist2;
  for(int i=0;i<n1;i++){
    for(int j=0;j<n2;j++){
      dist2=(xyz1[i*3+0]-xyz2[j*3+0])*(xyz1[i*3+0]-xyz2[j*3+0])+
            (xyz1[i*3+1]-xyz2[j*3+1])*(xyz1[i*3+1]-xyz2[j*3+1])+
            (xyz1[i*3+2]-xyz2[j*3+2])*(xyz1[i*3+2]-xyz2[j*3+2]);
      if(min_dist2>dist2) min_dist2=dist2;
    }
  }
  return sqrt(min_dist2);
  delete [] xyz1;
  delete [] xyz2;
}

double Molecule::get_distance_ilg_ilg(const int ilg1, const int ilg2) const 
{
  if(ilg1==ilg2) return 0.0;
  double * xyz1;
  double * xyz2;
  int n1=linkgroup[ilg1].get_n_atom_index();
  int n2=linkgroup[ilg2].get_n_atom_index();
  xyz1 = new double [n1*3];
  xyz2 = new double [n2*3];
  for(int i=0;i<n1;i++){
    xyz1[i*3+0]=atom[linkgroup[ilg1].get_atom_index(i)].get_R().get_x();
    xyz1[i*3+1]=atom[linkgroup[ilg1].get_atom_index(i)].get_R().get_y();
    xyz1[i*3+2]=atom[linkgroup[ilg1].get_atom_index(i)].get_R().get_z();
  }
  for(int i=0;i<n2;i++){
    xyz2[i*3+0]=atom[linkgroup[ilg2].get_atom_index(i)].get_R().get_x();
    xyz2[i*3+1]=atom[linkgroup[ilg2].get_atom_index(i)].get_R().get_y();
    xyz2[i*3+2]=atom[linkgroup[ilg2].get_atom_index(i)].get_R().get_z();
  }
  double min_dist2=999999.0;
  double dist2;
  for(int i=0;i<n1;i++){
    for(int j=0;j<n2;j++){
      dist2=(xyz1[i*3+0]-xyz2[j*3+0])*(xyz1[i*3+0]-xyz2[j*3+0])+
            (xyz1[i*3+1]-xyz2[j*3+1])*(xyz1[i*3+1]-xyz2[j*3+1])+
            (xyz1[i*3+2]-xyz2[j*3+2])*(xyz1[i*3+2]-xyz2[j*3+2]);
      if(min_dist2>dist2) min_dist2=dist2;
    }
  }
  return sqrt(min_dist2);
  delete [] xyz1;
  delete [] xyz2;
}

double Molecule::get_distance_ifrag_ifrag(const int ifrag1, const int ifrag2) const 
{
  double * xyz1;
  double * xyz2;
  int n1=fragment[ifrag1].get_n_atom_index();
  int n2=fragment[ifrag2].get_n_atom_index();
  xyz1 = new double [n1*3];
  xyz2 = new double [n2*3];
  for(int i=0;i<n1;i++){
    xyz1[i*3+0]=atom[fragment[ifrag1].get_atom_index(i)].get_R().get_x();
    xyz1[i*3+1]=atom[fragment[ifrag1].get_atom_index(i)].get_R().get_y();
    xyz1[i*3+2]=atom[fragment[ifrag1].get_atom_index(i)].get_R().get_z();
  }
  for(int i=0;i<n2;i++){
    xyz2[i*3+0]=atom[fragment[ifrag2].get_atom_index(i)].get_R().get_x();
    xyz2[i*3+1]=atom[fragment[ifrag2].get_atom_index(i)].get_R().get_y();
    xyz2[i*3+2]=atom[fragment[ifrag2].get_atom_index(i)].get_R().get_z();
  }
  double min_dist2=999999.0;
  double dist2;
  for(int i=0;i<n1;i++){
    for(int j=0;j<n2;j++){
      dist2=(xyz1[i*3+0]-xyz2[j*3+0])*(xyz1[i*3+0]-xyz2[j*3+0])+
            (xyz1[i*3+1]-xyz2[j*3+1])*(xyz1[i*3+1]-xyz2[j*3+1])+
            (xyz1[i*3+2]-xyz2[j*3+2])*(xyz1[i*3+2]-xyz2[j*3+2]);
      if(min_dist2>dist2) min_dist2=dist2;
    }
  }
  return sqrt(min_dist2);
  delete [] xyz1;
  delete [] xyz2;
}

void Molecule::chk_coord(string& s_out) const 
{
  char c[1024];

  /*----------------------*/
  /* chk bond in hydrogen */
  /*----------------------*/

  int * hbond0 = new int [get_natom()+1];
  int * hbond2 = new int [get_natom()+1];
  int n_hbond0;
  int n_hbond2;

  chk_bond_num_H (&n_hbond0,&n_hbond2,hbond0,hbond2);

  if(n_hbond0!=0){
    for(int i=0;i<n_hbond0;i++){
      sprintf(c,"   ** warning : atom No. %6d (H) has no chemical bond\n",hbond0[i]+1);
      s_out=s_out+c;
    }
  }
  if(n_hbond2!=0){
    for(int i=0;i<n_hbond2;i++){
      sprintf(c,"   ** warning : atom No. %6d (H) has too many chemical bonds\n",hbond2[i]+1);
      s_out=s_out+c;
    }
  }

  delete [] hbond0;
  delete [] hbond2;

  /*---------------------*/
  /* chk bond in carbond */
  /*---------------------*/

  int * hbond1 = new int [get_natom()+1];
  int * hbond5 = new int [get_natom()+1];
  int n_hbond1;
  int n_hbond5;

  chk_bond_num_C (&n_hbond1,&n_hbond5,hbond1,hbond5);

  if(n_hbond1!=0){
    for(int i=0;i<n_hbond1;i++){
      sprintf(c,"   ** warning : atom No. %6d (C) has too less chemical bonds\n",hbond1[i]+1);
      s_out=s_out+c;
    }
  }
  if(n_hbond5!=0){
    for(int i=0;i<n_hbond5;i++){
      sprintf(c,"   ** warning : atom No. %6d (C) has too many chemical bonds\n",hbond5[i]+1);
      s_out=s_out+c;
    }
  }
  
  delete [] hbond1;
  delete [] hbond5;
}

void Molecule::set_atom_from_trajectory(const Trajectory& t,
                                        const int itraj_,
                                        const int ierr)
{
  int na,nb,ia,ib;
  double x,y,z,vx,vy,vz,gx,gy,gz;
  int * iatom;
  int * av;
  double * x_;
  double * y_;
  double * z_;
  double * vx_;
  double * vy_;
  double * vz_;
  double * gx_;
  double * gy_;
  double * gz_;
  Dvector3D r;
  Dvector3D v;
  Dvector3D g;
  double R_type2=t.get_bond_color_R(2);
  double G_type2=t.get_bond_color_G(2);
  double B_type2=t.get_bond_color_B(2);
  na=t.get_atom_n(itraj_);
  if(na!=natom){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::set_atom_from_trajectory , na != natom " << endl ;
      cerr << "na in trajectory = " << na << " , natom in molecule = " << natom << endl ;
    }
    if(ierr==0) exit(1);
  }
  iatom=t.get_iatom_p     (itraj_);
  av   =t.get_iatom_view_p(itraj_);
  x_ =t.get_x_p(itraj_);
  y_ =t.get_y_p(itraj_);
  z_ =t.get_z_p(itraj_);
  vx_=t.get_vx_p(itraj_);
  vy_=t.get_vy_p(itraj_);
  vz_=t.get_vz_p(itraj_);
  gx_=t.get_gx_p(itraj_);
  gy_=t.get_gy_p(itraj_);
  gz_=t.get_gz_p(itraj_);
  for(int i=0;i<na;i++){
    ia=iatom[i];
    if(ia<0){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::set_atom_from_trajectory , ia < 0 " << endl ;
        cerr << "ia = " << ia << endl ;
      }
      if(ierr==0) exit(1);
    } 
    if(ia>natom){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::set_atom_from_trajectory , ia >= natom " << endl ;
        cerr << "ia = " << ia << " , natom = " << natom << endl ;
      }
      if(ierr==0) exit(1);
    }
    x =x_[i];
    y =y_[i];
    z =z_[i];
    vx =vx_[i];
    vy =vy_[i];
    vz =vz_[i];
    gx =gx_[i];
    gy =gy_[i];
    gz =gz_[i];
    r.set_xyz(x,y,z);
    v.set_xyz(vx,vy,vz);
    g.set_xyz(gx,gy,gz);
    atom[ia].set_R(r);
    atom[ia].set_V(v);
    atom[ia].set_G(g);
    if(av[i]==1){
    }
    if(av[i]==2){
      view_atom_set_color(i,R_type2,G_type2,B_type2);
    }
  }
}

void Molecule::set_bond_from_trajectory(const Trajectory& t,
                                        const int itraj_  ,
                                        const int ierr    )
{
  int * b1;
  int * b2;
  int * bv;
  int n;
  double R_type2=t.get_bond_color_R(2);
  double G_type2=t.get_bond_color_G(2);
  double B_type2=t.get_bond_color_B(2);
  b1 =t.get_ibond1_p    (itraj_,ierr);
  b2 =t.get_ibond2_p    (itraj_,ierr);
  bv =t.get_ibond_view_p(itraj_,ierr);
  clear_bond();
  n=t.get_bond_n(itraj_);
  for(int i=0;i<n;i++){
    add_bond(b1[i],b2[i],n);
    if(bv[i]==1){
      view_bond_set_dist_ratio_atomic(i);
      view_bond_set_color_atomic(i);
    } 
    else if(bv[i]==2){
      view_bond_set_color(i,R_type2,G_type2,B_type2);
    }
  }
}

void Molecule::make_random_velocity(const double t) const 
{
  Dvector3D v;
  int n;
  double x,y,z,sum,tt;

  std::random_device rnd;
  std::mt19937 mt((long int)rnd());
  //srand(time(NULL));

  for(int i=0;i<natom;i++){
    n=mt() ; x = (double) (n % 100) ; if(n%2==1) x = -1.0*x; //n=rand(); x = (double) (n % 100); if(n%2==1) x = -1.0*x;
    n=mt() ; y = (double) (n % 100) ; if(n%2==1) y = -1.0*y; //n=rand(); y = (double) (n % 100); if(n%2==1) y = -1.0*y;
    n=mt() ; z = (double) (n % 100) ; if(n%2==1) z = -1.0*z; //n=rand(); z = (double) (n % 100); if(n%2==1) z = -1.0*z;
    v.set_xyz(x,y,z);
    atom[i].set_V(v);
  }
  sum=0.0;
  for(int i=0;i<natom;i++){
    sum=sum+(x*x+y*y+z*z)*atom[i].get_mass_number();
  }
  tt=1.0/3.0/((double)natom)*0.3367967*sum;
  for(int i=0;i<natom;i++){
    x=atom[i].get_V().get_x()*sqrt(t/tt);
    y=atom[i].get_V().get_y()*sqrt(t/tt);
    z=atom[i].get_V().get_z()*sqrt(t/tt);
    v.set_xyz(x,y,z);
    atom[i].set_V(v);
  }
  sum=0.0;
  for(int i=0;i<natom;i++){
    sum=sum+(x*x+y*y+z*z)*atom[i].get_mass_number();
  }
  tt=1.0/3.0/((double)natom)*0.3367967*sum;
}

double analysis_HLA_angle(Dvector3D r1,
                          Dvector3D r2,
                          Dvector3D r3,
                          Dvector3D r4)
{
  Dvector3D r23;
  Dvector3D r21;
  Dvector3D r34;
  Dvector3D p_r21_r23;   // projection of r21 to r23
  Dvector3D p_r34_r23;   // projection of r34 to r23
  Dvector3D p_r21_ppr23; // projection of r21 to plane perpendicular to r23
  Dvector3D p_r34_ppr23; // projection of r34 to plane perpendicular to r23
  Dvector3D r2134;       // vector product of p_r21_ppr23 and p_r34_ppr23
  double angle,chk;

  r23.copy(r3-r2);
  r21.copy(r1-r2);
  r34.copy(r4-r3);
  p_r21_r23.copy(r23*(Dvector3D::dot(r23,r21)/(Dvector3D::norm(r23)*Dvector3D::norm(r23))));
  p_r34_r23.copy(r23*(Dvector3D::dot(r23,r34)/(Dvector3D::norm(r23)*Dvector3D::norm(r23))));
  p_r21_ppr23.copy(r21-p_r21_r23);
  p_r34_ppr23.copy(r34-p_r34_r23);
  r2134.copy(Dvector3D::Vproduct(p_r21_ppr23,p_r34_ppr23));
  angle=Dvector3D::angle(p_r21_ppr23,p_r34_ppr23);
  chk=Dvector3D::dot(r2134,r23)/(Dvector3D::norm(r2134)*Dvector3D::norm(r23));
  if      (-1.000001<chk && chk<-0.999999){return -angle;}
  else if ( 0.999999<chk && chk< 1.000001){return  angle;}
  else {
    cout << "error !! in analysis_HLA_angle" << endl ;
    exit(1);
    //return angle;
    return -1;
  }
}

void Molecule::analysis_HLA(string& s_out) const
{
  char c[1024];
  Dvector3D r;

  int ilg =nlinkgroup-1;
  int nb  =get_nbiounit_linkgroup(ilg);
  int ib_n=get_ibiounit_nterm(ilg);
  int ib_c=get_ibiounit_cterm(ilg);

  int       i_ca      [128]; // index      of CA atom in main chain 
  int       i_c       [128]; // index      of C  atom in main chain
  int       i_n       [128]; // index      of N  atom in main chain 
  int       i_nme_n        ; // index      of N  atom in main chain (NME)
  int       i_nme_ca       ; // index      of CA atom in main chain (NME)
  int       i_ace_c        ; // index      of C  atom in main chain (ACE)
  int       i_ace_ca       ; // index      of CA atom in main chain (ACE)

  Dvector3D R_ca      [128]; // coordinate of CA atom in main chain
  Dvector3D R_c       [128]; // coordinate of C  atom in main chain
  Dvector3D R_n       [128]; // coordinate of N  atom in main chain
  Dvector3D R_sc      [128]; // coordinate of side chain center of mass
  Dvector3D R_nme_n        ; // coordinate of N  atom in main chain (NME)
  Dvector3D R_nme_ca       ; // coordinate of CA atom in main chain (NME)
  Dvector3D R_ace_c        ; // coordinate of C  atom in main chain (ACE)
  Dvector3D R_ace_ca       ; // coordinate of CA atom in main chain (ACE)

  string    res_type_1[128]; // residue type 1 char
  string    res_type_3[128]; // residue type 3 char
  string    res_sc_pos[128]; // up or down for side chain
  string    res_n_pos [128]; // up or down for N
  string    res_c_pos [128]; // up or down for C
  string    UD_n           ; // U and D string for Nterm
  string    UD_c           ; // U and D string for Cterm
  int       chk_order_n    ; // CA order ( 1: good, 0: not good )
  int       chk_order_c    ; // CA order ( 1: good, 0: not good )

  double angle;
  Dvector3D r1,r2,r3,r4;
  
  int       chk_ace=-1     ;
  int       chk_nme=-1     ;

  // search region (3)
  double max_x_n_3 = -999.0;  double min_x_n_3 = +999.0;
  double max_y_n_3 = -999.0;  double min_y_n_3 = +999.0;
  double max_z_n_3 = -999.0;  double min_z_n_3 = +999.0;
  double max_x_c_3 = -999.0;  double min_x_c_3 = +999.0;
  double max_y_c_3 = -999.0;  double min_y_c_3 = +999.0;
  double max_z_c_3 = -999.0;  double min_z_c_3 = +999.0;

  // search region (4)
  double max_x_n_4 = -999.0;  double min_x_n_4 = +999.0;
  double max_y_n_4 = -999.0;  double min_y_n_4 = +999.0;
  double max_z_n_4 = -999.0;  double min_z_n_4 = +999.0;
  double max_x_c_4 = -999.0;  double min_x_c_4 = +999.0;
  double max_y_c_4 = -999.0;  double min_y_c_4 = +999.0;
  double max_z_c_4 = -999.0;  double min_z_c_4 = +999.0;

  // search region (5)
  double max_x_n_5 = -999.0;  double min_x_n_5 = +999.0;
  double max_y_n_5 = -999.0;  double min_y_n_5 = +999.0;
  double max_z_n_5 = -999.0;  double min_z_n_5 = +999.0;
  double max_x_c_5 = -999.0;  double min_x_c_5 = +999.0;
  double max_y_c_5 = -999.0;  double min_y_c_5 = +999.0;
  double max_z_c_5 = -999.0;  double min_z_c_5 = +999.0;

  for(int i=ib_n;i<=ib_c;i++){
    
    /////////////////////
    // amino acid type //
    /////////////////////

    res_type_1[i-ib_n]=biounit[i].get_abbr_1();
    res_type_3[i-ib_n]=biounit[i].get_abbr_3();

    ////////////////
    // main chain //
    ////////////////

    get_atom_index_biounit_type(1,&i,0,&i_ca[i-ib_n]);
    get_atom_index_biounit_type(1,&i,2,&i_n [i-ib_n]);
    get_atom_index_biounit_type(1,&i,1,&i_c [i-ib_n]);
    R_ca[i-ib_n].copy(atom[i_ca[i-ib_n]].get_R());
    R_n [i-ib_n].copy(atom[i_n [i-ib_n]].get_R());
    R_c [i-ib_n].copy(atom[i_c [i-ib_n]].get_R());
    if   (R_n[i-ib_n].get_z()>R_ca[i-ib_n].get_z()) {res_n_pos[i-ib_n]="U";}
    else                                            {res_n_pos[i-ib_n]="D";}
    if   (R_c[i-ib_n].get_z()>R_ca[i-ib_n].get_z()) {res_c_pos[i-ib_n]="U";}
    else                                            {res_c_pos[i-ib_n]="D";}

    ////////////////
    // side chain //
    ////////////////

    R_sc[i-ib_n].copy(get_biounit_center_of_mass_side(i));
    if(res_type_3[i-ib_n]!="GLY"){
      if   (R_sc[i-ib_n].get_z()>R_ca[i-ib_n].get_z()) {res_sc_pos[i-ib_n]="U";}
      else                                             {res_sc_pos[i-ib_n]="D";}
    } else {
      res_sc_pos[i-ib_n]="*";
    }

    ///////////////////
    // teriminal ACE //
    ///////////////////

    if(i==ib_n){
      if(get_atom_index_biounit_type(1,&i,83,&i_ace_ca)>0){
        chk_ace=i;
        get_atom_index_biounit_type(1,&i,81,&i_ace_c);
        R_ace_c .copy(atom[i_ace_c ].get_R());
        R_ace_ca.copy(atom[i_ace_ca].get_R());
      }
    }

    ///////////////////
    // teriminal NME //
    ///////////////////

    if(i==ib_c){
      if(get_atom_index_biounit_type(1,&i,89,&i_nme_ca)>0){
        chk_nme=i;
        get_atom_index_biounit_type(1,&i,87,&i_nme_n);
        R_nme_n .copy(atom[i_nme_n ].get_R());
        R_nme_ca.copy(atom[i_nme_ca].get_R());
      }
    }
  }

  ////////////////////
  // U and D string //
  ////////////////////

  if(chk_ace==-1){
    UD_n=res_n_pos[0].c_str();
    for(int i=ib_n;i<(ib_n+5);i++){
      if(i<=ib_c){
        UD_n=UD_n+res_sc_pos[i-ib_n].c_str();
      }
    } 
  }
  if(chk_nme==-1){
    UD_c=res_c_pos[ib_c-ib_n];
    for(int i=ib_c;i>(ib_c-5);i--){
      if(i>=ib_n){
        UD_c=res_sc_pos[i-ib_n].c_str()+UD_c;
      }
    }
  }

  ////////////////////////////////
  // CA oder (good or not good) //
  ////////////////////////////////

  if(chk_ace==-1){
    chk_order_n=1;
    if(R_n [0].get_x()<R_ca[0].get_x()){} else {chk_order_n=0;}
    if((ib_c-ib_n)>0){if(R_ca[0].get_x()<R_ca[1].get_x()){} else {chk_order_n=0;}}
    if((ib_c-ib_n)>1){if(R_ca[1].get_x()<R_ca[2].get_x()){} else {chk_order_n=0;}}
    if((ib_c-ib_n)>2){if(R_ca[2].get_x()<R_ca[3].get_x()){} else {chk_order_n=0;}}
    if((ib_c-ib_n)>3){if(R_ca[3].get_x()<R_ca[4].get_x()){} else {chk_order_n=0;}}
  }
  if(chk_nme==-1){
    chk_order_c=1;
    if(R_c [ib_c-ib_n-0].get_x()>R_ca[ib_c-ib_n-0].get_x()){} else {chk_order_c=0;}
    if((ib_c-ib_n)>0){if(R_ca[ib_c-ib_n-0].get_x()>R_ca[ib_c-ib_n-1].get_x()){} else {chk_order_c=0;}}
    if((ib_c-ib_n)>1){if(R_ca[ib_c-ib_n-1].get_x()>R_ca[ib_c-ib_n-2].get_x()){} else {chk_order_c=0;}}
    if((ib_c-ib_n)>2){if(R_ca[ib_c-ib_n-2].get_x()>R_ca[ib_c-ib_n-3].get_x()){} else {chk_order_c=0;}}
    if((ib_c-ib_n)>3){if(R_ca[ib_c-ib_n-3].get_x()>R_ca[ib_c-ib_n-4].get_x()){} else {chk_order_c=0;}}
  }

  ///////////////////////
  // search region (3) //
  ///////////////////////
  
  if(chk_ace==-1){
    if((ib_c-ib_n)>1){  
      for(int i=ib_n;i<ib_n+3;i++){
        for(int j=0;j<biounit[i].get_n_atom_index();j++){
          r.copy(atom[biounit[i].get_atom_index(j)].get_R());
          if(max_x_n_3<r.get_x()){max_x_n_3=r.get_x();}
          if(max_y_n_3<r.get_y()){max_y_n_3=r.get_y();}
          if(max_z_n_3<r.get_z()){max_z_n_3=r.get_z();}
          if(min_x_n_3>r.get_x()){min_x_n_3=r.get_x();}
          if(min_y_n_3>r.get_y()){min_y_n_3=r.get_y();}
          if(min_z_n_3>r.get_z()){min_z_n_3=r.get_z();}
        }
      }
      if((ib_c-ib_n)>2){  
        for(int j=0;j<biounit[ib_n+3].get_n_atom_index();j++){
          if(biounit[ib_n+3].get_atom_type(j)==0 ||
             biounit[ib_n+3].get_atom_type(j)==2 ||
             biounit[ib_n+3].get_atom_type(j)==5 ){
            r.copy(atom[biounit[ib_n+3].get_atom_index(j)].get_R());
            if(max_x_n_3<r.get_x()){max_x_n_3=r.get_x();}
            if(max_y_n_3<r.get_y()){max_y_n_3=r.get_y();}
            if(max_z_n_3<r.get_z()){max_z_n_3=r.get_z();}
            if(min_x_n_3>r.get_x()){min_x_n_3=r.get_x();}
            if(min_y_n_3>r.get_y()){min_y_n_3=r.get_y();}
            if(min_z_n_3>r.get_z()){min_z_n_3=r.get_z();}
          }
        }
      }
    }
  }
  if(chk_nme==-1){
    if((ib_c-ib_n)>1){  
      for(int i=ib_c-2;i<=ib_c;i++){
        for(int j=0;j<biounit[i].get_n_atom_index();j++){
          r.copy(atom[biounit[i].get_atom_index(j)].get_R());
          if(max_x_c_3<r.get_x()){max_x_c_3=r.get_x();}
          if(max_y_c_3<r.get_y()){max_y_c_3=r.get_y();}
          if(max_z_c_3<r.get_z()){max_z_c_3=r.get_z();}
          if(min_x_c_3>r.get_x()){min_x_c_3=r.get_x();}
          if(min_y_c_3>r.get_y()){min_y_c_3=r.get_y();}
          if(min_z_c_3>r.get_z()){min_z_c_3=r.get_z();}
        }
      }
      if((ib_c-ib_n)>2){  
        for(int j=0;j<biounit[ib_c-3].get_n_atom_index();j++){
          if(biounit[ib_c-3].get_atom_type(j)==0 ||
             biounit[ib_c-3].get_atom_type(j)==1 ||
             biounit[ib_c-3].get_atom_type(j)==4 ){
            r.copy(atom[biounit[ib_c-3].get_atom_index(j)].get_R());
            if(max_x_c_3<r.get_x()){max_x_c_3=r.get_x();}
            if(max_y_c_3<r.get_y()){max_y_c_3=r.get_y();}
            if(max_z_c_3<r.get_z()){max_z_c_3=r.get_z();}
            if(min_x_c_3>r.get_x()){min_x_c_3=r.get_x();}
            if(min_y_c_3>r.get_y()){min_y_c_3=r.get_y();}
            if(min_z_c_3>r.get_z()){min_z_c_3=r.get_z();}
          }
        }
      }
    }
  }

  ///////////////////////
  // search region (4) //
  ///////////////////////

  if(chk_ace==-1){
    if((ib_c-ib_n)>2){  
      for(int i=ib_n;i<ib_n+4;i++){
        for(int j=0;j<biounit[i].get_n_atom_index();j++){
          r.copy(atom[biounit[i].get_atom_index(j)].get_R());
          if(max_x_n_4<r.get_x()){max_x_n_4=r.get_x();}
          if(max_y_n_4<r.get_y()){max_y_n_4=r.get_y();}
          if(max_z_n_4<r.get_z()){max_z_n_4=r.get_z();}
          if(min_x_n_4>r.get_x()){min_x_n_4=r.get_x();}
          if(min_y_n_4>r.get_y()){min_y_n_4=r.get_y();}
          if(min_z_n_4>r.get_z()){min_z_n_4=r.get_z();}
        }
      }
      if((ib_c-ib_n)>3){ 
        for(int j=0;j<biounit[ib_n+4].get_n_atom_index();j++){
          if(biounit[ib_n+4].get_atom_type(j)==0 ||
             biounit[ib_n+4].get_atom_type(j)==2 ||
             biounit[ib_n+4].get_atom_type(j)==5 ){
            r.copy(atom[biounit[ib_n+4].get_atom_index(j)].get_R());
            if(max_x_n_4<r.get_x()){max_x_n_4=r.get_x();}
            if(max_y_n_4<r.get_y()){max_y_n_4=r.get_y();}
            if(max_z_n_4<r.get_z()){max_z_n_4=r.get_z();}
            if(min_x_n_4>r.get_x()){min_x_n_4=r.get_x();}
            if(min_y_n_4>r.get_y()){min_y_n_4=r.get_y();}
            if(min_z_n_4>r.get_z()){min_z_n_4=r.get_z();}
          }
        }
      }
    }
  }
  if(chk_nme==-1){
    if((ib_c-ib_n)>2){  
      for(int i=ib_c-3;i<=ib_c;i++){
        for(int j=0;j<biounit[i].get_n_atom_index();j++){
          r.copy(atom[biounit[i].get_atom_index(j)].get_R());
          if(max_x_c_4<r.get_x()){max_x_c_4=r.get_x();}
          if(max_y_c_4<r.get_y()){max_y_c_4=r.get_y();}
          if(max_z_c_4<r.get_z()){max_z_c_4=r.get_z();}
          if(min_x_c_4>r.get_x()){min_x_c_4=r.get_x();}
          if(min_y_c_4>r.get_y()){min_y_c_4=r.get_y();}
          if(min_z_c_4>r.get_z()){min_z_c_4=r.get_z();}
        }
      }
      if((ib_c-ib_n)>3){  
        for(int j=0;j<biounit[ib_c-4].get_n_atom_index();j++){
          if(biounit[ib_c-4].get_atom_type(j)==0 ||
             biounit[ib_c-4].get_atom_type(j)==1 ||
             biounit[ib_c-4].get_atom_type(j)==4 ){
            r.copy(atom[biounit[ib_c-4].get_atom_index(j)].get_R());
            if(max_x_c_4<r.get_x()){max_x_c_4=r.get_x();}
            if(max_y_c_4<r.get_y()){max_y_c_4=r.get_y();}
            if(max_z_c_4<r.get_z()){max_z_c_4=r.get_z();}
            if(min_x_c_4>r.get_x()){min_x_c_4=r.get_x();}
            if(min_y_c_4>r.get_y()){min_y_c_4=r.get_y();}
            if(min_z_c_4>r.get_z()){min_z_c_4=r.get_z();}
          }
        }
      }
    }
  }

  ///////////////////////
  // search region (5) //
  ///////////////////////

  if(chk_ace==-1){
    if((ib_c-ib_n)>3){  
      for(int i=ib_n;i<ib_n+5;i++){
        for(int j=0;j<biounit[i].get_n_atom_index();j++){
          r.copy(atom[biounit[i].get_atom_index(j)].get_R());
          if(max_x_n_5<r.get_x()){max_x_n_5=r.get_x();}
          if(max_y_n_5<r.get_y()){max_y_n_5=r.get_y();}
          if(max_z_n_5<r.get_z()){max_z_n_5=r.get_z();}
          if(min_x_n_5>r.get_x()){min_x_n_5=r.get_x();}
          if(min_y_n_5>r.get_y()){min_y_n_5=r.get_y();}
          if(min_z_n_5>r.get_z()){min_z_n_5=r.get_z();}
        }
      }
      if((ib_c-ib_n)>4){
        for(int j=0;j<biounit[ib_n+5].get_n_atom_index();j++){
          if(biounit[ib_n+5].get_atom_type(j)==0 ||
             biounit[ib_n+5].get_atom_type(j)==2 ||
             biounit[ib_n+5].get_atom_type(j)==5 ){
            r.copy(atom[biounit[ib_n+5].get_atom_index(j)].get_R());
            if(max_x_n_5<r.get_x()){max_x_n_5=r.get_x();}
            if(max_y_n_5<r.get_y()){max_y_n_5=r.get_y();}
            if(max_z_n_5<r.get_z()){max_z_n_5=r.get_z();}
            if(min_x_n_5>r.get_x()){min_x_n_5=r.get_x();}
            if(min_y_n_5>r.get_y()){min_y_n_5=r.get_y();}
            if(min_z_n_5>r.get_z()){min_z_n_5=r.get_z();}
          }
        }
      }
    }
  }
  if(chk_nme==-1){
    if((ib_c-ib_n)>3){  
      for(int i=ib_c-4;i<=ib_c;i++){
        for(int j=0;j<biounit[i].get_n_atom_index();j++){
          r.copy(atom[biounit[i].get_atom_index(j)].get_R());
          if(max_x_c_5<r.get_x()){max_x_c_5=r.get_x();}
          if(max_y_c_5<r.get_y()){max_y_c_5=r.get_y();}
          if(max_z_c_5<r.get_z()){max_z_c_5=r.get_z();}
          if(min_x_c_5>r.get_x()){min_x_c_5=r.get_x();}
          if(min_y_c_5>r.get_y()){min_y_c_5=r.get_y();}
          if(min_z_c_5>r.get_z()){min_z_c_5=r.get_z();}
        }
      }
      if((ib_c-ib_n)>4){  
        for(int j=0;j<biounit[ib_c-5].get_n_atom_index();j++){
          if(biounit[ib_c-5].get_atom_type(j)==0 ||
             biounit[ib_c-5].get_atom_type(j)==1 ||
             biounit[ib_c-5].get_atom_type(j)==4 ){
            r.copy(atom[biounit[ib_c-5].get_atom_index(j)].get_R());
            if(max_x_c_5<r.get_x()){max_x_c_5=r.get_x();}
            if(max_y_c_5<r.get_y()){max_y_c_5=r.get_y();}
            if(max_z_c_5<r.get_z()){max_z_c_5=r.get_z();}
            if(min_x_c_5>r.get_x()){min_x_c_5=r.get_x();}
            if(min_y_c_5>r.get_y()){min_y_c_5=r.get_y();}
            if(min_z_c_5>r.get_z()){min_z_c_5=r.get_z();}
          }
        }
      }
    }
  }

  ///////////////////////
  ///// print title /////
  ///////////////////////

  sprintf(c,"  -----------------\n"  );s_out=s_out+c;
  sprintf(c,"   # HLA ANALYSIS  \n"  );s_out=s_out+c;
  sprintf(c,"  -----------------\n\n");s_out=s_out+c;

  /////////////////////////
  ///// print peptide /////
  /////////////////////////

  sprintf(c,"    PEPTIDE: %2d %3d %3d ",ib_c-ib_n+1,ib_n,ib_c);s_out=s_out+c;
  for(int i=ib_n;i<=ib_c;i++){
    sprintf(c,"%1s",res_type_1[i-ib_n].c_str());
    s_out=s_out+c;
  }
  sprintf(c,"\n\n");
  s_out=s_out+c;

  ////////////////
  // coordinate //
  ////////////////

  if(chk_ace>=0){
    sprintf(c,"        %3s%2d%2d : ACE-CA %4d %8.4f %8.4f %8.4f\n",
            res_type_3[ib_n].c_str(),ib_n+1,(ib_c-ib_n+1)-(ib_n-ib_n+1)+1,i_ace_ca+1,
            R_ace_ca.get_x()*0.5291,R_ace_ca.get_y()*0.5291,R_ace_ca.get_z()*0.5291);
    s_out=s_out+c;
    sprintf(c,"        %3s%2d%2d : ACE-C  %4d %8.4f %8.4f %8.4f\n",
            res_type_3[ib_n].c_str(),ib_n+1,(ib_c-ib_n+1)-(ib_n-ib_n+1)+1,i_ace_c+1,
            R_ace_c.get_x()*0.5291,R_ace_c.get_y()*0.5291,R_ace_c.get_z()*0.5291);
    s_out=s_out+c;
  }
  for(int i=ib_n;i<=ib_c;i++){
    sprintf(c,"        %3s%2d%2d : N      %4d %8.4f %8.4f %8.4f %1s\n",
            res_type_3[i-ib_n].c_str(),i-ib_n+1,(ib_c-ib_n+1)-(i-ib_n+1)+1,i_n [i-ib_n]+1,
            R_n [i-ib_n].get_x()*0.5291,R_n [i-ib_n].get_y()*0.5291,R_n [i-ib_n].get_z()*0.5291,res_n_pos[i-ib_n].c_str());
    s_out=s_out+c;
    sprintf(c,"        %3s%2d%2d : CA     %4d %8.4f %8.4f %8.4f %1s\n",
            res_type_3[i-ib_n].c_str(),i-ib_n+1,(ib_c-ib_n+1)-(i-ib_n+1)+1,i_ca[i-ib_n]+1,
            R_ca[i-ib_n].get_x()*0.5291,R_ca[i-ib_n].get_y()*0.5291,R_ca[i-ib_n].get_z()*0.5291," ");
    s_out=s_out+c;
    sprintf(c,"        %3s%2d%2d : C      %4d %8.4f %8.4f %8.4f %1s\n",
            res_type_3[i-ib_n].c_str(),i-ib_n+1,(ib_c-ib_n+1)-(i-ib_n+1)+1,i_c [i-ib_n]+1,
            R_c [i-ib_n].get_x()*0.5291,R_c [i-ib_n].get_y()*0.5291,R_c [i-ib_n].get_z()*0.5291,res_c_pos[i-ib_n].c_str());
    s_out=s_out+c;
    sprintf(c,"        %3s%2d%2d : SC          %8.4f %8.4f %8.4f %1s\n",
            res_type_3[i-ib_n].c_str(),i-ib_n+1,(ib_c-ib_n+1)-(i-ib_n+1)+1,
            R_sc[i-ib_n].get_x()*0.5291,R_sc[i-ib_n].get_y()*0.5291,R_sc[i-ib_n].get_z()*0.5291,res_sc_pos[i-ib_n].c_str());
    s_out=s_out+c;
  }
  if(chk_nme>=0){
    sprintf(c,"        %3s%2d%2d : NME-N  %4d %8.4f %8.4f %8.4f\n",
            res_type_3[ib_c].c_str(),ib_c+1,(ib_c-ib_n+1)-(ib_c-ib_n+1)+1,i_nme_n+1,
            R_nme_n.get_x()*0.5291,R_nme_n.get_y()*0.5291,R_nme_n.get_z()*0.5291);
    s_out=s_out+c;
    sprintf(c,"        %3s%2d%2d : NME-CA %4d %8.4f %8.4f %8.4f\n",
            res_type_3[ib_c].c_str(),ib_c+1,(ib_c-ib_n+1)-(ib_c-ib_n+1)+1,i_nme_ca+1,
            R_nme_ca.get_x()*0.5291,R_nme_ca.get_y()*0.5291,R_nme_ca.get_z()*0.5291);
    s_out=s_out+c;
  }

  sprintf(c,"\n");
  s_out=s_out+c;

  /////////////////////
  // print yes or no //
  /////////////////////

  if(chk_ace==-1){
    if(chk_order_n==1) { sprintf(c,"        NTERM-ORDER: YES\n");}
    if(chk_order_n==0) { sprintf(c,"        NTERM-ORDER: NO \n");}
    s_out=s_out+c;
  }
  if(chk_nme==-1){
    if(chk_order_c==1) { sprintf(c,"        CTERM-ORDER: YES\n");}
    if(chk_order_c==0) { sprintf(c,"        CTERM-ORDER: NO \n");}
    s_out=s_out+c;
  }
  sprintf(c,"\n\n");
  s_out=s_out+c;

  ///////////////////////
  ///// print nterm /////
  ///////////////////////

  if(chk_ace==-1){
    
    sprintf(c,"    NTERM: ");
    s_out=s_out+c;
    for(int i=ib_n;i<ib_n+4;i++){
      sprintf(c,"%1s",res_type_1[i-ib_n].c_str());
      s_out=s_out+c;
    }
    sprintf(c,"\n\n");
    s_out=s_out+c;
    
    ////////////////
    // coordinate //
    ////////////////

    for(int i=ib_n;i<ib_n+5;i++){
      if(i<=ib_c){
        sprintf(c,"        %3s%2d   : N       %8.4f %8.4f %8.4f %1s\n",
                res_type_3[i-ib_n].c_str(),i-ib_n+1,
                R_n [i-ib_n].get_x()*0.5291,R_n [i-ib_n].get_y()*0.5291,R_n [i-ib_n].get_z()*0.5291,res_n_pos[i-ib_n].c_str());
        s_out=s_out+c;
        sprintf(c,"        %3s%2d   : CA      %8.4f %8.4f %8.4f %1s\n",
                res_type_3[i-ib_n].c_str(),i-ib_n+1,
                R_ca[i-ib_n].get_x()*0.5291,R_ca[i-ib_n].get_y()*0.5291,R_ca[i-ib_n].get_z()*0.5291,"");
        s_out=s_out+c;
        sprintf(c,"        %3s%2d   : C       %8.4f %8.4f %8.4f %1s\n",
                res_type_3[i-ib_n].c_str(),i-ib_n+1,
                R_c [i-ib_n].get_x()*0.5291,R_c [i-ib_n].get_y()*0.5291,R_c [i-ib_n].get_z()*0.5291,res_c_pos[i-ib_n].c_str());
        s_out=s_out+c;
        sprintf(c,"        %3s%2d   : SC      %8.4f %8.4f %8.4f %1s\n",
                res_type_3[i-ib_n].c_str(),i-ib_n+1,
                R_sc[i-ib_n].get_x()*0.5291,R_sc[i-ib_n].get_y()*0.5291,R_sc[i-ib_n].get_z()*0.5291,res_sc_pos[i-ib_n].c_str());
        s_out=s_out+c;
      }
    }
    if(chk_nme>=0){
      sprintf(c,"        %3s%2d   : NME-N   %8.4f %8.4f %8.4f\n",
              res_type_3[ib_c].c_str(),ib_c+1,
              R_nme_n.get_x()*0.5291,R_nme_n.get_y()*0.5291,R_nme_n.get_z()*0.5291);
      s_out=s_out+c;
      sprintf(c,"        %3s%2d   : NME-CA  %8.4f %8.4f %8.4f\n",
              res_type_3[ib_c].c_str(),ib_c+1,
              R_nme_ca.get_x()*0.5291,R_nme_ca.get_y()*0.5291,R_nme_ca.get_z()*0.5291);
      s_out=s_out+c;
    }
    sprintf(c,"\n");
    s_out=s_out+c;
    
    //////////////
    // distance //
    //////////////

    sprintf(c,"          CA-CA-CA------- %10.4f  %-3s---\n",(R_ca[2].get_x()-R_ca[0].get_x())*0.5291,UD_n.substr(1,3).c_str());s_out=s_out+c;
    sprintf(c,"        N-CA-CA-CA------- %10.4f %-4s---\n" ,(R_ca[2].get_x()-R_n [0].get_x())*0.5291,UD_n.substr(0,4).c_str());s_out=s_out+c;
    sprintf(c,"          CA-CA-CA-CA---- %10.4f  %-4s--\n" ,(R_ca[3].get_x()-R_ca[0].get_x())*0.5291,UD_n.substr(1,4).c_str());s_out=s_out+c;
    sprintf(c,"        N-CA-CA-CA-CA---- %10.4f %-5s--\n"  ,(R_ca[3].get_x()-R_n [0].get_x())*0.5291,UD_n.substr(0,5).c_str());s_out=s_out+c;
    sprintf(c,"          CA-CA-CA-CA-CA- %10.4f  %-5s-\n"  ,(R_ca[4].get_x()-R_ca[0].get_x())*0.5291,UD_n.substr(1,5).c_str());s_out=s_out+c;
    sprintf(c,"        N-CA-CA-CA-CA-CA- %10.4f %-6s-\n"   ,(R_ca[4].get_x()-R_n [0].get_x())*0.5291,UD_n.substr(0,6).c_str());s_out=s_out+c;
    sprintf(c,"\n");
    s_out=s_out+c;
    
    if((ib_c-ib_n)>1){  

      ///////////////////////
      // search region (3) //
      ///////////////////////

      sprintf(c,"        NTERM SERCH REGION (3) MIN: %10.4f %10.4f %10.4f\n",
              min_x_n_3*0.5291,
              min_y_n_3*0.5291,
              min_z_n_3*0.5291);
      s_out=s_out+c;
      sprintf(c,"        NTERM SERCH REGION (3) MAX: %10.4f %10.4f %10.4f\n",
              max_x_n_3*0.5291,
              max_y_n_3*0.5291,
              max_z_n_3*0.5291);
      s_out=s_out+c;
      sprintf(c,"        NTERM SERCH REGION (3) LEN: %10.4f %10.4f %10.4f\n",
              (max_x_n_3-min_x_n_3)*0.5291,
              (max_y_n_3-min_y_n_3)*0.5291,
              (max_z_n_3-min_z_n_3)*0.5291);
      s_out=s_out+c;
      sprintf(c,"\n");
      s_out=s_out+c;
    }
    
    if((ib_c-ib_n)>2){  

      ///////////////////////
      // search region (4) //
      ///////////////////////

      sprintf(c,"        NTERM SERCH REGION (4) MIN: %10.4f %10.4f %10.4f\n",
              min_x_n_4*0.5291,
              min_y_n_4*0.5291,
              min_z_n_4*0.5291);
      s_out=s_out+c;
      sprintf(c,"        NTERM SERCH REGION (4) MAX: %10.4f %10.4f %10.4f\n",
              max_x_n_4*0.5291,
              max_y_n_4*0.5291,
              max_z_n_4*0.5291);
      s_out=s_out+c;
      sprintf(c,"        NTERM SERCH REGION (4) LEN: %10.4f %10.4f %10.4f\n",
              (max_x_n_4-min_x_n_4)*0.5291,
              (max_y_n_4-min_y_n_4)*0.5291,
              (max_z_n_4-min_z_n_4)*0.5291);
      s_out=s_out+c;
      sprintf(c,"\n");
      s_out=s_out+c;
    }

    if((ib_c-ib_n)>3){  

      ///////////////////////
      // search region (5) //
      ///////////////////////

      sprintf(c,"        NTERM SERCH REGION (5) MIN: %10.4f %10.4f %10.4f\n",
              min_x_n_5*0.5291,
              min_y_n_5*0.5291,
              min_z_n_5*0.5291);
      s_out=s_out+c;
      sprintf(c,"        NTERM SERCH REGION (5) MAX: %10.4f %10.4f %10.4f\n",
              max_x_n_5*0.5291,
              max_y_n_5*0.5291,
              max_z_n_5*0.5291);
      s_out=s_out+c;
      sprintf(c,"        NTERM SERCH REGION (5) LEN: %10.4f %10.4f %10.4f\n",
              (max_x_n_5-min_x_n_5)*0.5291,
              (max_y_n_5-min_y_n_5)*0.5291,
              (max_z_n_5-min_z_n_5)*0.5291);
      s_out=s_out+c;
      sprintf(c,"\n");
      s_out=s_out+c;
    }

    /////////////////////////
    ///// angle (Nterm) /////
    /////////////////////////
    
    int i_ang;
    for(int i=ib_n;i<=(ib_n+4);i++){
      if((i+1-ib_n)<=ib_c){
        r1.copy(R_n [i  -ib_n]);
        r2.copy(R_ca[i  -ib_n]);
        r3.copy(R_c [i  -ib_n]);
        r4.copy(R_n [i+1-ib_n]);
        angle=analysis_HLA_angle(r1,r2,r3,r4);

        // ishi 20151117
        if (angle<0.0) {angle=angle+3.141592*2.0;}

        sprintf(c,"        NTERM ANG %1d%1d%1d%1d : N (%2d,%4d) - CA(%2d,%4d) - C (%2d,%4d) - N (%2d,%4d) : %8.2f\n",
                i  -ib_n+1,
                i  -ib_n+1,
                i  -ib_n+1,
                i+1-ib_n+1,
                i  -ib_n+1,i_n [i  -ib_n]+1,
                i  -ib_n+1,i_ca[i  -ib_n]+1,
                i  -ib_n+1,i_c [i  -ib_n]+1,
                i+1-ib_n+1,i_n [i+1-ib_n]+1,angle*180/3.141592);
        s_out=s_out+c;
        r1.copy(R_ca[i  -ib_n]);
        r2.copy(R_c [i  -ib_n]);
        r3.copy(R_n [i+1-ib_n]);
        r4.copy(R_ca[i+1-ib_n]);
        angle=analysis_HLA_angle(r1,r2,r3,r4);

        // ishi 20151117
        if (angle<0.0) {angle=angle+3.141592*2.0;}

        sprintf(c,"        NTERM ANG %1d%1d%1d%1d : CA(%2d,%4d) - C (%2d,%4d) - N (%2d,%4d) - CA(%2d,%4d) : %8.2f\n",
                i  -ib_n+1,
                i  -ib_n+1,
                i+1-ib_n+1,
                i+1-ib_n+1,
                i  -ib_n+1,i_ca[i  -ib_n]+1,
                i  -ib_n+1,i_c [i  -ib_n]+1,
                i+1-ib_n+1,i_n [i+1-ib_n]+1,
                i+1-ib_n+1,i_ca[i+1-ib_n]+1,angle*180/3.141592);
        s_out=s_out+c;
        r1.copy(R_c [i  -ib_n]);
        r2.copy(R_n [i+1-ib_n]);
        r3.copy(R_ca[i+1-ib_n]);
        r4.copy(R_c [i+1-ib_n]);
        angle=analysis_HLA_angle(r1,r2,r3,r4);

        // ishi 20151117
        if (angle<0.0) {angle=angle+3.141592*2.0;}

        sprintf(c,"        NTERM ANG %1d%1d%1d%1d : C (%2d,%4d) - N (%2d,%4d) - CA(%2d,%4d) - C (%2d,%4d) : %8.2f\n",
                i  -ib_n+1,
                i+1-ib_n+1,
                i+1-ib_n+1,
                i+1-ib_n+1,
                i  -ib_n+1,i_c [i  -ib_n]+1,
                i+1-ib_n+1,i_n [i+1-ib_n]+1,
                i+1-ib_n+1,i_ca[i+1-ib_n]+1,
                i+1-ib_n+1,i_c [i+1-ib_n]+1,angle*180/3.141592);
        s_out=s_out+c;
        i_ang=i;
      }
    }
    if(chk_nme>=0){
      int i=i_ang;
      r1.copy(R_n [i+1-ib_n]);
      r2.copy(R_ca[i+1-ib_n]);
      r3.copy(R_c [i+1-ib_n]);
      r4.copy(R_nme_n       );
      angle=analysis_HLA_angle(r1,r2,r3,r4);

      // ishi 20151117
      if (angle<0.0) {angle=angle+3.141592*2.0;}

      sprintf(c,"        NTERM ANG %1d%1d%1d%1d : N (%2d,%4d) - CA(%2d,%4d) - C (%2d,%4d) - N (%2d,%4d) : %8.2f\n",
              i+1-ib_n+1,
              i+1-ib_n+1,
              i+1-ib_n+1,
              i+2-ib_n+1,
              i+1-ib_n+1,i_n [i+1-ib_n]+1,
              i+1-ib_n+1,i_ca[i+1-ib_n]+1,
              i+1-ib_n+1,i_c [i+1-ib_n]+1,
              i+2-ib_n+1,i_nme_n       +1,angle*180/3.141592);
      s_out=s_out+c;
      r1.copy(R_ca[i+1-ib_n]);
      r2.copy(R_c [i+1-ib_n]);
      r3.copy(R_nme_n       );
      r4.copy(R_nme_ca      );
      angle=analysis_HLA_angle(r1,r2,r3,r4);

      // ishi 20151117
      if (angle<0.0) {angle=angle+3.141592*2.0;}

      sprintf(c,"        NTERM ANG %1d%1d%1d%1d : CA(%2d,%4d) - C (%2d,%4d) - N (%2d,%4d) - CA(%2d,%4d) : %8.2f\n",
              i+1-ib_n+1,
              i+1-ib_n+1,
              i+1-ib_n+1,
              i+2-ib_n+1,
              i+1-ib_n+1,i_ca[i+1-ib_n]+1,
              i+1-ib_n+1,i_c [i+1-ib_n]+1,
              i+2-ib_n+1,i_nme_n       +1,
              i+2-ib_n+1,i_nme_ca      +1,angle*180/3.141592);
      s_out=s_out+c;
    }
    sprintf(c,"\n");
    s_out=s_out+c;
  }

  /////////////////////////////
  ///// CTERM (print out) /////
  /////////////////////////////

  if(chk_nme==-1){
  
    sprintf(c,"    CTERM: ");
    s_out=s_out+c;
    for(int i=ib_c-3;i<=ib_c;i++){
      sprintf(c,"%1s",res_type_1[i-ib_n].c_str());
      s_out=s_out+c;
    }
    sprintf(c,"\n\n");
    s_out=s_out+c;
    
    ////////////////
    // coordinate //
    ////////////////

    if(chk_ace>=0){
      sprintf(c,"        %3s  %2d : ACE-CA  %8.4f %8.4f %8.4f\n",
              res_type_3[ib_n].c_str(),(ib_c-ib_n+1)-(0-ib_n+1)+1,
              R_ace_ca.get_x()*0.5291,R_ace_ca.get_y()*0.5291,R_ace_ca.get_z()*0.5291);
      s_out=s_out+c;
      sprintf(c,"        %3s  %2d : ACE-N   %8.4f %8.4f %8.4f\n",
              res_type_3[ib_n].c_str(),(ib_c-ib_n+1)-(0-ib_n+1)+1,
              R_ace_c.get_x()*0.5291,R_ace_c.get_y()*0.5291,R_ace_c.get_z()*0.5291);
      s_out=s_out+c;
    }
    for(int i=ib_c-4;i<=ib_c;i++){
      if(i>=0){
        sprintf(c,"        %3s  %2d : N       %8.4f %8.4f %8.4f %1s\n",
                res_type_3[i-ib_n].c_str(),(ib_c-ib_n+1)-(i-ib_n+1)+1,
                R_n [i-ib_n].get_x()*0.5291,R_n [i-ib_n].get_y()*0.5291,R_n [i-ib_n].get_z()*0.5291,res_n_pos[i-ib_n].c_str());
        s_out=s_out+c;
        sprintf(c,"        %3s  %2d : CA      %8.4f %8.4f %8.4f %1s\n",
                res_type_3[i-ib_n].c_str(),(ib_c-ib_n+1)-(i-ib_n+1)+1,
                R_ca[i-ib_n].get_x()*0.5291,R_ca[i-ib_n].get_y()*0.5291,R_ca[i-ib_n].get_z()*0.5291," ");
        s_out=s_out+c;
        sprintf(c,"        %3s  %2d : C       %8.4f %8.4f %8.4f %1s\n",
                res_type_3[i-ib_n].c_str(),(ib_c-ib_n+1)-(i-ib_n+1)+1,
                R_c [i-ib_n].get_x()*0.5291,R_c [i-ib_n].get_y()*0.5291,R_c [i-ib_n].get_z()*0.5291,res_c_pos[i-ib_n].c_str());
        s_out=s_out+c;
        sprintf(c,"        %3s  %2d : SC      %8.4f %8.4f %8.4f %1s\n",
                res_type_3[i-ib_n].c_str(),(ib_c-ib_n+1)-(i-ib_n+1)+1,
                R_sc[i-ib_n].get_x()*0.5291,R_sc[i-ib_n].get_y()*0.5291,R_sc[i-ib_n].get_z()*0.5291,res_sc_pos[i-ib_n].c_str());
        s_out=s_out+c;
      }
    }
    sprintf(c,"\n");
    s_out=s_out+c;
    
    //////////////
    // distance //
    //////////////

    sprintf(c,"        -------CA-CA-CA   %10.4f ---%3s \n",(R_ca[ib_c-ib_n+0].get_x()-R_ca[ib_c-ib_n-2].get_x())*0.5291,UD_c.substr(2,3).c_str());s_out=s_out+c;
    sprintf(c,"        -------CA-CA-CA-C %10.4f ---%4s\n" ,(R_c [ib_c-ib_n+0].get_x()-R_ca[ib_c-ib_n-2].get_x())*0.5291,UD_c.substr(2,4).c_str());s_out=s_out+c;
    sprintf(c,"        ----CA-CA-CA-CA   %10.4f --%4s \n" ,(R_ca[ib_c-ib_n+0].get_x()-R_ca[ib_c-ib_n-3].get_x())*0.5291,UD_c.substr(1,4).c_str());s_out=s_out+c;
    sprintf(c,"        ----CA-CA-CA-CA-C %10.4f --%5s\n"  ,(R_c [ib_c-ib_n+0].get_x()-R_ca[ib_c-ib_n-3].get_x())*0.5291,UD_c.substr(1,5).c_str());s_out=s_out+c;
    sprintf(c,"        -CA-CA-CA-CA-CA   %10.4f -%5s \n"  ,(R_ca[ib_c-ib_n+0].get_x()-R_ca[ib_c-ib_n-4].get_x())*0.5291,UD_c.substr(0,5).c_str());s_out=s_out+c;
    sprintf(c,"        -CA-CA-CA-CA-CA-C %10.4f -%6s\n"   ,(R_c [ib_c-ib_n+0].get_x()-R_ca[ib_c-ib_n-4].get_x())*0.5291,UD_c.substr(0,6).c_str());s_out=s_out+c;
    sprintf(c,"\n");
    s_out=s_out+c;
    
    if((ib_c-ib_n)>1){  

      ///////////////////////
      // search region (3) //
      ///////////////////////

      sprintf(c,"        CTERM SERCH REGION (3) MIN: %10.4f %10.4f %10.4f\n",
              min_x_c_3*0.5291,
              min_y_c_3*0.5291,
              min_z_c_3*0.5291);
      s_out=s_out+c;
      sprintf(c,"        CTERM SERCH REGION (3) MAX: %10.4f %10.4f %10.4f\n",
              max_x_c_3*0.5291,
              max_y_c_3*0.5291,
              max_z_c_3*0.5291);
      s_out=s_out+c;
      sprintf(c,"        CTERM SERCH REGION (3) LEN: %10.4f %10.4f %10.4f\n",
              (max_x_c_3-min_x_c_3)*0.5291,
              (max_y_c_3-min_y_c_3)*0.5291,
              (max_z_c_3-min_z_c_3)*0.5291);
      s_out=s_out+c;
      sprintf(c,"\n");
      s_out=s_out+c;
    }
    
    if((ib_c-ib_n)>2){  

      ///////////////////////
      // search region (4) //
      ///////////////////////

      sprintf(c,"        CTERM SERCH REGION (4) MIN: %10.4f %10.4f %10.4f\n",
              min_x_c_4*0.5291,
              min_y_c_4*0.5291,
              min_z_c_4*0.5291);
      s_out=s_out+c;
      sprintf(c,"        CTERM SERCH REGION (4) MAX: %10.4f %10.4f %10.4f\n",
              max_x_c_4*0.5291,
              max_y_c_4*0.5291,
              max_z_c_4*0.5291);
      s_out=s_out+c;
      sprintf(c,"        CTERM SERCH REGION (4) LEN: %10.4f %10.4f %10.4f\n",
              (max_x_c_4-min_x_c_4)*0.5291,
              (max_y_c_4-min_y_c_4)*0.5291,
              (max_z_c_4-min_z_c_4)*0.5291);
      s_out=s_out+c;
      sprintf(c,"\n");
      s_out=s_out+c;
    }

    if((ib_c-ib_n)>3){  

      ///////////////////////
      // search region (5) //
      ///////////////////////

      sprintf(c,"        CTERM SERCH REGION (5) MIN: %10.4f %10.4f %10.4f\n",
              min_x_c_5*0.5291,
              min_y_c_5*0.5291,
              min_z_c_5*0.5291);
      s_out=s_out+c;
      sprintf(c,"        CTERM SERCH REGION (5) MAX: %10.4f %10.4f %10.4f\n",
              max_x_c_5*0.5291,
              max_y_c_5*0.5291,
              max_z_c_5*0.5291);
      s_out=s_out+c;
      sprintf(c,"        CTERM SERCH REGION (5) LEN: %10.4f %10.4f %10.4f\n",
              (max_x_c_5-min_x_c_5)*0.5291,
              (max_y_c_5-min_y_c_5)*0.5291,
              (max_z_c_5-min_z_c_5)*0.5291);
      s_out=s_out+c;
      sprintf(c,"\n");
      s_out=s_out+c;
    }
    
    /////////////////////////
    ///// angle (Cterm) /////
    /////////////////////////

    int ang_ace_chk=0;
    for(int i=ib_c-5;i<=(ib_c-1);i++){
      if((-ib_n+i)>=0){
        if(ang_ace_chk==0){
          if(chk_ace>=0){
            r1.copy(R_ace_ca);
            r2.copy(R_ace_c);
            r3.copy(R_n [-ib_n+i  ]);
            r4.copy(R_ca[-ib_n+i  ]);
            angle=analysis_HLA_angle(r1,r2,r3,r4);

            // ishi 20151117
            if (angle<0.0) {angle=angle+3.141592*2.0;}

            sprintf(c,"        CTERM ANG %1d%1d%1d%1d : CA(%2d,%4d) - C (%2d,%4d) - N (%2d,%4d) - CA(%2d,%4d) : %8.2f\n",
                    ib_c-(i-1)+1,
                    ib_c-(i-1)+1,
                    ib_c-(i  )+1,
                    ib_c-(i  )+1,
                    ib_c-(i-1)+1,i_ace_ca       +1,
                    ib_c-(i-1)+1,i_ace_c        +1,
                    ib_c-(i  )+1,i_n [-ib_n+i  ]+1,
                    ib_c-(i  )+1,i_ca[-ib_n+i  ]+1,angle*180/3.141592);
            s_out=s_out+c;
            r1.copy(R_ace_c);
            r2.copy(R_n [-ib_n+i  ]);
            r3.copy(R_ca[-ib_n+i  ]);
            r4.copy(R_c [-ib_n+i  ]);
            angle=analysis_HLA_angle(r1,r2,r3,r4);

            // ishi 20151117
            if (angle<0.0) {angle=angle+3.141592*2.0;}

            sprintf(c,"        CTERM ANG %1d%1d%1d%1d : C (%2d,%4d) - N (%2d,%4d) - CA(%2d,%4d) - C (%2d,%4d) : %8.2f\n",
                    ib_c-(i-1)+1,
                    ib_c-(i  )+1,
                    ib_c-(i  )+1,
                    ib_c-(i  )+1,
                    ib_c-(i-1)+1,i_ace_c        +1,
                    ib_c-(i  )+1,i_n [-ib_n+i  ]+1,
                    ib_c-(i  )+1,i_ca[-ib_n+i  ]+1,
                    ib_c-(i  )+1,i_c [-ib_n+i  ]+1,angle*180/3.141592);
            s_out=s_out+c;
          }
          ang_ace_chk=1;
        }
        r1.copy(R_n [-ib_n+i  ]);
        r2.copy(R_ca[-ib_n+i  ]);
        r3.copy(R_c [-ib_n+i  ]);
        r4.copy(R_n [-ib_n+i+1]);
        angle=analysis_HLA_angle(r1,r2,r3,r4);

        // ishi 20151117
        if (angle<0.0) {angle=angle+3.141592*2.0;}

        sprintf(c,"        CTERM ANG %1d%1d%1d%1d : N (%2d,%4d) - CA(%2d,%4d) - C (%2d,%4d) - N (%2d,%4d) : %8.2f\n",
                ib_c-(i  )+1,
                ib_c-(i  )+1,
                ib_c-(i  )+1,
                ib_c-(i+1)+1,
                ib_c-(i  )+1,i_n [-ib_n+i  ]+1,
                ib_c-(i  )+1,i_ca[-ib_n+i  ]+1,
                ib_c-(i  )+1,i_c [-ib_n+i  ]+1,
                ib_c-(i+1)+1,i_n [-ib_n+i+1]+1,angle*180/3.141592);
        s_out=s_out+c;
        r1.copy(R_ca[-ib_n+i  ]);
        r2.copy(R_c [-ib_n+i  ]);
        r3.copy(R_n [-ib_n+i+1]);
        r4.copy(R_ca[-ib_n+i+1]);
        angle=analysis_HLA_angle(r1,r2,r3,r4);

        // ishi 20151117
        if (angle<0.0) {angle=angle+3.141592*2.0;}

        sprintf(c,"        CTERM ANG %1d%1d%1d%1d : CA(%2d,%4d) - C (%2d,%4d) - N (%2d,%4d) - CA(%2d,%4d) : %8.2f\n",
                ib_c-(i  )+1,
                ib_c-(i  )+1,
                ib_c-(i+1)+1,
                ib_c-(i+1)+1,
                ib_c-(i  )+1,i_ca[-ib_n+i  ]+1,
                ib_c-(i  )+1,i_c [-ib_n+i  ]+1,
                ib_c-(i+1)+1,i_n [-ib_n+i+1]+1,
                ib_c-(i+1)+1,i_ca[-ib_n+i+1]+1,angle*180/3.141592);
        s_out=s_out+c;
        r1.copy(R_c [-ib_n+i  ]);
        r2.copy(R_n [-ib_n+i+1]);
        r3.copy(R_ca[-ib_n+i+1]);
        r4.copy(R_c [-ib_n+i+1]);
        angle=analysis_HLA_angle(r1,r2,r3,r4);

        // ishi 20151117
        if (angle<0.0) {angle=angle+3.141592*2.0;}

        sprintf(c,"        CTERM ANG %1d%1d%1d%1d : C (%2d,%4d) - N (%2d,%4d) - CA(%2d,%4d) - C (%2d,%4d) : %8.2f\n",
                ib_c-(i  )+1,
                ib_c-(i+1)+1,
                ib_c-(i+1)+1,
                ib_c-(i+1)+1,
                ib_c-(i  )+1,i_c [-ib_n+i  ]+1,
                ib_c-(i+1)+1,i_n [-ib_n+i+1]+1,
                ib_c-(i+1)+1,i_ca[-ib_n+i+1]+1,
                ib_c-(i+1)+1,i_c [-ib_n+i+1]+1,angle*180/3.141592);
        s_out=s_out+c;
      }
    }
    sprintf(c,"\n");
    s_out=s_out+c;
  }
}


