
#include <cstdlib>
#include <cstdio>
#include <iostream>
#include <math.h>
#include "Molecule.h"
#include "ForceField.h"

extern int pv_debug; // for debug , 20160827

Molecule::Molecule()
{
  natom=0;
  nbond=0;
  nfragment=0;
  nlinkgroup=0;
  nbiounit=0;
  natom_buff=0;
  nbond_buff=0;
  nfragment_buff=0;
  nbiounit_buff=0;
  nlinkgroup_buff=0;

  // 20150121
  nangle=0;
  nangle_buff=0;
  ndihedral=0;
  ndihedral_buff=0;
}

Molecule::Molecule(const Molecule& m)
{
  natom=0;
  nbond=0;
  nfragment=0;
  nbiounit=0;
  nlinkgroup=0;
  natom_buff=0;
  nbond_buff=0;
  nfragment_buff=0;
  nbiounit_buff=0;
  nlinkgroup_buff=0;

  // 20150121
  nangle=0;
  nangle_buff=0;
  ndihedral=0;
  ndihedral_buff=0;

  copy(m);
}

Molecule::~Molecule()
{
  if(pv_debug==1){ cout << "ishi in  ~Molecule()" << endl ;}
  clear();
  if(pv_debug==1){ cout << "ishi out ~Molecule()" << endl ;}
}

void Molecule::copy(const Molecule& m)
{
  if(this==&m) return;

  create_atom(m.natom_buff);
  for(int i=0;i<m.natom;i++){
    atom[i].copy(m.atom[i]);
    natom++;
  }

  create_bond(m.nbond);
  for(int i=0;i<m.nbond;i++){
    bond[i].copy(m.bond[i]);
    nbond++;
  }

  create_fragment(m.nfragment);
  for(int i=0;i<m.nfragment;i++){
    fragment[i].copy(m.fragment[i]);
    nfragment++;
  }

  create_biounit(m.nbiounit);
  for(int i=0;i<m.nbiounit;i++){
    biounit[i].copy(m.biounit[i]);
    nbiounit++;
  }

  create_linkgroup(m.nlinkgroup);
  for(int i=0;i<m.nlinkgroup;i++){
    linkgroup[i].copy(m.linkgroup[i]);
    nlinkgroup++;
  }
}

void Molecule::copy_only(const Molecule& m)
{
  if(this==&m) return;

  create_atom(m.natom_buff);
  for(int i=0;i<m.natom;i++){
    atom[i].copy(m.atom[i]);
    natom++;
  }
  
  create_bond(m.nbond);
  for(int i=0;i<m.nbond;i++){
    bond[i].copy(m.bond[i]);
    nbond++;
  }

  create_fragment(m.nfragment);
  for(int i=0;i<m.nfragment;i++){
    fragment[i].copy(m.fragment[i]);
    nfragment++;
  }

  create_biounit(m.nbiounit);
  for(int i=0;i<m.nbiounit;i++){
    biounit[i].copy(m.biounit[i]);
    nbiounit++;
  }

  create_linkgroup(m.nlinkgroup);
  for(int i=0;i<m.nlinkgroup;i++){
    linkgroup[i].copy(m.linkgroup[i]);
    nlinkgroup++;
  }
}

void Molecule::copy_update_coord(const Molecule& m)
{
  for(int i=0;i<m.natom;i++){
    atom[i].set_R(m.atom[i].get_R());
  }
}

void Molecule::copy_update_bond(const Molecule& m)
{
  for(int i=0;i<m.nbond;i++){
    bond[i].set_iatom1(m.bond[i].get_iatom1());
    bond[i].set_iatom2(m.bond[i].get_iatom2());
  }
}

void Molecule::clear()
{
  if(pv_debug==1){ cout << "ishi in  Molecule::clear()" << endl ;}
  clear_atom     ();
  clear_bond     ();
  clear_fragment ();
  clear_biounit  ();
  clear_linkgroup();
  if(pv_debug==1){ cout << "ishi out Molecule::clear()" << endl ;}
}

void Molecule::add_chk_view_on ( const int itype,
                                 const int ilist)
{
  // type 0  : moleucle
  // type 1  : linkgroup
  // type 2  : biounit
  // type 3  : fragment
  // type 4  : main_chain
  // type 10 : atom
  // type 11 : bond

  if(itype==0){
    for(int i=0;i<natom;i++){
      atom[i].set_chk_view(1) ;
    }
  }
  if(itype==1){
    LinkGroup l(get_linkgroup(ilist));
    for(int i=0;i<l.get_n_atom_index();i++){
      atom[l.get_atom_index(i)].set_chk_view(1);
    }
  }
  if(itype==2){
    BioUnit b(get_biounit(ilist));
    for(int i=0;i<b.get_n_atom_index();i++){
      atom[b.get_atom_index(i)].set_chk_view(1);
    }
  }
  if(itype==3){
    Fragment f(get_fragment(ilist));
    for(int i=0;i<f.get_n_atom_index();i++){
      atom[f.get_atom_index(i)].set_chk_view(1);
    }
  }
  if(itype==4){
    for(int i=0;i<natom;i++){
      if(atom[i].get_bio1()<3){
        atom[i].set_chk_view(1);
      }
    }
  }
  if(itype==10){
    atom[ilist].set_chk_view(1);
  }
  if(itype==11){
    Bond b(get_bond(ilist));
    atom[b.get_iatom1()].set_chk_view(1);
    atom[b.get_iatom2()].set_chk_view(1);
  }
}

void Molecule::set_chk_view_on_selected()
{
  set_chk_view_bond_on_selected();
  set_chk_view_bio_on_selected();
  set_chk_view_frag_on_selected();
}

int Molecule::get_chk_view_atom ( const int iatom ) const 
{
  if(iatom>=natom){
    cerr << "error : Molecule::get_chk_view_atom , iatom >= natom " << endl ;
    cerr << "iatom = " << iatom << " , natom = " << natom << " , class = " << this << endl ;
    exit(1);
  }
  if(iatom<0){
    cerr << "error : Molecule::get_chk_view_atom , iatom < 0 " << endl ;
    cerr << "iatom = " << iatom << " , class = " << this << endl ;
    exit(1);
  }
  return atom[iatom].get_chk_view();
}

void Molecule::get_chk_view_atom_selected ( int* n   ,
                                            int* buf ) const
{
  (*n)=0;
  for(int i=0;i<natom;i++){
    if(atom[i].get_chk_view()==1){
      buf[(*n)]=i;
      (*n)++;
    }
  }
}

void Molecule::set_chk_view_atom_on( const int iatom )
{
  if(iatom>=natom){
    cerr << "error : Molecule::set_chk_view_atom_on , iatom >= natom " << endl ;
    cerr << "iatom = " << iatom << " , natom = " << natom << " , class = " << this << endl ;
    exit(1);
  }
  if(iatom<0){
    cerr << "error : Molecule::set_chk_view_atom_on , iatom < 0 " << endl ;
    cerr << "iatom = " << iatom << " , class = " << this << endl ;
    exit(1);
  }
  atom[iatom].set_chk_view(1);
}

void Molecule::set_chk_view_atom_on_all()
{
  for(int i=0;i<natom;i++){
    atom[i].set_chk_view(1);
  }
}

void Molecule::set_chk_view_atom_off( const int iatom )
{
  if(iatom>=natom){
    cerr << "error : Molecule::set_chk_view_atom_off , iatom >= chk_atom_n " << endl ;
    cerr << "iatom = " << iatom << " , natom = " << natom << " , class = " << this << endl ;
    exit(1);
  }
  if(iatom<0){
    cerr << "error : Molecule::set_chk_view_atom_off , iatom < 0 " << endl ;
    cerr << "iatom = " << iatom << " , class = " << this << endl ;
    exit(1);
  }
  atom[iatom].set_chk_view(-1);
}

void Molecule::set_chk_view_atom_off_all()
{
  for(int i=0;i<natom;i++){
    atom[i].set_chk_view(-1);
  }
}

int Molecule::get_chk_view_bond( const int ibond ) const 
{
  if(ibond>=nbond){
    cerr << "error : Molecule::get_chk_view_bond , ibond >= nbond " << endl ;
    cerr << "ibond = " << ibond << " , nbond = " << nbond << " , class = " << this << endl ;
    exit(1);
  }
  if(ibond<0){
    cerr << "error : Molecule::get_chk_view_bond , ibond <0 " << endl ;
    cerr << "ibond = " << ibond << " , class = " << this << endl ;
    exit(1);
  }
  return bond[ibond].get_chk_view();
}

void Molecule::set_chk_view_bond_on( const int ibond )
{
  if(ibond>=nbond){
    cerr << "error : Molecule::set_chk_view_bond_on , ibond >= nbond " << endl ;
    cerr << "ibond = " << ibond << " , nbond = " << nbond << " , class = " << this << endl ;
    exit(1);
  }
  if(ibond<0){
    cerr << "error : Molecule::set_chk_view_bond_on , ibond < 0 " << endl ;
    cerr << "ibond = " << ibond << " , class = " << this << endl ;
    exit(1);
  }
  bond[ibond].set_chk_view(1);
}

void Molecule::set_chk_view_bond_on_all()
{
  for(int i=0;i<nbond;i++){
    bond[i].set_chk_view(1);
  }
}

void Molecule::set_chk_view_bond_off( const int ibond )
{
  if(ibond>=nbond){
    cerr << "error : Molecule::set_chk_view_bond_off , ibond >= nbond " << endl ;
    cerr << "ibond = " << ibond << " , chk_bond_n = " << nbond << " , class = " << this << endl ;
    exit(1);
  }
  if(ibond<0){
    cerr << "error : Molecule::set_chk_view_bond_off , ibond < 0 " << endl ;
    cerr << "ibond = " << ibond << " , class = " << this << endl ;
    exit(1);
  }
  bond[ibond].set_chk_view(-1);
}

void Molecule::set_chk_view_bond_off_all()
{
  for(int i=0;i<nbond;i++){
    bond[i].set_chk_view(-1);
  }
}

void Molecule::set_chk_view_bond_on_selected()
{
  set_chk_view_bond_off_all();
  int atom1,atom2;
  for(int i=0;i<natom;i++){
    if(atom[i].get_chk_view()==1){
      for(int j=0;j<nbond;j++){
        atom1=bond[j].get_iatom1();
        atom2=bond[j].get_iatom2();
        if(atom1==i || atom2==i){
          bond[j].set_chk_view(1);
	}
      }
    }
  }
}

int Molecule::get_chk_view_bio( const int ibio ) const
{
  if(ibio>=nbiounit){
    cerr << "error : Molecule::get_chk_view_bio , ibio >= nbiounit " << endl ;
    cerr << "ibio = " << ibio << " , nbiounit = " << nbiounit << " , class = " << this << endl ;
    exit(1);
  }
  if(ibio<0){
    cerr << "error : Molecule::get_chk_view_bio , ibio >= 0 " << endl ;
    cerr << "ibio = " << ibio << " , class = " << this << endl ;
    exit(1);
  }
  return biounit[ibio].get_chk_view();
}
 
void Molecule::set_chk_view_bio_on( const int ibio )
{
  if(ibio>=nbiounit){
    cerr << "error : Molecule::set_chk_view_bio_on , ibio >= nbiounit " << endl ;
    cerr << "ibio = " << ibio << " , nbiounit = " << nbiounit << " , class = " << this << endl ;
    exit(1);
  }
  if(ibio<0){
    cerr << "error : Molecule::set_chk_view_bio_on , ibio < 0 " << endl ;
    cerr << "ibio = " << ibio << " , class = " << this << endl ;
    exit(1);
  }
  biounit[ibio].set_chk_view(1);
}

void Molecule::set_chk_view_bio_on_all()
{
  for(int i=0;i<nbiounit;i++){
    biounit[i].set_chk_view(1);
  }
}

void Molecule::set_chk_view_bio_off( const int ibio )
{
  if(ibio>=nbiounit){
    cerr << "error : Molecule::set_chk_view_bio_off , ibio >= nbiounit " << endl ;
    cerr << "ibio = " << ibio << " , nbiounit = " << nbiounit << " , class = " << this << endl ;
    exit(1);
  }
  if(ibio<0){
    cerr << "error : Molecule::set_chk_view_bio_off , ibio < 0 " << endl ;
    cerr << "ibio = " << ibio << " , class = " << this << endl ;
    exit(1);
  }
  biounit[ibio].set_chk_view(-1);
}

void Molecule::set_chk_view_bio_off_all()
{
  for(int i=0;i<nbiounit;i++){
    biounit[i].set_chk_view(-1);
  }
}

void Molecule::set_chk_view_bio_on_selected()
{
  set_chk_view_bio_off_all();
  int * list = new int [natom+1];
  get_list_atom_to_biounit(list);
  for(int i=0;i<natom;i++){
    if(atom[i].get_chk_view()==1){
      if(list[i]>=0){
	set_chk_view_bio_on(list[i]);
      }
    }
  }
  delete [] list ;
}

int Molecule::get_chk_view_frag( const int ifrag ) const
{
  if(ifrag>=nfragment){
    cerr << "error : Molecule::get_chk_view_frag , ifrag >= nfragment " << endl ;
    cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , class = " << this << endl ;
    exit(1);
  }
  if(ifrag<0){
    cerr << "error : Molecule::get_chk_view_frag , ifrag < 0 " << endl ;
    cerr << "ifrag = " << ifrag << " , class = " << this << endl ;
    exit(1);
  }
  return fragment[ifrag].get_chk_view();
}

void Molecule::set_chk_view_frag_on( const int ifrag )
{
  if(ifrag>=nfragment){
    cerr << "error : Molecule::set_chk_view_frag_on , ifrag >= nfragment " << endl ;
    cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , class = " << this << endl ;
    exit(1);
  }
  if(ifrag<0){
    cerr << "error : Molecule::set_chk_view_frag_on , ifrag < 0 " << endl ;
    cerr << "ifrag = " << ifrag << " , class = " << this << endl ;
    exit(1);
  }
  fragment[ifrag].set_chk_view(1);
}

void Molecule::set_chk_view_frag_on_all()
{
  for(int i=0;i<nfragment;i++){
    fragment[i].set_chk_view(1);
  }
}

void Molecule::set_chk_view_frag_off( const int ifrag )
{
  if(ifrag>=nfragment){
    cerr << "error : Molecule::set_chk_view_frag_off , ifrag >= nfragment " << endl ;
    cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , class = " << this << endl ;
    exit(1);
  }
  if(ifrag<0){
    cerr << "error : Molecule::set_chk_view_frag_off , ifrag < 0 " << endl ;
    cerr << "ifrag = " << ifrag << " , class = " << this << endl ;
    exit(1);
  }
  fragment[ifrag].set_chk_view(-1);
}

void Molecule::set_chk_view_frag_off_all()
{
  for(int i=0;i<nfragment;i++){
    fragment[i].set_chk_view(-1);
  }
}

void Molecule::set_chk_view_frag_on_selected()
{
  set_chk_view_frag_off_all() ;
  int * list = new int [natom];
  get_list_atom_to_fragment(list);
  for(int i=0;i<natom;i++){
    if(atom[i].get_chk_view()==1){
      if(list[i]>=0){
	set_chk_view_frag_on(list[i]);
      }
    }
  }
  delete [] list ;
}

void Molecule::get_bond_bond(const int ibond,
			     int * n1,
			     int * n2,
			     int * bond1_,
			     int * bond2_) const
{
  int j,ia1,ia2,ib1,ib2;
  ia1=bond[ibond].get_iatom1();
  ia2=bond[ibond].get_iatom2();
  *n1=0;
  *n2=0;
  for(int j=0;j<nbond;j++){
    ib1=bond[j].get_iatom1();
    ib2=bond[j].get_iatom2();
    if(ia1==ib1 && j!=ibond){bond1_[*n1]=j;(*n1)++;}
    if(ia1==ib2 && j!=ibond){bond1_[*n1]=j;(*n1)++;}
    if(ia2==ib1 && j!=ibond){bond2_[*n2]=j;(*n2)++;}
    if(ia2==ib2 && j!=ibond){bond2_[*n2]=j;(*n2)++;}
  }
}
