
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <sstream>
#include <fstream>
#include <math.h>
#include "Molecule.h"
#include "iString.h"

using namespace std ;

//---------------------------//
//----- read coordinate -----//
//---------------------------//

void Molecule::read_coord( const string& filename ,
                           const int type         ,
                           string& s_out          ,
			   const string& option1  ,
                           const string& file_string )
{

  // option1 == from-string --> read form string
  // option1 == ana-pdb --> pdb analysis

  if(type== 1){read_coord_paics_out  ( filename , s_out );}
  if(type== 2){read_coord_paics_in   ( filename , s_out );}
  if(type== 4){read_coord_gamess_out ( filename , s_out );}
  if(type== 5){read_coord_pdb        ( filename , s_out , option1 , file_string);}
  if(type== 6){read_coord_amber      ( filename , s_out );}
  if(type== 7){read_coord_xyz        ( filename , s_out );}
  if(type== 8){read_coord_siesta     ( filename , s_out );}
  if(type== 9){read_coord_mol2       ( filename , s_out );}
  if(type==10){read_coord_pdb        ( filename , s_out , "pdb-ana");}
  if(type==11){read_coord_pdb        ( filename , s_out , "pdb-ana");}
  if(type==99){read_coord_tmp        ( filename , s_out );}
  char c[128];
  sprintf(c,"  -----------------------------\n"                             );s_out=s_out+c;
  sprintf(c,"   # read molecule coordinate  \n"                             );s_out=s_out+c;
  sprintf(c,"  -----------------------------\n\n"                           );s_out=s_out+c;
  sprintf(c,"   number of atoms              = %6d\n"  ,get_natom()         );s_out=s_out+c;
  sprintf(c,"   number of atoms ( hydrogen ) = %6d\n\n",get_natom_atomic(1) );s_out=s_out+c;
}

void Molecule::read_coord_gamess_out(const string& filename,
                                     string& s_out)
{
  char line[1024];
  char buff[1024];
  double x,y,z,charge,dtmp;
  Atom a;
  Dvector3D r;
  int number,chk1;
  fstream file(filename.c_str(), std::ios::in);
  //if(file==NULL){ <--- 20150828
  if(!file){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return;
  }
  clear();
  chk1=0;
  while(! file.eof()){
    file.getline(line, sizeof(line));
    if(iFind(line, "ATOM      ATOMIC                      COORDINATES (BOHR)")>=0){
      chk1=1;
      file.getline(line, sizeof(line));
      file.getline(line, sizeof(line));
      while(!line[0]=='\0'){
        iPickUp(line,1,buff); sscanf(buff,"%lf",&dtmp);charge=dtmp; //iPickUp(line,1,buff); charge=atof(buff); 
        iPickUp(line,2,buff); sscanf(buff,"%lf",&dtmp);x     =dtmp; //iPickUp(line,2,buff); x     =atof(buff); 
        iPickUp(line,3,buff); sscanf(buff,"%lf",&dtmp);y     =dtmp; //iPickUp(line,3,buff); y     =atof(buff); 
        iPickUp(line,4,buff); sscanf(buff,"%lf",&dtmp);z     =dtmp; //iPickUp(line,4,buff); z     =atof(buff); 
        if(charge-floor(charge)>0.5){
          number = (int) (charge+0.9);
        } else {
          number = (int) charge;
        }
        r.set_xyz(x,y,z);
        a.set_atomic_number(number);
        a.set_R(r);
        add_atom(a);
        file.getline(line, sizeof(line));
      }
      if(chk1==1){break;}
    }
  }
  file.close();
}

void Molecule::read_coord_paics_out(const string& filename,
                                    string& s_out)
{
  char line[1024];
  char buff[1024];
  string s;
  Atom a;
  const double bohr=0.5291772108;
  int chk;
  double x,y,z,vx,vy,vz,dtmp;
  fstream file(filename.c_str(), std::ios::in);
  //if(file==NULL){ <--- 20150827
  if(!file){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  clear();
  chk=0;
  while(!file.eof()){
    file.getline(line, sizeof(line));
    //if(iFind(line,"< input coordinate of nucleus charge >")>=0 ||
    if((line[7]=='<' && line[29]=='n' && line[37]=='c')){
      chk=1;
      clear();
      for(int i=0;i<4;i++) file.getline(line, sizeof(line));
      while(!file.eof()){
        file.getline(line, sizeof(line));
        //if(iFind(line,"---------------------------------------------------------------------")>=0){
        if(line[8]=='-' && line[9]=='-'){
          break;
        } else {
          iPickUp(line,2,buff);s=buff;
          iPickUp(line,5,buff);sscanf(buff,"%lf",&dtmp); x=(dtmp/bohr); //iPickUp(line,5,buff);x=(atof(buff)/bohr);
          iPickUp(line,6,buff);sscanf(buff,"%lf",&dtmp); y=(dtmp/bohr); //iPickUp(line,6,buff);y=(atof(buff)/bohr);
          iPickUp(line,7,buff);sscanf(buff,"%lf",&dtmp); z=(dtmp/bohr); //iPickUp(line,7,buff);z=(atof(buff)/bohr);
          a.set_atomic_number(Atom::SymbolToNumber(s));
          a.set_R(Dvector3D(x,y,z));
          add_atom(a);
        }        
      }
    }
    if(chk==1){
      break;
    }
  }
  file.close();
}

void Molecule::read_coord_paics_in(const string& filename , string& s_out)
{
  char line[1024];
  char buff[1024];
  Atom a;
  double x,y,z,unit,dtmp;
  int atomic,n;
  string b;
  fstream file1(filename.c_str(), std::ios::in);
  //if(file1==NULL){ <--- 20150827
  if(!file1){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  unit=1.0;
  while(!file1.eof()){
    file1.getline(line, sizeof(line));
    for(int i=0;i<128;i++){
      line[i]=(char)toupper(line[i]);
    }
    if(iFind(line,"COORD_UNIT")>=0){
      iPickUp(line,1,buff);n=atoi(buff);
      if(n==1){
        unit=1.0/0.5291772108;   
      }
      break;
    }
  }
  file1.close();
  clear();
  fstream file2(filename.c_str(), std::ios::in);
  //if(file2==NULL){ <--- 20150827
  if(!file2){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  while(!file2.eof()){
    file2.getline(line, sizeof(line));
    for(int i=0;i<128;i++){
      line[i]=(char)toupper(line[i]);
    }
    if(iFind(line,"CHARGE")>=0 || iFind(line,"NUCLEUS")>=0){
      if(iFind(line,"EX_POINT_CHARGE")<0){        
        file2.getline(line, sizeof(line));
        iPickUp(line,0,buff);n=atoi(buff);
        for(int i=0;i<n;i++){
          file2.getline(line, sizeof(line));
          iPickUp(line,1,buff);atomic=atoi(buff);
          iPickUp(line,3,buff);sscanf(buff,"%lf",&dtmp); x=dtmp*unit; //iPickUp(line,3,buff);x=atof(buff)*unit;
          iPickUp(line,4,buff);sscanf(buff,"%lf",&dtmp); y=dtmp*unit; //iPickUp(line,4,buff);y=atof(buff)*unit;
          iPickUp(line,5,buff);sscanf(buff,"%lf",&dtmp); z=dtmp*unit; //iPickUp(line,5,buff);z=atof(buff)*unit;
          a.set_atomic_number(atomic);
          a.set_R(Dvector3D(x,y,z));
          add_atom(a);
        }
      }
    }
    if(iFind(line,"ATOM")>=0){
      if(iFind(line,"FRAG_ATOM")<0){
        file2.getline(line, sizeof(line));
        iPickUp(line,0,buff);n=atoi(buff);
        for(int i=0;i<n;i++){
          file2.getline(line, sizeof(line));
          iPickUp(line,1,buff);atomic=atoi(buff);
          iPickUp(line,2,buff);buff[strlen(buff)-4]='\0';b=buff;
          iPickUp(line,3,buff);sscanf(buff,"%lf",&dtmp); x=dtmp*unit; //iPickUp(line,3,buff);x=atof(buff)*unit;
          iPickUp(line,4,buff);sscanf(buff,"%lf",&dtmp); y=dtmp*unit; //iPickUp(line,4,buff);y=atof(buff)*unit;
          iPickUp(line,5,buff);sscanf(buff,"%lf",&dtmp); z=dtmp*unit; //iPickUp(line,5,buff);z=atof(buff)*unit;
          a.set_atomic_number(atomic);
          a.set_R(Dvector3D(x,y,z));
          a.set_basis(b);
          add_atom(a);
        }
      }
    }
  }
  file2.close();
}

void Molecule::read_coord_amber(const string& filename1,
                                string& s_out          )
{
  fstream file1(filename1.c_str(), std::ios::in);
  //if(file1==NULL){ <--- 20150827
  if(!file1){
    s_out = s_out + "\n warning , can not open the file : " + filename1 + "\n\n";
    return ;
  }
  char line[1024];
  char buff[1024];
  int nn,count1,count2;
  double x,y,z,dtmp;
  while(! file1.eof()){
    file1.getline(line,sizeof(line));
    file1.getline(line,sizeof(line));
    nn=atoi(iPickUp(line,0,buff));
    count1=0;
    count2=0;
    while(! file1.eof()){
      file1.getline(line,sizeof(line));
      sscanf(&line[0*12],"%lf",&dtmp);x=dtmp/0.5291772108; //x=atof(&line[0*12])/0.5291772108;
      sscanf(&line[1*12],"%lf",&dtmp);y=dtmp/0.5291772108; //y=atof(&line[1*12])/0.5291772108;
      sscanf(&line[2*12],"%lf",&dtmp);z=dtmp/0.5291772108; //z=atof(&line[2*12])/0.5291772108;
      set_atom_coordinate(count1,x,y,z);
      count1++; 
      if(count1==nn){
	break;
      }
      sscanf(&line[3*12],"%lf",&dtmp);x=dtmp/0.5291772108; //x=atof(&line[3*12])/0.5291772108;
      sscanf(&line[4*12],"%lf",&dtmp);y=dtmp/0.5291772108; //y=atof(&line[4*12])/0.5291772108;
      sscanf(&line[5*12],"%lf",&dtmp);z=dtmp/0.5291772108; //z=atof(&line[5*12])/0.5291772108;
      set_atom_coordinate(count1,x,y,z);
      count1++;
      if(count1==nn){
	break;
      }
    }
    if(count1>0){
      while(! file1.eof()){
	file1.getline(line,sizeof(line));
	sscanf(&line[0*12],"%lf",&dtmp);x=dtmp*20.455/0.5291772108; //x=atof(&line[0*12])*20.455/0.5291772108;
	sscanf(&line[1*12],"%lf",&dtmp);y=dtmp*20.455/0.5291772108; //y=atof(&line[1*12])*20.455/0.5291772108;
	sscanf(&line[2*12],"%lf",&dtmp);z=dtmp*20.455/0.5291772108; //z=atof(&line[2*12])*20.455/0.5291772108;
	set_atom_velocity(count2,x,y,z);
	count2++; 
	if(count2==nn){
	  break;
	}
	sscanf(&line[3*12],"%lf",&dtmp);x=dtmp*20.455/0.5291772108; //x=atof(&line[3*12])*20.455/0.5291772108;
	sscanf(&line[4*12],"%lf",&dtmp);y=dtmp*20.455/0.5291772108; //y=atof(&line[4*12])*20.455/0.5291772108;
	sscanf(&line[5*12],"%lf",&dtmp);z=dtmp*20.455/0.5291772108; //z=atof(&line[5*12])*20.455/0.5291772108;
	set_atom_velocity(count2,x,y,z);
	count2++;
	if(count2==nn){
	  break;
	}
      }
    }
    if(count1>0 && count2>0){
      break;
    }
  }
  file1.close();
  if(nn!=natom){
    clear();
  }
}

void Molecule::read_coord_pdb( const string& filename,
			       string& s_out,
			       const string& option,
                               const string& file_string)
{
  char line[1024];
  char buff[1024];
  char atyp[5];
  char rtyp[5];
  char seqn[5];
  char chin[5];
  string s_atyp,s_rtyp,s_seqn,s_chin,stmp;
  int atomic,chk,count,atm_type,res_type,chk_from_string;
  int char_pos,char_size,line_pos; // ishi 20150202
  Dvector3D r;
  Atom a;
  fstream file;

  if(file_string.size()>0){
    chk_from_string=1;
    char_pos=0;
    char_size=file_string.size();
  } else {
    chk_from_string=0;
  }

  if(chk_from_string==0){
    file.open(filename.c_str(), std::ios::in);
    //if(file==NULL){ <--- 20150827
    if(!file){
      s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
      return ;
    }
  }

  clear();
  count=0;
  while(1){

    if(chk_from_string==0){
      if(file.eof()){
        break;
      }
      file.getline(line,sizeof(line));
    }
    if(chk_from_string==1){
      if(char_pos==char_size){
        break;
      }
      line_pos=0;
      while(1){
        if(char_pos==char_size){
          line[line_pos]='\0';
          break;
        }
        line[line_pos]=file_string.at(char_pos) ;
        if(line[line_pos]=='\n'){
          line[line_pos]='\0';
          char_pos++;
          break;
        }
        if(line_pos>1000){
          line[line_pos]='\0';
          char_pos++;
          break;
        }
        line_pos++;
        char_pos++;
      }
      //cout << line << endl ;
    }


    if(line[0]=='A' || line[0]=='H'){
      for(int i=0;i<1024;i++){
        buff[i]=' ';
      }
      iPickUp(line,0,buff);
      if(iFind(buff,"ATOM")>=0 || iFind(buff,"HETATM")>=0){
        if(iPickUpNum(line)>4){
          chk=-1;
	  for(int j=0;j<5;j++){
	    atyp[j]='\0';
	    rtyp[j]='\0';
	    chin[j]='\0';
	    seqn[j]='\0';
	  }	    
          for(int j=0;j<4;j++){atyp[j]=line[12+j];}// [0-11] [12][13][14][15] [16-end] //
          for(int j=0;j<4;j++){rtyp[j]=line[16+j];}// [0-15] [16][17][18][19] [20-end] //
          for(int j=0;j<1;j++){chin[j]=line[21+j];}// [0-20] [21]             [22-end] //
          for(int j=0;j<3;j++){seqn[j]=line[23+j];}// [0-22] [23][24][25]     [26-end] // <-- res. number < 1000
        //for(int j=0;j<4;j++){seqn[j]=line[22+j];}// [0-21] [22][23][24][25] [26-end] // <-- res. number >=1000
          atomic=get_atomic_number_pdb(atyp,&chk);
          double dtmp;
          sscanf(&line[30],"%lf",&dtmp);r.set_x(dtmp/0.5291772108); //r.set_x(atof(&line[30])/0.5291772108);
          sscanf(&line[38],"%lf",&dtmp);r.set_y(dtmp/0.5291772108); //r.set_y(atof(&line[38])/0.5291772108);
          sscanf(&line[46],"%lf",&dtmp);r.set_z(dtmp/0.5291772108); //r.set_z(atof(&line[46])/0.5291772108);
          s_atyp=iTrim(atyp);
          s_rtyp=iTrim(rtyp);
          s_seqn=iTrim(seqn);
          s_chin=iTrim(chin);
          if(chk==1){

            // A*** is replaced 
            if(s_rtyp=="AGLY"){s_rtyp="GLY";}// s_out=s_out+"AGLY is replaced by GLY \n";}
            if(s_rtyp=="AALA"){s_rtyp="ALA";}// s_out=s_out+"AALA is replaced by ALA \n";}
            if(s_rtyp=="AVAL"){s_rtyp="VAL";}// s_out=s_out+"AVAL is replaced by VAL \n";}
            if(s_rtyp=="ALEU"){s_rtyp="LEU";}// s_out=s_out+"ALEU is replaced by LEU \n";}
            if(s_rtyp=="AILE"){s_rtyp="ILE";}// s_out=s_out+"AILE is replaced by ILE \n";}
            if(s_rtyp=="AMET"){s_rtyp="MET";}// s_out=s_out+"AMET is replaced by MET \n";}
            if(s_rtyp=="APRO"){s_rtyp="PRO";}// s_out=s_out+"APRO is replaced by PRO \n";}
            if(s_rtyp=="APHE"){s_rtyp="PHE";}// s_out=s_out+"APRO is replaced by PHE \n";}
            if(s_rtyp=="ATRP"){s_rtyp="TRP";}// s_out=s_out+"ATRP is replaced by TRP \n";}
            if(s_rtyp=="ASER"){s_rtyp="SER";}// s_out=s_out+"ASER is replaced by SER \n";}
            if(s_rtyp=="ATHR"){s_rtyp="THR";}// s_out=s_out+"ATHR is replaced by THR \n";}
            if(s_rtyp=="AASN"){s_rtyp="ASN";}// s_out=s_out+"AASN is replaced by ASN \n";}
            if(s_rtyp=="AGLN"){s_rtyp="GLN";}// s_out=s_out+"AGLN is replaced by GLN \n";}
            if(s_rtyp=="ATYR"){s_rtyp="TYR";}// s_out=s_out+"ATYR is replaced by TYR \n";}
            if(s_rtyp=="ACYS"){s_rtyp="CYS";}// s_out=s_out+"ACYS is replaced by CYS \n";}
            if(s_rtyp=="ALYS"){s_rtyp="LYS";}// s_out=s_out+"ALYS is replaced by LYS \n";}
            if(s_rtyp=="AARG"){s_rtyp="ARG";}// s_out=s_out+"AARG is replaced by ARG \n";}
            if(s_rtyp=="AHIS"){s_rtyp="HIS";}// s_out=s_out+"AHIS is replaced by HIS \n";}
            if(s_rtyp=="AASP"){s_rtyp="ASP";}// s_out=s_out+"AASP is replaced by ASP \n";}
            if(s_rtyp=="AGLU"){s_rtyp="GLU";}// s_out=s_out+"AGLU is replaced by GLU \n";}

            a.set_atomic_number(atomic);
            a.set_R(r);
	    a.set_link_read_pdb(count);
	    a.set_atyp_read_pdb(s_atyp.c_str()); 
	    a.set_rtyp_read_pdb(s_rtyp.c_str());
	    a.set_nseq_read_pdb(s_seqn.c_str());
	    a.set_chin_read_pdb(s_chin.c_str());

            if(0){}
            
            // B*** is replaced 
            else if(s_rtyp=="BGLY"){}// s_out=s_out+"BGLY is skipped \n" ; }
            else if(s_rtyp=="BALA"){}// s_out=s_out+"BALA is skipped \n" ; }
            else if(s_rtyp=="BVAL"){}// s_out=s_out+"BVAL is skipped \n" ; }
            else if(s_rtyp=="BLEU"){}// s_out=s_out+"BLEU is skipped \n" ; }
            else if(s_rtyp=="BILE"){}// s_out=s_out+"BILE is skipped \n" ; }
            else if(s_rtyp=="BMET"){}// s_out=s_out+"BMET is skipped \n" ; }
            else if(s_rtyp=="BPRO"){}// s_out=s_out+"BPRO is skipped \n" ; }
            else if(s_rtyp=="BPHE"){}// s_out=s_out+"BPHE is skipped \n" ; }
            else if(s_rtyp=="BTRP"){}// s_out=s_out+"BTRP is skipped \n" ; }
            else if(s_rtyp=="BSER"){}// s_out=s_out+"BSER is skipped \n" ; }
            else if(s_rtyp=="BTHR"){}// s_out=s_out+"BTHR is skipped \n" ; }
            else if(s_rtyp=="BASN"){}// s_out=s_out+"BASN is skipped \n" ; }
            else if(s_rtyp=="BGLN"){}// s_out=s_out+"BGLN is skipped \n" ; }
            else if(s_rtyp=="BTYR"){}// s_out=s_out+"BTYR is skipped \n" ; }
            else if(s_rtyp=="BCYS"){}// s_out=s_out+"BCYS is skipped \n" ; }
            else if(s_rtyp=="BLYS"){}// s_out=s_out+"BLYS is skipped \n" ; }
            else if(s_rtyp=="BARG"){}// s_out=s_out+"BARG is skipped \n" ; }
            else if(s_rtyp=="BHIS"){}// s_out=s_out+"BHIS is skipped \n" ; }
            else if(s_rtyp=="BASP"){}// s_out=s_out+"BASP is skipped \n" ; }
            else if(s_rtyp=="BGLU"){}// s_out=s_out+"BGLU is skipped \n" ; }
            
            else {
              
              if(option==""){
                add_atom(a,30000);
              }
              if(option=="pdb-ana"){
                atm_type=BioUnit::atm_type(s_atyp,s_rtyp);
                res_type=BioUnit::res_type(s_rtyp,&stmp);
                if(atm_type<0){
                  s_out=s_out+"    warning : not specified atom : " 
                    + s_atyp + " " + s_rtyp + " " + s_chin + s_seqn + "\n";
                  add_atom(a,30000);
                }
                else if(res_type<0){
                  s_out=s_out+"    warning : not specified residue : " 
                    + s_atyp + " " + s_rtyp + " " + s_chin + s_seqn + "\n";
                  add_atom(a,30000);
                } else {
                  a.set_rtyp_read_pdb(stmp.c_str());
                  add_atom(a,30000);
                }
              }
            }
            
          } else {
            s_out=s_out+"    warning : Molecule::read_coord_pdb : not specified  : "+line+"\n\n" ;
            a.set_atomic_number(0);
            a.set_R(r);
	    a.set_link_read_pdb(count);
	    a.set_atyp_read_pdb(s_atyp.c_str()); 
	    a.set_rtyp_read_pdb(s_rtyp.c_str());
	    a.set_nseq_read_pdb(s_seqn.c_str());
	    a.set_chin_read_pdb(s_chin.c_str());
            add_atom(a,30000);
          }
        }
      }
    }
    if(iFind(line,"TER")>=0){
      count++;
    }
  }
  file.close();
}

void Molecule::read_coord_xyz( const string& filename , string& s_out )
{
  char line[1024];
  int n;
  char atyp[5];
  double x,y,z;
  Dvector3D r;
  Atom a;
  fstream file(filename.c_str(), std::ios::in);
  
  //if(file==NULL){ <--- 20150827
  if(!file){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  clear();
  file >> n ; 
  for(int i=0;i<n;i++){
    if(file.eof()){break;}
    file >> atyp >> x >> y >> z ;
    a.set_atomic_number(Atom::SymbolToNumber(string(atyp)));
    r.set_x(x/0.5291772108);
    r.set_y(y/0.5291772108);
    r.set_z(z/0.5291772108);
    a.set_R(r);
    add_atom(a);
  }
  file.close();
}

void Molecule::read_coord_siesta( const string& filename , string& s_out )
{
  char line[1024];
  char buff[1024];
  int itmp1,itmp2,chk;
  int species[1024];
  double x,y,z,dtmp;
  Atom a;
  Dvector3D r;

  fstream file(filename.c_str(), std::ios::in);
  
  //if(file==NULL){ <--- 20150827
  if(!file){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  clear();
  chk=0;
  while(! file.eof()){
    file.getline(line,sizeof(line));
    if(iFind(line,"Species number")>=0){
      iPickUp(line,2,buff);itmp1=(atoi(buff));
      iPickUp(line,7,buff);itmp2=(atoi(buff));
      species[itmp1]=itmp2;
    }
    if(iFind(line,"Atomic coordinates (Bohr) and species")>=0){
      while(! file.eof()){
	file.getline(line,sizeof(line));
	if(iPickUpNum(line)<1){
	  chk=1;
	  break;
	}
	iPickUp(line,1,buff);sscanf(buff,"%lf",&dtmp);x=dtmp; //iPickUp(line,1,buff);x=atof(buff);
	iPickUp(line,2,buff);sscanf(buff,"%lf",&dtmp);y=dtmp; //iPickUp(line,2,buff);y=atof(buff);
	iPickUp(line,3,buff);sscanf(buff,"%lf",&dtmp);z=dtmp; //iPickUp(line,3,buff);z=atof(buff);
	iPickUp(line,4,buff);itmp1=atoi(buff);
	a.set_atomic_number(species[itmp1]);
	r.set_x(x); r.set_y(y); r.set_z(z); a.set_R(r);
	add_atom(a);
      }
    }
    if(chk==1) break;
  }
  file.close();
}

void Molecule::read_coord_mol2(const string& filename, string& s_out)
{
  char line[1024];
  char buff[1024];
  double x,y,z,dtmp;
  int ia,pos;
  Atom a;
  Dvector3D r;
  string ms1,ms2,ms3;

  fstream file(filename.c_str(), std::ios::in);
  
  //if(file==NULL){ <--- 20150827
  if(!file){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  clear();
  while(! file.eof()){
    file.getline(line,sizeof(line));
    if(iFind(line,"@<TRIPOS>ATOM")>=0){
      while(! file.eof()){
	file.getline(line,sizeof(line));
	if(iFind(line,"@<TRIPOS>")>=0){
	  break;
	}

        /////////////////////////
        // check atomic number //
        /////////////////////////
	iPickUp(line,1,buff);
        for(int i=0;i<5;i++){
          if(buff[i]=='1' || buff[i]=='2' || buff[i]=='3' || buff[i]=='4' || buff[i]=='5' ||
             buff[i]=='6' || buff[i]=='7' || buff[i]=='8' || buff[i]=='9' || buff[i]=='0' ){
            buff[i]='\0';
            break;
          }
        }
	if     (0                   ){        }
	else if(strcmp(buff,"H" )==0){ia =  1;}
	else if(strcmp(buff,"HE")==0){ia =  2;}	else if(strcmp(buff,"He")==0){ia =  2;}
	else if(strcmp(buff,"LI")==0){ia =  3;}	else if(strcmp(buff,"Li")==0){ia =  3;}
	else if(strcmp(buff,"BE")==0){ia =  4;}	else if(strcmp(buff,"Be")==0){ia =  4;}
	else if(strcmp(buff,"B" )==0){ia =  5;}
	else if(strcmp(buff,"C" )==0){ia =  6;}
	else if(strcmp(buff,"N" )==0){ia =  7;}
	else if(strcmp(buff,"O" )==0){ia =  8;}
	else if(strcmp(buff,"F" )==0){ia =  9;}
	else if(strcmp(buff,"NA")==0){ia = 11;}	else if(strcmp(buff,"Na")==0){ia = 11;}
	else if(strcmp(buff,"MG")==0){ia = 12;}	else if(strcmp(buff,"Mg")==0){ia = 12;}
	else if(strcmp(buff,"SI")==0){ia = 14;}	else if(strcmp(buff,"Si")==0){ia = 14;}
	else if(strcmp(buff,"P" )==0){ia = 15;}
	else if(strcmp(buff,"S" )==0){ia = 16;}
	else if(strcmp(buff,"CL")==0){ia = 17;}	else if(strcmp(buff,"Cl")==0){ia = 17;}
	else if(strcmp(buff,"K" )==0){ia = 19;}
	else if(strcmp(buff,"CA")==0){ia = 20;}	else if(strcmp(buff,"Ca")==0){ia = 20;}
	else if(strcmp(buff,"FE")==0){ia = 26;}	else if(strcmp(buff,"Fe")==0){ia = 26;}
	else if(strcmp(buff,"CO")==0){ia = 27;}	else if(strcmp(buff,"Co")==0){ia = 27;}
	else if(strcmp(buff,"NI")==0){ia = 28;}	else if(strcmp(buff,"Ni")==0){ia = 28;}
	else if(strcmp(buff,"ZN")==0){ia = 30;}	else if(strcmp(buff,"Zn")==0){ia = 30;}
	else if(strcmp(buff,"AS")==0){ia = 33;}	else if(strcmp(buff,"As")==0){ia = 33;}
	else if(strcmp(buff,"SE")==0){ia = 34;}	else if(strcmp(buff,"Se")==0){ia = 34;}
	else if(strcmp(buff,"BR")==0){ia = 35;}	else if(strcmp(buff,"Br")==0){ia = 35;}
	else if(strcmp(buff,"RU")==0){ia = 44;}	else if(strcmp(buff,"Ru")==0){ia = 44;}
	else if(strcmp(buff,"AG")==0){ia = 47;}	else if(strcmp(buff,"Ag")==0){ia = 47;}
        else if(strcmp(buff,"I" )==0){ia = 53;}
        else if(strcmp(buff,"CS")==0){ia = 55;} else if(strcmp(buff,"Cs")==0){ia = 55;}
        else if(strcmp(buff,"PT")==0){ia = 78;} else if(strcmp(buff,"Pt")==0){ia = 78;}
        else if(strcmp(buff,"AU")==0){ia = 79;} else if(strcmp(buff,"Au")==0){ia = 79;}
        else if(strcmp(buff,"HG")==0){ia = 80;} else if(strcmp(buff,"Hg")==0){ia = 80;}
        else if(strcmp(buff,"TL")==0){ia = 81;} else if(strcmp(buff,"Ti")==0){ia = 81;}
	else {
	  s_out=s_out+"warning : Molecule::read_coord_mol2 : not specified  : "+line+"\n\n" ;
	  ia=0;
	}

	iPickUp(line,2,buff);sscanf(buff,"%lf",&dtmp); x=dtmp/0.5291; //iPickUp(line,2,buff);x=atof(buff)/0.5291;
	iPickUp(line,3,buff);sscanf(buff,"%lf",&dtmp); y=dtmp/0.5291; //iPickUp(line,3,buff);y=atof(buff)/0.5291;
	iPickUp(line,4,buff);sscanf(buff,"%lf",&dtmp); z=dtmp/0.5291; //iPickUp(line,4,buff);z=atof(buff)/0.5291;
	iPickUp(line,1,buff);ms1=buff;
	iPickUp(line,5,buff);ms2=buff;
        pos=iPos(line,7); if(pos>=0){ms3=(&line[pos]);}else{ms3="";}
	a.set_atomic_number(ia);
	r.set_x(x); r.set_y(y); r.set_z(z); a.set_R(r);
        a.set_mol2_1(ms1);
        a.set_mol2_2(ms2);
        a.set_mol2_3(ms3);
	add_atom(a);
      }
    }
  }
  file.close();
}

//int Molecule::get_atom_type_pdb( const char* a, int* chk )
int Molecule::get_atomic_number_pdb( const char* a, int* chk )
{
  // a : 4 characters in a line of the pdb file : [12][13][14][15] //
  // chk==1 : success to get atomic number 
  // chk!=1 : not success to get atomic number
  char b[5];
  int atomic=0;
  int j=0;
  int chkchk=0;
  string s;
  for(int i=0;i<4;i++){
    if(chkchk==1 && a[i]==' ') {
      break;
    }
    if(a[i]!=' ') {
      if(chkchk==0){
        chkchk=1;
      }
      b[j]=a[i];
      j++;
    }
  }
  b[j]='\0';
  s=b;
  //
  if(s=="OH" ){atomic= 8;(*chk)=1;return atomic;} 
  if(s=="NH1"){atomic= 7;(*chk)=1;return atomic;} 
  if(s=="NH2"){atomic= 7;(*chk)=1;return atomic;} 
  if(s=="CH2"){atomic= 6;(*chk)=1;return atomic;} 
  if(s=="MG" ){atomic=12;(*chk)=1;return atomic;} 
  if(s=="Mg" ){atomic=12;(*chk)=1;return atomic;} 
  if(s=="Li" ){atomic= 3;(*chk)=1;return atomic;} 
  if(s=="FE" ){atomic=26;(*chk)=1;return atomic;} 
  if(s=="Fe" ){atomic=26;(*chk)=1;return atomic;} 
  if(s=="Ni" ){atomic=28;(*chk)=1;return atomic;} 
  if(s=="Pt" ){atomic=78;(*chk)=1;return atomic;} 
  if(s=="Ca" ){atomic=20;(*chk)=1;return atomic;} // <-- 20141106
  //
  if(a[0]=='C' && a[1]=='l' && a[2]=='-'              ) {atomic=17;(*chk)=1;return atomic;}
  if(a[0]=='C' && a[1]=='L' && a[2]=='-'              ) {atomic=17;(*chk)=1;return atomic;}
  if(a[0]=='N' && a[1]=='a' && a[2]=='+'              ) {atomic=11;(*chk)=1;return atomic;}
  if(a[0]=='N' && a[1]=='A' && a[2]=='+'              ) {atomic=11;(*chk)=1;return atomic;}
  if(             a[1]=='C' && a[2]=='l' && a[3]=='-' ) {atomic=17;(*chk)=1;return atomic;}
  if(             a[1]=='C' && a[2]=='L' && a[3]=='-' ) {atomic=17;(*chk)=1;return atomic;}
  if(             a[1]=='N' && a[2]=='a' && a[3]=='+' ) {atomic=11;(*chk)=1;return atomic;}
  if(             a[1]=='N' && a[2]=='A' && a[3]=='+' ) {atomic=11;(*chk)=1;return atomic;}
  if(a[0]=='B' && a[1]=='R'                           ) {atomic=35;(*chk)=1;return atomic;}
  if(a[0]=='C' && a[1]=='l'                           ) {atomic=17;(*chk)=1;return atomic;}
  if(a[0]=='C' && a[1]=='L'                           ) {atomic=17;(*chk)=1;return atomic;}
  if(a[0]=='H' && a[1]=='G'                           ) {atomic= 1;(*chk)=1;return atomic;}
  if(a[0]=='H' && a[1]=='D'                           ) {atomic= 1;(*chk)=1;return atomic;}
  if(a[0]=='H' && a[1]=='E'                           ) {atomic= 1;(*chk)=1;return atomic;}
  if(a[0]=='H' && a[1]=='N'                           ) {atomic= 1;(*chk)=1;return atomic;}
  if(a[0]=='H' && a[1]=='O'                           ) {atomic= 1;(*chk)=1;return atomic;}
  if(a[0]=='N' && a[1]=='a'                           ) {atomic=11;(*chk)=1;return atomic;}
  if(a[0]=='N' && a[1]=='A'                           ) {atomic=11;(*chk)=1;return atomic;}
  if(a[0]=='S' && a[1]=='E'                           ) {atomic=16;(*chk)=1;return atomic;}
  if(a[0]=='Z' && a[1]=='N'                           ) {atomic=30;(*chk)=1;return atomic;}
  if(             a[1]=='C' && a[2]=='l'              ) {atomic=17;(*chk)=1;return atomic;}
  if(             a[1]=='C' && a[2]=='L'              ) {atomic=17;(*chk)=1;return atomic;}
  if(             a[1]=='N' && a[2]=='a'              ) {atomic=11;(*chk)=1;return atomic;}
  if(             a[1]=='N' && a[2]=='A'              ) {atomic=11;(*chk)=1;return atomic;}
  if(             a[1]=='H'                           ) {atomic= 1;(*chk)=1;return atomic;}
  if(             a[1]=='C'                           ) {atomic= 6;(*chk)=1;return atomic;}
  if(             a[1]=='N'                           ) {atomic= 7;(*chk)=1;return atomic;}
  if(             a[1]=='O'                           ) {atomic= 8;(*chk)=1;return atomic;}
  if(             a[1]=='F'                           ) {atomic= 9;(*chk)=1;return atomic;}
  if(             a[1]=='P'                           ) {atomic=15;(*chk)=1;return atomic;}
  if(             a[1]=='S'                           ) {atomic=16;(*chk)=1;return atomic;}
  if(a[0]=='H'                                        ) {atomic= 1;(*chk)=1;return atomic;}
  if(a[0]=='C'                                        ) {atomic= 6;(*chk)=1;return atomic;}
  if(a[0]=='N'                                        ) {atomic= 7;(*chk)=1;return atomic;}
  if(a[0]=='O'                                        ) {atomic= 8;(*chk)=1;return atomic;}
  if(a[0]=='F'                                        ) {atomic= 9;(*chk)=1;return atomic;}
  if(a[0]=='P'                                        ) {atomic=15;(*chk)=1;return atomic;}
  if(a[0]=='S'                                        ) {atomic=16;(*chk)=1;return atomic;}
  if(a[0]=='K' && a[1]==' ' && a[2]==' ' && a[3]==' ' ) {atomic=19;(*chk)=1;return atomic;}
  if(a[0]==' ' && a[1]=='K' && a[2]==' ' && a[3]==' ' ) {atomic=19;(*chk)=1;return atomic;}
  if(a[0]==' ' && a[1]==' ' && a[2]=='K' && a[3]==' ' ) {atomic=19;(*chk)=1;return atomic;}
  if(a[0]==' ' && a[1]==' ' && a[2]==' ' && a[3]=='K' ) {atomic=19;(*chk)=1;return atomic;}
  //
  return atomic;
}

void Molecule::read_coord_tmp(const string& filename ,
			      string&       s_out    )
{
  fstream file1(filename.c_str(), std::ios::in);
  //if(file1==NULL){ <--- 20150827
  if(!file1){
    return ;
  }
  clear();
  char line[1024];
  char c   [1024];
  int chk,n1,n2,ia;
  file1 >> n1 ; // n_atom
  file1 >> n2 ; // n_traj
  double x,y,z;
  Atom a;
  Dvector3D r;
  for(int i=0;i<n1;i++){
    file1 >> ia ; 
    file1 >> x  ;
    file1 >> y  ;
    file1 >> z  ;
    r.set_x(x);
    r.set_y(y);
    r.set_z(z);
    a.set_atomic_number(ia);
    a.set_R(r);
    add_atom(a);
  }
  file1.close();
}


/*---------------------*/
/*----- read bond -----*/
/*---------------------*/

void Molecule::read_bond_mol2(const string& filename, string& s_out)
{
  char line[1024];
  char buff[1024];
  int n,a1,a2;
  string s3;
  fstream file(filename.c_str(), std::ios::in);
  
  //if(file==NULL){ <--- 20150827
  if(!file){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  clear_bond();
  clear_linkgroup();
  clear_biounit();
  while(! file.eof()){
    file.getline(line,sizeof(line));
    if(iFind(line,"@<TRIPOS>BOND")>=0){
      while(! file.eof()){
	file.getline(line,sizeof(line));
        n=iPickUpNum(line);
        if(n>=3){
          iPickUp(line,1,buff);a1=atoi(buff)-1;
          iPickUp(line,2,buff);a2=atoi(buff)-1;
        //iPickUp(line,1,buff);a1=atof(buff)-1;
        //iPickUp(line,2,buff);a2=atof(buff)-1;
	  iPickUp(line,3,buff);s3=buff;
          add_bond(a1,a2,s3);
        }
	if(iFind(line,"@<TRIPOS>")>=0){break;}
	if(iFind(line,"@<BUDE>")>=0){break;}
      }
    }
  }
  file.close();
}

/*-------------------------*/
/*----- read fragment -----*/
/*-------------------------*/

void Molecule::read_fragment(const string& filename,
                             const int     type    ,
                             string&       s_out   )
{
  if(type==1) {read_fragment_paics_in   (filename,s_out);}
  if(type==2) {read_fragment_paics_out_1(filename,s_out);}
  if(type==3) {read_fragment_paics_out_2(filename,s_out);}
}

void Molecule::read_fragment_paics_in( const string& filename , string& s_out )
{
  char line[1024];
  char buff[1024];
  int nelec,ncharge,natom_in,natom_add,nbasis_in,nbasis_add,iatom;
  fstream file(filename.c_str(), std::ios::in);
  //if(file==NULL){ <--- 20150827
  if(!file){ 
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  clear_fragment();
  int * chk_atom_key_word = new int [natom+1];
  for(int i=0;i<natom;i++){
    chk_atom_key_word[i]=0;
  }
  while(!file.eof()){
    file.getline(line, sizeof(line));
    if(iFind(line,"FRAG_DEF")>=0 || iFind(line,"FRAG ")>=0){
      if(iFind(line,"FIELD_PROP_FRAG")<0){
        iPickUp(line,1,buff);nelec     =atoi(buff);
        iPickUp(line,2,buff);natom_in  =atoi(buff);
        iPickUp(line,4,buff);natom_add =atoi(buff);
        iPickUp(line,3,buff);nbasis_in =atoi(buff);
        iPickUp(line,5,buff);nbasis_add=atoi(buff);
        Fragment f;
        f.push_back_elec(nelec);
        for(int i=0;i<natom_in;i++){
          file >> buff;
          iatom=atoi(buff);
          f.push_back_atom_index(iatom-1);
        }
        for(int i=0;i<nbasis_in;i++){
          file >> buff;
        }
        for(int i=0;i<natom_add;i++){
          file >> buff;
          iatom=atoi(buff);
          f.push_back_atom_index_link(iatom-1,-1);
        }
        for(int i=0;i<nbasis_add;i++){
          file >> buff;
        }
        push_back_fragment(f);
      }
    }
    if(iFind(line,"FRAG_ATOM")>=0){
      chk_atom_key_word[nfragment]=1;
      iPickUp(line,1,buff);ncharge   =atoi(buff);
      iPickUp(line,2,buff);natom_in  =atoi(buff);
      iPickUp(line,3,buff);natom_add =atoi(buff);
      Fragment f;
      nelec=-ncharge;
      for(int i=0;i<natom_in;i++){
        file >> buff;
        iatom=atoi(buff);
        f.push_back_atom_index(iatom-1);
	nelec=nelec+atom[iatom-1].get_atomic_number();
      }
      for(int i=0;i<natom_add;i++){
        file >> buff;
        iatom=atoi(buff);
        f.push_back_atom_index_link(iatom-1,-1);
	nelec++;
      }
      f.push_back_elec(nelec);
      push_back_fragment(f);
    }
  }
  file.close();
  for(int i=0;i<nfragment;i++){
    if(chk_atom_key_word[i]==1){
      for(int j=0;j<fragment[i].get_n_link();j++){
	int chk=0;
	iatom=fragment[i].get_atom_index_link1(j);
	for(int k=0;k<nfragment;k++){
	  if(i!=k){
	    for(int l=0;l<fragment[k].get_n_atom_index();l++){
	      if(iatom==fragment[k].get_atom_index(l)){
		fragment[k].pick_up_elec(1);
		chk=1;
	      }
	      if(chk==1) break;
	    }
	  }
	  if(chk==1) break;
	}
      }
    }
  }
  delete [] chk_atom_key_word;
}

void Molecule::read_fragment_paics_out_1(const string& filename,
                                         string& s_out         )
{
  char line[1024];
  char buff[1024];
  int n,count,nelec,natom_in,iatom_in,natom_add,chk1;
  fstream file(filename.c_str(), std::ios::in);
  //if(file==NULL){ <--- 20150827
  if(!file){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  clear_fragment();
  chk1=0;
  while(!file.eof()){
    file.getline(line, sizeof(line));
    //if(iFind(line,"     < input fragment >"      )){
    if(line[7]=='<' && line[15]=='f' && line[24]=='>'){
      file.getline(line, sizeof(line));
      file.getline(line, sizeof(line));
      iPickUp(line,4,buff);n=atoi(buff);
      file.getline(line, sizeof(line));
      chk1=1;
    }
    if(chk1==1){
      for(int i=0;i<n;i++){
        Fragment f;
        file.getline(line, sizeof(line));
        iPickUp(line, 4,buff) ; nelec    =atoi(buff);
        iPickUp(line, 7,buff) ; natom_in =atoi(buff);
        iPickUp(line,10,buff) ; natom_add=atoi(buff);
        f.push_back_elec(nelec);
        if(natom_add==0){
          file.getline(line,sizeof(line));
        } else {
          count=0;
          for(int k=0;k<((natom_add-1)/10+1);k++){
            file.getline(line,sizeof(line));
            for(int j=0;j<10;j++){
              if(k==0)iPickUp(line,j+3,buff);
              if(k!=0)iPickUp(line,j+0,buff);
              f.push_back_atom_index_link(atoi(buff)-1,-1);
              count++;
              if(count==natom_add) break;
            }
          }
        }
        count=0;
        for(int k=0;k<((natom_in-1)/10+1);k++){
          file.getline(line,sizeof(line));
          for(int j=0;j<10;j++){
            if(k==0){
              iPickUp(line,j+3,buff);
              iatom_in=atoi(buff);
              f.push_back_atom_index(iatom_in-1);
              count++;
            }
            if(k!=0){
              iPickUp(line,j+0,buff);
              iatom_in=atoi(buff);
              f.push_back_atom_index(iatom_in-1);
              count++;
            }
            if(count==natom_in) break;
          }
        }
        push_back_fragment(f);
	file.getline(line, sizeof(line));
      }
      break;
    }
  }
  file.close();
}

void Molecule::read_fragment_paics_out_2( const string& filename , string& s_out )
{
  char line[1024];
  char buff[1024];
  int n,count,nelec,natom_in,iatom_in,natom_add,chk1;
  fstream file(filename.c_str(), std::ios::in);
  //if(file==NULL){ <--- 20150827
  if(!file){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  clear_fragment();
  chk1=0;
  while(!file.eof()){
    file.getline(line, sizeof(line));
    if(iFind(line,"< auto fragment update >")>=0){
      while(!file.eof()){
        file.getline(line, sizeof(line));
        if(iFind(line,"number of fragment =")>=0){
          iPickUp(line,4,buff);n=atoi(buff);
          file.getline(line, sizeof(line));
          chk1=1;
          break;
        }
      }
    }
    if(chk1==1){
      for(int i=0;i<n;i++){
        Fragment f;
        file.getline(line, sizeof(line));
        iPickUp(line, 4,buff) ; nelec    =atoi(buff);
        iPickUp(line, 7,buff) ; natom_in =atoi(buff);
        iPickUp(line,10,buff) ; natom_add=atoi(buff);
        f.push_back_elec(nelec);
        if(natom_add==0){
          file.getline(line,sizeof(line));
        } else {
          count=0;
          for(int k=0;k<((natom_add-1)/10+1);k++){
            file.getline(line,sizeof(line));
            for(int j=0;j<10;j++){
              if(k==0)iPickUp(line,j+3,buff);
              if(k!=0)iPickUp(line,j+0,buff);
              f.push_back_atom_index_link(atoi(buff)-1,-1);
              count++;
              if(count==natom_add) break;
            }
          }
        }
        count=0;
        for(int k=0;k<((natom_in-1)/10+1);k++){
          file.getline(line,sizeof(line));
          for(int j=0;j<10;j++){
            if(k==0){
              iPickUp(line,j+3,buff);
              iatom_in=atoi(buff);
              f.push_back_atom_index(iatom_in-1);
              count++;
            }
            if(k!=0){
              iPickUp(line,j+0,buff);
              iatom_in=atoi(buff);
              f.push_back_atom_index(iatom_in-1);
              count++;
            }
            if(count==natom_in) break;
          }
        }
        push_back_fragment(f); 
        file.getline(line, sizeof(line));
      }
      break;
    }
  }
  file.close();
}

void Molecule::read_amber_top(const string& filename,
                              string& s_out         )
{
  fstream file2(filename.c_str(), std::ios::in);
  //if(file2==NULL){ <--- 20150827
  if(!file2){
    s_out = s_out + "\n warning , can not open the file : " + filename + "\n\n";
    return ;
  }
  clear();
  char line[1024];
  char buff[1024];
  string atyp;
  Atom a;
  int n,chk,chkchk,atomic;
  while(! file2.eof()){
    chkchk=0;
    for(int i=0;i<1024;i++)buff[i]=' ';
    file2.getline(line,sizeof(line));
    iPickUp(line,1,buff);
    if( iFind(buff,"POINTERS")>=0 ) {
      file2.getline(line,sizeof(line));
      file2.getline(line,sizeof(line));
      iPickUp(line,0,buff);
      n=atoi(buff);
    }
    if( iFind(buff,"ATOM_NAME")>=0 ) {
      file2.getline(line,sizeof(line));
      for(int i=0;i<n;i++){
        file2.width(4);
	file2 >> atyp ;
	chk=-1;
	//atomic=get_atom_type_pdb(atyp.c_str(),&chk);
	atomic=get_atomic_number_pdb(atyp.c_str(),&chk);
	if(chk==1){
	  a.set_atomic_number(atomic);
	  add_atom(a,30000);
	}
      }
      chkchk=1;
    }
    if(chkchk==1){
      break;
    }
  }
  file2.close();
}

/*
void Molecule::read_amber_ff_type( const string& filename,
                                   string&       s_out)
{
  fstream file(filename.c_str(), std::ios::in);
  char line[1024];
  char buff[1024];
  int n,ia;
  double chg;
  while(!file.eof()){
    for(int i=0;i<1024;i++) buff[i]=' ';
    file.getline(line,sizeof(line));
    if( iFind(line,"ATOM_TYPE")>=0 ) {
      iPickUp(line,1,buff);
      n=atoi(buff);
      for(int i=0;i<n;i++){
        file.getline(line,sizeof(line));
        iPickUp(line,0,buff); ia=atoi(buff)-1;
        iPickUp(line,1,buff); atom[ia].set_amber_ff_type(buff);
        iPickUp(line,2,buff); chg=atof(buff); atom[ia].set_amber_ff_chg(chg);  // <--- "atof" is not available in Android
      }
    }
  }
  file.close();
}
*/
