
#include <cstdlib>
#include <cstdio>
#include <cmath>
#include <iostream>
#include <fstream>
#include <sstream>
#include "ForceField.h"
#include "iString.h"
#include "Molecule.h"

void ForceField::amber_force_field_calc_energy ( const Molecule&   m           ,
                                                 double          * bnd_eng     ,
                                                 double          * ang_eng     ,
                                                 double          * dih_eng     ,
                                                 double          * chg_eng_exc ,
                                                 double          * chg_eng_14p ,
                                                 double          * vdw_eng_exc ,
                                                 double          * vdw_eng_14p ,
                                                 const double    * atm_coeff_A ,
                                                 const double    * atm_coeff_B ) const
//const int         chk_bnd     ,
//const int         chk_ang     ,
//const int         chk_dih     ,
//const int         chk_exc     ,
//const int         chk_14p     ,  
//const int         calc_type   ,
//const double      max         ) const 
{
  double * x;
  double * y;
  double * z;
  int natm=m.get_natom();
  x = new double [natm+1];
  y = new double [natm+1];
  z = new double [natm+1];
  for(int i=0;i<natm;i++){
    x[i]=m.get_atom(i).get_R().get_x();
    y[i]=m.get_atom(i).get_R().get_y();
    z[i]=m.get_atom(i).get_R().get_z();
  }
  amber_force_field_calc_energy(natm,
                                x,y,z,
                                bnd_eng,
                                ang_eng,
                                dih_eng,
                                chg_eng_exc,
                                chg_eng_14p,
                                vdw_eng_exc,
                                vdw_eng_14p,
                                atm_coeff_A,
                                atm_coeff_B);
  //chk_bnd,
  //chk_ang,
  //chk_dih,
  //chk_exc,
  //chk_14p,                                
  //calc_type,
  //max);
  delete [] x;
  delete [] y;
  delete [] z;
}

void ForceField::amber_force_field_calc_energy ( const int      n           ,
                                                 const double * x           ,
                                                 const double * y           ,
                                                 const double * z           ,
                                                 double       * bnd_eng     ,
                                                 double       * ang_eng     ,
                                                 double       * dih_eng     ,
                                                 double       * chg_eng_exc ,
                                                 double       * chg_eng_14p ,
                                                 double       * vdw_eng_exc ,
                                                 double       * vdw_eng_14p ,
                                                 const double * atm_coeff_A ,
                                                 const double * atm_coeff_B ) const 
                                                 //const int      chk_bnd     ,
                                                 //const int      chk_ang     ,
                                                 //const int      chk_dih     ,
                                                 //const int      chk_exc     ,
                                                 //const int      chk_14p     ,  
                                                 //const int      calc_type   ,
                                                 //const double   max_au      ) const 
{
  // chk_bnd   = 0 : not calculate bond energy (default=1)
  // chk_ang   = 0 : not calculate angle energy (default=1)
  // chk_dih   = 0 : not calculate dihedral energy (default=1)
  // chk_exc   = 0 : not calculate excluded pair (default=1)
  // chk_14p   = 0 : not calculate 14-pair (default=1)
  // calc_type = 0 : normal energy calculation
  // calc_type = 1 : stop energy calculation in energy > max (a.u.)
  // dist_out      : in angstrome
 
  int i,j,k,chk,pp1,pp2;
  double x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4,sum_v,v,v1,v2,sum_v1,sum_v2;
  double d,ang,r1,r3,dot,r12_x,r12_y,r12_z,r32_x,r32_y,r32_z,dih,r32n,n_12_32_n_32_34,nn1232,nn3234,r,r12_r32,r32_r34;
  double r34_x,r34_y,r34_z,n_12_32_x,n_12_32_y,n_12_32_z,n_32_34_x,n_32_34_y,n_32_34_z,n_x,n_y,n_z,coeffA,coeffB;
  char c[1024];

  //const double max_kcal_mol=max_au*627.51;
  double * x_ = new double [n];
  double * y_ = new double [n];
  double * z_ = new double [n];

  for(int i=0;i<n;i++){
    x_[i]=x[i]*0.5291772108;
    y_[i]=y[i]*0.5291772108;
    z_[i]=z[i]*0.5291772108;
  }

  (*bnd_eng)=0.0;
  (*ang_eng)=0.0;
  (*dih_eng)=0.0;
  (*chg_eng_exc)=0.0;
  (*chg_eng_14p)=0.0;
  (*vdw_eng_exc)=0.0;
  (*vdw_eng_14p)=0.0;

  //if(chk_exc==1){
  sum_v1=0.0;
  sum_v2=0.0;
  for(i=0;i<natm;i++){
    x1=x_[i];
    y1=y_[i];
    z1=z_[i];
    for(j=(i+1);j<natm;j++){
      chk=0;
      for(k=0;k<atm_excluded_n[i];k++){
        if(j==atm_excluded[i][k]){
          chk=1;
        }
      }
      if(chk==0){
        x2=x_[j];
        y2=y_[j];
        z2=z_[j];
        
        //////////////////
        // distance (d) //
        //////////////////
        d=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
        
        ////////////////////////////////////////////////
        // non-bonding energy (exclude 12,13,14 pair) //
        ////////////////////////////////////////////////
        if(atm_coeff_A==NULL){
          coeffA=sqrt(atm_vdw_e[i]*atm_vdw_e[j])*pow(atm_vdw_r[i]+atm_vdw_r[j],12);
        } else {
          coeffA=atm_coeff_A[i*n+j];
        }
        if(atm_coeff_B==NULL){
          coeffB=2.0*sqrt(atm_vdw_e[i]*atm_vdw_e[j])*pow(atm_vdw_r[i]+atm_vdw_r[j], 6);
        } else {
          coeffB=atm_coeff_B[i*n+j];
        }
        v1=coeffA/pow(d,12)-coeffB/pow(d,6);
        v2=(atm_chg[i]*18.2223)*(atm_chg[j]*18.2223)/d;
        sum_v1=sum_v1+v1;
        sum_v2=sum_v2+v2;

        //cout.width(10);cout << i+1;
        //cout.width(10);cout << j+1;
        //cout.width(10);cout << atm_vdw_e[i];
        //cout.width(10);cout << atm_vdw_e[j];
        //cout.width(10);cout << atm_vdw_r[i];
        //cout.width(10);cout << atm_vdw_r[j];
        //cout.width(10);cout << coeffA ;
        //cout.width(10);cout << coeffB ;
        //cout.width(10);cout << atm_chg[i];
        //cout.width(10);cout << atm_chg[j];
        //cout.width(10);cout << d ;
        //cout.width(16);cout << v1 ;
        //cout.width(16);cout << sum_v1 ;
        //cout << endl ;
        
        /////////////////////////////
        // stop energy calculation //
        /////////////////////////////
        //if(calc_type==1){ 
        //  if((sum_v1+sum_v2)>max_kcal_mol){
        //    (*vdw_eng_exc)=sum_v1/627.51;
        //    (*chg_eng_exc)=sum_v2/627.51;
        //    return;
        //  }
        //}
      }
    }
  }
  (*vdw_eng_exc)=sum_v1/627.51;
  (*chg_eng_exc)=sum_v2/627.51;
  //}
  
  //if(chk_14p==1){
  sum_v1=0.0;
  sum_v2=0.0;
  for(i=0;i<np14;i++){
    x1=x_[p14_1[i]];
    y1=y_[p14_1[i]];
    z1=z_[p14_1[i]];
    x2=x_[p14_2[i]];
    y2=y_[p14_2[i]];
    z2=z_[p14_2[i]];
    
    //////////////////
    // distance (d) //
    //////////////////
    d=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
    
    ///////////////////////////////////////
    // non-bonding energy (only 14 pair) //
    ///////////////////////////////////////
    coeffA=    sqrt(atm_vdw_e[p14_1[i]]*atm_vdw_e[p14_2[i]])*pow(atm_vdw_r[p14_1[i]]+atm_vdw_r[p14_2[i]],12);
    coeffB=2.0*sqrt(atm_vdw_e[p14_1[i]]*atm_vdw_e[p14_2[i]])*pow(atm_vdw_r[p14_1[i]]+atm_vdw_r[p14_2[i]], 6);
    v1=coeffA/pow(d,12)-coeffB/pow(d,6);
    v2=(atm_chg[p14_1[i]]*18.2223)*(atm_chg[p14_2[i]]*18.2223)/d;
    sum_v1=sum_v1+v1/2.00;
    sum_v2=sum_v2+v2/1.20;
    
    /////////////////////////////
    // stop energy calculation //
    /////////////////////////////
    //if(calc_type==1){
    //  if(((*vdw_eng_exc)+(*chg_eng_exc)+sum_v1+sum_v2)>max_kcal_mol){
    //    (*vdw_eng_14p)=sum_v1/627.51;
    //    (*chg_eng_14p)=sum_v2/627.51;
    //    return;
    //  }
    //}
  }
  (*vdw_eng_14p)=sum_v1/627.51;
  (*chg_eng_14p)=sum_v2/627.51;
  //}
  
  //if(chk_bnd==1){
  sum_v=0.0;
  for(i=0;i<nbnd;i++){
    
    //////////////////
    // distance (d) //
    //////////////////
    x1=x_[bnd1[i]];
    y1=y_[bnd1[i]];
    z1=z_[bnd1[i]];
    x2=x_[bnd2[i]];
    y2=y_[bnd2[i]];
    z2=z_[bnd2[i]];
    d=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
    
    /////////////////
    // bond energy //
    /////////////////
    v=bnd_coeff[i]*(d-bnd_eq[i])*(d-bnd_eq[i]);
    sum_v=sum_v+v;
  }
  (*bnd_eng)=sum_v/627.51;
  //}
  
  //if(chk_ang==1){
  sum_v=0.0;
  for(i=0;i<nang;i++){
    
    /////////////////
    // angle (ang) //
    /////////////////
    x1=x_[ang1[i]];
    y1=y_[ang1[i]];
    z1=z_[ang1[i]];
    x2=x_[ang2[i]];
    y2=y_[ang2[i]];
    z2=z_[ang2[i]];
    x3=x_[ang3[i]];
    y3=y_[ang3[i]];
    z3=z_[ang3[i]];
    
    r12_x=x2-x1; r12_y=y2-y1; r12_z=z2-z1;
    r32_x=x2-x3; r32_y=y2-y3; r32_z=z2-z3;
    r1=sqrt(r12_x*r12_x+r12_y*r12_y+r12_z*r12_z);
    r3=sqrt(r32_x*r32_x+r32_y*r32_y+r32_z*r32_z);
    dot   =(r12_x*r32_x+r12_y*r32_y+r12_z*r32_z);
    ang=acos(dot/(r1*r3));
    
    //////////////////
    // angle energy //
    //////////////////
    v=ang_coeff[i]*(ang-ang_eq[i]*3.141594/180.0)*(ang-ang_eq[i]*3.141594/180.0);
    sum_v=sum_v+v;
  }
  (*ang_eng)=sum_v/627.51;
  //}
  
  //if(chk_dih==1){
  sum_v=0.0;
  for(i=0;i<ndih;i++){
    
    ////////////////////
    // dihedral (dih) //
    ////////////////////
    x1=x_[dih1[i]];
    y1=y_[dih1[i]];
    z1=z_[dih1[i]];
    x2=x_[dih2[i]];
    y2=y_[dih2[i]];
    z2=z_[dih2[i]];
    x3=x_[dih3[i]];
    y3=y_[dih3[i]];
    z3=z_[dih3[i]];
    x4=x_[dih4[i]];
    y4=y_[dih4[i]];
    z4=z_[dih4[i]];
    
    r12_x=x2-x1; r12_y=y2-y1; r12_z=z2-z1;
    r32_x=x2-x3; r32_y=y2-y3; r32_z=z2-z3;
    r34_x=x4-x3; r34_y=y4-y3; r34_z=z4-z3;
    r12_r32=r12_x*r32_x+r12_y*r32_y+r12_z*r32_z;
    r32_r34=r32_x*r34_x+r32_y*r34_y+r32_z*r34_z;
    if(-0.00000001 < r12_r32 && r12_r32 < +0.00000001) { printf("r12_r32 = 0.0 (1,2,3 are linear) !!\n"); exit(1);}
    if(-0.00000001 < r32_r34 && r32_r34 < +0.00000001) { printf("r32_r34 = 0.0 (2,3,4 are linear) !!\n"); exit(1);}
    n_12_32_x=r12_y*r32_z-r12_z*r32_y; n_12_32_y=r12_z*r32_x-r12_x*r32_z; n_12_32_z=r12_x*r32_y-r12_y*r32_x;
    n_32_34_x=r32_y*r34_z-r32_z*r34_y; n_32_34_y=r32_z*r34_x-r32_x*r34_z; n_32_34_z=r32_x*r34_y-r32_y*r34_x;
    nn1232=sqrt(n_12_32_x*n_12_32_x+n_12_32_y*n_12_32_y+n_12_32_z*n_12_32_z);
    nn3234=sqrt(n_32_34_x*n_32_34_x+n_32_34_y*n_32_34_y+n_32_34_z*n_32_34_z);
    n_12_32_n_32_34=n_12_32_x*n_32_34_x+n_12_32_y*n_32_34_y+n_12_32_z*n_32_34_z;
    if     (n_12_32_n_32_34/(nn1232*nn3234) >  0.99999999){ dih=0.0      ; /*  0.0*/ }
    else if(n_12_32_n_32_34/(nn1232*nn3234) < -0.99999999){ dih=3.141592 ; /*180.0*/ }
    else {
      n_x=n_12_32_y*n_32_34_z-n_12_32_z*n_32_34_y;
      n_y=n_12_32_z*n_32_34_x-n_12_32_x*n_32_34_z;
      n_z=n_12_32_x*n_32_34_y-n_12_32_y*n_32_34_x;
      r32n=r32_x*n_x+r32_y*n_y+r32_z*n_z;
      if  (r32n < 0.0){dih= -(-1.0)*acos(n_12_32_n_32_34/(nn1232*nn3234));}
      else            {dih= -(+1.0)*acos(n_12_32_n_32_34/(nn1232*nn3234));}
    }
    
    /////////////////////
    // dihedral energy //
    /////////////////////
    v=(dih_coeff[i]/dih_div[i])*(1.0+cos(dih_phase[i]*dih-dih_eq[i]*3.141594/180.00));
    sum_v=sum_v+v;
  }
  (*dih_eng)=sum_v/627.51;
  //}
  
  delete [] x_;
  delete [] y_;
  delete [] z_;
}

void ForceField::amber_force_field_calc_gradient ( const Molecule&   m               ,
                                                   double          * grad_x          ,
                                                   double          * grad_y          ,
                                                   double          * grad_z          ) const 
{
  double * x;
  double * y;
  double * z;
  int natm=m.get_natom();
  x = new double [natm+1];
  y = new double [natm+1];
  z = new double [natm+1];
  for(int i=0;i<natm;i++){
    x[i]=m.get_atom(i).get_R().get_x();
    y[i]=m.get_atom(i).get_R().get_y();
    z[i]=m.get_atom(i).get_R().get_z();
  }
  amber_force_field_calc_gradient(x,y,z,
                                  grad_x,
                                  grad_y,
                                  grad_z);
  delete [] x;
  delete [] y;
  delete [] z;
}

void ForceField::amber_force_field_calc_gradient ( const double    * x               ,
                                                   const double    * y               ,
                                                   const double    * z               ,
                                                   double          * grad_x          ,
                                                   double          * grad_y          ,
                                                   double          * grad_z          ) const 
{
  int i,j,k,chk,ia1,ia2,ia3,ia4,imax;
  double x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4,ang123,d12,d32,dot,r12_r32,r32_r34,nn1232,nn3234,n_12_32_n_32_34,r32n,dih,fa,buf,coeffA,coeffB,gmax;
  double f1[3],f2[3],f3[3],f4[3],r12[3],r32[3],r34[3],n_12_32[3],n_32_34[3],n[3],fb[3],fc[3],r13[3],r24[3];

  for(i=0;i<natm;i++){
    grad_x[i]=0;
    grad_y[i]=0;
    grad_z[i]=0;
  }

  /////////////////////////////////
  // force from bond interaction //
  ///////////////////////////////// 

  for(i=0;i<nbnd;i++){

    // atomic coordinate
    ia1=bnd1[i];
    ia2=bnd2[i];
    x1=x[ia1]*0.5291772108;
    y1=y[ia1]*0.5291772108;
    z1=z[ia1]*0.5291772108;
    x2=x[ia2]*0.5291772108;
    y2=y[ia2]*0.5291772108;
    z2=z[ia2]*0.5291772108;

    // distance
    r12[0]=x2-x1;
    r12[1]=y2-y1;
    r12[2]=z2-z1;
    d12=sqrt(r12[0]*r12[0]+r12[1]*r12[1]+r12[2]*r12[2]);

    // force
    f1[0]=2.0*bnd_coeff[i]*(d12-bnd_eq[i])*(r12[0]/d12);
    f1[1]=2.0*bnd_coeff[i]*(d12-bnd_eq[i])*(r12[1]/d12);
    f1[2]=2.0*bnd_coeff[i]*(d12-bnd_eq[i])*(r12[2]/d12);
    f2[0]=-f1[0];
    f2[1]=-f1[1];
    f2[2]=-f1[2];

    // gradient
    grad_x[ia1]=grad_x[ia1]+(-f1[0]);
    grad_y[ia1]=grad_y[ia1]+(-f1[1]);
    grad_z[ia1]=grad_z[ia1]+(-f1[2]);
    grad_x[ia2]=grad_x[ia2]+(-f2[0]);
    grad_y[ia2]=grad_y[ia2]+(-f2[1]);
    grad_z[ia2]=grad_z[ia2]+(-f2[2]);
  }

  //////////////////////////////////
  // force from angle interaction //
  //////////////////////////////////

  for(i=0;i<nang;i++){

    // atomic coordinate
    ia1=ang1[i];
    ia2=ang2[i];
    ia3=ang3[i];
    x1=x[ia1]*0.5291772108;
    y1=y[ia1]*0.5291772108;
    z1=z[ia1]*0.5291772108;
    x2=x[ia2]*0.5291772108;
    y2=y[ia2]*0.5291772108;
    z2=z[ia2]*0.5291772108;
    x3=x[ia3]*0.5291772108;
    y3=y[ia3]*0.5291772108;
    z3=z[ia3]*0.5291772108;

    // angle
    r12[0]=x2-x1;
    r12[1]=y2-y1;
    r12[2]=z2-z1;
    r32[0]=x2-x3;
    r32[1]=y2-y3;
    r32[2]=z2-z3;
    d12=sqrt(r12[0]*r12[0]+r12[1]*r12[1]+r12[2]*r12[2]);
    d32=sqrt(r32[0]*r32[0]+r32[1]*r32[1]+r32[2]*r32[2]);
    dot=(r12[0]*r32[0]+r12[1]*r32[1]+r12[2]*r32[2]);
    ang123=acos(dot/(d12*d32));

    // force
    f1[0]=-2.0*ang_coeff[i]*(ang123-ang_eq[i]*3.141594/180.0)*(1.0/(d12*sin(ang123)))*(r32[0]/d32-(dot/(d12*d32))*(r12[0]/d12));
    f1[1]=-2.0*ang_coeff[i]*(ang123-ang_eq[i]*3.141594/180.0)*(1.0/(d12*sin(ang123)))*(r32[1]/d32-(dot/(d12*d32))*(r12[1]/d12));
    f1[2]=-2.0*ang_coeff[i]*(ang123-ang_eq[i]*3.141594/180.0)*(1.0/(d12*sin(ang123)))*(r32[2]/d32-(dot/(d12*d32))*(r12[2]/d12));
    f3[0]=-2.0*ang_coeff[i]*(ang123-ang_eq[i]*3.141594/180.0)*(1.0/(d32*sin(ang123)))*(r12[0]/d12-(dot/(d12*d32))*(r32[0]/d32));
    f3[1]=-2.0*ang_coeff[i]*(ang123-ang_eq[i]*3.141594/180.0)*(1.0/(d32*sin(ang123)))*(r12[1]/d12-(dot/(d12*d32))*(r32[1]/d32));
    f3[2]=-2.0*ang_coeff[i]*(ang123-ang_eq[i]*3.141594/180.0)*(1.0/(d32*sin(ang123)))*(r12[2]/d12-(dot/(d12*d32))*(r32[2]/d32));
    f2[0]=-f1[0]-f3[0];
    f2[1]=-f1[1]-f3[1];
    f2[2]=-f1[2]-f3[2];

    // gradient
    grad_x[ia1]=grad_x[ia1]+(-f1[0]);
    grad_y[ia1]=grad_y[ia1]+(-f1[1]);
    grad_z[ia1]=grad_z[ia1]+(-f1[2]);
    grad_x[ia2]=grad_x[ia2]+(-f2[0]);
    grad_y[ia2]=grad_y[ia2]+(-f2[1]);
    grad_z[ia2]=grad_z[ia2]+(-f2[2]);
    grad_x[ia3]=grad_x[ia3]+(-f3[0]);
    grad_y[ia3]=grad_y[ia3]+(-f3[1]);
    grad_z[ia3]=grad_z[ia3]+(-f3[2]);
  }

  /////////////////////////////////////
  // force from dihedral interaction //
  /////////////////////////////////////

  for(i=0;i<ndih;i++){

    // atomic coordinate
    ia1=dih1[i];
    ia2=dih2[i];
    ia3=dih3[i];
    ia4=dih4[i];
    x1=x[ia1]*0.5291772108;
    y1=y[ia1]*0.5291772108;
    z1=z[ia1]*0.5291772108;
    x2=x[ia2]*0.5291772108;
    y2=y[ia2]*0.5291772108;
    z2=z[ia2]*0.5291772108;
    x3=x[ia3]*0.5291772108;
    y3=y[ia3]*0.5291772108;
    z3=z[ia3]*0.5291772108;
    x4=x[ia4]*0.5291772108;
    y4=y[ia4]*0.5291772108;
    z4=z[ia4]*0.5291772108;

    // dihedral
    r12[0]=x2-x1; r12[1]=y2-y1; r12[2]=z2-z1;
    r32[0]=x2-x3; r32[1]=y2-y3; r32[2]=z2-z3;
    r34[0]=x4-x3; r34[1]=y4-y3; r34[2]=z4-z3;
    r13[0]=x3-x1; r13[1]=y3-y1; r13[2]=z3-z1;
    r24[0]=x4-x2; r24[1]=y4-y2; r24[2]=z4-z2;
    r12_r32=r12[0]*r32[0]+r12[1]*r32[1]+r12[2]*r32[2];
    r32_r34=r32[0]*r34[0]+r32[1]*r34[1]+r32[2]*r34[2];
    if(-0.00000001 < r12_r32 && r12_r32 < +0.00000001) {printf("r12_r32 = 0.0 !! in dihedral calculation\n"); exit(1);}
    if(-0.00000001 < r32_r34 && r32_r34 < +0.00000001) {printf("r32_r34 = 0.0 !! in dihedral calculation\n"); exit(1);}
    n_12_32[0]=r12[1]*r32[2]-r12[2]*r32[1];
    n_12_32[1]=r12[2]*r32[0]-r12[0]*r32[2];
    n_12_32[2]=r12[0]*r32[1]-r12[1]*r32[0];
    n_32_34[0]=r32[1]*r34[2]-r32[2]*r34[1];
    n_32_34[1]=r32[2]*r34[0]-r32[0]*r34[2];
    n_32_34[2]=r32[0]*r34[1]-r32[1]*r34[0];
    nn1232=sqrt(n_12_32[0]*n_12_32[0]+n_12_32[1]*n_12_32[1]+n_12_32[2]*n_12_32[2]);
    nn3234=sqrt(n_32_34[0]*n_32_34[0]+n_32_34[1]*n_32_34[1]+n_32_34[2]*n_32_34[2]);
    n_12_32_n_32_34=n_12_32[0]*n_32_34[0]+n_12_32[1]*n_32_34[1]+n_12_32[2]*n_32_34[2];
    if     (n_12_32_n_32_34/(nn1232*nn3234) >  0.99999999){ dih=0.0      ; /*  0.0*/ }
    else if(n_12_32_n_32_34/(nn1232*nn3234) < -0.99999999){ dih=3.141592 ; /*180.0*/ }
    else {
      n[0]=n_12_32[1]*n_32_34[2]-n_12_32[2]*n_32_34[1];
      n[1]=n_12_32[2]*n_32_34[0]-n_12_32[0]*n_32_34[2];
      n[2]=n_12_32[0]*n_32_34[1]-n_12_32[1]*n_32_34[0];
      r32n=r32[0]*n[0]+r32[1]*n[1]+r32[2]*n[2];
      if  (r32n < 0.0){dih= -(-1.0)*acos(n_12_32_n_32_34/(nn1232*nn3234));}
      else            {dih= -(+1.0)*acos(n_12_32_n_32_34/(nn1232*nn3234));}
    }

    // force
    fb[0]=(n_32_34[0]/nn3234-cos(dih)*n_12_32[0]/nn1232)/nn1232;
    fb[1]=(n_32_34[1]/nn3234-cos(dih)*n_12_32[1]/nn1232)/nn1232;
    fb[2]=(n_32_34[2]/nn3234-cos(dih)*n_12_32[2]/nn1232)/nn1232;
    fc[0]=(n_12_32[0]/nn1232-cos(dih)*n_32_34[0]/nn3234)/nn3234;
    fc[1]=(n_12_32[1]/nn1232-cos(dih)*n_32_34[1]/nn3234)/nn3234;
    fc[2]=(n_12_32[2]/nn1232-cos(dih)*n_32_34[2]/nn3234)/nn3234;
    if(fabs(dih_eq[i]-0.0)<1.0E-10 || fabs(dih_eq[i]-180.0)<1.0E-10){
      if     (fabs(dih_phase[i]-1.0)<1.0E-10){buf=cos(dih_eq[i]*3.141594/180.00)                                           ;}
      else if(fabs(dih_phase[i]-2.0)<1.0E-10){buf=cos(dih_eq[i]*3.141594/180.00)*(2.0*cos(dih))                            ;}
      else if(fabs(dih_phase[i]-3.0)<1.0E-10){buf=cos(dih_eq[i]*3.141594/180.00)*(3.0-4.0*sin(dih)*sin(dih))               ;}
      else if(fabs(dih_phase[i]-4.0)<1.0E-10){buf=cos(dih_eq[i]*3.141594/180.00)*(4.0*cos(dih)*(2.0*cos(dih)*cos(dih)-1.0));}
      else {
        printf("ff_dih_phase[i]=%10.4f, not 1.0 or 2.0 ro 3.0 or 4.0 in dihedral calculation\n",dih_phase[i]); exit(1);
      }
      fa=-1.0*(dih_coeff[i]/dih_div[i])*dih_phase[i]*buf;
    } else {
      printf("ff_dih_eq[i]=%10.4f, not 0.0 or 180.0 in dihedral calculation\n",dih_eq[i]); exit(1);
    }
    f1[0]=fa*(fb[1]*r32[2]-fb[2]*r32[1]);
    f1[1]=fa*(fb[2]*r32[0]-fb[0]*r32[2]);
    f1[2]=fa*(fb[0]*r32[1]-fb[1]*r32[0]);
    f2[0]=fa*(-1.0*(fc[1]*r34[2]-fc[2]*r34[1])+(fb[1]*r13[2]-fb[2]*r13[1]));
    f2[1]=fa*(-1.0*(fc[2]*r34[0]-fc[0]*r34[2])+(fb[2]*r13[0]-fb[0]*r13[2]));
    f2[2]=fa*(-1.0*(fc[0]*r34[1]-fc[1]*r34[0])+(fb[0]*r13[1]-fb[1]*r13[0]));
    f3[0]=fa*(-1.0*(fb[1]*r12[2]-fb[2]*r12[1])+(fc[1]*r24[2]-fc[2]*r24[1]));
    f3[1]=fa*(-1.0*(fb[2]*r12[0]-fb[0]*r12[2])+(fc[2]*r24[0]-fc[0]*r24[2]));
    f3[2]=fa*(-1.0*(fb[0]*r12[1]-fb[1]*r12[0])+(fc[0]*r24[1]-fc[1]*r24[0]));
    f4[0]=fa*(fc[1]*r32[2]-fc[2]*r32[1]);
    f4[1]=fa*(fc[2]*r32[0]-fc[0]*r32[2]);
    f4[2]=fa*(fc[0]*r32[1]-fc[1]*r32[0]);

    // gradient
    grad_x[ia1]=grad_x[ia1]+(-f1[0]);
    grad_y[ia1]=grad_y[ia1]+(-f1[1]);
    grad_z[ia1]=grad_z[ia1]+(-f1[2]);
    grad_x[ia2]=grad_x[ia2]+(-f2[0]);
    grad_y[ia2]=grad_y[ia2]+(-f2[1]);
    grad_z[ia2]=grad_z[ia2]+(-f2[2]);
    grad_x[ia3]=grad_x[ia3]+(-f3[0]);
    grad_y[ia3]=grad_y[ia3]+(-f3[1]);
    grad_z[ia3]=grad_z[ia3]+(-f3[2]);
    grad_x[ia4]=grad_x[ia4]+(-f4[0]);
    grad_y[ia4]=grad_y[ia4]+(-f4[1]);
    grad_z[ia4]=grad_z[ia4]+(-f4[2]);
  }

  /////////////////////////////////////////////////////////////
  // force from non-bonding interactin (not 12, 13, 14 pair) //
  /////////////////////////////////////////////////////////////

  for(i=0;i<natm;i++){
    for(j=(i+1);j<natm;j++){
      chk=0;
      for(k=0;k<atm_excluded_n[i];k++){
        if(j==atm_excluded[i][k]){
          chk=1;
	}
      }
      if(chk==0){

        // atomic coordinate
        ia1=i;
        ia2=j;
        x1=x[ia1]*0.5291772108;
        y1=y[ia1]*0.5291772108;
        z1=z[ia1]*0.5291772108;
        x2=x[ia2]*0.5291772108;
        y2=y[ia2]*0.5291772108;
        z2=z[ia2]*0.5291772108;
        
        // distance
        r12[0]=x2-x1;
        r12[1]=y2-y1;
        r12[2]=z2-z1;
        d12=sqrt(r12[0]*r12[0]+r12[1]*r12[1]+r12[2]*r12[2]);

        // force (van der Waals)
        coeffA=    sqrt(atm_vdw_e[ia1]*atm_vdw_e[ia2])*pow(atm_vdw_r[ia1]+atm_vdw_r[ia2],12);
        coeffB=2.0*sqrt(atm_vdw_e[ia1]*atm_vdw_e[ia2])*pow(atm_vdw_r[ia1]+atm_vdw_r[ia2], 6);
        f1[0]=-(12.0*coeffA/pow(d12,14)*r12[0]-6.0*coeffB/pow(d12,8)*r12[0]);
        f1[1]=-(12.0*coeffA/pow(d12,14)*r12[1]-6.0*coeffB/pow(d12,8)*r12[1]);
        f1[2]=-(12.0*coeffA/pow(d12,14)*r12[2]-6.0*coeffB/pow(d12,8)*r12[2]);
        f2[0]=-1.0*f1[0];
        f2[1]=-1.0*f1[1];
        f2[2]=-1.0*f1[2];

        // gradient (van der Waals)
        grad_x[ia1]=grad_x[ia1]+(-f1[0]);
        grad_y[ia1]=grad_y[ia1]+(-f1[1]);
        grad_z[ia1]=grad_z[ia1]+(-f1[2]);
        grad_x[ia2]=grad_x[ia2]+(-f2[0]);
        grad_y[ia2]=grad_y[ia2]+(-f2[1]);
        grad_z[ia2]=grad_z[ia2]+(-f2[2]);

        // force (electrostatic)
        f1[0]=-(atm_chg[ia1]*18.2223)*(atm_chg[ia2]*18.2223)/(d12*d12*d12)*r12[0];
        f1[1]=-(atm_chg[ia1]*18.2223)*(atm_chg[ia2]*18.2223)/(d12*d12*d12)*r12[1];
        f1[2]=-(atm_chg[ia1]*18.2223)*(atm_chg[ia2]*18.2223)/(d12*d12*d12)*r12[2];
        f2[0]=-1.0*f1[0];
        f2[1]=-1.0*f1[1];
        f2[2]=-1.0*f1[2];

        // gradient (electrostatic)
        grad_x[ia1]=grad_x[ia1]+(-f1[0]);
        grad_y[ia1]=grad_y[ia1]+(-f1[1]);
        grad_z[ia1]=grad_z[ia1]+(-f1[2]);
        grad_x[ia2]=grad_x[ia2]+(-f2[0]);
        grad_y[ia2]=grad_y[ia2]+(-f2[1]);
        grad_z[ia2]=grad_z[ia2]+(-f2[2]);
      }
    }
  }

  //////////////////////////////////////////////////////
  // force from non-bonding interactin (only 14 pair) //
  //////////////////////////////////////////////////////

  for(i=0;i<np14;i++){

    // atomic coordinate
    ia1=p14_1[i];
    ia2=p14_2[i];
    x1=x[ia1]*0.5291772108;
    y1=y[ia1]*0.5291772108;
    z1=z[ia1]*0.5291772108;
    x2=x[ia2]*0.5291772108;
    y2=y[ia2]*0.5291772108;
    z2=z[ia2]*0.5291772108;

    // distance
    r12[0]=x2-x1; r12[1]=y2-y1; r12[2]=z2-z1;
    d12=sqrt(r12[0]*r12[0]+r12[1]*r12[1]+r12[2]*r12[2]);

    // force (van der Waals)
    coeffA=    sqrt(atm_vdw_e[ia1]*atm_vdw_e[ia2])*pow(atm_vdw_r[ia1]+atm_vdw_r[ia2],12);
    coeffB=2.0*sqrt(atm_vdw_e[ia1]*atm_vdw_e[ia2])*pow(atm_vdw_r[ia1]+atm_vdw_r[ia2], 6);
    f1[0]=-(12.0*coeffA/pow(d12,14)*r12[0]-6.0*coeffB/pow(d12,8)*r12[0])/2.0;
    f1[1]=-(12.0*coeffA/pow(d12,14)*r12[1]-6.0*coeffB/pow(d12,8)*r12[1])/2.0;
    f1[2]=-(12.0*coeffA/pow(d12,14)*r12[2]-6.0*coeffB/pow(d12,8)*r12[2])/2.0;
    f2[0]=-1.0*f1[0];
    f2[1]=-1.0*f1[1];
    f2[2]=-1.0*f1[2];

    // gradient (van der Waals)
    grad_x[ia1]=grad_x[ia1]+(-f1[0]);
    grad_y[ia1]=grad_y[ia1]+(-f1[1]);
    grad_z[ia1]=grad_z[ia1]+(-f1[2]);
    grad_x[ia2]=grad_x[ia2]+(-f2[0]);
    grad_y[ia2]=grad_y[ia2]+(-f2[1]);
    grad_z[ia2]=grad_z[ia2]+(-f2[2]);
    
    // force (electrostatic)
    f1[0]=-((atm_chg[ia1]*18.2223)*(atm_chg[ia2]*18.2223)/(d12*d12*d12)*r12[0])/1.2;
    f1[1]=-((atm_chg[ia1]*18.2223)*(atm_chg[ia2]*18.2223)/(d12*d12*d12)*r12[1])/1.2;
    f1[2]=-((atm_chg[ia1]*18.2223)*(atm_chg[ia2]*18.2223)/(d12*d12*d12)*r12[2])/1.2;
    f2[0]=-1.0*f1[0];
    f2[1]=-1.0*f1[1];
    f2[2]=-1.0*f1[2];

    // gradient (electrostatic)
    grad_x[ia1]=grad_x[ia1]+(-f1[0]);
    grad_y[ia1]=grad_y[ia1]+(-f1[1]);
    grad_z[ia1]=grad_z[ia1]+(-f1[2]);
    grad_x[ia2]=grad_x[ia2]+(-f2[0]);
    grad_y[ia2]=grad_y[ia2]+(-f2[1]);
    grad_z[ia2]=grad_z[ia2]+(-f2[2]);
  }
}

string ForceField::amber_force_field_get_atom_type ( const int ai ,
                                                     const int tm ,
                                                     const int info_1)
{
  /* term == 0  : not term */
  /* term == 1  : N-term   */
  /* term == 2  : C-term   */
  /* term == 61 : ACE-N-term   */
  /* term == 62 : NME-C-term   */
  /* info_1 (1800 <= ai <=1899) 0:HID, 1:HIE, 2:HIP */
  /* info_1 (1500 <= ai <=1599) 0:CYS, 1:CYX        */
  
  string s;

  if (ai==   0)              {s="CT"; return s;}  
  if (ai==   1)              {s="C" ; return s;}
  if (ai==   2 && tm==1)     {s="N3"; return s;}  
  if (ai==   2)              {s="N" ; return s;}
  if (ai==   3 && tm==1)     {s="HP"; return s;}  
  if (ai==   3)              {s="H1"; return s;}  
  if (ai==   4 && tm==2)     {s="O2"; return s;}  
  if (ai==   4)              {s="O" ; return s;}  
  if (ai==   5)              {s="H" ; return s;}  
  if (ai==   6)              {}
  if (ai==   7)              {}
  if (ai==   8 && tm==1)     {s="HP"; return s;}
  if (ai==   8         )     {s="H1"; return s;}
  if (ai==   9 && tm==1)     {s="HP"; return s;}
  if (ai==   9         )     {s="H1"; return s;}
  if (ai==  10)              {s="H" ; return s;}
  if (ai==  11)              {s="H" ; return s;}
  if (ai==  12)              {s="H" ; return s;}

  // ACE (20150120)
  if (ai==  81 && tm==61)    {s="C" ; return s;} // C (same as 1)
  if (ai==  82 && tm==61)    {s="O" ; return s;} // O (same as 4)
  if (ai==  83 && tm==61)    {s="CT"; return s;} // CH3
  if (ai==  84 && tm==61)    {s="HC"; return s;} // H31
  if (ai==  85 && tm==61)    {s="HC"; return s;} // H32
  if (ai==  86 && tm==61)    {s="HC"; return s;} // H33

  // NME (20150120)
  if (ai==  87 && tm==62)    {s="N" ; return s;} // N (same as 2)
  if (ai==  88 && tm==62)    {s="H" ; return s;} // H (same as 5)
  if (ai==  89 && tm==62)    {s="CT"; return s;} // CH3
  if (ai==  90 && tm==62)    {s="H1"; return s;} // H31
  if (ai==  91 && tm==62)    {s="H1"; return s;} // H32
  if (ai==  92 && tm==62)    {s="H1"; return s;} // H33

  // ala
  if (ai== 205)              {s="CT"; return s;} //CB 
  if (ai== 206)              {s="HC"; return s;} //HB1
  if (ai== 207)              {s="HC"; return s;} //HB2
  if (ai== 208)              {s="HC"; return s;} //HB3

  // val
  if (ai== 305)              {s="CT"; return s;} //CB
  if (ai== 306)              {s="CT"; return s;} //CG1
  if (ai== 307)              {s="CT"; return s;} //CG2
  if (ai== 308)              {s="HC"; return s;} //HB
  if (ai== 309)              {s="HC"; return s;} //HG11
  if (ai== 310)              {s="HC"; return s;} //HG12
  if (ai== 311)              {s="HC"; return s;} //HG13
  if (ai== 312)              {s="HC"; return s;} //HG21
  if (ai== 313)              {s="HC"; return s;} //HG22
  if (ai== 314)              {s="HC"; return s;} //HG23

  // leu
  if (ai== 405)              {s="CT"; return s;} // CB
  if (ai== 406)              {s="CT"; return s;} // CG
  if (ai== 407)              {s="CT"; return s;} // CD1
  if (ai== 408)              {s="CT"; return s;} // CD2
  if (ai== 409)              {s="HC"; return s;} // HB2
  if (ai== 410)              {s="HC"; return s;} // HB3
  if (ai== 411)              {s="HC"; return s;} // HG
  if (ai== 412)              {s="HC"; return s;} // HD11
  if (ai== 413)              {s="HC"; return s;} // HD12
  if (ai== 414)              {s="HC"; return s;} // HD13
  if (ai== 415)              {s="HC"; return s;} // HD21
  if (ai== 416)              {s="HC"; return s;} // HD22
  if (ai== 417)              {s="HC"; return s;} // HD23

  // ile
  if (ai== 505)              {s="CT"; return s;} // CB
  if (ai== 506)              {s="CT"; return s;} // CG2
  if (ai== 507)              {s="CT"; return s;} // CG1
  if (ai== 508)              {s="CT"; return s;} // CD1
  if (ai== 509)              {s="HC"; return s;} // HB
  if (ai== 510)              {s="HC"; return s;} // HG21
  if (ai== 511)              {s="HC"; return s;} // HG22
  if (ai== 512)              {s="HC"; return s;} // HG23
  if (ai== 513)              {s="HC"; return s;} // HG12
  if (ai== 514)              {s="HC"; return s;} // HG13
  if (ai== 515)              {s="HC"; return s;} // HD11
  if (ai== 516)              {s="HC"; return s;} // HD12
  if (ai== 517)              {s="HC"; return s;} // HD13

  // met
  if (ai== 605)              {s="CT"; return s;} // CB
  if (ai== 606)              {s="CT"; return s;} // CG
  if (ai== 607)              {s="S" ; return s;} // SD
  if (ai== 608)              {s="CT"; return s;} // CE
  if (ai== 609)              {s="HC"; return s;} // HB2
  if (ai== 610)              {s="HC"; return s;} // HB3
  if (ai== 611)              {s="H1"; return s;} // HG2
  if (ai== 612)              {s="H1"; return s;} // HG3
  if (ai== 613)              {s="H1"; return s;} // HE1
  if (ai== 614)              {s="H1"; return s;} // HE2
  if (ai== 615)              {s="H1"; return s;} // HE3

  // pro
  if (ai== 705)              {s="CT"; return s;} // CB
  if (ai== 706)              {s="CT"; return s;} // CG
  if (ai== 707)              {s="CT"; return s;} // CD
  if (ai== 708)              {s="HC"; return s;} // HB2
  if (ai== 709)              {s="HC"; return s;} // HB3
  if (ai== 710)              {s="HC"; return s;} // HG2
  if (ai== 711)              {s="HC"; return s;} // HG3
  if (ai== 712 && tm==1)     {s="HP"; return s;} // HD2
  if (ai== 712)              {s="H1"; return s;} // HD2
  if (ai== 713 && tm==1)     {s="HP"; return s;} // HD3
  if (ai== 713)              {s="H1"; return s;} // HD3

  // phe
  if (ai== 805)              {s="CT"; return s;} // CB
  if (ai== 806)              {s="CA"; return s;} // CG
  if (ai== 807)              {s="CA"; return s;} // CD1
  if (ai== 808)              {s="CA"; return s;} // CD2
  if (ai== 809)              {s="CA"; return s;} // CE1
  if (ai== 810)              {s="CA"; return s;} // CE2
  if (ai== 811)              {s="CA"; return s;} // CZ
  if (ai== 812)              {s="HC"; return s;} // HB2
  if (ai== 813)              {s="HC"; return s;} // HB3
  if (ai== 814)              {s="HA"; return s;} // HD1
  if (ai== 815)              {s="HA"; return s;} // HD2
  if (ai== 816)              {s="HA"; return s;} // HE1
  if (ai== 817)              {s="HA"; return s;} // HE2
  if (ai== 818)              {s="HA"; return s;} // HZ

  // trp
  if (ai== 905)              {s="CT"; return s;} // CB
  if (ai== 906)              {s="C*"; return s;} // CG
  if (ai== 907)              {s="CW"; return s;} // CD1
  if (ai== 908)              {s="CB"; return s;} // CD2
  if (ai== 909)              {s="NA"; return s;} // NE1
  if (ai== 910)              {s="CN"; return s;} // CE2
  if (ai== 911)              {s="CA"; return s;} // CE3
  if (ai== 912)              {s="CA"; return s;} // CZ3
  if (ai== 913)              {s="CA"; return s;} // CH2
  if (ai== 914)              {s="CA"; return s;} // CZ2
  if (ai== 915)              {s="HC"; return s;} // HB2
  if (ai== 916)              {s="HC"; return s;} // HB3
  if (ai== 917)              {s="H4"; return s;} // HD1
  if (ai== 918)              {s="H" ; return s;} // HE1
  if (ai== 919)              {s="HA"; return s;} // HE3
  if (ai== 920)              {s="HA"; return s;} // HZ3
  if (ai== 921)              {s="HA"; return s;} // HH2
  if (ai== 922)              {s="HA"; return s;} // HZ2

  // ser
  if (ai==1005)              {s="CT"; return s;} // CB
  if (ai==1006)              {s="OH"; return s;} // OG
  if (ai==1007)              {s="H1"; return s;} // HB2
  if (ai==1008)              {s="H1"; return s;} // HB3
  if (ai==1009)              {s="HO"; return s;} // HG

  // thr
  if (ai==1105)              {s="CT"; return s;} // CB
  if (ai==1106)              {s="CT"; return s;} // CG2
  if (ai==1107)              {s="OH"; return s;} // OG1
  if (ai==1108)              {s="H1"; return s;} // HB
  if (ai==1109)              {s="HC"; return s;} // HG21
  if (ai==1110)              {s="HC"; return s;} // HG22
  if (ai==1111)              {s="HC"; return s;} // HG23
  if (ai==1112)              {s="HO"; return s;} // HG1

  // asn
  if (ai==1205)              {s="CT"; return s;} // CB
  if (ai==1206)              {s="C" ; return s;} // CG
  if (ai==1207)              {s="N" ; return s;} // ND2
  if (ai==1208)              {s="O" ; return s;} // OD1
  if (ai==1209)              {s="HC"; return s;} // HB2
  if (ai==1210)              {s="HC"; return s;} // HB3
  if (ai==1211)              {s="H" ; return s;} // HD21
  if (ai==1212)              {s="H" ; return s;} // HD22

  // gln
  if (ai==1305)              {s="CT"; return s;} // CB
  if (ai==1306)              {s="CT"; return s;} // CG
  if (ai==1307)              {s="C" ; return s;} // CD
  if (ai==1308)              {s="N" ; return s;} // NE2
  if (ai==1309)              {s="O" ; return s;} // OE1
  if (ai==1310)              {s="HC"; return s;} // HB2
  if (ai==1311)              {s="HC"; return s;} // HB3
  if (ai==1312)              {s="HC"; return s;} // HG2
  if (ai==1313)              {s="HC"; return s;} // HG3
  if (ai==1314)              {s="H" ; return s;} // HE21
  if (ai==1315)              {s="H" ; return s;} // HE22

  // tyr
  if (ai==1405)              {s="CT"; return s;} // CB
  if (ai==1406)              {s="CA"; return s;} // CG
  if (ai==1407)              {s="CA"; return s;} // CD1
  if (ai==1408)              {s="CA"; return s;} // CD2
  if (ai==1409)              {s="CA"; return s;} // CE1
  if (ai==1410)              {s="CA"; return s;} // CE2
  if (ai==1411)              {s="C" ; return s;} // CZ
  if (ai==1412)              {s="OH"; return s;} // OH
  if (ai==1413)              {s="HC"; return s;} // HB2
  if (ai==1414)              {s="HC"; return s;} // HB3
  if (ai==1415)              {s="HA"; return s;} // HD1
  if (ai==1416)              {s="HA"; return s;} // HD2
  if (ai==1417)              {s="HA"; return s;} // HE1
  if (ai==1418)              {s="HA"; return s;} // HE2
  if (ai==1419)              {s="HO"; return s;} // HH

  // cys
  if (ai==1505)              {s="CT"; return s;} // CB
  if (ai==1506 && info_1==0) {s="SH"; return s;} // SG (CYS)
  if (ai==1506 && info_1==1) {s="S" ; return s;} // SG (CYX)
  if (ai==1507)              {s="H1"; return s;} // HB2
  if (ai==1508)              {s="H1"; return s;} // HB3
  if (ai==1509)              {s="HS"; return s;} // HG

  // lys
  if (ai==1605)              {s="CT"; return s;} // CB
  if (ai==1606)              {s="CT"; return s;} // CG
  if (ai==1607)              {s="CT"; return s;} // CD
  if (ai==1608)              {s="CT"; return s;} // CE
  if (ai==1609)              {s="N3"; return s;} // NZ
  if (ai==1610)              {s="HC"; return s;} // HB2
  if (ai==1611)              {s="HC"; return s;} // HB3
  if (ai==1612)              {s="HC"; return s;} // HG2
  if (ai==1613)              {s="HC"; return s;} // HG3
  if (ai==1614)              {s="HC"; return s;} // HD2
  if (ai==1615)              {s="HC"; return s;} // HD3
  if (ai==1616)              {s="HP"; return s;} // HE2
  if (ai==1617)              {s="HP"; return s;} // HE3
  if (ai==1618)              {s="H" ; return s;} // HZ1
  if (ai==1619)              {s="H" ; return s;} // HZ2
  if (ai==1620)              {s="H" ; return s;} // HZ3

  // arg
  if (ai==1705)              {s="CT"; return s;} // CB
  if (ai==1706)              {s="CT"; return s;} // CG
  if (ai==1707)              {s="CT"; return s;} // CD
  if (ai==1708)              {s="N2"; return s;} // NE
  if (ai==1709)              {s="CA"; return s;} // CZ
  if (ai==1710)              {s="N2"; return s;} // NH1
  if (ai==1711)              {s="N2"; return s;} // NH2
  if (ai==1712)              {s="HC"; return s;} // HB2
  if (ai==1713)              {s="HC"; return s;} // HB3
  if (ai==1714)              {s="HC"; return s;} // HG2
  if (ai==1715)              {s="HC"; return s;} // HG3
  if (ai==1716)              {s="H1"; return s;} // HD2
  if (ai==1717)              {s="H1"; return s;} // HD3
  if (ai==1718)              {s="H" ; return s;} // HE
  if (ai==1719)              {s="H" ; return s;} // HH11
  if (ai==1720)              {s="H" ; return s;} // HH12
  if (ai==1721)              {s="H" ; return s;} // HH21
  if (ai==1722)              {s="H" ; return s;} // HH22

  // asp
  if (ai==1905)              {s="CT"; return s;} // CB
  if (ai==1906)              {s="C" ; return s;} // CG
  if (ai==1907)              {s="O2"; return s;} // OD1
  if (ai==1908)              {s="O2"; return s;} // OD2
  if (ai==1909)              {s="HC"; return s;} // HB2
  if (ai==1910)              {s="HC"; return s;} // HB3
  //if (ai==1911)              {}
  //if (ai==1912)              {}

  // glu
  if (ai==2005)              {s="CT"; return s;} // CB
  if (ai==2006)              {s="CT"; return s;} // CG
  if (ai==2007)              {s="C" ; return s;} // CD
  if (ai==2008)              {s="O2"; return s;} // OE1
  if (ai==2009)              {s="O2"; return s;} // OE2
  if (ai==2010)              {s="HC"; return s;} // HB2
  if (ai==2011)              {s="HC"; return s;} // HB3
  if (ai==2012)              {s="HC"; return s;} // HG2
  if (ai==2013)              {s="HC"; return s;} // HG3
  //if (ai==2014)              {}
  //if (ai==2015)              {}
  
  // his (hid,hie,hip)
  if (ai==1805)              {s="CT"; return s;} // CB
  if (ai==1806)              {s="CC"; return s;} // CG
  if (ai==1807 && info_1==0) {s="CV"; return s;} // CD2
  if (ai==1807 && info_1==1) {s="CW"; return s;} // CD2
  if (ai==1807 && info_1==2) {s="CW"; return s;} // CD2
  if (ai==1808 && info_1==0) {s="NA"; return s;} // ND1
  if (ai==1808 && info_1==1) {s="NB"; return s;} // ND1
  if (ai==1808 && info_1==2) {s="NA"; return s;} // ND1
  if (ai==1809 && info_1==0) {s="NB"; return s;} // NE2
  if (ai==1809 && info_1==1) {s="NA"; return s;} // NE2
  if (ai==1809 && info_1==2) {s="NA"; return s;} // NE2
  if (ai==1810)              {s="CR"; return s;} // CE1
  if (ai==1811)              {s="HC"; return s;} // HB2
  if (ai==1812)              {s="HC"; return s;} // HB3
  if (ai==1813)              {s="H4"; return s;} // HD2
  if (ai==1814)              {s="H" ; return s;} // HD1
  if (ai==1815)              {s="H5"; return s;} // HE1
  if (ai==1816)              {s="H" ; return s;} // HE2

  // WATER
  if (ai==6501)              {s="OW"; return s;} // 
  if (ai==6502)              {s="HW"; return s;} //
  if (ai==6503)              {s="HW"; return s;} //

  cout << "amber_type , ai=" << ai << " , tm=" << tm << " , info_1=" << info_1 << " is not defined!" << endl ; 
  s="**";
  return s;
}

int ForceField::amber_force_field_bond(const string   a1,
				       const string   a2,
				       double       * coeff,
				       double       * eq)
{
  // (parm99.dat)
  if((a1=="C"  && a2=="O" ) || (a2=="C"  && a1=="O" )){*coeff=5.700E+02;*eq=1.229E+00;return 1;}
  if((a1=="C"  && a2=="N" ) || (a2=="C"  && a1=="N" )){*coeff=4.900E+02;*eq=1.335E+00;return 1;}
  if((a1=="CT" && a2=="HP") || (a2=="CT" && a1=="HP")){*coeff=3.400E+02;*eq=1.090E+00;return 1;}
  if((a1=="CT" && a2=="C" ) || (a2=="CT" && a1=="C" )){*coeff=3.170E+02;*eq=1.522E+00;return 1;}
  if((a1=="N3" && a2=="H" ) || (a2=="N3" && a1=="H" )){*coeff=4.340E+02;*eq=1.010E+00;return 1;}
  if((a1=="N3" && a2=="CT") || (a2=="N3" && a1=="CT")){*coeff=3.670E+02;*eq=1.471E+00;return 1;}
  if((a1=="C"  && a2=="O2") || (a2=="C"  && a1=="O2")){*coeff=6.560E+02;*eq=1.250E+00;return 1;}
  if((a1=="CT" && a2=="H1") || (a2=="CT" && a1=="H1")){*coeff=3.400E+02;*eq=1.090E+00;return 1;}
  if((a1=="N"  && a2=="H" ) || (a2=="N"  && a1=="H" )){*coeff=4.340E+02;*eq=1.010E+00;return 1;}
  if((a1=="N"  && a2=="CT") || (a2=="N"  && a1=="CT")){*coeff=3.370E+02;*eq=1.449E+00;return 1;}
  if((a1=="CT" && a2=="CT") || (a2=="CT" && a1=="CT")){*coeff=310.0    ;*eq=1.526    ;return 1;}
  if((a1=="CT" && a2=="HC") || (a2=="CT" && a1=="HC")){*coeff=340.0    ;*eq=1.090    ;return 1;}
  if((a1=="CT" && a2=="S" ) || (a2=="CT" && a1=="S" )){*coeff=227.0    ;*eq=1.810    ;return 1;}
  if((a1=="CA" && a2=="CT") || (a2=="CA" && a1=="CT")){*coeff=317.0    ;*eq=1.510    ;return 1;}
  if((a1=="CA" && a2=="CA") || (a2=="CA" && a1=="CA")){*coeff=469.0    ;*eq=1.400    ;return 1;}
  if((a1=="CA" && a2=="HA") || (a2=="CA" && a1=="HA")){*coeff=367.0    ;*eq=1.080    ;return 1;}
  if((a1=="C*" && a2=="CB") || (a2=="C*" && a1=="CB")){*coeff=388.0    ;*eq=1.459    ;return 1;}
  if((a1=="C*" && a2=="CT") || (a2=="C*" && a1=="CT")){*coeff=317.0    ;*eq=1.495    ;return 1;}
  if((a1=="C*" && a2=="CW") || (a2=="C*" && a1=="CW")){*coeff=546.0    ;*eq=1.352    ;return 1;}
  if((a1=="CB" && a2=="CN") || (a2=="CB" && a1=="CN")){*coeff=447.0    ;*eq=1.419    ;return 1;}
  if((a1=="CW" && a2=="H4") || (a2=="CW" && a1=="H4")){*coeff=367.0    ;*eq=1.080    ;return 1;}
  if((a1=="CW" && a2=="NA") || (a2=="CW" && a1=="NA")){*coeff=427.0    ;*eq=1.381    ;return 1;}
  if((a1=="H"  && a2=="NA") || (a2=="H"  && a1=="NA")){*coeff=434.0    ;*eq=1.010    ;return 1;}
  if((a1=="CN" && a2=="NA") || (a2=="CN" && a1=="NA")){*coeff=428.0    ;*eq=1.380    ;return 1;}
  if((a1=="CA" && a2=="CN") || (a2=="CA" && a1=="CN")){*coeff=469.0    ;*eq=1.400    ;return 1;}
  if((a1=="CA" && a2=="CB") || (a2=="CA" && a1=="CB")){*coeff=469.0    ;*eq=1.404    ;return 1;}
  if((a1=="CT" && a2=="OH") || (a2=="CT" && a1=="OH")){*coeff=320.0    ;*eq=1.410    ;return 1;}
  if((a1=="HO" && a2=="OH") || (a2=="HO" && a1=="OH")){*coeff=553.0    ;*eq=0.960    ;return 1;}
  if((a1=="C"  && a2=="CA") || (a2=="C"  && a1=="CA")){*coeff=469.0    ;*eq=1.409    ;return 1;}
  if((a1=="C"  && a2=="OH") || (a2=="C"  && a1=="OH")){*coeff=450.0    ;*eq=1.364    ;return 1;}
  if((a1=="CT" && a2=="SH") || (a2=="CT" && a1=="SH")){*coeff=237.0    ;*eq=1.810    ;return 1;}
  if((a1=="HS" && a2=="SH") || (a2=="HS" && a1=="SH")){*coeff=274.0    ;*eq=1.336    ;return 1;}
  if((a1=="CT" && a2=="N2") || (a2=="CT" && a1=="N2")){*coeff=337.0    ;*eq=1.463    ;return 1;}
  if((a1=="H"  && a2=="N2") || (a2=="H"  && a1=="N2")){*coeff=434.0    ;*eq=1.010    ;return 1;}
  if((a1=="CA" && a2=="N2") || (a2=="CA" && a1=="N2")){*coeff=481.0    ;*eq=1.340    ;return 1;}
  if((a1=="CC" && a2=="CT") || (a2=="CC" && a1=="CT")){*coeff=317.0    ;*eq=1.504    ;return 1;}
  if((a1=="CC" && a2=="NA") || (a2=="CC" && a1=="NA")){*coeff=422.0    ;*eq=1.385    ;return 1;}
  if((a1=="CC" && a2=="CV") || (a2=="CC" && a1=="CV")){*coeff=512.0    ;*eq=1.375    ;return 1;}
  if((a1=="CR" && a2=="NA") || (a2=="CR" && a1=="NA")){*coeff=477.0    ;*eq=1.343    ;return 1;}
  if((a1=="CR" && a2=="H5") || (a2=="CR" && a1=="H5")){*coeff=367.0    ;*eq=1.080    ;return 1;}
  if((a1=="CR" && a2=="NB") || (a2=="CR" && a1=="NB")){*coeff=488.0    ;*eq=1.335    ;return 1;}
  if((a1=="CV" && a2=="NB") || (a2=="CV" && a1=="NB")){*coeff=410.0    ;*eq=1.394    ;return 1;}
  if((a1=="CV" && a2=="H4") || (a2=="CV" && a1=="H4")){*coeff=367.0    ;*eq=1.080    ;return 1;}
  if((a1=="CC" && a2=="NB") || (a2=="CC" && a1=="NB")){*coeff=410.0    ;*eq=1.394    ;return 1;}
  if((a1=="CC" && a2=="CW") || (a2=="CC" && a1=="CW")){*coeff=518.0    ;*eq=1.371    ;return 1;}
  if((a1=="S"  && a2=="S" ) || (a2=="S"  && a1=="S" )){*coeff=166.0    ;*eq=2.038    ;return 1;}
  if((a1=="OW" && a2=="HW") || (a2=="OW" && a1=="HW")){*coeff=553.0    ;*eq=0.9572   ;return 1;} // TIP3P water
  if((a1=="HW" && a2=="HW") || (a2=="HW" && a1=="HW")){*coeff=553.0    ;*eq=1.5136   ;return 1;} // TIP3P water

  cout << "amber_force_field_bond , " << a1 << " - " << a2 << " is not defined!" << endl ;

  return -1;
}

int ForceField::amber_force_field_angle(const string a1,
					const string a2,
					const string a3,
					double * coeff,
					double * eq)
{
  // (parm99.dat)
  if(a2=="C"  && ((a1=="N"  && a3=="O" ) || (a3=="N"  && a1=="O" ))){*coeff= 80.0;*eq=122.90;return 1;} // N  C  O     80.0   122.90
  if(a2=="N"  && ((a1=="C"  && a3=="H" ) || (a3=="C"  && a1=="H" ))){*coeff= 50.0;*eq=120.00;return 1;} // C  N  H     50.0   120.00
  if(a2=="N"  && ((a1=="C"  && a3=="CT") || (a3=="C"  && a1=="CT"))){*coeff= 50.0;*eq=121.90;return 1;} // C  N  CT    50.0   121.90
  if(a2=="CT" && ((a1=="C"  && a3=="HP") || (a3=="C"  && a1=="HP"))){*coeff= 50.0;*eq=109.50;return 1;} // C  CT HP    50.0   109.50
  if(a2=="CT" && ((a1=="N3" && a3=="HP") || (a3=="N3" && a1=="HP"))){*coeff= 50.0;*eq=109.50;return 1;} // N3 CT HP    50.0   109.50
  if(a2=="CT" && ((a1=="HP" && a3=="HP") || (a3=="HP" && a1=="HP"))){*coeff= 35.0;*eq=109.50;return 1;} // HP CT HP    35.0   109.50
  if(a2=="C"  && ((a1=="CT" && a3=="O" ) || (a3=="CT" && a1=="O" ))){*coeff= 80.0;*eq=120.40;return 1;} // CT C  O     80.0   120.40
  if(a2=="C"  && ((a1=="CT" && a3=="N" ) || (a3=="CT" && a1=="N" ))){*coeff= 70.0;*eq=116.60;return 1;} // CT C  N     70.0   116.60
  if(a2=="N3" && ((a1=="CT" && a3=="CT") || (a3=="CT" && a1=="CT"))){*coeff= 50.0;*eq=109.50;return 1;} // CT N3 CT    50.0   109.50
  if(a2=="N3" && ((a1=="CT" && a3=="H" ) || (a3=="CT" && a1=="H" ))){*coeff= 50.0;*eq=109.50;return 1;} // CT N3 H     50.0   109.50
  if(a2=="N3" && ((a1=="H"  && a3=="H" ) || (a3=="H"  && a1=="H" ))){*coeff= 35.0;*eq=109.50;return 1;} // H  N3 H     35.0   109.50
  if(a2=="CT" && ((a1=="N3" && a3=="C" ) || (a3=="N3" && a1=="C" ))){*coeff= 80.0;*eq=111.20;return 1;} // N3 CT C     80.0   111.20
  if(a2=="C"  && ((a1=="O2" && a3=="O2") || (a3=="O2" && a1=="O2"))){*coeff= 80.0;*eq=126.00;return 1;} // O2 C  O2    80.0   126.00
  if(a2=="CT" && ((a1=="H1" && a3=="C" ) || (a3=="H1" && a1=="C" ))){*coeff= 50.0;*eq=109.50;return 1;} // H1 CT C     50.0   109.50
  if(a2=="CT" && ((a1=="H1" && a3=="H1") || (a3=="H1" && a1=="H1"))){*coeff= 35.0;*eq=109.50;return 1;} // H1 CT H1    35.0   109.50
  if(a2=="C"  && ((a1=="CT" && a3=="O2") || (a3=="CT" && a1=="O2"))){*coeff= 70.0;*eq=117.00;return 1;} // CT C  O2    70.0   117.00
  if(a2=="N"  && ((a1=="H"  && a3=="CT") || (a3=="H"  && a1=="CT"))){*coeff= 50.0;*eq=118.04;return 1;} // H  N  CT    50.0   118.04
  if(a2=="CT" && ((a1=="N"  && a3=="H1") || (a3=="N"  && a1=="H1"))){*coeff= 50.0;*eq=109.50;return 1;} // N  CT H1    50.0   109.50
  if(a2=="CT" && ((a1=="N"  && a3=="C" ) || (a3=="N"  && a1=="C" ))){*coeff= 63.0;*eq=110.10;return 1;} // N  CT C     63.0   110.10
  if(a2=="CT" && ((a1=="CT" && a3=="N3") || (a3=="CT" && a1=="N3"))){*coeff= 80.0;*eq=111.20;return 1;} // CT CT N3    80.0   111.20
  if(a2=="CT" && ((a1=="CT" && a3=="HP") || (a3=="CT" && a1=="HP"))){*coeff= 50.0;*eq=109.50;return 1;} // CT CT HP    50.0   109.50
  if(a2=="CT" && ((a1=="C"  && a3=="CT") || (a3=="C"  && a1=="CT"))){*coeff= 63.0;*eq=111.10;return 1;} // C  CT CT    63.0   111.10
  if(a2=="CT" && ((a1=="CT" && a3=="HC") || (a3=="CT" && a1=="HC"))){*coeff= 50.0;*eq=109.50;return 1;} // CT-CT-HC    50.0   109.50
  if(a2=="CT" && ((a1=="HC" && a3=="HC") || (a3=="HC" && a1=="HC"))){*coeff= 35.0;*eq=109.50;return 1;} // HC-CT-HC    35.0   109.50
  if(a2=="CT" && ((a1=="CT" && a3=="N" ) || (a3=="CT" && a1=="N" ))){*coeff= 80.0;*eq=109.70;return 1;} // CT-CT-N     80.0   109.70
  if(a2=="CT" && ((a1=="CT" && a3=="H1") || (a3=="CT" && a1=="H1"))){*coeff= 50.0;*eq=109.50;return 1;} // CT-CT-H1    50.0   109.50
  if(a2=="CT" && ((a1=="CT" && a3=="CT") || (a3=="CT" && a1=="CT"))){*coeff= 40.0;*eq=109.50;return 1;} // CT-CT-CT    40.0   109.50
  if(a2=="CT" && ((a1=="CT" && a3=="S" ) || (a3=="CT" && a1=="S" ))){*coeff= 50.0;*eq=114.70;return 1;} // CT-CT-S     50.0   114.70
  if(a2=="CT" && ((a1=="H1" && a3=="S" ) || (a3=="H1" && a1=="S" ))){*coeff= 50.0;*eq=109.50;return 1;} // H1-CT-S     50.0   109.50
  if(a2=="S"  && ((a1=="CT" && a3=="CT") || (a3=="CT" && a1=="CT"))){*coeff= 62.0;*eq= 98.90;return 1;} // CT-S -CT    62.0    98.90
  if(a2=="N"  && ((a1=="CT" && a3=="CT") || (a3=="CT" && a1=="CT"))){*coeff= 50.0;*eq=118.00;return 1;} // CT-N -CT    50.0   118.00
  if(a2=="CT" && ((a1=="CT" && a3=="CA") || (a3=="CT" && a1=="CA"))){*coeff= 63.0;*eq=114.00;return 1;} // CT-CT-CA    63.0   114.00
  if(a2=="CT" && ((a1=="CA" && a3=="HC") || (a3=="CA" && a1=="HC"))){*coeff= 50.0;*eq=109.50;return 1;} // CA-CT-HC    50.0   109.50
  if(a2=="CA" && ((a1=="CA" && a3=="CT") || (a3=="CA" && a1=="CT"))){*coeff= 70.0;*eq=120.00;return 1;} // CA-CA-CT    70.0   120.00
  if(a2=="CA" && ((a1=="CA" && a3=="CA") || (a3=="CA" && a1=="CA"))){*coeff= 63.0;*eq=120.00;return 1;} // CA-CA-CA    63.0   120.00
  if(a2=="CA" && ((a1=="CA" && a3=="HA") || (a3=="CA" && a1=="HA"))){*coeff= 50.0;*eq=120.00;return 1;} // CA-CA-HA    50.0   120.00
  if(a2=="CT" && ((a1=="C*" && a3=="CT") || (a3=="C*" && a1=="CT"))){*coeff= 63.0;*eq=115.60;return 1;} // C*-CT-CT    63.0   115.60
  if(a2=="CT" && ((a1=="C*" && a3=="HC") || (a3=="C*" && a1=="HC"))){*coeff= 50.0;*eq=109.50;return 1;} // C*-CT-HC    50.0   109.50
  if(a2=="C*" && ((a1=="CT" && a3=="CW") || (a3=="CT" && a1=="CW"))){*coeff= 70.0;*eq=125.00;return 1;} // CT-C*-CW    70.0   125.00
  if(a2=="C*" && ((a1=="CB" && a3=="CT") || (a3=="CB" && a1=="CT"))){*coeff= 70.0;*eq=128.60;return 1;} // CB-C*-CT    70.0   128.60
  if(a2=="C*" && ((a1=="CB" && a3=="CW") || (a3=="CB" && a1=="CW"))){*coeff= 63.0;*eq=106.40;return 1;} // CB-C*-CW    63.0   106.40
  if(a2=="CW" && ((a1=="C*" && a3=="H4") || (a3=="C*" && a1=="H4"))){*coeff= 50.0;*eq=120.00;return 1;} // C*-CW-H4    50.0   120.00
  if(a2=="CW" && ((a1=="C*" && a3=="NA") || (a3=="C*" && a1=="NA"))){*coeff= 70.0;*eq=108.70;return 1;} // C*-CW-NA    70.0   108.70
  if(a2=="CB" && ((a1=="C*" && a3=="CA") || (a3=="C*" && a1=="CA"))){*coeff= 63.0;*eq=134.90;return 1;} // C*-CB-CA    63.0   134.90
  if(a2=="CB" && ((a1=="C*" && a3=="CN") || (a3=="C*" && a1=="CN"))){*coeff= 63.0;*eq=108.80;return 1;} // C*-CB-CN    63.0   108.80
  if(a2=="CW" && ((a1=="H4" && a3=="NA") || (a3=="H4" && a1=="NA"))){*coeff= 50.0;*eq=120.00;return 1;} // H4-CW-NA    50.0   120.00
  if(a2=="NA" && ((a1=="CW" && a3=="H" ) || (a3=="CW" && a1=="H" ))){*coeff= 50.0;*eq=120.00;return 1;} // CW-NA-H     50.0   120.00
  if(a2=="NA" && ((a1=="CN" && a3=="CW") || (a3=="CN" && a1=="CW"))){*coeff= 70.0;*eq=111.60;return 1;} // CN-NA-CW    70.0   111.60
  if(a2=="CN" && ((a1=="CN" && a3=="H" ) || (a3=="CN" && a1=="H" ))){*coeff= 50.0;*eq=123.10;return 1;} // CN-NA-H     50.0   123.10
  if(a2=="CN" && ((a1=="CA" && a3=="NA") || (a3=="CA" && a1=="NA"))){*coeff= 70.0;*eq=132.80;return 1;} // CA-CN-NA    70.0   132.80
  if(a2=="CN" && ((a1=="CB" && a3=="NA") || (a3=="CB" && a1=="NA"))){*coeff= 70.0;*eq=104.40;return 1;} // CB-CN-NA    70.0   104.40
  if(a2=="CN" && ((a1=="CA" && a3=="CB") || (a3=="CA" && a1=="CB"))){*coeff= 63.0;*eq=122.70;return 1;} // CA-CN-CB    63.0   122.70
  if(a2=="CA" && ((a1=="CA" && a3=="CB") || (a3=="CA" && a1=="CB"))){*coeff= 63.0;*eq=120.00;return 1;} // CA-CA-CB    63.0   120.00
  if(a2=="CA" && ((a1=="CN" && a3=="HA") || (a3=="CN" && a1=="HA"))){*coeff= 50.0;*eq=120.00;return 1;} // CN-CA-HA    50.0   120.00
  if(a2=="CA" && ((a1=="CA" && a3=="CN") || (a3=="CA" && a1=="CN"))){*coeff= 63.0;*eq=120.00;return 1;} // CA-CA-CN    63.0   120.00
  if(a2=="CB" && ((a1=="CA" && a3=="CN") || (a3=="CA" && a1=="CN"))){*coeff= 63.0;*eq=116.20;return 1;} // CA-CB-CN    63.0   116.20
  if(a2=="CA" && ((a1=="CB" && a3=="HA") || (a3=="CB" && a1=="HA"))){*coeff= 50.0;*eq=120.00;return 1;} // CB-CA-HA    50.0   120.00
  if(a2=="NA" && ((a1=="CN" && a3=="H" ) || (a3=="CN" && a1=="H" ))){*coeff= 50.0;*eq=123.10;return 1;} // CN-NA-H     50.0   123.10
  if(a2=="CT" && ((a1=="CT" && a3=="OH") || (a3=="CT" && a1=="OH"))){*coeff= 50.0;*eq=109.50;return 1;} // CT-CT-OH    50.0   109.50
  if(a2=="CT" && ((a1=="H1" && a3=="OH") || (a3=="H1" && a1=="OH"))){*coeff= 50.0;*eq=109.50;return 1;} // H1-CT-OH    50.0   109.50
  if(a2=="OH" && ((a1=="CT" && a3=="HO") || (a3=="CT" && a1=="HO"))){*coeff= 55.0;*eq=108.50;return 1;} // CT-OH-HO    55.0   108.50
  if(a2=="CT" && ((a1=="C"  && a3=="HC") || (a3=="C"  && a1=="HC"))){*coeff= 50.0;*eq=109.50;return 1;} // C -CT-HC    50.0   109.50
  if(a2=="N"  && ((a1=="H"  && a3=="H" ) || (a3=="H"  && a1=="H" ))){*coeff= 35.0;*eq=120.00;return 1;} // H -N -H     35.0   120.00
  if(a2=="CA" && ((a1=="C"  && a3=="CA") || (a3=="C"  && a1=="CA"))){*coeff= 63.0;*eq=120.00;return 1;} // C -CA-CA    63.0   120.00
  if(a2=="CA" && ((a1=="C"  && a3=="HA") || (a3=="C"  && a1=="HA"))){*coeff= 50.0;*eq=120.00;return 1;} // C -CA-HA    50.0   120.00
  if(a2=="C"  && ((a1=="CA" && a3=="OH") || (a3=="CA" && a1=="OH"))){*coeff= 70.0;*eq=120.00;return 1;} // CA-C -OH    70.0   120.00
  if(a2=="C"  && ((a1=="CA" && a3=="CA") || (a3=="CA" && a1=="CA"))){*coeff= 63.0;*eq=120.00;return 1;} // CA-C -CA    63.0   120.00    
  if(a2=="OH" && ((a1=="C"  && a3=="HO") || (a3=="C"  && a1=="HO"))){*coeff= 50.0;*eq=113.00;return 1;} // C -OH-HO    50.0   113.00
  if(a2=="CT" && ((a1=="CT" && a3=="SH") || (a3=="CT" && a1=="SH"))){*coeff= 50.0;*eq=108.60;return 1;} // CT-CT-SH    50.0   108.60
  if(a2=="CT" && ((a1=="H1" && a3=="SH") || (a3=="H1" && a1=="SH"))){*coeff= 50.0;*eq=109.50;return 1;} // H1-CT-SH    50.0   109.50    
  if(a2=="SH" && ((a1=="CT" && a3=="HS") || (a3=="CT" && a1=="HS"))){*coeff= 43.0;*eq= 96.00;return 1;} // CT-SH-HS    43.0    96.00   
  if(a2=="CT" && ((a1=="CT" && a3=="N2") || (a3=="CT" && a1=="N2"))){*coeff= 80.0;*eq=111.20;return 1;} // CT-CT-N2    80.0   111.20
  if(a2=="CT" && ((a1=="H1" && a3=="N2") || (a3=="H1" && a1=="N2"))){*coeff= 50.0;*eq=109.50;return 1;} // H1-CT-N2    50.0   109.50
  if(a2=="N2" && ((a1=="CT" && a3=="H" ) || (a3=="CT" && a1=="H" ))){*coeff= 50.0;*eq=118.40;return 1;} // CT-N2-H     50.0   118.40
  if(a2=="N2" && ((a1=="CA" && a3=="CT") || (a3=="CA" && a1=="CT"))){*coeff= 50.0;*eq=123.20;return 1;} // CA-N2-CT    50.0   123.20
  if(a2=="N2" && ((a1=="CA" && a3=="H" ) || (a3=="CA" && a1=="H" ))){*coeff= 50.0;*eq=120.00;return 1;} // CA-N2-H     50.0   120.00
  if(a2=="CA" && ((a1=="N2" && a3=="N2") || (a3=="N2" && a1=="N2"))){*coeff= 70.0;*eq=120.00;return 1;} // N2-CA-N2    70.0   120.00
  if(a2=="N2" && ((a1=="H"  && a3=="H" ) || (a3=="H"  && a3=="H" ))){*coeff= 35.0;*eq=120.00;return 1;} // H -N2-H     35.0   120.00
  if(a2=="CT" && ((a1=="CC" && a3=="CT") || (a3=="CC" && a1=="CT"))){*coeff= 63.0;*eq=113.10;return 1;} // CC-CT-CT    63.0   113.10
  if(a2=="CT" && ((a1=="CC" && a3=="HC") || (a3=="CC" && a1=="HC"))){*coeff= 50.0;*eq=109.50;return 1;} // CC-CT-HC    50.0   109.50
  if(a2=="CC" && ((a1=="CT" && a3=="NA") || (a3=="CT" && a1=="NA"))){*coeff= 70.0;*eq=120.00;return 1;} // CT-CC-NA    70.0   120.00
  if(a2=="CC" && ((a1=="CT" && a3=="CV") || (a3=="CT" && a1=="CV"))){*coeff= 70.0;*eq=120.00;return 1;} // CT-CC-CV    70.0   120.00
  if(a2=="CC" && ((a1=="CV" && a3=="NA") || (a3=="CV" && a1=="NA"))){*coeff= 70.0;*eq=120.00;return 1;} // CV-CC-NA    70.0   120.00
  if(a2=="NA" && ((a1=="CC" && a3=="H" ) || (a3=="CC" && a1=="H" ))){*coeff= 50.0;*eq=120.00;return 1;} // CC-NA-H     50.0   120.00
  if(a2=="NA" && ((a1=="CC" && a3=="CR") || (a3=="CC" && a1=="CR"))){*coeff= 70.0;*eq=120.00;return 1;} // CC-NA-CR    70.0   120.00
  if(a2=="CV" && ((a1=="CC" && a3=="NB") || (a3=="CC" && a1=="NB"))){*coeff= 70.0;*eq=120.00;return 1;} // CC-CV-NB    70.0   120.00
  if(a2=="CV" && ((a1=="CC" && a3=="H4") || (a3=="CC" && a1=="H4"))){*coeff= 50.0;*eq=120.00;return 1;} // CC-CV-H4    50.0   120.00
  if(a2=="NA" && ((a1=="CR" && a3=="H" ) || (a3=="CR" && a1=="H" ))){*coeff= 50.0;*eq=120.00;return 1;} // CR-NA-H     50.0   120.00
  if(a2=="CR" && ((a1=="H5" && a3=="NA") || (a3=="H5" && a1=="NA"))){*coeff= 50.0;*eq=120.00;return 1;} // H5-CR-NA    50.0   120.00
  if(a2=="CR" && ((a1=="NA" && a3=="NB") || (a3=="NA" && a1=="NB"))){*coeff= 70.0;*eq=120.00;return 1;} // NA-CR-NB    70.0   120.00
  if(a2=="CR" && ((a1=="H5" && a3=="NB") || (a3=="H5" && a1=="NB"))){*coeff= 50.0;*eq=120.00;return 1;} // H5-CR-NB    50.0   120.00
  if(a2=="NB" && ((a1=="CR" && a3=="CV") || (a3=="CR" && a1=="CV"))){*coeff= 70.0;*eq=117.00;return 1;} // CR-NB-CV    70.0   117.00
  if(a2=="CV" && ((a1=="H4" && a3=="NB") || (a3=="H4" && a1=="NB"))){*coeff= 50.0;*eq=120.00;return 1;} // H4-CV-NB    50.0   120.00
  if(a2=="CC" && ((a1=="CT" && a3=="NB") || (a3=="CT" && a1=="NB"))){*coeff= 70.0;*eq=120.00;return 1;} // CT-CC-NB    70.0   120.00
  if(a2=="CC" && ((a1=="CT" && a3=="CW") || (a3=="CT" && a1=="CW"))){*coeff= 70.0;*eq=120.00;return 1;} // CT-CC-CW    70.0   120.00
  if(a2=="CC" && ((a1=="CW" && a3=="NB") || (a3=="CW" && a1=="NB"))){*coeff= 70.0;*eq=120.00;return 1;} // CW-CC-NB    70.0   120.00
  if(a2=="NB" && ((a1=="CC" && a3=="CR") || (a3=="CC" && a1=="CR"))){*coeff= 70.0;*eq=117.00;return 1;} // CC-NB-CR    70.0   117.00
  if(a2=="CW" && ((a1=="CC" && a3=="NA") || (a3=="CC" && a1=="NA"))){*coeff= 70.0;*eq=120.00;return 1;} // CC-CW-NA    70.0   120.00
  if(a2=="CW" && ((a1=="CC" && a3=="H4") || (a3=="CC" && a1=="H4"))){*coeff= 50.0;*eq=120.00;return 1;} // CC-CW-H4    50.0   120.00
  if(a2=="NA" && ((a1=="CR" && a3=="CW") || (a3=="CR" && a1=="CW"))){*coeff= 70.0;*eq=120.00;return 1;} // CR-NA-CW    70.0   120.00
  if(a2=="CC" && ((a1=="CW" && a3=="NA") || (a3=="CW" && a1=="NA"))){*coeff= 70.0;*eq=120.00;return 1;} // CW-CC-NA    70.0   120.00
  if(a2=="CR" && ((a1=="NA" && a3=="NA") || (a3=="NA" && a1=="NA"))){*coeff= 70.0;*eq=120.00;return 1;} // NA-CR-NA    70.0   120.00
  if(a2=="S"  && ((a1=="CT" && a3=="S" ) || (a3=="CT" && a1=="S" ))){*coeff= 68.0;*eq=103.70;return 1;} // CT-S -S     68.0   103.70
//if(a2=="OW" && ((a1=="HW" && a3=="HW") || (a3=="HW" && a1=="HW"))){*coeff=100.0;*eq=104.52;return 1;} // HW-OW-HW   100.0   104.52  <-- TIP3P
  if(a2=="OW" && ((a1=="HW" && a3=="HW") || (a3=="HW" && a1=="HW"))){*coeff=  0.0;*eq=104.52;return 1;} // HW-OW-HW   100.0   104.52  <-- TIP3P angle is ignored in AMBER !!
  if(a2=="HW" && ((a1=="HW" && a3=="OW") || (a3=="HW" && a1=="OW"))){*coeff=  0.0;*eq=127.74;return 1;} // HW-HW-OW     0.0   127.74

  cout << "amber_force_field_angle , " << a1 << " - " << a2 << " - " << a3 << " is not defined!" << endl ;

  return -1;
}

int ForceField::amber_force_field_dihedral(const string   a1,
					   const string   a2,
					   const string   a3,
					   const string   a4,
					   const int      improper,
					   int          * n,
					   double       * coeff,
					   double       * div,
					   double       * eq,
					   double       * phase)
{
  if(improper==0){
    // frcmod.ff99SB
    if(1==d4_("C" ,"N" ,"CT","C" ,a1,a2,a3,a4  )){*n=4; div[0]=1; coeff[0]= 0.000; eq[0]=  0.0; phase[0]=1.0;
                                                        div[1]=1; coeff[1]= 0.270; eq[1]=  0.0; phase[1]=2.0;
							div[2]=1; coeff[2]= 0.420; eq[2]=  0.0; phase[2]=3.0;
							div[3]=1; coeff[3]= 0.000; eq[3]=  0.0; phase[3]=4.0; return 1;}
    if(1==d4_("N" ,"CT","C" ,"N" ,a1,a2,a3,a4  )){*n=4; div[0]=1; coeff[0]= 0.450; eq[0]=180.0; phase[0]=1.0;
                                                        div[1]=1; coeff[1]= 1.580; eq[1]=180.0; phase[1]=2.0;
							div[2]=1; coeff[2]= 0.550; eq[2]=180.0; phase[2]=3.0;
							div[3]=1; coeff[3]= 0.000; eq[3]=  0.0; phase[3]=4.0; return 1;}
    if(1==d4_("CT","CT","N" ,"C" ,a1,a2,a3,a4  )){*n=4; div[0]=1; coeff[0]= 0.000; eq[0]=  0.0; phase[0]=4.0;
                                                        div[1]=1; coeff[1]= 0.400; eq[1]=  0.0; phase[1]=3.0; 
							div[2]=1; coeff[2]= 2.000; eq[2]=  0.0; phase[2]=2.0;
							div[3]=1; coeff[3]= 2.000; eq[3]=  0.0; phase[3]=1.0; return 1;}
    if(1==d4_("CT","CT","C" ,"N" ,a1,a2,a3,a4  )){*n=4; div[0]=1; coeff[0]= 0.000; eq[0]=  0.0; phase[0]=4.0;
                                                        div[1]=1; coeff[1]= 0.400; eq[1]=  0.0; phase[1]=3.0;         
							div[2]=1; coeff[2]= 0.200; eq[2]=  0.0; phase[2]=2.0;
							div[3]=1; coeff[3]= 0.200; eq[3]=  0.0; phase[3]=1.0; return 1;}
    // parm99.dat
    if(1==d4_("H" ,"N" ,"C" ,"O" ,a1,a2,a3,a4  )){*n=2; div[0]=1; coeff[0]= 2.000; eq[0]=  0.0; phase[0]=1.0;
                                                        div[1]=1; coeff[1]= 2.500; eq[1]=180.0; phase[1]=2.0; return 1;}    
    if(1==d4_("H1","CT","C" ,"O" ,a1,a2,a3,a4  )){*n=3; div[0]=1; coeff[0]= 0.800; eq[0]=  0.0; phase[0]=1.0;
                                                        div[1]=1; coeff[1]= 0.000; eq[1]=  0.0; phase[1]=2.0;
						        div[2]=1; coeff[2]= 0.080; eq[2]=180.0; phase[2]=3.0; return 1;}
    if(1==d4_("HC","CT","CT","HC",a1,a2,a3,a4  )){*n=1; div[0]=1; coeff[0]= 0.150; eq[0]=  0.0; phase[0]=3.0; return 1;}
    if(1==d4_("HC","CT","CT","CT",a1,a2,a3,a4  )){*n=1; div[0]=1; coeff[0]= 0.160; eq[0]=  0.0; phase[0]=3.0; return 1;}
    if(1==d4_("CT","CT","CT","CT",a1,a2,a3,a4  )){*n=3; div[0]=1; coeff[0]= 0.180; eq[0]=  0.0; phase[0]=3.0;
                                                        div[1]=1; coeff[1]= 0.250; eq[1]=180.0; phase[1]=2.0;
                                                        div[2]=1; coeff[2]= 0.200; eq[2]=180.0; phase[2]=1.0; return 1;}
    if(1==d4_("N" ,"CT","C" ,"N" ,a1,a2,a3,a4  )){*n=2; div[0]=1; coeff[0]= 1.700; eq[0]=180.0; phase[0]=1.0; 
                                                        div[1]=1; coeff[1]= 2.000; eq[1]=180.0; phase[1]=2.0; return 1;}
    if(1==d4_("HC","CT","C" ,"O" ,a1,a2,a3,a4  )){*n=3; div[0]=1; coeff[0]= 0.800; eq[0]=  0.0; phase[0]=1.0;
                                                        div[1]=1; coeff[1]= 0.000; eq[1]=  0.0; phase[1]=2.0;
						        div[2]=1; coeff[2]= 0.080; eq[2]=180.0; phase[2]=3.0; return 1;}
    if(1==d4_("OH","CT","CT","OH",a1,a2,a3,a4  )){*n=2; div[0]=1; coeff[0]= 0.144; eq[0]=  0.0; phase[0]=3.0;
                                                        div[1]=1; coeff[1]= 1.175; eq[1]=  0.0; phase[1]=2.0; return 1;}
    if(1==d4_("H1","CT","CT","OH",a1,a2,a3,a4  )){*n=2; div[0]=1; coeff[0]= 0.000; eq[0]=  0.0; phase[0]=3.0;
                                                        div[1]=1; coeff[1]= 0.25 ; eq[1]=  0.0; phase[1]=1.0; return 1;}
    if(1==d4_("HC","CT","CT","OH",a1,a2,a3,a4  )){*n=2; div[0]=1; coeff[0]= 0.000; eq[0]=  0.0; phase[0]=3.0;
                                                        div[1]=1; coeff[1]= 0.25 ; eq[1]=  0.0; phase[1]=1.0; return 1;}
    if(1==d4_("HO","OH","CT","CT",a1,a2,a3,a4  )){*n=2; div[0]=1; coeff[0]= 0.16 ; eq[0]=  0.0; phase[0]=3.0;
                                                        div[1]=1; coeff[1]= 0.25 ; eq[1]=  0.0; phase[1]=1.0; return 1;}
    if(1==d4_("HO","OH","C" ,"O" ,a1,a2,a3,a4  )){*n=2; div[0]=1; coeff[0]= 2.30 ; eq[0]=180.0; phase[0]=2.0;
                                                        div[1]=1; coeff[1]= 1.90 ; eq[1]=  0.0; phase[1]=1.0; return 1;}
    if(1==d4_("CT","S" ,"S" ,"CT",a1,a2,a3,a4  )){*n=2; div[0]=1; coeff[0]= 3.50 ; eq[0]=  0.0; phase[0]=2.0;
                                                        div[1]=1; coeff[1]= 0.60 ; eq[1]=  0.0; phase[1]=3.0; return 1;}
    // X-A-B-X
    if(1==d4_("X" ,"C" ,"N" ,"X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]=10.00 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"C" ,"CT","X" ,a1,a2,a3,a4  )){*n=1; div[0]=6; coeff[0]= 0.00 ; eq[0]=  0.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CT","N" ,"X" ,a1,a2,a3,a4  )){*n=1; div[0]=6; coeff[0]= 0.00 ; eq[0]=  0.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CT","N3","X" ,a1,a2,a3,a4  )){*n=1; div[0]=9; coeff[0]= 1.40 ; eq[0]=  0.0; phase[0]=3.0; return 1;}
    if(1==d4_("X" ,"CT","CT","X" ,a1,a2,a3,a4  )){*n=1; div[0]=9; coeff[0]= 1.40 ; eq[0]=  0.0; phase[0]=3.0; return 1;}
    if(1==d4_("X" ,"CT","S" ,"X" ,a1,a2,a3,a4  )){*n=1; div[0]=3; coeff[0]= 1.00 ; eq[0]=  0.0; phase[0]=3.0; return 1;}
    if(1==d4_("X" ,"CA","CT","X" ,a1,a2,a3,a4  )){*n=1; div[0]=6; coeff[0]= 0.00 ; eq[0]=  0.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CA","CA","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]=14.50 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CA","CB","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]=14.00 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CB","CN","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]=12.00 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"C*","CW","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]=26.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CA","CN","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]=14.50 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"C*","CB","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]= 6.70 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CW","NA","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]= 6.00 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"C*","CW","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]=26.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"C*","CT","X" ,a1,a2,a3,a4  )){*n=1; div[0]=6; coeff[0]= 0.00 ; eq[0]=  0.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CN","NA","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]= 6.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CW","NA","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]= 6.00 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"C*","CT","X" ,a1,a2,a3,a4  )){*n=1; div[0]=6; coeff[0]= 0.00 ; eq[0]=  0.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CT","OH","X" ,a1,a2,a3,a4  )){*n=1; div[0]=3; coeff[0]= 0.50 ; eq[0]=  0.0; phase[0]=3.0; return 1;}
    if(1==d4_("X" ,"C" ,"CA","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]=14.50 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"C" ,"OH","X" ,a1,a2,a3,a4  )){*n=1; div[0]=2; coeff[0]= 4.60 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CT","SH","X" ,a1,a2,a3,a4  )){*n=1; div[0]=3; coeff[0]= 0.75 ; eq[0]=  0.0; phase[0]=3.0; return 1;}
    if(1==d4_("X" ,"CT","N2","X" ,a1,a2,a3,a4  )){*n=1; div[0]=6; coeff[0]= 0.00 ; eq[0]=  0.0; phase[0]=3.0; return 1;}
    if(1==d4_("X" ,"CA","N2","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]= 9.60 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CC","CT","X" ,a1,a2,a3,a4  )){*n=1; div[0]=6; coeff[0]= 0.00 ; eq[0]=  0.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CC","NA","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]= 5.60 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CC","CV","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]=20.60 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CR","NA","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]= 9.30 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CR","NB","X" ,a1,a2,a3,a4  )){*n=1; div[0]=2; coeff[0]=10.00 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CV","NB","X" ,a1,a2,a3,a4  )){*n=1; div[0]=2; coeff[0]= 4.80 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CC","NB","X" ,a1,a2,a3,a4  )){*n=1; div[0]=2; coeff[0]= 4.80 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CC","CW","X" ,a1,a2,a3,a4  )){*n=1; div[0]=4; coeff[0]=21.50 ; eq[0]=180.0; phase[0]=2.0; return 1;}
  }
  if(improper==1){
    if(1==d4_("NA","CV","CC","CT",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;} // <-- CT-CV-CC-NA parameter ??
    if(1==d4_("NB","CW","CC","CT",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;} // <-- CT-CW-CC-NB parameter ??
    if(1==d4_("NA","CW","CC","CT",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;} // <-- CT-CW-CC-NA parameter ??
    if(1==d4_("C" ,"CT","N" ,"H" ,a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("CB","CT","C*","CW",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("CA","CA","CA","CT",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("CA","CA","CA","OH",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("Br","CA","CA","CA",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("CA","CA","C" ,"OH",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    // X-A-B-C
    if(1==d4_("X" ,"O2","C" ,"O2",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]=10.50 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"CT","N" ,"CT",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.00 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"N2","CA","N2",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]=10.50 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    // X-X-A-B
    if(1==d4_("X" ,"X" ,"C" ,"O" ,a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]=10.50 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"X" ,"CA","HA",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"X" ,"NA","H" ,a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.00 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"X" ,"CW","H4",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"X" ,"N" ,"H" ,a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.00 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"X" ,"N2","H" ,a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.00 ; eq[0]=180.0; phase[0]=2.0; return 1;}
    if(1==d4_("X" ,"X" ,"CV","H4",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;} 
    if(1==d4_("X" ,"X" ,"CR","H5",a1,a2,a3,a4,1)){*n=1; div[0]=1; coeff[0]= 1.10 ; eq[0]=180.0; phase[0]=2.0; return 1;}
  }

  cout << "amber_force_field_dihedral , " << a1 << " - " << a2 << " - " << a3 << " - " << a4 << " is not defined!" << endl ;

  return -1;
}

void ForceField::amber_force_field_get_atm_coeff_A(double * coeff_A_) const 
{
  int i,j,k,chk;
  for(i=0;i<natm;i++){
    for(j=0;j<natm;j++){
      coeff_A_[i*natm+j]=sqrt(atm_vdw_e[i]*atm_vdw_e[j])*pow(atm_vdw_r[i]+atm_vdw_r[j],12);
    }
  }
}

void ForceField::amber_force_field_get_atm_coeff_B( double * coeff_B_) const 
{
  int i,j,k,chk;
  for(i=0;i<natm;i++){
    for(j=0;j<natm;j++){
      coeff_B_[i*natm+j]=2.0*sqrt(atm_vdw_e[i]*atm_vdw_e[j])*pow(atm_vdw_r[i]+atm_vdw_r[j], 6);
    }
  }
}


void ForceField::amber_force_field_get_improper (const int   rt,
						 const int   tn,
						 const int   tc,
						 const int   his,
						 int       * n,
						 int       * a1,
						 int       * a2,
						 int       * a3,
						 int       * a4,
						 int       * b1,
						 int       * b2,
						 int       * b3,
						 int       * b4)
{
  // rt   ---> residue index
  // tn   ---> n-terminal type
  // tc   ---> c-terminal type
  // hist ---> hist type

  (*n)=0;
  if(rt==700){
    if(tn==-1 && tc==-1)  { a1[*n]=   1; a2[*n]= 707; a3[*n]=   2; a4[*n]=   0; b1[*n]--; (*n)++;  // -M(C)      CD         N          CA
                            a1[*n]=   0; a2[*n]=   2; a3[*n]=   1; a4[*n]=   4; b2[*n]++; (*n)++;} // CA         +M(N)      C          O
    if(tn>0)              { a1[*n]=   0; a2[*n]=   2; a3[*n]=   1; a4[*n]=   4; b2[*n]++; (*n)++;} // CA         +M(N)      C          O
    if(tc>0)              { a1[*n]=   1; a2[*n]= 707; a3[*n]=   2; a4[*n]=   0; b1[*n]--; (*n)++;  // -M(C)      CD         N          CA
                            a1[*n]=   0; a2[*n]=   4; a3[*n]=   1; a4[*n]=   4;           (*n)++;} // CA         OXT        C          O          <-- OXT and O !!
  } else {
    if(tn==-1 && tc==-1)  { a1[*n]=   1; a2[*n]=   0; a3[*n]=   2; a4[*n]=   5; b1[*n]--; (*n)++;  // -M(C)      CA         N          H
                            a1[*n]=   0; a2[*n]=   2; a3[*n]=   1; a4[*n]=   4; b2[*n]++; (*n)++;} // CA         +M(N)      C          O
    if(tn==6)             {                                                                      }
    if(tc==6)             {                                                                      }
    if(tn>0 && tn!=6)     { a1[*n]=   0; a2[*n]=   2; a3[*n]=   1; a4[*n]=   4; b2[*n]++; (*n)++;} // CA         +M(N)      C          O
    if(tc>0 && tc!=6)     { a1[*n]=   1; a2[*n]=   0; a3[*n]=   2; a4[*n]=   5; b1[*n]--; (*n)++;  // -M(C)      CA         N          H
                            a1[*n]=   0; a2[*n]=   4; a3[*n]=   1; a4[*n]=   4;           (*n)++;} // CA         OXT        C          O          <-- OXT and O !!
    if(rt==800)           { a1[*n]= 806; a2[*n]= 810; a3[*n]= 808; a4[*n]= 815;           (*n)++;  // CG  < 806> CE2 < 810> CD2 < 808> HD2 < 815>
                          //a1[*n]= 808; a2[*n]= 811; a3[*n]= 810; a4[*n]= 817;           (*n)++;  // CD2 < 808> CZ  < 811> CE2 < 810> HE2 < 817> <-- definition in all_amino94.dat, but not used !!
                            a1[*n]= 811; a2[*n]= 808; a3[*n]= 810; a4[*n]= 817;           (*n)++;  // CZ  < 811> CD2 < 808> CE2 < 810> HE2 < 817> <-- AMBER uses this definition !!
			    a1[*n]= 809; a2[*n]= 810; a3[*n]= 811; a4[*n]= 818;           (*n)++;  // CE1 < 809> CE2 < 810> CZ  < 811> HZ  < 818>
			    a1[*n]= 807; a2[*n]= 811; a3[*n]= 809; a4[*n]= 816;           (*n)++;  // CD1 < 807> CZ  < 811> CE1 < 809> HE1 < 816>
			    a1[*n]= 806; a2[*n]= 809; a3[*n]= 807; a4[*n]= 814;           (*n)++;  // CG  < 806> CE1 < 809> CD1 < 807> HD1 < 814>
			    a1[*n]= 807; a2[*n]= 808; a3[*n]= 806; a4[*n]= 805;           (*n)++;} // CD1 < 807> CD2 < 808> CG  < 806> CB  < 805>
    if(rt==900)           { a1[*n]= 907; a2[*n]= 910; a3[*n]= 909; a4[*n]= 918;           (*n)++;  // CD1 < 907> CE2 < 910> NE1 < 909> HE1 < 918>
                            a1[*n]= 910; a2[*n]= 913; a3[*n]= 914; a4[*n]= 922;           (*n)++;  // CE2 < 910> CH2 < 913> CZ2 < 914> HZ2 < 922>
			    a1[*n]= 914; a2[*n]= 912; a3[*n]= 913; a4[*n]= 921;           (*n)++;  // CZ2 < 914> CZ3 < 912> CH2 < 913> HH2 < 921>
			    a1[*n]= 913; a2[*n]= 911; a3[*n]= 912; a4[*n]= 920;           (*n)++;  // CH2 < 913> CE3 < 911> CZ3 < 912> HZ3 < 920>
			    a1[*n]= 912; a2[*n]= 908; a3[*n]= 911; a4[*n]= 919;           (*n)++;  // CZ3 < 912> CD2 < 908> CE3 < 911> HE3 < 919>
			    a1[*n]= 906; a2[*n]= 909; a3[*n]= 907; a4[*n]= 917;           (*n)++;  // CG  < 906> NE1 < 909> CD1 < 907> HD1 < 917>
			  //a1[*n]= 907; a2[*n]= 908; a3[*n]= 906; a4[*n]= 905;           (*n)++;  // CD1 < 907> CD2 < 908> CG  < 906> CB  < 905> <-- definition in all_amino94.dat, but not used !!
			    a1[*n]= 908; a2[*n]= 905; a3[*n]= 906; a4[*n]= 907;           (*n)++;} // CD2 < 908> CB  < 905> CG  < 906> CD1 < 907> <-- AMBER uses this definition !!
    if(rt==1200)          { a1[*n]=1205; a2[*n]=1207; a3[*n]=1206; a4[*n]=1208;           (*n)++;  // CB  <1205> ND2 <1207> CG  <1206> OD1 <1208>
                            a1[*n]=1206; a2[*n]=1211; a3[*n]=1207; a4[*n]=1212;           (*n)++;} // CG  <1206> HD21<1211> ND2 <1207> HD22<1212>
    if(rt==1300)          { a1[*n]=1306; a2[*n]=1308; a3[*n]=1307; a4[*n]=1309;           (*n)++;  // CG  <1306> NE2 <1308> CD  <1307> OE1 <1309>
                            a1[*n]=1307; a2[*n]=1314; a3[*n]=1308; a4[*n]=1315;           (*n)++;} // CD  <1307> HE21<1314> NE2 <1308> HE22<1315>
    if(rt==1400)          { a1[*n]=1406; a2[*n]=1410; a3[*n]=1408; a4[*n]=1416;           (*n)++;  // CG  <1406> CE2 <1410> CD2 <1408> HD2 <1416>
                          //a1[*n]=1408; a2[*n]=1411; a3[*n]=1410; a4[*n]=1418;           (*n)++;  // CD2 <1408> CZ  <1411> CE2 <1410> HE2 <1418> <-- definition in all_amino94.dat, but not used !!
                            a1[*n]=1411; a2[*n]=1408; a3[*n]=1410; a4[*n]=1418;           (*n)++;  // CZ  <1411> CD2 <1408> CE2 <1410> HE2 <1418> <-- AMBER uses this definition !!
		            a1[*n]=1407; a2[*n]=1411; a3[*n]=1409; a4[*n]=1417;           (*n)++;  // CD1 <1407> CZ  <1411> CE1 <1409> HE1 <1417>
		            a1[*n]=1406; a2[*n]=1409; a3[*n]=1407; a4[*n]=1415;           (*n)++;  // CG  <1406> CE1 <1409> CD1 <1407> HD1 <1415>
		            a1[*n]=1407; a2[*n]=1408; a3[*n]=1406; a4[*n]=1405;           (*n)++;  // CD1 <1407> CD2 <1408> CG  <1406> CB  <1405>
		            a1[*n]=1409; a2[*n]=1410; a3[*n]=1411; a4[*n]=1412;           (*n)++;} // CE1 <1409> CE2 <1410> CZ  <1411> OH  <1412>
    if(rt==1700)          { a1[*n]=1708; a2[*n]=1710; a3[*n]=1709; a4[*n]=1711;           (*n)++;  // NE  <1708> NH1 <1710> CZ  <1709> NH2 <1711>
                            a1[*n]=1707; a2[*n]=1709; a3[*n]=1708; a4[*n]=1718;           (*n)++;  // CD  <1707> CZ  <1709> NE  <1708> HE  <1718>
			    a1[*n]=1709; a2[*n]=1719; a3[*n]=1710; a4[*n]=1720;           (*n)++;  // CZ  <1709> HH11<1719> NH1 <1710> HH12<1720>
			    a1[*n]=1709; a2[*n]=1721; a3[*n]=1711; a4[*n]=1722;           (*n)++;} // CZ  <1709> HH21<1721> NH2 <1711> HH22<1722>
    if(rt==1800 && his==0){ a1[*n]=1806; a2[*n]=1810; a3[*n]=1808; a4[*n]=1814;           (*n)++;  // CG  <1806> CE1 <1810> ND1 <1808> HD1 <1814>
                            a1[*n]=1806; a2[*n]=1809; a3[*n]=1807; a4[*n]=1813;           (*n)++;  // CG  <1806> NE2 <1809> CD2 <1807> HD2 <1813>
			    a1[*n]=1808; a2[*n]=1809; a3[*n]=1810; a4[*n]=1815;           (*n)++;  // ND1 <1808> NE2 <1809> CE1 <1810> HE1 <1815>
			    a1[*n]=1808; a2[*n]=1807; a3[*n]=1806; a4[*n]=1805;           (*n)++;} // ND1 <1808> CD2 <1807> CG  <1806> CB  <1805>
    if(rt==1800 && his==1){ a1[*n]=1810; a2[*n]=1807; a3[*n]=1809; a4[*n]=1816;           (*n)++;  // CE1 <1810> CD2 <1807> NE2 <1809> HE2 <1816>
                            a1[*n]=1806; a2[*n]=1809; a3[*n]=1807; a4[*n]=1813;           (*n)++;  // CG  <1806> NE2 <1809> CD2 <1807> HD2 <1813>
			    a1[*n]=1808; a2[*n]=1809; a3[*n]=1810; a4[*n]=1815;           (*n)++;  // ND1 <1808> NE2 <1809> CE1 <1810> HE1 <1815>
			    a1[*n]=1808; a2[*n]=1807; a3[*n]=1806; a4[*n]=1805;           (*n)++;} // ND1 <1808> CD2 <1807> CG  <1806> CB  <1805>
    if(rt==1800 && his==2){ a1[*n]=1806; a2[*n]=1810; a3[*n]=1808; a4[*n]=1814;           (*n)++;  // CG  <1806> CE1 <1810> ND1 <1808> HD1 <1814>
                            a1[*n]=1810; a2[*n]=1807; a3[*n]=1809; a4[*n]=1816;           (*n)++;  // CE1 <1810> CD2 <1807> NE2 <1809> HE2 <1816>
			    a1[*n]=1806; a2[*n]=1809; a3[*n]=1807; a4[*n]=1813;           (*n)++;  // CG  <1806> NE2 <1809> CD2 <1807> HD2 <1813>
			    a1[*n]=1808; a2[*n]=1809; a3[*n]=1810; a4[*n]=1815;           (*n)++;  // ND1 <1808> NE2 <1809> CE1 <1810> HE1 <1815>
			    a1[*n]=1808; a2[*n]=1807; a3[*n]=1806; a4[*n]=1805;           (*n)++;} // ND1 <1808> CD2 <1807> CG  <1806> CB  <1805>
    if(rt==1900)          { a1[*n]=1905; a2[*n]=1907; a3[*n]=1906; a4[*n]=1908;           (*n)++;} // CB  <1905> OD1 <1907> CG  <1906> OD2 <1908>
    if(rt==2000)          { a1[*n]=2006; a2[*n]=2008; a3[*n]=2007; a4[*n]=2009;           (*n)++;} // CG  <2006> OE1 <2008> CD  <2007> OE2 <2009>
  }
}

int ForceField::amber_force_field_charge(const string   a1,
					 const string   pdb_in,
					 const int      res_type,
					 const int      terminal_,
					 double       * out_ch,
					 const int      info_1)
{
  // terminal_ == 0  : not terminal
  // terminal_ == 1  ; N-terminal
  // terminal_ == 2  ; C-terminal
  // terminal_ == 61  ; ACE_N-terminal
  // terminal_ == 62  ; NME_C-terminal
  
  int terminal,ace,nme;
  string pdb=iTrim(pdb_in.c_str());

  terminal=terminal_;
  ace=0;  if(terminal_==61){terminal=0;ace=1;}
  nme=0;  if(terminal_==62){terminal=0;nme=1;}

  // ACE (20150120)
  if(ace==1 && terminal==0){
    if(pdb=="HH31" && a1=="HC"){*out_ch=  0.11230; return 1;}
    if(pdb=="CH3"  && a1=="CT"){*out_ch= -0.36620; return 1;}
    if(pdb=="HH32" && a1=="HC"){*out_ch=  0.11230; return 1;}
    if(pdb=="HH33" && a1=="HC"){*out_ch=  0.11230; return 1;}
    if(pdb=="C"    && a1=="C" ){*out_ch=  0.59720; return 1;}
    if(pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
  }

  // NME (20150120)
  if(nme==1 && terminal==0){
    if(pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(pdb=="CH3"  && a1=="CT"){*out_ch= -0.14900; return 1;}
    if(pdb=="HH31" && a1=="H1"){*out_ch=  0.09760; return 1;}
    if(pdb=="HH32" && a1=="H1"){*out_ch=  0.09760; return 1;}
    if(pdb=="HH33" && a1=="H1"){*out_ch=  0.09760; return 1;}
  }

  if(res_type== 100){
    // GLY
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.02520; return 1;}
    if(terminal==0 && pdb=="HA2"  && a1=="H1"){*out_ch=  0.06980; return 1;}
    if(terminal==0 && pdb=="HA3"  && a1=="H1"){*out_ch=  0.06980; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NGLY
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.29430; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.16420; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.16420; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.16420; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch= -0.01000; return 1;}
    if(terminal==1 && pdb=="HA2"  && a1=="HP"){*out_ch=  0.08950; return 1;}
    if(terminal==1 && pdb=="HA3"  && a1=="HP"){*out_ch=  0.08950; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61630; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57220; return 1;}
    // CGLY
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.24930; return 1;}
    if(terminal==2 && pdb=="HA2"  && a1=="H1"){*out_ch=  0.10560; return 1;}
    if(terminal==2 && pdb=="HA3"  && a1=="H1"){*out_ch=  0.10560; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.72310; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.78550; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.78550; return 1;}
  }

  if(res_type== 200){
    // ALA
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch=  0.03370; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.08230; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.18250; return 1;}
    if(terminal==0 && pdb=="HB1"  && a1=="HC"){*out_ch=  0.06030; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.06030; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.06030; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NALA
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.14140; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.19970; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.19970; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.19970; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.09620; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.08890; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch= -0.05970; return 1;}
    if(terminal==1 && pdb=="HB1"  && a1=="HC"){*out_ch=  0.03000; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.03000; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.03000; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61630; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57220; return 1;}
    // CALA
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.17470; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.10670; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.20930; return 1;}
    if(terminal==2 && pdb=="HB1"  && a1=="HC"){*out_ch=  0.07640; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.07640; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.07640; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.77310; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.80550; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.80550; return 1;}
  }

  if(res_type== 300){
    // VAL
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.08750; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.09690; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch=  0.29850; return 1;}
    if(terminal==0 && pdb=="HB"   && a1=="HC"){*out_ch= -0.02970; return 1;}
    if(terminal==0 && pdb=="CG1"  && a1=="CT"){*out_ch= -0.31920; return 1;}
    if(terminal==0 && pdb=="HG11" && a1=="HC"){*out_ch=  0.07910; return 1;}
    if(terminal==0 && pdb=="HG12" && a1=="HC"){*out_ch=  0.07910; return 1;}
    if(terminal==0 && pdb=="HG13" && a1=="HC"){*out_ch=  0.07910; return 1;}
    if(terminal==0 && pdb=="CG2"  && a1=="CT"){*out_ch= -0.31920; return 1;}
    if(terminal==0 && pdb=="HG21" && a1=="HC"){*out_ch=  0.07910; return 1;}
    if(terminal==0 && pdb=="HG22" && a1=="HC"){*out_ch=  0.07910; return 1;}
    if(terminal==0 && pdb=="HG23" && a1=="HC"){*out_ch=  0.07910; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NVAL
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.05770; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.22720; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.22720; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.22720; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch= -0.00540; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.10930; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.31960; return 1;}
    if(terminal==1 && pdb=="HB"   && a1=="HC"){*out_ch= -0.02210; return 1;}
    if(terminal==1 && pdb=="CG1"  && a1=="CT"){*out_ch= -0.31290; return 1;}
    if(terminal==1 && pdb=="HG11" && a1=="HC"){*out_ch=  0.07350; return 1;}
    if(terminal==1 && pdb=="HG12" && a1=="HC"){*out_ch=  0.07350; return 1;}
    if(terminal==1 && pdb=="HG13" && a1=="HC"){*out_ch=  0.07350; return 1;}
    if(terminal==1 && pdb=="CG2"  && a1=="CT"){*out_ch= -0.31290; return 1;}
    if(terminal==1 && pdb=="HG21" && a1=="HC"){*out_ch=  0.07350; return 1;}
    if(terminal==1 && pdb=="HG22" && a1=="HC"){*out_ch=  0.07350; return 1;}
    if(terminal==1 && pdb=="HG23" && a1=="HC"){*out_ch=  0.07350; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61630; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57220; return 1;}
    // CVAL
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.34380; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.14380; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch=  0.19400; return 1;}
    if(terminal==2 && pdb=="HB"   && a1=="HC"){*out_ch=  0.03080; return 1;}
    if(terminal==2 && pdb=="CG1"  && a1=="CT"){*out_ch= -0.30640; return 1;}
    if(terminal==2 && pdb=="HG11" && a1=="HC"){*out_ch=  0.08360; return 1;}
    if(terminal==2 && pdb=="HG12" && a1=="HC"){*out_ch=  0.08360; return 1;}
    if(terminal==2 && pdb=="HG13" && a1=="HC"){*out_ch=  0.08360; return 1;}
    if(terminal==2 && pdb=="CG2"  && a1=="CT"){*out_ch= -0.30640; return 1;}
    if(terminal==2 && pdb=="HG21" && a1=="HC"){*out_ch=  0.08360; return 1;}
    if(terminal==2 && pdb=="HG22" && a1=="HC"){*out_ch=  0.08360; return 1;}
    if(terminal==2 && pdb=="HG23" && a1=="HC"){*out_ch=  0.08360; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.83500; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.81730; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.81730; return 1;}
  }
  
  if(res_type== 400){
    // LEU
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.05180; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.09220; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.11020; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.04570; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.04570; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CT"){*out_ch=  0.35310; return 1;}
    if(terminal==0 && pdb=="HG"   && a1=="HC"){*out_ch= -0.03610; return 1;}
    if(terminal==0 && pdb=="CD1"  && a1=="CT"){*out_ch= -0.41210; return 1;}
    if(terminal==0 && pdb=="HD11" && a1=="HC"){*out_ch=  0.10000; return 1;}
    if(terminal==0 && pdb=="HD12" && a1=="HC"){*out_ch=  0.10000; return 1;}
    if(terminal==0 && pdb=="HD13" && a1=="HC"){*out_ch=  0.10000; return 1;}
    if(terminal==0 && pdb=="CD2"  && a1=="CT"){*out_ch= -0.41210; return 1;}
    if(terminal==0 && pdb=="HD21" && a1=="HC"){*out_ch=  0.10000; return 1;}
    if(terminal==0 && pdb=="HD22" && a1=="HC"){*out_ch=  0.10000; return 1;}
    if(terminal==0 && pdb=="HD23" && a1=="HC"){*out_ch=  0.10000; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NLEU
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.10100; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.21480; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.21480; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.21480; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.01040; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.10530; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch= -0.02440; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.02560; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.02560; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CT"){*out_ch=  0.34210; return 1;}
    if(terminal==1 && pdb=="HG"   && a1=="HC"){*out_ch= -0.03800; return 1;}
    if(terminal==1 && pdb=="CD1"  && a1=="CT"){*out_ch= -0.41060; return 1;}
    if(terminal==1 && pdb=="HD11" && a1=="HC"){*out_ch=  0.09800; return 1;}
    if(terminal==1 && pdb=="HD12" && a1=="HC"){*out_ch=  0.09800; return 1;}
    if(terminal==1 && pdb=="HD13" && a1=="HC"){*out_ch=  0.09800; return 1;}
    if(terminal==1 && pdb=="CD2"  && a1=="CT"){*out_ch= -0.41040; return 1;}
    if(terminal==1 && pdb=="HD21" && a1=="HC"){*out_ch=  0.09800; return 1;}
    if(terminal==1 && pdb=="HD22" && a1=="HC"){*out_ch=  0.09800; return 1;}
    if(terminal==1 && pdb=="HD23" && a1=="HC"){*out_ch=  0.09800; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CLEU
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.28470; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.13460; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.24690; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.09740; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.09740; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CT"){*out_ch=  0.37060; return 1;}
    if(terminal==2 && pdb=="HG"   && a1=="HC"){*out_ch= -0.03740; return 1;}
    if(terminal==2 && pdb=="CD1"  && a1=="CT"){*out_ch= -0.41630; return 1;}
    if(terminal==2 && pdb=="HD11" && a1=="HC"){*out_ch=  0.10380; return 1;}
    if(terminal==2 && pdb=="HD12" && a1=="HC"){*out_ch=  0.10380; return 1;}
    if(terminal==2 && pdb=="HD13" && a1=="HC"){*out_ch=  0.10380; return 1;}
    if(terminal==2 && pdb=="CD2"  && a1=="CT"){*out_ch= -0.41630; return 1;}
    if(terminal==2 && pdb=="HD21" && a1=="HC"){*out_ch=  0.10380; return 1;}
    if(terminal==2 && pdb=="HD22" && a1=="HC"){*out_ch=  0.10380; return 1;}
    if(terminal==2 && pdb=="HD23" && a1=="HC"){*out_ch=  0.10380; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.83260; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.81990; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.81990; return 1;}
  }
  
  if(res_type== 500){
    // ILE
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.05970; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.08690; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch=  0.13030; return 1;}
    if(terminal==0 && pdb=="HB"   && a1=="HC"){*out_ch=  0.01870; return 1;}
    if(terminal==0 && pdb=="CG2"  && a1=="CT"){*out_ch= -0.32040; return 1;}
    if(terminal==0 && pdb=="HG21" && a1=="HC"){*out_ch=  0.08820; return 1;}
    if(terminal==0 && pdb=="HG22" && a1=="HC"){*out_ch=  0.08820; return 1;}
    if(terminal==0 && pdb=="HG23" && a1=="HC"){*out_ch=  0.08820; return 1;}
    if(terminal==0 && pdb=="CG1"  && a1=="CT"){*out_ch= -0.04300; return 1;}
    if(terminal==0 && pdb=="HG12" && a1=="HC"){*out_ch=  0.02360; return 1;}
    if(terminal==0 && pdb=="HG13" && a1=="HC"){*out_ch=  0.02360; return 1;}
    if(terminal==0 && pdb=="CD1"  && a1=="CT"){*out_ch= -0.06600; return 1;}
    if(terminal==0 && pdb=="HD11" && a1=="HC"){*out_ch=  0.01860; return 1;}
    if(terminal==0 && pdb=="HD12" && a1=="HC"){*out_ch=  0.01860; return 1;}
    if(terminal==0 && pdb=="HD13" && a1=="HC"){*out_ch=  0.01860; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NILE
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.03110; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.23290; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.23290; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.23290; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.02570; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.10310; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.18850; return 1;}
    if(terminal==1 && pdb=="HB"   && a1=="HC"){*out_ch=  0.02130; return 1;}
    if(terminal==1 && pdb=="CG2"  && a1=="CT"){*out_ch= -0.37200; return 1;}
    if(terminal==1 && pdb=="HG21" && a1=="HC"){*out_ch=  0.09470; return 1;}
    if(terminal==1 && pdb=="HG22" && a1=="HC"){*out_ch=  0.09470; return 1;}
    if(terminal==1 && pdb=="HG23" && a1=="HC"){*out_ch=  0.09470; return 1;}
    if(terminal==1 && pdb=="CG1"  && a1=="CT"){*out_ch= -0.03870; return 1;}
    if(terminal==1 && pdb=="HG12" && a1=="HC"){*out_ch=  0.02010; return 1;}
    if(terminal==1 && pdb=="HG13" && a1=="HC"){*out_ch=  0.02010; return 1;}
    if(terminal==1 && pdb=="CD1"  && a1=="CT"){*out_ch= -0.09080; return 1;}
    if(terminal==1 && pdb=="HD11" && a1=="HC"){*out_ch=  0.02260; return 1;}
    if(terminal==1 && pdb=="HD12" && a1=="HC"){*out_ch=  0.02260; return 1;}
    if(terminal==1 && pdb=="HD13" && a1=="HC"){*out_ch=  0.02260; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CILE
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.31000; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.13750; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch=  0.03630; return 1;}
    if(terminal==2 && pdb=="HB"   && a1=="HC"){*out_ch=  0.07660; return 1;}
    if(terminal==2 && pdb=="CG2"  && a1=="CT"){*out_ch= -0.34980; return 1;}
    if(terminal==2 && pdb=="HG21" && a1=="HC"){*out_ch=  0.10210; return 1;}
    if(terminal==2 && pdb=="HG22" && a1=="HC"){*out_ch=  0.10210; return 1;}
    if(terminal==2 && pdb=="HG23" && a1=="HC"){*out_ch=  0.10210; return 1;}
    if(terminal==2 && pdb=="CG1"  && a1=="CT"){*out_ch= -0.03230; return 1;}
    if(terminal==2 && pdb=="HG12" && a1=="HC"){*out_ch=  0.03210; return 1;}
    if(terminal==2 && pdb=="HG13" && a1=="HC"){*out_ch=  0.03210; return 1;}
    if(terminal==2 && pdb=="CD1"  && a1=="CT"){*out_ch= -0.06990; return 1;}
    if(terminal==2 && pdb=="HD11" && a1=="HC"){*out_ch=  0.01960; return 1;}
    if(terminal==2 && pdb=="HD12" && a1=="HC"){*out_ch=  0.01960; return 1;}
    if(terminal==2 && pdb=="HD13" && a1=="HC"){*out_ch=  0.01960; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.83430; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.81900; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.81900; return 1;}
  }
  
  if(res_type== 600){
    // MET
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.02370; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.08800; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch=  0.03420; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.02410; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.02410; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CT"){*out_ch=  0.00180; return 1;}
    if(terminal==0 && pdb=="HG2"  && a1=="H1"){*out_ch=  0.04400; return 1;}
    if(terminal==0 && pdb=="HG3"  && a1=="H1"){*out_ch=  0.04400; return 1;}
    if(terminal==0 && pdb=="SD"   && a1=="S" ){*out_ch= -0.27370; return 1;}
    if(terminal==0 && pdb=="CE"   && a1=="CT"){*out_ch= -0.05360; return 1;}
    if(terminal==0 && pdb=="HE1"  && a1=="H1"){*out_ch=  0.06840; return 1;}
    if(terminal==0 && pdb=="HE2"  && a1=="H1"){*out_ch=  0.06840; return 1;}
    if(terminal==0 && pdb=="HE3"  && a1=="H1"){*out_ch=  0.06840; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NMET
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.15920; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.19840; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.19840; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.19840; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.02210; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.11160; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.08650; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.01250; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.01250; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CT"){*out_ch=  0.03340; return 1;}
    if(terminal==1 && pdb=="HG2"  && a1=="H1"){*out_ch=  0.02920; return 1;}
    if(terminal==1 && pdb=="HG3"  && a1=="H1"){*out_ch=  0.02920; return 1;}
    if(terminal==1 && pdb=="SD"   && a1=="S" ){*out_ch= -0.27740; return 1;}
    if(terminal==1 && pdb=="CE"   && a1=="CT"){*out_ch= -0.03410; return 1;}
    if(terminal==1 && pdb=="HE1"  && a1=="H1"){*out_ch=  0.05970; return 1;}
    if(terminal==1 && pdb=="HE2"  && a1=="H1"){*out_ch=  0.05970; return 1;}
    if(terminal==1 && pdb=="HE3"  && a1=="H1"){*out_ch=  0.05970; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CMET
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.25970; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.12770; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.02360; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.04800; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.04800; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CT"){*out_ch=  0.04920; return 1;}
    if(terminal==2 && pdb=="HG2"  && a1=="H1"){*out_ch=  0.03170; return 1;}
    if(terminal==2 && pdb=="HG3"  && a1=="H1"){*out_ch=  0.03170; return 1;}
    if(terminal==2 && pdb=="SD"   && a1=="S" ){*out_ch= -0.26920; return 1;}
    if(terminal==2 && pdb=="CE"   && a1=="CT"){*out_ch= -0.03760; return 1;}
    if(terminal==2 && pdb=="HE1"  && a1=="H1"){*out_ch=  0.06250; return 1;}
    if(terminal==2 && pdb=="HE2"  && a1=="H1"){*out_ch=  0.06250; return 1;}
    if(terminal==2 && pdb=="HE3"  && a1=="H1"){*out_ch=  0.06250; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.80130; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.81050; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.81050; return 1;}
  }
  
  if(res_type== 700){
    // PRO
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.25480; return 1;}
    if(terminal==0 && pdb=="CD"   && a1=="CT"){*out_ch=  0.01920; return 1;}
    if(terminal==0 && pdb=="HD2"  && a1=="H1"){*out_ch=  0.03910; return 1;}
    if(terminal==0 && pdb=="HD3"  && a1=="H1"){*out_ch=  0.03910; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CT"){*out_ch=  0.01890; return 1;}
    if(terminal==0 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.02130; return 1;}
    if(terminal==0 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.02130; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.00700; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.02530; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.02530; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.02660; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.06410; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.58960; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.57480; return 1;}
    // NPRO
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch= -0.20200; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.31200; return 1;} // <--- NOT PDB TAG OF AMBER (ONLY PAICS) !!
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.31200; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.31200; return 1;}
    if(terminal==1 && pdb=="CD"   && a1=="CT"){*out_ch= -0.01200; return 1;}
    if(terminal==1 && pdb=="HD2"  && a1=="HP"){*out_ch=  0.10000; return 1;}
    if(terminal==1 && pdb=="HD3"  && a1=="HP"){*out_ch=  0.10000; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CT"){*out_ch= -0.12100; return 1;}
    if(terminal==1 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.10000; return 1;}
    if(terminal==1 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.10000; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch= -0.11500; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.10000; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.10000; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.10000; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.10000; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.52600; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.50000; return 1;}
    // CPRO
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.28020; return 1;}
    if(terminal==2 && pdb=="CD"   && a1=="CT"){*out_ch=  0.04340; return 1;}
    if(terminal==2 && pdb=="HD2"  && a1=="H1"){*out_ch=  0.03310; return 1;}
    if(terminal==2 && pdb=="HD3"  && a1=="H1"){*out_ch=  0.03310; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CT"){*out_ch=  0.04660; return 1;}
    if(terminal==2 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.01720; return 1;}
    if(terminal==2 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.01720; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.05430; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.03810; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.03810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.13360; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.07760; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.66310; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.76970; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.76970; return 1;}
  }
  
  if(res_type== 800){
    // PHE
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.00240; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.09780; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.03430; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.02950; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.02950; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CA"){*out_ch=  0.01180; return 1;}
    if(terminal==0 && pdb=="CD1"  && a1=="CA"){*out_ch= -0.12560; return 1;}
    if(terminal==0 && pdb=="HD1"  && a1=="HA"){*out_ch=  0.13300; return 1;}
    if(terminal==0 && pdb=="CE1"  && a1=="CA"){*out_ch= -0.17040; return 1;}
    if(terminal==0 && pdb=="HE1"  && a1=="HA"){*out_ch=  0.14300; return 1;}
    if(terminal==0 && pdb=="CZ"   && a1=="CA"){*out_ch= -0.10720; return 1;}
    if(terminal==0 && pdb=="HZ"   && a1=="HA"){*out_ch=  0.12970; return 1;}
    if(terminal==0 && pdb=="CE2"  && a1=="CA"){*out_ch= -0.17040; return 1;}
    if(terminal==0 && pdb=="HE2"  && a1=="HA"){*out_ch=  0.14300; return 1;}
    if(terminal==0 && pdb=="CD2"  && a1=="CA"){*out_ch= -0.12560; return 1;}
    if(terminal==0 && pdb=="HD2"  && a1=="HA"){*out_ch=  0.13300; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NPHE
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.17370; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.19210; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.19210; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.19210; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.07330; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.10410; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.03300; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.01040; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.01040; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CA"){*out_ch=  0.00310; return 1;}
    if(terminal==1 && pdb=="CD1"  && a1=="CA"){*out_ch= -0.13920; return 1;}
    if(terminal==1 && pdb=="HD1"  && a1=="HA"){*out_ch=  0.13740; return 1;}
    if(terminal==1 && pdb=="CE1"  && a1=="CA"){*out_ch= -0.16020; return 1;}
    if(terminal==1 && pdb=="HE1"  && a1=="HA"){*out_ch=  0.14330; return 1;}
    if(terminal==1 && pdb=="CZ"   && a1=="CA"){*out_ch= -0.12080; return 1;}
    if(terminal==1 && pdb=="HZ"   && a1=="HA"){*out_ch=  0.13290; return 1;}
    if(terminal==1 && pdb=="CE2"  && a1=="CA"){*out_ch= -0.16030; return 1;}
    if(terminal==1 && pdb=="HE2"  && a1=="HA"){*out_ch=  0.14330; return 1;}
    if(terminal==1 && pdb=="CD2"  && a1=="CA"){*out_ch= -0.13910; return 1;}
    if(terminal==1 && pdb=="HD2"  && a1=="HA"){*out_ch=  0.13740; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CPHE
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.18250; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.10980; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.09590; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.04430; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.04430; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CA"){*out_ch=  0.05520; return 1;}
    if(terminal==2 && pdb=="CD1"  && a1=="CA"){*out_ch= -0.13000; return 1;}
    if(terminal==2 && pdb=="HD1"  && a1=="HA"){*out_ch=  0.14080; return 1;}
    if(terminal==2 && pdb=="CE1"  && a1=="CA"){*out_ch= -0.18470; return 1;}
    if(terminal==2 && pdb=="HE1"  && a1=="HA"){*out_ch=  0.14610; return 1;}
    if(terminal==2 && pdb=="CZ"   && a1=="CA"){*out_ch= -0.09440; return 1;}
    if(terminal==2 && pdb=="HZ"   && a1=="HA"){*out_ch=  0.12800; return 1;}
    if(terminal==2 && pdb=="CE2"  && a1=="CA"){*out_ch= -0.18470; return 1;}
    if(terminal==2 && pdb=="HE2"  && a1=="HA"){*out_ch=  0.14610; return 1;}
    if(terminal==2 && pdb=="CD2"  && a1=="CA"){*out_ch= -0.13000; return 1;}
    if(terminal==2 && pdb=="HD2"  && a1=="HA"){*out_ch=  0.14080; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.76600; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.80260; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.80260; return 1;}
  }
  
  if(res_type== 900){
    // TRP
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.02750; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.11230; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.00500; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.03390; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.03390; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="C*"){*out_ch= -0.14150; return 1;}
    if(terminal==0 && pdb=="CD1"  && a1=="CW"){*out_ch= -0.16380; return 1;}
    if(terminal==0 && pdb=="HD1"  && a1=="H4"){*out_ch=  0.20620; return 1;}
    if(terminal==0 && pdb=="NE1"  && a1=="NA"){*out_ch= -0.34180; return 1;}
    if(terminal==0 && pdb=="HE1"  && a1=="H" ){*out_ch=  0.34120; return 1;}
    if(terminal==0 && pdb=="CE2"  && a1=="CN"){*out_ch=  0.13800; return 1;}
    if(terminal==0 && pdb=="CZ2"  && a1=="CA"){*out_ch= -0.26010; return 1;}
    if(terminal==0 && pdb=="HZ2"  && a1=="HA"){*out_ch=  0.15720; return 1;}
    if(terminal==0 && pdb=="CH2"  && a1=="CA"){*out_ch= -0.11340; return 1;}
    if(terminal==0 && pdb=="HH2"  && a1=="HA"){*out_ch=  0.14170; return 1;}
    if(terminal==0 && pdb=="CZ3"  && a1=="CA"){*out_ch= -0.19720; return 1;}
    if(terminal==0 && pdb=="HZ3"  && a1=="HA"){*out_ch=  0.14470; return 1;}
    if(terminal==0 && pdb=="CE3"  && a1=="CA"){*out_ch= -0.23870; return 1;}
    if(terminal==0 && pdb=="HE3"  && a1=="HA"){*out_ch=  0.17000; return 1;}
    if(terminal==0 && pdb=="CD2"  && a1=="CB"){*out_ch=  0.12430; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NTRP
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.19130; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.18880; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.18880; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.18880; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.04210; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.11620; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.05430; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.02220; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.02220; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="C*"){*out_ch= -0.16540; return 1;}
    if(terminal==1 && pdb=="CD1"  && a1=="CW"){*out_ch= -0.17880; return 1;}
    if(terminal==1 && pdb=="HD1"  && a1=="H4"){*out_ch=  0.21950; return 1;}
    if(terminal==1 && pdb=="NE1"  && a1=="NA"){*out_ch= -0.34440; return 1;}
    if(terminal==1 && pdb=="HE1"  && a1=="H" ){*out_ch=  0.34120; return 1;}
    if(terminal==1 && pdb=="CE2"  && a1=="CN"){*out_ch=  0.15750; return 1;}
    if(terminal==1 && pdb=="CZ2"  && a1=="CA"){*out_ch= -0.27100; return 1;}
    if(terminal==1 && pdb=="HZ2"  && a1=="HA"){*out_ch=  0.15890; return 1;}
    if(terminal==1 && pdb=="CH2"  && a1=="CA"){*out_ch= -0.10800; return 1;}
    if(terminal==1 && pdb=="HH2"  && a1=="HA"){*out_ch=  0.14110; return 1;}
    if(terminal==1 && pdb=="CZ3"  && a1=="CA"){*out_ch= -0.20340; return 1;}
    if(terminal==1 && pdb=="HZ3"  && a1=="HA"){*out_ch=  0.14580; return 1;}
    if(terminal==1 && pdb=="CE3"  && a1=="CA"){*out_ch= -0.22650; return 1;}
    if(terminal==1 && pdb=="HE3"  && a1=="HA"){*out_ch=  0.16460; return 1;}
    if(terminal==1 && pdb=="CD2"  && a1=="CB"){*out_ch=  0.11320; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CTRP
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.20840; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.12720; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.07420; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.04970; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.04970; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="C*"){*out_ch= -0.07960; return 1;}
    if(terminal==2 && pdb=="CD1"  && a1=="CW"){*out_ch= -0.18080; return 1;}
    if(terminal==2 && pdb=="HD1"  && a1=="H4"){*out_ch=  0.20430; return 1;}
    if(terminal==2 && pdb=="NE1"  && a1=="NA"){*out_ch= -0.33160; return 1;}
    if(terminal==2 && pdb=="HE1"  && a1=="H" ){*out_ch=  0.34130; return 1;}
    if(terminal==2 && pdb=="CE2"  && a1=="CN"){*out_ch=  0.12220; return 1;}
    if(terminal==2 && pdb=="CZ2"  && a1=="CA"){*out_ch= -0.25940; return 1;}
    if(terminal==2 && pdb=="HZ2"  && a1=="HA"){*out_ch=  0.15670; return 1;}
    if(terminal==2 && pdb=="CH2"  && a1=="CA"){*out_ch= -0.10200; return 1;}
    if(terminal==2 && pdb=="HH2"  && a1=="HA"){*out_ch=  0.14010; return 1;}
    if(terminal==2 && pdb=="CZ3"  && a1=="CA"){*out_ch= -0.22870; return 1;}
    if(terminal==2 && pdb=="HZ3"  && a1=="HA"){*out_ch=  0.15070; return 1;}
    if(terminal==2 && pdb=="CE3"  && a1=="CA"){*out_ch= -0.18370; return 1;}
    if(terminal==2 && pdb=="HE3"  && a1=="HA"){*out_ch=  0.14910; return 1;}
    if(terminal==2 && pdb=="CD2"  && a1=="CB"){*out_ch=  0.10780; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.76580; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.80110; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.80110; return 1;}
  }
  
  if(res_type==1000){
    // SER
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.02490; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.08430; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch=  0.21170; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="H1"){*out_ch=  0.03520; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="H1"){*out_ch=  0.03520; return 1;}
    if(terminal==0 && pdb=="OG"   && a1=="OH"){*out_ch= -0.65460; return 1;}
    if(terminal==0 && pdb=="HG"   && a1=="HO"){*out_ch=  0.42750; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NSER
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.18490; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.18980; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.18980; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.18980; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.05670; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.07820; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.25960; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="H1"){*out_ch=  0.02730; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="H1"){*out_ch=  0.02730; return 1;}
    if(terminal==1 && pdb=="OG"   && a1=="OH"){*out_ch= -0.67140; return 1;}
    if(terminal==1 && pdb=="HG"   && a1=="HO"){*out_ch=  0.42390; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61630; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57220; return 1;}
    // CSER
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.27220; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.13040; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch=  0.11230; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="H1"){*out_ch=  0.08130; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="H1"){*out_ch=  0.08130; return 1;}
    if(terminal==2 && pdb=="OG"   && a1=="OH"){*out_ch= -0.65140; return 1;}
    if(terminal==2 && pdb=="HG"   && a1=="HO"){*out_ch=  0.44740; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.81130; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.81320; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.81320; return 1;}
  }

  if(res_type==1100){
    // THR
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.03890; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.10070; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch=  0.36540; return 1;}
    if(terminal==0 && pdb=="HB"   && a1=="H1"){*out_ch=  0.00430; return 1;}
    if(terminal==0 && pdb=="CG2"  && a1=="CT"){*out_ch= -0.24380; return 1;}
    if(terminal==0 && pdb=="HG21" && a1=="HC"){*out_ch=  0.06420; return 1;}
    if(terminal==0 && pdb=="HG22" && a1=="HC"){*out_ch=  0.06420; return 1;}
    if(terminal==0 && pdb=="HG23" && a1=="HC"){*out_ch=  0.06420; return 1;}
    if(terminal==0 && pdb=="OG1"  && a1=="OH"){*out_ch= -0.67610; return 1;}
    if(terminal==0 && pdb=="HG1"  && a1=="HO"){*out_ch=  0.41020; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NTHR
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.18120; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.19340; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.19340; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.19340; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.00340; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.10870; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.45140; return 1;}
    if(terminal==1 && pdb=="HB"   && a1=="H1"){*out_ch= -0.03230; return 1;}
    if(terminal==1 && pdb=="CG2"  && a1=="CT"){*out_ch= -0.25540; return 1;}
    if(terminal==1 && pdb=="HG21" && a1=="HC"){*out_ch=  0.06270; return 1;}
    if(terminal==1 && pdb=="HG22" && a1=="HC"){*out_ch=  0.06270; return 1;}
    if(terminal==1 && pdb=="HG23" && a1=="HC"){*out_ch=  0.06270; return 1;}
    if(terminal==1 && pdb=="OG1"  && a1=="OH"){*out_ch= -0.67640; return 1;}
    if(terminal==1 && pdb=="HG1"  && a1=="HO"){*out_ch=  0.40700; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61630; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57220; return 1;}
    // CTHR
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.24200; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.12070; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch=  0.30250; return 1;}
    if(terminal==2 && pdb=="HB"   && a1=="H1"){*out_ch=  0.00780; return 1;}
    if(terminal==2 && pdb=="CG2"  && a1=="CT"){*out_ch= -0.18530; return 1;}
    if(terminal==2 && pdb=="HG21" && a1=="HC"){*out_ch=  0.05860; return 1;}
    if(terminal==2 && pdb=="HG22" && a1=="HC"){*out_ch=  0.05860; return 1;}
    if(terminal==2 && pdb=="HG23" && a1=="HC"){*out_ch=  0.05860; return 1;}
    if(terminal==2 && pdb=="OG1"  && a1=="OH"){*out_ch= -0.64960; return 1;}
    if(terminal==2 && pdb=="HG1"  && a1=="HO"){*out_ch=  0.41190; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.78100; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.80440; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.80440; return 1;}
  }
  
  if(res_type==1200){
    // ASN
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch=  0.01430; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.10480; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.20410; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.07970; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.07970; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="C" ){*out_ch=  0.71300; return 1;}
    if(terminal==0 && pdb=="OD1"  && a1=="O" ){*out_ch= -0.59310; return 1;}
    if(terminal==0 && pdb=="ND2"  && a1=="N" ){*out_ch= -0.91910; return 1;}
    if(terminal==0 && pdb=="HD21" && a1=="H" ){*out_ch=  0.41960; return 1;}
    if(terminal==0 && pdb=="HD22" && a1=="H" ){*out_ch=  0.41960; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NASN
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.18010; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.19210; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.19210; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.19210; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.03680; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.12310; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch= -0.02830; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.05150; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.05150; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="C" ){*out_ch=  0.58330; return 1;}
    if(terminal==1 && pdb=="OD1"  && a1=="O" ){*out_ch= -0.57440; return 1;}
    if(terminal==1 && pdb=="ND2"  && a1=="N" ){*out_ch= -0.86340; return 1;}
    if(terminal==1 && pdb=="HD21" && a1=="H" ){*out_ch=  0.40970; return 1;}
    if(terminal==1 && pdb=="HD22" && a1=="H" ){*out_ch=  0.40970; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61630; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57220; return 1;}
    // CASN
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.20800; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.13580; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.22990; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.10230; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.10230; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="C" ){*out_ch=  0.71530; return 1;}
    if(terminal==2 && pdb=="OD1"  && a1=="O" ){*out_ch= -0.60100; return 1;}
    if(terminal==2 && pdb=="ND2"  && a1=="N" ){*out_ch= -0.90840; return 1;}
    if(terminal==2 && pdb=="HD21" && a1=="H" ){*out_ch=  0.41500; return 1;}
    if(terminal==2 && pdb=="HD22" && a1=="H" ){*out_ch=  0.41500; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.80500; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.81470; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.81470; return 1;}
  }
  
  if(res_type==1300){
    // GLN
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.00310; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.08500; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.00360; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.01710; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.01710; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CT"){*out_ch= -0.06450; return 1;}
    if(terminal==0 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.03520; return 1;}
    if(terminal==0 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.03520; return 1;}
    if(terminal==0 && pdb=="CD"   && a1=="C" ){*out_ch=  0.69510; return 1;}
    if(terminal==0 && pdb=="OE1"  && a1=="O" ){*out_ch= -0.60860; return 1;}
    if(terminal==0 && pdb=="NE2"  && a1=="N" ){*out_ch= -0.94070; return 1;}
    if(terminal==0 && pdb=="HE21" && a1=="H" ){*out_ch=  0.42510; return 1;}
    if(terminal==0 && pdb=="HE22" && a1=="H" ){*out_ch=  0.42510; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NGLN
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.14930; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.19960; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.19960; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.19960; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.05360; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.10150; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.06510; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.00500; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.00500; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CT"){*out_ch= -0.09030; return 1;}
    if(terminal==1 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.03310; return 1;}
    if(terminal==1 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.03310; return 1;}
    if(terminal==1 && pdb=="CD"   && a1=="C" ){*out_ch=  0.73540; return 1;}
    if(terminal==1 && pdb=="OE1"  && a1=="O" ){*out_ch= -0.61330; return 1;}
    if(terminal==1 && pdb=="NE2"  && a1=="N" ){*out_ch= -1.00310; return 1;}
    if(terminal==1 && pdb=="HE21" && a1=="H" ){*out_ch=  0.44290; return 1;}
    if(terminal==1 && pdb=="HE22" && a1=="H" ){*out_ch=  0.44290; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CGLN
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.22480; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.12320; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.06640; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.04520; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.04520; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CT"){*out_ch= -0.02100; return 1;}
    if(terminal==2 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.02030; return 1;}
    if(terminal==2 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.02030; return 1;}
    if(terminal==2 && pdb=="CD"   && a1=="C" ){*out_ch=  0.70930; return 1;}
    if(terminal==2 && pdb=="OE1"  && a1=="O" ){*out_ch= -0.60980; return 1;}
    if(terminal==2 && pdb=="NE2"  && a1=="N" ){*out_ch= -0.95740; return 1;}
    if(terminal==2 && pdb=="HE21" && a1=="H" ){*out_ch=  0.43040; return 1;}
    if(terminal==2 && pdb=="HE22" && a1=="H" ){*out_ch=  0.43040; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.77750; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.80420; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.80420; return 1;}
  }
  
  if(res_type==1400){
    // TYR
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.00140; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.08760; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.01520; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.02950; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.02950; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CA"){*out_ch= -0.00110; return 1;}
    if(terminal==0 && pdb=="CD1"  && a1=="CA"){*out_ch= -0.19060; return 1;}
    if(terminal==0 && pdb=="HD1"  && a1=="HA"){*out_ch=  0.16990; return 1;}
    if(terminal==0 && pdb=="CE1"  && a1=="CA"){*out_ch= -0.23410; return 1;}
    if(terminal==0 && pdb=="HE1"  && a1=="HA"){*out_ch=  0.16560; return 1;}
    if(terminal==0 && pdb=="CZ"   && a1=="C" ){*out_ch=  0.32260; return 1;}
    if(terminal==0 && pdb=="OH"   && a1=="OH"){*out_ch= -0.55790; return 1;}
    if(terminal==0 && pdb=="HH"   && a1=="HO"){*out_ch=  0.39920; return 1;}
    if(terminal==0 && pdb=="CE2"  && a1=="CA"){*out_ch= -0.23410; return 1;}
    if(terminal==0 && pdb=="HE2"  && a1=="HA"){*out_ch=  0.16560; return 1;}
    if(terminal==0 && pdb=="CD2"  && a1=="CA"){*out_ch= -0.19060; return 1;}
    if(terminal==0 && pdb=="HD2"  && a1=="HA"){*out_ch=  0.16990; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NTYR
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.19400; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.18730; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.18730; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.18730; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.05700; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.09830; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.06590; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.01020; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.01020; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CA"){*out_ch= -0.02050; return 1;}
    if(terminal==1 && pdb=="CD1"  && a1=="CA"){*out_ch= -0.20020; return 1;}
    if(terminal==1 && pdb=="HD1"  && a1=="HA"){*out_ch=  0.17200; return 1;}
    if(terminal==1 && pdb=="CE1"  && a1=="CA"){*out_ch= -0.22390; return 1;}
    if(terminal==1 && pdb=="HE1"  && a1=="HA"){*out_ch=  0.16500; return 1;}
    if(terminal==1 && pdb=="CZ"   && a1=="C" ){*out_ch=  0.31390; return 1;}
    if(terminal==1 && pdb=="OH"   && a1=="OH"){*out_ch= -0.55780; return 1;}
    if(terminal==1 && pdb=="HH"   && a1=="HO"){*out_ch=  0.40010; return 1;}
    if(terminal==1 && pdb=="CE2"  && a1=="CA"){*out_ch= -0.22390; return 1;}
    if(terminal==1 && pdb=="HE2"  && a1=="HA"){*out_ch=  0.16500; return 1;}
    if(terminal==1 && pdb=="CD2"  && a1=="CA"){*out_ch= -0.20020; return 1;}
    if(terminal==1 && pdb=="HD2"  && a1=="HA"){*out_ch=  0.17200; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CTYR
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.20150; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.10920; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.07520; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.04900; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.04900; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CA"){*out_ch=  0.02430; return 1;}
    if(terminal==2 && pdb=="CD1"  && a1=="CA"){*out_ch= -0.19220; return 1;}
    if(terminal==2 && pdb=="HD1"  && a1=="HA"){*out_ch=  0.17800; return 1;}
    if(terminal==2 && pdb=="CE1"  && a1=="CA"){*out_ch= -0.24580; return 1;}
    if(terminal==2 && pdb=="HE1"  && a1=="HA"){*out_ch=  0.16730; return 1;}
    if(terminal==2 && pdb=="CZ"   && a1=="C" ){*out_ch=  0.33950; return 1;}
    if(terminal==2 && pdb=="OH"   && a1=="OH"){*out_ch= -0.56430; return 1;}
    if(terminal==2 && pdb=="HH"   && a1=="HO"){*out_ch=  0.40170; return 1;}
    if(terminal==2 && pdb=="CE2"  && a1=="CA"){*out_ch= -0.24580; return 1;}
    if(terminal==2 && pdb=="HE2"  && a1=="HA"){*out_ch=  0.16730; return 1;}
    if(terminal==2 && pdb=="CD2"  && a1=="CA"){*out_ch= -0.19220; return 1;}
    if(terminal==2 && pdb=="HD2"  && a1=="HA"){*out_ch=  0.17800; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.78170; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.80700; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.80700; return 1;}
  }
  
  // CYS
  if(res_type==1500){
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch=  0.02130; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.11240; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.12310; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="H1"){*out_ch=  0.11120; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="H1"){*out_ch=  0.11120; return 1;}
    if(terminal==0 && pdb=="SG"   && a1=="SH"){*out_ch= -0.31190; return 1;}
    if(terminal==0 && pdb=="HG"   && a1=="HS"){*out_ch=  0.19330; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NCYS
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.13250; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.20230; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.20230; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.20230; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.09270; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.14110; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch= -0.11950; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="H1"){*out_ch=  0.11880; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="H1"){*out_ch=  0.11880; return 1;}
    if(terminal==1 && pdb=="SG"   && a1=="SH"){*out_ch= -0.32980; return 1;}
    if(terminal==1 && pdb=="HG"   && a1=="HS"){*out_ch=  0.19750; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CCYS
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.16350; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.13960; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.19960; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="H1"){*out_ch=  0.14370; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="H1"){*out_ch=  0.14370; return 1;}
    if(terminal==2 && pdb=="SG"   && a1=="SH"){*out_ch= -0.31020; return 1;}
    if(terminal==2 && pdb=="HG"   && a1=="HS"){*out_ch=  0.20680; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.74970; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.79810; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.79810; return 1;}
  }
  
  if(res_type==1550){
    // CYX
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch=  0.04290; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.07660; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.07900; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="H1"){*out_ch=  0.09100; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="H1"){*out_ch=  0.09100; return 1;}
    if(terminal==0 && pdb=="SG"   && a1=="S" ){*out_ch= -0.10810; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NCYX
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.20690; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.18150; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.18150; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.18150; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.10550; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.09220; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch= -0.02770; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="H1"){*out_ch=  0.06800; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="H1"){*out_ch=  0.06800; return 1;}
    if(terminal==1 && pdb=="SG"   && a1=="S" ){*out_ch= -0.09840; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CCYX
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.13180; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.09380; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.19430; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="H1"){*out_ch=  0.12280; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="H1"){*out_ch=  0.12280; return 1;}
    if(terminal==2 && pdb=="SG"   && a1=="S" ){*out_ch= -0.05290; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.76180; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.80410; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.80410; return 1;}
  }
  
  if(res_type==1600){
    // LYS
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.34790; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27470; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.24000; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.14260; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.00940; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.03620; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.03620; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CT"){*out_ch=  0.01870; return 1;}
    if(terminal==0 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.01030; return 1;}
    if(terminal==0 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.01030; return 1;}
    if(terminal==0 && pdb=="CD"   && a1=="CT"){*out_ch= -0.04790; return 1;}
    if(terminal==0 && pdb=="HD2"  && a1=="HC"){*out_ch=  0.06210; return 1;}
    if(terminal==0 && pdb=="HD3"  && a1=="HC"){*out_ch=  0.06210; return 1;}
    if(terminal==0 && pdb=="CE"   && a1=="CT"){*out_ch= -0.01430; return 1;}
    if(terminal==0 && pdb=="HE2"  && a1=="HP"){*out_ch=  0.11350; return 1;}
    if(terminal==0 && pdb=="HE3"  && a1=="HP"){*out_ch=  0.11350; return 1;}
    if(terminal==0 && pdb=="NZ"   && a1=="N3"){*out_ch= -0.38540; return 1;}
    if(terminal==0 && pdb=="HZ1"  && a1=="H" ){*out_ch=  0.34000; return 1;}
    if(terminal==0 && pdb=="HZ2"  && a1=="H" ){*out_ch=  0.34000; return 1;}
    if(terminal==0 && pdb=="HZ3"  && a1=="H" ){*out_ch=  0.34000; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.73410; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.58940; return 1;}
    // NLYS
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.09660; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.21650; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.21650; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.21650; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch= -0.00150; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.11800; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.02120; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.02830; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.02830; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CT"){*out_ch= -0.00480; return 1;}
    if(terminal==1 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.01210; return 1;}
    if(terminal==1 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.01210; return 1;}
    if(terminal==1 && pdb=="CD"   && a1=="CT"){*out_ch= -0.06080; return 1;}
    if(terminal==1 && pdb=="HD2"  && a1=="HC"){*out_ch=  0.06330; return 1;}
    if(terminal==1 && pdb=="HD3"  && a1=="HC"){*out_ch=  0.06330; return 1;}
    if(terminal==1 && pdb=="CE"   && a1=="CT"){*out_ch= -0.01810; return 1;}
    if(terminal==1 && pdb=="HE2"  && a1=="HP"){*out_ch=  0.11710; return 1;}
    if(terminal==1 && pdb=="HE3"  && a1=="HP"){*out_ch=  0.11710; return 1;}
    if(terminal==1 && pdb=="NZ"   && a1=="N3"){*out_ch= -0.37640; return 1;}
    if(terminal==1 && pdb=="HZ1"  && a1=="H" ){*out_ch=  0.33820; return 1;}
    if(terminal==1 && pdb=="HZ2"  && a1=="H" ){*out_ch=  0.33820; return 1;}
    if(terminal==1 && pdb=="HZ3"  && a1=="H" ){*out_ch=  0.33820; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.72140; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.60130; return 1;}
    // CLYS
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.34810; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.27640; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.29030; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.14380; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.05380; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.04820; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.04820; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CT"){*out_ch=  0.02270; return 1;}
    if(terminal==2 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.01340; return 1;}
    if(terminal==2 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.01340; return 1;}
    if(terminal==2 && pdb=="CD"   && a1=="CT"){*out_ch= -0.03920; return 1;}
    if(terminal==2 && pdb=="HD2"  && a1=="HC"){*out_ch=  0.06110; return 1;}
    if(terminal==2 && pdb=="HD3"  && a1=="HC"){*out_ch=  0.06110; return 1;}
    if(terminal==2 && pdb=="CE"   && a1=="CT"){*out_ch= -0.01760; return 1;}
    if(terminal==2 && pdb=="HE2"  && a1=="HP"){*out_ch=  0.11210; return 1;}
    if(terminal==2 && pdb=="HE3"  && a1=="HP"){*out_ch=  0.11210; return 1;}
    if(terminal==2 && pdb=="NZ"   && a1=="N3"){*out_ch= -0.37410; return 1;}
    if(terminal==2 && pdb=="HZ1"  && a1=="H" ){*out_ch=  0.33740; return 1;}
    if(terminal==2 && pdb=="HZ2"  && a1=="H" ){*out_ch=  0.33740; return 1;}
    if(terminal==2 && pdb=="HZ3"  && a1=="H" ){*out_ch=  0.33740; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.84880; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.82520; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.82520; return 1;}
  }
  
  if(res_type==1700){
    // ARG
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.34790; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27470; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.26370; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.15600; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.00070; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.03270; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.03270; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CT"){*out_ch=  0.03900; return 1;}
    if(terminal==0 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.02850; return 1;}
    if(terminal==0 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.02850; return 1;}
    if(terminal==0 && pdb=="CD"   && a1=="CT"){*out_ch=  0.04860; return 1;}
    if(terminal==0 && pdb=="HD2"  && a1=="H1"){*out_ch=  0.06870; return 1;}
    if(terminal==0 && pdb=="HD3"  && a1=="H1"){*out_ch=  0.06870; return 1;}
    if(terminal==0 && pdb=="NE"   && a1=="N2"){*out_ch= -0.52950; return 1;}
    if(terminal==0 && pdb=="HE"   && a1=="H" ){*out_ch=  0.34560; return 1;}
    if(terminal==0 && pdb=="CZ"   && a1=="CA"){*out_ch=  0.80760; return 1;}
    if(terminal==0 && pdb=="NH1"  && a1=="N2"){*out_ch= -0.86270; return 1;}
    if(terminal==0 && pdb=="HH11" && a1=="H" ){*out_ch=  0.44780; return 1;}
    if(terminal==0 && pdb=="HH12" && a1=="H" ){*out_ch=  0.44780; return 1;}
    if(terminal==0 && pdb=="NH2"  && a1=="N2"){*out_ch= -0.86270; return 1;}
    if(terminal==0 && pdb=="HH21" && a1=="H" ){*out_ch=  0.44780; return 1;}
    if(terminal==0 && pdb=="HH22" && a1=="H" ){*out_ch=  0.44780; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.73410; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.58940; return 1;}
    // NARG
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.13050; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.20830; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.20830; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.20830; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch= -0.02230; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.12420; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.01180; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.02260; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.02260; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CT"){*out_ch=  0.02360; return 1;}
    if(terminal==1 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.03090; return 1;}
    if(terminal==1 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.03090; return 1;}
    if(terminal==1 && pdb=="CD"   && a1=="CT"){*out_ch=  0.09350; return 1;}
    if(terminal==1 && pdb=="HD2"  && a1=="H1"){*out_ch=  0.05270; return 1;}
    if(terminal==1 && pdb=="HD3"  && a1=="H1"){*out_ch=  0.05270; return 1;}
    if(terminal==1 && pdb=="NE"   && a1=="N2"){*out_ch= -0.56500; return 1;}
    if(terminal==1 && pdb=="HE"   && a1=="H" ){*out_ch=  0.35920; return 1;}
    if(terminal==1 && pdb=="CZ"   && a1=="CA"){*out_ch=  0.82810; return 1;}
    if(terminal==1 && pdb=="NH1"  && a1=="N2"){*out_ch= -0.86930; return 1;}
    if(terminal==1 && pdb=="HH11" && a1=="H" ){*out_ch=  0.44940; return 1;}
    if(terminal==1 && pdb=="HH12" && a1=="H" ){*out_ch=  0.44940; return 1;}
    if(terminal==1 && pdb=="NH2"  && a1=="N2"){*out_ch= -0.86930; return 1;}
    if(terminal==1 && pdb=="HH21" && a1=="H" ){*out_ch=  0.44940; return 1;}
    if(terminal==1 && pdb=="HH22" && a1=="H" ){*out_ch=  0.44940; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.72140; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.60130; return 1;}
    // CARG
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.34810; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.27640; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.30680; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.14470; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.03740; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.03710; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.03710; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CT"){*out_ch=  0.07440; return 1;}
    if(terminal==2 && pdb=="HG2"  && a1=="HC"){*out_ch=  0.01850; return 1;}
    if(terminal==2 && pdb=="HG3"  && a1=="HC"){*out_ch=  0.01850; return 1;}
    if(terminal==2 && pdb=="CD"   && a1=="CT"){*out_ch=  0.11140; return 1;}
    if(terminal==2 && pdb=="HD2"  && a1=="H1"){*out_ch=  0.04680; return 1;}
    if(terminal==2 && pdb=="HD3"  && a1=="H1"){*out_ch=  0.04680; return 1;}
    if(terminal==2 && pdb=="NE"   && a1=="N2"){*out_ch= -0.55640; return 1;}
    if(terminal==2 && pdb=="HE"   && a1=="H" ){*out_ch=  0.34790; return 1;}
    if(terminal==2 && pdb=="CZ"   && a1=="CA"){*out_ch=  0.83680; return 1;}
    if(terminal==2 && pdb=="NH1"  && a1=="N2"){*out_ch= -0.87370; return 1;}
    if(terminal==2 && pdb=="HH11" && a1=="H" ){*out_ch=  0.44930; return 1;}
    if(terminal==2 && pdb=="HH12" && a1=="H" ){*out_ch=  0.44930; return 1;}
    if(terminal==2 && pdb=="NH2"  && a1=="N2"){*out_ch= -0.87370; return 1;}
    if(terminal==2 && pdb=="HH21" && a1=="H" ){*out_ch=  0.44930; return 1;}
    if(terminal==2 && pdb=="HH22" && a1=="H" ){*out_ch=  0.44930; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.85570; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.82660; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.82660; return 1;}
  }

  if(res_type==1800 && info_1==0){
    // HID
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch=  0.01880; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.08810; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.04620; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.04020; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.04020; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CC"){*out_ch= -0.02660; return 1;}
    if(terminal==0 && pdb=="ND1"  && a1=="NA"){*out_ch= -0.38110; return 1;}
    if(terminal==0 && pdb=="HD1"  && a1=="H" ){*out_ch=  0.36490; return 1;}
    if(terminal==0 && pdb=="CE1"  && a1=="CR"){*out_ch=  0.20570; return 1;}
    if(terminal==0 && pdb=="HE1"  && a1=="H5"){*out_ch=  0.13920; return 1;}
    if(terminal==0 && pdb=="NE2"  && a1=="NB"){*out_ch= -0.57270; return 1;}
    if(terminal==0 && pdb=="CD2"  && a1=="CV"){*out_ch=  0.12920; return 1;}
    if(terminal==0 && pdb=="HD2"  && a1=="H4"){*out_ch=  0.11470; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NHID
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.15420; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.19630; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.19630; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.19630; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.09640; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.09580; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.02590; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.02090; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.02090; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CC"){*out_ch= -0.03990; return 1;}
    if(terminal==1 && pdb=="ND1"  && a1=="NA"){*out_ch= -0.38190; return 1;}
    if(terminal==1 && pdb=="HD1"  && a1=="H" ){*out_ch=  0.36320; return 1;}
    if(terminal==1 && pdb=="CE1"  && a1=="CR"){*out_ch=  0.21270; return 1;}
    if(terminal==1 && pdb=="HE1"  && a1=="H5"){*out_ch=  0.13850; return 1;}
    if(terminal==1 && pdb=="NE2"  && a1=="NB"){*out_ch= -0.57110; return 1;}
    if(terminal==1 && pdb=="CD2"  && a1=="CV"){*out_ch=  0.10460; return 1;}
    if(terminal==1 && pdb=="HD2"  && a1=="H4"){*out_ch=  0.12990; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CHID
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.17390; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.11000; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.10460; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.05650; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.05650; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CC"){*out_ch=  0.02930; return 1;}
    if(terminal==2 && pdb=="ND1"  && a1=="NA"){*out_ch= -0.38920; return 1;}
    if(terminal==2 && pdb=="HD1"  && a1=="H" ){*out_ch=  0.37550; return 1;}
    if(terminal==2 && pdb=="CE1"  && a1=="CR"){*out_ch=  0.19250; return 1;}
    if(terminal==2 && pdb=="HE1"  && a1=="H5"){*out_ch=  0.14180; return 1;}
    if(terminal==2 && pdb=="NE2"  && a1=="NB"){*out_ch= -0.56290; return 1;}
    if(terminal==2 && pdb=="CD2"  && a1=="CV"){*out_ch=  0.10010; return 1;}
    if(terminal==2 && pdb=="HD2"  && a1=="H4"){*out_ch=  0.12410; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.76150; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.80160; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.80160; return 1;}
  }
  if(res_type==1800 && info_1==1){
    // HIE
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.41570; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27190; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.05810; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.13600; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.00740; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.03670; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.03670; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CC"){*out_ch=  0.18680; return 1;}
    if(terminal==0 && pdb=="ND1"  && a1=="NB"){*out_ch= -0.54320; return 1;}
    if(terminal==0 && pdb=="CE1"  && a1=="CR"){*out_ch=  0.16350; return 1;}
    if(terminal==0 && pdb=="HE1"  && a1=="H5"){*out_ch=  0.14350; return 1;}
    if(terminal==0 && pdb=="NE2"  && a1=="NA"){*out_ch= -0.27950; return 1;}
    if(terminal==0 && pdb=="HE2"  && a1=="H" ){*out_ch=  0.33390; return 1;}
    if(terminal==0 && pdb=="CD2"  && a1=="CW"){*out_ch= -0.22070; return 1;}
    if(terminal==0 && pdb=="HD2"  && a1=="H4"){*out_ch=  0.18620; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.59730; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.56790; return 1;}
    // NHIE 
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.14720; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.20160; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.20160; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.20160; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.02360; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.13800; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.04890; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.02230; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.02230; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CC"){*out_ch=  0.17400; return 1;}
    if(terminal==1 && pdb=="ND1"  && a1=="NB"){*out_ch= -0.55790; return 1;}
    if(terminal==1 && pdb=="CE1"  && a1=="CR"){*out_ch=  0.18040; return 1;}
    if(terminal==1 && pdb=="HE1"  && a1=="H5"){*out_ch=  0.13970; return 1;}
    if(terminal==1 && pdb=="NE2"  && a1=="NA"){*out_ch= -0.27810; return 1;}
    if(terminal==1 && pdb=="HE2"  && a1=="H" ){*out_ch=  0.33240; return 1;}
    if(terminal==1 && pdb=="CD2"  && a1=="CW"){*out_ch= -0.23490; return 1;}
    if(terminal==1 && pdb=="HD2"  && a1=="H4"){*out_ch=  0.19630; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.61230; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.57130; return 1;}
    // CHIE
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.38210; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.26810; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.26990; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.16500; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.10680; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.06200; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.06200; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CC"){*out_ch=  0.27240; return 1;}
    if(terminal==2 && pdb=="ND1"  && a1=="NB"){*out_ch= -0.55170; return 1;}
    if(terminal==2 && pdb=="CE1"  && a1=="CR"){*out_ch=  0.15580; return 1;}
    if(terminal==2 && pdb=="HE1"  && a1=="H5"){*out_ch=  0.14480; return 1;}
    if(terminal==2 && pdb=="NE2"  && a1=="NA"){*out_ch= -0.26700; return 1;}
    if(terminal==2 && pdb=="HE2"  && a1=="H" ){*out_ch=  0.33190; return 1;}
    if(terminal==2 && pdb=="CD2"  && a1=="CW"){*out_ch= -0.25880; return 1;}
    if(terminal==2 && pdb=="HD2"  && a1=="H4"){*out_ch=  0.19570; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.79160; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.80650; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.80650; return 1;}
  }
  
  if(res_type==1800 && info_1==2){
    // HIP
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.34790; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.27470; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch= -0.13540; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.12120; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.04140; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.08100; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.08100; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CC"){*out_ch= -0.00120; return 1;}
    if(terminal==0 && pdb=="ND1"  && a1=="NA"){*out_ch= -0.15130; return 1;}
    if(terminal==0 && pdb=="HD1"  && a1=="H" ){*out_ch=  0.38660; return 1;}
    if(terminal==0 && pdb=="CE1"  && a1=="CR"){*out_ch= -0.01700; return 1;}
    if(terminal==0 && pdb=="HE1"  && a1=="H5"){*out_ch=  0.26810; return 1;}
    if(terminal==0 && pdb=="NE2"  && a1=="NA"){*out_ch= -0.17180; return 1;}
    if(terminal==0 && pdb=="HE2"  && a1=="H" ){*out_ch=  0.39110; return 1;}
    if(terminal==0 && pdb=="CD2"  && a1=="CW"){*out_ch= -0.11410; return 1;}
    if(terminal==0 && pdb=="HD2"  && a1=="H4"){*out_ch=  0.23170; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.73410; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.58940; return 1;}
    // NHIP
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.25600; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.17040; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.17040; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.17040; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.05810; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.10470; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.04840; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.05310; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.05310; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CC"){*out_ch= -0.02360; return 1;}
    if(terminal==1 && pdb=="ND1"  && a1=="NA"){*out_ch= -0.15100; return 1;}
    if(terminal==1 && pdb=="HD1"  && a1=="H" ){*out_ch=  0.38210; return 1;}
    if(terminal==1 && pdb=="CE1"  && a1=="CR"){*out_ch= -0.00110; return 1;}
    if(terminal==1 && pdb=="HE1"  && a1=="H5"){*out_ch=  0.26450; return 1;}
    if(terminal==1 && pdb=="NE2"  && a1=="NA"){*out_ch= -0.17390; return 1;}
    if(terminal==1 && pdb=="HE2"  && a1=="H" ){*out_ch=  0.39210; return 1;}
    if(terminal==1 && pdb=="CD2"  && a1=="CW"){*out_ch= -0.14330; return 1;}
    if(terminal==1 && pdb=="HD2"  && a1=="H4"){*out_ch=  0.24950; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.72140; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.60130; return 1;}
    // CHIP
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.34810; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.27640; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.14450; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.11150; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.08000; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch=  0.08680; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch=  0.08680; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CC"){*out_ch=  0.02980; return 1;}
    if(terminal==2 && pdb=="ND1"  && a1=="NA"){*out_ch= -0.15010; return 1;}
    if(terminal==2 && pdb=="HD1"  && a1=="H" ){*out_ch=  0.38830; return 1;}
    if(terminal==2 && pdb=="CE1"  && a1=="CR"){*out_ch= -0.02510; return 1;}
    if(terminal==2 && pdb=="HE1"  && a1=="H5"){*out_ch=  0.26940; return 1;}
    if(terminal==2 && pdb=="NE2"  && a1=="NA"){*out_ch= -0.16830; return 1;}
    if(terminal==2 && pdb=="HE2"  && a1=="H" ){*out_ch=  0.39130; return 1;}
    if(terminal==2 && pdb=="CD2"  && a1=="CW"){*out_ch= -0.12560; return 1;}
    if(terminal==2 && pdb=="HD2"  && a1=="H4"){*out_ch=  0.23360; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.80320; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.81770; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.81770; return 1;}
  }
  
  if(res_type==1900){
    // ASP
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.51630; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.29360; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch=  0.03810; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.08800; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch= -0.03030; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch= -0.01220; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch= -0.01220; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="C" ){*out_ch=  0.79940; return 1;}
    if(terminal==0 && pdb=="OD1"  && a1=="O2"){*out_ch= -0.80140; return 1;}
    if(terminal==0 && pdb=="OD2"  && a1=="O2"){*out_ch= -0.80140; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.53660; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.58190; return 1;}
    // NASP
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.07820; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.22000; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.22000; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.22000; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.02920; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.11410; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch= -0.02350; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch= -0.01690; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch= -0.01690; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="C" ){*out_ch=  0.81940; return 1;}
    if(terminal==1 && pdb=="OD1"  && a1=="O2"){*out_ch= -0.80840; return 1;}
    if(terminal==1 && pdb=="OD2"  && a1=="O2"){*out_ch= -0.80840; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.56210; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.58890; return 1;}
    // CASP
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.51920; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.30550; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.18170; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.10460; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch= -0.06770; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch= -0.02120; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch= -0.02120; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="C" ){*out_ch=  0.88510; return 1;}
    if(terminal==2 && pdb=="OD1"  && a1=="O2"){*out_ch= -0.81620; return 1;}
    if(terminal==2 && pdb=="OD2"  && a1=="O2"){*out_ch= -0.81620; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.72560; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.78870; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.78870; return 1;}
  }
  
  if(res_type==2000){
    // GLU
    if(terminal==0 && pdb=="N"    && a1=="N" ){*out_ch= -0.51630; return 1;}
    if(terminal==0 && pdb=="H"    && a1=="H" ){*out_ch=  0.29360; return 1;}
    if(terminal==0 && pdb=="CA"   && a1=="CT"){*out_ch=  0.03970; return 1;}
    if(terminal==0 && pdb=="HA"   && a1=="H1"){*out_ch=  0.11050; return 1;}
    if(terminal==0 && pdb=="CB"   && a1=="CT"){*out_ch=  0.05600; return 1;}
    if(terminal==0 && pdb=="HB2"  && a1=="HC"){*out_ch= -0.01730; return 1;}
    if(terminal==0 && pdb=="HB3"  && a1=="HC"){*out_ch= -0.01730; return 1;}
    if(terminal==0 && pdb=="CG"   && a1=="CT"){*out_ch=  0.01360; return 1;}
    if(terminal==0 && pdb=="HG2"  && a1=="HC"){*out_ch= -0.04250; return 1;}
    if(terminal==0 && pdb=="HG3"  && a1=="HC"){*out_ch= -0.04250; return 1;}
    if(terminal==0 && pdb=="CD"   && a1=="C" ){*out_ch=  0.80540; return 1;}
    if(terminal==0 && pdb=="OE1"  && a1=="O2"){*out_ch= -0.81880; return 1;}
    if(terminal==0 && pdb=="OE2"  && a1=="O2"){*out_ch= -0.81880; return 1;}
    if(terminal==0 && pdb=="C"    && a1=="C" ){*out_ch=  0.53660; return 1;}
    if(terminal==0 && pdb=="O"    && a1=="O" ){*out_ch= -0.58190; return 1;}
    // NGLU
    if(terminal==1 && pdb=="N"    && a1=="N3"){*out_ch=  0.00170; return 1;}
    if(terminal==1 && pdb=="H1"   && a1=="H" ){*out_ch=  0.23910; return 1;}
    if(terminal==1 && pdb=="H2"   && a1=="H" ){*out_ch=  0.23910; return 1;}
    if(terminal==1 && pdb=="H3"   && a1=="H" ){*out_ch=  0.23910; return 1;}
    if(terminal==1 && pdb=="CA"   && a1=="CT"){*out_ch=  0.05880; return 1;}
    if(terminal==1 && pdb=="HA"   && a1=="HP"){*out_ch=  0.12020; return 1;}
    if(terminal==1 && pdb=="CB"   && a1=="CT"){*out_ch=  0.09090; return 1;}
    if(terminal==1 && pdb=="HB2"  && a1=="HC"){*out_ch= -0.02320; return 1;}
    if(terminal==1 && pdb=="HB3"  && a1=="HC"){*out_ch= -0.02320; return 1;}
    if(terminal==1 && pdb=="CG"   && a1=="CT"){*out_ch= -0.02360; return 1;}
    if(terminal==1 && pdb=="HG2"  && a1=="HC"){*out_ch= -0.03150; return 1;}
    if(terminal==1 && pdb=="HG3"  && a1=="HC"){*out_ch= -0.03150; return 1;}
    if(terminal==1 && pdb=="CD"   && a1=="C" ){*out_ch=  0.80870; return 1;}
    if(terminal==1 && pdb=="OE1"  && a1=="O2"){*out_ch= -0.81890; return 1;}
    if(terminal==1 && pdb=="OE2"  && a1=="O2"){*out_ch= -0.81890; return 1;}
    if(terminal==1 && pdb=="C"    && a1=="C" ){*out_ch=  0.56210; return 1;}
    if(terminal==1 && pdb=="O"    && a1=="O" ){*out_ch= -0.58890; return 1;}
    // CGLU
    if(terminal==2 && pdb=="N"    && a1=="N" ){*out_ch= -0.51920; return 1;}
    if(terminal==2 && pdb=="H"    && a1=="H" ){*out_ch=  0.30550; return 1;}
    if(terminal==2 && pdb=="CA"   && a1=="CT"){*out_ch= -0.20590; return 1;}
    if(terminal==2 && pdb=="HA"   && a1=="H1"){*out_ch=  0.13990; return 1;}
    if(terminal==2 && pdb=="CB"   && a1=="CT"){*out_ch=  0.00710; return 1;}
    if(terminal==2 && pdb=="HB2"  && a1=="HC"){*out_ch= -0.00780; return 1;}
    if(terminal==2 && pdb=="HB3"  && a1=="HC"){*out_ch= -0.00780; return 1;}
    if(terminal==2 && pdb=="CG"   && a1=="CT"){*out_ch=  0.06750; return 1;}
    if(terminal==2 && pdb=="HG2"  && a1=="HC"){*out_ch= -0.05480; return 1;}
    if(terminal==2 && pdb=="HG3"  && a1=="HC"){*out_ch= -0.05480; return 1;}
    if(terminal==2 && pdb=="CD"   && a1=="C" ){*out_ch=  0.81830; return 1;}
    if(terminal==2 && pdb=="OE1"  && a1=="O2"){*out_ch= -0.82200; return 1;}
    if(terminal==2 && pdb=="OE2"  && a1=="O2"){*out_ch= -0.82200; return 1;}
    if(terminal==2 && pdb=="C"    && a1=="C" ){*out_ch=  0.74200; return 1;}
    if(terminal==2 && pdb=="O"    && a1=="O2"){*out_ch= -0.79300; return 1;}
    if(terminal==2 && pdb=="OXT"  && a1=="O2"){*out_ch= -0.79300; return 1;}
  }
    
  if(a1=="OW"){*out_ch= -0.834000; return 1;}
  if(a1=="HW"){*out_ch=  0.417000; return 1;}
  if(a1=="HW"){*out_ch=  0.417000; return 1;}
  
  cout << "amber_force_field_charge , (a1,pdb,res_type,terminal)=(" ;
  cout << a1       << " ";
  cout << pdb      << " ";
  cout << res_type << " ";
  cout << terminal << ")";
  cout << " is not defined !" << endl ;

  return -1;
}

int ForceField::amber_force_field_vdw(const string   a1,
				      double       * out_r,
				      double       * out_e)
{
  // (parm99.dat)
  if(a1=="N" ){*out_r=  1.8240; *out_e=  0.1700; return 1;}
  if(a1=="H" ){*out_r=  0.6000; *out_e=  0.0157; return 1;}
  if(a1=="CT"){*out_r=  1.9080; *out_e=  0.1094; return 1;}
  if(a1=="H1"){*out_r=  1.3870; *out_e=  0.0157; return 1;}
  if(a1=="C" ){*out_r=  1.9080; *out_e=  0.0860; return 1;}
  if(a1=="O" ){*out_r=  1.6612; *out_e=  0.2100; return 1;}
  if(a1=="O2"){*out_r=  1.6612; *out_e=  0.2100; return 1;}
  if(a1=="N3"){*out_r=  1.8240; *out_e=  0.1700; return 1;}
  if(a1=="HP"){*out_r=  1.1000; *out_e=  0.0157; return 1;}
  if(a1=="HC"){*out_r=  1.4870; *out_e=  0.0157; return 1;}
  if(a1=="S" ){*out_r=  2.0000; *out_e=  0.2500; return 1;}
  if(a1=="HA"){*out_r=  1.4590; *out_e=  0.0150; return 1;}
  if(a1=="CA"){*out_r=  1.9080; *out_e=  0.0860; return 1;} // same as C*
  if(a1=="C*"){*out_r=  1.9080; *out_e=  0.0860; return 1;} // same as C*
  if(a1=="CW"){*out_r=  1.9080; *out_e=  0.0860; return 1;} // same as C*
  if(a1=="CN"){*out_r=  1.9080; *out_e=  0.0860; return 1;} // same as C*
  if(a1=="CB"){*out_r=  1.9080; *out_e=  0.0860; return 1;} // same as C*
  if(a1=="H4"){*out_r=  1.4090; *out_e=  0.0150; return 1;}
  if(a1=="NA"){*out_r=  1.8240; *out_e=  0.1700; return 1;}
  if(a1=="HO"){*out_r=  0.0000; *out_e=  0.0000; return 1;}
  if(a1=="OH"){*out_r=  1.7210; *out_e=  0.2104; return 1;} 
  if(a1=="SH"){*out_r=  2.0000; *out_e=  0.2500; return 1;}
  if(a1=="HS"){*out_r=  0.6000; *out_e=  0.0157; return 1;}
  if(a1=="N2"){*out_r=  1.8240; *out_e=  0.1700; return 1;} // same as N
  if(a1=="CC"){*out_r=  1.9080; *out_e=  0.0860; return 1;} // same as C*
  if(a1=="CR"){*out_r=  1.9080; *out_e=  0.0860; return 1;} // same as C*
  if(a1=="H5"){*out_r=  1.3590; *out_e=  0.0150; return 1;}
  if(a1=="NB"){*out_r=  1.8240; *out_e=  0.1700; return 1;} // same as N
  if(a1=="CV"){*out_r=  1.9080; *out_e=  0.0860; return 1;} // same as C*
  if(a1=="OW"){*out_r=  1.7683; *out_e=  0.1520; return 1;} // TIP3P water model
  if(a1=="HW"){*out_r=  0.0000; *out_e=  0.0000; return 1;} // TIP3P water model

  cout << "amber_force_field_vdw , " << a1 << " is not defined!" << endl ;

  return -1;
}

void ForceField::amber_force_field_get_atom_info ( const Molecule&   m           ,
                                                   const int         iatom       ,
                                                   int             * ptyp_       ,
                                                   string          * pdb_        ,
                                                   int             * ibu_        ,
                                                   int             * res_type_   ,
                                                   int             * t_          ,
                                                   int             * his_        ,
                                                   string          * amber_      ) //const 
{
  int a,ptyp,ibu,res_type,t,his,tc,tn,his1814,his1816;
  string pdb,amber;

  a=iatom;
  ptyp=m.get_atom(a).get_bio1();
  pdb=BioUnit::pdb_tag(ptyp);
  ibu=m.get_atom(a).get_bio3();
  if(ibu>=0){res_type=m.get_biounit(ibu).get_type();}
  tc=0;
  if(ibu>=0){
    if(m.get_biounit(ibu).get_terminal_c()>0){
      tc=1;
      if(m.get_biounit(ibu).get_terminal_c()==6){tc=6;} // 20150120
    }
  }
  tn=0;
  if(ibu>=0){
    if(m.get_biounit(ibu).get_terminal_n()>0){
      tn=1; 
      if(m.get_biounit(ibu).get_terminal_n()==6){tn=6;} // 20150120
    }
  }
  t=0;
  if(ibu>=0){
    if(tn>0) {t= 1;}
    if(tc>0) {t= 2;}
    if(tn==6){t=61;} // ACE , 20150120
    if(tc==6){t=62;} // NME , 20150120
  }
  if(ibu>=0 && res_type==1800){
    his1814=m.get_biounit(ibu).get_h1814();
    his1816=m.get_biounit(ibu).get_h1816();
    if     (his1814>=0 && his1816< 0) {his=0;} // HID
    else if(his1814 <0 && his1816>=0) {his=1;} // HIE
    else if(his1814>=0 && his1816>=0) {his=2;} // HIP
    else {
      cout << "error in his check!! @ print dihedral type1 , his1814 , his1816 = " << his1814 << "," << his1816 << endl ;
      exit(1);
    }
    amber=ForceField::amber_force_field_get_atom_type(ptyp,t,his);
  } else if(ibu>=0 && res_type==1550){
    amber=ForceField::amber_force_field_get_atom_type(ptyp,t,1);
  } else {
    amber=ForceField::amber_force_field_get_atom_type(ptyp,t);
  }
  *ptyp_     = ptyp;
  *pdb_      = pdb;
  *ibu_      = ibu;
  *res_type_ = res_type;
  *t_        = t;
  *his_      = his;
  *amber_    = amber;
}

void ForceField::amber_force_field_make_param_atom ( const Molecule& m )
{
  int a1,ptyp1,ibu1,tc1,tn1,t1,res_type1,his1;
  int out_n,natom,chk,chkchk1,chkchk2,chkchk3,chkchk4,ia1,ia2,n_sum,ang1,ang2,ang3,dih1,dih2,dih3,dih4;
  double charge,vdw_r,vdw_e;
  string amber1,pdb1;
  int * out;

  out = new int [m.get_natom()+1];
  
  for(int i=0;i<m.get_natom();i++){

    a1=i; amber_force_field_get_atom_info (m,a1,&ptyp1,&pdb1,&ibu1,&res_type1,&t1,&his1,&amber1);
    
    if(amber1 != "**"){

      //////////////////
      ///// charge /////
      //////////////////

      chk=ForceField::amber_force_field_charge (amber1,pdb1,res_type1,t1,&charge,his1);
      if(chk<0){
        cout << "error ForceField::amber_force_field_make_param_atom (1)" ;
        cout.width(6); cout << amber1 ;
        cout.width(6); cout << res_type1 ;
        cout.width(6); cout << t1 ;
        cout << endl ;
        exit(1);
      }

      /////////////////////////
      ///// van der Waals /////
      /////////////////////////

      chk=ForceField::amber_force_field_vdw(amber1,&vdw_r,&vdw_e);
      if(chk<0){
        cout << "error ForceField::amber_force_field_make_param_atom (2)" ;
        cout.width(6); cout << amber1 ;
        cout.width(6); cout << res_type1 ;
        cout.width(6); cout << t1 ;
        cout << endl ;
        exit(1);
      }
    }

    //////////////////////////
    ///// excluded atoms /////
    //////////////////////////

    out_n=0; // number of excluded atom !! 

    for(int j=0;j<m.get_nbond();j++){
      ia1=m.get_bond(j).get_iatom1();
      ia2=m.get_bond(j).get_iatom2();
      chk=0;
      if(ia1==i){chk=1;}
      if(ia2==i){chk=1;}
      if(chk==1){
	chkchk1=0; for(int k=0;k<out_n;k++){if(out[k]==ia1 || i==ia1){chkchk1=1;break;}}
	chkchk2=0; for(int k=0;k<out_n;k++){if(out[k]==ia2 || i==ia2){chkchk2=1;break;}}
	if(chkchk1==0 && ia1>i){out[out_n]=ia1;out_n++;}
	if(chkchk2==0 && ia2>i){out[out_n]=ia2;out_n++;}
      }
    }
    for(int j=0;j<m.get_nangle();j++){
      chk=0;
      ang1=m.get_angle_iatom1(j);
      ang2=m.get_angle_iatom2(j);
      ang3=m.get_angle_iatom3(j);
      if(ang1==i){chk=1;}
      if(ang2==i){chk=1;}
      if(ang3==i){chk=1;}
      if(chk==1){
	chkchk1=0; for(int k=0;k<out_n;k++){if(out[k]==ang1 || i==ang1){chkchk1=1;break;}}
	chkchk2=0; for(int k=0;k<out_n;k++){if(out[k]==ang2 || i==ang2){chkchk2=1;break;}}
	chkchk3=0; for(int k=0;k<out_n;k++){if(out[k]==ang3 || i==ang2){chkchk3=1;break;}}
	if(chkchk1==0 && ang1>i){out[out_n]=ang1;out_n++;}
	if(chkchk2==0 && ang2>i){out[out_n]=ang2;out_n++;}
	if(chkchk3==0 && ang3>i){out[out_n]=ang3;out_n++;}
      }
    }
    for(int j=0;j<m.get_ndihedral();j++){
      chk=0;
      dih1=m.get_dihedral_iatom1(j);
      dih2=m.get_dihedral_iatom2(j);
      dih3=m.get_dihedral_iatom3(j);
      dih4=m.get_dihedral_iatom4(j);
      if(dih1==i){chk=1;}
      if(dih2==i){chk=1;}
      if(dih3==i){chk=1;}
      if(dih4==i){chk=1;}
      if(chk==1){
	chkchk1=0; for(int k=0;k<out_n;k++){if(out[k]==dih1 || i==dih1){chkchk1=1;break;}}
	chkchk2=0; for(int k=0;k<out_n;k++){if(out[k]==dih2 || i==dih2){chkchk2=1;break;}}
	chkchk3=0; for(int k=0;k<out_n;k++){if(out[k]==dih3 || i==dih3){chkchk3=1;break;}}
	chkchk4=0; for(int k=0;k<out_n;k++){if(out[k]==dih4 || i==dih4){chkchk4=1;break;}}
	if(chkchk1==0 && dih1>i){out[out_n]=dih1;out_n++;}
	if(chkchk2==0 && dih2>i){out[out_n]=dih2;out_n++;}
	if(chkchk3==0 && dih3>i){out[out_n]=dih3;out_n++;}
	if(chkchk4==0 && dih4>i){out[out_n]=dih4;out_n++;}
      }
    }
    
    /////////////////////////
    ///// add parameter /////
    /////////////////////////
    
    push_back_atm(a1,ptyp1,amber1,pdb1,charge,vdw_r,vdw_e,out_n,out);    
  }
  delete [] out;
}

void ForceField::amber_force_field_make_param_bond( const Molecule& m )
{
  int a1,ptyp1,ibu1,tc1,tn1,t1,res_type1,his1; string amber1,pdb1;
  int a2,ptyp2,ibu2,tc2,tn2,t2,res_type2,his2; string amber2,pdb2;
  int chk;
  double coeff,eq;

  for(int i=0;i<m.get_nbond();i++){

    a1=m.get_bond(i).get_iatom1(); amber_force_field_get_atom_info (m,a1,&ptyp1,&pdb1,&ibu1,&res_type1,&t1,&his1,&amber1);
    a2=m.get_bond(i).get_iatom2(); amber_force_field_get_atom_info (m,a2,&ptyp2,&pdb2,&ibu2,&res_type2,&t2,&his2,&amber2);

    if(amber1 != "**" && amber2 != "**" ){
      chk=amber_force_field_bond(amber1,amber2,&coeff,&eq);
      if(chk<0){
	cout << "error ForceField::amber_force_field_make_param_bond" ;
	cout.width(6); cout << amber1 ;
	cout.width(6); cout << amber2 ;
	cout << endl ;
	exit(1);
      }
    }
    push_back_bnd(a1,a2,coeff,eq);
  }
}

void ForceField::amber_force_field_make_param_angle( const Molecule& m )
{
  int a1,ptyp1,ibu1,tc1,tn1,t1,res_type1,his1; string amber1,pdb1;
  int a2,ptyp2,ibu2,tc2,tn2,t2,res_type2,his2; string amber2,pdb2;
  int a3,ptyp3,ibu3,tc3,tn3,t3,res_type3,his3; string amber3,pdb3;
  int chk;
  double coeff;
  double eq;

  for(int i=0;i<m.get_nangle();i++){

    a1=m.get_angle_iatom1(i); amber_force_field_get_atom_info (m,a1,&ptyp1,&pdb1,&ibu1,&res_type1,&t1,&his1,&amber1);
    a2=m.get_angle_iatom2(i); amber_force_field_get_atom_info (m,a2,&ptyp2,&pdb2,&ibu2,&res_type2,&t2,&his2,&amber2);
    a3=m.get_angle_iatom3(i); amber_force_field_get_atom_info (m,a3,&ptyp3,&pdb3,&ibu3,&res_type3,&t3,&his3,&amber3);

    if(amber1 != "**" && amber2 != "**" && amber3 != "**" ){
      chk=amber_force_field_angle(amber1,amber2,amber3,&coeff,&eq);
      if(chk<0){
	cout << "error ForceField::amber_force_field_make_param_angle" << endl ;
	cout.width( 6); cout << i ;
	cout.width( 6); cout << amber1 ;
	cout.width( 6); cout << amber2 ;
	cout.width( 6); cout << amber3 ;
	cout.width( 6); cout << a1 ;
	cout.width( 6); cout << a2 ;
	cout.width( 6); cout << a3 ;
	cout << endl ;
	exit(1);
      }
    }
    push_back_ang(a1,a2,a3,coeff,eq);
  }
}

void ForceField::amber_force_field_make_param_dihedral ( const Molecule& m )
{
  int a1,ptyp1,ibu1,tc1,tn1,t1,res_type1,his1; string amber1,pdb1;
  int a2,ptyp2,ibu2,tc2,tn2,t2,res_type2,his2; string amber2,pdb2;
  int a3,ptyp3,ibu3,tc3,tn3,t3,res_type3,his3; string amber3,pdb3;
  int a4,ptyp4,ibu4,tc4,tn4,t4,res_type4,his4; string amber4,pdb4;
  int chk,improper,n;
  double coeff[16];
  double div[16];
  double eq[16];
  double phase[16];

  for(int i=0;i<m.get_ndihedral();i++){

    a1=m.get_dihedral_iatom1(i); amber_force_field_get_atom_info (m,a1,&ptyp1,&pdb1,&ibu1,&res_type1,&t1,&his1,&amber1);
    a2=m.get_dihedral_iatom2(i); amber_force_field_get_atom_info (m,a2,&ptyp2,&pdb1,&ibu2,&res_type2,&t2,&his2,&amber2);
    a3=m.get_dihedral_iatom3(i); amber_force_field_get_atom_info (m,a3,&ptyp3,&pdb1,&ibu3,&res_type3,&t3,&his3,&amber3);
    a4=m.get_dihedral_iatom4(i); amber_force_field_get_atom_info (m,a4,&ptyp4,&pdb1,&ibu4,&res_type4,&t4,&his4,&amber4);

    improper=m.get_dihedral_chk_improper(i);
    n=0;
    if(amber1 != "**" && amber2 != "**" && amber3 != "**" && amber4 != "**" ){
      chk=amber_force_field_dihedral(amber1,amber2,amber3,amber4,improper,&n,coeff,div,eq,phase);
      if(chk<0){
	cout << "error!! in force_field_dihedral" << endl ;
	cout.width( 6); cout << amber1 ;
	cout.width( 6); cout << amber2 ;
	cout.width( 6); cout << amber3 ;
	cout.width( 6); cout << amber4 ;
	cout << endl ;
        exit(1);
      }
    }
    for(int j=0;j<n;j++){
      push_back_dih(a1,a2,a3,a4,coeff[j],div[j],eq[j],phase[j],improper);
    }
  }
}

void ForceField::amber_force_field_make_14_pair ( const Molecule& m )
{
  int ia1,ia2,ang1,ang2,bond1,bond2,chk,chkchk,chkchkchk;

  for(int i=0;i<m.get_ndihedral();i++){
    if(m.get_dihedral_chk_improper(i)==0){
      ia1=m.get_dihedral_iatom1(i);
      ia2=m.get_dihedral_iatom4(i);
      chk=0;
      for(int j=0;j<(np14);j++){
        if(ia1==p14_1[j] && ia2==p14_2[j]){chk=1;}
        if(ia2==p14_1[j] && ia1==p14_2[j]){chk=1;}
      }
      if(chk==0){
        chkchk=0;
        for(int j=0;j<m.get_nangle();j++){
          ang1=m.get_angle_iatom1(j);
          ang2=m.get_angle_iatom3(j);
          if(ia1==ang1 && ia2==ang2){chkchk=1;}
          if(ia1==ang2 && ia2==ang1){chkchk=1;}
        }
        if(chkchk==0){
          chkchkchk=0;
          for(int j=0;j<m.get_nbond();j++){
            bond1=m.get_bond(j).get_iatom1();
            bond2=m.get_bond(j).get_iatom2();
            if(ia1==bond1 && ia2==bond2){chkchkchk=1;}
            if(ia1==bond2 && ia2==bond1){chkchkchk=1;}
          }
          if(chkchkchk==0){
            push_back_p14(ia1,ia2);
          } else {
            //cout << "find 1-2 pair" << endl ;
          }
        } else {
          //cout << "find 1-3 pair" << endl ;
        }
      }
    }
  }  
}

int ForceField::d4_(const string a1,
		    const string a2,
		    const string a3,
		    const string a4,
		    const string b1,
		    const string b2,
		    const string b3,
		    const string b4,
		    const int    improper)
{
  // improper==0 : not improper
  // improper==1 ; improper

  // ABCD
  if(a1!="X" && a2!="X" && a3!="X" && a4!="X") {
    if(a1==b1 && a2==b2 && a3==b3  && a4==b4 ) {                 return 1;}
    if(a1==b4 && a2==b3 && a3==b2  && a4==b1 ) {if(improper!=1) {return 1;}}
    return 0;
  }

  // XBCD
  if(a1=="X" && a2!="X" && a3!="X" && a4!="X") {
    if(          a2==b2 && a3==b3  && a4==b4 ) {                 return 1;}
    if(          a2==b3 && a3==b2  && a4==b1 ) {if(improper!=1) {return 1;}}
    return 0;
  }

  // XBCX
  if(a1=="X" && a2!="X" && a3!="X" && a4=="X") {
    if(          a2==b2 && a3==b3            ) {                 return 1;}
    if(          a2==b3 && a3==b2            ) {if(improper!=1) {return 1;}} 
    return 0;
  }
  
  // XXCD
  if(a1=="X" && a2=="X" && a3!="X" && a4!="X") {
    if(                    a3==b3 && a4==b4  ) {                 return 1;}
    if(                    a3==b2 && a4==b1  ) {if(improper!=1) {return 1;}}
    return 0;
  }

  cout << "d4_ , not defined ABCD type , (a1,a2,a3,a4)=";
  cout.width(6); cout << a1 ;
  cout.width(6); cout << a2 ;
  cout.width(6); cout << a3 ;
  cout.width(6); cout << a4 ;
  cout << endl ;
  return -1;
}

void ForceField::write_amber_force_field_paics( const string filename ) const 
{
  ofstream fout(filename.c_str(), ios_base::out);  
  fout << get_amber_force_field_paics() ;
  fout.close();
}

string ForceField::get_amber_force_field_paics() const 
{
  stringstream ss;
  int n=0;
  int n_sum=0;

  ss.width(8); ss << natm;
  ss.width(8); ss << nbnd;
  ss.width(8); ss << nang;
  ss.width(8); ss << ndih;
  ss.width(8); ss << np14;
  ss << endl ;

  ss << "ATOM_PARAMETER" << endl ;
  for(int i=0;i<natm;i++){
    ss.setf(ios::fixed,ios::floatfield);
    ss.precision(6);
    ss.width( 6); ss << atm1[i]+1;
    ss.width( 6); ss << atm_paics[i];
    ss.width( 6); ss << atm_amber[i];
    ss.width( 6); ss << iTrim(atm_pdb[i].c_str());
    ss.width(12); ss << atm_chg[i];
    ss.width(12); ss << atm_vdw_r[i];
    ss.width(12); ss << atm_vdw_e[i];
    ss << endl ;
  }

  ss << "BOND_PARAMETER" << endl ;
  for(int i=0;i<nbnd;i++){
    ss.setf(ios::fixed,ios::floatfield);
    ss.precision(6);
    ss.width( 6); ss << bnd1[i]+1;
    ss.width( 6); ss << bnd2[i]+1;
    ss.width(12); ss << bnd_coeff[i];
    ss.width(12); ss << bnd_eq[i];
    ss << endl ;
  }

  ss << "ANGLE_PARAMETER" << endl ;
  for(int i=0;i<nang;i++){
    ss.setf(ios::fixed,ios::floatfield);
    ss.precision(6);
    ss.width( 6); ss << ang1[i]+1;
    ss.width( 6); ss << ang2[i]+1;
    ss.width( 6); ss << ang3[i]+1;
    ss.width(12); ss << ang_coeff[i];
    ss.width(12); ss << ang_eq[i];
    ss << endl ;
  }

  ss << "DIHEDRAL_PARAMETER" << endl ;
  for(int i=0;i<ndih;i++){
    ss.setf(ios::fixed,ios::floatfield);
    ss.precision(6);
    ss.width( 6); ss << dih1[i]+1;
    ss.width( 6); ss << dih2[i]+1;
    ss.width( 6); ss << dih3[i]+1;
    ss.width( 6); ss << dih4[i]+1;
    ss.width( 6); ss << dih_improper[i] ;
    ss.width(12); ss << dih_coeff[i];
    ss.width(12); ss << dih_eq[i];
    ss.width(12); ss << dih_div[i];
    ss.width(12); ss << dih_phase[i];
    ss << endl ;
  }

  ss << "EXCLUDED_ATOM_NUBMER" << endl ;
  for(int i=0;i<natm;i++){
    ss.width(6); ss << atm_excluded_n[i] ;
    if((i+1)%10==0 && (i+1)!=natm){
      ss << endl ;
    }
  }
  ss << endl ;

  ss << "EXCLUDED_ATOM" << endl ;
  for(int i=0;i<natm;i++){
    n_sum=n_sum+atm_excluded_n[i];
  }
  for(int i=0;i<natm;i++){
    for(int j=0;j<atm_excluded_n[i];j++){
      ss.width(6); ss << atm_excluded[i][j] ;
      n++;
      if(n%10==0 && n!=n_sum){
	ss << endl ;
      }
    }
  }
  ss << endl ;

  ss << "14_PAIR" << endl ;
  for(int i=0;i<np14;i++){
    ss.width(6); ss << p14_1[i];
    ss.width(6); ss << p14_2[i];
    if( (i+1)%5==0 && (i+1)!=np14){
      ss << endl ;
    }
  }
  ss << endl ;

  return ss.str();
}
