#include <fstream>
#include <iostream>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <random>
#include <sstream>
#include "Molecule.h"
#include "docking.h"

double docking_ang_min_rmsd(const int      docking_rot_bond_n,
                            const int      n_ang_1,
                            const double * ang_1,
                            const double * ang_2)
{
  double rmsd,ang_diff;
  double ang_min_rmsd=999999999.0;
  for(int k=0;k<n_ang_1;k++){
    rmsd=0.0;
    for(int l=0;l<docking_rot_bond_n;l++){
      ang_diff=ang_1[k*docking_rot_bond_n+l]-ang_2[0*docking_rot_bond_n+l];
      if(ang_diff<-180.0){ang_diff=ang_diff+360.0;}
      if(ang_diff> 180.0){ang_diff=ang_diff-360.0;}
      rmsd=rmsd+fabs(ang_diff);
    }
    rmsd=rmsd/(double)docking_rot_bond_n;
    if(rmsd<ang_min_rmsd){
      ang_min_rmsd=rmsd;
    }
  }
  return ang_min_rmsd;
}

double docking_ang_min_rmsd_main(const int      docking_rot_bond_n,
                                 const int      docking_rot_bond_main_n,
                                 const int    * docking_rot_bond_main,
                                 const int      n_ang_1,
                                 const double * ang_1,
                                 const double * ang_2)
{
  double rmsd,ang_diff;
  double ang_min_rmsd=999999999.0;
  for(int k=0;k<n_ang_1;k++){
    rmsd=0.0;
    for(int l=0;l<docking_rot_bond_main_n;l++){
      ang_diff=ang_1[k*docking_rot_bond_n+docking_rot_bond_main[l]]-ang_2[0*docking_rot_bond_n+docking_rot_bond_main[l]];
      if(ang_diff<-180.0){ang_diff=ang_diff+360.0;}
      if(ang_diff> 180.0){ang_diff=ang_diff-360.0;}
      rmsd=rmsd+fabs(ang_diff);
    }
    rmsd=rmsd/(double)docking_rot_bond_main_n;
    if(rmsd<ang_min_rmsd){
      ang_min_rmsd=rmsd;
    }
  }
  return ang_min_rmsd;
}

double docking_pos_min_rmsd(const int      n_pos_1,
                            const double * pos_1,
                            const double * pos_2)
{
  double rmsd,pos_diff,x2,y2,z2,x1,y1,z1;
  double pos_min_rmsd=999999999.0;
  x2=pos_2[0];
  y2=pos_2[1];
  z2=pos_2[2];
  for(int k=0;k<n_pos_1;k++){
    x1=pos_1[3*k+0];
    y1=pos_1[3*k+1];
    z1=pos_1[3*k+2];
    rmsd=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
    if(rmsd<pos_min_rmsd){
      pos_min_rmsd=rmsd;
    }
  }
  return pos_min_rmsd;
}

void docking_get_ligand_info(const string& file_1,
                             const string& file_2,
                             int         * atom_n,
                             int         * type_n,
                             int         * bond_n,
                             int         * rot_bond_n,
                             int         * rot_bond_main_n,
                             int         * pair_n)
{
  stringstream ss1;
  stringstream ss2;
  string sbuf;
  string type[128];
  int n,chk;

  n=0;
  ss1 << file_1;
  ss1 >> sbuf;
  ss1 >> *atom_n;
  for(int i=0;i<(*atom_n);i++){
    ss1 >> sbuf ;
    ss1 >> sbuf ;
    ss1 >> sbuf ;
    chk=0;
    for(int i=0;i<n;i++){
      if(type[i]==sbuf){
        chk=1;
        break;
      }
    }
    if(chk==0){
      type[n]=sbuf;
      ss1 >> sbuf ;
      ss1 >> sbuf ;
      ss1 >> sbuf ;
      ss1 >> sbuf ;
      ss1 >> sbuf ;
      ss1 >> sbuf ;
      n++;
    } else {
      ss1 >> sbuf ;
      ss1 >> sbuf ;
      ss1 >> sbuf ;
      ss1 >> sbuf ;
      ss1 >> sbuf ;
      ss1 >> sbuf ;
    }
  }
  (*type_n)=n;
  ss2 << file_2 ;
  ss2 >> sbuf;
  ss2 >> *bond_n;
  for(int i=0;i<(*bond_n);i++){
    ss2 >> sbuf;
    ss2 >> sbuf;
  }
  ss2 >> sbuf;
  ss2 >> sbuf;
  ss2 >> sbuf;
  ss2 >> *rot_bond_n ;
  while (1) {
    ss2 >> sbuf;
    if(sbuf=="ATOM_PAIR"){
      ss2 >> *pair_n;
      break;
    }
  }
  while (1) {
    ss2 >> sbuf;
    if(sbuf=="ROT_BOND_MAIN"){
      ss2 >> *rot_bond_main_n;
      break;
    }
  }    
}

void docking_set_ligand_info(const string& file_1,
                             const string& file_2,
                             int         * atomic,
                             string      * type,
                             double      * x,
                             double      * y,
                             double      * z,
                             double      * chg,
                             double      * vdw_e,
                             double      * vdw_r,
                             int         * bond_1,
                             int         * bond_2,
                             int         * center_atom,
                             string      * all_type,
                             double      * all_type_vdw_e,
                             double      * all_type_vdw_r,
                             int         * rot_bond,
                             int         * rot_bond_main,
                             int         * rot_bond_atom_n,
                             int         * rot_bond_atom,
                             int         * pair_1,
                             int         * pair_2,
                             double      * pair_coeffA,
                             double      * pair_coeffB)
{
  stringstream ss1;
  stringstream ss2;
  char buf[1024];
  int atom_n,bond_n,rot_bond_n,pair_n,rot_bond_main_n,chk;

  ss1 << file_1;
  ss1 >> buf;
  ss1 >> atom_n ;
  for(int i=0;i<atom_n;i++){
    ss1 >> buf ;
    ss1 >> atomic [i] ;
    ss1 >> type [i] ;
    ss1 >> x [i] ;
    ss1 >> y [i] ;
    ss1 >> z [i] ;
    ss1 >> chg [i] ;
    ss1 >> vdw_r [i] ;
    ss1 >> vdw_e [i] ;
    x[i]=x[i]/0.5291 ;
    y[i]=y[i]/0.5291 ;
    z[i]=z[i]/0.5291 ;
  }

  int n=0;
  for(int i=0;i<atom_n;i++){
    chk=0;
    for(int j=0;j<n;j++){
      if(all_type[j]==type[i]){
        chk=1;
        break;
      }
    }
    if(chk==0){
      all_type      [n]=type [i];
      all_type_vdw_r[n]=vdw_r[i];
      all_type_vdw_e[n]=vdw_e[i];
      n++;
    }
  }

  ss2 << file_2 ;
  ss2 >> buf;
  ss2 >> bond_n;
  for(int i=0;i<bond_n;i++){
    ss2 >> bond_1[i] ;
    ss2 >> bond_2[i] ;
    bond_1[i]-- ;
    bond_2[i]-- ;
  }
  ss2 >> buf;
  ss2 >> *center_atom ;
  (*center_atom)-- ;
  ss2 >> buf ;
  ss2 >> rot_bond_n ;
  for(int i=0;i<rot_bond_n;i++){
    ss2 >> rot_bond[i];
    rot_bond[i]--;
  }
  ss2 >> buf ;
  for(int i=0;i<rot_bond_n;i++){
    ss2 >> rot_bond_atom_n[i];
  }
  ss2 >> buf ;
  for(int i=0;i<rot_bond_n;i++){
    for(int j=0;j<rot_bond_atom_n[i];j++){
      ss2 >> rot_bond_atom[i*atom_n+j];
      rot_bond_atom[i*atom_n+j]--;
    }
  }
  ss2 >> buf ;
  ss2 >> pair_n;
  for(int i=0;i<pair_n;i++){
    ss2 >> pair_1[i];
    ss2 >> pair_2[i];
    pair_1[i]--;
    pair_2[i]--;
  }
  ss2 >> buf ;
  ss2 >> rot_bond_main_n;
  for(int i=0;i<rot_bond_main_n;i++){
    ss2 >> rot_bond_main[i];
  }

  double vdw_e1,vdw_e2,vdw_r1,vdw_r2;
  for(int j=0;j<pair_n;j++){
    vdw_e1=vdw_e[pair_1[j]];
    vdw_e2=vdw_e[pair_2[j]];
    vdw_r1=vdw_r[pair_1[j]]*0.5;
    vdw_r2=vdw_r[pair_2[j]]*0.5;
    pair_coeffA[j]=    sqrt(vdw_e1*vdw_e2)*pow((vdw_r1+vdw_r2),12);
    pair_coeffB[j]=2.0*sqrt(vdw_e1*vdw_e2)*pow((vdw_r1+vdw_r2), 6);
  }

}

void docking_get_protein_info(const string& file_3,
                              int * atom_n,
                              int * bond_n)
{
  stringstream ss;
  string sbuf;
  int atomic;
  double x,y,z;
  Molecule m;
  Atom a;
  Dvector3D r;
  ss << file_3;
  ss >> sbuf;
  ss >> *atom_n;
  for(int i=0;i<(*atom_n);i++){
    ss >> sbuf;
    ss >> atomic;
    ss >> sbuf;
    ss >> x ;
    ss >> y ;
    ss >> z ;
    ss >> sbuf;
    ss >> sbuf;
    ss >> sbuf;
    a.set_atomic_number(atomic);
    r.set_xyz(x/0.5291,y/0.5291,z/0.5291);
    a.set_R(r);
    m.add_atom(a);
  }
  m.add_bond_auto();
  *bond_n = m.get_nbond() ;
}

void docking_set_protein_info(const string& file_3,
                              const int     docking_pro_n,
                              const int     docking_pro_bond_n,
                              int    *      docking_pro_main_bond_n,
                              int    *      docking_pro_atomic,
                              int    *      docking_pro_biounit,
                              int    *      docking_pro_bond_1,
                              int    *      docking_pro_bond_2,
                              int    *      docking_pro_main_bond_1,
                              int    *      docking_pro_main_bond_2,
                              double *      docking_pro_x,
                              double *      docking_pro_y,
                              double *      docking_pro_z,
                              double *      docking_pro_chg,
                              double *      docking_pro_vdw_e,
                              double *      docking_pro_vdw_r)
{
  stringstream ss;
  string sbuf;
  Atom a;
  Molecule m;
  Dvector3D r;
  ss << file_3;
  ss >> sbuf;
  ss >> sbuf;
  for(int i=0;i<docking_pro_n;i++){
    ss >> sbuf;
    ss >> docking_pro_atomic[i];
    ss >> sbuf;
    ss >> docking_pro_x     [i];
    ss >> docking_pro_y     [i];
    ss >> docking_pro_z     [i];
    ss >> docking_pro_chg   [i];
    ss >> docking_pro_vdw_r [i];
    ss >> docking_pro_vdw_e [i];
    docking_pro_x[i]=docking_pro_x[i]/0.5291;
    docking_pro_y[i]=docking_pro_y[i]/0.5291;
    docking_pro_z[i]=docking_pro_z[i]/0.5291;
    a.set_atomic_number(docking_pro_atomic[i]);
    r.set_xyz(docking_pro_x[i],docking_pro_y[i],docking_pro_z[i]);
    a.set_R(r);
    m.add_atom(a);
  }
  ss >> sbuf;
  ss >> sbuf;
  for(int i=0;i<docking_pro_n;i++){
    ss >> sbuf;
    ss >> docking_pro_biounit[i];
  }
  m.add_bond_auto();
  int ib1,ib2,ibio1,ibio2,n;
  n=0;
  for(int i=0;i<docking_pro_bond_n;i++){
    ib1=m.get_bond_iatom1(i);
    ib2=m.get_bond_iatom2(i);
    ibio1=docking_pro_biounit[ib1];
    ibio2=docking_pro_biounit[ib2];
    docking_pro_bond_1[i]=ib1;
    docking_pro_bond_2[i]=ib2;
    if((ibio1==0 || ibio1==1 || ibio1==2) && (ibio2==0 || ibio2==1 || ibio2==2)){
      docking_pro_main_bond_1[n]=ib1;
      docking_pro_main_bond_2[n]=ib2;
      n++;
    }
  }
  (*docking_pro_main_bond_n)=n;
}

void docking_energy_internal(const int      conf_n,
                             const int      atom_n,
                             const int      pair_n,
                             const int    * pair_1,
                             const int    * pair_2,
                             const double * x,
                             const double * y,
                             const double * z,
                             const double * chg,
                             const double * vdw_e,
                             const double * vdw_r,
                             const double * coeffA,
                             const double * coeffB,
                             const double * external_eng,
                             const double   external_eng_max,
                             double       * eng_chg,
                             double       * eng_vdw,
                             long int     * n_internal)
{
  double d,x1,y1,z1,x2,y2,z2,vdw_e1,vdw_e2,vdw_r1,vdw_r2,coeffA_,coeffB_,chg1,chg2,v1,v2;
  for(int i=0;i<conf_n;i++){
    if(external_eng[i]<external_eng_max){
      eng_chg[i]=0.0;
      eng_vdw[i]=0.0;
      for(int j=0;j<pair_n;j++){
        x1=x[atom_n*i+pair_1[j]];
        y1=y[atom_n*i+pair_1[j]];
        z1=z[atom_n*i+pair_1[j]];
        x2=x[atom_n*i+pair_2[j]];
        y2=y[atom_n*i+pair_2[j]];
        z2=z[atom_n*i+pair_2[j]];
        d=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2))*0.5291772108;
        chg1=chg[pair_1[j]];
        chg2=chg[pair_2[j]];
        coeffA_=coeffA[j];
        coeffB_=coeffB[j];
        v1=coeffA_/pow(d,12)-coeffB_/pow(d,6);
        v2=0.0;
        eng_vdw[i]=eng_vdw[i]+v1;
        eng_chg[i]=eng_chg[i]+v2;
      }
      (*n_internal)++;
    }
  }
}

void docking_energy_external(const int      conf_n,
                             const int      atom_n,
                             const double * x,
                             const double * y,
                             const double * z,
                             const int    * ix,
                             const int    * iy,
                             const int    * iz,
                             const double * chg,
                             const double * vdw_e,
                             const double * vdw_r,
                             const int      pro_atom_n,
                             const double * pro_x,
                             const double * pro_y,
                             const double * pro_z,
                             const double * pro_chg,
                             const double * pro_vdw_e,
                             const double * pro_vdw_r,
                             double       * eng_chg,
                             double       * eng_vdw,
                             long int     * n_external)
{
  double d,x1,y1,z1,x2,y2,z2,vdw_e1,vdw_e2,vdw_r1,vdw_r2,coeffA,coeffB,chg1,chg2,v1,v2;
  int nx1,ny1,nz1,ntmp;
  ntmp=0;
  for(int i=0;i<conf_n;i++){
    eng_chg[i]=0.0;
    eng_vdw[i]=0.0;
    for(int j=0;j<atom_n;j++){
      nx1=ix[i*atom_n+j];
      ny1=iy[i*atom_n+j];
      nz1=iz[i*atom_n+j];
      if(nx1>=0 && ny1>=0 && nz1>=0){
        for(int k=0;k<pro_atom_n;k++){
          x1=x[atom_n*i+j];
          y1=y[atom_n*i+j];
          z1=z[atom_n*i+j];
          x2=pro_x[k];
          y2=pro_y[k];
          z2=pro_z[k];
          vdw_e1=vdw_e[j];
          vdw_r1=vdw_r[j]*0.5;
          chg1=chg[j];
          vdw_e2=pro_vdw_e[k];
          vdw_r2=pro_vdw_r[k]*0.5;
          chg2=chg[k];
          d=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2))*0.5291772108;
          coeffA=    sqrt(vdw_e1*vdw_e2)*pow((vdw_r1+vdw_r2),12);
          coeffB=2.0*sqrt(vdw_e1*vdw_e2)*pow((vdw_r1+vdw_r2), 6);
          v1=coeffA/pow(d,12)-coeffB/pow(d,6);
          v2=0.0;
          eng_vdw[i]=eng_vdw[i]+v1;
          eng_chg[i]=eng_chg[i]+v2;
        }
      } else {
        ntmp++;
        v1=999999.0;
        v2=0.0;
        eng_vdw[i]=eng_vdw[i]+v1;
        eng_chg[i]=eng_chg[i]+v2;
      }
    }
    (*n_external)++;
  }
}

void docking_energy_external_grid(const int      conf_n,
                                  const int      atom_n,
                                  const double * x,
                                  const double * y,
                                  const double * z,
                                  const int    * type_i,
                                  const int      grid_ntype,
                                  const int      grid_nx,
                                  const int      grid_ny,
                                  const int      grid_nz,
                                  const double   grid_0x,
                                  const double   grid_0y,
                                  const double   grid_0z,
                                  const double   grid_d,
                                  const double * grid,
                                  const int    * ix,
                                  const int    * iy,
                                  const int    * iz,
                                  const double * dx,
                                  const double * dy,
                                  const double * dz,
                                  double       * eng_chg,
                                  double       * eng_vdw,
                                  long int     * n_external)
{
  int nx1,nx2,ny1,ny2,nz1,nz2,ntmp;
  double v1,d,dx1,dx2,dy1,dy2,dz1,dz2,x1,y1,z1,lx,ly,lz,lx_max,ly_max,lz_max;
  double v_111,v_112,v_121,v_122,v_211,v_212,v_221,v_222,v_x11,v_x12,v_x21,v_x22,v_xy1,v_xy2,v_xyz;
  int n1=(grid_nz+1);
  int n2=(grid_nz+1)*(grid_ny+1);
  int n3=(grid_nz+1)*(grid_ny+1)*(grid_nz+1);
  ntmp=0;
  for(int i=0;i<conf_n;i++){
    eng_chg[i]=0.0;
    eng_vdw[i]=0.0;
    for(int j=0;j<atom_n;j++){
      nx1=ix[i*atom_n+j];
      ny1=iy[i*atom_n+j];
      nz1=iz[i*atom_n+j];
      if(nx1>=0 && ny1>=0 && nz1>=0){
        nx2=nx1+1;
        ny2=ny1+1;
        nz2=nz1+1;
        dx1=dx[i*atom_n+j];
        dy1=dy[i*atom_n+j];
        dz1=dz[i*atom_n+j];
        v_111=grid[ type_i[j] * n3 + nx1 * n2 + ny1 * n1 + nz1 ];
        v_112=grid[ type_i[j] * n3 + nx1 * n2 + ny1 * n1 + nz2 ];
        v_121=grid[ type_i[j] * n3 + nx1 * n2 + ny2 * n1 + nz1 ];
        v_122=grid[ type_i[j] * n3 + nx1 * n2 + ny2 * n1 + nz2 ];
        v_211=grid[ type_i[j] * n3 + nx2 * n2 + ny1 * n1 + nz1 ];
        v_212=grid[ type_i[j] * n3 + nx2 * n2 + ny1 * n1 + nz2 ];
        v_221=grid[ type_i[j] * n3 + nx2 * n2 + ny2 * n1 + nz1 ];
        v_222=grid[ type_i[j] * n3 + nx2 * n2 + ny2 * n1 + nz2 ];
        v_x11=v_111+dx1*(v_211-v_111);
        v_x12=v_112+dx1*(v_212-v_112);
        v_x21=v_121+dx1*(v_221-v_121);
        v_x22=v_122+dx1*(v_222-v_122);
        v_xy1=v_x11+dy1*(v_x21-v_x11);
        v_xy2=v_x12+dy1*(v_x22-v_x12);
        v_xyz=v_xy1+dz1*(v_xy2-v_xy1);
        v1=v_xyz;
      } else {
        ntmp++;
        v1=999999.0;
      }
      eng_vdw[i]=eng_vdw[i]+v1;
    }
    (*n_external)++;
  }
}

void docking_make_conformation_2(const int      conf_n,
                                 const int      atom_n,
                                 const int    * atomic,
                                 const int      center_atom,
                                 const double * x_orig,
                                 const double * y_orig,
                                 const double * z_orig,
                                 const int    * bond_1,
                                 const int    * bond_2,
                                 const int      rot_bond_n,
                                 const int    * rot_bond,
                                 const int    * rot_bond_atom_n,
                                 const int    * rot_bond_atom,
                                 const double * par_ang,
                                 const double * par_pos,
                                 const double * par_rot,
                                 double       * x_out,
                                 double       * y_out,
                                 double       * z_out)
{
  Molecule m;
  Atom a;
  Dvector3D r,r1,r2,r12,axis;
  int a1,a2;
  double ax,ay,az;

  for(int i=0;i<conf_n;i++){
    
    /////////////////////////////
    // set original coordinate //
    /////////////////////////////

    m.clear();
    for(int j=0;j<atom_n;j++){
      a.set_atomic_number(atomic[j]);
      r.set_xyz(x_orig[j],y_orig[j],z_orig[j]);
      a.set_R(r);
      m.add_atom(a);
    }

    ///////////////////
    // bond rotation //
    ///////////////////

    for(int j=0;j<rot_bond_n;j++){
      a1=bond_1[rot_bond[j]];
      a2=bond_2[rot_bond[j]];
      r1.copy(m.get_atom(a1).get_R());
      r2.copy(m.get_atom(a2).get_R());
      r12=(r2-r1);
      for(int k=0;k<rot_bond_atom_n[j];k++){
        m.rotate_atom(rot_bond_atom[j*atom_n+k],r12,par_ang[i*rot_bond_n+j]*3.141592/180.0,r1);
      }
    }

    //////////////
    // rotation //
    //////////////

    ax=par_rot[i*4+0];
    ay=par_rot[i*4+1];
    az=par_rot[i*4+2];
    axis.set_xyz(ax,ay,az);
    r.set_xyz(x_orig[center_atom],y_orig[center_atom],z_orig[center_atom]);
    m.rotate(axis,par_rot[i*4+3],r);
    
    ///////////////
    // translate //
    ///////////////

    r1.set_xyz(par_pos[i*3+0]-x_orig[center_atom],par_pos[i*3+1]-y_orig[center_atom],par_pos[i*3+2]-z_orig[center_atom]);
    m.translate(r1);

    ///////////////////////////
    // set output coordinate //
    ///////////////////////////

    for(int j=0;j<atom_n;j++){
      r.copy(m.get_atom(j).get_R());
      x_out[i*atom_n+j]=r.get_x();
      y_out[i*atom_n+j]=r.get_y();
      z_out[i*atom_n+j]=r.get_z();
    }
  }
}




void docking_make_conformation(const int      conf_n,
                               const int      atom_n,
                               const int    * atomic,
                               const int      center_atom,
                               const double * x_orig,
                               const double * y_orig,
                               const double * z_orig,
                               const int    * bond_1,
                               const int    * bond_2,
                               const int      rot_bond_n,
                               const int    * rot_bond,
                               const int    * rot_bond_atom_n,
                               const int    * rot_bond_atom,
                               const double * par_ang,
                               const double * par_pos,
                               const double * par_rot,
                               double       * x_out,
                               double       * y_out,
                               double       * z_out)
{
  double r1_x,r1_y,r1_z;
  double r2_x,r2_y,r2_z;
  double r12_x,r12_y,r12_z;
  int a1,a2;
  double ax,ay,az;

  int iatom;
  double axis_xy_x;
  double axis_xy_y;
  double axis_xy_z;

  double r_atom_x;
  double r_atom_y;
  double r_atom_z;
  double r_atom_new_x;
  double r_atom_new_y;
  double r_atom_new_z;
  double ang;
  double ang1;
  double ang2;
  double p_cos_a;
  double p_sin_a;
  double m_cos_a;
  double m_sin_a;
  double p_cos_a1;
  double p_sin_a1;
  double m_cos_a1;
  double m_sin_a1;
  double p_cos_a2;
  double p_sin_a2;
  double m_cos_a2;
  double m_sin_a2;
  
  double r_tmp0_x;
  double r_tmp0_y;
  double r_tmp0_z;
  double r_tmp1_x;
  double r_tmp1_y;
  double r_tmp1_z;
  double r_tmp2_x;
  double r_tmp2_y;
  double r_tmp2_z;
  double r_tmp3_x;
  double r_tmp3_y;
  double r_tmp3_z;
  double dtmp;
  
  double * m_atom_x ;
  double * m_atom_y ;
  double * m_atom_z ;

  m_atom_x = new double [atom_n+1];
  m_atom_y = new double [atom_n+1];
  m_atom_z = new double [atom_n+1];

  for(int i=0;i<conf_n;i++){
    
    /////////////////////////////
    // set original coordinate //
    /////////////////////////////

    for(int j=0;j<atom_n;j++){
      m_atom_x[j]=x_orig[j];
      m_atom_y[j]=y_orig[j];
      m_atom_z[j]=z_orig[j];
    }

    ///////////////////
    // bond rotation //
    ///////////////////

    for(int j=0;j<rot_bond_n;j++){
      a1=bond_1[rot_bond[j]];
      a2=bond_2[rot_bond[j]];
      r1_x=m_atom_x[a1];
      r1_y=m_atom_y[a1];
      r1_z=m_atom_z[a1];
      r2_x=m_atom_x[a2];
      r2_y=m_atom_y[a2];
      r2_z=m_atom_z[a2];
      r12_x=r2_x-r1_x;
      r12_y=r2_y-r1_y;
      r12_z=r2_z-r1_z;
      axis_xy_x=r12_x;
      axis_xy_y=r12_y;

      ang=par_ang[i*rot_bond_n+j]*3.141592/180.0;

      dtmp=(axis_xy_x)/sqrt((axis_xy_x*axis_xy_x+axis_xy_y*axis_xy_y));
      if     ( 0.9999999 < dtmp && dtmp <  1.0000001 ) {ang1=0.0;}
      else if(-1.0000001 < dtmp && dtmp < -0.9999999 ) {ang1=3.14159265359;}
      else                                             {ang1=acos(dtmp);}
 
      dtmp=(r12_z)/sqrt((r12_x*r12_x+r12_y*r12_y+r12_z*r12_z));
      if     ( 0.9999999 < dtmp && dtmp <  1.0000001 ) {ang2=0.0;}
      else if(-1.0000001 < dtmp && dtmp < -0.9999999 ) {ang2=3.14159265359;}
      else                                             {ang2=acos(dtmp);}

      p_cos_a  =  cos( ang );
      p_sin_a  =  sin( ang );
      m_cos_a  =  cos(-ang );
      m_sin_a  =  sin(-ang );
      p_cos_a1 =  cos( ang1); 
      p_sin_a1 =  sin( ang1); 
      m_cos_a1 =  cos(-ang1); 
      m_sin_a1 =  sin(-ang1); 
      p_cos_a2 =  cos( ang2);
      p_sin_a2 =  sin( ang2);
      m_cos_a2 =  cos(-ang2);
      m_sin_a2 =  sin(-ang2);

      for(int k=0;k<rot_bond_atom_n[j];k++){
        iatom=rot_bond_atom[j*atom_n+k];
        r_atom_x=m_atom_x[iatom]-r1_x;
        r_atom_y=m_atom_y[iatom]-r1_y;
        r_atom_z=m_atom_z[iatom]-r1_z;
        if(-0.000001 < r12_x && r12_x < 0.0000001 &&
           -0.000001 < r12_y && r12_y < 0.0000001 &&
           -0.000001 < r12_z && r12_z < 0.0000001){
            cerr << "error : Dvector3D::rotation(const Dvector3D& axis, const Dvector3D r2, const double ang) : " ;
            cerr << "axis is zero vector !!" << endl ;
            exit(1);
        } 
        else if(-0.0000001 < r12_y && r12_y < 0.000001 &&
                -0.0000001 < r12_z && r12_z < 0.000001){
          if(r12_x>=0.0){
            r_atom_new_x=r_atom_x;
            r_atom_new_y=r_atom_y*cos( ang)-r_atom_z*sin( ang);
            r_atom_new_z=r_atom_y*sin( ang)+r_atom_z*cos( ang);
          }
          else{
            r_atom_new_x=r_atom_x;
            r_atom_new_y=r_atom_y*cos(-ang)-r_atom_z*sin(-ang);
            r_atom_new_z=r_atom_y*sin(-ang)+r_atom_z*cos(-ang);
          }
        }
        else if(-0.0000001 < r12_x && r12_x < 0.000001 &&
                -0.0000001 < r12_z && r12_z < 0.000001){
          if(r12_y>=0.0){
            r_atom_new_z=r_atom_z*cos( ang)-r_atom_x*sin( ang);
            r_atom_new_y=r_atom_y;
            r_atom_new_x=r_atom_z*sin( ang)+r_atom_x*cos( ang);
          }
          else{
            r_atom_new_z=r_atom_z*cos(-ang)-r_atom_x*sin(-ang);
            r_atom_new_y=r_atom_y;
            r_atom_new_x=r_atom_z*sin(-ang)+r_atom_x*cos(-ang);
          }
        }
        else if(-0.000001 < r12_x && r12_x < 0.000001 &&
                -0.000001 < r12_y && r12_y < 0.000001){
          if(r12_z>=0.0){
            r_atom_new_x=r_atom_x*cos( ang)-r_atom_y*sin( ang);
            r_atom_new_y=r_atom_x*sin( ang)+r_atom_y*cos( ang);
            r_atom_new_z=r_atom_z;
          }
          else{
            r_atom_new_x=r_atom_x*cos(-ang)-r_atom_y*sin(-ang);
            r_atom_new_y=r_atom_x*sin(-ang)+r_atom_y*cos(-ang);
            r_atom_new_z=r_atom_z;
          }
        }
        else if(r12_y>=0.0){
          r_tmp0_x    =r_atom_x*m_cos_a1-r_atom_y*m_sin_a1;
          r_tmp0_y    =r_atom_x*m_sin_a1+r_atom_y*m_cos_a1;
          r_tmp0_z    =r_atom_z;
          r_tmp1_z    =r_tmp0_z*m_cos_a2-r_tmp0_x*m_sin_a2;
          r_tmp1_y    =r_tmp0_y;
          r_tmp1_x    =r_tmp0_z*m_sin_a2+r_tmp0_x*m_cos_a2;
          r_tmp2_x    =r_tmp1_x*p_cos_a -r_tmp1_y*p_sin_a ;
          r_tmp2_y    =r_tmp1_x*p_sin_a +r_tmp1_y*p_cos_a ;
          r_tmp2_z    =r_tmp1_z;
          r_tmp3_z    =r_tmp2_z*p_cos_a2-r_tmp2_x*p_sin_a2;
          r_tmp3_y    =r_tmp2_y;
          r_tmp3_x    =r_tmp2_z*p_sin_a2+r_tmp2_x*p_cos_a2;
          r_atom_new_x=r_tmp3_x*p_cos_a1-r_tmp3_y*p_sin_a1;
          r_atom_new_y=r_tmp3_x*p_sin_a1+r_tmp3_y*p_cos_a1;
          r_atom_new_z=r_tmp3_z;
        }
        else{
          r_tmp0_x    =r_atom_x*p_cos_a1-r_atom_y*p_sin_a1;
          r_tmp0_y    =r_atom_x*p_sin_a1+r_atom_y*p_cos_a1;
          r_tmp0_z    =r_atom_z;
          r_tmp1_z    =r_tmp0_z*m_cos_a2-r_tmp0_x*m_sin_a2;
          r_tmp1_y    =r_tmp0_y;
          r_tmp1_x    =r_tmp0_z*m_sin_a2+r_tmp0_x*m_cos_a2;
          r_tmp2_x    =r_tmp1_x*p_cos_a -r_tmp1_y*p_sin_a ;
          r_tmp2_y    =r_tmp1_x*p_sin_a +r_tmp1_y*p_cos_a ;
          r_tmp2_z    =r_tmp1_z;
          r_tmp3_z    =r_tmp2_z*p_cos_a2-r_tmp2_x*p_sin_a2;
          r_tmp3_y    =r_tmp2_y;
          r_tmp3_x    =r_tmp2_z*p_sin_a2+r_tmp2_x*p_cos_a2;
          r_atom_new_x=r_tmp3_x*m_cos_a1-r_tmp3_y*m_sin_a1;
          r_atom_new_y=r_tmp3_x*m_sin_a1+r_tmp3_y*m_cos_a1;
          r_atom_new_z=r_tmp3_z;
        }
        m_atom_x[iatom]=r1_x+r_atom_new_x;
        m_atom_y[iatom]=r1_y+r_atom_new_y;
        m_atom_z[iatom]=r1_z+r_atom_new_z;
      }
    }

    //////////////
    // rotation //
    //////////////

    ax=par_rot[i*4+0];
    ay=par_rot[i*4+1];
    az=par_rot[i*4+2];
    r1_x=x_orig[center_atom];
    r1_y=y_orig[center_atom];
    r1_z=z_orig[center_atom];
    r12_x=ax;
    r12_y=ay;
    r12_z=az;
    axis_xy_x=r12_x;
    axis_xy_y=r12_y;
    ang=par_rot[i*4+3];

    dtmp=(axis_xy_x)/sqrt((axis_xy_x*axis_xy_x+axis_xy_y*axis_xy_y));
    if     ( 0.9999999 < dtmp && dtmp <  1.0000001 ) {ang1=0.0;}
    else if(-1.0000001 < dtmp && dtmp < -0.9999999 ) {ang1=3.14159265359;}
    else                                             {ang1=acos(dtmp);}
 
    dtmp=(r12_z)/sqrt((r12_x*r12_x+r12_y*r12_y+r12_z*r12_z));
    if     ( 0.9999999 < dtmp && dtmp <  1.0000001 ) {ang2=0.0;}
    else if(-1.0000001 < dtmp && dtmp < -0.9999999 ) {ang2=3.14159265359;}
    else                                             {ang2=acos(dtmp);}

    p_cos_a  =  cos( ang );
    p_sin_a  =  sin( ang );
    m_cos_a  =  cos(-ang );
    m_sin_a  =  sin(-ang );
    p_cos_a1 =  cos( ang1); 
    p_sin_a1 =  sin( ang1); 
    m_cos_a1 =  cos(-ang1); 
    m_sin_a1 =  sin(-ang1); 
    p_cos_a2 =  cos( ang2);
    p_sin_a2 =  sin( ang2);
    m_cos_a2 =  cos(-ang2);
    m_sin_a2 =  sin(-ang2);

    for(int j=0;j<atom_n;j++){
      iatom=j;
      r_atom_x=m_atom_x[iatom]-r1_x;
      r_atom_y=m_atom_y[iatom]-r1_y;
      r_atom_z=m_atom_z[iatom]-r1_z;
      if(-0.000001 < r12_x && r12_x < 0.0000001 &&
         -0.000001 < r12_y && r12_y < 0.0000001 &&
         -0.000001 < r12_z && r12_z < 0.0000001){
        cerr << "error : Dvector3D::rotation(const Dvector3D& axis, const Dvector3D r2, const double ang) : " ;
        cerr << "axis is zero vector !!" << endl ;
        exit(1);
      } 
      else if(-0.0000001 < r12_y && r12_y < 0.000001 &&
              -0.0000001 < r12_z && r12_z < 0.000001){
        if(r12_x>=0.0){
          r_atom_new_x=r_atom_x;
          r_atom_new_y=r_atom_y*cos( ang)-r_atom_z*sin( ang);
          r_atom_new_z=r_atom_y*sin( ang)+r_atom_z*cos( ang);
        }
        else{
          r_atom_new_x=r_atom_x;
          r_atom_new_y=r_atom_y*cos(-ang)-r_atom_z*sin(-ang);
          r_atom_new_z=r_atom_y*sin(-ang)+r_atom_z*cos(-ang);
        }
      }
      else if(-0.0000001 < r12_x && r12_x < 0.000001 &&
              -0.0000001 < r12_z && r12_z < 0.000001){
        if(r12_y>=0.0){
          r_atom_new_z=r_atom_z*cos( ang)-r_atom_x*sin( ang);
          r_atom_new_y=r_atom_y;
          r_atom_new_x=r_atom_z*sin( ang)+r_atom_x*cos( ang);
        }
        else{
          r_atom_new_z=r_atom_z*cos(-ang)-r_atom_x*sin(-ang);
          r_atom_new_y=r_atom_y;
          r_atom_new_x=r_atom_z*sin(-ang)+r_atom_x*cos(-ang);
        }
      }
      else if(-0.000001 < r12_x && r12_x < 0.000001 &&
              -0.000001 < r12_y && r12_y < 0.000001){
        if(r12_z>=0.0){
          r_atom_new_x=r_atom_x*cos( ang)-r_atom_y*sin( ang);
          r_atom_new_y=r_atom_x*sin( ang)+r_atom_y*cos( ang);
          r_atom_new_z=r_atom_z;
        }
        else{
          r_atom_new_x=r_atom_x*cos(-ang)-r_atom_y*sin(-ang);
          r_atom_new_y=r_atom_x*sin(-ang)+r_atom_y*cos(-ang);
          r_atom_new_z=r_atom_z;
        }
      }
      else if(r12_y>=0.0){
        r_tmp0_x    =r_atom_x*m_cos_a1-r_atom_y*m_sin_a1;
        r_tmp0_y    =r_atom_x*m_sin_a1+r_atom_y*m_cos_a1;
        r_tmp0_z    =r_atom_z;
        r_tmp1_z    =r_tmp0_z*m_cos_a2-r_tmp0_x*m_sin_a2;
        r_tmp1_y    =r_tmp0_y;
        r_tmp1_x    =r_tmp0_z*m_sin_a2+r_tmp0_x*m_cos_a2;
        r_tmp2_x    =r_tmp1_x*p_cos_a -r_tmp1_y*p_sin_a ;
        r_tmp2_y    =r_tmp1_x*p_sin_a +r_tmp1_y*p_cos_a ;
        r_tmp2_z    =r_tmp1_z;
        r_tmp3_z    =r_tmp2_z*p_cos_a2-r_tmp2_x*p_sin_a2;
        r_tmp3_y    =r_tmp2_y;
        r_tmp3_x    =r_tmp2_z*p_sin_a2+r_tmp2_x*p_cos_a2;
        r_atom_new_x=r_tmp3_x*p_cos_a1-r_tmp3_y*p_sin_a1;
        r_atom_new_y=r_tmp3_x*p_sin_a1+r_tmp3_y*p_cos_a1;
        r_atom_new_z=r_tmp3_z;
      }
      else{
        r_tmp0_x    =r_atom_x*p_cos_a1-r_atom_y*p_sin_a1;
        r_tmp0_y    =r_atom_x*p_sin_a1+r_atom_y*p_cos_a1;
        r_tmp0_z    =r_atom_z;
        r_tmp1_z    =r_tmp0_z*m_cos_a2-r_tmp0_x*m_sin_a2;
        r_tmp1_y    =r_tmp0_y;
        r_tmp1_x    =r_tmp0_z*m_sin_a2+r_tmp0_x*m_cos_a2;
        r_tmp2_x    =r_tmp1_x*p_cos_a -r_tmp1_y*p_sin_a ;
        r_tmp2_y    =r_tmp1_x*p_sin_a +r_tmp1_y*p_cos_a ;
        r_tmp2_z    =r_tmp1_z;
        r_tmp3_z    =r_tmp2_z*p_cos_a2-r_tmp2_x*p_sin_a2;
        r_tmp3_y    =r_tmp2_y;
        r_tmp3_x    =r_tmp2_z*p_sin_a2+r_tmp2_x*p_cos_a2;
        r_atom_new_x=r_tmp3_x*m_cos_a1-r_tmp3_y*m_sin_a1;
        r_atom_new_y=r_tmp3_x*m_sin_a1+r_tmp3_y*m_cos_a1;
        r_atom_new_z=r_tmp3_z;
      }
      m_atom_x[iatom]=r1_x+r_atom_new_x;
      m_atom_y[iatom]=r1_y+r_atom_new_y;
      m_atom_z[iatom]=r1_z+r_atom_new_z;
    }
    
    ///////////////
    // translate //
    ///////////////

    r1_x=par_pos[i*3+0]-x_orig[center_atom];
    r1_y=par_pos[i*3+1]-y_orig[center_atom];
    r1_z=par_pos[i*3+2]-z_orig[center_atom];

    for(int j=0;j<atom_n;j++){
      r_atom_x    =m_atom_x[j];
      r_atom_y    =m_atom_y[j];
      r_atom_z    =m_atom_z[j];
      r_atom_new_x=r_atom_x+r1_x;
      r_atom_new_y=r_atom_y+r1_y;
      r_atom_new_z=r_atom_z+r1_z;
      m_atom_x[j] =r_atom_new_x;
      m_atom_y[j] =r_atom_new_y;
      m_atom_z[j] =r_atom_new_z;
    }

    ///////////////////////////
    // set output coordinate //
    ///////////////////////////

    for(int j=0;j<atom_n;j++){
      x_out[i*atom_n+j]=m_atom_x[j];
      y_out[i*atom_n+j]=m_atom_y[j];
      z_out[i*atom_n+j]=m_atom_z[j];
    }
  }

  delete [] m_atom_x ;
  delete [] m_atom_y ;
  delete [] m_atom_z ;
}

void docking_make_grid_position(const int      conf_n,
                                const int      atom_n,
                                const int      grid_nx,
                                const int      grid_ny,
                                const int      grid_nz,
                                const double   grid_0x,
                                const double   grid_0y,
                                const double   grid_0z,
                                const double   grid_d,                                     
                                const double * x_out,
                                const double * y_out,
                                const double * z_out,
                                int          * ix_out,
                                int          * iy_out,
                                int          * iz_out,
                                double       * dx_out,
                                double       * dy_out,
                                double       * dz_out)
{
  int nx1,nx2,ny1,ny2,nz1,nz2;
  double v1,d,dx1,dx2,dy1,dy2,dz1,dz2,x1,y1,z1,lx,ly,lz,lx_max,ly_max,lz_max;

  lx_max=grid_nx*grid_d;
  ly_max=grid_ny*grid_d;
  lz_max=grid_nz*grid_d;

  for(int i=0;i<conf_n;i++){
    for(int j=0;j<atom_n;j++){
      x1=x_out[atom_n*i+j];
      y1=y_out[atom_n*i+j];
      z1=z_out[atom_n*i+j];
      lx=x1-grid_0x;
      ly=y1-grid_0y;
      lz=z1-grid_0z;
      if(0.0<lx && lx<lx_max && 
         0.0<ly && ly<ly_max &&
         0.0<lz && lz<ly_max ){

        nx2=0;
        d=0.0;
        for(int k=0;k<grid_nx;k++){
          d+=grid_d;
          nx2++;
          if(d>lx){
            dx1=(grid_d-(d-lx))/grid_d;
            break;
          }
        }
        nx1=nx2-1;
        ix_out[atom_n*i+j]=nx1;
        dx_out[atom_n*i+j]=dx1;

        ny2=0;
        d=0.0;
        for(int k=0;k<grid_ny;k++){
          d+=grid_d;
          ny2++;
          if(d>ly){
            dy1=(grid_d-(d-ly))/grid_d;
            break;
          }
        }
        ny1=ny2-1;
        iy_out[atom_n*i+j]=ny1;
        dy_out[atom_n*i+j]=dy1;

        nz2=0;
        d=0.0;
        for(int k=0;k<grid_nz;k++){
          d+=grid_d;
          nz2++;
          if(d>lz){
            dz1=(grid_d-(d-lz))/grid_d;
            break;
          }
        }
        nz1=nz2-1;
        iz_out[atom_n*i+j]=nz1;
        dz_out[atom_n*i+j]=dz1;

      } else {
        ix_out[atom_n*i+j]=-1;
        iy_out[atom_n*i+j]=-1;
        iz_out[atom_n*i+j]=-1;
      }
    }
  }
}

void docking_setup_par(const int         grid_use,
                       const long int    random_seed,
                       const int         atom_n,
                       const int       * atomic,
                       const int         center_atom,
                       const double    * x,
                       const double    * y,
                       const double    * z,
                       const int       * bond_1,
                       const int       * bond_2,
                       const int         pair_n,
                       const int       * pair_1,
                       const int       * pair_2,
                       const double    * chg,
                       const double    * vdw_e,
                       const double    * vdw_r,
                       const double    * coeffA,
                       const double    * coeffB,
                       const int         pro_atom_n,
                       const double    * pro_x,
                       const double    * pro_y,
                       const double    * pro_z,
                       const double    * pro_chg,
                       const double    * pro_vdw_e,
                       const double    * pro_vdw_r,
                       const int       * type_i,
                       const int         grid_ntype,
                       const int         grid_nx,
                       const int         grid_ny,
                       const int         grid_nz,
                       const double      grid_d,
                       const double    * grid,                            
                       const int         par_n,
                       const int         chk_debug_compare,
                       double          * par_ang,
                       double          * par_pos,
                       double          * par_rot,
                       double          * region,
                       double          * par_eng,
                       double          * par_eng_intra,
                       double          * par_eng_intra_chg,
                       double          * par_eng_intra_vdw,
                       double          * par_eng_inter,
                       double          * par_eng_inter_chg,
                       double          * par_eng_inter_vdw,
                       double          * par_x,
                       double          * par_y,
                       double          * par_z,
                       int               rot_bond_n,
                       int             * rot_bond,
                       int             * rot_bond_atom_n,
                       int             * rot_bond_atom,
                       long int        * n_internal,
                       long int        * n_external,
                       clock_t         * t1,
                       clock_t         * t2,
                       clock_t         * t3)
{
  clock_t tt1,tt2;
  std::mt19937 mt(random_seed);

  int *    ix                 = new int    [par_n*atom_n+1];
  int *    iy                 = new int    [par_n*atom_n+1];
  int *    iz                 = new int    [par_n*atom_n+1];
  double * dx                 = new double [par_n*atom_n+1];
  double * dy                 = new double [par_n*atom_n+1];
  double * dz                 = new double [par_n*atom_n+1];
  double * eng_inter_vdw      = new double [par_n+1];
  double * eng_inter_vdw_grid = new double [par_n+1];
  double * eng_inter_chg      = new double [par_n+1];
  double * eng_inter_chg_grid = new double [par_n+1];
  
  //////////////////////
  ///// make angle /////
  //////////////////////

  for(int j=0;j<rot_bond_n;j++){
    par_ang[0*rot_bond_n+j]=0.0;
  }
  for(int i=1;i<par_n;i++){
    for(int j=0;j<rot_bond_n;j++){
      par_ang[i*rot_bond_n+j]=double(mt()%360)-180.0;
    }
  }

  /////////////////////
  ///// make pos. /////
  /////////////////////
  
  int nx=roundf((float)((region[1]-region[0])/0.1));
  int ny=roundf((float)((region[3]-region[2])/0.1));
  int nz=roundf((float)((region[5]-region[4])/0.1));
  //int nx=1; // for window compile (dummy)
  //int ny=1; // for window compile (dummy)
  //int nz=1; // for window compile (dummy)

  for(int i=0;i<par_n;i++){
    par_pos[i*3+0]=region[0]+(double(mt()%nx)*0.1);
    par_pos[i*3+1]=region[2]+(double(mt()%ny)*0.1);
    par_pos[i*3+2]=region[4]+(double(mt()%nz)*0.1);
  }

  /////////////////////
  ///// make rot. /////
  /////////////////////

  for(int i=0;i<par_n;i++){
    par_rot[i*4+0]=  -1.0+(double(mt()%200)*0.01);
    par_rot[i*4+1]=  -1.0+(double(mt()%200)*0.01);
    par_rot[i*4+2]=  -1.0+(double(mt()%200)*0.01);
    par_rot[i*4+3]=-180.0+(double(mt()%360));
  }
  
  ///////////////////////
  // make conformation //
  ///////////////////////

  tt1=clock();
  docking_make_conformation(par_n,
                            atom_n,
                            atomic,
                            center_atom,
                            x,
                            y,
                            z,
                            bond_1,
                            bond_2,
                            rot_bond_n,
                            rot_bond,
                            rot_bond_atom_n,
                            rot_bond_atom,
                            par_ang,
                            par_pos,
                            par_rot,
                            par_x,
                            par_y,
                            par_z);
  
  tt2=clock();
  (*t3)=(*t3)+(tt2-tt1);
  
  //////////////////////////
  // make external energy //
  //////////////////////////
  
  tt1=clock();
  docking_make_grid_position(par_n,
                             atom_n,
                             grid_nx,
                             grid_ny,
                             grid_nz,
                             region[0],
                             region[2],
                             region[4],
                             grid_d,
                             par_x,
                             par_y,
                             par_z,
                             ix,
                             iy,
                             iz,
                             dx,
                             dy,
                             dz);
  if(grid_use==0 || chk_debug_compare==1){
    docking_energy_external(par_n,
                            atom_n,
                            par_x,
                            par_y,
                            par_z,                               
                            ix,
                            iy,
                            iz,
                            chg,
                            vdw_e,
                            vdw_r,
                            pro_atom_n,
                            pro_x,
                            pro_y,
                            pro_z,
                            pro_chg,
                            pro_vdw_e,
                            pro_vdw_r,
                            eng_inter_chg,
                            eng_inter_vdw,
                            n_external);
  }
  if(grid_use!=0){
    docking_energy_external_grid(par_n,
                                 atom_n,
                                 par_x,
                                 par_y,
                                 par_z,
                                 type_i,
                                 grid_ntype,
                                 grid_nx,
                                 grid_ny,
                                 grid_nz,
                                 region[0],
                                 region[2],
                                 region[4],
                                 grid_d,
                                 grid,
                                 ix,
                                 iy,
                                 iz,
                                 dx,
                                 dy,
                                 dz,
                                 eng_inter_chg_grid,
                                 eng_inter_vdw_grid,
                                 n_external);
  }
  if(grid_use==0){
    for(int i=0;i<par_n;i++){
      par_eng_inter_vdw[i]=eng_inter_vdw[i];
      par_eng_inter_chg[i]=eng_inter_chg[i];
      par_eng_inter[i]=par_eng_inter_vdw[i]+par_eng_inter_chg[i];
    }
  } else {
    for(int i=0;i<par_n;i++){
      par_eng_inter_vdw[i]=eng_inter_vdw_grid[i];
      par_eng_inter_chg[i]=eng_inter_chg_grid[i];
      par_eng_inter[i]=par_eng_inter_vdw[i]+par_eng_inter_chg[i];

    }
  }
  if(chk_debug_compare==1){
      cout << endl ;
      for(int i=0;i<par_n;i++){
        cout.width(10); cout << i ;
        cout.width(20); cout << eng_inter_vdw     [i];
        cout.width(20); cout << eng_inter_vdw_grid[i];
        cout << endl ;
      }
      cout << endl ;
  }
  tt2=clock();
  (*t1)=(*t1)+(tt2-tt1);

  //////////////////////////
  // make internal energy //
  //////////////////////////
  
  tt1=clock();
  docking_energy_internal (par_n,
                           atom_n,
                           pair_n,
                           pair_1,
                           pair_2,
                           par_x,
                           par_y,
                           par_z,
                           chg,
                           vdw_e,
                           vdw_r,
                           coeffA,
                           coeffB,
                           par_eng_inter,
                           1.0E+99,
                           par_eng_intra_chg,
                           par_eng_intra_vdw,
                           n_internal);
  for(int i=0;i<par_n;i++){
    par_eng_intra[i]=par_eng_intra_chg[i]+par_eng_intra_vdw[i];
  }
  tt2=clock();
  (*t2)=(*t2)+(tt2-tt1);

  //////////////////////
  // make par. energy //
  //////////////////////

  for(int i=0;i<par_n;i++){
    par_eng[i]=par_eng_intra[i]+par_eng_inter[i];
  }

  delete [] ix ;
  delete [] iy ;
  delete [] iz ;
  delete [] dx ;
  delete [] dy ;
  delete [] dz ;
  delete [] eng_inter_vdw ;
  delete [] eng_inter_vdw_grid ;
  delete [] eng_inter_chg ;
  delete [] eng_inter_chg_grid ;
}

void docking_make_chi_angle(const long int random_seed,
                            const int      docking_rot_bond_n,
                            const int      docking_par_n,
                            const double * docking_par_ang,
                            const double * par_pos,
                            const double * par_rot,
                            const double * region,
                            const int      docking_chi_n,
                            double *       docking_chi_ang,
                            double *       chi_pos,
                            double *       chi_rot)
{
  std::mt19937 mt(random_seed);                          

  int nx=roundf((float)((region[1]-region[0])/0.1));
  int ny=roundf((float)((region[3]-region[2])/0.1));
  int nz=roundf((float)((region[5]-region[4])/0.1));
  //int nx=1; // for window compile (dummy)
  //int ny=1; // for window compile (dummy)
  //int nz=1; // for window compile (dummy)

  int nn;
  int angle_change_rate=5;
  nn=0;
  for(int i=0;i<docking_par_n;i++){
    for(int j=0;j<docking_rot_bond_n;j++){
      docking_chi_ang[nn*docking_rot_bond_n+j]=docking_par_ang[i*docking_rot_bond_n+j];
    }
    chi_pos[i*3+0]=par_pos[i*3+0];
    chi_pos[i*3+1]=par_pos[i*3+1];
    chi_pos[i*3+2]=par_pos[i*3+2];
    chi_rot[i*4+0]=par_rot[i*4+0];
    chi_rot[i*4+1]=par_rot[i*4+1];
    chi_rot[i*4+2]=par_rot[i*4+2];
    chi_rot[i*4+3]=par_rot[i*4+3];
    nn++;
  }
  for(int i=0;i<docking_par_n;i++){
    for(int j=0;j<(docking_chi_n/docking_par_n-1);j++){
      for(int k=0;k<docking_rot_bond_n;k++){
        if((mt()%angle_change_rate)==0){
          docking_chi_ang[nn*docking_rot_bond_n+k]=double(mt()%360)-180.0;
        } else {
          docking_chi_ang[nn*docking_rot_bond_n+k]=docking_par_ang[i*docking_rot_bond_n+k];
        }
      }
      if((mt()%angle_change_rate)==0){chi_pos[nn*3+0]=region[0]+(double(mt() %nx)*0.10);} else {chi_pos[nn*3+0]=par_pos[i*3+0];}
      if((mt()%angle_change_rate)==0){chi_pos[nn*3+1]=region[2]+(double(mt() %ny)*0.10);} else {chi_pos[nn*3+1]=par_pos[i*3+1];}
      if((mt()%angle_change_rate)==0){chi_pos[nn*3+2]=region[4]+(double(mt()% nz)*0.10);} else {chi_pos[nn*3+2]=par_pos[i*3+2];}
      if((mt()%angle_change_rate)==0){chi_rot[nn*4+0]=     -1.0+(double(mt()%200)*0.01);} else {chi_rot[nn*4+0]=par_rot[i*4+0];}
      if((mt()%angle_change_rate)==0){chi_rot[nn*4+1]=     -1.0+(double(mt()%200)*0.01);} else {chi_rot[nn*4+1]=par_rot[i*4+1];}
      if((mt()%angle_change_rate)==0){chi_rot[nn*4+2]=     -1.0+(double(mt()%200)*0.01);} else {chi_rot[nn*4+2]=par_rot[i*4+2];}
      if((mt()%angle_change_rate)==0){chi_rot[nn*4+3]=   -180.0+(double(mt()%360)     );} else {chi_rot[nn*4+3]=par_rot[i*4+3];}
      nn++;
    }
  }
}

void docking_sort_chi(const int docking_na,
                      const int docking_rot_bond_n,
                      const int docking_chi_n,
                      double *  docking_chi_ang,
                      double *  chi_pos,
                      double *  chi_rot,
                      double *  docking_chi_eng,
                      double *  docking_chi_eng_intra,
                      double *  docking_chi_eng_inter,
                      double *  docking_chi_x,
                      double *  docking_chi_y,
                      double *  docking_chi_z)
{
  int min_i,na;
  double dtmp;
  na=docking_na;
  for(int i=0;i<docking_chi_n;i++){
    min_i=i;
    for(int j=i+1;j<docking_chi_n;j++){
      if(docking_chi_eng[min_i]>docking_chi_eng[j]){
        min_i=j;
      }
    }
    // exchange ang
    for(int j=0;j<docking_rot_bond_n;j++){
      dtmp=docking_chi_ang[i*docking_rot_bond_n+j];
      docking_chi_ang[i*docking_rot_bond_n+j]=docking_chi_ang[min_i*docking_rot_bond_n+j];
      docking_chi_ang[min_i*docking_rot_bond_n+j]=dtmp;
    }
    // exchange pos
    for(int j=0;j<3;j++){
      dtmp=chi_pos[i*3+j];
      chi_pos[i*3+j]=chi_pos[min_i*3+j];
      chi_pos[min_i*3+j]=dtmp;
    }
    // exchange pos
    for(int j=0;j<4;j++){
      dtmp=chi_rot[i*4+j];
      chi_rot[i*4+j]=chi_rot[min_i*4+j];
      chi_rot[min_i*4+j]=dtmp;
    }
    // exchange total eng
    dtmp=docking_chi_eng[i];
    docking_chi_eng[i]=docking_chi_eng[min_i];
    docking_chi_eng[min_i]=dtmp;
    // exchange intra eng
    dtmp=docking_chi_eng_intra[i];
    docking_chi_eng_intra[i]=docking_chi_eng_intra[min_i];
    docking_chi_eng_intra[min_i]=dtmp;
    // exchange inter eng
    dtmp=docking_chi_eng_inter[i];
    docking_chi_eng_inter[i]=docking_chi_eng_inter[min_i];
    docking_chi_eng_inter[min_i]=dtmp;
    // exchange coord-x
    for(int j=0;j<na;j++){
      dtmp=docking_chi_x[na*i+j];
      docking_chi_x[na*i+j]=docking_chi_x[na*min_i+j];
      docking_chi_x[na*min_i+j]=dtmp;
    }
    // exchange coord-y
    for(int j=0;j<na;j++){
      dtmp=docking_chi_y[na*i+j];
      docking_chi_y[na*i+j]=docking_chi_y[na*min_i+j];
      docking_chi_y[na*min_i+j]=dtmp;
    }
    // exchange coord-z
    for(int j=0;j<na;j++){
      dtmp=docking_chi_z[na*i+j];
      docking_chi_z[na*i+j]=docking_chi_z[na*min_i+j];
      docking_chi_z[na*min_i+j]=dtmp;
    }
  }
}

void docking_select_par(const int      docking_na,
                        const int      docking_rot_bond_n,
                        const int      docking_rot_bond_main_n,
                        const int *    docking_rot_bond_main,
                        const int      docking_alpha_n,
                        const int *    docking_alpha,
                        const int      docking_par_n,
                        double *       docking_par_ang,
                        double *       par_pos,
                        double *       par_rot,
                        double *       docking_par_eng,
                        double *       docking_par_eng_intra,
                        double *       docking_par_eng_inter,
                        double *       docking_par_x,
                        double *       docking_par_y,
                        double *       docking_par_z,
                        const int      docking_chi_n,
                        const double * docking_chi_ang,
                        const double * chi_pos,
                        const double * chi_rot,
                        const double * docking_chi_eng,
                        const double * docking_chi_eng_intra,
                        const double * docking_chi_eng_inter,
                        const double * docking_chi_x,
                        const double * docking_chi_y,
                        const double * docking_chi_z)
{
  int na,chkchk,chk;
  int    * chi_selected;
  double * chi_ang_min_rmsd_main;
  double * chi_ang_min_rmsd;
  double * chi_pos_min_rmsd;
  double ang_rmsd_max=10.0; // degree
  double ang_rmsd_th;
  double pos_rmsd_max= 2.0 / 0.5291 ; // bohr
  double pos_rmsd_th;
  int select_2nd=docking_par_n/2;

  na=docking_na;

  chi_ang_min_rmsd_main = new double [docking_chi_n+1];
  chi_ang_min_rmsd      = new double [docking_chi_n+1];
  chi_pos_min_rmsd      = new double [docking_chi_n+1];
  chi_selected          = new int    [docking_chi_n+1];

  for(int i=0;i<docking_chi_n;i++){
    chi_selected[i]=0;
  }
  for(int i=0;i<docking_par_n;i++){

    ////////////////////
    // calc min. rmsd //
    ////////////////////

    for(int j=0;j<docking_chi_n;j++){

      chi_ang_min_rmsd_main[j]=
	docking_ang_min_rmsd_main(docking_rot_bond_n,
                                  docking_rot_bond_main_n,
                                  docking_rot_bond_main,
                                  i,
                                  &docking_par_ang[0*docking_rot_bond_n],
                                  &docking_chi_ang[j*docking_rot_bond_n]);
      chi_ang_min_rmsd[j]=
        docking_ang_min_rmsd     (docking_rot_bond_n,
                                  i,
                                  &docking_par_ang[0*docking_rot_bond_n],
                                  &docking_chi_ang[j*docking_rot_bond_n]);

      chi_pos_min_rmsd[j]=
        docking_pos_min_rmsd     (i,
                                  &par_pos[0*3],
                                  &chi_pos[j*3]);
    }

    /////////////////
    // select par. //
    /////////////////

    ang_rmsd_th=ang_rmsd_max;
    pos_rmsd_th=pos_rmsd_max;

    while (1) {

      chkchk=0;
      for(int k=0;k<docking_chi_n;k++){

        chk=0;

        if(i<select_2nd){if( chi_selected[k]==0 && (chi_ang_min_rmsd[k]>ang_rmsd_th) ){chk=1;}}
        else            {if( chi_selected[k]==0 && (chi_pos_min_rmsd[k]>pos_rmsd_th) ){chk=1;}}

        if(chk==1){
          // copy eng
          docking_par_eng[i]      =docking_chi_eng[k];
          docking_par_eng_intra[i]=docking_chi_eng_intra[k];
          docking_par_eng_inter[i]=docking_chi_eng_inter[k];
          // copy xyz
          for(int j=0;j<na;j++){
            docking_par_x[na*i+j]=docking_chi_x[na*k+j];
            docking_par_y[na*i+j]=docking_chi_y[na*k+j];
            docking_par_z[na*i+j]=docking_chi_z[na*k+j];
          }
          // copy ang
          for(int j=0;j<docking_rot_bond_n;j++){
            docking_par_ang[i*docking_rot_bond_n+j]=docking_chi_ang[k*docking_rot_bond_n+j];
          }
          // copy pos
          for(int j=0;j<3;j++){
            par_pos[i*3+j]=chi_pos[k*3+j];
          }
          // copy rot
          for(int j=0;j<4;j++){
            par_rot[i*4+j]=chi_rot[k*4+j];
          }
          chi_selected[k]=1;
          chkchk=1;
          break;
        }
        
      }
      if(chkchk==1){
        break;
      } else {
        if(i<select_2nd){ang_rmsd_th=ang_rmsd_th*0.5;}
        else            {pos_rmsd_th=pos_rmsd_th*0.5;}
        cout << "ishi , change threshold , ang_rmsd_th (degree) = " << ang_rmsd_th << " , pos_rmsd_th (A) = " << pos_rmsd_th * 0.5291 << endl ; 
      }
    }
  }
  delete [] chi_ang_min_rmsd_main;
  delete [] chi_ang_min_rmsd;
  delete [] chi_pos_min_rmsd;
  delete [] chi_selected;
}

void docking_next_par(const int         grid_use,
                      const long int    random_seed,
                      const int         atom_n,
                      const int       * atomic,
                      const int         center_atom,
                      const double    * x,
                      const double    * y,
                      const double    * z,
                      const int         bond_n,
                      const int       * bond_1,
                      const int       * bond_2,
                      const int         pair_n,
                      const int       * pair_1,
                      const int       * pair_2,
                      const double    * chg,
                      const double    * vdw_e,
                      const double    * vdw_r,
                      const double    * coeffA,
                      const double    * coeffB,
                      const int         pro_atom_n,
                      const double    * pro_x,
                      const double    * pro_y,
                      const double    * pro_z,
                      const double    * pro_chg,
                      const double    * pro_vdw_e,
                      const double    * pro_vdw_r,
                      const int       * type_i,
                      const int         grid_ntype,
                      const int         grid_nx,
                      const int         grid_ny,
                      const int         grid_nz,
                      const double      grid_d,
                      const double    * grid,
                      const int         rot_bond_n,
                      const int       * rot_bond,
                      const int         rot_bond_main_n,
                      const int       * rot_bond_main,
                      const int       * rot_bond_atom_n,
                      const int       * rot_bond_atom,
                      const int         alpha_n,
                      const int       * alpha,
                      const int         par_n,
                      const int         chk_debug_compare,
                      double          * par_ang,
                      double          * par_pos,
                      double          * par_rot,
                      double          * region,
                      double          * par_eng,
                      double          * par_eng_intra,
                      double          * par_eng_inter,
                      double          * par_x,
                      double          * par_y,
                      double          * par_z,
                      const int         chi_n,
                      double          * chi_x,
                      double          * chi_y,
                      double          * chi_z,
                      double          * chi_eng,
                      double          * chi_eng_intra,
                      double          * chi_eng_inter,
                      long int        * n_internal,
                      long int        * n_external,
                      clock_t         * t1,
                      clock_t         * t2,
                      clock_t         * t3,
                      clock_t         * t4)
{
  clock_t tt1,tt2;

  double * chi_ang              = new double [chi_n*rot_bond_n];
  double * chi_pos              = new double [chi_n*3+1];
  double * chi_rot              = new double [chi_n*4+1];
//double * chi_eng              = new double [chi_n]; 
//double * chi_eng_intra        = new double [chi_n]; 
  double * chi_eng_intra_chg    = new double [chi_n]; 
  double * chi_eng_intra_vdw    = new double [chi_n]; 
//double * chi_eng_inter        = new double [chi_n]; 
  double * chi_eng_inter_chg    = new double [chi_n]; 
  double * chi_eng_inter_vdw    = new double [chi_n]; 
  int *    ix                   = new int    [chi_n*atom_n+1];
  int *    iy                   = new int    [chi_n*atom_n+1];
  int *    iz                   = new int    [chi_n*atom_n+1];
  double * dx                   = new double [chi_n*atom_n+1];
  double * dy                   = new double [chi_n*atom_n+1];
  double * dz                   = new double [chi_n*atom_n+1];
  double * eng_inter_vdw        = new double [chi_n+1];
  double * eng_inter_chg        = new double [chi_n+1];
  double * eng_inter_vdw_grid   = new double [chi_n+1];
  double * eng_inter_chg_grid   = new double [chi_n+1];

  //cout << par_eng_inter[par_n-1] << endl ;

  /////////////////////
  // make chi. angle //
  /////////////////////

  docking_make_chi_angle(random_seed,
                         rot_bond_n,
                         par_n,
                         par_ang,
                         par_pos,
                         par_rot,
                         region,
                         chi_n,
                         chi_ang,
                         chi_pos,
                         chi_rot);
  
  ////////////////////////////
  // make chi. conformation //
  ////////////////////////////
  
  tt1=clock();
  docking_make_conformation(chi_n,
                            atom_n,
                            atomic,
                            center_atom,
                            x,
                            y,
                            z,
                            bond_1,
                            bond_2,
                            rot_bond_n,
                            rot_bond,
                            rot_bond_atom_n,
                            rot_bond_atom,
                            chi_ang,
                            chi_pos,
                            chi_rot,
                            chi_x,
                            chi_y,
                            chi_z);
  
  tt2=clock();
  (*t3)=(*t3)+(tt2-tt1);

  ///////////////////////////////
  // make chi. external energy //
  ///////////////////////////////

  tt1=clock();
  docking_make_grid_position(chi_n,
                             atom_n,
                             grid_nx,
                             grid_ny,
                             grid_nz,
                             region[0],
                             region[2],
                             region[4],
                             grid_d,
                             chi_x,
                             chi_y,
                             chi_z,
                             ix,
                             iy,
                             iz,
                             dx,
                             dy,
                             dz);
  if(grid_use==0 || chk_debug_compare==1){
    docking_energy_external(chi_n,
                            atom_n,
                            chi_x,
                            chi_y,
                            chi_z,
                            ix,
                            iy,
                            iz,
                            chg,
                            vdw_e,
                            vdw_r,
                            pro_atom_n,
                            pro_x,
                            pro_y,
                            pro_z,
                            pro_chg,
                            pro_vdw_e,
                            pro_vdw_r,
                            eng_inter_chg,
                            eng_inter_vdw,
                            n_external);
  }
  if(grid_use!=0){
    docking_energy_external_grid(chi_n,
                                 atom_n,
                                 chi_x,
                                 chi_y,
                                 chi_z,
                                 type_i,
                                 grid_ntype,
                                 grid_nx,
                                 grid_ny,
                                 grid_nz,
                                 region[0],
                                 region[2],
                                 region[4],
                                 grid_d,
                                 grid,
                                 ix,
                                 iy,
                                 iz,
                                 dx,
                                 dy,
                                 dz,
                                 eng_inter_chg_grid,
                                 eng_inter_vdw_grid,
                                 n_external);
  }
  if(grid_use==0){
    for(int i=0;i<chi_n;i++){
      chi_eng_inter_vdw[i]=eng_inter_vdw[i];
      chi_eng_inter_chg[i]=eng_inter_chg[i];
      chi_eng_inter[i]=chi_eng_inter_vdw[i]+chi_eng_inter_chg[i];
    }
  } else {
    for(int i=0;i<chi_n;i++){
      chi_eng_inter_vdw[i]=eng_inter_vdw_grid[i];
      chi_eng_inter_chg[i]=eng_inter_chg_grid[i];
      chi_eng_inter[i]=chi_eng_inter_vdw[i]+chi_eng_inter_chg[i];
    }
  }
  if(chk_debug_compare==1){
      cout << endl ;
      for(int i=0;i<chi_n;i++){
        cout.width(10); cout << i ;
        cout.width(20); cout << eng_inter_vdw     [i];
        cout.width(20); cout << eng_inter_vdw_grid[i];
        cout << endl ;
      }
      cout << endl ;
  }
  tt2=clock();
  (*t1)=(*t1)+(tt2-tt1);

  ///////////////////////////////
  // make chi. internal energy //
  ///////////////////////////////
  
  tt1=clock();
  docking_energy_internal (chi_n,
                           atom_n,
                           pair_n,
                           pair_1,
                           pair_2,
                           chi_x,
                           chi_y,
                           chi_z,
                           chg,
                           vdw_e,
                           vdw_r,
                           coeffA,
                           coeffB,
                           chi_eng_inter,
                           (par_eng_inter[par_n-1]+1.0E+2),
                           chi_eng_intra_chg,
                           chi_eng_intra_vdw,
                           n_internal);
  for(int i=0;i<chi_n;i++){
    chi_eng_intra[i]=chi_eng_intra_chg[i]+chi_eng_intra_vdw[i];
  }
  tt2=clock();
  (*t2)=(*t2)+(tt2-tt1);
  
  //////////////////////
  // make chi. energy //
  //////////////////////
  
  for(int i=0;i<chi_n;i++){
    chi_eng[i]=chi_eng_intra[i]+chi_eng_inter[i];
  }
  
  ///////////////
  // sort chi. //
  ///////////////
  
  tt1=clock();
  docking_sort_chi (atom_n,
                    rot_bond_n,
                    chi_n,
                    chi_ang,
                    chi_pos,
                    chi_rot,
                    chi_eng,
                    chi_eng_intra,
                    chi_eng_inter,
                    chi_x,
                    chi_y,
                    chi_z);

  /////////////////
  // select par. //
  /////////////////

  docking_select_par (atom_n,
                      rot_bond_n,
                      rot_bond_main_n,
                      rot_bond_main,
                      alpha_n,
                      alpha,
                      par_n,
                      par_ang,
                      par_pos,
                      par_rot,
                      par_eng,
                      par_eng_intra,
                      par_eng_inter,
                      par_x,
                      par_y,
                      par_z,
                      chi_n,
                      chi_ang,
                      chi_pos,
                      chi_rot,
                      chi_eng,
                      chi_eng_intra,
                      chi_eng_inter,
                      chi_x,
                      chi_y,
                      chi_z);
  tt2=clock();
  (*t4)=(*t4)+(tt2-tt1);
  
  delete [] chi_ang;
  delete [] chi_pos;
  delete [] chi_rot;
  //delete [] chi_eng;
  //delete [] chi_eng_inter;
  delete [] chi_eng_inter_chg;
  delete [] chi_eng_inter_vdw;
  //delete [] chi_eng_intra;
  delete [] chi_eng_intra_chg;
  delete [] chi_eng_intra_vdw;
  delete [] ix ;
  delete [] iy ;
  delete [] iz ;
  delete [] dx ;
  delete [] dy ;
  delete [] dz ;
  delete [] eng_inter_vdw;
  delete [] eng_inter_chg;
  delete [] eng_inter_vdw_grid;
  delete [] eng_inter_chg_grid;
}

void docking_print_par(const int      na,
                       const int      n,
                       const int      n_itr,
                       const int      i_start,
                       const int      docking_rot_bond_n,
                       const int      docking_rot_bond_main_n,
                       const int    * docking_rot_bond_main,
                       const int      docking_alpha_n,
                       const int    * docking_alpha,
                       const double * docking_par_ang,
                       const double * docking_par_pos,
                       const double * docking_par_rot,
                       const double * docking_par_eng,
                       const double * docking_par_eng_intra,
                       const double * docking_par_eng_inter,
                       const int      flag,
                       string&        s_out)
{
  // flag = 1 : normal print out
  // flag = 2 : small print out
  // flag = 3 : only energy
  
  stringstream ss;
  double ang_min_rmsd_main,ang_min_rmsd,pos_min_rmsd;

  for(int i=i_start;i<(i_start+n);i++){

    ////////////////////
    // calc. min rmsd //
    ////////////////////

    ang_min_rmsd_main=
      docking_ang_min_rmsd_main(docking_rot_bond_n,
                                docking_rot_bond_main_n,
                                docking_rot_bond_main,
                                i,
                                &docking_par_ang[0*docking_rot_bond_n],
                                &docking_par_ang[i*docking_rot_bond_n]);
    
    ang_min_rmsd=
      docking_ang_min_rmsd     (docking_rot_bond_n,
                                i,
                                &docking_par_ang[0*docking_rot_bond_n],
                                &docking_par_ang[i*docking_rot_bond_n]);

    pos_min_rmsd=
      docking_pos_min_rmsd     (i,
                                &docking_par_pos[0*3],
                                &docking_par_pos[i*3]);
    
    ///////////
    // print //
    ///////////

    ss << " par :";
    ss.width(8); ss << n_itr ;
    ss.precision(3);
    if(docking_par_eng[i]      <1.0E+5){ss.width(12); ss << fixed      << docking_par_eng      [i];}
    else                                  {ss.width(12); ss << scientific << docking_par_eng      [i];}
    if(docking_par_eng_intra[i]<1.0E+5){ss.width(12); ss << fixed      << docking_par_eng_intra[i];}
    else                                  {ss.width(12); ss << scientific << docking_par_eng_intra[i];}
    if(docking_par_eng_inter[i]<1.0E+5){ss.width(12); ss << fixed      << docking_par_eng_inter[i];}
    else                                  {ss.width(12); ss << scientific << docking_par_eng_inter[i];}
    if(flag==1 || flag==2) {
      ss.precision(2);
      ss.setf(ios::fixed,ios::floatfield);
      if(i==0){
        ss.width(8);ss << "-" ;
        ss.width(8);ss << "-" ;
        ss.width(8);ss << "-" ;
      } else {
        if(docking_rot_bond_main_n>0){
          ss.width(8);ss << ang_min_rmsd_main;
        } else {
          ss.width(8);ss << "-" ;
        }          
        ss.width(8);ss << ang_min_rmsd;
        ss.width(8);ss << pos_min_rmsd * 0.5291 ;
      }
    }
    if(flag==1){
      ss.width(8); ss << docking_par_pos[i*3+0] * 0.5291;
      ss.width(8); ss << docking_par_pos[i*3+1] * 0.5291;
      ss.width(8); ss << docking_par_pos[i*3+2] * 0.5291;
      ss.width(8); ss << docking_par_rot[i*4+0];
      ss.width(8); ss << docking_par_rot[i*4+1];
      ss.width(8); ss << docking_par_rot[i*4+2];
      ss.width(8); ss << docking_par_rot[i*4+3];
      ss << endl ;
      ss.width(74); ss << "" ;
      ss.precision(1);
      for(int j=0;j<docking_rot_bond_n;j++){
        ss.width(8); ss << docking_par_ang[i*docking_rot_bond_n+j] ;
        if((j+1)%7==0){
          ss << endl; 
          if((j+1)!=docking_rot_bond_n){
            ss.width(74); ss << "" ;
          }
        }
      }
      if((docking_rot_bond_n%7)!=0){
        ss << endl; 
      }
    } else {
      ss << endl ;
    }
  }
  ss << endl; 
  s_out=ss.str();
}


void docking_set_grid_from_file(const string filename,
                                string * grid_type,
                                double * grid,
                                double * grid_region)
{
  int n1,n2,n3;
  int grid_type_n;
  int grid_nx;
  int grid_ny;
  int grid_nz;
  double grid_center_x;
  double grid_center_y;
  double grid_center_z;
  double grid_d;
  ifstream fin(filename.c_str(), ios_base::in);
  fin >> grid_type_n ;
  fin >> grid_nx ;
  fin >> grid_ny ;
  fin >> grid_nz ;
  fin >> grid_center_x ; grid_center_x = grid_center_x / 0.5291;
  fin >> grid_center_y ; grid_center_y = grid_center_y / 0.5291;
  fin >> grid_center_z ; grid_center_z = grid_center_z / 0.5291;
  fin >> grid_d        ; grid_d        = grid_d        / 0.5291;
  for(int i=0;i<grid_type_n;i++){
    fin >> grid_type[i] ;
  }
  n1=(grid_nz+1);
  n2=(grid_nz+1)*(grid_ny+1);
  n3=(grid_nz+1)*(grid_ny+1)*(grid_nz+1);
  for(int i=0;i<grid_type_n;i++){
    for(int ix=0;ix<=grid_nx;ix++){
      for(int iy=0;iy<=grid_ny;iy++){
        for(int iz=0;iz<=grid_nz;iz++){
          fin >> grid[i*n3+ix*n2+iy*n1+iz];
        }
      }
    }
  }
  grid_region[0]=grid_center_x-(grid_nx/2)*grid_d;
  grid_region[1]=grid_center_x+(grid_nx/2)*grid_d;
  grid_region[2]=grid_center_y-(grid_ny/2)*grid_d;
  grid_region[3]=grid_center_y+(grid_ny/2)*grid_d;
  grid_region[4]=grid_center_z-(grid_nz/2)*grid_d;
  grid_region[5]=grid_center_z+(grid_nz/2)*grid_d;
  fin.close();
}

void docking_set_grid_from_file_head_no_grid_value(const string file_head,
                                                   string * grid_type,
                                                   double * grid_region)
{
  int n1,n2,n3;
  int grid_type_n;
  int grid_nx;
  int grid_ny;
  int grid_nz;
  double grid_center_x;
  double grid_center_y;
  double grid_center_z;
  double grid_d;
  stringstream ss1;
  ss1 << file_head;
  ss1 >> grid_type_n ;
  ss1 >> grid_nx ;
  ss1 >> grid_ny ;
  ss1 >> grid_nz ;
  ss1 >> grid_center_x ; grid_center_x = grid_center_x / 0.5291;
  ss1 >> grid_center_y ; grid_center_y = grid_center_y / 0.5291;
  ss1 >> grid_center_z ; grid_center_z = grid_center_z / 0.5291;
  ss1 >> grid_d        ; grid_d        = grid_d        / 0.5291;
  for(int i=0;i<grid_type_n;i++){
    ss1 >> grid_type[i] ;
  }
  grid_region[0]=grid_center_x-(grid_nx/2)*grid_d;
  grid_region[1]=grid_center_x+(grid_nx/2)*grid_d;
  grid_region[2]=grid_center_y-(grid_ny/2)*grid_d;
  grid_region[3]=grid_center_y+(grid_ny/2)*grid_d;
  grid_region[4]=grid_center_z-(grid_nz/2)*grid_d;
  grid_region[5]=grid_center_z+(grid_nz/2)*grid_d;
}

void docking_get_grid_info_from_file_head(const string file_head,
                                          int *    grid_nx,
                                          int *    grid_ny,
                                          int *    grid_nz,
                                          double * grid_center_x,
                                          double * grid_center_y,
                                          double * grid_center_z,
                                          double * grid_d,
                                          int *    grid_n,
                                          int *    grid_type_n)
{
  stringstream ss1;
  ss1 << file_head;
  ss1 >> *grid_type_n ;
  ss1 >> *grid_nx ;
  ss1 >> *grid_ny ;
  ss1 >> *grid_nz ;
  ss1 >> *grid_center_x ; (*grid_center_x) = (*grid_center_x) / 0.5291;
  ss1 >> *grid_center_y ; (*grid_center_y) = (*grid_center_y) / 0.5291;
  ss1 >> *grid_center_z ; (*grid_center_z) = (*grid_center_z) / 0.5291;
  ss1 >> *grid_d        ; (*grid_d)        = (*grid_d)        / 0.5291;
  *grid_n=((*grid_nx)+1)*((*grid_ny)+1)*((*grid_nz)+1);
}

void docking_get_grid_info_from_file(const string filename,
                                     int *    grid_nx,
                                     int *    grid_ny,
                                     int *    grid_nz,
                                     double * grid_center_x,
                                     double * grid_center_y,
                                     double * grid_center_z,
                                     double * grid_d,
                                     int *    grid_n,
                                     int *    grid_type_n)
{
  ifstream fin(filename.c_str(), ios_base::in);
  fin >> *grid_type_n ;
  fin >> *grid_nx ;
  fin >> *grid_ny ;
  fin >> *grid_nz ;
  fin >> *grid_center_x ; (*grid_center_x) = (*grid_center_x) / 0.5291;
  fin >> *grid_center_y ; (*grid_center_y) = (*grid_center_y) / 0.5291;
  fin >> *grid_center_z ; (*grid_center_z) = (*grid_center_z) / 0.5291;
  fin >> *grid_d        ; (*grid_d)        = (*grid_d)        / 0.5291;
  fin.close();
  *grid_n=((*grid_nx)+1)*((*grid_ny)+1)*((*grid_nz)+1);
}

void docking_write_grid(const string  filename,
                        const int     grid_type_n,
                        const int     grid_nx,
                        const int     grid_ny,
                        const int     grid_nz,
                        const double  grid_center_x,
                        const double  grid_center_y,
                        const double  grid_center_z,
                        const double  grid_d,
                        const string* grid_type,
                        const double* grid)
{
  int n,n1,n2,n3;
  ofstream fout(filename.c_str(), ios_base::out);
  fout.width( 6); fout << grid_type_n ;
  fout.width( 6); fout << grid_nx ;
  fout.width( 6); fout << grid_ny ;
  fout.width( 6); fout << grid_nz ;
  fout.precision(6);
  fout.width(16); fout << scientific << grid_center_x * 0.5291;
  fout.width(16); fout << scientific << grid_center_y * 0.5291;
  fout.width(16); fout << scientific << grid_center_z * 0.5291;
  fout.width(16); fout << scientific << grid_d        * 0.5291;
  fout << endl ;
  for(int i=0;i<grid_type_n;i++){
    fout.width(6); fout << grid_type[i] ;
  }
  fout << endl ;
  n1=(grid_nz+1);
  n2=(grid_nz+1)*(grid_ny+1);
  n3=(grid_nz+1)*(grid_ny+1)*(grid_nz+1);
  for(int i=0;i<grid_type_n;i++){
    n=0;
    for(int ix=0;ix<=grid_nx;ix++){
      for(int iy=0;iy<=grid_ny;iy++){
        for(int iz=0;iz<=grid_nz;iz++){
          fout.width(16); fout << scientific << grid[i*n3+ix*n2+iy*n1+iz];
          n++;
          if(n%10==0){
            fout << endl ;
          }
        }
      }
    }
    if(n3%10!=0){
      fout << endl ;
    }
  }
  fout.close();
}

void docking_set_grid(const int     grid_type_n,
                      const int     grid_nx,
                      const int     grid_ny,
                      const int     grid_nz,
                      const double  grid_center_x,
                      const double  grid_center_y,
                      const double  grid_center_z,
                      const double  grid_d,
                      const int     all_type_n,
                      const string* all_type,
                      const double* all_type_vdw_e,
                      const double* all_type_vdw_r,
                      const int     pro_atom_n,
                      const double* pro_x,
                      const double* pro_y,
                      const double* pro_z,
                      const double* pro_vdw_e,
                      const double* pro_vdw_r,                             
                      string *      grid_type,
                      double *      grid,
                      double *      grid_region)
{
  int n1,n2,n3;
  double chk,x1,y1,z1,x2,y2,z2,d,coeffA_,coeffB_,v1,vdw_e1,vdw_r1,vdw_e2,vdw_r2,grid_0x,grid_0y,grid_0z;
  double * coeffA;
  double * coeffB;

  n1=grid_nz+1;
  n2=(grid_nz+1)*(grid_ny+1);
  n3=(grid_nz+1)*(grid_ny+1)*(grid_nz+1);

  grid_region[0]=grid_center_x-(grid_nx/2)*grid_d;
  grid_region[1]=grid_center_x+(grid_nx/2)*grid_d;
  grid_region[2]=grid_center_y-(grid_ny/2)*grid_d;
  grid_region[3]=grid_center_y+(grid_ny/2)*grid_d;
  grid_region[4]=grid_center_z-(grid_nz/2)*grid_d;
  grid_region[5]=grid_center_z+(grid_nz/2)*grid_d;
   
  ////////////////////////////////////////////////////////////////
  ///// grid calculation form ligand and protein information /////
  ////////////////////////////////////////////////////////////////

  grid_0x=grid_region[0];
  grid_0y=grid_region[2];
  grid_0z=grid_region[4];

  for(int itype=0;itype<all_type_n;itype++){
    grid_type[itype]=all_type[itype];
  }
  coeffA=new double [pro_atom_n*all_type_n+1];
  coeffB=new double [pro_atom_n*all_type_n+1];
  for(int k=0;k<pro_atom_n;k++){
    vdw_e2=pro_vdw_e[k];
    vdw_r2=pro_vdw_r[k]*0.5;
    for(int itype=0;itype<all_type_n;itype++){
      vdw_e1=all_type_vdw_e[itype];
      vdw_r1=all_type_vdw_r[itype]*0.5;
      coeffA[all_type_n*k+itype]=    sqrt(vdw_e1*vdw_e2)*pow((vdw_r1+vdw_r2),12);
      coeffB[all_type_n*k+itype]=2.0*sqrt(vdw_e1*vdw_e2)*pow((vdw_r1+vdw_r2), 6);
    }
  }
  for(int ix=0;ix<=grid_nx;ix++){
    for(int iy=0;iy<=grid_ny;iy++){
      for(int iz=0;iz<=grid_nz;iz++){
        x1=grid_0x+ix*grid_d;
        y1=grid_0y+iy*grid_d;
        z1=grid_0z+iz*grid_d;
        for(int k=0;k<pro_atom_n;k++){
          x2=pro_x[k];
          y2=pro_y[k];
          z2=pro_z[k];
          d=sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2))*0.5291772108;
          for(int itype=0;itype<all_type_n;itype++){
            coeffA_=coeffA[all_type_n*k+itype];
            coeffB_=coeffB[all_type_n*k+itype];
            v1=coeffA_/pow(d,12)-coeffB_/pow(d,6);
            grid[itype*n3+ix*n2+iy*n1+iz]=grid[itype*n3+ix*n2+iy*n1+iz]+v1;
          }
        }
      }
    }
    cout << " make grid : " ;
    cout.width(4) ; cout << ix      ;  cout << " / ";
    cout.width(4) ; cout << grid_nx ;
    cout << endl ;
  }
  delete [] coeffA;
  delete [] coeffB;
}

void docking_assign_atom_type_to_grid(const int     lig_atom_n,
                                      const string* lig_atom_type,
                                      const int     grid_type_n,
                                      const string* grid_type,
                                      int *         lig_atom_type_i)
{
  int chk;
  
  for(int i=0;i<lig_atom_n;i++){
    chk=0;
    for(int j=0;j<grid_type_n;j++){
      if(lig_atom_type[i]==grid_type[j]){
        lig_atom_type_i[i]=j;
        chk=1;
        break;
      }
    }
    if(chk==0){
      cout << "assign error!" << endl ;
      exit(1);
    }
  }
}

void docking_write_pdb_1(const string   filename,
                         const int      atom_n,
                         const int    * atomic,
                         const int      center_atom,
                         const double * x_orig,
                         const double * y_orig,
                         const double * z_orig,
                         const int      bond_n,
                         const int    * bond_1,
                         const int    * bond_2,
                         const int      rot_bond_n,
                         const int    * rot_bond,
                         const int    * rot_bond_atom_n,
                         const int    * rot_bond_atom,
                         const int      grid_nx,
                         const int      grid_ny,
                         const int      grid_nz,
                         const double   grid_0x,
                         const double   grid_0y,
                         const double   grid_0z,
                         const double   grid_d,                                    
                         const double * par_ang,
                         const double * par_pos,
                         const double * par_rot)
{
  Molecule m;
  Atom a;
  Dvector3D r,r1,r2,r12;
  int a1,a2;
  string s_out;
  double * x_out  ;
  double * y_out  ;
  double * z_out  ;
  int    * ix_out ;
  int    * iy_out ;
  int    * iz_out ;
  double * dx_out ;
  double * dy_out ;
  double * dz_out ;
  
  x_out  = new double [atom_n+1];
  y_out  = new double [atom_n+1];
  z_out  = new double [atom_n+1];
  ix_out = new int    [atom_n+1];
  iy_out = new int    [atom_n+1];
  iz_out = new int    [atom_n+1];
  dx_out = new double [atom_n+1];
  dy_out = new double [atom_n+1];
  dz_out = new double [atom_n+1];

  docking_make_conformation(1,
                            atom_n,
                            atomic,
                            center_atom,
                            x_orig,
                            y_orig,
                            z_orig,
                            bond_1,
                            bond_2,
                            rot_bond_n,
                            rot_bond,
                            rot_bond_atom_n,
                            rot_bond_atom,
                            par_ang,
                            par_pos,
                            par_rot,
                            x_out,
                            y_out,
                            z_out);
  
  m.clear();
  for(int j=0;j<atom_n;j++){
    a.set_atomic_number(atomic[j]);
    r.set_xyz(x_out[j],y_out[j],z_out[j]);
    a.set_R(r);
    m.add_atom(a);
  }
  for(int j=0;j<bond_n;j++){
    a1=bond_1[j];
    a2=bond_2[j];
    m.add_bond(a1,a2);
  }

  m.make_linkgroup(s_out);
  m.write_pdb(filename);

  delete [] x_out  ;
  delete [] y_out  ;
  delete [] z_out  ;
  delete [] ix_out ;
  delete [] iy_out ;
  delete [] iz_out ;
  delete [] dx_out ;
  delete [] dy_out ;
  delete [] dz_out ;
}
