
#include <cstdio>
#include <cstdlib>
#include <math.h>
#include "Molecule.h"
#include "BioUnit.h"
#include "iString.h"


/* private method */
void Molecule::create_biounit(const int n)
{
  clear_biounit();
  if(n==0) return ;
  biounit = new BioUnit [n];
  nbiounit_buff=n;
}

/* private method */
void Molecule::resize_biounit(const int n)
{
  int tmp_nbiounit=nbiounit;
  BioUnit* tmp_biounit;
  tmp_biounit = new BioUnit [tmp_nbiounit+1];
  for(int i=0;i<tmp_nbiounit;i++){
    tmp_biounit[i].copy(biounit[i]);
  }
  create_biounit(n);
  if(n>tmp_nbiounit){
    for(int i=0;i<tmp_nbiounit;i++){
      push_back_biounit(tmp_biounit[i]);
    }
  } else {
    for(int i=0;i<n;i++){
      push_back_biounit(tmp_biounit[i]);
    }
  }
  delete [] tmp_biounit ;
}

/* private method */  
void Molecule::push_back_biounit ( const BioUnit& bu ,
                                   const int buff_increase )
{
  int type,iatm,ityp,ialp,itmp;
  if(nbiounit==nbiounit_buff){
    resize_biounit(nbiounit+buff_increase);
  }
  biounit[nbiounit].copy(bu);
  itmp=bu.get_c0();
  type=bu.get_type();
  if(itmp!=-1){
    ialp=bu.get_atom_index(itmp);
  } else {
    ialp=-1;
  }
  for(int j=0;j<bu.get_n_atom_index();j++){
    iatm=bu.get_atom_index(j);
    ityp=bu.get_atom_type(j);
    atom[iatm].set_bio1(ityp);
    atom[iatm].set_bio2(ialp);
    atom[iatm].set_bio3(nbiounit);
    atom[iatm].set_bio4(type);
  }
  nbiounit++;
} 

/* private method */
void Molecule::pick_up_biounit (const int ibiounit,
                                const int ierr)
{
  if(ibiounit<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::pick_up_biounit , ibiounit < 0 " << endl; 
      cerr << "ibiounit = " << ibiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibiounit>=nbiounit){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::pick_up_biounit : ibiounit >= nbiounit" << endl ;
      cerr << "ibiounit = " << ibiounit << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  for(int i=0;i<biounit[ibiounit].get_n_atom_index();i++){
    atom[biounit[ibiounit].get_atom_index(i)].set_bio1(-1);
    atom[biounit[ibiounit].get_atom_index(i)].set_bio2(-1);
    atom[biounit[ibiounit].get_atom_index(i)].set_bio3(-1);
    atom[biounit[ibiounit].get_atom_index(i)].set_bio4(-1);
  }
  for(int i=ibiounit;i<nbiounit-1;i++){
    biounit[i].copy(biounit[i+1]);
  }
  nbiounit--;
  int in,ic;
  for(int i=0;i<nbiounit;i++){
    in=biounit[i].get_link_unit_n();
    ic=biounit[i].get_link_unit_c();
    if(in> ibiounit){biounit[i].set_link_unit_n(in-1);}
    if(ic> ibiounit){biounit[i].set_link_unit_c(ic-1);}
    // 20130919
    if(in==ibiounit){biounit[i].set_link_unit_n(-1);}
    if(ic==ibiounit){biounit[i].set_link_unit_c(-1);}
  }
}

void Molecule::clear_biounit()
{
  if(nbiounit==0) return ;
  for(int i=(nbiounit-1);i>=0;i--){
    pick_up_biounit(i);
  }
  delete [] biounit ;
  nbiounit_buff=0;
}

int Molecule::get_ibiounit_from_iatom(const int iatom) const
{
  if(iatom<0){
    cerr << "error in get_ibiounit_from_iatom , iatom < 0" << endl ;
    cerr << "iatom = " << iatom << " , this = " << this << endl ;
    exit(1);
  }
  if(iatom>=natom){
    cerr << "error in get_ibiounit_from_iatom , iatom >= natom" << endl ;
    cerr << "iatom = " << iatom << " , natom " << natom << " , this = " << this << endl ;
    exit(1);
  }
  for(int i=0;i<nbiounit;i++){
    for(int j=0;j<biounit[i].get_n_atom_index();j++){
      if(iatom==biounit[i].get_atom_index(j)){
        return i;
      }
    }
  }
  return -1;
}

int Molecule::get_ibiounit_from_fragment(const int ifrag,
                                         int *     nbio ,
                                         int *     ibio ) const
{
  if(ifrag<0){
    cerr << "error , Molecule::get_ibiounit_from_fragment , ifrag < 0" << endl ;
    cerr << "ifrag = " << ifrag << " , this = " << this << endl ;
    exit(1);
  }
  if(ifrag>=nfragment){
    cerr << "error , Molecule::get_ibiounit_from_fragment , ifrag >= nfragment" << endl ;
    cerr << "ifrag = " << ifrag << " , nfragment = " << nfragment << " , this = " << this << endl ;
    exit(1);
  }
  int a,ia,t;
  int ibio_tmp=-1;
  if(nbio!=NULL) *nbio=0;
  for(int i=0;i<nbiounit;i++){
    t=biounit[i].get_type();
    ia=-1;
    if     (100 <=t && t<4000) ia=biounit[i].get_c0 ();
    else if(4000<=t && t<5000) ia=biounit[i].get_c20();
    else if(5000<=t          ) ia=biounit[i].get_n26();
    if(ia>=0){
      a=biounit[i].get_atom_index(ia);
      for(int j=0;j<fragment[ifrag].get_n_atom_index();j++){
        if(fragment[ifrag].get_atom_index(j)==a){
          if(nbio==NULL){
            return i;
          } else {
            if(ibio_tmp<0) ibio_tmp=i;
            if(ibio!=NULL) ibio[*nbio]=i;
            (*nbio)++;
          }
        }
      }
    }
  }
  if(nbio==NULL) return -1;
  if(*nbio==0) return -1;
  return ibio_tmp;
}

int Molecule::get_ibiounit_ss_bond(const int ibu) const
{
  int ia1,ia2,b1,b2,jbu;
  double dist,l1,l2,x1,y1,z1,x2,y2,z2;

  ia1=-1;
  if(biounit[ibu].get_type()==1550){
    for(int i=0;i<biounit[ibu].get_n_atom_index();i++){
      if(atom[biounit[ibu].get_atom_index(i)].get_atomic_number()==16){
        ia1=biounit[ibu].get_atom_index(i);
        break;
      }
    }
    if(ia1<0) {
      return -1;
    }
    /* ishi 20100726 */
    for(int i=0;i<nbond;i++){
      b1=bond[i].get_iatom1();
      b2=bond[i].get_iatom2();
      ia2=-1;
      if(b1==ia1){
        ia2=b2;
      }
      if(b2==ia1){
        ia2=b1;
      }
      if(ia2>=0){
        if(atom[ia2].get_atomic_number()==16){
          jbu=get_ibiounit_from_iatom(ia2);
          if(biounit[ibu].get_type()==1550){
            return jbu;
          }
        }
      }
    }
  }
  return -1;
}

int Molecule::get_biounit_type_from_iatom ( const int iatom ) const 
{
  int ibiounit = get_ibiounit_from_iatom(iatom);
  if(ibiounit==-1) return -1;
  return biounit[ibiounit].get_type();
}

int Molecule::get_biounit_atom_type_from_iatom ( const int iatom ) const
{
  int ibiounit = get_ibiounit_from_iatom(iatom);
  int atom_index;
  for(int i=0;i< biounit[ibiounit].get_n_atom_index();i++){
    atom_index=biounit[ibiounit].get_atom_index(i);
    if(atom_index==iatom){
      return biounit[ibiounit].get_atom_type(i);
    }
  }
  return -1;
}

int Molecule::get_nbiounit ( ) const
{
  return nbiounit ;
}

int Molecule::get_atom_index_biounit(const int nbio, 
                                     const int* ibio,
                                     int* out,
                                     const int ierr) const
{
  for(int i=0;i<nbio;i++){
    if(ibio[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::get_atom_index_biounit (2), ibio[i] < 0 " << endl; 
        cerr << "ibio[i] = " << ibio[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(ibio[i]>=nbiounit){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::get_atom_index_biounit (2), ibio[i] >= nbiounit" << endl ;
        cerr << "ibio[i] = " << ibio[i] << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  int n=0;
  for(int j=0;j<nbio;j++){
    if(0<=ibio[j] && ibio[j]<nbiounit){
      for(int i=0;i<biounit[ibio[j]].get_n_atom_index();i++){
        out[n]=biounit[ibio[j]].get_atom_index(i);
        n++;
      }
    }
  }
  return n;
}

int Molecule::get_atom_index_biounit_main(const int nbio,
                                          const int* ibio,
                                          int* out,
                                          const int ierr) const
{
  for(int i=0;i<nbio;i++){
    if(ibio[i]<0){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::get_atom_index_biounit_main (2), ibio[i] < 0 " << endl; 
        cerr << "ibio[i] = " << ibio[i] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
    if(ibio[i]>=nbiounit){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::get_atom_index_biounit_main (2), ibio[i] >= nbiounit" << endl ;
        cerr << "ibio[i] = " << ibio[i] << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    }
  }
  int n=0;
  for(int j=0;j<nbio;j++){
    if(0<=ibio[j] && ibio[j]<nbiounit){
      for(int i=0;i<biounit[ibio[j]].get_n_atom_index();i++){
        if(0<=biounit[ibio[j]].get_atom_type(i) && biounit[ibio[j]].get_atom_type(i)<=7){
          out[n]=biounit[ibio[j]].get_atom_index(i);
          n++;
        }
      }
    }
  }
  return n;
}

int Molecule::get_atom_index_biounit_side(const int nbio, 
                                          const int* ibio,
                                          int* out,
                                          const int ierr) const
{
  int n=0;
  for(int j=0;j<nbio;j++){
    if(ibio[j]<0){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::get_atom_index_biounit_side (2), ibio[j] < 0 " << endl; 
        cerr << "ibio[j] = " << ibio[j] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    } else if(ibio[j]>=nbiounit){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::get_atom_index_biounit_side (2), ibio[j] >= nbiounit" << endl ;
        cerr << "ibio[j] = " << ibio << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    } else {
      for(int i=0;i<biounit[ibio[j]].get_n_atom_index();i++){
        if((100<=biounit[ibio[j]].get_atom_type(i) && biounit[ibio[j]].get_atom_type(i)<=2100) ||
           (biounit[ibio[j]].get_atom_type(i)==-999)){
          out[n]=biounit[ibio[j]].get_atom_index(i);
          n++;
        }
      }
    }
  }
  return n;
}

int Molecule::get_atom_index_biounit_type(const int nbio,
                                          const int * ibio,
                                          const int type_,
                                          int* out,
                                          const int ierr) const 
{
  int n=0;
  for(int j=0;j<nbio;j++){
    if(ibio[j]<0){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::get_atom_index_biounit_type, ibio[j] < 0 " << endl; 
        cerr << "ibio[j] = " << ibio[j] << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    } else if(ibio[j]>=nbiounit){
      if(ierr==0 || ierr==1){
        cerr << "error in Molecule::get_atom_index_biounit_type, ibio[j] >= nbiounit" << endl ;
        cerr << "ibio[j] = " << ibio << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
      }
      if(ierr==0) exit(1);
    } else {
      for(int i=0;i<biounit[ibio[j]].get_n_atom_index();i++){
        if(type_==biounit[ibio[j]].get_atom_type(i)){
          out[n]=biounit[ibio[j]].get_atom_index(i);
          n++;
        }
      }
    }
  }
  return n;
}

BioUnit Molecule::get_biounit(const int ibio,
                              const int ierr) const
{
  if(ibio<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit, ibio < 0 " << endl; 
      cerr << "ibio = " << ibio << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibio>=nbiounit){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit, ibio >= nbiounit" << endl ;
      cerr << "ibio = " << ibio << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return biounit[ibio];
}

int Molecule::get_biounit_seq(const int ibio,
                              const int ierr) const
{
  if(ibio<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_seq, ibio < 0 " << endl; 
      cerr << "ibio = " << ibio << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibio>=nbiounit){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_seq, ibio >= nbiounit" << endl ;
      cerr << "ibio = " << ibio << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return biounit[ibio].get_seq();
}

string Molecule::get_biounit_abbr_3(const int ibio,
                                    const int ierr) const
{
  if(ibio<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_abbr_3, ibio < 0 " << endl; 
      cerr << "ibio = " << ibio << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibio>=nbiounit){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_abbr_3, ibio >= nbiounit" << endl ;
      cerr << "ibio = " << ibio << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  return biounit[ibio].get_abbr_3();
}

double Molecule::get_biounit_distance_from_atom ( const int ibu,
						  const int ja,
						  int *     iatom_,
						  const int ierr) const 
{
  if(ibu>nbiounit || ja>natom){
    // error
    return 0.0 ;
  }
  int n1=biounit[ibu].get_n_atom_index();
  Dvector3D r1;
  Dvector3D r2;
  double x1,x2,y1,y2,z1,z2,dist2,min_dist2;
  int min_iatom;
  min_dist2=999999.0;
  r2.copy(atom[ja].get_R());
  x2=r2.get_x();
  y2=r2.get_y();
  z2=r2.get_z();
  for(int i=0;i<n1;i++){
    r1.copy(atom[biounit[ibu].get_atom_index(i)].get_R());
    x1=r1.get_x();
    y1=r1.get_y();
    z1=r1.get_z();
    dist2=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
    if(dist2<min_dist2) {
      min_dist2=dist2;
      min_iatom=i;
    }
  }
  if(iatom_!=NULL){
    iatom_[0]=min_iatom;
  }
  //cout << "aaa " << ibu << " " << ja << " " << min_dist2 << endl ;
  return sqrt(min_dist2);
}

double Molecule::get_biounit_distance_from_linkgroup (const int ibu, 
                                                      const int jlink) const
{
  if(ibu>nbiounit || jlink>nlinkgroup){
    // error
    return 0.0 ;
  }
  int n1=biounit  [ibu  ].get_n_atom_index();
  int n2=linkgroup[jlink].get_n_atom_index();
  Dvector3D r1;
  Dvector3D r2;
  double x1,x2,y1,y2,z1,z2,dist2,min_dist2;
  min_dist2=999999.0;
  for(int i=0;i<n1;i++){
    r1.copy(atom[biounit[ibu].get_atom_index(i)].get_R());
    x1=r1.get_x();
    y1=r1.get_y();
    z1=r1.get_z();
    for(int j=0;j<n2;j++){
      r2.copy(atom[linkgroup[jlink].get_atom_index(j)].get_R());
      x2=r2.get_x();
      y2=r2.get_y();
      z2=r2.get_z();
      dist2=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
      if(dist2<min_dist2) min_dist2=dist2;
    }
  }
  return sqrt(min_dist2);
}

double Molecule::get_biounit_distance_from_biounit (const int ibu, 
                                                    const int jbu,
						    int * iatom_,
						    int * jatom_,
						    const int ierr) const
{
  if(ibu>nbiounit || jbu>nbiounit){
    // error
    return 0.0 ;
  }
  int n1=biounit[ibu].get_n_atom_index();
  int n2=biounit[jbu].get_n_atom_index();
  int min_ia,min_ja;
  Dvector3D r1;
  Dvector3D r2;
  double x1,x2,y1,y2,z1,z2,dist2,min_dist2;
  min_dist2=999999.0;
  min_ia=0;
  min_ja=0;
  for(int i=0;i<n1;i++){
    r1.copy(atom[biounit[ibu].get_atom_index(i)].get_R());
    x1=r1.get_x();
    y1=r1.get_y();
    z1=r1.get_z();
    for(int j=0;j<n2;j++){
      r2.copy(atom[biounit[jbu].get_atom_index(j)].get_R());
      x2=r2.get_x();
      y2=r2.get_y();
      z2=r2.get_z();
      dist2=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
      if(dist2<min_dist2) {
	min_dist2=dist2;
	min_ia=i;
	min_ja=j;
      }
    }
  }
  if(iatom_!=NULL && jatom_!=NULL){
    iatom_[0]=min_ia;
    jatom_[0]=min_ja;
  }
  return sqrt(min_dist2);
}

double Molecule::get_biounit_distance_from_fragment (const int ibu, 
                                                     const int jfrag) const
{
  if(ibu>nbiounit || jfrag>nfragment){
    // error
    return 0.0 ;
  }
  int n1=biounit [ibu  ].get_n_atom_index();
  int n2=fragment[jfrag].get_n_atom_index();
  Dvector3D r1;
  Dvector3D r2;
  double x1,x2,y1,y2,z1,z2,dist2,min_dist2;
  min_dist2=999999.0;
  for(int i=0;i<n1;i++){
    r1.copy(atom[biounit[ibu].get_atom_index(i)].get_R());
    x1=r1.get_x();
    y1=r1.get_y();
    z1=r1.get_z();
    for(int j=0;j<n2;j++){
      r2.copy(atom[fragment[jfrag].get_atom_index(j)].get_R());
      x2=r2.get_x();
      y2=r2.get_y();
      z2=r2.get_z();
      dist2=(x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2);
      if(dist2<min_dist2) min_dist2=dist2;
    }
  }
  return sqrt(min_dist2);
}

Dvector3D Molecule::get_biounit_center_of_mass( const int ibio ,
                                                const int ierr ) const
{
  if(ibio<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_center_of_mass, ibio < 0 " << endl; 
      cerr << "ibio = " << ibio << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibio>=nbiounit){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_center_of_mass, ibio >= nbiounit" << endl ;
      cerr << "ibio = " << ibio << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  double sumMass = get_biounit_total_mass(ibio);
  Dvector3D sum(0.00,0.00,0.00);
  int iatom;
  for(int i=0;i<biounit[ibio].get_n_atom_index();i++){
    iatom=biounit[ibio].get_atom_index(i);
    sum=sum+atom[iatom].get_R()*atom[iatom].get_mass_number();
  }
  if(-0.0000001 < sumMass && sumMass < 0.000001){
    //return innerAxis_origin;
  } else {
    return sum/sumMass;
  }
  return sum;
}




Dvector3D Molecule::get_biounit_center_of_mass_side( const int ibio ,
                                                     const int ierr ) const
{
  if(ibio<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_center_of_mass_side, ibio < 0 " << endl; 
      cerr << "ibio = " << ibio << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibio>=nbiounit){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_center_of_mass_side, ibio >= nbiounit" << endl ;
      cerr << "ibio = " << ibio << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  double sumMass = 0.0;
  Dvector3D sum(0.00,0.00,0.00);
  int iatom,type;
  for(int i=0;i<biounit[ibio].get_n_atom_index();i++){
    iatom=biounit[ibio].get_atom_index(i);
    type=biounit[ibio].get_atom_type(i);
    if(type>=100){
      //cout << " " << type << " " << iatom+1 << endl ;
      sum=sum+atom[iatom].get_R()*atom[iatom].get_mass_number();
      sumMass=sumMass+atom[iatom].get_mass_number();
    }
  }
  if(-0.0000001 < sumMass && sumMass < 0.000001){
    //return innerAxis_origin;
  } else {
    return sum/sumMass;
  }
  return sum;
}




Dvector3D Molecule::get_biounit_center_of_mass( const int nbu,
                                                const int* list_in) const
{
  int ibio;
  int iatom;
  double sumMass=0.0;
  Dvector3D sum(0.00,0.00,0.00);
  for(int i=0;i<nbu;i++){
    ibio=list_in[i];
    sumMass=sumMass+get_biounit_total_mass(ibio);
    for(int i=0;i<biounit[ibio].get_n_atom_index();i++){
      iatom=biounit[ibio].get_atom_index(i);
      sum=sum+atom[iatom].get_R()*atom[iatom].get_mass_number();
    }
  }
  if(-0.0000001 < sumMass && sumMass < 0.000001){
    //return innerAxis_origin;
  } else {
    return sum/sumMass;
  }
  return sum;
}

double Molecule::get_biounit_total_mass( const int ibio ,
                                         const int ierr ) const
{
  if(ibio<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_total_mass, ibio < 0 " << endl; 
      cerr << "ibio = " << ibio << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ibio>=nbiounit){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_total_mass, ibio >= nbiounit" << endl ;
      cerr << "ibio = " << ibio << " , nbiounit = " << nbiounit << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  double sum=0.0;
  for(int i=0;i<biounit[ibio].get_n_atom_index();i++){
    sum=sum+atom[biounit[ibio].get_atom_index(i)].get_mass_number();
  }
  return sum;
}

void Molecule::make_biounit_from_structure(string& s_out)
{
  clear_biounit();
  for(int i=0;i<get_nlinkgroup();i++){
    biounit_analysis(i,s_out);
  }
}

void Molecule::make_biounit_from_structure(const int nlg,
                                           const int* ilg,
                                           string& s_out)
{
  for(int i=0;i<nlg;i++){
    clear_biounit_linkgroup(ilg[i]);
    biounit_analysis(ilg[i],s_out);
  }
}

void Molecule::make_biounit_from_pdb_tag(string& s_out)
{
  clear_biounit();
  for(int i=0;i<get_nlinkgroup();i++){
    biounit_analysis_pdb_tag(i,s_out);
  }
}

void Molecule::make_biounit_from_pdb_tag(const int nlg,
                                         const int* ilg,
                                         string& s_out)
{
  for(int i=0;i<get_nlinkgroup();i++){
    clear_biounit_linkgroup(ilg[i]);
    biounit_analysis_pdb_tag(ilg[i],s_out);
  }
}

// private method //
void Molecule::biounit_analysis(const int igroup,
                                string& s_out)
{
  if(linkgroup[igroup].get_n_atom_index()<6){
    return ;
  }

  char c[1024];
  string ws="";
  int nw=0;

  sprintf(c,"\n"                                              ); s_out=s_out+c;
  sprintf(c,"  -----------------------------------\n"         ); s_out=s_out+c;
  sprintf(c,"   # make bio-unit @ link-group %d   \n",igroup+1); s_out=s_out+c;
  sprintf(c,"  -----------------------------------\n\n"       ); s_out=s_out+c;
  sprintf(c,"   number of atoms              = "              ); s_out=s_out+c;
  sprintf(c,"%8d\n",get_natom_linkgroup(igroup)               ); s_out=s_out+c;
  sprintf(c,"   number of atoms ( hydrogen ) = "              ); s_out=s_out+c;
  sprintf(c,"%8d\n\n",get_natom_linkgroup_atomic(igroup,1)    ); s_out=s_out+c;

  ///////////////////////////////////
  // allocation and initialization //
  ///////////////////////////////////

  int * group = new int [natom+1] ;
  int ** classification = new int* [natom+1] ;
  int imax=get_max_iatom_from_ilinkgroup(igroup);
  int imin=get_min_iatom_from_ilinkgroup(igroup);

  for(int i=0;i<natom;i++){
    group[i]=-1;
    if(imin<=i && i<=imax){
      if(chk_iatom_in_ilinkgroup(i,igroup)>=0){
        group[i]=igroup;
      }
    }
    classification[i] = new int [2] ;
    classification[i][0] = -1; // atom classification
    classification[i][1] = -1; // alpha carbon belonging
  }

  /////////////////////////////////
  // biounit atom classification //
  /////////////////////////////////

  biounit_atom_classification(igroup,group,classification,s_out,&nw,ws);
  
  //////////////////////////////////
  // check not classificated atom //
  //////////////////////////////////
  
  int n,nn,nb;
  nn=0;
  for(int j=0;j<natom;j++){
    if(group[j]==igroup && classification[j][0]==-1){
      nn++;
    }
  }
  sprintf(c,"   not classificated atom : %6d\n",nn);s_out=s_out+c;
  n=0;
  for(int j=0;j<natom;j++){
    if(group[j]==igroup && classification[j][0]==-1){
      sprintf(c,"%8d",j+1);s_out=s_out+c;
      n++;
      if(n%10==0 && n!=nn){
        s_out=s_out+"\n";
      }
    }
  }
  if(nn!=0){
    s_out=s_out+"\n";
  }
  s_out=s_out+"\n";

  BioUnit bu;
  int nbiounit_old=nbiounit;
  n =0;
  nn=0;
  for(int j=0;j<natom;j++){
    if(group[j]==igroup){
      
      //---------//
      // peptide //
      //---------//
      
      if(classification[j][0]==0){

	//////////////////////////
        // pick up alpha carbon //
	//////////////////////////

        bu.clear();

	//////////////////////
	// set biounit type //
	//////////////////////

	string s1,s3;
        bu.set_type(classification[j][1]);
	s3=BioUnit::res_abbr_3(classification[j][1]); bu.set_abbr_3(s3);
	s1=BioUnit::res_abbr_1(classification[j][1]); bu.set_abbr_1(s1);
	if(s3=="***"){
	  sprintf(c,"   ** warning : unknown residue was found, biounit No. %6d (C-alpha atom No. %6d)\n",nbiounit+1,j+1);
          ws=ws+c;nw++;
	}

	///////////////////////////////
	// push back atom to biounit //
	///////////////////////////////

        bu.push_back_atom_index(j,classification[j][0]);
        int nO4=0;
        int nH5=0;
        int nH6=0;
        int nH7=0;
        int nC81=0;
        int nN87=0;
        int nbond_C1=0;
        int nbond_N2=0;
        for(int k=0;k<natom;k++){
          if(classification[k][0]!=0 && classification[k][1]==j){
            bu.push_back_atom_index(k,classification[k][0]);
            if(classification[k][0]==4) nO4++;
            if(classification[k][0]==5) nH5++;
            if(classification[k][0]==6) nH6++;
            if(classification[k][0]==7) nH7++;
            if(classification[k][0]==81) nC81++; // 20130815 for Nterm -COCH3
            if(classification[k][0]==87) nN87++; // 20130815 for Cterm -CNCH3
          }
        }
        for(int k=0;k<natom;k++){
          if(classification[k][0]==2 && classification[k][1]==j){
            nbond_N2=atom[k].get_bond_atom_n();
          }
          if(classification[k][0]==1 && classification[k][1]==j){
            nbond_C1=atom[k].get_bond_atom_n();
          }
        }

	////////////////////////////////
	// check termintal of peptide //
	////////////////////////////////
	
        if(nC81==1)                     {bu.set_terminal_n(6);} // N-term ( -COCH3 )
        if(nN87==1)                     {bu.set_terminal_c(6);} // C-term ( -CNCH3 )
        if(nH5==1 && bu.get_type()==700){bu.set_terminal_n(1);} // N-term ( +0     )
        if(nH5==2 && bu.get_type()!=700){bu.set_terminal_n(1);} // N-term ( +0     )  
        if(nH5==2 && bu.get_type()==700){bu.set_terminal_n(2);} // N-term ( +1     )
        if(nH5==3 && bu.get_type()!=700){bu.set_terminal_n(2);} // N-term ( +1     )
        if(nO4==2 && nH6==1)            {bu.set_terminal_c(1);} // C-term ( +0     )
        if(nO4==2 && nH6==0)            {bu.set_terminal_c(2);} // C-term ( -1     )
        if(nH7==1){
          bu.set_terminal_c(4);// C-term (cap)
        }
        if(nbond_N2==2){
	  if(nH5==1 || (nH5==0 && bu.get_type()==700)){
	    bu.set_terminal_n(3);// N-term (cut)
	    sprintf(c,"   ** warning : N-term of link-group No. %6d may be incorrect, atom No. %6d\n",igroup+1,j+1);
	    ws=ws+c;nw++;
	  } 	  
	  if(nH5==0 && bu.get_type()!=700){// not N-term
	    sprintf(c,"   ** warning : main-chain of link-group No. %6d may be incorrect, atom No. %6d\n",igroup+1,j+1);
	    ws=ws+c;nw++;
	  }
        } 
        if(nbond_C1==2){
          bu.set_terminal_c(3);// C-term (cut)
          sprintf(c,"   ** warning : C-term of link-group No. %6d may be incorrect, atom No. %6d\n",igroup+1,j+1);
          ws=ws+c;nw++;
        }

	//////////////////////////
        // check for HID ro HIE //
	//////////////////////////

        if(bu.get_type()==1800){
          int h1814=bu.get_h1814();
          int h1816=bu.get_h1816();
          if     (h1814>=0 && h1816<0){bu.set_abbr_3("HID");}
          else if(h1816>=0 && h1814<0){bu.set_abbr_3("HIE");}
          else{
          }
        }

	////////////////////////////////////
        // push backu biounit to molecule //
	////////////////////////////////////

        push_back_biounit(bu);
        nn++;
      }

      //---------------------------//
      // dna deoxyribose or ribose //
      //---------------------------//
      
      if(classification[j][0]==20){
        bu.clear();
        //bu.set_abbr_3("DOX"); // ishi 20160315
        bu.set_abbr_1(  "X");
        bu.set_type(classification[j][1]);
        bu.push_back_atom_index(j,classification[j][0]);
        int n28=0;
        int n30=0;
        for(int k=0;k<natom;k++){
          if(classification[k][0]!=20 && classification[k][1]==j){
            bu.push_back_atom_index(k,classification[k][0]);
            if(classification[k][0]==28) n28++;
            if(classification[k][0]==30) n30++;
            if(classification[k][0]==35) {bu.set_abbr_3("RBO");} // ishi 20160315
            if(classification[k][0]==43) {bu.set_abbr_3("DOX");} // ishi 20160315
          }
        }
        if(n28>0) {bu.set_terminal_5(1);}
        if(n30>0) {bu.set_terminal_3(1);}
        // push back
        push_back_biounit(bu);
        nn++;
      }

      //----------//
      // dna base //
      //----------//
      
      if(classification[j][0]==26){
        bu.clear();
        bu.set_type(classification[j][1]);
        switch(classification[j][1]){
        case 5000: bu.set_abbr_3("  A") ; bu.set_abbr_1("A"); break;
        case 5100: bu.set_abbr_3("  G") ; bu.set_abbr_1("G"); break;
        case 5200: bu.set_abbr_3("  T") ; bu.set_abbr_1("T"); break;
        case 5300: bu.set_abbr_3("  C") ; bu.set_abbr_1("C"); break;
        case 5400: bu.set_abbr_3("  U") ; bu.set_abbr_1("U"); break;
        default  : bu.set_abbr_3("***") ; bu.set_abbr_1("*");
          sprintf(c,"   ** warning : unknown DNA-base was found around atom No. %6d\n",j+1);
          ws=ws+c;nw++;
          break;
        }
        bu.push_back_atom_index(j,classification[j][0]);
        for(int k=0;k<natom;k++){
          if(classification[k][0]!=26 && classification[k][1]==j){
            bu.push_back_atom_index(k,classification[k][0]);
          }
        }
        push_back_biounit(bu);
        nn++;
      }
    }
  }
  sprintf(c,"   %d biounits were generated\n\n",nn);s_out=s_out+c;

  for(int i=0;i<natom;i++) { delete [] classification[i] ; }
  delete [] classification;
  delete [] group;
  
  //-------------//
  // set ss bond //
  //-------------//

  for(int i=nbiounit_old;i<nbiounit;i++){
    if(biounit[i].get_type()==1550){
      int ibu=get_ibiounit_ss_bond(i);
      if(ibu>=0){
        biounit[i  ].set_ss_bond(ibu);
        biounit[ibu].set_ss_bond(i  );
      }
    }
  }
  
  int a2,a2_biounit;
  
  //-------------------------//
  // set link unit : peptide //
  //-------------------------//

  int c1_atom,c1_atom_tmp,n2_atom,n2_atom_tmp;
  for(int i=nbiounit_old;i<nbiounit;i++){
    if((100<=biounit[i].get_type() && biounit[i].get_type()<4000) || biounit[i].get_type()== -999){
      if(biounit[i].get_terminal_n()<0){
        n2_atom_tmp=biounit[i].get_n2();
        if(n2_atom_tmp<0){
          sprintf(c,"   ** warning : \"N\" in main-chain of the bio-unit No. %d was not found\n",i+1);
          ws=ws+c;nw++;
          break;
        }
        n2_atom=biounit[i].get_atom_index(n2_atom_tmp);
        nb=atom[n2_atom].get_bond_atom_n();
        for(int j=0;j<nb;j++){
          a2=atom[n2_atom].get_bond_atom(j);
          a2_biounit=get_ibiounit_from_iatom(a2);
          if(a2_biounit!=i){
            biounit[i].set_link_unit_n(a2_biounit);
          }
        }
      } 
      if(biounit[i].get_terminal_c()<0){
        c1_atom_tmp=biounit[i].get_c1();
        if(c1_atom_tmp<0){
          sprintf(c,"   ** warning : \"C\" in main-chain of the bio-unit No. %d was not found\n",i+1);
          ws=ws+c;nw++;
          break;
        }
        c1_atom=biounit[i].get_atom_index(c1_atom_tmp);
        nb=atom[c1_atom].get_bond_atom_n();
        for(int j=0;j<nb;j++){
          a2=atom[c1_atom].get_bond_atom(j);
          a2_biounit=get_ibiounit_from_iatom(a2);
          if(a2_biounit!=i){
            biounit[i].set_link_unit_c(a2_biounit);
          }
        }
      }
    }
  }

  //-----------------------------//
  // set link unit : deoxyribose //
  //-----------------------------//
  
  int o31_atom,o31_atom_tmp,c23_atom,c23_atom_tmp,c25_atom,c25_atom_tmp;
  for(int i=nbiounit_old;i<nbiounit;i++){
    //if(4000<=biounit[i].get_type() && biounit[i].get_type()<4001){ // ishi 20160315
    if(4000<=biounit[i].get_type() && biounit[i].get_type()<5000){ 
      if(biounit[i].get_terminal_5()<0){
        o31_atom_tmp=biounit[i].get_o31();
        if(o31_atom_tmp<0){
          sprintf(c,"   ** warning : structure around \"O3*\" of the bio-unit No. %d may be incorrect\n",
                  //igroup+1,
		  i+1);
          ws=ws+c;nw++;
          break;
        }
        o31_atom=biounit[i].get_atom_index(o31_atom_tmp);
        nb=atom[o31_atom].get_bond_atom_n();
        for(int j=0;j<nb;j++){
          a2=atom[o31_atom].get_bond_atom(j);
          a2_biounit=get_ibiounit_from_iatom(a2);
          if(a2_biounit!=i){
            biounit[i].set_link_unit_5(a2_biounit);
          }
        }
      }
      if(biounit[i].get_terminal_3()<0){
        c23_atom_tmp=biounit[i].get_c23();
        if(c23_atom_tmp<0){
          sprintf(c,"   ** warning : structure around \"C3*\" of the bio-unit No. %d may be incorrect\n",
                  //igroup+1,
		  i+1);
          ws=ws+c;nw++;
          break;
        }
        c23_atom=biounit[i].get_atom_index(c23_atom_tmp);
        nb=atom[c23_atom].get_bond_atom_n();
        for(int j=0;j<nb;j++){
          a2=atom[c23_atom].get_bond_atom(j);
          a2_biounit=get_ibiounit_from_iatom(a2);
          if(a2_biounit!=i){
            biounit[i].set_link_unit_3(a2_biounit);
          }
        }

      }
      c25_atom_tmp=biounit[i].get_c25();
      if(c25_atom_tmp<0){
        sprintf(c,"   ** warning : \"C1*\" of the bio-unit No. %d was not found\n",i+1);
        ws=ws+c;nw++;
        break;
      }
      c25_atom=biounit[i].get_atom_index(c25_atom_tmp);
      nb=atom[c25_atom].get_bond_atom_n();
      for(int j=0;j<nb;j++){
        a2=atom[c25_atom].get_bond_atom(j);
        a2_biounit=get_ibiounit_from_iatom(a2);
        if(a2_biounit!=i){
          biounit[i].set_link_unit_b(a2_biounit);
        }
      }
    } 
  }

  //----------------------//
  // set link unit : base //
  //----------------------//

  int n26_atom,n26_atom_tmp;
  for(int i=nbiounit_old;i<nbiounit;i++){
    if(5000<=biounit[i].get_type() && biounit[i].get_type()<6000){
      n26_atom_tmp=biounit[i].get_n26();
      if(n26_atom_tmp<0){
        sprintf(c,"   ** warning : \"N9\" of the bio-unit No. %d was not found\n",i+1);
        ws=ws+c;nw++;
        break;
      }
      n26_atom=biounit[i].get_atom_index(n26_atom_tmp);
      nb=atom[n26_atom].get_bond_atom_n();
      for(int j=0;j<nb;j++){
        a2=atom[n26_atom].get_bond_atom(j);
        a2_biounit=get_ibiounit_from_iatom(a2);
        if(a2_biounit!=i){
          biounit[i].set_link_unit_d(a2_biounit);
        }
      }      
    }
  }

  //-------------------------------//
  // check biounit link : pepteide //
  //-------------------------------//

  int ibu_c,chk_cterm;
  int ibu=get_ibiounit_nterm(igroup);
  int nbu=get_nbiounit_linkgroup(igroup);
  int count=0;
  while(1){
    if(ibu<0){
      break;
    }
    if(count>nbu){
      sprintf(c,"   ** warning : main-chain may be incorrect\n");
      ws=ws+c;nw++;
      break;
    }
    BioUnit bu(get_biounit(ibu));
    chk_cterm=0;
    if(bu.get_terminal_c()==1){chk_cterm=1;}
    if(bu.get_terminal_c()==2){chk_cterm=1;}
    if(bu.get_terminal_c()==3){chk_cterm=1;}
    if(bu.get_terminal_c()==4){chk_cterm=1;}
    if(bu.get_terminal_c()==6){chk_cterm=1;}
    ibu_c=bu.get_link_unit_c();
    if(chk_cterm!=1){
      if(ibu_c<0){
        sprintf(c,"   ** warning : main-chain around bio-unit No. %6d may be incorrect in link-goroup No. %6d \n",
		ibu+1,igroup+1);
        ws=ws+c;nw++;
      }
    }
    ibu=ibu_c;
    count++;
  }
  if(nw>0){
    s_out=s_out+ws+"\n";
  }
}

// private method //
void Molecule::biounit_atom_classification(const int igroup        ,
                                           int*      group         ,
                                           int**     classification,
                                           string&   s_out         ,
                                           int*      nw            ,
                                           string&   ws            ) const
{
  int residue,dna,iatom,jatom,katom,latom,matom,matom2,chk,bond1,bond2,ib,chk1,chk2,chk3,n,na;
  char c[128];

  //if(chk_dna(igroup,group)){
  //if(chk_dna(igroup,group,1)){
  if(chk_dna(igroup,group,2)){
        
    //---------------//
    // chk 5' carbon //
    //---------------//
    
    int dna1,dna2,num1,num2,idna;
    int *  itmp1       = new int   [natom+1];
    int *  itmp2       = new int   [natom+1];
    int *  type1       = new int   [natom+1];
    int *  type2       = new int   [natom+1];
    int ** belong_atom = new int * [2]    ;
    for(int i=0;i<2;i++){
      belong_atom[i] = new int [2];
    }
        
    n=0;
    idna=0;
    for(int i=0;i<natom;i++){
      
      if(group[i]==igroup){
        if(chk_5d_carbon(i,belong_atom)){
          n++;
          iatom=i;
          for(int j=0;j<2;j++){
            if(belong_atom[j][1]==21) jatom=belong_atom[j][0];
            if(belong_atom[j][1]==22) katom=belong_atom[j][0];
          }
          num1=0;
          num2=0;
          for(int j=0;j<natom;j++){
            itmp1[j]=-1;
            itmp2[j]=-1;
            type1[j]=-1;
            type2[j]=-1;
          }

          examine_dna(iatom,jatom,katom,&dna1,&dna2,&num1,&num2,itmp1,itmp2,type1,type2);

          if(num1>0){
            classification[iatom][0]=type1[0]; // 5'carbon 
            classification[iatom][1]=dna1;     // 5'carbon
            for(int j=1;j<num1;j++){
              classification[itmp1[j]][0]=type1[j];
              classification[itmp1[j]][1]=iatom;
            }
          }
          if(num2>0){
            classification[itmp2[0]][0]=type2[0]; // N26 (base)
            classification[itmp2[0]][1]=dna2;     // N26 (base)
            for(int j=1;j<num2;j++){
              classification[itmp2[j]][0]=type2[j];
              classification[itmp2[j]][1]=itmp2[0];
            }
          }
          if(num1>=0 || num2>=0){
            idna++;
            sprintf(c,"%8d",idna)                                ; s_out=s_out+c;
            sprintf(c,"   Deoxyribose    , 5'C : %8d",i+1)       ; s_out=s_out+c;
            sprintf(c," , natom = %6d\n",num1)                   ; s_out=s_out+c;
            sprintf(c,"        ")                                ; s_out=s_out+c;
            sprintf(c,"   Base           ,  -N : %8d",itmp2[0]+1); s_out=s_out+c;
            sprintf(c," , natom = %6d , type = %6d\n",num2,dna2) ; s_out=s_out+c;
          }
        }
      }
    }      
    for(int i=0;i<2;i++) delete [] belong_atom[i];
    delete [] belong_atom;
    delete [] itmp1;
    delete [] itmp2;
    delete [] type1;
    delete [] type2;
    if(n>0){
        s_out=s_out+"\n";
    }
    
  } else {

    //--------------------//
    // check alpha carbon //
    //--------------------//
    
    int num,ialpha;
    int n_not_bond,atomic1,atomic2,bond1,bond2;
    int * type = new int [natom+1];
    int * itmp = new int [natom+1];
    int * not_bond_1 = new int [nbond+1];
    int * not_bond_2 = new int [nbond+1];
    int ** belong_atom = new int* [4];
    for(int i=0;i<4;i++){
      belong_atom[i] = new int [2];
    }

    n=0;
    na=0;
    ialpha=0;
    n_not_bond=0;

    for(int i=0;i<nbond;i++){
      bond1=bond[i].get_iatom1();
      bond2=bond[i].get_iatom2();
      atomic1=atom[bond1].get_atomic_number();
      atomic2=atom[bond2].get_atomic_number();
      if(atomic1==16 && atomic2==16){
        not_bond_1[n_not_bond]=bond1;
        not_bond_2[n_not_bond]=bond2;
        cout << n_not_bond << endl ;
        n_not_bond++;
      }
      if((atomic1==16 && atomic2==30)||(atomic2==16 && atomic1==30)){
        not_bond_1[n_not_bond]=bond1;
        not_bond_2[n_not_bond]=bond2;
        cout << n_not_bond << endl ;
        n_not_bond++;
      }
    }
    
    for(int i=0;i<natom;i++){

      if(group[i]==igroup){

        na++;
        if(chk_alpha_carbon(i,belong_atom)){

          n++;
          iatom=  i;  // atom type =  0 ( alpha carbon )
          jatom= -1;  // atom type =  2
          katom= -1;  // atom type = 99
          latom= -1;  // atom type =  1
          matom= -1;  // atom type =  3 ( atom type = 8 only for gly )
          matom2=-1;  // atom type =  3 ( atom type = 9 only for gly )
          for(int j=0;j<4;j++){
            if(belong_atom[j][1]== 2){jatom=belong_atom[j][0];} // N(main) atom
            if(belong_atom[j][1]==99){katom=belong_atom[j][0];} // C(side) atom
            if(belong_atom[j][1]== 1){latom=belong_atom[j][0];} // C(main) atom
            if(belong_atom[j][1]== 3){
              if(matom!=-1){
                matom2=belong_atom[j][0]; // H(main) atom ( only for GLY )
              } else {
                matom =belong_atom[j][0]; // H(main) atom
              }
            }
          }
          num=0;
          for(int j=0;j<natom;j++){
            type[j]=-1;
          }
          
          examine_residue(iatom,jatom,katom,latom,matom,n_not_bond,not_bond_1,not_bond_2,
                          &residue,&num,itmp,type);

          if(num>100){
            sprintf(c,"   ** warning : residue including %8d atom may have structual problems\n",iatom+1);
            ws=ws+c;(*nw)++;
          }

          if(num<100){
            classification[iatom][0]=0      ; // C-alpha
            classification[iatom][1]=residue; // C-alpha
            for(int k=1;k<num;k++){
              if(type[k]<0){
                sprintf(c,"   ** warning : %8d atom is not classified\n",itmp[k]+1);
                ws=ws+c;(*nw)++;
              }
              classification[itmp[k]][0]=type[k];
              classification[itmp[k]][1]=iatom;
            } 
          }

          if(katom==-1){
            // only for GLY
            classification[matom ][0]=8;
            classification[matom ][1]=iatom;
            classification[matom2][0]=9;
            classification[matom2][1]=iatom;
          }
          ialpha++;
          sprintf(c,"%8d",ialpha)                 ; s_out=s_out+c;
          sprintf(c,"   C-alpha : %8d",i+1)       ; s_out=s_out+c;
          sprintf(c," , C : %8d",latom+1)         ; s_out=s_out+c;
          sprintf(c," , N : %8d",jatom+1)         ; s_out=s_out+c;
          if(katom!=-1){
            sprintf(c," , C-side : %8d\n",katom+1); s_out=s_out+c;
          } else {
            sprintf(c," ,          %8s\n","")     ; s_out=s_out+c;
          }
        }
      }
    }
    
    for(int i=0;i<4;i++){
      delete [] belong_atom[i] ;
    }
    delete [] not_bond_1 ;
    delete [] not_bond_2 ;
    delete [] belong_atom ;
    delete [] type ;
    delete [] itmp ;
    if(n>0){
      s_out=s_out+"\n";
    }
  }
}

// private method //
bool Molecule::chk_alpha_carbon( const int iatom, 
                                 int** belong_atom ) const
{
  int nbond_,an, nH, nC, nN, n, nH_, nC_, nN_, nO_, chk, chk_carbon_peptide;
  int bond_[4];
  int atomic_[4];

  if(atom[iatom].get_not_ca()==1) return false ;

  if(atom[iatom].get_atomic_number()!=6) return false ;
  
  nbond_ =atom[iatom].get_bond_atom_n();
  if(nbond_!=4) {
    return false;
  }

  for(int i=0;i<nbond_;i++){
    bond_  [i]=atom[iatom].get_bond_atom(i);
    atomic_[i]=atom[bond_[i]].get_atomic_number();
  }

  nH=0; nC=0; nN=0;
  for(int i=0;i<nbond_;i++){
    if(atomic_[i]==1) {nH++;}
    if(atomic_[i]==6) {nC++;}
    if(atomic_[i]==7) {nN++;}
  }
  if( ! ( (nH==1 && nC==2 && nN==1) || (nH==2 && nC==1 && nN==1) ) ) {
    return false;
  }
  //
  if(nH==2){
    chk=-1;
    for(int i=0;i<4;i++){
      if(atomic_[i]==6){
        nH_=0; nC_=0; nN_=0, nO_=0; n=0;
        n=atom[bond_[i]].get_bond_atom_n();
        for(int j=0;j<n;j++){
          an=atom[atom[bond_[i]].get_bond_atom(j)].get_atomic_number();
          if(an==1) nH_++;
          if(an==6) nC_++;
          if(an==7) nN_++;
          if(an==8) nO_++;
        }
        if(n==2 && nH_==0 && nC_==1 && nN_==0 && nO_==1){ chk=1; chk_carbon_peptide=i; }
        if(n==3 && nH_==0 && nC_==1 && nN_==1 && nO_==1){ chk=1; chk_carbon_peptide=i; }
        if(n==3 && nH_==0 && nC_==1 && nN_==0 && nO_==2){ chk=1; chk_carbon_peptide=i; }
        if(n==3 && nH_==1 && nC_==1 && nN_==0 && nO_==1){ chk=1; chk_carbon_peptide=i; }
      }
    }
    if(chk==-1){
      return false;
    }
    chk=-1;
    for(int i=0;i<4;i++){
      if(atomic_[i]==7){
        nH_=0; nC_=0; nN_=0; nO_=0; n=0;
        n=atom[bond_[i]].get_bond_atom_n();
        for(int j=0;j<n;j++){
          an=atom[atom[bond_[i]].get_bond_atom(j)].get_atomic_number();
          if(an==1) nH_++;
          if(an==6) nC_++;
          if(an==7) nN_++;
          if(an==8) nO_++;
        }
        if(n==2 && nH_==1 && nC_==1 && nN_==0 && nO_==0){ chk=1; } // 20100422
        if(n==3 && nH_==1 && nC_==2 && nN_==0 && nO_==0){ chk=1; }
        if(n==3 && nH_==2 && nC_==1 && nN_==0 && nO_==0){ chk=1; }
        if(n==4 && nH_==3 && nC_==1 && nN_==0 && nO_==0){ chk=1; }
      }
    }
    if(chk==-1){
      return false;
    }
  }
  if(nH==1){
    chk=-1;
    for(int i=0;i<4;i++){
      if(atomic_[i]==6){
        nH_=0; nC_=0; nN_=0; nO_=0; n=0;
        n=atom[bond_[i]].get_bond_atom_n();
        for(int j=0;j<n;j++){
          an=atom[atom[bond_[i]].get_bond_atom(j)].get_atomic_number();
          if(an==1) nH_++;
          if(an==6) nC_++;
          if(an==7) nN_++;
          if(an==8) nO_++;
        }
        if(n==2 && nH_==0 && nC_==1 && nN_==0 && nO_==1){ chk=1; chk_carbon_peptide=i; }
        if(n==3 && nH_==0 && nC_==1 && nN_==1 && nO_==1){ chk=1; chk_carbon_peptide=i; }
        if(n==3 && nH_==0 && nC_==1 && nN_==0 && nO_==2){ chk=1; chk_carbon_peptide=i; }
        if(n==3 && nH_==1 && nC_==1 && nN_==0 && nO_==1){ chk=1; chk_carbon_peptide=i; }
      }
    }
    if(chk==-1){
      return false;
    }
    chk=-1;
    for(int i=0;i<4;i++){
      if(atomic_[i]==7){
        nH_=0; nC_=0; nN_=0; nO_=0; n=0;
        n=atom[bond_[i]].get_bond_atom_n();
        for(int j=0;j<n;j++){
          an=atom[atom[bond_[i]].get_bond_atom(j)].get_atomic_number();
          if(an==1) nH_++;
          if(an==6) nC_++;
          if(an==7) nN_++;
          if(an==8) nO_++;
        }
        if(n==2 && nH_==1 && nC_==1 && nN_==0 && nO_==0){ chk=1; }
        if(n==3 && nH_==1 && nC_==2 && nN_==0 && nO_==0){ chk=1; }
        if(n==3 && nH_==2 && nC_==1 && nN_==0 && nO_==0){ chk=1; }
        if(n==4 && nH_==3 && nC_==1 && nN_==0 && nO_==0){ chk=1; }
        if(n==2 && nH_==0 && nC_==2 && nN_==0 && nO_==0){ chk=1; }
        if(n==3 && nH_==0 && nC_==3 && nN_==0 && nO_==0){ chk=1; }
        if(n==3 && nH_==1 && nC_==2 && nN_==0 && nO_==0){ chk=1; }
        if(n==4 && nH_==2 && nC_==2 && nN_==0 && nO_==0){ chk=1; }
      }
    }
    if(chk==-1){
      return false;
    }
  }
  for(int i=0;i<4;i++) {
    belong_atom[i][0]=bond_[i];
    if(atomic_[i]==1) belong_atom[i][1]=3;
    if(atomic_[i]==7) belong_atom[i][1]=2;
    if(atomic_[i]==6 && chk_carbon_peptide==i) belong_atom[i][1]=1;
    if(atomic_[i]==6 && chk_carbon_peptide!=i) belong_atom[i][1]=99;
  }
  return true;
}

// private method //
void Molecule::examine_residue(const int iatom   , // C (alpha) :  0
                               const int jatom   , // N (main ) :  2
                               const int katom   , // C (side ) : 99
                               const int latom   , // C (main ) :  1 
                               const int matom   , // H (main ) :  3
                               const int n_not_bond,
                               const int * not_bond_1,
                               const int * not_bond_2,
                               int *     residue ,
                               int *     num     ,
                               int *     group   ,
                               int *     type    ) const
{
  int nC,nH,nN,nO,nS,tmp_nH,nb,nCb,nNb,nHb,nOb,nSb,a;
  int k,nb1,nb2,nb3,a1,a2,a3,c81,c83,n87,c89;

  *residue=-999;
  *num=0;
  group[*num]=iatom; *num=*num+1; if(*num >= natom) return ; // 0 : C-alpha(main)
  group[*num]=jatom; *num=*num+1; if(*num >= natom) return ; // 2 : N(main)

  if(katom!=-1){

    bond_trace(katom, num, group, n_not_bond,not_bond_1,not_bond_2);
    if(*num >= natom) {return ;}

    nC  =-1;                             // number of C atom (0:C-alpha(main)) 
    nN  =-1;                             // number of N atom (2:N(main)) 
    nH  = 0;                             // number of H atom
    nO  = 0;                             // number of O atom
    nS  = 0;                             // number of S atom
    nHb = 0;                             // number of bond in H atom                   // ishi 20130914
    nCb =-atom[iatom].get_bond_atom_n(); // number of bond in C atom (0:C-alpha(main)) // ishi 20130914 
    nNb =-atom[jatom].get_bond_atom_n(); // number of bond in N atom (2:N(main))       // ishi 20130914 
    nOb = 0;                             // number of bond in O atom                   // ishi 20130914
    nSb = 0;                             // number of bond in S atom                   // ishi 20130914
    for(int i=0; i<*num; i++){
      if(atom[group[i]].get_atomic_number()==1)  {nH++;nHb=nHb+atom[group[i]].get_bond_atom_n();}
      if(atom[group[i]].get_atomic_number()==6)  {nC++;nCb=nCb+atom[group[i]].get_bond_atom_n();}
      if(atom[group[i]].get_atomic_number()==7)  {nN++;nNb=nNb+atom[group[i]].get_bond_atom_n();}
      if(atom[group[i]].get_atomic_number()==8)  {nO++;nOb=nOb+atom[group[i]].get_bond_atom_n();}
      if(atom[group[i]].get_atomic_number()==16) {nS++;nSb=nSb+atom[group[i]].get_bond_atom_n();}
    }

    // Ala
    if(nC == 1 && nH == 3 && nN ==0 && nO ==0 && nS ==0 && 
       nCb== 4 && nHb== 3 && nNb==0 && nOb==0 && nSb==0) *residue= 200;
    // Val
    if(nC == 3 && nH == 7 && nN ==0 && nO ==0 && nS ==0 && 
       nCb==12 && nHb== 7 && nNb==0 && nOb==0 && nSb==0) *residue= 300;
    // Met
    if(nC == 3 && nH == 7 && nN ==0 && nO ==0 && nS ==1 && 
       nCb==12 && nHb== 7 && nNb==0 && nOb==0 && nSb==2) *residue= 600;
    // Pro 
    if(nC == 3 && nH == 6 && nN ==0 && nO ==0 && nS ==0 && 
       nCb==12 && nHb== 6 && nNb==0 && nOb==0 && nSb==0) *residue= 700;
    // Phe
    if(nC == 7 && nH == 7 && nN ==0 && nO ==0 && nS ==0 && 
       nCb==22 && nHb== 7 && nNb==0 && nOb==0 && nSb==0) *residue= 800;
    // Trp
    if(nC == 9 && nH == 8 && nN ==1 && nO ==0 && nS ==0 && 
       nCb==28 && nHb== 8 && nNb==3 && nOb==0 && nSb==0) *residue= 900;
    // Ser
    if(nC == 1 && nH == 3 && nN ==0 && nO ==1 && nS ==0 && 
       nCb== 4 && nHb== 3 && nNb==0 && nOb==2 && nSb==0) *residue=1000;
    // Thr
    if(nC == 2 && nH == 5 && nN ==0 && nO ==1 && nS ==0 && 
       nCb== 8 && nHb== 5 && nNb==0 && nOb==2 && nSb==0) *residue=1100;
    // Asn
    if(nC == 2 && nH == 4 && nN ==1 && nO ==1 && nS ==0 && 
       nCb== 7 && nHb== 4 && nNb==3 && nOb==1 && nSb==0) *residue=1200;
    // Gln
    if(nC == 3 && nH == 6 && nN ==1 && nO ==1 && nS ==0 && 
       nCb==11 && nHb== 6 && nNb==3 && nOb==1 && nSb==0) *residue=1300;
    // Tyr
    if(nC == 7 && nH == 7 && nN ==0 && nO ==1 && nS ==0 && 
       nCb==22 && nHb== 7 && nNb==0 && nOb==2 && nSb==0) *residue=1400;
    // Cys
    if(nC==  1 && nH==  3 && nN ==0 && nO ==0 && nS ==1 && 
       nCb== 4 && nHb== 3 && nNb==0 && nOb==0 && nSb==2) *residue=1500;
    // Cys ( S-S , S-Z )
    if(nC == 1 && nH == 2 && nN ==0 && nO ==0 && nS ==1 && 
       nCb== 4 && nHb== 2 && nNb==0 && nOb==0 && nSb==2) *residue=1550;
    // Lys ( --NH3+    )
    if(nC == 4 && nH ==11 && nN ==1 && nO ==0 && nS ==0 && 
       nCb==16 && nHb==11 && nNb==4 && nOb==0 && nSb==0) *residue=1600;
    // Lys ( --NH2     )
    if(nC == 4 && nH ==10 && nN ==1 && nO ==0 && nS ==0
                                                       ) *residue=1601;
    // Arg ( --NH2+    )
    if(nC == 4 && nH ==11 && nN ==3 && nO ==0 && nS ==0 && 
       nCb==15 && nHb==11 && nNb==9 && nOb==0 && nSb==0) *residue=1700;
    // Arg ( --NH      )
    if(nC == 4 && nH ==10 && nN ==3 && nO ==0 && nS ==0
                                                       ) *residue=1701;
    // His ( HID , HIE )
    if(nC == 4 && nH == 5 && nN ==2 && nO ==0 && nS ==0
                                                       ) *residue=1800;
    // His ( HIP       )
    //if(nC == 4 && nH == 6 && nN== 2 && nO ==0 && nS ==0
    //                                                   ) *residue=1850; // <-- ishiishi
    if(nC == 4 && nH == 6 && nN== 2 && nO ==0 && nS ==0
                                                       ) *residue=1800; // <-- ishiishi , 20140324
    // Asp ( --COO-    )
    if(nC == 2 && nH == 2 && nN ==0 && nO ==2 && nS ==0 && 
       nCb== 7 && nHb== 2 && nNb==0 && nOb==2 && nSb==0) *residue=1900;
    // Asp ( --COOH    )
    if(nC == 2 && nH == 3 && nN ==0 && nO ==2 && nS ==0
                                                       ) *residue=1901;
    // Glu ( --COO-    )
    if(nC == 3 && nH == 4 && nN ==0 && nO== 2 && nS ==0 && 
       nCb==11 && nHb== 4 && nNb==0 && nOb==2 && nSb==0) *residue=2000;
    // Glu ( --COOH    )
    if(nC == 3 && nH == 5 && nN ==0 && nO ==2 && nS ==0
                                                       ) *residue=2001;
    // Ile or Leu
    if(nC==  4 && nH == 9 && nN ==0 && nO ==0 && nS ==0 &&
       nCb==16 && nHb== 9 && nNb==0 && nOb==0 && nSb==0) {
      tmp_nH=0;
      nb=atom[katom].get_bond_atom_n();
      for(int i=0;i<nb;i++){
        a=atom[katom].get_bond_atom(i);
        if(atom[a].get_atomic_number()==1){
          tmp_nH++;
        }
      }
      if(tmp_nH==1) *residue=500;
      if(tmp_nH==2) *residue=400;
    }
  } else {
    *residue=100;
  }

  examine_side(iatom,jatom,katom,*residue,*num,group,type);
  if(*num >= natom) {return ;}

  group[*num]=latom;(*num)++; if(*num >= natom) {return ;}
  group[*num]=matom;(*num)++; if(*num >= natom) {return ;}
  
  for(int i=0;i<(*num);i++){
    if     (group[i]==iatom) type[i]= 0;
    else if(group[i]==jatom) type[i]= 2;
    else if(group[i]==latom) type[i]= 1;
    else if(group[i]==matom) type[i]= 3;
    else {
      // 20130712 
      //if(type[i]<0){
      //  type[i]=(*residue);
      //}
    }
  }
  
  /* search 4O */
  nb=atom[latom].get_bond_atom_n();
  for(int i=0;i<nb;i++){
    a=atom[latom].get_bond_atom(i);
    if(atom[a].get_atomic_number()==8){
      group[*num]=a;
      type [*num]=4;
      (*num)++; if(*num >= natom) {return ;}
    }
  }

  /* search 7H */
  nb=atom[latom].get_bond_atom_n();
  for(int i=0;i<nb;i++){
    a=atom[latom].get_bond_atom(i);
    if(atom[a].get_atomic_number()==1){
      group[*num]=a;
      type [*num]=7;
      (*num)++; if(*num >= natom) {return ;}
    }
  }  

  /* search 5H */
  nb=atom[jatom].get_bond_atom_n();
  for(int i=0;i<nb;i++){
    a=atom[jatom].get_bond_atom(i);
    if(atom[a].get_atomic_number()==1){
      group[*num]=a;
      type [*num]=5;
      (*num)++; if(*num >= natom) {return ;}
    }
  }

  /* search 6H */
  int n=(*num);
  for(int j=0;j<n;j++){
    if(type[j]==4){
      nb=atom[group[j]].get_bond_atom_n();
      for(int i=0;i<nb;i++){
        a=atom[group[j]].get_bond_atom(i);
        if(atom[a].get_atomic_number()==1){
          group[*num]=a;
          type [*num]=6;
          (*num)++; if(*num >= natom) {return ;}
        }
      }
    }
  }

  /* search 81C,83C Nterm -COCH3 (20130815) */
  c81=-1;
  c83=-1;
  nb1=atom[jatom].get_bond_atom_n();
  for(int i=0;i<nb1;i++){
    a1=atom[jatom].get_bond_atom(i);
    if(a1!=iatom && atom[a].get_atomic_number()==6){
      nb2=atom[a1].get_bond_atom_n();
      for(int j=0;j<nb2;j++){
        a2=atom[a1].get_bond_atom(j);
        if(atom[a2].get_atomic_number()==6){
          nb3=atom[a2].get_bond_atom_n();
          if(nb3==4){
            nH=0;
            for(k=0;k<nb3;k++){
              a3=atom[a2].get_bond_atom(k);
              if(atom[a3].get_atomic_number()==1){
                nH++;
              }
            }
            if(nH==3){
              c81=a1;group[*num]=c81;type[*num]=81;(*num)++; if(*num >= natom) {return ;}
              c83=a2;group[*num]=c83;type[*num]=83;(*num)++; if(*num >= natom) {return ;}
            }
          }
        }
      }
    }
  }
  
  /* search 82O Nterm -COCH3 (20130815) */
  if(c81!=-1){
    nb=atom[c81].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      a=atom[c81].get_bond_atom(i);
      if(atom[a].get_atomic_number()==8){
        group[*num]=a;
        type [*num]=82;
        (*num)++; if(*num >= natom) {return ;}
      }
    }
  }

  /* search 84H Nterm -COCH3 (20130815) */
  if(c83!=-1){
    nb=atom[c83].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      a=atom[c83].get_bond_atom(i);
      if(atom[a].get_atomic_number()==1){
        group[*num]=a;
        type [*num]=84;
        (*num)++; if(*num >= natom) {return ;}
      }
    }   
  }

  /* search 87N,89C Cterm -CNCH3 (20130815) */
  n87=-1;
  c89=-1;
  nb1=atom[latom].get_bond_atom_n();
  for(int i=0;i<nb1;i++){
    a1=atom[latom].get_bond_atom(i);
    if(atom[a1].get_atomic_number()==7){
      nb2=atom[a1].get_bond_atom_n();
      for(int j=0;j<nb2;j++){
        a2=atom[a1].get_bond_atom(j);
        if(a2!=latom && atom[a2].get_atomic_number()==6){
          nb3=atom[a2].get_bond_atom_n();
          if(nb3==4){
            nH=0;
            for(k=0;k<nb3;k++){
              a3=atom[a2].get_bond_atom(k);
              if(atom[a3].get_atomic_number()==1){
                nH++;
              }
            }
            if(nH==3){
              n87=a1;group[*num]=n87;type[*num]=87;(*num)++; if(*num >= natom) {return ;}
              c89=a2;group[*num]=c89;type[*num]=89;(*num)++; if(*num >= natom) {return ;}
            }
          }
        }
      }
    }
  }
  
  /* search 88H Cterm -CNCH3 (20130815) */
  if(n87!=-1){
    nb=atom[n87].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      a=atom[n87].get_bond_atom(i);
      if(atom[a].get_atomic_number()==1){
        group[*num]=a;
        type [*num]=88;
        (*num)++; if(*num >= natom) {return ;}
      }
    }
  }

  /* search 90H Cterm -CNCH3 (20130815) */
  if(c89!=-1){
    nb=atom[c89].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      a=atom[c89].get_bond_atom(i);
      if(atom[a].get_atomic_number()==1){
        group[*num]=a;
        type [*num]=90;
        (*num)++; if(*num >= natom) {return ;}
      }
    }   
  }
}

/* private method */
void Molecule::examine_side(const int   iatom   , // C (alpha) :  0
                            const int   jatom   , // N (main ) :  2
                            const int   katom   , // C (side ) : 99
                            const int   res     ,
                            const int   num     ,
                            const int * group   ,
                            int       * type    ) const 
{
  int bond1,bond2,nb,chk1,chk2,nb_;
  int * b1;
  int * b2;
  int * a1;
  int * a2;

  // 20130914
  //b1 = new int [natom+1];
  //b2 = new int [natom+2];
  //a1 = new int [natom+1]; // atomic number
  //a2 = new int [natom+2]; // atomic number
  b1 = new int [nbond+1];
  b2 = new int [nbond+1];
  a1 = new int [nbond+1]; // atomic number
  a2 = new int [nbond+1]; // atomic number

  nb=0;
  for(int i=0;i<nbond;i++){
    bond1=bond[i].get_iatom1();
    bond2=bond[i].get_iatom2();
    chk1=-1;
    chk2=-1;
    for(int j=0;j<num;j++){
      if(bond1==group[j] && group[j]!=iatom && group[j]!=jatom) chk1=j;
      if(bond2==group[j] && group[j]!=iatom && group[j]!=jatom) chk2=j;
    }
    if(chk1>-1 && chk2>-1){
      b1[nb]=chk1;
      b2[nb]=chk2;
      a1[nb]=atom[bond1].get_atomic_number();
      a2[nb]=atom[bond2].get_atomic_number();
      nb++;
    }
  }
  
  if(res== 200) examine_ala(nb,b1,b2,a1,a2,type); // ALA
  if(res== 300) examine_val(nb,b1,b2,a1,a2,type); // VAL
  if(res== 400) examine_leu(nb,b1,b2,a1,a2,type); // LEU
  if(res== 500) examine_ile(nb,b1,b2,a1,a2,type); // ILE
  if(res== 600) examine_met(nb,b1,b2,a1,a2,type); // MET
  if(res== 700) examine_pro(nb,b1,b2,a1,a2,type); // PRO
  if(res== 800) examine_phe(nb,b1,b2,a1,a2,type); // PHE
  if(res== 900) examine_trp(nb,b1,b2,a1,a2,type); // TRP
  if(res==1000) examine_ser(nb,b1,b2,a1,a2,type); // SER
  if(res==1100) examine_thr(nb,b1,b2,a1,a2,type); // THR
  if(res==1200) examine_asn(nb,b1,b2,a1,a2,type); // ASN
  if(res==1300) examine_gln(nb,b1,b2,a1,a2,type); // GLN
  if(res==1400) examine_tyr(nb,b1,b2,a1,a2,type); // TYR
  if(res==1500) examine_cys(nb,b1,b2,a1,a2,type); // CYS
  if(res==1550) examine_cys(nb,b1,b2,a1,a2,type); // CYS ( S-S    )
  if(res==1600) examine_lys(nb,b1,b2,a1,a2,type); // LYS ( --NH3+ )
  if(res==1601) examine_lys(nb,b1,b2,a1,a2,type); // LYS ( --NH2  ) 
  if(res==1700) examine_arg(nb,b1,b2,a1,a2,type); // ARG ( --NH2+ )
  if(res==1701) examine_arg(nb,b1,b2,a1,a2,type); // ARG ( --NH   )
  if(res==1800) examine_his(nb,b1,b2,a1,a2,type); // HIS 
  //if(res==1850) examine_his(nb,b1,b2,a1,a2,type); // HIP            // <-- ishiishi , 20140324
  if(res==1900) examine_asp(nb,b1,b2,a1,a2,type); // ASP ( --COO- )
  if(res==1901) examine_asp(nb,b1,b2,a1,a2,type); // ASP ( --COOH )
  if(res==2000) examine_glu(nb,b1,b2,a1,a2,type); // GLU ( --COO- )
  if(res==2001) examine_glu(nb,b1,b2,a1,a2,type); // GLU ( --COOH )

  delete [] b1;
  delete [] b2;
  delete [] a1;
  delete [] a2;
}

void Molecule::examine_ala( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  4];for(int i=0;i<(  4);i++)a  [i]=-1;
  int chk[2+4];for(int i=0;i<(2+4);i++)chk[i]= 0;
  a[0]=2;chk[2]=1;
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0]){if(chk[b2[i]]==0){a[1]=b2[i];chk[b2[i]]=1;break;}}
    if(b2[i]==a[0]){if(chk[b1[i]]==0){a[1]=b1[i];chk[b1[i]]=1;break;}}
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0]){if(chk[b2[i]]==0){a[2]=b2[i];chk[b2[i]]=1;break;}}
    if(b2[i]==a[0]){if(chk[b1[i]]==0){a[2]=b1[i];chk[b1[i]]=1;break;}}
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0]){if(chk[b2[i]]==0){a[3]=b2[i];chk[b2[i]]=1;break;}}
    if(b2[i]==a[0]){if(chk[b1[i]]==0){a[3]=b1[i];chk[b1[i]]=1;break;}}
  }
  if(a[0]>=0) type[a[0]]=205;
  if(a[1]>=0) type[a[1]]=206;
  if(a[2]>=0) type[a[2]]=207;
  if(a[3]>=0) type[a[3]]=208;
}

void Molecule::examine_val( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  10];for(int i=0;i<(  10);i++)a  [i]=-1;
  int chk[2+10];for(int i=0;i<(2+10);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 305 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]==6){if(chk[b2[i]]==0){a[1]=b2[i];chk[b2[i]]=1;break;}} // 306 C
    if(b2[i]==a[0] && a1[i]==6){if(chk[b1[i]]==0){a[1]=b1[i];chk[b1[i]]=1;break;}} // 306 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]==6){if(chk[b2[i]]==0){a[2]=b2[i];chk[b2[i]]=1;break;}} // 307 C
    if(b2[i]==a[0] && a1[i]==6){if(chk[b1[i]]==0){a[2]=b1[i];chk[b1[i]]=1;break;}} // 307 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]==1){if(chk[b2[i]]==0){a[3]=b2[i];chk[b2[i]]=1;break;}} // 308 H
    if(b2[i]==a[0] && a1[i]==1){if(chk[b1[i]]==0){a[3]=b1[i];chk[b1[i]]=1;break;}} // 308 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==1){if(chk[b2[i]]==0){a[4]=b2[i];chk[b2[i]]=1;break;}} // 309 H
    if(b2[i]==a[1] && a1[i]==1){if(chk[b1[i]]==0){a[4]=b1[i];chk[b1[i]]=1;break;}} // 309 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==1){if(chk[b2[i]]==0){a[5]=b2[i];chk[b2[i]]=1;break;}} // 310 H
    if(b2[i]==a[1] && a1[i]==1){if(chk[b1[i]]==0){a[5]=b1[i];chk[b1[i]]=1;break;}} // 310 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==1){if(chk[b2[i]]==0){a[6]=b2[i];chk[b2[i]]=1;break;}} // 311 H
    if(b2[i]==a[1] && a1[i]==1){if(chk[b1[i]]==0){a[6]=b1[i];chk[b1[i]]=1;break;}} // 311 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]==1){if(chk[b2[i]]==0){a[7]=b2[i];chk[b2[i]]=1;break;}} // 312 H
    if(b2[i]==a[2] && a1[i]==1){if(chk[b1[i]]==0){a[7]=b1[i];chk[b1[i]]=1;break;}} // 312 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]==1){if(chk[b2[i]]==0){a[8]=b2[i];chk[b2[i]]=1;break;}} // 313 H
    if(b2[i]==a[2] && a1[i]==1){if(chk[b1[i]]==0){a[8]=b1[i];chk[b1[i]]=1;break;}} // 313 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]==1){if(chk[b2[i]]==0){a[9]=b2[i];chk[b2[i]]=1;break;}} // 314 H
    if(b2[i]==a[2] && a1[i]==1){if(chk[b1[i]]==0){a[9]=b1[i];chk[b1[i]]=1;break;}} // 314 H
  }
  if(a[0]>=0) type[a[0]]=305;
  if(a[1]>=0) type[a[1]]=306;
  if(a[2]>=0) type[a[2]]=307;
  if(a[3]>=0) type[a[3]]=308;
  if(a[4]>=0) type[a[4]]=309;
  if(a[5]>=0) type[a[5]]=310;
  if(a[6]>=0) type[a[6]]=311;
  if(a[7]>=0) type[a[7]]=312;
  if(a[8]>=0) type[a[8]]=313;
  if(a[9]>=0) type[a[9]]=314;
}

void Molecule::examine_leu( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  13];for(int i=0;i<(  13);i++)a  [i]=-1;
  int chk[2+13];for(int i=0;i<(2+13);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 405 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]==6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 406 C
    if(b2[i]==a[0] && a1[i]==6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 406 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 407 C
    if(b2[i]==a[1] && a1[i]==6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 407 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 408 C
    if(b2[i]==a[1] && a1[i]==6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 408 C
  }

  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]==1){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 409 H
    if(b2[i]==a[0] && a1[i]==1){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 409 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]==1){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 410 H
    if(b2[i]==a[0] && a1[i]==1){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 410 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 411 H
    if(b2[i]==a[1] && a1[i]==1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 411 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]==1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 412 H
    if(b2[i]==a[2] && a1[i]==1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 412 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]==1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 413 H
    if(b2[i]==a[2] && a1[i]==1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 413 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]==1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 414 H
    if(b2[i]==a[2] && a1[i]==1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 414 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]==1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 415 H
    if(b2[i]==a[3] && a1[i]==1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 415 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]==1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 416 H
    if(b2[i]==a[3] && a1[i]==1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 417 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]==1){if(chk[b2[i]]==0){a[12]=b2[i];chk[b2[i]]=1;break;}} // 417 H
    if(b2[i]==a[3] && a1[i]==1){if(chk[b1[i]]==0){a[12]=b1[i];chk[b1[i]]=1;break;}} // 417 H
  }
  if(a[ 0]>=0) type[a[ 0]]=405;
  if(a[ 1]>=0) type[a[ 1]]=406;
  if(a[ 2]>=0) type[a[ 2]]=407;
  if(a[ 3]>=0) type[a[ 3]]=408;  
  if(a[ 4]>=0) type[a[ 4]]=409;
  if(a[ 5]>=0) type[a[ 5]]=410;
  if(a[ 6]>=0) type[a[ 6]]=411;
  if(a[ 7]>=0) type[a[ 7]]=412;
  if(a[ 8]>=0) type[a[ 8]]=413;
  if(a[ 9]>=0) type[a[ 9]]=414;
  if(a[10]>=0) type[a[10]]=415;
  if(a[11]>=0) type[a[11]]=416; 
  if(a[12]>=0) type[a[12]]=417;
}

void Molecule::examine_ile( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  13];for(int i=0;i<(  13);i++)a  [i]=-1;
  int chk[2+13];for(int i=0;i<(2+13);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 505 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]==6){
      int nH=0;
      for(int j=0;j<nb;j++){
        if(b1[j]==b2[i] && a2[j]==1) nH++;
        if(b2[j]==b2[i] && a1[j]==1) nH++;
      }
      if(nH==3){
        if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;} // 506 C
      }
    }
    if(b2[i]==a[0] && a1[i]==6){
      int nH=0;
      for(int j=0;j<nb;j++){
        if(b1[j]==b1[i] && a2[j]==1) nH++;
        if(b2[j]==b1[i] && a1[j]==1) nH++;
      }
      if(nH==3){
        if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;} // 506 C
      }
    }
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]==6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 507 C
    if(b2[i]==a[0] && a1[i]==6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 507 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]==6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 508 C
    if(b2[i]==a[2] && a1[i]==6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 508 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]==1){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 509 H
    if(b2[i]==a[0] && a1[i]==1){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 509 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==1){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 510 H
    if(b2[i]==a[1] && a1[i]==1){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 510 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 511 H
    if(b2[i]==a[1] && a1[i]==1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 511 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 512 H
    if(b2[i]==a[1] && a1[i]==1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 512 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]==1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 513 H
    if(b2[i]==a[2] && a1[i]==1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 513 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]==1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 514 H
    if(b2[i]==a[2] && a1[i]==1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 514 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]==1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 515 H
    if(b2[i]==a[3] && a1[i]==1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 515 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]==1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 516 H
    if(b2[i]==a[3] && a1[i]==1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 516 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]==1){if(chk[b2[i]]==0){a[12]=b2[i];chk[b2[i]]=1;break;}} // 517 H
    if(b2[i]==a[3] && a1[i]==1){if(chk[b1[i]]==0){a[12]=b1[i];chk[b1[i]]=1;break;}} // 517 H
  }
  if(a[ 0]>=0) type[a[ 0]]=505;
  if(a[ 1]>=0) type[a[ 1]]=506;
  if(a[ 2]>=0) type[a[ 2]]=507;
  if(a[ 3]>=0) type[a[ 3]]=508;  
  if(a[ 4]>=0) type[a[ 4]]=509;
  if(a[ 5]>=0) type[a[ 5]]=510;
  if(a[ 6]>=0) type[a[ 6]]=511;
  if(a[ 7]>=0) type[a[ 7]]=512;
  if(a[ 8]>=0) type[a[ 8]]=513;
  if(a[ 9]>=0) type[a[ 9]]=514;
  if(a[10]>=0) type[a[10]]=515;
  if(a[11]>=0) type[a[11]]=516; 
  if(a[12]>=0) type[a[12]]=517;
}

void Molecule::examine_met( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  11];for(int i=0;i<(  11);i++)a  [i]=-1;
  int chk[2+11];for(int i=0;i<(2+11);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 605 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 606 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 606 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==16){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 607 S
    if(b2[i]==a[1] && a1[i]==16){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 607 S
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 608 C
    if(b2[i]==a[2] && a1[i]== 6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 608 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 609 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 609 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 610 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 610 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 611 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 611 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 612 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 612 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 613 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 613 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 614 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 614 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 615 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 615 H
  }
  if(a[ 0]>=0) type[a[ 0]]=605;
  if(a[ 1]>=0) type[a[ 1]]=606;
  if(a[ 2]>=0) type[a[ 2]]=607;
  if(a[ 3]>=0) type[a[ 3]]=608;  
  if(a[ 4]>=0) type[a[ 4]]=609;
  if(a[ 5]>=0) type[a[ 5]]=610;
  if(a[ 6]>=0) type[a[ 6]]=611;
  if(a[ 7]>=0) type[a[ 7]]=612;
  if(a[ 8]>=0) type[a[ 8]]=613;
  if(a[ 9]>=0) type[a[ 9]]=614;
  if(a[10]>=0) type[a[10]]=615;
}

void Molecule::examine_pro( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  9];for(int i=0;i<(  9);i++)a  [i]=-1;
  int chk[2+9];for(int i=0;i<(2+9);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 705 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 706 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 706 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 707 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 707 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 708 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 708 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 709 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 709 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 710 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 710 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 711 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 711 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 712 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 712 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 713 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 713 H
  }
  if(a[ 0]>=0) type[a[ 0]]=705;
  if(a[ 1]>=0) type[a[ 1]]=706;
  if(a[ 2]>=0) type[a[ 2]]=707;
  if(a[ 3]>=0) type[a[ 3]]=708;  
  if(a[ 4]>=0) type[a[ 4]]=709;
  if(a[ 5]>=0) type[a[ 5]]=710;
  if(a[ 6]>=0) type[a[ 6]]=711;
  if(a[ 7]>=0) type[a[ 7]]=712;
  if(a[ 8]>=0) type[a[ 8]]=713;
}

void Molecule::examine_phe( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  14];for(int i=0;i<(  14);i++)a  [i]=-1;
  int chk[2+14];for(int i=0;i<(2+14);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 805 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 806 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 806 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 807 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 807 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 808 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 808 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 6){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 809 C
    if(b2[i]==a[2] && a1[i]== 6){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 809 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 6){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 810 C
    if(b2[i]==a[3] && a1[i]== 6){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 810 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 6){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 811 C
    if(b2[i]==a[4] && a1[i]== 6){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 811 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 812 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 812 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 813 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 813 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 814 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 814 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 815 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 815 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 816 H
    if(b2[i]==a[4] && a1[i]== 1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 816 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 1){if(chk[b2[i]]==0){a[12]=b2[i];chk[b2[i]]=1;break;}} // 817 H
    if(b2[i]==a[5] && a1[i]== 1){if(chk[b1[i]]==0){a[12]=b1[i];chk[b1[i]]=1;break;}} // 817 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 1){if(chk[b2[i]]==0){a[13]=b2[i];chk[b2[i]]=1;break;}} // 818 H
    if(b2[i]==a[6] && a1[i]== 1){if(chk[b1[i]]==0){a[13]=b1[i];chk[b1[i]]=1;break;}} // 818 H
  }
  if(a[ 0]>=0) type[a[ 0]]=805;
  if(a[ 1]>=0) type[a[ 1]]=806;
  if(a[ 2]>=0) type[a[ 2]]=807;
  if(a[ 3]>=0) type[a[ 3]]=808;  
  if(a[ 4]>=0) type[a[ 4]]=809;
  if(a[ 5]>=0) type[a[ 5]]=810;
  if(a[ 6]>=0) type[a[ 6]]=811;
  if(a[ 7]>=0) type[a[ 7]]=812;
  if(a[ 8]>=0) type[a[ 8]]=813;
  if(a[ 9]>=0) type[a[ 9]]=814;
  if(a[10]>=0) type[a[10]]=815;
  if(a[11]>=0) type[a[11]]=816;
  if(a[12]>=0) type[a[12]]=817;
  if(a[13]>=0) type[a[13]]=818;
}

void Molecule::examine_trp( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  18];for(int i=0;i<(  18);i++)a  [i]=-1;
  int chk[2+18];for(int i=0;i<(2+18);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 905 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 906 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 906 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]==6){
      int nH=0;
      for(int j=0;j<nb;j++){
        if(b1[j]==b2[i] && a2[j]==1) nH++;
        if(b2[j]==b2[i] && a1[j]==1) nH++;
      }
      if(nH==1){
        if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;} // 907 C
      }
    }
    if(b2[i]==a[1] && a1[i]==6){
      int nH=0;
      for(int j=0;j<nb;j++){
        if(b1[j]==b1[i] && a2[j]==1) nH++;
        if(b2[j]==b1[i] && a1[j]==1) nH++;
      }
      if(nH==1){
        if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;} // 907 C
      }
    }
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 908 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 908 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 7){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 909 N
    if(b2[i]==a[2] && a1[i]== 7){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 909 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 6){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 910 C
    if(b2[i]==a[4] && a1[i]== 6){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 910 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 6){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 911 C
    if(b2[i]==a[3] && a1[i]== 6){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 911 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 6){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 912 C
    if(b2[i]==a[6] && a1[i]== 6){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 912 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[7] && a2[i]== 6){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 913 C
    if(b2[i]==a[7] && a1[i]== 6){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 913 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[8] && a2[i]== 6){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 914 C
    if(b2[i]==a[8] && a1[i]== 6){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 914 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 915 C
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 915 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 916 C
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 916 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[12]=b2[i];chk[b2[i]]=1;break;}} // 917 C
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[12]=b1[i];chk[b1[i]]=1;break;}} // 917 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 1){if(chk[b2[i]]==0){a[13]=b2[i];chk[b2[i]]=1;break;}} // 918 C
    if(b2[i]==a[4] && a1[i]== 1){if(chk[b1[i]]==0){a[13]=b1[i];chk[b1[i]]=1;break;}} // 918 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 1){if(chk[b2[i]]==0){a[14]=b2[i];chk[b2[i]]=1;break;}} // 919 C
    if(b2[i]==a[6] && a1[i]== 1){if(chk[b1[i]]==0){a[14]=b1[i];chk[b1[i]]=1;break;}} // 919 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[7] && a2[i]== 1){if(chk[b2[i]]==0){a[15]=b2[i];chk[b2[i]]=1;break;}} // 920 C
    if(b2[i]==a[7] && a1[i]== 1){if(chk[b1[i]]==0){a[15]=b1[i];chk[b1[i]]=1;break;}} // 920 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[8] && a2[i]== 1){if(chk[b2[i]]==0){a[16]=b2[i];chk[b2[i]]=1;break;}} // 921 C
    if(b2[i]==a[8] && a1[i]== 1){if(chk[b1[i]]==0){a[16]=b1[i];chk[b1[i]]=1;break;}} // 921 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[9] && a2[i]== 1){if(chk[b2[i]]==0){a[17]=b2[i];chk[b2[i]]=1;break;}} // 922 C
    if(b2[i]==a[9] && a1[i]== 1){if(chk[b1[i]]==0){a[17]=b1[i];chk[b1[i]]=1;break;}} // 922 C
  }
  if(a[ 0]>=0) type[a[ 0]]=905;
  if(a[ 1]>=0) type[a[ 1]]=906;
  if(a[ 2]>=0) type[a[ 2]]=907;
  if(a[ 3]>=0) type[a[ 3]]=908;  
  if(a[ 4]>=0) type[a[ 4]]=909;
  if(a[ 5]>=0) type[a[ 5]]=910;
  if(a[ 6]>=0) type[a[ 6]]=911;
  if(a[ 7]>=0) type[a[ 7]]=912;
  if(a[ 8]>=0) type[a[ 8]]=913;
  if(a[ 9]>=0) type[a[ 9]]=914;
  if(a[10]>=0) type[a[10]]=915;
  if(a[11]>=0) type[a[11]]=916;
  if(a[12]>=0) type[a[12]]=917;
  if(a[13]>=0) type[a[13]]=918;
  if(a[14]>=0) type[a[14]]=919;
  if(a[15]>=0) type[a[15]]=920;
  if(a[16]>=0) type[a[16]]=921;
  if(a[17]>=0) type[a[17]]=922;
}

void Molecule::examine_ser( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  5];for(int i=0;i<(  5);i++)a  [i]=-1;
  int chk[2+5];for(int i=0;i<(2+5);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 1005 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 8){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1006 O
    if(b2[i]==a[0] && a1[i]== 8){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1006 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 1007 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 1007 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 1008 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 1008 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 1009 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 1009 H
  }
  if(a[ 0]>=0) type[a[ 0]]=1005;
  if(a[ 1]>=0) type[a[ 1]]=1006;
  if(a[ 2]>=0) type[a[ 2]]=1007;
  if(a[ 3]>=0) type[a[ 3]]=1008;  
  if(a[ 4]>=0) type[a[ 4]]=1009;
}

void Molecule::examine_thr( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  8];for(int i=0;i<(  8);i++)a  [i]=-1;
  int chk[2+8];for(int i=0;i<(2+8);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 1105 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1106 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1106 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 8){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 1107 O
    if(b2[i]==a[0] && a1[i]== 8){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 1107 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 1108 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 1108 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 1109 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 1109 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 1110 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 1110 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 1111 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 1111 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 1112 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 1112 H
  }
  if(a[ 0]>=0) type[a[ 0]]=1105;
  if(a[ 1]>=0) type[a[ 1]]=1106;
  if(a[ 2]>=0) type[a[ 2]]=1107;
  if(a[ 3]>=0) type[a[ 3]]=1108;  
  if(a[ 4]>=0) type[a[ 4]]=1109;
  if(a[ 5]>=0) type[a[ 5]]=1110;
  if(a[ 6]>=0) type[a[ 6]]=1111;
  if(a[ 7]>=0) type[a[ 7]]=1112;
}

void Molecule::examine_asn( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  8];for(int i=0;i<(  8);i++)a  [i]=-1;
  int chk[2+8];for(int i=0;i<(2+8);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 1205 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1206 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1206 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 7){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 1207 N
    if(b2[i]==a[1] && a1[i]== 7){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 1207 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 8){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 1208 O
    if(b2[i]==a[1] && a1[i]== 8){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 1208 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 1209 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 1209 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 1210 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 1210 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 1211 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 1211 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 1212 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 1212 H
  }
  if(a[ 0]>=0) type[a[ 0]]=1205;
  if(a[ 1]>=0) type[a[ 1]]=1206;
  if(a[ 2]>=0) type[a[ 2]]=1207;
  if(a[ 3]>=0) type[a[ 3]]=1208;  
  if(a[ 4]>=0) type[a[ 4]]=1209;
  if(a[ 5]>=0) type[a[ 5]]=1210;
  if(a[ 6]>=0) type[a[ 6]]=1211;
  if(a[ 7]>=0) type[a[ 7]]=1212;
}

void Molecule::examine_gln( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  11];for(int i=0;i<(  11);i++)a  [i]=-1;
  int chk[2+11];for(int i=0;i<(2+11);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 1305 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1306 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1306 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 1307 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 1307 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 7){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 1308 N
    if(b2[i]==a[2] && a1[i]== 7){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 1308 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 8){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 1309 O
    if(b2[i]==a[2] && a1[i]== 8){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 1309 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 1310 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 1310 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 1311 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 1311 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 1312 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 1312 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 1313 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 1313 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 1314 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 1314 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 1315 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 1315 H
  }
  if(a[ 0]>=0) type[a[ 0]]=1305;
  if(a[ 1]>=0) type[a[ 1]]=1306;
  if(a[ 2]>=0) type[a[ 2]]=1307;
  if(a[ 3]>=0) type[a[ 3]]=1308;  
  if(a[ 4]>=0) type[a[ 4]]=1309;
  if(a[ 5]>=0) type[a[ 5]]=1310;
  if(a[ 6]>=0) type[a[ 6]]=1311;
  if(a[ 7]>=0) type[a[ 7]]=1312;
  if(a[ 8]>=0) type[a[ 8]]=1313;
  if(a[ 9]>=0) type[a[ 9]]=1314;
  if(a[10]>=0) type[a[10]]=1315;
}

void Molecule::examine_tyr( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  15];for(int i=0;i<(  15);i++)a  [i]=-1;
  int chk[2+15];for(int i=0;i<(2+15);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 1405 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1406 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1406 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 1407 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 1407 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 1408 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 1408 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 6){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 1409 C
    if(b2[i]==a[2] && a1[i]== 6){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 1409 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 6){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 1410 C
    if(b2[i]==a[3] && a1[i]== 6){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 1410 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 6){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 1411 C
    if(b2[i]==a[4] && a1[i]== 6){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 1411 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 8){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 1412 C
    if(b2[i]==a[6] && a1[i]== 8){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 1412 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 1413 C
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 1413 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 1414 C
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 1414 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 1415 C
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 1415 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 1416 C
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 1416 C
  }
  for(int i=0;i<nb;i++){ 
    if(b1[i]==a[4] && a2[i]== 1){if(chk[b2[i]]==0){a[12]=b2[i];chk[b2[i]]=1;break;}} // 1417 C
    if(b2[i]==a[4] && a1[i]== 1){if(chk[b1[i]]==0){a[12]=b1[i];chk[b1[i]]=1;break;}} // 1417 C
  }
  for(int i=0;i<nb;i++){ 
    if(b1[i]==a[5] && a2[i]== 1){if(chk[b2[i]]==0){a[13]=b2[i];chk[b2[i]]=1;break;}} // 1418 C
    if(b2[i]==a[5] && a1[i]== 1){if(chk[b1[i]]==0){a[13]=b1[i];chk[b1[i]]=1;break;}} // 1418 C
  }
  for(int i=0;i<nb;i++){ 
    if(b1[i]==a[7] && a2[i]== 1){if(chk[b2[i]]==0){a[14]=b2[i];chk[b2[i]]=1;break;}} // 1419 C
    if(b2[i]==a[7] && a1[i]== 1){if(chk[b1[i]]==0){a[14]=b1[i];chk[b1[i]]=1;break;}} // 1419 C
  }
  if(a[ 0]>=0) type[a[ 0]]=1405;
  if(a[ 1]>=0) type[a[ 1]]=1406;
  if(a[ 2]>=0) type[a[ 2]]=1407;
  if(a[ 3]>=0) type[a[ 3]]=1408;  
  if(a[ 4]>=0) type[a[ 4]]=1409;
  if(a[ 5]>=0) type[a[ 5]]=1410;
  if(a[ 6]>=0) type[a[ 6]]=1411;
  if(a[ 7]>=0) type[a[ 7]]=1412;
  if(a[ 8]>=0) type[a[ 8]]=1413;
  if(a[ 9]>=0) type[a[ 9]]=1414;
  if(a[10]>=0) type[a[10]]=1415;
  if(a[11]>=0) type[a[11]]=1416;
  if(a[12]>=0) type[a[12]]=1417;
  if(a[13]>=0) type[a[13]]=1418;
  if(a[14]>=0) type[a[14]]=1419;
}

void Molecule::examine_cys( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  16];for(int i=0;i<(  16);i++)a  [i]=-1;
  int chk[2+16];for(int i=0;i<(2+16);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 1505 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]==16){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1506 S
    if(b2[i]==a[0] && a1[i]==16){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1506 S
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 1507 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 1507 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 1508 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 1508 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 1509 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 1509 H
  }
  if(a[ 0]>=0) type[a[ 0]]=1505;
  if(a[ 1]>=0) type[a[ 1]]=1506;
  if(a[ 2]>=0) type[a[ 2]]=1507;
  if(a[ 3]>=0) type[a[ 3]]=1508;  
  if(a[ 4]>=0) type[a[ 4]]=1509;
}

void Molecule::examine_lys( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  16];for(int i=0;i<(  16);i++)a  [i]=-1;
  int chk[2+16];for(int i=0;i<(2+16);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 1605 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1606 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1606 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 1607 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 1607 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 1608 C
    if(b2[i]==a[2] && a1[i]== 6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 1608 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 7){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 1609 N
    if(b2[i]==a[3] && a1[i]== 7){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 1609 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 1610 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 1610 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 1611 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 1611 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 1612 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 1612 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 1613 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 1613 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 1614 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 1614 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 1615 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 1615 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 1616 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 1616 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[12]=b2[i];chk[b2[i]]=1;break;}} // 1617 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[12]=b1[i];chk[b1[i]]=1;break;}} // 1617 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 1){if(chk[b2[i]]==0){a[13]=b2[i];chk[b2[i]]=1;break;}} // 1618 H
    if(b2[i]==a[4] && a1[i]== 1){if(chk[b1[i]]==0){a[13]=b1[i];chk[b1[i]]=1;break;}} // 1618 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 1){if(chk[b2[i]]==0){a[14]=b2[i];chk[b2[i]]=1;break;}} // 1619 H
    if(b2[i]==a[4] && a1[i]== 1){if(chk[b1[i]]==0){a[14]=b1[i];chk[b1[i]]=1;break;}} // 1619 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 1){if(chk[b2[i]]==0){a[15]=b2[i];chk[b2[i]]=1;break;}} // 1620 H
    if(b2[i]==a[4] && a1[i]== 1){if(chk[b1[i]]==0){a[15]=b1[i];chk[b1[i]]=1;break;}} // 1620 H
  }
  if(a[ 0]>=0) type[a[ 0]]=1605;
  if(a[ 1]>=0) type[a[ 1]]=1606;
  if(a[ 2]>=0) type[a[ 2]]=1607;
  if(a[ 3]>=0) type[a[ 3]]=1608;  
  if(a[ 4]>=0) type[a[ 4]]=1609;
  if(a[ 5]>=0) type[a[ 5]]=1610;
  if(a[ 6]>=0) type[a[ 6]]=1611;
  if(a[ 7]>=0) type[a[ 7]]=1612;
  if(a[ 8]>=0) type[a[ 8]]=1613;  
  if(a[ 9]>=0) type[a[ 9]]=1614;
  if(a[10]>=0) type[a[10]]=1615;
  if(a[11]>=0) type[a[11]]=1616;
  if(a[12]>=0) type[a[12]]=1617;
  if(a[13]>=0) type[a[13]]=1618; 
  if(a[14]>=0) type[a[14]]=1619;
  if(a[15]>=0) type[a[15]]=1620;
}

void Molecule::examine_arg( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  18];for(int i=0;i<(  18);i++)a  [i]=-1;
  int chk[2+18];for(int i=0;i<(2+18);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 1705 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1706 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1706 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1706 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1706 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 1707 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 1707 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 7){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 1708 C
    if(b2[i]==a[2] && a1[i]== 7){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 1708 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 6){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 1709 C
    if(b2[i]==a[3] && a1[i]== 6){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 1709 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 7){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 1710 N
    if(b2[i]==a[4] && a1[i]== 7){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 1710 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 7){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 1711 N
    if(b2[i]==a[4] && a1[i]== 7){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 1711 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 1712 N
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 1712 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 1713 N
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 1713 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 1714 N
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 1714 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 1715 N
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 1715 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 1716 N
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 1716 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[12]=b2[i];chk[b2[i]]=1;break;}} // 1717 N
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[12]=b1[i];chk[b1[i]]=1;break;}} // 1717 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[13]=b2[i];chk[b2[i]]=1;break;}} // 1718 N
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[13]=b1[i];chk[b1[i]]=1;break;}} // 1718 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 1){if(chk[b2[i]]==0){a[14]=b2[i];chk[b2[i]]=1;break;}} // 1719 N
    if(b2[i]==a[5] && a1[i]== 1){if(chk[b1[i]]==0){a[14]=b1[i];chk[b1[i]]=1;break;}} // 1719 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 1){if(chk[b2[i]]==0){a[15]=b2[i];chk[b2[i]]=1;break;}} // 1720 N
    if(b2[i]==a[5] && a1[i]== 1){if(chk[b1[i]]==0){a[15]=b1[i];chk[b1[i]]=1;break;}} // 1720 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 1){if(chk[b2[i]]==0){a[16]=b2[i];chk[b2[i]]=1;break;}} // 1721 N
    if(b2[i]==a[6] && a1[i]== 1){if(chk[b1[i]]==0){a[16]=b1[i];chk[b1[i]]=1;break;}} // 1721 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 1){if(chk[b2[i]]==0){a[17]=b2[i];chk[b2[i]]=1;break;}} // 1722 N
    if(b2[i]==a[6] && a1[i]== 1){if(chk[b1[i]]==0){a[17]=b1[i];chk[b1[i]]=1;break;}} // 1722 N
  }
  if(a[ 0]>=0) type[a[ 0]]=1705;
  if(a[ 1]>=0) type[a[ 1]]=1706;
  if(a[ 2]>=0) type[a[ 2]]=1707;
  if(a[ 3]>=0) type[a[ 3]]=1708;  
  if(a[ 4]>=0) type[a[ 4]]=1709;
  if(a[ 5]>=0) type[a[ 5]]=1710;
  if(a[ 6]>=0) type[a[ 6]]=1711;
  if(a[ 7]>=0) type[a[ 7]]=1712;
  if(a[ 8]>=0) type[a[ 8]]=1713;  
  if(a[ 9]>=0) type[a[ 9]]=1714;
  if(a[10]>=0) type[a[10]]=1715;
  if(a[11]>=0) type[a[11]]=1716;
  if(a[12]>=0) type[a[12]]=1717;
  if(a[13]>=0) type[a[13]]=1718; 
  if(a[14]>=0) type[a[14]]=1719;
  if(a[15]>=0) type[a[15]]=1720;
  if(a[16]>=0) type[a[16]]=1721;
  if(a[17]>=0) type[a[17]]=1722;
}

void Molecule::examine_his( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  12];for(int i=0;i<(  12);i++)a  [i]=-1;
  int chk[2+12];for(int i=0;i<(2+12);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 1805 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1806 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1806 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 1807 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 1807 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 7){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 1808 N
    if(b2[i]==a[1] && a1[i]== 7){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 1808 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 7){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 1809 N
    if(b2[i]==a[2] && a1[i]== 7){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 1809 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 6){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 1810 C
    if(b2[i]==a[3] && a1[i]== 6){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 1810 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 1811 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 1811 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 1812 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 1812 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 1813 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 1813 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 1814 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 1814 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 1815 H
    if(b2[i]==a[5] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 1815 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 1816 H
    if(b2[i]==a[4] && a1[i]== 1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 1816 H
  }
  if(a[ 0]>=0) type[a[ 0]]=1805;
  if(a[ 1]>=0) type[a[ 1]]=1806;
  if(a[ 2]>=0) type[a[ 2]]=1807;
  if(a[ 3]>=0) type[a[ 3]]=1808;  
  if(a[ 4]>=0) type[a[ 4]]=1809;
  if(a[ 5]>=0) type[a[ 5]]=1810;
  if(a[ 6]>=0) type[a[ 6]]=1811;
  if(a[ 7]>=0) type[a[ 7]]=1812;
  if(a[ 8]>=0) type[a[ 8]]=1813;  
  if(a[ 9]>=0) type[a[ 9]]=1814;
  if(a[10]>=0) type[a[10]]=1815;
  if(a[11]>=0) type[a[11]]=1816;
}

void Molecule::examine_asp( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  8];for(int i=0;i<(  8);i++)a  [i]=-1;
  int chk[2+8];for(int i=0;i<(2+8);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 1905 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 1906 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 1906 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 8){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 1907 O
    if(b2[i]==a[1] && a1[i]== 8){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 1907 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 8){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 1908 O
    if(b2[i]==a[1] && a1[i]== 8){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 1908 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 1909 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 1909 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 1910 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 1910 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 1911 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 1911 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 1912 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 1912 H
  }
  if(a[ 0]>=0) type[a[ 0]]=1905;
  if(a[ 1]>=0) type[a[ 1]]=1906;
  if(a[ 2]>=0) type[a[ 2]]=1907;
  if(a[ 3]>=0) type[a[ 3]]=1908;  
  if(a[ 4]>=0) type[a[ 4]]=1909;
  if(a[ 5]>=0) type[a[ 5]]=1910;
  if(a[ 6]>=0) type[a[ 6]]=1911;
  if(a[ 7]>=0) type[a[ 7]]=1912;
}

void Molecule::examine_glu( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  11];for(int i=0;i<(  11);i++)a  [i]=-1;
  int chk[2+11];for(int i=0;i<(2+11);i++)chk[i]= 0;
  a[0]=2;chk[2]=1; // 2005 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 1]=b2[i];chk[b2[i]]=1;break;}} // 2006 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 1]=b1[i];chk[b1[i]]=1;break;}} // 2006 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 2007 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 2007 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 8){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 2008 O
    if(b2[i]==a[2] && a1[i]== 8){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 2008 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 8){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 2009 O
    if(b2[i]==a[2] && a1[i]== 8){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 2009 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 2010 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 2010 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 1){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 2011 H
    if(b2[i]==a[0] && a1[i]== 1){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 2011 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 2012 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 2012 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 2013 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 2013 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 2014 H
    if(b2[i]==a[3] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 2014 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 2015 H
    if(b2[i]==a[4] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 2015 H
  }
  if(a[ 0]>=0) type[a[ 0]]=2005;
  if(a[ 1]>=0) type[a[ 1]]=2006;
  if(a[ 2]>=0) type[a[ 2]]=2007;
  if(a[ 3]>=0) type[a[ 3]]=2008;  
  if(a[ 4]>=0) type[a[ 4]]=2009;
  if(a[ 5]>=0) type[a[ 5]]=2010;
  if(a[ 6]>=0) type[a[ 6]]=2011;
  if(a[ 7]>=0) type[a[ 7]]=2012;
  if(a[ 8]>=0) type[a[ 8]]=2013;
  if(a[ 9]>=0) type[a[ 9]]=2014;
  if(a[10]>=0) type[a[10]]=2015;
}

// private method //
bool Molecule::chk_dna( const int  igroup,
                        const int* group,
                        const int  n_po4) const 
{
  int nn,nb,bond2;
  int n=0;
  //cout << linkgroup[igroup].get_not_dna() << endl ;
  if(linkgroup[igroup].get_not_dna()==1){
    return false;
  }
  for(int i=0;i<natom;i++){
    if(group[i]==igroup){
      if(atom[i].get_atomic_number()==15){
        nn=0;
        nb=atom[i].get_bond_atom_n();
        for(int j=0;j<nb;j++){
          bond2=atom[i].get_bond_atom(j);
          if(atom[bond2].get_atomic_number()==8) nn++;
        }
        if(nn==4){
          n++;
        }
      }
    }
  }
  if(n>=n_po4){
    return true ;
  } else {
    return false;
  }
}

// private method //
bool Molecule::chk_5d_carbon( const int iatom, 
                              int** belong_atom ) const
{
  if(atom[iatom].get_atomic_number()!=6) {
    return false ;
  }

  int nb,a,nH,nC,nO,n;
  int bond_  [4];
  int atomic_[4];

  nb=atom[iatom].get_bond_atom_n();

  if(nb!=4) {
    return false;
  }
  
  for(int i=0;i<4;i++){
    a=atom[iatom].get_bond_atom(i);
    bond_[i]=a;
    atomic_[i]=atom[a].get_atomic_number();
  }

  nH=0; nC=0; nO=0;
  for(int i=0;i<4;i++){
    if(atomic_[i]==1) {nH++;}
    if(atomic_[i]==6) {nC++;}
    if(atomic_[i]==8) {nO++;}
  }
  if( ! (nH==2 && nC==1 && nO==1) ) {
    return false;
  }

  n=0;
  for(int i=0;i<4;i++) {
    if(atomic_[i]==8) {belong_atom[n][0]=bond_[i];belong_atom[n][1]=21;n++;}
    if(atomic_[i]==6) {belong_atom[n][0]=bond_[i];belong_atom[n][1]=22;n++;}
  }

  return true;
}

void Molecule::examine_dna( const int iatom20 ,
                            const int iatom21 ,
                            const int iatom22 ,
                            int*      dna1    ,
                            int*      dna2    ,
                            int*      num1    ,
                            int*      num2    ,
                            int*      group1  ,       // itmp1
                            int*      group2  ,       // itmp2
                            int*      atype1  ,       // type1
                            int*      atype2  ) const // type2
{
  int bond1,bond2,bond3,bond4,iO1,iO2,iC1,iP1,nC,nH,nN,nO,nb,n,nn;
  int iatom23,iatom24,iatom25,iatom26,iatom27,iatom28,iatom29,iatom30;
  int iatom32,iatom33,iatom34,iatom35,iatom36,iatom37,iatom38,iatom39;
  int iatom40,iatom41,iatom42,iatom43,iatom44;
  int not_bond_1[3];
  int not_bond_2[3];

  *num1=0;
  *num2=0;
  iatom23=-1;
  iatom24=-1;
  iatom25=-1;
  iatom26=-1;
  iatom27=-1;
  iatom28=-1;
  iatom29=-1;
  iatom30=-1;
  iO1=-1;
  iO2=-1;
  iC1=-1;
  iP1=-1;
  iatom32=-1;
  iatom33=-1;
  iatom34=-1;
  iatom35=-1;
  iatom36=-1;
  iatom37=-1;
  iatom38=-1;
  iatom39=-1;
  iatom40=-1;
  iatom41=-1;
  iatom42=-1;
  iatom43=-1;
  iatom44=-1;

  /* search 3'carbond (23) */
  if(iatom22>=0){
    nb=atom[iatom22].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom22].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==6 && bond2!=iatom20){
        iatom23=bond2;
        break;
      }
    }
  }

  /* search 2'carbond (24) */
  if(iatom23>=0){
    nb=atom[iatom23].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom23].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==6 && bond2!=iatom22){
        iatom24=bond2;
        break;
      }
    }
  }

  /* search 1'carbond (25) */
  if(iatom24>=0){
    nb=atom[iatom24].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom24].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==6 && bond2!=iatom23){
      iatom25=bond2;
      break;
      }
    }
  }

  /* search N (26) */
  if(iatom25>=0){
    nb=atom[iatom25].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom25].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==7){
        iatom26=bond2;break;
      }
    }
  }
  
  /* search P (27) */
  if(iatom21>=0){
    nb=atom[iatom21].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom21].get_bond_atom(i);
      if(bond2!=iatom20){
        iatom27=bond2;
        break;
      }
    }
  }

  if(iatom27>=0){
    if(atom[iatom27].get_atomic_number()==15){
      
      /* search iO1 (31) */
      nb=atom[iatom27].get_bond_atom_n();
      for(int i=0;i<nb;i++){
        bond2=atom[iatom27].get_bond_atom(i);
        nn=0;
        if(bond2!=iatom21){
          nn=atom[bond2].get_bond_atom_n();
        }
        if(nn==2){
          iO1=bond2;
          break;
        }
      }

      /* search iC1 ( atom in other fragment ) */
      if(iO1>=0){
        nb=atom[iO1].get_bond_atom_n();
        for(int i=0;i<nb;i++){
          bond2=atom[iO1].get_bond_atom(i);
          if(bond2!=iatom27){
            iC1=bond2;
            break;
          }
        }
      }
    } else {
      /* search H (28) */
      iatom28=iatom27;
      iatom27=-1;
    }
  }
  
  /* search iO2 ( atom in other fragment ) */
  if(iatom23>=0){
    nb=atom[iatom23].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom23].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==8){
        iO2=bond2;
        break;
      }
    }
  }

  /* search iP1 ( atom in other fragment ) */
  if(iO2>=0){
    nb=atom[iO2].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iO2].get_bond_atom(i);
      if(bond2!=iatom23){
        iP1=bond2;
        break;
      }
    }
  }

  if(iP1>=0){
    if(atom[iP1].get_atomic_number()!=15){
      iatom29=iO2;
      iatom30=iP1;
      iO2=-1;
      iP1=-1;
    }
  }

  /* search O (32) */
  if(iatom27>=0){
    nb=atom[iatom27].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom27].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==8 && bond2!=iatom21 && bond2!=iO1){
        iatom32=bond2;
        break;
      }
    }
  }

  /* search O (33) */
  if(iatom27>=0){
    nb=atom[iatom27].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom27].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==8 && bond2!=iatom21 && bond2!=iO1 && bond2!=iatom32){
        iatom33=bond2;
        break;
      }
    }
  }

  /* search O (34) */
  if(iatom22>=0){
    nb=atom[iatom22].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom22].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==8){
        iatom34=bond2;
        break;
      }
    }
  }

  /* search O (35) */
  if(iatom24>=0){
    nb=atom[iatom24].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom24].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==8){
        iatom35=bond2;
        break;
      }
    }
  }

  /* search H (36) */
  if(iatom20>=0){
    nb=atom[iatom20].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom20].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==1){
        iatom36=bond2;
        break;
      }
    }
  }

  /* search H (37) */
  if(iatom20>=0){
    nb=atom[iatom20].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom20].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==1 && bond2!=iatom36){
        iatom37=bond2;
        break;
      }
    }
  }

  /* search H (38) */
  if(iatom22>=0){
    nb=atom[iatom22].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom22].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==1){
        iatom38=bond2;
        break;
      }
    }
  }

  /* search H (39) */
  if(iatom23>=0){
    nb=atom[iatom23].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom23].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==1){
        iatom39=bond2;
        break;
      }
    }
  }

  /* search H (40) */
  if(iatom24>=0){
    nb=atom[iatom24].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom24].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==1){
        iatom40=bond2;
        break;
      }
    }
  }

  /* search H (43) */
  if(iatom24>=0){
    nb=atom[iatom24].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom24].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==1 && bond2!=iatom40){
        iatom43=bond2;
        break;
      }
    }
  }

  /* search H (41) */
  if(iatom35>=0){
    nb=atom[iatom35].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom35].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==1){
        iatom41=bond2;
        break;
      }
    }
  }

  /* search H (42) */
  if(iatom25>=0){
    nb=atom[iatom25].get_bond_atom_n();
    for(int i=0;i<nb;i++){
      bond2=atom[iatom25].get_bond_atom(i);
      if(atom[bond2].get_atomic_number()==1){
        iatom42=bond2;
        break;
      }  
    }
  }

  /* search O (44) */
  if(iatom28!=-1){
    iatom44=iatom21;
  }

  n=0;
  not_bond_1[n]=iatom25;
  not_bond_2[n]=iatom26;
  n++;
  if(iO1!=-1){
    not_bond_1[n]=iO1;
    not_bond_2[n]=iC1;
    n++;
  }
  if(iO2!=-1){
    not_bond_1[n]=iatom23;
    not_bond_2[n]=iO2;
    n++;
  }
  bond_trace(iatom20,num1,group1,n,not_bond_1,not_bond_2);
  //cout << "*num1 = " << (*num1) << endl ;
  
  *dna1=4000;
  if (iatom35!=-1 && iatom41!=-1) {*dna1=4001;}  // ribose (20160315)

  if(iatom26>=0){
    n=0;
    not_bond_1[n]=iatom26;
    not_bond_2[n]=iatom25;
    n++;
    bond_trace(iatom26,num2,group2,n,not_bond_1,not_bond_2);
    nC=0;nN=0;nH=0;nO=0;
    for(int i=0;i<(*num2);i++){
      if(atom[group2[i]].get_atomic_number()==1) nH++;
      if(atom[group2[i]].get_atomic_number()==6) nC++;
      if(atom[group2[i]].get_atomic_number()==7) nN++;
      if(atom[group2[i]].get_atomic_number()==8) nO++;
    }
    if     (nC==5 && nH==4  && nN==5 && nO==0) {*dna2=5000;} // adenin   , A
    else if(nC==5 && nH==4  && nN==5 && nO==1) {*dna2=5100;} // guanine  , G
    else if(nC==5 && nH==5  && nN==2 && nO==2) {*dna2=5200;} // thymine  , T
    else if(nC==4 && nH==4  && nN==3 && nO==1) {*dna2=5300;} // cytosine , C
    else if(nC==4 && nH==3  && nN==2 && nO==2) {*dna2=5400;} // uracil   , U
    else {
      *dna2=5900;
    }
    examine_base(*dna2,*num2,group2,atype2);
  }
  //cout << "*num2 = " << (*num2) << endl ;


  for(int i=0;i<(*num1);i++){
    if     (group1[i]==iatom20) atype1[i]=20;
    else if(group1[i]==iatom22) atype1[i]=22;
    else if(group1[i]==iatom23) atype1[i]=23;
    else if(group1[i]==iatom24) atype1[i]=24;
    else if(group1[i]==iatom25) atype1[i]=25;
    else if(group1[i]==iatom27) atype1[i]=27;
    else if(group1[i]==iatom28) atype1[i]=28;
    else if(group1[i]==iatom29) atype1[i]=29;
    else if(group1[i]==iatom30) atype1[i]=30;
    else if(group1[i]==iO1    ) atype1[i]=31;
    else if(group1[i]==iatom32) atype1[i]=32;
    else if(group1[i]==iatom33) atype1[i]=33;
    else if(group1[i]==iatom34) atype1[i]=34;
    else if(group1[i]==iatom35) atype1[i]=35;
    else if(group1[i]==iatom36) atype1[i]=36;
    else if(group1[i]==iatom37) atype1[i]=37;
    else if(group1[i]==iatom38) atype1[i]=38;
    else if(group1[i]==iatom39) atype1[i]=39;
    else if(group1[i]==iatom40) atype1[i]=40;
    else if(group1[i]==iatom41) atype1[i]=41;
    else if(group1[i]==iatom42) atype1[i]=42;
    else if(group1[i]==iatom43) atype1[i]=43;
    else if(group1[i]==iatom44 || 
            group1[i]==iatom21) {
      if(iatom44!=-1){
        atype1[i]=44;
      } else {
        atype1[i]=21;
      }
    }
    else {
      atype1[i]=(*dna1);
    }
  }
  for(int i=0;i<(*num2);i++){
    if(atype2[i]<0){
      atype2[i]=(*dna2);
    }
  }
}

void Molecule::examine_base(const int   dna2  ,
                            const int   num   ,
                            const int * group ,
                            int       * type  ) const
{
  int bond1,bond2,nb,chk1,chk2;
  int * b1;
  int * b2;
  int * a1;
  int * a2;

  b1 = new int [natom+1];
  b2 = new int [natom+2];
  a1 = new int [natom+1]; // atomic number
  a2 = new int [natom+2]; // atomic number

  nb=0;
  for(int i=0;i<nbond;i++){
    bond1=bond[i].get_iatom1();
    bond2=bond[i].get_iatom2();
    chk1=-1;
    chk2=-1;
    for(int j=0;j<num;j++){
      if(bond1==group[j]) chk1=j;
      if(bond2==group[j]) chk2=j;
    }
    if(chk1>-1 && chk2>-1){
      b1[nb]=chk1;
      b2[nb]=chk2;
      a1[nb]=atom[bond1].get_atomic_number();
      a2[nb]=atom[bond2].get_atomic_number();
      nb++;
    }
  }

  if(dna2==5000) examine_ade(nb,b1,b2,a1,a2,type);
  if(dna2==5100) examine_gua(nb,b1,b2,a1,a2,type);
  if(dna2==5200) examine_thy(nb,b1,b2,a1,a2,type);
  if(dna2==5300) examine_cyt(nb,b1,b2,a1,a2,type);
  if(dna2==5400) examine_ura(nb,b1,b2,a1,a2,type);

  delete [] b1;
  delete [] b2;
  delete [] a1;
  delete [] a2;
}

void Molecule::examine_ade( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  14];for(int i=0;i<(  14);i++)a  [i]=-1;
  int chk[  14];for(int i=0;i<(  14);i++)chk[i]= 0;
  a[0]=0;chk[0]=1;

  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){
      if(chk[b2[i]]==0){
        int nC=0;
        for(int j=0;j<nb;j++){
          if(b1[j]==b2[i] && a2[j]==6) nC++;
          if(b2[j]==b2[i] && a1[j]==6) nC++;
        }
        if(nC==0){
          a[ 1]=b2[i];chk[b2[i]]=1;break;
        }
      } 
    } // 5001 C
    if(b2[i]==a[0] && a1[i]== 6){
      if(chk[b1[i]]==0){
        int nC=0;
        for(int j=0;j<nb;j++){
          if(b1[j]==b1[i] && a2[j]==6) nC++;
          if(b2[j]==b1[i] && a1[j]==6) nC++;
        }
        if(nC==0){
          a[ 1]=b1[i];chk[b1[i]]=1;break;
        }
      }
    }
  } // 5001 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 7){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 5002 N
    if(b2[i]==a[1] && a1[i]== 7){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 5002 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 5003 C
    if(b2[i]==a[2] && a1[i]== 6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 5003 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 5004 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 5004 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 7){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 5005 N
    if(b2[i]==a[4] && a1[i]== 7){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 5005 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 6){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 5006 C
    if(b2[i]==a[5] && a1[i]== 6){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 5006 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 7){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 5007 N
    if(b2[i]==a[6] && a1[i]== 7){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 5007 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[7] && a2[i]== 6){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 5008 C
    if(b2[i]==a[7] && a1[i]== 6){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 5008 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[8] && a2[i]== 7){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 5009 N
    if(b2[i]==a[8] && a1[i]== 7){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 5009 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 5010 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 2010 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 5011 H
    if(b2[i]==a[6] && a1[i]== 1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 5011 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[9] && a2[i]== 1){if(chk[b2[i]]==0){a[12]=b2[i];chk[b2[i]]=1;break;}} // 5012 H
    if(b2[i]==a[9] && a1[i]== 1){if(chk[b1[i]]==0){a[12]=b1[i];chk[b1[i]]=1;break;}} // 5012 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[9] && a2[i]== 1){if(chk[b2[i]]==0){a[13]=b2[i];chk[b2[i]]=1;break;}} // 5013 H
    if(b2[i]==a[9] && a1[i]== 1){if(chk[b1[i]]==0){a[13]=b1[i];chk[b1[i]]=1;break;}} // 5013 H
  }
  if(a[ 0]>=0) type[a[ 0]]=  26;
  if(a[ 1]>=0) type[a[ 1]]=5001;
  if(a[ 2]>=0) type[a[ 2]]=5002;
  if(a[ 3]>=0) type[a[ 3]]=5003;  
  if(a[ 4]>=0) type[a[ 4]]=5004;
  if(a[ 5]>=0) type[a[ 5]]=5005;
  if(a[ 6]>=0) type[a[ 6]]=5006;
  if(a[ 7]>=0) type[a[ 7]]=5007;
  if(a[ 8]>=0) type[a[ 8]]=5008;
  if(a[ 9]>=0) type[a[ 9]]=5009;
  if(a[10]>=0) type[a[10]]=5010;
  if(a[11]>=0) type[a[11]]=5011;
  if(a[12]>=0) type[a[12]]=5012;
  if(a[13]>=0) type[a[13]]=5013;
}

void Molecule::examine_gua( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  15];for(int i=0;i<(  15);i++)a  [i]=-1;
  int chk[  15];for(int i=0;i<(  15);i++)chk[i]= 0;
  a[0]=0;chk[0]=1;

  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){
      if(chk[b2[i]]==0){
        int nC=0;
        for(int j=0;j<nb;j++){
          if(b1[j]==b2[i] && a2[j]==6) nC++;
          if(b2[j]==b2[i] && a1[j]==6) nC++;
        }
        if(nC==0){
          a[ 1]=b2[i];chk[b2[i]]=1;break;
        }
      } 
    } // 5101 C
    if(b2[i]==a[0] && a1[i]== 6){
      if(chk[b1[i]]==0){
        int nC=0;
        for(int j=0;j<nb;j++){
          if(b1[j]==b1[i] && a2[j]==6) nC++;
          if(b2[j]==b1[i] && a1[j]==6) nC++;
        }
        if(nC==0){
          a[ 1]=b1[i];chk[b1[i]]=1;break;
        }
      }
    }
  } // 5101 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 7){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 5102 N
    if(b2[i]==a[1] && a1[i]== 7){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 5102 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 5103 C
    if(b2[i]==a[2] && a1[i]== 6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 5103 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 5104 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 5104 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 7){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 5105 N
    if(b2[i]==a[4] && a1[i]== 7){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 5105 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 6){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 5106 C
    if(b2[i]==a[5] && a1[i]== 6){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 5106 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 6){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 5108 C
    if(b2[i]==a[3] && a1[i]== 6){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 5108 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[8] && a2[i]== 7){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 5107 C
    if(b2[i]==a[8] && a1[i]== 7){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 5107 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 7){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 5109 C
    if(b2[i]==a[6] && a1[i]== 7){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 5109 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[8] && a2[i]== 8){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 5110 O
    if(b2[i]==a[8] && a1[i]== 8){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 5110 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 5111 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 5111 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[9] && a2[i]== 1){if(chk[b2[i]]==0){a[12]=b2[i];chk[b2[i]]=1;break;}} // 5112 H
    if(b2[i]==a[9] && a1[i]== 1){if(chk[b1[i]]==0){a[12]=b1[i];chk[b1[i]]=1;break;}} // 5112 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[9] && a2[i]== 1){if(chk[b2[i]]==0){a[13]=b2[i];chk[b2[i]]=1;break;}} // 5113 H
    if(b2[i]==a[9] && a1[i]== 1){if(chk[b1[i]]==0){a[13]=b1[i];chk[b1[i]]=1;break;}} // 5113 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[7] && a2[i]== 1){if(chk[b2[i]]==0){a[14]=b2[i];chk[b2[i]]=1;break;}} // 5114 H
    if(b2[i]==a[7] && a1[i]== 1){if(chk[b1[i]]==0){a[14]=b1[i];chk[b1[i]]=1;break;}} // 5114 H
  }
  if(a[ 0]>=0) type[a[ 0]]=  26;
  if(a[ 1]>=0) type[a[ 1]]=5101;
  if(a[ 2]>=0) type[a[ 2]]=5102;
  if(a[ 3]>=0) type[a[ 3]]=5103;  
  if(a[ 4]>=0) type[a[ 4]]=5104;
  if(a[ 5]>=0) type[a[ 5]]=5105;
  if(a[ 6]>=0) type[a[ 6]]=5106;
  if(a[ 7]>=0) type[a[ 7]]=5107;
  if(a[ 8]>=0) type[a[ 8]]=5108;
  if(a[ 9]>=0) type[a[ 9]]=5109;
  if(a[10]>=0) type[a[10]]=5110;
  if(a[11]>=0) type[a[11]]=5111;
  if(a[12]>=0) type[a[12]]=5112;
  if(a[13]>=0) type[a[13]]=5113;
  if(a[14]>=0) type[a[14]]=5114;
}

void Molecule::examine_thy( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  14];for(int i=0;i<(  14);i++)a  [i]=-1;
  int chk[  14];for(int i=0;i<(  14);i++)chk[i]= 0;
  a[0]=0;chk[0]=1;

  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){
      if(chk[b2[i]]==0){
        int nC=0;
        for(int j=0;j<nb;j++){
          if(b1[j]==b2[i] && a2[j]==6) nC++;
          if(b2[j]==b2[i] && a1[j]==6) nC++;
        }
        if(nC==1){
          a[ 1]=b2[i];chk[b2[i]]=1;break;
        }
      } 
    } // 5201 C
    if(b2[i]==a[0] && a1[i]== 6){
      if(chk[b1[i]]==0){
        int nC=0;
        for(int j=0;j<nb;j++){
          if(b1[j]==b1[i] && a2[j]==6) nC++;
          if(b2[j]==b1[i] && a1[j]==6) nC++;
        }
        if(nC==1){
          a[ 1]=b1[i];chk[b1[i]]=1;break;
        }
      }
    }
  } // 5201 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 5202 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 5202 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 5205 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 5205 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 7){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 5204 C
    if(b2[i]==a[5] && a1[i]== 7){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 5204 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 5203 C
    if(b2[i]==a[4] && a1[i]== 6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 5203 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 6){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 5206 C
    if(b2[i]==a[2] && a1[i]== 6){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 5206 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 8){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 5207 C
    if(b2[i]==a[3] && a1[i]== 8){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 5207 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 8){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 5208 C
    if(b2[i]==a[5] && a1[i]== 8){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 5208 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 5209 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 5209 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 5210 H
    if(b2[i]==a[6] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 5210 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 5211 H
    if(b2[i]==a[6] && a1[i]== 1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 5211 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 1){if(chk[b2[i]]==0){a[12]=b2[i];chk[b2[i]]=1;break;}} // 5212 H
    if(b2[i]==a[6] && a1[i]== 1){if(chk[b1[i]]==0){a[12]=b1[i];chk[b1[i]]=1;break;}} // 5212 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 1){if(chk[b2[i]]==0){a[13]=b2[i];chk[b2[i]]=1;break;}} // 5213 H
    if(b2[i]==a[4] && a1[i]== 1){if(chk[b1[i]]==0){a[13]=b1[i];chk[b1[i]]=1;break;}} // 5213 H
  }
  if(a[ 0]>=0) type[a[ 0]]=  26;
  if(a[ 1]>=0) type[a[ 1]]=5201;
  if(a[ 2]>=0) type[a[ 2]]=5202;
  if(a[ 3]>=0) type[a[ 3]]=5203;  
  if(a[ 4]>=0) type[a[ 4]]=5204;
  if(a[ 5]>=0) type[a[ 5]]=5205;
  if(a[ 6]>=0) type[a[ 6]]=5206;
  if(a[ 7]>=0) type[a[ 7]]=5207;
  if(a[ 8]>=0) type[a[ 8]]=5208;
  if(a[ 9]>=0) type[a[ 9]]=5209;
  if(a[10]>=0) type[a[10]]=5210;
  if(a[11]>=0) type[a[11]]=5211;
  if(a[12]>=0) type[a[12]]=5212;
  if(a[13]>=0) type[a[13]]=5213;
}

void Molecule::examine_cyt( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  12];for(int i=0;i<(  12);i++)a  [i]=-1;
  int chk[  12];for(int i=0;i<(  12);i++)chk[i]= 0;
  a[0]=0;chk[0]=1;

  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){
      if(chk[b2[i]]==0){
        int nC=0;
        for(int j=0;j<nb;j++){
          if(b1[j]==b2[i] && a2[j]==6) nC++;
          if(b2[j]==b2[i] && a1[j]==6) nC++;
        }
        if(nC==1){
          a[ 1]=b2[i];chk[b2[i]]=1;break;
        }
      } 
    } // 5301 C
    if(b2[i]==a[0] && a1[i]== 6){
      if(chk[b1[i]]==0){
        int nC=0;
        for(int j=0;j<nb;j++){
          if(b1[j]==b1[i] && a2[j]==6) nC++;
          if(b2[j]==b1[i] && a1[j]==6) nC++;
        }
        if(nC==1){
          a[ 1]=b1[i];chk[b1[i]]=1;break;
        }
      }
    }
  } // 5301 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 5302 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 5302 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 5303 C
    if(b2[i]==a[2] && a1[i]== 6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 5303 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 5305 C
    if(b2[i]==a[0] && a1[i]== 6){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 5505 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 7){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 5304 N
    if(b2[i]==a[5] && a1[i]== 7){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 5304 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 7){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 5306 N
    if(b2[i]==a[3] && a1[i]== 7){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 5306 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 8){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 5307 O
    if(b2[i]==a[5] && a1[i]== 8){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 5307 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 5308 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 5308 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 5309 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 5309 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 5310 H
    if(b2[i]==a[6] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 5310 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[6] && a2[i]== 1){if(chk[b2[i]]==0){a[11]=b2[i];chk[b2[i]]=1;break;}} // 5311 H
    if(b2[i]==a[6] && a1[i]== 1){if(chk[b1[i]]==0){a[11]=b1[i];chk[b1[i]]=1;break;}} // 5311 H
  }
  if(a[ 0]>=0) type[a[ 0]]=  26;
  if(a[ 1]>=0) type[a[ 1]]=5301;
  if(a[ 2]>=0) type[a[ 2]]=5302;
  if(a[ 3]>=0) type[a[ 3]]=5303;  
  if(a[ 4]>=0) type[a[ 4]]=5304;
  if(a[ 5]>=0) type[a[ 5]]=5305;
  if(a[ 6]>=0) type[a[ 6]]=5306;
  if(a[ 7]>=0) type[a[ 7]]=5307;
  if(a[ 8]>=0) type[a[ 8]]=5308;
  if(a[ 9]>=0) type[a[ 9]]=5309;
  if(a[10]>=0) type[a[10]]=5310;
  if(a[11]>=0) type[a[11]]=5311;
}

void Molecule::examine_ura( const int   nb,
                            const int * b1,
                            const int * b2,
                            const int * a1,
                            const int * a2,
                            int       * type) const 
{
  int a  [  11];for(int i=0;i<(  11);i++)a  [i]=-1;
  int chk[  11];for(int i=0;i<(  11);i++)chk[i]= 0;
  a[0]=0;chk[0]=1;

  for(int i=0;i<nb;i++){
    if(b1[i]==a[0] && a2[i]== 6){
      if(chk[b2[i]]==0){
        int nC=0;
        for(int j=0;j<nb;j++){
          if(b1[j]==b2[i] && a2[j]==6) nC++;
          if(b2[j]==b2[i] && a1[j]==6) nC++;
        }
        if(nC==1){
          a[ 1]=b2[i];chk[b2[i]]=1;break;
        }
      } 
    } // 5401 C
    if(b2[i]==a[0] && a1[i]== 6){
      if(chk[b1[i]]==0){
        int nC=0;
        for(int j=0;j<nb;j++){
          if(b1[j]==b1[i] && a2[j]==6) nC++;
          if(b2[j]==b1[i] && a1[j]==6) nC++;
        }
        if(nC==1){
          a[ 1]=b1[i];chk[b1[i]]=1;break;
        }
      }
    }
  } // 5401 C
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 6){if(chk[b2[i]]==0){a[ 2]=b2[i];chk[b2[i]]=1;break;}} // 5402 C
    if(b2[i]==a[1] && a1[i]== 6){if(chk[b1[i]]==0){a[ 2]=b1[i];chk[b1[i]]=1;break;}} // 5402 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 6){if(chk[b2[i]]==0){a[ 3]=b2[i];chk[b2[i]]=1;break;}} // 5403 C
    if(b2[i]==a[2] && a1[i]== 6){if(chk[b1[i]]==0){a[ 3]=b1[i];chk[b1[i]]=1;break;}} // 5403 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 7){if(chk[b2[i]]==0){a[ 4]=b2[i];chk[b2[i]]=1;break;}} // 5404 C
    if(b2[i]==a[3] && a1[i]== 7){if(chk[b1[i]]==0){a[ 4]=b1[i];chk[b1[i]]=1;break;}} // 5404 C
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 6){if(chk[b2[i]]==0){a[ 5]=b2[i];chk[b2[i]]=1;break;}} // 5405 N
    if(b2[i]==a[4] && a1[i]== 6){if(chk[b1[i]]==0){a[ 5]=b1[i];chk[b1[i]]=1;break;}} // 5405 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[3] && a2[i]== 8){if(chk[b2[i]]==0){a[ 6]=b2[i];chk[b2[i]]=1;break;}} // 5406 N
    if(b2[i]==a[3] && a1[i]== 8){if(chk[b1[i]]==0){a[ 6]=b1[i];chk[b1[i]]=1;break;}} // 5406 N
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[5] && a2[i]== 8){if(chk[b2[i]]==0){a[ 7]=b2[i];chk[b2[i]]=1;break;}} // 5407 O
    if(b2[i]==a[5] && a1[i]== 8){if(chk[b1[i]]==0){a[ 7]=b1[i];chk[b1[i]]=1;break;}} // 5407 O
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[1] && a2[i]== 1){if(chk[b2[i]]==0){a[ 8]=b2[i];chk[b2[i]]=1;break;}} // 5408 H
    if(b2[i]==a[1] && a1[i]== 1){if(chk[b1[i]]==0){a[ 8]=b1[i];chk[b1[i]]=1;break;}} // 5408 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[2] && a2[i]== 1){if(chk[b2[i]]==0){a[ 9]=b2[i];chk[b2[i]]=1;break;}} // 5409 H
    if(b2[i]==a[2] && a1[i]== 1){if(chk[b1[i]]==0){a[ 9]=b1[i];chk[b1[i]]=1;break;}} // 5409 H
  }
  for(int i=0;i<nb;i++){
    if(b1[i]==a[4] && a2[i]== 1){if(chk[b2[i]]==0){a[10]=b2[i];chk[b2[i]]=1;break;}} // 5410 H
    if(b2[i]==a[4] && a1[i]== 1){if(chk[b1[i]]==0){a[10]=b1[i];chk[b1[i]]=1;break;}} // 5410 H
  }
  if(a[ 0]>=0) type[a[ 0]]=  26;
  if(a[ 1]>=0) type[a[ 1]]=5401;
  if(a[ 2]>=0) type[a[ 2]]=5402;
  if(a[ 3]>=0) type[a[ 3]]=5403;  
  if(a[ 4]>=0) type[a[ 4]]=5404;
  if(a[ 5]>=0) type[a[ 5]]=5405;
  if(a[ 6]>=0) type[a[ 6]]=5406;
  if(a[ 7]>=0) type[a[ 7]]=5407;
  if(a[ 8]>=0) type[a[ 8]]=5408;
  if(a[ 9]>=0) type[a[ 9]]=5409;
  if(a[10]>=0) type[a[10]]=5410;
}

Atom Molecule::get_H_cap_atom_nterm(const int ib) const 
{
  int a1,a2,n2,n2_tmp,nb;
  int b[2];
  Atom a;
  n2_tmp=biounit[ib].get_n2();
  if(n2_tmp>=0){
    n2=biounit[ib].get_atom_index(n2_tmp);
    nb=atom[n2].get_bond_atom_n();
    if(nb==2){
      for(int i=0;i<nb;i++){
        b[i]=atom[n2].get_bond_atom(i);
      }
      Dvector3D r1,r2,r3,r12,r13,r123,r,hcoord;
      r1=atom[n2  ].get_R();
      r2=atom[b[0]].get_R();
      r3=atom[b[1]].get_R();
      r12=r2-r1;
      r13=r3-r1;
      r123=r12+r13;
      r=((r123/Dvector3D::norm(r123))*(-1.05/0.5291));
      hcoord=r1+r;
      a.set_atomic_number(1);
      a.set_R(hcoord);
      return a;
    }
  }
  return a;
}

// private method //
void Molecule::biounit_analysis_pdb_tag(const int igroup,
					string& s_out   )
{
  char c[128];
  int alink1,alink2,atm_type1,atm_type2,res_type1,res_type2,res_nterm,res_cterm;
  int c1_atom,c1_atom_tmp,n2_atom,n2_atom_tmp,nb,a2,a2_biounit;
  int imin,itmp1,itmp2;
  string artyp1,anseq1,achin1,aatyp1,artyp2,anseq2,achin2,aatyp2,err_s;
  BioUnit bu;
  int * check;
  int * atom_order;
  int * nseq_order;

  //cout << "Molecule::biounit_analysis_pdb_tag start" << endl ;

  sprintf(c,"\n"                                                         ); s_out=s_out+c;
  sprintf(c,"  ----------------------------------------------\n"         ); s_out=s_out+c;
  sprintf(c,"   # make bio-unit from pdb-tag @ link-group %d \n",igroup+1); s_out=s_out+c;
  sprintf(c,"  ----------------------------------------------\n\n"       ); s_out=s_out+c;
  sprintf(c,"   number of atoms              = "                         ); s_out=s_out+c;
  sprintf(c,"%8d\n",get_natom_linkgroup(igroup)                          ); s_out=s_out+c;
  sprintf(c,"   number of atoms ( hydrogen ) = "                         ); s_out=s_out+c;
  sprintf(c,"%8d\n\n",get_natom_linkgroup_atomic(igroup,1)               ); s_out=s_out+c;

  atom_order = new int [natom+1];
  nseq_order = new int [natom+1];
  check      = new int [natom+1];

  for(int i=0;i<natom;i++){
    check[i]=0;
  }

  // sort
  for(int i=0;i<natom;i++){
    atom_order[i]=i;
    nseq_order[i]=atoi(atom[i].get_nseq_read_pdb().c_str());
  }
  for(int i=0;i<natom;i++){
    imin=i;
    for(int j=(i+1);j<natom;j++){
      if(nseq_order[j]<nseq_order[imin]){
        imin=j;
      }
    }
    itmp1=atom_order[i];
    itmp2=nseq_order[i];
    atom_order[i]=atom_order[imin];
    nseq_order[i]=nseq_order[imin];
    atom_order[imin]=itmp1;
    nseq_order[imin]=itmp2;
  }
  
  res_nterm=nbiounit;
  for(int iorder=0;iorder<natom;iorder++){
    int i=atom_order[iorder];
    if(atom[i].get_lnk1()==igroup){
      if(check[i]==0){
        if(artyp1!="ACE"){
          bu.clear();
        }
	alink1=atom[i].get_link_read_pdb();
	artyp1=atom[i].get_rtyp_read_pdb();
	anseq1=atom[i].get_nseq_read_pdb();
	achin1=atom[i].get_chin_read_pdb();
	aatyp1=atom[i].get_atyp_read_pdb();
        if(artyp1=="NME"){
          bu.copy(biounit[nbiounit-1]);
          pick_up_biounit(nbiounit-1);
        }
	atm_type1=BioUnit::atm_type(aatyp1,artyp1);
        //cout << i << " " << aatyp1 << " " << artyp1 << " " << atm_type1 << endl ; // ishi
	res_type1=BioUnit::res_type(artyp1);
	if(atm_type1==-1){
	  if(res_type1>=0){
	    sprintf(c,"   ** warning : atom = %8d , %s dose not have atom type %s\n",i+1,artyp1.c_str(),aatyp1.c_str()); s_out=s_out+c;
	  }
	}
        if(res_type1 != -1){
          bu.push_back_atom_index(i,atm_type1);
          check[i]=1;
          for(int jorder=iorder+1;jorder<natom;jorder++){
            int j=atom_order[jorder];
            if(atom[j].get_lnk1()==igroup){
              alink2=atom[j].get_link_read_pdb();
              artyp2=atom[j].get_rtyp_read_pdb();
              anseq2=atom[j].get_nseq_read_pdb();
              achin2=atom[j].get_chin_read_pdb();
              aatyp2=atom[j].get_atyp_read_pdb();
              if(alink1==alink2 && artyp1==artyp2 && anseq1==anseq2 && achin1==achin2){
                atm_type2=BioUnit::atm_type(aatyp2,artyp2);
                //cout << j << " " << aatyp2 << " " << artyp2 << " " << atm_type2 << endl ; // ishi
                res_type2=BioUnit::res_type(artyp2);
		if(atm_type2==-1){
		  if(res_type2>=0){
		    sprintf(c,"   ** warning : atom = %8d , %s dose not have atom type %s\n",j+1,artyp2.c_str(),aatyp2.c_str()); s_out=s_out+c;
		  }
		}
                bu.push_back_atom_index(j,atm_type2);
                check[j]=1;
              }
            }
          }
          if(artyp1!="ACE" && artyp1!="NME"){
            bu.set_type(res_type1);
            bu.set_abbr_3(BioUnit::res_abbr_3(res_type1));
            bu.set_abbr_1(BioUnit::res_abbr_1(res_type1));
            if(res_type1==1800){ 
              int chk_hd1=0;
              int chk_he2=0;
              for(int j=0;j<bu.get_n_atom_index();j++){
                if(bu.get_atom_type(j)==1814){chk_hd1=1;}
                if(bu.get_atom_type(j)==1816){chk_he2=1;}
              }
              if(chk_hd1==1 && chk_he2==0){bu.set_type(1800);bu.set_abbr_3("HID");}
              if(chk_hd1==0 && chk_he2==1){bu.set_type(1800);bu.set_abbr_3("HIE");}
              if(chk_hd1==1 && chk_he2==1){bu.set_type(1800);bu.set_abbr_3("HIP");} 
            }
          }
          if(artyp1!="ACE"){
            push_back_biounit(bu);
          }
        }
      }
    }
  }
  res_cterm=nbiounit-1;

  delete [] check;
  delete [] atom_order;
  delete [] nseq_order;

  if(nbiounit<1){
    return ;
  }

  //////////////////
  // set terminal //
  //////////////////
  
  biounit[res_nterm].set_terminal_n(999); // 999 = terminal index for pdb-read // ishi 20140718
  biounit[res_cterm].set_terminal_c(999); // 999 = terminal index for pdb-read // ishi 20140718

  //////////////
  // set link //
  //////////////
 
  for(int i=res_nterm;i<=res_cterm;i++){
    if((100<=biounit[i].get_type() && biounit[i].get_type()<4000) || biounit[i].get_type()== -999){
      if(biounit[i].get_terminal_n()<0){
        n2_atom_tmp=biounit[i].get_n2();
        if(n2_atom_tmp<0){
          break;
        } else {
	  n2_atom=biounit[i].get_atom_index(n2_atom_tmp);
	  nb=atom[n2_atom].get_bond_atom_n();
	  for(int j=0;j<nb;j++){
	    a2=atom[n2_atom].get_bond_atom(j);
	    a2_biounit=get_ibiounit_from_iatom(a2);
	    if(a2_biounit!=i){
	      biounit[i].set_link_unit_n(a2_biounit);
	    }
	  }
	} 
      }
      if(biounit[i].get_terminal_c()<0){
        c1_atom_tmp=biounit[i].get_c1();
        if(c1_atom_tmp<0){
          break;
        } else {
	  c1_atom=biounit[i].get_atom_index(c1_atom_tmp);
	  nb=atom[c1_atom].get_bond_atom_n();
	  for(int j=0;j<nb;j++){
	    a2=atom[c1_atom].get_bond_atom(j);
	    a2_biounit=get_ibiounit_from_iatom(a2);
	    if(a2_biounit!=i){
	      biounit[i].set_link_unit_c(a2_biounit);
	    }
	  }
	}
      }
    }
  }

  ////////////////////////////
  // set ss-bond (20130827) //
  ////////////////////////////

  int bond1,bond2,bio1,bio2;
  for(int i=0;i<nbond;i++){
    bond1=bond[i].get_iatom1();
    bond2=bond[i].get_iatom2();
    if(atom[bond1].get_atomic_number()==16 && atom[bond2].get_atomic_number()==16){
      bio1=get_ibiounit_from_iatom(bond1);
      bio2=get_ibiounit_from_iatom(bond2);
      if(bio1>=0 && bio2>=0){
        biounit[bio1].set_type(1550); biounit[bio1].set_abbr_3("CYX");
        biounit[bio2].set_type(1550); biounit[bio2].set_abbr_3("CYX");
        biounit[bio1].set_ss_bond(bio2);
        biounit[bio2].set_ss_bond(bio1);
      }
    }
  }

  //cout << "Molecule::biounit_analysis_pdb_tag end" << endl ;
}

Atom Molecule::get_H_cap_atom_cterm(const int ib) const 
{
  int a1,a2,c1,nb,c1_tmp;
  int b[2];
  Atom a;
  c1_tmp=biounit[ib].get_c1();
  if(c1_tmp>=0){
    c1=biounit[ib].get_atom_index(c1_tmp);
    nb=atom[c1].get_bond_atom_n();
    if(nb==2){
      for(int i=0;i<nb;i++){
        b[i]=atom[c1].get_bond_atom(i);
      }
      Dvector3D r1,r2,r3,r12,r13,r123,r,hcoord;
      r1=atom[c1  ].get_R();
      r2=atom[b[0]].get_R();
      r3=atom[b[1]].get_R();
      r12=r2-r1;
      r13=r3-r1;
      r123=r12+r13;
      r=((r123/Dvector3D::norm(r123))*(-1.05/0.5291));
      hcoord=r1+r;
      a.set_atomic_number(1);
      a.set_R(hcoord);
      return a;
    }
  }
  return a;
}

int Molecule::chk_iatom_in_ibiounit(const int iatom,
                                    const int ibu) const
{
  for(int i=0;i<biounit[ibu].get_n_atom_index();i++){
    if(iatom==biounit[ibu].get_atom_index(i)) return i;
  }
  return -1;
}

int Molecule::get_ibiounit_dist_in_linkgroup(const double dist,
                                             const int nl,
                                             const int* in,
                                             int* out) const
{
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nbiounit();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_biounit_distance_from_linkgroup(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==1){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ibiounit_dist_in_biounit(const double dist,
                                           const int nl,
                                           const int* in,
                                           int* out) const
{
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nbiounit();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_biounit_distance_from_biounit(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==1){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ibiounit_dist_in_fragment(const double dist,
                                            const int nl,
                                            const int* in,
                                            int* out) const
{
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nbiounit();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_biounit_distance_from_fragment(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==1){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ibiounit_dist_out_linkgroup(const double dist,
                                              const int nl,
                                              const int * in,
                                              int* out) const
{
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nbiounit();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_biounit_distance_from_linkgroup(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==0){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ibiounit_dist_out_biounit(const double dist,
                                            const int nl,
                                            const int* in,
                                            int* out) const 
{
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nbiounit();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_biounit_distance_from_biounit(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==0){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

int Molecule::get_ibiounit_dist_out_fragment(const double dist,
                                             const int    nl,
                                             const int*   in,
                                             int*         out) const
{
  int n,chk;
  double d;
  n=0;
  for(int i=0;i<get_nbiounit();i++){
    chk=0;
    for(int j=0;j<nl;j++){
      d=get_biounit_distance_from_fragment(i,in[j]);
      if(d<dist) {
        chk=1;
        break;
      }
    }
    if(chk==0){
      out[n]=i;
      n++;
    }    
  }
  return n;
}

void Molecule::delete_biounit(const int nl,
                              const int* in)
{
  int * list_atom = new int [get_natom()];
  int n_list_atom=0;
  for(int i=0;i<nl;i++){
    for(int j=0;j<get_biounit(in[i]).get_n_atom_index();j++){
      list_atom[n_list_atom]=get_biounit(in[i]).get_atom_index(j);
      n_list_atom++;
    }
  }
  delete_atom(n_list_atom,list_atom);
  delete [] list_atom;
}

void Molecule::biounit_H_cap()
{
  Atom * a=new Atom [2*get_nbiounit()+1];
  int na=0;
  for(int i=0;i<get_nbiounit();i++){
    if(get_biounit(i).get_terminal_n()==3){
      a[na].copy(get_H_cap_atom_nterm(i));
      na++;
    }
  }
  for(int i=0;i<get_nbiounit();i++){
    if(get_biounit(i).get_terminal_c()==3){
      a[na].copy(get_H_cap_atom_cterm(i));
      na++;
    }
  }
  for(int i=0;i<na;i++){
    add_atom(a[i],3000);
  }
  delete [] a ;
}

void Molecule::get_list_atom_to_biounit(int* list) const 
{
  for(int i=0;i<natom;i++){
    list[i]=-1;
  }
  for(int i=0;i<nbiounit;i++){
    for(int j=0;j<biounit[i].get_n_atom_index();j++){
      list[biounit[i].get_atom_index(j)]=i;
    }
  }
}

string Molecule::get_biounit_info_str_ibio(const int ilink,
                                           const int ibio,
                                           const int nseq) const
{
  string buf;
  int ibio_ss;
  if((100<=biounit[ibio].get_type() && biounit[ibio].get_type()<4000) || 
     biounit[ibio].get_type()==-999){
    buf=buf+iIntToString(ilink+1,4)+", "+biounit[ibio].get_abbr_3()+" "+
      iIntToString(biounit[ibio].get_seq(),3,'0');
    if(biounit[ibio].get_terminal_n()==  1){buf=buf+" N(+0)";}
    if(biounit[ibio].get_terminal_n()==  2){buf=buf+" N(+1)";}
    if(biounit[ibio].get_terminal_n()==  3){buf=buf+" Ncut ";}
    if(biounit[ibio].get_terminal_n()==  6){buf=buf+" NACE ";} // 20130815
    if(biounit[ibio].get_terminal_n()==999){buf=buf+" Nterm";}
    if(biounit[ibio].get_terminal_c()==  1){buf=buf+" C(+0)";}
    if(biounit[ibio].get_terminal_c()==  2){buf=buf+" C(-1)";}
    if(biounit[ibio].get_terminal_c()==  3){buf=buf+" Ccut ";}
    if(biounit[ibio].get_terminal_c()==  4){buf=buf+" Ccap ";}
    if(biounit[ibio].get_terminal_c()==  6){buf=buf+" CNME ";} // 20130815
    if(biounit[ibio].get_terminal_c()==999){buf=buf+" Cterm";}
    if(biounit[ibio].get_ss_bond()>=0){
      ibio_ss=biounit[ibio].get_ss_bond();
      buf=buf+" SS-"+iIntToString(biounit[ibio_ss].get_seq(),3,'0');
    }
  }
  if(4000<=biounit[ibio].get_type() && biounit[ibio].get_type()<5000){
    buf=buf+iIntToString(ilink+1,4)+", ";
    if(biounit[ibio].get_type()==4000){buf=buf+"DOX";}
    if(biounit[ibio].get_type()==4001){buf=buf+"RBO";}
    buf=buf+iIntToString(biounit[ibio].get_seq(),4,'0');
    if(biounit[ibio].get_terminal_5()==1){buf=buf+" 5";}
    if(biounit[ibio].get_terminal_3()==1){buf=buf+" 3";}
  }
  if(5000<=biounit[ibio].get_type() && biounit[ibio].get_type()<5900){
    buf=buf+iIntToString(ilink+1,4)+", "+biounit[ibio].get_abbr_1()+"  "+
      //iIntToString(biounit[ibio].get_seq(),3,'0'); // ishi 20160315
      iIntToString(biounit[ibio].get_seq(),4,'0');
  }
  if(biounit[ibio].get_type()==5900){
    buf=buf+iIntToString(ilink+1,4)+", "+"***"+
      iIntToString(biounit[ibio].get_seq(),3,'0');
  }
  return buf;
}

string Molecule::get_biounit_info_str_iseq(const int ilink, 
                                           const int ibio,
                                           const int ierr) const
{
  if(ilink<0){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_info_str , ilink << 0" << endl ;
      cerr << "ilink = " << ilink << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  if(ilink>=nlinkgroup){
    if(ierr==0 || ierr==1){
      cerr << "error in Molecule::get_biounit_info_str , ilink >= nlinkgroup" << endl ;
      cerr << "ilink = " << ilink << " , nlinkgroup = " << nlinkgroup << " , this = " << this << endl ;
    }
    if(ierr==0) exit(1);
  }
  string buf;
  int ibu,jbu,nbu,n,m;
  n=0;
  m=0;
  nbu=get_nbiounit();
  ibu=get_ibiounit_nterm(ilink);
  while(1){
    if(ibu<0){break;}
    if(n>nbu){break;}
    if(n==ibio){
      buf=get_biounit_info_str_ibio(ilink,ibu,n);
      break;
    }
    ibu=get_biounit(ibu).get_link_unit_c();
    n++;
  }
  ibu=get_ibiounit_5term(ilink);
  while(1){
    if(ibu<0){break;}
    if(n>nbu){break;}
    if(n==ibio){
      buf=get_biounit_info_str_ibio(ilink,ibu,m);
      break;
    }
    jbu=get_biounit(ibu).get_link_unit_b();
    n++;    
    if(n==ibio){
      buf=get_biounit_info_str_ibio(ilink,jbu,m);
      break;
    }
    ibu=get_biounit(ibu).get_link_unit_3();
    n++;
    m++;
  }
  return buf;
}

void Molecule::get_biounit_info_str_all(const int ilink,
                                        string * s_info) const 
{
  int ibu,jbu,nbu,n,m;
  n=0;
  m=0;
  nbu=get_nbiounit();
  ibu=get_ibiounit_nterm(ilink);

  //cout << "ishi in get_biounit_infor_str_all() " << endl ;

  /////////////
  // peptide //
  /////////////

  while(1){
    if(ibu<0){break;}
    if(n>nbu){break;}
    s_info[n]=iIntToString(ibu+1,6)+", "+get_biounit_info_str_ibio(ilink,ibu,n);
    ibu=get_biounit(ibu).get_link_unit_c();
    n++;
  }
  ibu=get_ibiounit_5term(ilink);
  while(1){
    if(ibu<0){break;}
    if(n>nbu){break;}
    s_info[n]=iIntToString(ibu+1,6)+", "+get_biounit_info_str_ibio(ilink,ibu,m);
    //cout << get_biounit_info_str_ibio(ilink,ibu,m) << endl ;
    jbu=get_biounit(ibu).get_link_unit_b();
    n++;
    s_info[n]=iIntToString(jbu+1,6)+", "+get_biounit_info_str_ibio(ilink,jbu,m);
    ibu=get_biounit(ibu).get_link_unit_3();
    n++;
    m++;
  }
}

void Molecule::set_biounit_seq_1( const int ilink )
{
  //cout << "ishi in Molecule::set_biounit_seq_1(const int)" << endl ; 
  int n,ibu,jbu;
  ibu=get_ibiounit_nterm(ilink);
  n=0;
  while(1){
    if(ibu<0)     {break;}
    if(n>nbiounit){break;}
    biounit[ibu].set_seq(linkgroup[ilink].get_residue_index()+n);
    ibu=biounit[ibu].get_link_unit_c();
    n++;
  }
  ibu=get_ibiounit_5term(ilink);
  while(1){
    if(ibu<0)     {break;}
    if(n>nbiounit){break;}
    biounit[ibu].set_seq(linkgroup[ilink].get_residue_index()+n);
    jbu=get_biounit(ibu).get_link_unit_b();
    biounit[jbu].set_seq(linkgroup[ilink].get_residue_index()+n);
    ibu=get_biounit(ibu).get_link_unit_3();
    n++;
  }
}

void Molecule::set_biounit_seq_1( )
{
  //cout << "ishi in Molecule::set_biounit_seq_1()" << endl ; 
  for(int i=0;i<nlinkgroup;i++){
    set_biounit_seq_1(i);
  }
}

void Molecule::set_biounit_seq_1(const int nlg,
                                 const int* ilg)
{
  //cout << "ishi in Molecule::set_biounit_seq_1(const int, const int*)" << endl ; 
  for(int i=0;i<nlg;i++){
    set_biounit_seq_1(ilg[i]);
  }
}

void Molecule::set_biounit_seq_2( const int ilink )
{
  //cout << "ishi in Molecule::set_biounit_seq_2(const int)" << endl ; 
  int ia1,ia2;
  string s;
  for(int i=0;i<nbiounit;i++){
    ia1=biounit[i].get_c0();
    if(ia1>=0){
      ia2=biounit[i].get_atom_index(ia1);
      if(ilink==atom[ia2].get_lnk1()){
	s=atom[ia2].get_nseq_read_pdb();
	if(s!=""){
	  biounit[i].set_seq(atoi(s.c_str()));
	}
      }
    }
  }
}

void Molecule::set_biounit_seq_2( )
{
  cout << "ishi in Molecule::set_biounit_seq_2()" << endl ; 
  for(int i=0;i<nlinkgroup;i++){
    set_biounit_seq_2(i);
  }
}

void Molecule::set_biounit_seq_2( const int nlg,
                                  const int* ilg)
{
  cout << "ishi in Molecule::set_biounit_seq_2(const int, const int*)" << endl ; 
  for(int i=0;i<nlg;i++){
    set_biounit_seq_2(ilg[i]);
  }
}
